
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//
//  Abstract/Purpose:
//    Header file for Data Engine 5.0 or greater. This file defines the
//    manageable data objects which are monitored by the engine.
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCDATAOBJ_H_INCLUDED__
#define __DCDATAOBJ_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>


#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(1)


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Limits
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Minimum size in bytes of data object */
//---------------------------------------------------------------------
#define MIN_DATA_OBJECT_SIZE			(sizeof(DataObjHeader))

//---------------------------------------------------------------------
/** Maximum size in bytes of data object (default) including header */
//---------------------------------------------------------------------
#define MAX_DATA_OBJECT_SIZE			(4096)

//---------------------------------------------------------------------
/** Maximum size in bytes of the data object body */
//---------------------------------------------------------------------
#define MAX_DATA_OBJECT_BODY_SIZE		(MAX_DATA_OBJECT_SIZE - MIN_DATA_OBJECT_SIZE)


//---------------------------------------------------------------------
/**
 * Creator ID definition used to set:
 * ObjID.ObjIDUnion.ObjIDCreatorStruct.creatorID
 * ObjID.ObjIDUnion.InnerObjIDStruct.popID
 * ObjID.ObjIDUnion.ObjIDTypeInstStruct
 * ObjID.ObjIDUnion.asu8[3]
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Public creator ID (processed by data manager) */
//---------------------------------------------------------------------
#define DATA_CREATOR_ID_PUBLIC			(0)

//---------------------------------------------------------------------
/** Reserved creator ID (future use by data manager) */
//---------------------------------------------------------------------
#define DATA_CREATOR_ID_RESERVED1		(255)


//---------------------------------------------------------------------
/**
 * Predefined public data object OIDs should be recognized
 * by all creators/populators
 * @note if (ObjID.ObjIDUnion.InnerObjIDStruct.popID == DATA_CREATOR_ID_PUBLIC)
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Invalid oid */
//---------------------------------------------------------------------
#define DATA_OBJ_OID_INVALID			((u32)0)

//---------------------------------------------------------------------
/** Public oid to denote root of the object tree. This object is cached */
//---------------------------------------------------------------------
#define DATA_OBJ_OID_ROOT				((u32)1)

//---------------------------------------------------------------------
/** Public oid for the main chassis object. This object is cached */
//---------------------------------------------------------------------
#define DATA_OBJ_OID_MAIN_CHASSIS		((u32)2)



//---------------------------------------------------------------------
/**
 * Data Object types for assignment by Data Manager team
 * Definition of manageable object types (DataObjHeader.objType)
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 0x0000 is an invalid object type.
 *  No object can contain this type. This is ignored. */
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_INVALID			(0x0000)


//---------------------------------------------------------------------
/** 0x0001 - 0x0010 are predefined base object types defined by Data Manager.
 *  They cause a specific action to be taken when encountered within the Data Manager. */
//---------------------------------------------------------------------

/** Minimum in predefined base range */
#define DATA_OBJ_TYPE_BASE_LO			(0x0001)

/** Maximum in predefined base range */
#define DATA_OBJ_TYPE_BASE_HI			(0x0010)


//---------------------------------------------------------------------
/** 0x0011 - 0x011F are reserved object types for Disney (HIP) range 1 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 1 */
#define DATA_OBJ_TYPE_HIP_LO_1			(0x0011)

/** Maximum in Disney (HIP) range 1 */
#define DATA_OBJ_TYPE_HIP_HI_1			(0x011F)


//---------------------------------------------------------------------
/** 0x0120 - 0x012F are reserved object types for DRAC (DRAC 3/4/5) */
//---------------------------------------------------------------------

/** Minimum in DRAC (DRAC 3/4/5) range */
#define DATA_OBJ_TYPE_DRSC_LO_1			(0x0120)

/** Maximum in DRAC (DRAC 3/4/5) range */
#define DATA_OBJ_TYPE_DRSC_HI_1			(0x012F)


//---------------------------------------------------------------------
/** 0x0130 - 0x02FF are reserved object types for Disney (HIP) range 2 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 2 */
#define DATA_OBJ_TYPE_HIP_LO_2			(0x0130)

/** Maximum in Disney (HIP) range 2 */
#define DATA_OBJ_TYPE_HIP_HI_2			(0x02FF)


//---------------------------------------------------------------------
/** 0x0300 - 0x04FF are reserved object types for Storage Services */
//---------------------------------------------------------------------

/** Minimum in Storage Services range */
#define DATA_OBJ_TYPE_STORAGE_SVCS_LO	(0x0300)

/** Maximum in Storage Services range */
#define DATA_OBJ_TYPE_STORAGE_SVCS_HI	(0x04FF)


//---------------------------------------------------------------------
/** 0x0500 - 0x05FF are reserved object types for SMASH OOB */
//---------------------------------------------------------------------

/** Minimum in SMASH OOB range */
#define DATA_OBJ_TYPE_SMASHOOB_SVCS_LO	(0x0500)

/** Maximum in SMASH OOB range */
#define DATA_OBJ_TYPE_SMASHOOB_SVCS_HI	(0x05FF)


//---------------------------------------------------------------------
/** 0x0600 - 0x06FF are reserved object types for Generic range */
//---------------------------------------------------------------------

/** Minimum in Generic range */
#define DATA_OBJ_TYPE_GENERIC_LO		(0x0600)

/** Maximum in Generic range */
#define DATA_OBJ_TYPE_GENERIC_HI		(0x06FF)


//---------------------------------------------------------------------
/** 0x0700 - 0x07FF are reserved object types for LC 1.x */
//---------------------------------------------------------------------

/** Minimum in License Management range */
#define DATA_OBJ_TYPE_LC_1_X_LO			(0x0700)

/** Maximum in License Management range */
#define DATA_OBJ_TYPE_LC_1_X_HI			(0x07FF)


//---------------------------------------------------------------------
/** 0x0800 - 0x08FF are reserved object types for Software Inventory */
//---------------------------------------------------------------------

/** Minimum in Software Inventory range */
#define DATA_OBJ_TYPE_SOFTWARE_INV_LO	(0x0800)

/** Maximum in Software Inventory range */
#define DATA_OBJ_TYPE_SOFTWARE_INV_HI	(0x08FF)


//---------------------------------------------------------------------
/** 0x0900 - 0x09FF are reserved object types for License Management */
//---------------------------------------------------------------------

/** Minimum in License Management range */
#define DATA_OBJ_TYPE_LICENSE_MGMT_LO	(0x0900)

/** Maximum in License Management range */
#define DATA_OBJ_TYPE_LICENSE_MGMT_HI	(0x09FF)


//---------------------------------------------------------------------
/** 0x0A00 - 0x7FFF are object types available for assignment by Data Manager team.
 *  Contact Data Manager team to be assigned an object type range. */
//---------------------------------------------------------------------

/** Minimum in next available object type range 1 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_LO_1	(0x0A00)

/** Maximum in next available object type range 1 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_HI_1	(0x7FFF)


//---------------------------------------------------------------------
/** 0x8000 - 0x8FFF are reserved object types for Disney (HIP) range 3 for private objects */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 3 for private objects */
#define DATA_OBJ_TYPE_HIP_LO_3			(0x8000)

/** Maximum in Disney (HIP) range 3 for private objects */
#define DATA_OBJ_TYPE_HIP_HI_3			(0x8FFF)


//---------------------------------------------------------------------
/** 0x9000 - 0xAFFF are object types available for assignment by Data Manager team.
 *  Should be considered reserved by all other teams. */
//---------------------------------------------------------------------

/** Minimum in next available object type range 2 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_LO_2	(0x9000)

/** Maximum in next available object type range 2 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_HI_2   (0xAFFF)


//---------------------------------------------------------------------
/** 0xB000 - 0xBFFF are reserved object types for Disney (HIP) RCI BIOS Setup */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) RCI BIOS Setup object range */
#define DATA_OBJ_TYPE_HIP_RCI_BIOS_SETUP_LO	(0xB000)

/** Maximum in Disney (HIP) RCI BIOS Setup object range */
#define DATA_OBJ_TYPE_HIP_RCI_BIOS_SETUP_HI	(0xBFFF)


//---------------------------------------------------------------------
/** 0xC000 - 0xFFFD are object types available for assignment by Data Manager team.
 *  Highest range that can be assigned by Data Manager team.
 *  Should be considered reserved by all other teams. */
//---------------------------------------------------------------------

/** Minimum in next available object type range 3 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_LO_3	(0xC000)

/** Maximum in next available object type range 3 */
#define DATA_OBJ_TYPE_NEXT_AVAILABLE_HI_3	(0xFFFD)


//---------------------------------------------------------------------
/** 0xFFFE is reserved by Data Manager for further redefinition of the DataObjHeader */
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_RESERVED1			(0xFFFE)


//---------------------------------------------------------------------
/** 0xFFFF is reserved by Data Manager.  Mostly liked to be used by test tools. */
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_RESERVED2			(0xFFFF)



//--------------------------------------------------------------------
/**
 * Base object types defined and internally
 * supported by the Data Manager
 */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Object body is of type DataObjRoot */
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_ROOT				(DATA_OBJ_TYPE_BASE_LO + 0x0)

//---------------------------------------------------------------------
/** Data Object body is of type DataObjRedundancy */
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_REDUNDANCY		(DATA_OBJ_TYPE_BASE_LO + 0x1)



//--------------------------------------------------------------------
/**
 * Definition of timeouts used by OBJ_REFRESH_TX.
 * Choose prime numbers so that Data Object creates
 * derivative timers from the basic ones provided
 */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** 1 second refresh interval */
//---------------------------------------------------------------------
#define DO_REFRESH_TIMEOUT_SECONDS_T0	(1)

//---------------------------------------------------------------------
/** 5 second refresh interval */
//---------------------------------------------------------------------
#define DO_REFRESH_TIMEOUT_SECONDS_T1	(5)

//---------------------------------------------------------------------
/** 23 second refresh interval (prime number) */
//---------------------------------------------------------------------
#define DO_REFRESH_TIMEOUT_SECONDS_T2	(23)

//---------------------------------------------------------------------
/** 61 second refresh interval (prime number) */
//---------------------------------------------------------------------
#define DO_REFRESH_TIMEOUT_SECONDS_T3	(61)



//--------------------------------------------------------------------
/** Definition of manageable object refresh interval (DataObjHeader.refreshInterval) */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** Never poll for refresh */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_NONE			(0)

//---------------------------------------------------------------------
/** Poll for refresh on TIMEOUT_SECONDS_T0 seconds */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_T0				BIT(0)

//---------------------------------------------------------------------
/** Poll for refresh on TIMEOUT_SECONDS_T1 seconds */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_T1				BIT(1)

//---------------------------------------------------------------------
/** Poll for refresh on TIMEOUT_SECONDS_T2 seconds */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_T2				BIT(2)

//---------------------------------------------------------------------
/** Poll for refresh on TIMEOUT_SECONDS_T3 seconds */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_T3				BIT(3)

//---------------------------------------------------------------------
/** Reserved for future use */
//---------------------------------------------------------------------
#define DATA_OBJ_REFRESH_RESERVED		(BIT(4) | \
										BIT(5) | \
										BIT(6) | \
										BIT(7))


//--------------------------------------------------------------------
/** Definition of manageable object status (DataObjHeader.objStatus) */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** The status of the object is not one of below, NOT (unknown, ok, warning, critical and non-recoverable) */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_OTHER			(0)

//---------------------------------------------------------------------
/** The status of the object is unknown (not known or monitored) */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_UNKNOWN			(1)

//---------------------------------------------------------------------
/** The status of the object is all ok */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_OK				(2)

//---------------------------------------------------------------------
/** The status of the object is warning */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_NON_CRITICAL	(3)

//---------------------------------------------------------------------
/** The status of the object is critical */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_CRITICAL		(4)

//---------------------------------------------------------------------
/** The status of the object is non-recoverable (dead) */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_NON_RECOVERABLE	(5)

//---------------------------------------------------------------------
/** Low end of reserved status */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_RESERVED_LO		(6)

//---------------------------------------------------------------------
/** High end of reserved status */
//---------------------------------------------------------------------
#define DATA_OBJ_STATUS_RESERVED_HI		(255)



//--------------------------------------------------------------------
/**
 * Definition of manageable bitwise ORed object flags
 * (DataObjHeader.objFlags)
 * Flag Set to: 0 = Not Enabled  1 = Enabled
 */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** No property flags set */
//---------------------------------------------------------------------
#define DATA_OBJ_FLAG_NONE				(0)

//---------------------------------------------------------------------
/** The object status is managed by the populator. Else managed by Data Manager */
//---------------------------------------------------------------------
#define DATA_OBJ_FLAG_STATUS_POPMNGD	BIT(0)

//---------------------------------------------------------------------
/** The object contents besides the header not cached by Data Manager */
//---------------------------------------------------------------------
#define DATA_OBJ_FLAG_NON_CACHED		BIT(1)

//---------------------------------------------------------------------
/** If set by the creator then refresh is deferred to request for this data object. @note Valid only if the object is cached. Flag should be cleared by Data Populator as required on return. */
//---------------------------------------------------------------------
#define DATA_OBJ_FLAG_REFRESH_DEFERRED	BIT(2)

//---------------------------------------------------------------------
/** Reserved for future use */
//---------------------------------------------------------------------
#define DATA_OBJ_FLAG_RESERVED			(BIT(3) | \
										BIT(4) | \
										BIT(5) | \
										BIT(6) | \
										BIT(7))



//--------------------------------------------------------------------
/** Definition of the redundancy status (DataObjRedundancy.redStatus) */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** The status of the object is not one of below */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_OTHER		(0)

//---------------------------------------------------------------------
/** Redundancy obj status unknown */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_UNKNOWN		(1)

//---------------------------------------------------------------------
/** Redundancy obj status does not apply or not redundant */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_NA			(2)

//---------------------------------------------------------------------
/** Redundancy obj taken offline */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_OFFLINE		(3)

//---------------------------------------------------------------------
/** Obj fully redundant */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_FULL		(4)

//---------------------------------------------------------------------
/** Obj redundancy has been degraded */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_DEGRADED	(5)

//---------------------------------------------------------------------
/** Obj redundancy has been lost */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_LOST		(6)



//--------------------------------------------------------------------
/** Definition of the redundancy count (DataObjRedundancy.redCount) */
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** Used to indicate that the redundancy count (redCount) is unknown */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_REDCOUNT_UNKNOWN	(0)

//---------------------------------------------------------------------
/** Used to check that at least two units are needed to achieve redundancy */
//---------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_REDCOUNT_MIN		(2)



//--------------------------------------------------------------------
/**
 * Macro obtains pointer of type redundancy data object body
 * (DataObjRedundancy *) given a pointer to the data object header
 */
//--------------------------------------------------------------------
#define DATA_OBJ_REDUNDANCY_PTR_BY_HDR(pOBJHDR) ((DataObjRedundancy *)((u8 *)pOBJHDR + MIN_DATA_OBJECT_SIZE))


//--------------------------------------------------------------------
/**
 * Macro obtains pointer of type root data object body
 * (DataObjRoot *) given a pointer to the data object header
 */
//--------------------------------------------------------------------
#define DATA_OBJ_ROOT_PTR_BY_HDR(pOBJHDR) ((DataObjRoot *)((u8 *)pOBJHDR + MIN_DATA_OBJECT_SIZE))



//--------------------------------------------------------------------
/** 
 * Definitions of Field Identifiers (FID) reserved
 * objects defined in this header
 */
//--------------------------------------------------------------------

/** Field Identifier for InnerObjIDStruct */
#define FID_DE_InnerObjIDStruct					(0x3000)
/** Field Identifier for ObjIDCreatorStruct */
#define FID_DE_ObjIDCreatorStruct				(0x3001)
/** Field Identifier for ObjIDTypeInstStruct */
#define FID_DE_ObjIDTypeInstStruct				(0x3002)
/** Field Identifier for ObjIDUnion */
#define FID_DE_ObjIDUnion						(0x3003)
/** Field Identifier for asu32 */
#define FID_DE_asu32							(0x3004)
/** Field Identifier for asu8 */
#define FID_DE_asu8								(0x3005)
/** Field Identifier for creatorID */
#define FID_DE_creatorID						(0x3006)
/** Field Identifier for objCount */
#define FID_DE_objCount							(0x3007)
/** Field Identifier for objFlags */
#define FID_DE_objFlags							(0x3008)
/** Field Identifier for objID */
#define FID_DE_objID							(0x3009)
/** Field Identifier for objID1 */
#define FID_DE_objID1							(0x300a)
/** Field Identifier for objID2 */
#define FID_DE_objID2							(0x300b)
/** Field Identifier for objID3 */
#define FID_DE_objID3							(0x300c)
/** Field Identifier for objInst */
#define FID_DE_objInst							(0x300d)
/** Field Identifier for objSize */
#define FID_DE_objSize							(0x300e)
/** Field Identifier for objStatussd */
#define FID_DE_objStatussd						(0x300f)
/** Field Identifier for objType */
#define FID_DE_objType							(0x3010)
/** Field Identifier for offsetRedName */
#define FID_DE_offsetRedName					(0x3011)
/** Field Identifier for popID */
#define FID_DE_popID							(0x3012)
/** Field Identifier for redCount */
#define FID_DE_redCount							(0x3013)
/** Field Identifier for redStatus */
#define FID_DE_redStatus						(0x3014)
/** Field Identifier for refreshInterval */
#define FID_DE_refreshInterval					(0x3015)


//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Structure that apply to all data objects
//--------------------------------------------------------------------


//--------------------------------------------------------------------
/**
 * All data objects are keyed by a unique object id
 */
//--------------------------------------------------------------------
typedef struct _ObjID
{
	/** choose a representation that fits to your application */
    union _ObjIDUnion
	{
		/** Object ID for reference to as an u32 value */
		u32 asu32;

		/** index 0-2 = sub ID index 3 = Populator/Creator ID */
		u8 asu8[4];

		/** Object ID for reference to as individual IDs and a Data Populator ID */
		struct _InnerObjIDStruct
		{
			/** value for use by creator */
			u8 objID3;
			/** value for use by creator */
			u8 objID2;
			/** value for use by creator */
			u8 objID1;
			/** Should be ID assigned by Data Manager to the populator generating object. <= MAX_POP_ID */
			u8 popID;
		} InnerObjIDStruct;

		/** Object ID for reference to as type, instance and a Data Populator ID */
		struct _ObjIDTypeInstStruct
		{
			/** may be used by data object creator to store object type */
			u16 objType;
			/** May be used by data object creator to store instance of the object when more than one object of the same type exists */
			u8  objInst;
			/** Should be ID assigned by Data Manager to the creator (populator) generating object */
			u8  popID;
		} ObjIDTypeInstStruct;

		/** Object ID for reference to as individual IDs and a Data Populator ID */
		struct _ObjIDCreatorStruct
		{
			/** value for use by creator */
			u8 objID3;
			/** value for use by creator */
			u8 objID2;
			/** value for use by creator */
			u8 objID1;
			/** should be ID assigned by data manager to the creator generating object < DATA_CREATOR_ID_RESERVED1 */
			u8 creatorID;
		} ObjIDCreatorStruct;
    } ObjIDUnion;
} ObjID;


//--------------------------------------------------------------------
/**
 * Specifies list of object OIDs requested or submited
 */
//--------------------------------------------------------------------
typedef struct _ObjList
{
	/** Number of objects in list. */
	u32 objCount;
	/** First object in list followed by variable number of objects. */
	ObjID objID[1];
} ObjList;


//--------------------------------------------------------------------
/** 
 * Structure the is present at the begining of a Data Object.
 * All data objects contain a header followed by a body. A Data Object
 * is used to indicate a widget in the Data Manager and
 * its Data Populators and Data Consumers
 */
//--------------------------------------------------------------------
typedef struct _DataObjHeader
{
	/** Total size of the object, inclusive of objSize. */
	u32 objSize;
	/** Object ID unique on system */
	ObjID objID;
	/** Type of manageable object */
	u16 objType;
	/** Status of the managed object. NOTE: Leaf objects by populator and others by data manager */
	u8 objStatus;
	/** Flags that control the properties of this object */
	u8 objFlags;
	/** Refresh timer for the object (See: SM_API_POP_REFRESH_OBJ). Can be changed on refresh. Defined by OBJ_REFRESH_TXXX */
	u8 refreshInterval;
	/** Reserved for future use. Should be set to 0 by creator. Checked by Data Manager. */
	u8 reservedAlign[3];
} DataObjHeader;



//--------------------------------------------------------------------
/** Base Object structures */
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * The data object body structure of DATA_OBJ_TYPE_ROOT
 */
//---------------------------------------------------------------------
typedef struct _DataObjRoot
{
	/** Reserved for future use set to 0 */
	u32 reserved;
} DataObjRoot;


//---------------------------------------------------------------------
/**
 * The data object body structure of DATA_OBJ_TYPE_ROOT
 * used to represent redundancy for cooling and power
 */
//---------------------------------------------------------------------
typedef struct _DataObjRedundancy
{
	/** redundancy status */
	u8 redStatus;
	/** sub obj type of redundncy device fan0x17, ps0x15, sd0x34 */
	u8 subType;
	/** if not set to DATA_OBJ_REDUNDANCY_REDCOUNT_UNKNOWN then indicates number of devices required for redundancy >= DATA_OBJ_REDUNDANCY_REDCOUNT_MIN */
	u16 redCount;
	/** Offset of the name of this redundancy unit */
	u32 offsetRedName;
	booln eventCapability; //Bit to check whether events/alerts capability is enabled or disabled
	u8 reservedAlignment[2];		// set to 0
} DataObjRedundancy;


//---------------------------------------------------------------------


#pragma pack()


#ifdef __cplusplus
}
#endif

#endif //  __DCDATAOBJ_H_INCLUDED__

