
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
//
// Abstract/Purpose:
//
// Header file for Data Populator to Data Manager Dispatch API interface
// structures, defines and macros
//
// Environment:
//
// WINNT, Netware and OS2
//
//--------------------------------------------------------------------

/** @file */

#ifndef __DCDPDMDISPATCHDEF_H_INCLUDED__
#define __DCDPDMDISPATCHDEF_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// SDK files
//--------------------------------------------------------------------
#include <dcossdk.h>


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>
#include <dcsmdispatchdef.h>
#include <dcdataevent.h>



#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** List specifies parent OID(s) in list. 
 * Certain types of OID or ObjList that can be specified. 
 */
//---------------------------------------------------------------------
#define DPDMD_OID_LIST_TYPE_PARENT	(0)

//---------------------------------------------------------------------
/** List specifies child OID(s) in list. 
 * Certain types of OID or ObjList that can be specified. 
 */
//---------------------------------------------------------------------
#define DPDMD_OID_LIST_TYPE_CHILD	(1)



//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//---------------------------------------------------------------------
// Data Populator(DP) to Data Manager(DM) dispatch API functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Data Populator to Data Manager dispatch interface.
 * Interface structure is to used to implement an API exposed by  
 * the Data Manager to the Data Populator on SM_DRT_POP_LOAD
 * This interface allows the data populator to invoke services 
 * exported within the data manager for data object and event 
 * search and manipulation
 */
//---------------------------------------------------------------------
struct _SMDPDMDispatchTable
{

	//---------------------------------------------------------------------
	// Interface version information
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Gets Major Version of this interface.
	 * This API is called to obtain the major version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Major version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetMajorVersion)(void);

	//---------------------------------------------------------------------
	/** Gets Major Version of this interface.
	 * This API is called to obtain the minor version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Minor version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetMinorVersion)(void);

	//---------------------------------------------------------------------
	/** Gets Revision Version of this interface.
	 * This API is called to obtain the revision version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Revision version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetRevision)(void);


	//---------------------------------------------------------------------
	// Interface heap API
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Interface heap memory free funtion.
	 * This API is called by to free dynamically allocated memory. This 
	 * is used to free memory returned from any API which does NOT have a
	 * type set to "const".
	 * @note The caller cannot use the C-Lib "free" API method to release 
	 * memory returned by this module. 
	 * The memory returned from this module may not be compatible with 
	 * the C-Lib "free" API method and may lead to undefined execution.
	 *
	 * @param pMem [in] Specifies a pointer to a previously returned memory 
	 * that is to be freed.
	 *
	*/
	//---------------------------------------------------------------------
    void (_cdecl *FreeGeneric)(_IN void *pMem);


	//---------------------------------------------------------------------
	// Data manager information API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Gets ready state of Data Manager.
	 * This API is called to see if the the Data Manager server is 
	 * running and ready.
	 *
	 * @return If the Data Manager is ready to serve Data Objects this API 
	 * method ready, returns TRUE else returns FALSE.
	 *
	 */
	//---------------------------------------------------------------------
	booln (_cdecl *IsDataManagerReady)(void);


	//---------------------------------------------------------------------
	// Data Populator information API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** This method is called to return the populator unique ID number.
	 * This API is called to obtain the unique identifier assigne to this 
	 * Data Populator by the Data Manager.
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @return Data Populator ID on success. DATA_CREATOR_ID_PUBLIC on error.
	 *
	 */
	//---------------------------------------------------------------------
	u8 (_cdecl *GetPopulatorID)(_IN const SMDPDMDispatchEnv *pDPDMDE);


	//---------------------------------------------------------------------
	//
	// Request and response allocation API function prototypes
	// use FreeGeneric to free returned memory
	//
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Allocate a largest sized request buffer.
	 * This API method will allocate and return a pointer to the largest 
	 * request structure that can be used. The request structure 
	 * returned should be freed using FreeGeneric. This API will 
	 * return NULL if the memory could not be allocated.
	 *
	 * @param pReqSize [out] Pointer to a variable that receives the
	 * size in bytes of the request structure returned.
	 *
	 * @return pointer to request structure whose memory has 
	 * been allocated or NULL if memory could not be allocated.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	void * (_cdecl *AllocSMReq)(_OUT u32 *pReqSize);

	//---------------------------------------------------------------------
	/** Allocate a largest sized response buffer.
	 * This API method will allocate and return a pointer to the largest 
	 * response structure that can be used. The response structure 
	 * returned should be freed using FreeGeneric. This API will 
	 * return NULL if the memory could not be allocated.
	 *
	 * @param pRspSize [out] Pointer to a variable that receives the
	 * size in bytes of the response structure returned.
	 *
	 * @return pointer to response structure whose memory has 
	 * been allocated or NULL if memory could not be allocated.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	void * (_cdecl *AllocSMRsp)(_OUT u32 *pRspSize);


	//---------------------------------------------------------------------
	// Data Object and Event Allocation API
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Allocate a largest sized data object buffer.
	 * This API method will allocate and return a pointer to the largest 
	 * data object structure that can be used. The data object structure 
	 * returned should be freed using FreeGeneric. This API will 
	 * return NULL if the memory could not be allocated.
	 *
	 * @param pDataObjSize [out] Pointer to a variable that receives the
	 * size in bytes of the data object structure returned.
	 *
	 * @return pointer to data object structure whose memory has 
	 * been allocated or NULL if memory could not be allocated.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	DataObjHeader * (_cdecl *AllocDataObject)(_OUT u32 *pDataObjSize);

	//---------------------------------------------------------------------
	/** Allocate a largest sized event object buffer.
	 * This API method will allocate and return a pointer to the largest 
	 * event object structure that can be used. The event object structure 
	 * returned should be freed using FreeGeneric. This API will 
	 * return NULL if the memory could not be allocated.
	 *
	 * @param pDataEvtSize [out] Pointer to a variable that receives the
	 * size in bytes of the event object structure returned.
	 *
	 * @return pointer to event object structure whose memory has 
	 * been allocated or NULL if memory could not be allocated.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	DataEventHeader * (_cdecl *AllocDataEvent)(_OUT u32 *pDataEvtSize);


	//---------------------------------------------------------------------
	// Object and list Re-allocation API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Duplicate the required sized data object list.
	 * Use these functions if needing to conserve memory buffers returned
	 * by the manager and object access API functions described 
	 * due to the need for caching lists and objects by the caller
	 * returned memory should be freed using FreeGeneric
	 *
	 * @param pMOL [in] Pointer to a object id list
	 *
	 * @param pReqListSize [out] Recieves the size of object list returned.
	 *
	 * @return List of object IDs that are populated in the input list.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *AllocAndCopyRequiredObjList)(_IN const ObjList *pMOL, 
		_OUT u32 *pReqListSize);

	//---------------------------------------------------------------------
	/** Duplicate the required sized data object.
	 * Use these functions if needing to conserve memory buffers returned
	 * by the manager and object access API functions described 
	 * due to the need for caching lists and objects by the caller
	 * returned memory should be freed using FreeGeneric
	 *
	 * @param pMOH [in] Pointer to a data object.
	 *
	 * @return Data Object within a buffer required to contain it.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	DataObjHeader * (_cdecl *AllocAndCopyRequiredDataObj)(_IN const DataObjHeader *pMOH);

	//---------------------------------------------------------------------
	/** Duplicate the required sized data event.
	 * Use these functions if needing to conserve memory buffers returned
	 * by the manager and object access API functions described 
	 * due to the need for caching lists and objects by the caller
	 * returned memory should be freed using FreeGeneric
	 *
	 * @param pMEH [in] Pointer to a data event.
	 *
	 * @return Data Object within a buffer required to contain it.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	DataEventHeader * (_cdecl *AllocAndCopyRequiredDataEvt)(_IN const DataEventHeader *pMEH);


	//---------------------------------------------------------------------
	// Data Object, Event and Manager access API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Get a list of parent OIDs for specified OID. 
	 * This API is called to retrieve all parent objects (OID's) from a 
	 * specific OID of a data object.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 *
	 * @return Pointer to a data structure containing parent object 
	 * identifier lists returned by the Data Manager for this OID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no parent objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListParentOID)(_IN const ObjID *pOID);
	
	//---------------------------------------------------------------------
	/** Get a list of child OIDs for specified OID. 
	 * This API is called to retrieve all child objects (OID's) from a 
	 * specific OID of a data object.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 *
	 * @return Pointer to a data structure containing child object 
	 * identifier lists returned by the Data Manager for this OID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no child objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListChildOID)(_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** Get a list of parent/grandparent OIDs for specified OID and type.
	 * This API is called to retrieve all parent objects (OID's) from a
	 * specific OID and type of a data object.
	 * This search routine examines parents and grandparents to satisfy 
	 * the search criteria.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 * 
	 * @param objType [in] Specifies the type of object to list. 
	 * Data Object types are defined in the dcdataobj.h header 
	 * (i.e. DATA_OBJ_TYPE_ROOT) 
	 *
	 * @return Pointer to a data structure containing parent object 
	 * identifier lists returned by the Data Manager for this OID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no child objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListParentOIDByType)(_IN const ObjID *pOID, 
		_IN u16 objType);

	//---------------------------------------------------------------------
	/** Get a list of child/grandchild OIDs for specified OID and type.
	 * This API is called to retrieve all child objects (OID's) from a
	 * specific OID and type of a data object.
	 * This search routine examines children and grandchildren to satisfy 
	 * the search criteria.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 * 
	 * @param objType [in] Specifies the type of object to list. 
	 * Data Object types are defined in the dcdataobj.h header 
	 * (i.e. DATA_OBJ_TYPE_ROOT) 
	 *
	 * @return Pointer to a data structure containing child object 
	 * identifier lists returned by the Data Manager for this OID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no child objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListChildOIDByType)(_IN const ObjID *pOID, 
		_IN u16 objType);

	//---------------------------------------------------------------------
	/** Lists child/grandchild OIDs for specified OID, type and objStatus.
	 * This API is called to retrieve all child  objects (OID's) from a
	 * specific OID and type and status of a data object.
	 * This search routine examines children and grandchildren to satisfy 
	 * the search criteria.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 * 
	 * @param objType [in] Specifies the type of object to list. 
	 * Data Object types are defined in the dcdataobj.h header 
	 * (i.e. DATA_OBJ_TYPE_ROOT)
	 *
	 * @param worseThan [in] Specifies the search criteria of the 
	 * objStatus field to retrieve (TRUE or FALSE). If TRUE then 
	 * object status searched should be equal or worse than the given 
	 * status. FALSE if exact match desired.
	 *
	 * @param objStatus [in] This the object status field to 
	 * examine for a match. Specifies the status of the object being 
	 * searched for.  Specifies the health status in time associated with 
	 * the Data Object and the properties described by it. 
	 *
	 * @return Pointer to a data structure containing child object 
	 * identifier lists returned by the Data Manager for this OID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no child objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListChildOIDByStatus)(_IN const ObjID *pOID, 
		_IN u16 objType, 
		_IN booln worseThan, 
		_IN u8 objStatus);

	//---------------------------------------------------------------------
	/** This API is called to retrieve specified Data Object. 
	 * The structure associated with a specific object identifier (OID)
	 * is retreived.
	 * This search is the fastest one of all searches.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 *
	 * @return Pointer to the specified Data Object. This structure has 
	 * a header and specific parameters to describe an object. NULL is 
	 * returned if the Data Object could not be found or memory could not 
	 * be allocated.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	DataObjHeader * (_cdecl *GetDataObjByOID)(_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** This API is called to perform set operations specified Data Object. 
	 * The API is called after setting up the request object. 
	 *
	 * @param pSReq [in] pointer to request structure.
	 *
	 * @param reqSize [in] Specifies the size in bytes of the request 
	 * structure.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocSMReq
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *SetObjByReq)(_IN const SMReqHeaderSet *pSReq, 
		_IN u32 reqSize);

	//---------------------------------------------------------------------
	/** This API is called to perform pass-through operations on the 
	 * specified Data Object. The API is called after setting up the 
	 * request object. On completion it returns the completion status 
	 * along with the response from the Data Object if any. 
	 *
	 * @param pOPTReq [in] Pointer to request structure.
	 *
	 * @param reqSize [in] Specifies the size in bytes of the request 
	 * structure.
	 *
	 * @param pRsp [out] Pointer to response structure.
	 *
	 * @param rspSize [in] Specifies the size in bytes of the response 
	 * structure.
	 *
	 * @param pBytesReturned [out] Recieves the size in bytes returned in 
	 * the response structure.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocSMReq, AllocSMRsp
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *PassThruObjByReq)(_IN const SMReqHeaderPassThru *pOPTReq, 
		_IN u32 reqSize, 
		_OUT void *pRsp, 
		_IN u32 rspSize, 
		_OUT u32 *pBytesReturned);


	//---------------------------------------------------------------------
	// Data object create update and destroy API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to create multiple 
	 * data objects. The objects are created as children of the parents 
	 * specified in the parent list.
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param ppDOH [in] Points to an array of Data Object pointers to 
	 * be created.
	 *
	 * @param pNumInDOHList [in] Specifies number of Data Objects
	 * submitted in ppDOH array.
	 *
	 * @param pParentOL [in] Specifies the list of parent OIDs under
	 * which the Data Objects will be created.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocDataObject, AllocSMReq
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjCreateMultiple)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const DataObjHeader **ppDOH, 
		_IN const u32 *pNumInDOHList, 
		_IN const ObjList *pParentOL);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to create a single 
	 * data object. The objects are created as a child of the parent 
	 * specified in the parent OID.
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pDOH [in] Points to a Data Object to be created.
	 *
	 * @param pParentOID [in] Specifies the parent OIDs under which the 
	 * Data Object will be created.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see DataObjCreateMultiple
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjCreateSingle)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const DataObjHeader *pDOH, 
		_IN const ObjID *pParentOID);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to refresh multiple 
	 * data objects. 
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param ppDOH [in] Points to an array of Data Object pointers to 
	 * be refreshed.
	 *
	 * @param pNumInDOHList [in] Specifies number of Data Objects
	 * submitted in ppDOH array.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocDataObject
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjRefreshMultiple)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const DataObjHeader **ppDOH, 
		_IN const u32 *pNumInDOHList);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to refresh a single 
	 * data object. 
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pDOH [in] Points to a Data Object to be refreshed.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see DataObjRefreshMultiple
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjRefreshSingle)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const DataObjHeader *pDOH);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to destroy a single 
	 * data object. 
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pOL [in] Points to a list of OIDs (Data Objects) to be deleted.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see DataObjDestroyMultiple
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjDestroyMultiple)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const ObjList *pOL);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to destroy a single 
	 * data object. 
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pOID [in] Points to an OID (Data Object) to be deleted.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see DataObjDestroyMultiple
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DataObjDestroySingle)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to create multiple 
	 * branches (links) between data objects. The objects branches are 
	 * created as children or as parents based on type specified.
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pOL [in] Points to a list of OIDs (Data Objects) to be branched.
	 *
	 * @param pListType [in] pointer to the type of object list specified: 
	 * DPDMD_OID_LIST_TYPE_PARENT to branch the list to pOID as parents or 
	 * DPDMD_OID_LIST_TYPE_CHILD to branch the list to pOID as children.
	 *
	 * @param pOID [in] Points to an OID (Data Object) to be brached to.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocSMReq
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOBranchCreateMultiple)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const ObjList *pOL, 
		_IN const s32 *pListType, 
		_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** This API is called by the Data Populator to destroy multiple 
	 * branches (unlinks) between data objects. The objects branches are 
	 * destroyed as children or as parents based on type specified.
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pOL [in] Points to a list of OIDs (Data Objects) to be branched.
	 *
	 * @param pListType [in] pointer to the type of object list specified: 
	 * DPDMD_OID_LIST_TYPE_PARENT to unbranch the list to pOID as parents or 
	 * DPDMD_OID_LIST_TYPE_CHILD to unbranch the list to pOID as children.
	 *
	 * @param pOID [in] Points to an OID (Data Object) to be brached to.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocSMReq
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOBranchDestroyMultiple)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_IN const ObjList *pOL, 
		_IN const s32 *pListType, 
		_IN const ObjID *pOID);


	//---------------------------------------------------------------------
	// Data event generation API function prototypes
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** This API allocates and copies the passed event and then queues it 
	 * for further processing by the Data Manager. Then the event is 
	 * queued and processed by the Data Manager. While duplicating the 
	 * event the method fills in mandatory timestamp and reserved feilds 
	 * which is output into the buffer. 
	 *
	 * @param pDPDMDE [in] Specifies the environmental pointer submited to 
	 * the Data Populator during an SM_DRT_POP_LOAD request. 
	 *
	 * @param pDEH [in] Points to an Data Event to be processed. 
	 * pDEH is validated upon entry for size and field values. 
	 * MIN_DATA_EVENT_SIZE < pDEH->evtSize < MAX_DATA_EVENT_SIZE (or INI defined value). 
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error (validation or allocation) returns the status code. 
	 *
	 * @see AllocDataEvent
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DESubmitSingle)(_IN const SMDPDMDispatchEnv *pDPDMDE, 
		_INOUT DataEventHeader *pDEH);


	//---------------------------------------------------------------------
	// Populator support API function prototypes
	// These are routines typically needed by any populator performing work
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Returns an allocated UTF8 string copy of the UCS2 string contained 
	 * within a Data Object at offset determined by offsetStr after 
	 * conversion. NULL is returned if memory could not be allocated or
	 * converted.
	 *
	 * @param pDOH [in] Pointer to the DataObjHeader.
	 *
	 * @param offsetStr [in] offset of the string in the DataObject from the start of pDOH.
	 *
	 * @return Allocated UTF8 string copy of the UCS2 string supplied.
	 * Free returned memory when done with use.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *DOGetByOffsetUTF8Str)(_IN const DataObjHeader *pDOH, 
		_IN u32 offsetStr);

	//---------------------------------------------------------------------
	/** Returns the UTF8 time string corresponding to the input UTC UNIX 
	 * time from EPOCH as a 64-bit number. This is usefull in outputing 
	 * the time stamped into the DataEventHeader or other data objects 
	 * like ESM event logs.
	 *
	 * @param timeVal [in] Number of seconds elapsed since UNIX EPOCH.
	 *
	 * @return Allocated UTF8 string copy of UNIX time string converted.
	 * Free returned memory when done with use.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *DOGetObjTimeUTF8Str)(_IN s64 timeVal);

	//---------------------------------------------------------------------
	/** Returns the alias name of the entity (Populator, Manager, other) that 
	 * created the Data Object.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 * 
	 * @return Allocated UTF8 string copy of creator entity name.
	 * Free returned memory when done with use.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *DOGetCreatorAliasNameByOID)(_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** Returns the display name of the entity (Populator, Manager, other) that 
	 * created the Data Object.
	 *
	 * @param pOID [in] Pointer to OID of Data object.
	 * 
	 * @return Allocated UTF8 string copy of creator entity name.
	 * Free returned memory when done with use.
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *DOGetCreatorDisplayNameByOID)(_IN const ObjID *pOID);

	//---------------------------------------------------------------------
	/** Returns a reference/pointer to the body/payload 
	 * portion of the object memory pointed to by the parameter pDOH. 
	 * It is not a new allocation to prevent unnecessary copies. 
	 * The optional parameter pBodySize will be set to the size in 
	 * bytes of the body/payload. Return value is NULL if body 
	 * absent or is malformed.
	 * @note Do NOT call FreeGeneric or any free on the returned memory 
	 * since it is just a reference pointer from input pDOH.
	 *
	 * @param pDOH [in] Pointer to the DataObjHeader.
	 *
	 * @param pBodySize [out] Recieves the size in bytes of the body/payload.
	 *
	 * @return Pointer to memory that represent the body/payload of Data 
	 * Object.
	 *
	 */
	//---------------------------------------------------------------------
	const void * (_cdecl *DOGetObjBody)(_IN const DataObjHeader *pDOH, 
		_OUT u32 *pBodySize);


	//---------------------------------------------------------------------
	/** This function initializes a object list that the populator is 
	 * required to return to the Data Manager on a 
	 * SM_DRT_COM_LIST_CHILDREN_OID.
	 *
	 * @param pOL [out] Points to a list of OIDs (Data Objects) to be branched.
	 *
	 * @param pOLSize [in] Specifies size of the objList.
	 *
	 * @param pOID [in] Points to an OID (Data Object) to be brached to.
	 *
	 * @param pBytesReturned [out] Recieves number of bytes returned in pOL.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see AllocSMReq
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOObjListClear)(_OUT ObjList *pOL, 
		_IN const u32 *pOLSize, 
		_OUT u32 *pBytesReturned);

	//---------------------------------------------------------------------
	/** This function appends an OID to the end of the object list that 
	 * the populator returns to the Data Manager typically on a 
	 * SM_DRT_COM_LIST_CHILDREN_OID. Be sure to call DOObjListClear 
	 * before calling this routine.
	 *
	 * @param pOIDToAppend [in] unique identifier of the object to append 
	 * to the end of the list.
	 *
	 * @param pOL [in/out] Points to a list of OIDs (Data Objects) to be 
	 * branched.
	 *
	 * @param pOLSize [in] Specifies size of the objList.
	 *
	 * @param pBytesReturned [out] Recieves number of bytes returned in pOL.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 * @see DOObjListClear
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOObjListAppendOID)(_IN const ObjID *pOIDToAppend, 
		_INOUT ObjList *pOL, 
		_IN const u32 *pOLSize, 
		_OUT u32 *pBytesReturned);

	//---------------------------------------------------------------------
	/** This function appends an UTF8 string after conversion to UCS2 to 
	 * a passed filled in Data Object header at the end and returns the 
	 * offset from the beggining to the sting. This routine requires the 
	 * header fields be setup and the size of Data Object be current in
	 * order to append the string to object pDOH->objSize is setup on 
	 * return.
	 *
	 * @param pDOH [in/out] Points to Data Object to which the string is 
	 * appended.
	 *
	 * @param pDOHBufSize [in] Size of buffer pointed to by pDOH. 
	 *
	 * @param pOffsetStrVar [out] offset at which the UTF8 string 
	 * was appended.
	 *
	 * @param pUTF8Str [in] Points to UTF8 string to be appended.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOAppendUTF8Str)(_INOUT DataObjHeader *pDOH, 
		_IN const u32 *pDOHBufSize, 
		_OUT u32 *pOffsetStrVar, 
		_IN const astring *pUTF8Str);

	//---------------------------------------------------------------------
	/** This function appends an UCS2 string without conversion to 
	 * a passed filled in Data Object header at the end and returns the 
	 * offset from the beggining to the sting. This routine requires the 
	 * header fields be setup and the size of Data Object be current in
	 * order to append the string to object pDOH->objSize is setup on 
	 * return.
	 *
	 * @param pDOH [in/out] Points to Data Object to which the string is 
	 * appended.
	 *
	 * @param pDOHBufSize [in] Size of buffer pointed to by pDOH. 
	 *
	 * @param pOffsetStrVar [out] offset at which the UTF8 string 
	 * was appended.
	 *
	 * @param pUCS2Str [in] Points to UCS2 string to be appended.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
	 *
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *DOAppendUCS2Str)(_INOUT DataObjHeader *pDOH, 
		_IN const u32 *pDOHBufSize, 
		_OUT u32 *pOffsetStrVar, 
		_IN const ustring *pUCS2Str);

	//---------------------------------------------------------------------
	/** Get a list of OIDs created by specified creator ID
	 * This API is called to retrieve all objects (OID's) that match
	 * a specific creator ID.
	 * @note These routines return more than required memory on several
	 * occassions and need to be combined with Re-allocation API if 
	 * memory footprint needs to be minimum while caching objects and 
	 * lists.
	 *
	 * @param creatorID [in] ID used to denote the creator in an OID.
	 *
	 * @return Pointer to a data structure containing object
	 * identifier lists returned by the Data Manager macthed for 
	 * this creator ID. 
	 * This structure has members defining the number of objects in 
	 * the list and the objects ID's. NULL Memory could not be allocated 
	 * or initialized or there are no objects satisfying the search
	 * criteria. 
	 *
	 * @see FreeGeneric
	 *
	 */
	//---------------------------------------------------------------------
	ObjList * (_cdecl *ListCreatorOIDByID)(_IN u8 creatorID);

}; // _SMDPDMDispatchTable


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif //  __DCDPDMDISPATCHDEF_H_INCLUDED__

