
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Data Engine Config Library API functions
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCECFLAPI_H_INCLUDED__
#define __DCECFLAPI_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcecfldef.h>
#include <dcsmsecurity.h>
#include <dcsmstatus.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Function Prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Data Engine Security Configuration
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// ECFLSecGetUserAccessPrivLevel
/** 
 * Security: Get user access privilege level
 *
 * This API returns the user privilege level required to access the
 * Data Manager.
 *
 * @note SM_PRIVLEVEL_ADMINISTRATOR allows access only by users
 *		with Administrator privileges.  SM_PRIVLEVEL_POWERUSER allows
 *		access by users with Administrator or Power User privileges.
 *		SM_PRIVLEVEL_USER allows access by users with Administrator,
 *		Power User or User privileges.
 * 
 * @param pPrivLevel [out] Pointer to u32 variable that receives the
 *		privilege level. \n
 *		\n
 *		Possible privilege level values (defined in dcsmsecurity.h): \n
 *		SM_PRIVLEVEL_ADMINISTRATOR \n
 *		SM_PRIVLEVEL_POWERUSER \n
 *		SM_PRIVLEVEL_USER \n
 *
 * @return SM_STATUS_SUCCESS if privilege level is returned successfully.
 *
 * @return Other status code defined in dcsmstatus.h if other error
 *		occurs.  Privilege level is not returned.
 *
 * @sa ECFLSecSetUserAccessPrivLevel
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLSecGetUserAccessPrivLevel(_OUT u32 *pPrivLevel);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLSECGETUSERACCESSPRIVLEVEL)(_OUT u32 *pPrivLevel);

/** Method string definition */
#define FSTRECFLSECGETUSERACCESSPRIVLEVEL	"ECFLSecGetUserAccessPrivLevel"


//---------------------------------------------------------------------
// ECFLSecSetUserAccessPrivLevel
/** 
 * Security: Set user access privilege level.
 *
 * This API sets the user privilege level required to access the
 * Data Manager.
 *
 * @note SM_PRIVLEVEL_ADMINISTRATOR allows access only by users
 *		with Administrator privileges.  SM_PRIVLEVEL_POWERUSER allows
 *		access by users with Administrator or Power User privileges.
 *		SM_PRIVLEVEL_USER allows access by users with Administrator,
 *		Power User or User privileges.
 * 
 * @param privLevel [in] Specifies the privilege level. \n
 *		\n
 *		Supported privilege level values (defined in dcsmsecurity.h): \n
 *		SM_PRIVLEVEL_ADMINISTRATOR \n
 *		SM_PRIVLEVEL_POWERUSER \n
 *		SM_PRIVLEVEL_USER \n
 *
 * @return SM_STATUS_SUCCESS if privilege level is set successfully.
 *
 * @return SM_STATUS_INVALID_PARAMETER if invalid parameter is passed
 *		to the API.  Privilege level is not changed.
 *
 * @return Other status code defined in dcsmstatus.h if other error
 *		occurs.  Privilege level is not changed.
 *
 * @sa ECFLSecGetUserAccessPrivLevel
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLSecSetUserAccessPrivLevel(_IN u32 privLevel);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLSECSETUSERACCESSPRIVLEVEL)(_IN u32 privLevel);

/** Method string definition */
#define FSTRECFLSECSETUSERACCESSPRIVLEVEL	"ECFLSecSetUserAccessPrivLevel"


//---------------------------------------------------------------------
// ECFLSecGetUserGroupPrivLevel
/** 
 * Security: Get user group privilege level
 *
 * This API returns the current privilege level for the specified
 * user group.
 * 
 * @param userGroup [in] Specifies the user group. \n
 *		\n
 *		Supported user group values (defined in dcsmsecurity.h): \n
 *		SM_USERGROUP_USER \n
 *		SM_USERGROUP_POWERUSER \n
 *		SM_USERGROUP_ADMINISTRATOR \n\n
 * 
 * @param pPrivLevel [out] Pointer to u32 variable that receives the
 *		privilege level. \n
 *		\n
 *		Possible privilege level values (defined in dcsmsecurity.h): \n
 *		SM_PRIVLEVEL_USER \n
 *		SM_PRIVLEVEL_POWERUSER \n
 *		SM_PRIVLEVEL_ADMINISTRATOR \n
 *
 * @return SM_STATUS_SUCCESS if privilege level is returned successfully.
 *
 * @return SM_STATUS_INVALID_PARAMETER if invalid parameter is passed
 *		to the API.  Privilege level is not returned.
 *
 * @return Other status code defined in dcsmstatus.h if other error
 *		occurs.  Privilege level is not returned.
 *
 * @sa ECFLSecSetUserGroupPrivLevel
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLSecGetUserGroupPrivLevel(_IN u32 userGroup,
											   _OUT u32 *pPrivLevel);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLSECGETUSERGROUPPRIVLEVEL)(_IN u32 userGroup,
													    _OUT u32 *pPrivLevel);

/** Method string definition */
#define FSTRECFLSECGETUSERGROUPPRIVLEVEL	"ECFLSecGetUserGroupPrivLevel"


//---------------------------------------------------------------------
// ECFLSecSetUserGroupPrivLevel
/** 
 * Security: Set user group privilege level.
 *
 * This API sets the privilege level for the specified user group.
 *
 * @note The privilege level cannot be set below the default for the
 *		specified user group.
 * 
 * @param userGroup [in] Specifies the user group. \n
 *		\n
 *		Supported user group values (defined in dcsmsecurity.h): \n
 *		SM_USERGROUP_USER  (not allowed by default)\n
 *		SM_USERGROUP_POWERUSER \n
 *		SM_USERGROUP_ADMINISTRATOR \n\n
 * 
 * @param privLevel [in] Specifies the privilege level. \n
 *		\n
 *		Supported privilege level values (defined in dcsmsecurity.h): \n
 *		SM_PRIVLEVEL_USER \n
 *		SM_PRIVLEVEL_POWERUSER \n
 *		SM_PRIVLEVEL_ADMINISTRATOR \n
 *
 * @return SM_STATUS_SUCCESS if privilege level is set successfully.
 *
 * @return SM_STATUS_INVALID_PARAMETER if invalid parameter is passed
 *		to the API.  Privilege level is not changed.
 *
 * @return Other status code defined in dcsmstatus.h if other error
 *		occurs.  Privilege level is not changed.
 *
 * @sa ECFLSecGetUserGroupPrivLevel
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLSecSetUserGroupPrivLevel(_IN u32 userGroup,
											   _IN u32 privLevel);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLSECSETUSERGROUPPRIVLEVEL)(_IN u32 userGroup,
														_IN u32 privLevel);

/** Method string definition */
#define FSTRECFLSECSETUSERGROUPPRIVLEVEL	"ECFLSecSetUserGroupPrivLevel"


//---------------------------------------------------------------------
// Data Engine SNMP Configuration
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// ECFLEnableSNMP
/** 
 * SNMP: Enable Data Engine SNMP.
 *
 * This API enables Data Engine SNMP support.
 *
 * The OS SNMP agent configuration is updated if needed.
 * 
 * @param pDisplayMsg [out] Pointer to variable to indicate display or not.
 * 
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLEnableSNMP(_OUT booln *pDisplayMsg);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLENABLESNMP)(_OUT booln *pDisplayMsg);

/** Method string definition */
#define FSTRECFLENABLESNMP	"ECFLEnableSNMP"


//---------------------------------------------------------------------
// ECFLDisableSNMP
/** 
 * SNMP: Disable Data Engine SNMP.
 *
 * This API disables Data Engine SNMP support.
 *
 * The OS SNMP agent configuration is updated if needed.
 * 
 * @param pDisplayMsg [out] Pointer to variable to indicate display or not.
 * 
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLDisableSNMP(_OUT booln *pDisplayMsg);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLDISABLESNMP)(_OUT booln *pDisplayMsg);

/** Method string definition */
#define FSTRECFLDISABLESNMP	"ECFLDisableSNMP"


//---------------------------------------------------------------------
// ECFLGetSNMPStatus
/** 
 * SNMP: Get Data Engine SNMP status.
 * 
 * This API returns whether Data Engine SNMP support is enabled or disabled.
 * 
 * @param pDisplayMsg [out] Pointer to variable to indicate display or not.
 * 
 * @param pEnableStatus [out] Pointer to variable that receives status.
 *		TRUE indicates enabled, and FALSE indicates disabled.
 *
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLGetSNMPStatus(_OUT booln *pDisplayMsg,
									_OUT booln *pEnableStatus);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLGETSNMPSTATUS)(_OUT booln *pDisplayMsg,
											 _OUT booln *pEnableStatus);

/** Method string definition */
#define FSTRECFLGETSNMPSTATUS	"ECFLGetSNMPStatus"


//---------------------------------------------------------------------
// ECFLEnableSNMPSets
/** 
 * SNMP: Enable Data Engine SNMP Sets.
 * 
 * This API enables SNMP Sets in Data Engine SNMP support.  
 * When enabled, SNMP Set requests are dispatched to MIB implementers.
 * 
 * @param bForce [in] Indicates whether the INI file should be created.
 * 
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLEnableSNMPSets(_IN booln bForce);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLENABLESNMPSETS)(_IN booln bForce);

/** Method string definition */
#define FSTRECFLENABLESNMPSETS	"ECFLEnableSNMPSets"


//---------------------------------------------------------------------
// ECFLDisableSNMPSets
/** 
 * SNMP: Disable Data Engine SNMP Sets.
 * 
 * This API disables SNMP Sets in Data Engine SNMP support.  
 * When disabled, SNMP Set requests are not dispatched to MIB implementers.
 * 
 * @param bForce [in] Indicates whether the INI file should be created.
 * 
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLDisableSNMPSets(_IN booln bForce);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLDISABLESNMPSETS)(_IN booln bForce);

/** Method string definition */
#define FSTRECFLDISABLESNMPSETS	"ECFLDisableSNMPSets"


//---------------------------------------------------------------------
// ECFLGetSNMPSetsStatus
/** 
 * SNMP: Get Data Engine SNMP Sets status.
 * 
 * This API returns whether SNMP Sets are enabled or disabled in
 * Data Engine SNMP support.
 * 
 * @param bForce [in] Indicates whether the INI file should be created.
 * 
 * @param pEnableStatus [out] Pointer to variable that receives status. 
 *		TRUE indicates enabled, and FALSE indicates disabled.
 *
 * @return On success returns SM_STATUS_SUCCESS.  
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl ECFLGetSNMPSetsStatus(_IN booln bForce,
										_OUT booln *pEnableStatus);

/** Method type definition */
typedef s32 (_cdecl *FPROCECFLGETSNMPSETSSTATUS)(_IN booln bForce,
												 _OUT booln *pEnableStatus);

/** Method string definition */
#define FSTRECFLGETSNMPSETSSTATUS	"ECFLGetSNMPSetsStatus"


//---------------------------------------------------------------------
// End Prototypes
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCECFLAPI_H_INCLUDED__

