
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 9 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Data Engine Abstract Data Accessor public Definitions
//
//  See Also:
//    Please see Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCEDADEF_H_INCLUDED__
#define __DCEDADEF_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

//--------------------------------------------------------------------
// Definitions for commands
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Data Accessor Command Declaration specifier */
//---------------------------------------------------------------------
#define NVP_NAME_DA_CMD							"omacmd"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration user rights specifier */
//---------------------------------------------------------------------
#define NVP_NAME_USERRIGHTS						"omausrmask"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration user information specifier */
//---------------------------------------------------------------------
#define NVP_NAME_USERINFO						"omausrinfo"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration monitor performance specifier */
//---------------------------------------------------------------------
#define NVP_NAME_PERFMON						"omaperfmon"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration echo command specifier */
//---------------------------------------------------------------------
#define NVP_NAME_ECHO_CMD						"omacmdecho"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration set error level specifier */
//---------------------------------------------------------------------
#define NVP_NAME_ERR_LVL						"omaerrlevel"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration redirect to script path file name specifier */
//---------------------------------------------------------------------
#define NVP_NAME_SCRIPTPFN						"omascriptpfn"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration redirect for command listing specifier */
//---------------------------------------------------------------------
#define NVP_CMD_LIST_HELP						NVP_NAME_DA_CMD "=?"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration get command help specifier */
//---------------------------------------------------------------------
#define NVP_CMD_HELP							"help=?"

//---------------------------------------------------------------------
/** Data Accessor Command Declaration specifier (legacy support) */
//---------------------------------------------------------------------
#define NVPAIR_CMD_NAME_ASTR					NVP_NAME_DA_CMD

//---------------------------------------------------------------------
/** Data Accessor Command Declaration user rights specifier (legacy support) */
//---------------------------------------------------------------------
#define NVPAIR_USERRIGHTS_NAME					NVP_NAME_USERRIGHTS

//---------------------------------------------------------------------
/** Data Accessor Command Declaration user information specifier (legacy support) */
//---------------------------------------------------------------------
#define NVPAIR_USERINFO_NAME					NVP_NAME_USERINFO


//---------------------------------------------------------------------
//  Access Rights Bitmap
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Accessor User Rights No Access mask */
//---------------------------------------------------------------------
#define OMA_ACCESS_RIGHTS_NONE					(0)

//---------------------------------------------------------------------
/** Data Accessor User Rights Read Access mask */
//---------------------------------------------------------------------
#define OMA_ACCESS_RIGHTS_READ					(BIT(0))

//---------------------------------------------------------------------
/** Data Accessor User Rights Write Access mask */
//---------------------------------------------------------------------
#define OMA_ACCESS_RIGHTS_WRITE					(BIT(1))

//---------------------------------------------------------------------
/** Data Accessor User Rights Admin Access mask */
//---------------------------------------------------------------------
#define OMA_ACCESS_RIGHTS_ADMIN					(BIT(2))


//---------------------------------------------------------------------
//  User Access Rights
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Accessor User Rights No Access */
//---------------------------------------------------------------------
#define OMA_USER_RIGHTS_NONE					(OMA_ACCESS_RIGHTS_NONE)

//---------------------------------------------------------------------
/** Data Accessor User Rights User level Access */
//---------------------------------------------------------------------
#define OMA_USER_RIGHTS_USER					(OMA_ACCESS_RIGHTS_READ)

//---------------------------------------------------------------------
/** Data Accessor User Rights Power User level Access */
//---------------------------------------------------------------------
#define OMA_USER_RIGHTS_POWERUSER				(OMA_ACCESS_RIGHTS_READ | \
												OMA_ACCESS_RIGHTS_WRITE)

//---------------------------------------------------------------------
/** Data Accessor User Rights Admin level Access */
//---------------------------------------------------------------------
#define OMA_USER_RIGHTS_ADMIN					(OMA_ACCESS_RIGHTS_READ | \
												OMA_ACCESS_RIGHTS_WRITE | \
												OMA_ACCESS_RIGHTS_ADMIN)


//---------------------------------------------------------------------
// Function Signatures exported by all Data Accessors
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Accessor Interface send command function prototype
 * Receives a name-value-pair string and marshals the command request 
 * to the appropriate function. This is the entry function to the data 
 * accessor that processes commands based on a set of name value pairs 
 * (NVP). The return value is an XML containing the requested data and 
 * status information. Any errors preventing the generation of the XML 
 * returns NULL. Some errors including the data accessor in undefined 
 * state may return an invalid XML. Always release the returned memory 
 * with FPROCDCDAFREEDATA
 *
 * @param numNVPair [in] The number of items in the name-value-pair 
 *        string ppNVPair
 * 
 * @param ppNVPair [in] The name-value-pair string
 * 
 * @return On success, returns the result XML string. Use the free data 
 *         interface to release memory returned. On error, returns NULL 
 *         or XML string containing error information. 
 *
 * @see FPROCDCDAFREEDATA
 */
//---------------------------------------------------------------------
typedef astring * (_cdecl *FPROCDCDASENDCMD)(_IN s32 numNVPair,
											 _IN astring **ppNVPair);

//---------------------------------------------------------------------
/** Data Accessor Interface free data function prototype
 * Releases the heap memory returned from the send command function.  
 * The behavior is undetermined if the reference value provided did 
 * not come from the send command.
 *
 * @param pData [in] Pointer to the heap memory returned from the 
 *        send command function
 * 
 * @return none
 *
 * @see FPROCDCDASENDCMD
 */
//---------------------------------------------------------------------
typedef void (_cdecl *FPROCDCDAFREEDATA)(_IN astring *pData);

//---------------------------------------------------------------------
/** Data Accessor Interface destroy function prototype
 * Performs necessary cleanup, the scope of the cleanup is 
 * implementation specific. Implementation and/or exposure of this
 * function is optional.
 *
 * @return none
 */
//---------------------------------------------------------------------
typedef void (_cdecl *FPROCDCDADESTROY)(void);


//---------------------------------------------------------------------
// Function string signature template defined by all Data Accessors
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Accessor Send Command string template used to form the export symbol name */
//---------------------------------------------------------------------
#define OMA_DA_SENDCMD_TEMPLATE					"%s_sendCmd"

//---------------------------------------------------------------------
/** Data Accessor Free Data string template used to form the export symbol name */
//---------------------------------------------------------------------
#define OMA_DA_FREEDATA_TEMPLATE				"%s_freeData"

//---------------------------------------------------------------------
/** Data Accessor Destroy string template used to form the export symbol name */
//---------------------------------------------------------------------
#define OMA_DA_DESTROY_TEMPLATE					"%s_destroy"


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCEDADEF_H_INCLUDED__
