
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 9 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Data Engine Abstract Data Accessor public API
//
//  See Also:
//    Please see Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCEDAPI_H_INCLUDED__
#define __DCEDAPI_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <dcedadef.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Macro definitions that must be defined in the projects of
// all data accessors.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Compile time validation ensuring implementation defines the send command function prototype name */
//---------------------------------------------------------------------
#ifndef DATAACCESSOR_SENDCMD
#error DATAACCESSOR_SENDCMD not defined by makefile. Example DATAACCESSOR_SENDCMD=hipda32_sendCmd
#endif

//---------------------------------------------------------------------
/** Compile time validation ensuring implementation defines the free data function prototype name */
//---------------------------------------------------------------------
#ifndef DATAACCESSOR_FREEDATA
#error DATAACCESSOR_FREEDATA not defined by makefile. Example DATAACCESSOR_FREEDATA=hipda32_freeData
#endif

//---------------------------------------------------------------------
/** Compile time validation ensuring implementation defines the destroy function prototype name */
//---------------------------------------------------------------------
#ifndef DATAACCESSOR_DESTROY
#error DATAACCESSOR_DESTROY not defined by makefile. Example DATAACCESSOR_DESTROY=hipda32_destroy
#endif


//---------------------------------------------------------------------
// Function Prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Data Accessor Interface send command exported function
 * Receives a name-value-pair string and marshals the command request 
 * to the appropriate function. This is the entry function to the data 
 * accessor that processes commands based on a set of name value pairs 
 * (NVP). The return value is an XML containing the requested data and 
 * status information. Any errors preventing the generation of the XML 
 * returns NULL. Some errors including the data accessor in undefined 
 * state may return an invalid XML. Always release the returned memory 
 * with DATAACCESSOR_FREEDATA
 *
 * @param numNVPair [in] The number of items in the name-value-pair 
 *        string ppNVPair
 * 
 * @param ppNVPair [in] The name-value-pair string
 * 
 * @return On success, returns the result XML string. Use the free data 
 *         interface to release memory returned. On error, returns NULL 
 *         or XML string containing error information. 
 *
 * @see DATAACCESSOR_FREEDATA
 */
//---------------------------------------------------------------------
SMINTF astring * _cdecl DATAACCESSOR_SENDCMD(_IN s32 numNVPair, 
											 _IN astring **ppNVPair);


//---------------------------------------------------------------------
/** Data Accessor Interface free data exported function
 * Releases the heap memory returned from the send command function.  
 * The behavior is undetermined if the reference value provided did 
 * not come from the send command.
 *
 * @param pData [in] Pointer to the heap memory returned from the 
 *        send command function
 * 
 * @return none
 *
 * @see DATAACCESSOR_SENDCMD
 */
//---------------------------------------------------------------------
SMINTF void _cdecl DATAACCESSOR_FREEDATA(_IN astring *pData);


//---------------------------------------------------------------------
/** Data Accessor Interface destroy exported function
 * Performs necessary cleanup, the scope of the cleanup is 
 * implementation specific. Implementation and/or exposure of this
 * function is optional.
 *
 * @return none
 */
//---------------------------------------------------------------------
SMINTF void _cdecl DATAACCESSOR_DESTROY(void);


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCEDAPI_H_INCLUDED__
