
//-------------------------------------------------------------------------
//
//                 Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 19 $
//
//  Last Modified By / On:
//    $Author: Chris_poblete $ / $Date: 8/18/03 5:58p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Native Data Exchange public Definitions
//
//  See Also:
//    Please see Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

#ifndef __DCEDAPLUGINDEF_H_INCLUDED__
#define __DCEDAPLUGINDEF_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcndxdef.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Structure definitions
//---------------------------------------------------------------------

typedef void HMapDB;


typedef struct _StrHeader
{
	const astring *pStr;
} StrHeader;


typedef struct _StrBitmapu32
{
	const astring *pStr; // bit as string
	u32 bitValue; // bit value
} StrBitmapu32;


typedef struct _ObjListContainer
{
	u32 listCount; // number of elements in the list
	u32 objUnitSize; // unit size of the object
	// object list memory starts here
	// memory size of this object is 
	// ((listCount * objUnitSize) + sizeof(ObjListContainer))
	// the object size must be aligned properly for IA64 support
} ObjListContainer;


typedef struct _BitmapObj
{
	astring *pBitName;
	u8 bitPosition;
	u8 reservedAlign[3]; // alignment for next data segment
} BitmapObj;


typedef struct _RRObjTypeList
{
	u16 objType;
	booln bEnabled;
} RRObjTypeList;

typedef struct _RRLogParamList
{
	astring *pParamName;
	astring *pParamObjNameMap;
	u16 logMode;
	u8 reservedAlign[2]; // alignment for next data segment
} RRLogParamList;

typedef struct _SMRRspObj
{
	astring *pRspObjName;
	astring *pRspFollowupVar;
	booln bRspSDO;
	u8 reservedAlign[2]; // alignment for next data segment
} SMRRspObj;

typedef struct _SMRRLogObj
{
	ObjListContainer *pLogParamList; // RRLogParamList
	u32 logEventID;
	u16 logCategory;
	u16 logTypeOnErr;
	u16 logTypeOnSuc;
	booln bLogEnabled;
} SMRRLogObj;

typedef struct _SMReqRsp
{
	SMRRspObj *pRspObj;
	SMRRLogObj *pLogObj;
	ObjListContainer *pObjTypeList; // RRObjTypeList
	astring *pRRID; // INDEX
	astring *pDescription;
	astring *pDAPluginName;
	astring *pReqObjName;
	astring *pReqFollowupVar;
	astring *pReqFollowupVarDefVal;
	u32 reqType;
	s16 daobjStatus; 
	booln bReqSDO;
	booln bIsPassThru;
} SMReqRsp;


typedef struct _DOReqRspData
{
	_IN const SMReqRsp *pSRR; // the req rsp property object
	_IN NDXNamespace anchorNS; // user provided namespace
	_IN u32 numIterate; // iteration count for followup request processing
	_IN u16 productID; // product identifier of pSRR
	_IN u8 reservedAlign[2]; // alignment for next data segment
} DOReqRspData;


typedef struct _DBAccessData
{
	const void *pThreadID; // calling thread id
	HMapDB *pDirectory; // database
	s8 dbAccess; // access state
	u8 reservedAlign[3]; // alignment for next data segment
} DBAccessData;

typedef struct _DAReqRsp 
{
	_IN astring **ppNVPair; // NVP data array
	_OUT SMXMLStrBuf *pXMLBuf; // XML buffer
	_PRIV SMXMLStrBuf *pAttrBuf; // XML attribute buffer
	_PRIV SMXMLStrBuf *pErrorBuf; // XML error buffer
	_PRIV astring *pSBuf512; // all purpose string buffer
	_PRIV DBAccessData dad; // database access data
	_IN SMXGValOpts xvo; // XML option
	_IN s32 numNVPair; // NVP array size
	_PRIV s32 sizeSBuf512; // all purpose string buffer size
	_IN u8 userErrLvl; // user error level 
	_PRIV u8 reservedAlign[3]; // alignment for next data segment
} DAReqRsp;


typedef struct _PluginDADispatchTable
{
	// ----------------------------------------------------------------
	// Free memory returned by functions in this table
	// ----------------------------------------------------------------

	void (_cdecl *PluginDAFreeGeneric)(_IN void *pData);

	// ----------------------------------------------------------------
	// Generic utilities
	// ----------------------------------------------------------------

	u32 (_cdecl *SizeofSMVType)(_IN u8 smvType);

	u32 (_cdecl *SizeofSDOType)(_IN u8 sdoType);

	// ----------------------------------------------------------------
	// XML utilities
	// ----------------------------------------------------------------

	void (_cdecl *DAXMLAddSMStatusEx)(_IN DAReqRsp *pDRR,
		_OUT SMXMLStrBuf *pXMLBuf,
		_IN s32 smStatus);

	// ----------------------------------------------------------------
	// NVP utilities
	// ----------------------------------------------------------------

	u8 (_cdecl *NVPGet_u8)(_IN s32 numNVPair,
		_IN astring **ppNVPair,
		_IN const astring *pNVPName,
		_IN u8 defaultValue);

	booln (_cdecl *NVPGet_booln)(_IN s32 numNVPair,
		_IN astring **ppNVPair,
		_IN const astring *pNVPName,
		_IN booln defaultValue);

	s32 (_cdecl *NVPGet_s32)(_IN s32 numNVPair,
		_IN astring **ppNVPair,
		_IN const astring *pNVPName,
		_IN s32 defaultValue);

	u32 (_cdecl *NVPGet_u32)(_IN s32 numNVPair,
		_IN astring **ppNVPair,
		_IN const astring *pNVPName,
		_IN u32 defaultValue);

	s32 (_cdecl *NVPRGetBitmapu8)(_IN DAReqRsp *pDRR,
		_IN const astring *pObjectName,
		_IN const astring *pVarName,
		_IN u16 objTypeOpt,
		_INOUT u8 *pValue);

	// ----------------------------------------------------------------
	// Command Logging
	// ----------------------------------------------------------------

	astring * (_cdecl *GetXMLLogPathFileName)(_IN const DBAccessData *pDAD,
		_IN u16 productID);

	s32 (_cdecl *DAAppendToXMLLog)(_IN const DBAccessData *pDAD,
		_IN const astring *pPathFileName, 
		_IN u16 type, 
		_IN u16 category, 
		_IN u32 eventID, 
		_IN const astring *pUserInfo, 
		_IN const astring *pSource, 
		_IN const astring *pEventDesc, 
		_IN const void *pData, 
		_IN u32 dataBufSize, 
		_IN u32 maxLogSize);

	s32 (_cdecl *SetAppendCmdLog)(_OUT SMXMLStrBuf *pXMLBuf,
		_IN SMXMLStrBuf *pAttrBuf,
		_IN const astring *pParamName, 
		_IN const void *pOldVal,
		_IN u32 oldValSize,
		_IN const void *pNewVal,
		_IN u32 newValSize,
		_IN u8 smvtype);

	void (_cdecl *BitmapPropertyAppendCmdLog)(_IN DAReqRsp *pDRR,
		_IN u32 oldValue,
		_IN u32 newValue,
		_IN const astring *pObjectName,
		_IN const astring *pVarName,
		_IN u16 objTypeOpt,
		_INOUT SMXMLStrBuf *pXMLBuf);
	
	void (_cdecl *BitmapSetAppendCmdLog)(_IN DAReqRsp *pDRR,
		_IN u32 oldValue,
		_IN u32 newValue,
		_IN StrBitmapu32 *pBMS,
		_IN u32 bmsCount,
		_INOUT SMXMLStrBuf *pXMLBuf);

	// ----------------------------------------------------------------
	// DA Configuration
	// ----------------------------------------------------------------

	const ObjListContainer * (_cdecl *HResolveBitmapPropertyBitmapList)(_IN DBAccessData *pDAD,
		_IN const astring *pObjName,
		_IN const astring *pFieldName,
		_IN u16 objTypeOpt);

	const astring * (_cdecl *DResolveFieldIDToFieldName)(_IN const DBAccessData *pDAD,
		_IN u16 fieldID);

} PluginDADispatchTable;


typedef struct _DAPluginReqRsp
{
	_IN const astring *pCmdName;
	_IN const astring *pUserName;
	_IN const PluginDADispatchTable *pPDF;
	_IN DOReqRspData *pRRData;
	_INOUT DAReqRsp *pDRR;
} DAPluginReqRsp;


//---------------------------------------------------------------------
// Function Signatures
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCEDAPLUGINDEF_H_INCLUDED__
