
//--------------------------------------------------------------------
//
//          DELL INCORPORATED PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Incorporated and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 1 $
//
//  Last Modified By / On:
//    $Author: Tushar_Oza $ / $Date: 11/18/03 3:08a $
//
//  Author / Created:
//    Tushar Oza / 2003 Nov 18
//
//  Environment:
//    Linux, Netware, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//--------------------------------------------------------------------

/** @file */


#ifndef __DCEPEMDISPATCHDEF_H_INCLUDED__
#define __DCEPEMDISPATCHDEF_H_INCLUDED__


//--------------------------------------------------------------------
//  Includes this file is dependent on
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// SDK files
//--------------------------------------------------------------------
#include <dcossdk.h>


//--------------------------------------------------------------------
// Standard C include files
//--------------------------------------------------------------------
#include <limits.h>


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>
#include <dcsmepim.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// DEFINITIONS
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Basic log tag name to be used by all event implementers 
 * that require to commit events to OS/XML logs. 
 */
//---------------------------------------------------------------------
#define EPM_OS_LOG_TAG						"epmoslogtag"


//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Event Plug-in Implementer(EPI) to Event Plug-in Manager(EM) invoke API functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Event Implementer to Event Manager dispatch interface
 * This interface structure is to used to implement an API exposed by
 * the event plug-in manager to the event plug-in implementer on SM_EPDRT_EPI_LOAD
 * This interface allows the event plug-in implementer to invoke services 
 * exported within the event plug-in manager
 */
//---------------------------------------------------------------------
struct _SMEPEMDispatchTable
{

	//-----------------------------------------------------------------
	// Interface version information
	//-----------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Gets Major Version of this interface.
	 * This API is called to obtain the major version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Major version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetMajorVersion)(void);


	//---------------------------------------------------------------------
	/** Gets Minor Version of this interface.
	 * This API is called to obtain the minor version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Minor version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetMinorVersion)(void);


	//---------------------------------------------------------------------
	/** Gets Revision Version of this interface.
	 * This API is called to obtain the revision version 
	 * number. This can be used to perform version control for the 
	 * module and to determine which features are supported.
	 *
	 * @return Revision version of the interface.
	 *
	*/
	//---------------------------------------------------------------------
    u32 (_cdecl *GetRevision)(void);


	//-----------------------------------------------------------------
	// Event Plug-in Implementer information API function prototypes
	//-----------------------------------------------------------------

	//---------------------------------------------------------------------
	/** Interface heap memory free funtion.
	 * This API is called to free dynamically allocated memory. This 
	 * is used to free memory returned from any API which does NOT have a
	 * type set to "const".
	 * @note The caller cannot use the C-Lib "free" API method to release 
	 * memory returned by this module. 
	 * The memory returned from this module may not be compatible with 
	 * the C-Lib "free" API method and may lead to undefined execution.
	 *
	 * @param pMem [in] Specifies a pointer to a previously returned memory 
	 * that is to be freed.
	 *
	*/
	//---------------------------------------------------------------------
    void (_cdecl *FreeGeneric)(_IN void *pMem);


	//---------------------------------------------------------------------
	/** This method is called to get the Event Source Name
	 *
	 * @return Pointer to Event Source Name UTF-8 string.
	 * Do not use FreeGeneric or C-Lib "free" API method
	 * to free returned memory when done with use. This is a const memory
	 *
	 * @see FreeGeneric
	 */
	//---------------------------------------------------------------------
	const astring * (_cdecl *GetEventSourceName)(void);


	//---------------------------------------------------------------------
	/** This method is called to return to the EPI a unique ID number.
	 * This API is called to obtain the unique identifier assigned to this 
	 * Event Plugin Implementer by the Event Plugin Manager.
	 *
	 * @param pEPEMDE [in] Specifies the environmental pointer submitted to 
	 * the Event Plugin Implementer during an SM_EPDRT_EPI_LOAD request.
	 *
	 * @return EPI ID on success. EVTIMP_PLUGIN_ID_PUBLIC on error.
	 *
	 */
	//---------------------------------------------------------------------
	u32 (_cdecl *GetepiID)(_IN const SMEPEMDispatchEnv *pEPEMDE);


	//---------------------------------------------------------------------
	/** This method is called to return a NULL-separated-value (NSV) of OS locale language names.
	 * The returned string is UTF-8 Double-Null-Terminated (DNT) value
	 * The default is 'en_US'. This is first in the NULL-separated-value (NSV).
	 * The OS Locale is appended to the list only if not same as default
	 *
	 * @param pUTF8BufSize [in/out] Specifies the output buffer size on return
	 * from the call. This return size can be used by the caller to parse the
	 * NSV UTF8 returned string.
	 *
	 * @return NULL-separated-value (NSV) of OS locale language names.
	 *
	 * @see FreeGeneric
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *GetOSLocaleNameUTF8List)(_INOUT u32 *pUTF8BufSize);


	//---------------------------------------------------------------------
	/** Allocate and initializes the Event Message Data structure.
	 * This API method will allocate and return a pointer to the 
	 * Event Message Data structure that can be used.
	 * The Event Message Data structure returned should be freed using
	 * FreeEventMessageData.
	 * This API will return NULL if the memory could not be allocated.
 	 * @note Do NOT call FreeGeneric or any free on the returned memory 
	 * since it is allocated as a special structure.
	 * Event Plugin Implementers fill in this structure and pass
	 * it to the LogEventDataByLogTagName routine for
	 * EPM_OS_LOG_TAG log tag type
	 *
	 * @param memAllocSize [in] A variable that receives the
	 * size in bytes of the memory to be allocated.
	 *
	 * @return pointer to Event Message Data structure whose memory has 
	 * been allocated or NULL if memory could not be allocated.
	 *
	 * @see FreeEventMessageData, LogEventDataByLogTagName
	 *
	 */
	//---------------------------------------------------------------------
	EventMessageData * (_cdecl *AllocEventMessageData)(_IN u32 memAllocSize);


	//---------------------------------------------------------------------
	/** Free the Event Message Data Structure reset the memory to NULL pointers.
	 * This API is called to free dynamically allocated EventMessageData memory.
	 * This is used to free memory returned from AllocEventMessageData API.
	 * @note The caller cannot use the C-Lib "free" API or the FreeGeneric API
	 * method to release memory returned by AllocEventMessageData API. 
	 *
	 * @param pEMD [in] Specifies a pointer to a previously returned EventMessageData 
	 * memory that is to be freed.
	 *
	 * @see AllocEventMessageData
	*/
	//---------------------------------------------------------------------
	void (_cdecl *FreeEventMessageData)(_IN EventMessageData *pEMD);


	//---------------------------------------------------------------------
	/** This method is called by the EPI to register a call-back service routine.
	 * E.g.: committing to OS Logs, Appending to SEL logs etc.
	 *
	 * @param pUTF8LogTag [in] Specifies the input buffer that contains the
	 * UTF8 logtagname.
	 *
	 * @param pfnLogRoutine [in] Callback Function that is 
	 * to receive event message data. The function should 
	 * return SM_STATUS_SUCCESS if the event data should continue to be 
	 * passed on to other registered callback functions. The callback function should 
	 * return status other than SM_STATUS_SUCCESS if the event data should
	 * not be passed on to other registered functions.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns 
	 * the status code. SM_STATUS_INVALID_PARAMETER implies invalid 
	 * argument passed to API function. SM_STATUS_UNSUCCESSFUL indicates
	 * specified callback function failed to register. Refer to dcsmstatus.h
	 * for all other appropriate return codes.
	 *
	 * @see LogTagNameRemove
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *LogTagNameInsert)(_IN const astring *pUTF8LogTag, 
		_IN const PFNLOGROUTINE pfnLogRoutine);


	//---------------------------------------------------------------------
	/** This method is called to return a NULL-separated-value (NSV) of registered log tag names.
	 * The returned string is UTF-8 Double-Null-Terminated (DNT) value
	 * Number of available logtags is returned in pNumLogTags
	 *
	 * @param pUTF8BufSize [in/out] Specifies the output buffer size on return
	 * from the call. This return size can be used by the caller to parse the
	 * NSV UTF8 returned string.
	 *
	 * @param pNumLogTags [in/out] Specifies the number of logtags in the returned buffer
	 *
	 * @return NULL-separated-value (NSV) of registered log tag names.
	 *
	 * @see FreeGeneric
	 */
	//---------------------------------------------------------------------
	astring * (_cdecl *LogTagNameGetUTF8List)(_INOUT u32 *pUTF8BufSize, 
											_INOUT u32 *pNumLogTags);


	//---------------------------------------------------------------------
	/** This method is called by the EPI to deregister a call-back service routine.
	 * E.g.: committing to OS Logs, Appending to SEL logs etc.
	 *
	 * @param pUTF8LogTag [in] Specifies the input buffer that contains the
	 * UTF8 logtagname to be removed.
	 *
	 * @param pfnLogRoutine [in] Callback Function that is 
	 * to receive event message data. The function should 
	 * return SM_STATUS_SUCCESS if the event data should continue to be 
	 * passed on to other registered callback functions. The callback function should 
	 * return status other than SM_STATUS_SUCCESS if the event data should
	 * not be passed on to other registered functions.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns 
	 * the status code. SM_STATUS_INVALID_PARAMETER implies invalid 
	 * argument passed to API function. SM_STATUS_UNSUCCESSFUL indicates
	 * specified callback function failed to register. Refer to dcsmstatus.h
	 * for all other appropriate return codes.
	 *
	 * @see LogTagNameInsert
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *LogTagNameRemove)(_IN const astring *pUTF8LogTag, 
								_IN const PFNLOGROUTINE pfnLogRoutine);


	//---------------------------------------------------------------------
	/** This method is called by the EPI to log events. Needs to supply tagname and data.
	 * E.g.: committing to OS Logs, Appending to SEL logs etc.
	 * This is an asynchronous method. It is implementers responsibility to make
	 * sense of the pData (implementer implies the registered callback function)
	 *
	 * @param pUTF8LogTag [in] Specifies the input buffer that contains the
	 * UTF8 logtagname to be removed.
	 *
	 * @param pLRDData [in] Log routine callback Function that is 
	 * to receive event message data. The function should 
	 * return SM_STATUS_SUCCESS if the event data should continue to be 
	 * passed on to other registered callback functions. The callback function should 
	 * return status other than SM_STATUS_SUCCESS if the event data should
	 * not be passed on to other registered functions.
	 *
	 * @return On success returns SM_STATUS_SUCCESS. On error returns 
	 * the status code. SM_STATUS_INVALID_PARAMETER implies invalid 
	 * argument passed to API function. 
	 * Refer to dcsmstatus.h for all other appropriate return codes.
	 *
	 * @see LogTagNameInsert, AllocEventMessageData, FreeEventMessageData
	 */
	//---------------------------------------------------------------------
	s32 (_cdecl *LogEventDataByLogTagName)(_IN const astring *pUTF8LogTag, 
		_IN LogRoutineData *pLRDData);


}; // _SMEPEMDispatchTable



#ifdef __cplusplus
}
#endif


#endif //  __DCEPEMDISPATCHDEF_H_INCLUDED__

