
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2006 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Generic object definitions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

/** @file */

#ifndef __DCGENOBJ_H_INCLUDED__
#define __DCGENOBJ_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcdataobj.h>


#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(1)


//---------------------------------------------------------------------
// Definitions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Generic Object Types
//---------------------------------------------------------------------

/** Event Filter Configuration object (GenEvtFilterCfgObj) */
#define GEN_OBJ_TYPE_EVT_FILTER_CFG		(DATA_OBJ_TYPE_GENERIC_LO + 0x0)

// NOTE: DATA_OBJ_TYPE_GENERIC_LO + 0xFF is maximum in Generic range


//---------------------------------------------------------------------
// Limits
//---------------------------------------------------------------------

/** Max size in bytes of a Generic object (GenObject) */
#define MAX_GEN_OBJECT_SIZE				(MAX_DATA_OBJECT_SIZE)


//---------------------------------------------------------------------
// Structures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 
 * GenEvtFilterCfgObj:
 * Data object body of GEN_OBJ_TYPE_EVT_FILTER_CFG
 */
//---------------------------------------------------------------------
typedef struct _GenEvtFilterCfgObj
{
	/** Reserved for future use set to 0 */
	u32 reserved;
} GenEvtFilterCfgObj;
 

//---------------------------------------------------------------------
/** 
 * GenObject:
 * Generic data object
 */
//---------------------------------------------------------------------
typedef struct _GenObject
{
	/** Data object header */
	DataObjHeader objHeader;

	/** Data object body */
	union _GenObjectBody
	{
		/** Start of data buffer after header */
		u8 byte;

		/** Event filter configuration object */
		GenEvtFilterCfgObj evtFilterCfgObj;

		/** Anchors the size of this structure */
		// NOTE: This union element anchors the 
		// size of this structure and should not
		// be changed without clear understanding
		// NOTE: We cannot enable this size here
		// since the some bytes are used by
		// surrounding structures like ObjStatusChange
	    // _PRIV u8 sizeAnchor[MAX_GEN_OBJECT_SIZE - sizeof(DataObjHeader) - 20];
		// NOTE: Please read info about sizeAnchor
	} objBody;
} GenObject;


//---------------------------------------------------------------------
// End Definitions
//---------------------------------------------------------------------


#pragma pack()

#ifdef __cplusplus
}
#endif


#endif // __DCGENOBJ_H_INCLUDED__

