
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2006 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Generic object request definitions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

/** @file */

#ifndef __DCGENREQ_H_INCLUDED__
#define __DCGENREQ_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcdataevent.h>
#include <dcdataobj.h>
#include <dcsmdispatchdef.h>


#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(1)


//---------------------------------------------------------------------
// Definitions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Generic Set Types
//---------------------------------------------------------------------

/** Set type for setting event filter */
#define GEN_ST_SET_EVT_FILTER			(SM_SET_TYPE_GENERIC_LO + 0x0)

// NOTE: SM_SET_TYPE_GENERIC_LO + 0x3FF is maximum in Generic range


//---------------------------------------------------------------------
// Generic Pass Thru Types
//---------------------------------------------------------------------

// NOTE: SM_PASSTHRU_TYPE_GENERIC_LO + 0x3FF is maximum in Generic range


//---------------------------------------------------------------------
// Limits
//---------------------------------------------------------------------

/** Max size in bytes of Generic request/reponse buffer */
#define MAX_GEN_REQRSP_BUF_SIZE			(MAX(MAX_DATA_OBJECT_SIZE, MAX_DATA_EVENT_SIZE))

/** Max size in bytes of event filter value to be stored into an ini file including the NULL terminator */
#define MAX_EVTFILTER_INI_VALUE_SIZE	(1024) 


//---------------------------------------------------------------------
// Structures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * GenObjSetReq:
 * Parameters passed on Generic object set request
 * (SM_DRT_DMG_SET_OBJ_BY_OID)
 */
//---------------------------------------------------------------------
typedef struct _GenObjSetReq
{
	/** Set request header */
	SMReqHeaderSet setHdr;

	/** Set request data */
	union _GenObjSetReqData
	{
		/** Parameters passed on GEN_SET_EVT_FILTER */
		struct _GenSetEvtFilter
		{
			/** Event filter INI section name */
			astring sectionName[MAX_INI_SECKEY_NAME_SIZE];
			/** Event filter INI key name */
			astring keyName[MAX_INI_SECKEY_NAME_SIZE];
			/** Event filter INI value */
			astring value[MAX_EVTFILTER_INI_VALUE_SIZE];
		} evtFilter;
	} setData;
} GenObjSetReq;


//---------------------------------------------------------------------
/**
 * GenObjPassThru:
 * Parameters passed on Generic object pass thru request
 * (SM_DRT_COM_PASS_THRU_BY_OID)
 */
//---------------------------------------------------------------------
typedef struct _GenObjPassThru
{
	/** Pass thru request header */
	SMReqHeaderPassThru ptHdr;

	/** Pass thru request data */
	union _GenObjPassThruData
	{
		/** Anchors the size of this structure */
    	_PRIV u8 sizeAnchor[MAX_GEN_REQRSP_BUF_SIZE - sizeof(SMReqHeaderPassThru)];
   	} ptData;
} GenObjPassThru;


//---------------------------------------------------------------------
/**
 * GenApiReq:
 * Request structure for Generic API requests
 */
//---------------------------------------------------------------------
typedef struct _IN _GenApiReq
{
	/** Request data */
	union _GenApiReqData
	{
		/** Parameters passed on SM_DRT_DMG_SET_OBJ_BY_OID */
		GenObjSetReq setReq;

		/** Parameters passed on SM_DRT_COM_PASS_THRU_BY_OID */
		GenObjPassThru passThru;

		/** Anchors the size of this structure */
	    _PRIV u8 sizeAnchor[MAX_GEN_REQRSP_BUF_SIZE];
	} reqData;
} GenApiReq;


//---------------------------------------------------------------------
/**
 * GenApiRsp:
 * Response structure for Generic API requests
 */
//---------------------------------------------------------------------
typedef struct _OUT _GenApiRsp
{
	/** Response data */
	union _GenApiRspData
	{
		/** Anchors the size of this structure */
	    _PRIV u8 sizeAnchor[MAX_GEN_REQRSP_BUF_SIZE];
	} rspData;
} GenApiRsp;


//---------------------------------------------------------------------
// End Definitions
//---------------------------------------------------------------------


#pragma pack()

#ifdef __cplusplus
}
#endif


#endif // __DCGENREQ_H_INCLUDED__

