
//-------------------------------------------------------------------------
//
//                  Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCISOCPPSECURECOMPAT_H_INCLUDED__
#define __DCISOCPPSECURECOMPAT_H_INCLUDED__

//--------------------------------------------------------------------
// This file is not for Windows environment
//--------------------------------------------------------------------
#if !defined(SYSTEM_TYPE_WINNT) || defined(_IA64_)

//-------------------------------------------------------------------------
//  Includes this file is dependent on
//-------------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------------
// POSIX functions deprecated by Windows compiler: 
// When using common POSIX function names and use Windows compiler you may
// find documentation warning such as "This POSIX function is deprecated
// beginning in Visual C++ 2005. Use the ISO C++ conformant _getch instead."
//---------------------------------------------------------------------------
// To allow "common code" development that utilizes Windows' POSIX
// functions, the following alias definitions are provided for non-Windows
// environment. To the left is Windows' POSIX functions while to the right 
// is the common POSIX function name.
//---------------------------------------------------------------------------

#define _chmod 			chmod
#define _access 		access
#define _unlink			unlink
#define _getch			getch
#define _stricmp		stricmp
#define _close			close
#define _tzset			tzset


//---------------------------------------------------------------------------
// STRING functions deprecated by Windows compiler:
// When using functions that contain a string as parameter such as strcpy
// and use Windows compiler you may find documentation warning such as
// "These functions are deprecated because more secure versions are 
// available..."
//---------------------------------------------------------------------------
// To allow "common code" development that utilizes Windows' secure
// "safe string" functions, the following function prototypes are provided
// for non-Windows environment. These functions do not invoke an invalid
// parameter handler similar to the Windows' functions.
//---------------------------------------------------------------------------

// Functions that have no change in arguments
#define sscanf_s		sscanf

#if defined(_UNICODE)
#define _tcscat_s		wcscat_s
#define _tcscpy_s		wcscpy_s
#define _tgetenv_s		_wgetenv_s
#define _stprintf_s		swprintf_s
#else
#define _tcscat_s		strcat_s
#define _tcscpy_s		strcpy_s
#define _tgetenv_s		getenv_s
#define _stprintf_s		sprintf_s
#endif

// Size constant declarations used by functions below
#define TIME_STRING_BUF_LEN			26

// Type definitions used by functions below
typedef int	errno_t;


//---------------------------------------------------------------------
// strcpy_s
//---------------------------------------------------------------------
errno_t _cdecl strcpy_s(_OUT char *strDest,
						_IN size_t sizeInBytes,			//added: size of strDest
						_IN const char *strSource
						);

//---------------------------------------------------------------------
// wcscpy_s
//---------------------------------------------------------------------
errno_t _cdecl wcscpy_s(_OUT wchar_t *strDestination,
						_IN size_t sizeInWords,			//added: size of strDestination
						_IN const wchar_t *strSource
						);

//---------------------------------------------------------------------
// strcat_s
//---------------------------------------------------------------------
errno_t _cdecl strcat_s(_INOUT char *strDestination,
						_IN size_t sizeInBytes,			//added: size of strDestination
						_IN const char *strSource
						);

//---------------------------------------------------------------------
// wcscat_s
//---------------------------------------------------------------------
errno_t _cdecl wcscat_s(_INOUT wchar_t *strDestination,
						_IN size_t sizeInWords,			//added: size of strDestination
						_IN const wchar_t *strSource
						);

//---------------------------------------------------------------------
// strncat_s
//---------------------------------------------------------------------
errno_t _cdecl strncat_s(_OUT char *strDestination,
						 _IN size_t sizeInBytes,		//added: size of strDestination
   						 _IN const char *strSource,
						 _IN size_t count);

//---------------------------------------------------------------------
// wcsncat_s
//---------------------------------------------------------------------
errno_t _cdecl wcsncat_s(_INOUT wchar_t *strDest,
						 _IN size_t bufferSizeInWords,	//added: size of bufferSizeInWords
						 _IN const wchar_t *strSource,
						 _IN size_t count
						 );

//---------------------------------------------------------------------
// strncpy_s
//---------------------------------------------------------------------
errno_t _cdecl strncpy_s(_OUT char *strDest,
						 _IN size_t sizeInWords,		//added: size of strDest
						 _IN const char *strSource,
						 _IN size_t count
						 );

//---------------------------------------------------------------------
// wcsncpy_s
//---------------------------------------------------------------------
errno_t _cdecl wcsncpy_s(_OUT wchar_t *strDest,
   						 _IN size_t sizeInWords,		//added: size of sizeInWords
   						 _IN const wchar_t *strSource,
						 _IN size_t count
						 );

//---------------------------------------------------------------------
// strtok_s
//---------------------------------------------------------------------
char* _cdecl strtok_s(_INOUT char *strToken, 
					  _IN const char *strDelimit, 
					  _INOUT char **context				//added: makes possible to parse two strings simultaneously in the same thread
					  );

//---------------------------------------------------------------------
// wcstok_s
//---------------------------------------------------------------------
wchar_t* _cdecl wcstok_s(_INOUT wchar_t *strToken,
						 _IN const wchar_t *strDelimit, 
						 _INOUT wchar_t **context		//added: makes possible to parse two strings simultaneously in the same thread
						 );

//---------------------------------------------------------------------
// _wcsset_s
//---------------------------------------------------------------------
errno_t _cdecl _wcsset_s(_OUT wchar_t *str,
						 _IN size_t sizeInWords,		//added: size of str
						 _IN wchar_t c
						 );

//---------------------------------------------------------------------
// _wcsnset_s
//---------------------------------------------------------------------
errno_t _cdecl _wcsnset_s(_OUT wchar_t *str,
						  _IN size_t sizeInWords,		//added: size of str
						  _IN wchar_t c,
						  _IN size_t count
						  );

//---------------------------------------------------------------------
// _ultoa_s
//---------------------------------------------------------------------
errno_t _ultoa_s(_IN unsigned long value,
				 _OUT char *str,
				 _IN size_t sizeOfstr,					//added: size of str
				 _IN int radix
				 );

//---------------------------------------------------------------------
// _ltoa_s
//---------------------------------------------------------------------
errno_t _cdecl _ltoa_s(_IN long value,
					   _OUT char *str,
					   _IN size_t sizeOfstr,			//added: size of str
					   _IN int radix
					   );

//---------------------------------------------------------------------
// _snprintf_s
//---------------------------------------------------------------------
int _cdecl _snprintf_s(_OUT char* pBuffer, 
					   _IN size_t sizeOfBuffer,			//added: size of pBuffer
					   _IN size_t count, 
					   _IN const char* pformat, 
					   _IN ...
					   );

//---------------------------------------------------------------------
// sprintf_s
//---------------------------------------------------------------------
int _cdecl sprintf_s(_OUT char *pBuffer, 
					 _IN size_t sizeInBytes,			//added: size of pBuffer
					 _IN const char *pFormat, 
					 _IN ... );

//---------------------------------------------------------------------
// swprintf_s
//---------------------------------------------------------------------
int _cdecl swprintf_s(_OUT wchar_t *pBuffer, 
					  _IN size_t sizeInWords,			//added: size of pBuffer
					  _IN const wchar_t *pFormat, 
					  _IN ... );

//---------------------------------------------------------------------
// tmpnam_s
//---------------------------------------------------------------------
errno_t _cdecl tmpnam_s(_OUT char * pBuffer, 
						_IN size_t sizeInBytes			//added: size of pBuffer
						);

//---------------------------------------------------------------------
// ctime_s
//---------------------------------------------------------------------
errno_t _cdecl ctime_s(_OUT char* pBuffer, 
					   _IN size_t sizeInBytes,			//added: size of pBuffer
					   _IN const time_t* ptime
					   );

//---------------------------------------------------------------------
// asctime_s
//---------------------------------------------------------------------
errno_t _cdecl asctime_s(_OUT char* pBuffer, 
						 _IN size_t sizeInBytes,		//added: size of pBuffer
						 _IN const struct tm* ptm
						 );

//---------------------------------------------------------------------
// fopen_s
// NOTE: fopen returns a pointer which can be checked for failure
//       using NULL. NULL is mapped to 0. However, in fopen_s the
//       return is errno_t which treat 0 as success.
//---------------------------------------------------------------------
errno_t _cdecl fopen_s(_OUT FILE** ppFile,				//added: return value
					   _IN const char* pFilename, 
					   _IN const char* pMode
					   );

//---------------------------------------------------------------------
// getenv_s
//---------------------------------------------------------------------
errno_t _cdecl getenv_s(_OUT size_t *pReturnValue,		//added: The buffer size needed, or 0 if the variable is not found.
						_OUT char *pBuffer,				//added: return value
						_IN size_t sizeInBytes,			//added: size of pBuffer
						_IN const char *pVarname);

//---------------------------------------------------------------------
// _wgetenv_s
//---------------------------------------------------------------------
errno_t _cdecl _wgetenv_s(_OUT size_t *pReturnValue,	//added: The buffer size needed, or 0 if the variable is not found.
						  _OUT wchar_t *pBuffer,		//added: return value
						  _IN size_t sizeInWords,		//added: size of pBuffer in words
						  _IN const wchar_t *pVarname);

//---------------------------------------------------------------------
// _dupenv_s
// NOTE: Varname not found not an error. Buffer and size cleared, returns zero
//---------------------------------------------------------------------
errno_t _cdecl _dupenv_s(_OUT char** ppBuffer,			//added: return value
						 _OUT size_t* pSizeInBytes,		//added: size of return value
						 _IN const char* pVarName
						 );

//---------------------------------------------------------------------
// _sopen_s
//---------------------------------------------------------------------
errno_t _cdecl _sopen_s(_OUT int* pfh,					//added: return value
						_IN const char* pFilename, 
						_IN int oflag, 
						_IN int shflag, 
						_IN int pmode);

//---------------------------------------------------------------------
// localtime_s
//---------------------------------------------------------------------
errno_t _cdecl localtime_s(_OUT struct tm *pTM,			//added
						   _IN const time_t *pTime);

//---------------------------------------------------------------------
// _get_timezone
//---------------------------------------------------------------------
errno_t _cdecl _get_timezone(_OUT long *pSeconds);


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif

//--------------------------------------------------------------------
// This file is not for Windows environment
//--------------------------------------------------------------------
#endif // #if !defined(SYSTEM_TYPE_WINNT) || defined(_IA64_)


#endif // __DCISOCPPSECURECOMPAT_H_INCLUDED__

