
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    Revision: 1
//
//  Last Modified By / On:
//    Author: Ryan Brown / Date: 2012 Jul 25
//
//  Author:
//    Data Manager
//
//  Environment:
//    Linux / Windows
//
//  User/Kernel Mode:
//    User
//
//  Abstract/Purpose:
//    Data Manager multi-langugage Message Registry library interface.
//    The registry provides a mapping from a unique identifier with 
//    optional keys to a set of UTF-8 encoded strings. 
//
//  See Also:
//    dcmsgregapi.h
//
//  Notes:
//    For more information refer to Data Manager Message Registry HLD.
//
//--------------------------------------------------------------------

#ifndef __DCMSGREGDEF_H_INCLUDED__
#define __DCMSGREGDEF_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif

//---------------------------------------------------------------------
//  Macro definitions
//---------------------------------------------------------------------

// Range limits

#define MSGREG_MSGID_STR_MAXLEN              (32)
#define MSGREG_SEVERITY_STR_MAXLEN           (32)
#define MSGREG_CATEGORY_STR_MAXLEN           (32)
#define MSGREG_MESSAGE_STR_MAXLEN            (512)
#define MSGREG_RECACTION_STR_MAXLEN          (2048)
#define MSGREG_DETAILEDDESC_STR_MAXLEN       (2048)

// Supported ISO 639-1 language codes

#define MSGREG_LOCALE_CHINESE                "zh"
#define MSGREG_LOCALE_ENGLISH                "en"
#define MSGREG_LOCALE_FRENCH                 "fr"
#define MSGREG_LOCALE_GERMAN                 "de"
#define MSGREG_LOCALE_JAPANESE               "ja"
#define MSGREG_LOCALE_SPANISH                "es"

//---------------------------------------------------------------------
//  Type definitions
//---------------------------------------------------------------------

typedef struct _MessageContent
{
   astring msgID[MSGREG_MSGID_STR_MAXLEN];
   astring severity[MSGREG_SEVERITY_STR_MAXLEN];
   astring category[MSGREG_CATEGORY_STR_MAXLEN];
   astring message[MSGREG_MESSAGE_STR_MAXLEN];   // includes argument substitutions
   astring recommendedAction[MSGREG_RECACTION_STR_MAXLEN];
   astring detailedDesc[MSGREG_DETAILEDDESC_STR_MAXLEN];
} MessageContent;

typedef struct _MessageContentWN
{
   astring msgID[MSGREG_MSGID_STR_MAXLEN];
   u32 severity;
   u32 category;
   astring message[MSGREG_MESSAGE_STR_MAXLEN];   // includes argument substitutions
   astring recommendedAction[MSGREG_RECACTION_STR_MAXLEN];
   astring detailedDesc[MSGREG_DETAILEDDESC_STR_MAXLEN];
} MessageContentWN;

//---------------------------------------------------------------------
// Null-Separated Value (NSV) String
//---------------------------------------------------------------------

// Represents Null-separated value list string in UTF-8 format.
// For example in memory, 'Broadcom\0network\0interface\0\0'

typedef astring nsvastring;

// Represents the object to create Null-separated value list
// string in UTF-8 format.

typedef struct _NSVObj
{
   u32 valuesize;
   u32 usedsize;
   nsvastring *pstrvalue;
} NSVObj;


//---------------------------------------------------------------------
/**
 * Conditional statement to check if the null-terminiated string
 * value is not empty.
 *
 * @param PX [in,required] of type pointer to nsvastring.
 *
 * @sa PNSVSTRING_NEXT
 *
 */
#define PNSVSTRING_ISNOTEMPTY(PX)       (*PX)


//---------------------------------------------------------------------
/**
 * Move the pointer to the next item in the null-terminated string
 * value. An example usage:
 *     nsvastring *palist;
 *     // add value to the list
 *     while (PNSVSTRING_ISNOTEMPTY(palist)) {
 *        // palist contains an element in the list
 *        PNSVSTRING_NEXT(palist);
 *     } // while
 *
 * @param PX [in,required] of type pointer to nsvastring.
 *
 * @sa PNSVSTRING_ISNOTEMPTY
 *
 */
#define PNSVSTRING_NEXT(PX)             PX = (char *)((u8 *)PX + (strlen(PX) + 1))


//---------------------------------------------------------------------
/**
 * Conditional statement to check if the null-separated string list
 * in the object is not empty.
 *
 * @param X [in,required] of type NSVObj.
 *
 */
#define NSVO_ISNOTEMPTY(X)             (X.usedsize > 1)


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif

#endif //__DCMSGREGDEF_H_INCLUDED__
