
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 14 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Native Data Exchange public API
//
//  See Also:
//    Please see Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCNDXAPI_H_INCLUDED__
#define __DCNDXAPI_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <dcndxdef.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Function Prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Convert C memory to SDO by object name
 * NDX function to convert C memory to self-describing object (SDO)
 * by object name. The object name will be used to determine the 
 * configuration data necessary for conversion.
 *
 * @param pCBuffer [in] Pointer to valid C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param pObjName [in] The object name representing pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pSDB [out] Pointer to SDOBinary user memory. May be NULL if 
 *        requesting output size.
 *
 * @param pSDBSize [inout] Pointer to the size in bytes of the pSDB if not 
 *        NULL. It is reset to the actual size of the result value.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeCObjToSDO(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN const astring *pObjName,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);

//---------------------------------------------------------------------
/** Convert C memory to SDO by object type
 * NDX function to convert C memory to self-describing object (SDO)
 * by object type. The object type will be used to determine the 
 * configuration data necessary for conversion.
 *
 * @param pCBuffer [in] Pointer to valid C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param objType [in] The object type representing pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pSDB [out] Pointer to SDOBinary user memory. May be NULL if 
 *        requesting output size.
 *
 * @param pSDBSize [inout] Pointer to the size in bytes of the pSDB if not 
 *        NULL. It is reset to the actual size of the result value.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeCObjTypedToSDO(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);

//---------------------------------------------------------------------
/** Convert DataObjHeader C memory to SDO
 * NDX function to convert DataObjHeader C memory to self-describing 
 * object (SDO). The DataObjHeader includes object type information
 * that will be used to determine the configuration data necessary
 * for conversion.
 *
 * @param pCBuffer [in] Pointer to valid DataObjHeader C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param bIncludeBody [in] Flag to indicate include the body of 
 *        DataObjHeader in the conversion.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pSDB [out] Pointer to SDOBinary user memory. May be NULL if 
 *        requesting output size.
 *
 * @param pSDBSize [inout] Pointer to the size in bytes of the pSDB if not 
 *        NULL. It is reset to the actual size of the result value.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 *
 * @see DataObjHeader
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeDOHToSDO(
				_IN const DataObjHeader *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN booln bIncludeBody,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);


//---------------------------------------------------------------------
/** Convert C memory to XML by object name
 * NDX function to convert C memory to XML by object name. The object 
 * name will be used to determine the configuration data necessary for 
 * conversion.
 *
 * @param pCBuffer [in] Pointer to valid C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param pObjName [in] The object name representing pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pXMLBuf [out] Pointer to the result XML memory in user space.
 *
 * @param pAttrBufOpt [in] Pointer to attribute buffer in user space.
 *        This is optional. If not NULL, this buffer is used as temporary
 *        buffer throughout the conversion process. It prevents multiple
 *        allocations when it is needed.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeCObjToXML(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN const astring *pObjName,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);

//---------------------------------------------------------------------
/** Convert C memory to XML by object type
 * NDX function to convert C memory to XML by object type. The object 
 * type will be used to determine the configuration data necessary for 
 * conversion.
 *
 * @param pCBuffer [in] Pointer to valid C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param objType [in] The object type representing pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pXMLBuf [out] Pointer to the result XML memory in user space.
 *
 * @param pAttrBufOpt [in] Pointer to attribute buffer in user space.
 *        This is optional. If not NULL, this buffer is used as temporary
 *        buffer throughout the conversion process. It prevents multiple
 *        allocations when it is needed.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeCObjTypedToXML(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);

//---------------------------------------------------------------------
/** Convert DataObjHeader C memory to XML
 * NDX function to convert DataObjHeader C memory to XML. The 
 * DataObjHeader includes object type information that will be used 
 * to determine the configuration data necessary for conversion.
 *
 * @param pCBuffer [in] Pointer to valid DataObjHeader C memory.
 *
 * @param sizeCBuffer [in] The size in bytes of pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pXMLBuf [out] Pointer to the result XML memory in user space.
 *
 * @param pAttrBufOpt [in] Pointer to attribute buffer in user space.
 *        This is optional. If not NULL, this buffer is used as temporary
 *        buffer throughout the conversion process. It prevents multiple
 *        allocations when it is needed.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 *
 * @see DataObjHeader
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeDOHToXML(
				_IN const DataObjHeader *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN booln bIncludeBody,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);


//---------------------------------------------------------------------
/** Convert self-describing memory (SDO) to XML by object type.
 * NDX function to convert self-describing memory (SDO) to XML by object
 * type. The object type and field IDs within the SDO are used to 
 * determine the configuration data necessary for conversion.
 *
 * @param pSDB [in] Pointer to valid SDO memory.
 *
 * @param sdbSize [in] The size in bytes of pSDB.
 *
 * @param objType [in] The object type representing pCBuffer.
 *
 * @param pNDXOpts [in] Pointer to NDX options. See definition of NDXOptions.
 *
 * @param pXMLBuf [out] Pointer to the result XML memory in user space.
 *
 * @param pAttrBufOpt [in] Pointer to attribute buffer in user space.
 *        This is optional. If not NULL, this buffer is used as temporary
 *        buffer throughout the conversion process. It prevents multiple
 *        allocations when it is needed.
 * 
 * @return SM_STATUS_SUCCESS on success, specific SM_STATUS code on error.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl NDXchangeSDOToXML(
				_IN SDOBinary *pSDB,
				_IN u32 sdbSize,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);


//---------------------------------------------------------------------
/** Free the memory containing the namespace object
 * NDX function to free the memory containing the namespace object 
 * returned from the functions NDXchangeOIDToONS and NDXchangeONSToOID.
 *
 * @param pXNS [in] Pointer to valid NDXNamespace memory.
 *
 * @see NDXNamespace
 */
//---------------------------------------------------------------------
SMINTF void _cdecl NDXNamespaceFree(_IN NDXNamespace *pXNS);


//---------------------------------------------------------------------
/** Get namespace object information from object ID (OID)
 * NDX function to get the namespace object information from object
 * ID (OID).
 *
 * @param oid [in] object ID
 *
 * @return Pointer to the namespace object
 *
 * @see NDXNamespaceFree
 * @see NDXNamespace
 */
//---------------------------------------------------------------------
SMINTF NDXNamespace * _cdecl NDXchangeOIDToONS(_IN u32 oid);


//---------------------------------------------------------------------
/** Get namespace object information from namespace string
 * NDX function to get the namespace object information from namespace
 * string.
 *
 * @param pNamespace [in] namespace string
 *
 * @return Pointer to the namespace object
 *
 * @see NDXNamespaceFree
 * @see NDXNamespace
 */
//---------------------------------------------------------------------
SMINTF NDXNamespace * _cdecl NDXchangeONSToOID(_IN const astring *pNamespace);


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif



#endif // __DCNDXAPI_H_INCLUDED__
