
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 19 $
//
//  Last Modified By / On:
//    $Author: Chris_poblete $ / $Date: 8/18/03 5:58p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Native Data Exchange public Definitions
//
//  See Also:
//    Please see Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

#ifndef __DCNDXDEF_H_INCLUDED__
#define __DCNDXDEF_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcdataobj.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

// ENV name to reference path to NDX directory
#define ENV_NDX_PATH							"DCNDXPATH"

// CXM file extension
#define EXT_NDX_CXM								".cxm"

// path definitions
#define SUBPATH_NDX								"ndx"
#define FILE_SUFFIX_NDX							"ndx" SM_EXT_INI
#define FILE_SUFFIX_FID							"fid" SM_EXT_INI

// NDX section definitions
#define INI_SEC_NDX_FIDMAP						"FIDMap"
#define INI_SEC_NDX_FIDLOAD						"FIDLoad"
#define INI_SEC_NDX_DAPLUGIN					"DAPlugin"
#define INI_SEC_NDX_OBJTYPETOSTR				"TypeNumToStr"
#define INI_SEC_NDX_OBJTYPETONAME				"ObjTypeToName"
#define INI_SEC_NDX_OBJNAMETOCXM				"ObjNameToCXM"

// NDX registration section
#define INI_SEC_NDX_REGISTRY					"NDXRegistry"


//---------------------------------------------------------------------
// Structure definitions
//---------------------------------------------------------------------

typedef struct _NDXOptions
{
	const astring *pMapPathFileName; // absolute pathfilename to map file
	const astring *pStylePathFileName; // absolute pathfilename to style file
	astring *pSkipObjectListNTSL; // list of objects to skip as NULL-terminated string list
} NDXOptions;


typedef struct _NDXNamespace
{
	astring *pNamespace; // object namespace
	astring *pTypeName; // object type name
	u32 sizeNamespace; // namespace size including NULL terminator
	u32 sizeTypeName; // typename size including NULL terminator
	s32 instance; // object instance
	ObjID objid; // object ID
} NDXNamespace;


//---------------------------------------------------------------------
// Function Signatures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// CObject to SDO
//---------------------------------------------------------------------

#define NDXCHANGECOBJTOSDO_STR  "NDXchangeCObjToSDO"
typedef s32 (_cdecl *FPROCNDXCHANGECOBJTOSDO)(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN const astring *pObjName,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);

#define NDXCHANGECOBJTYPEDTOSDO_STR  "NDXchangeCObjTypedToSDO"
typedef s32 (_cdecl *FPROCNDXCHANGECOBJTYPEDTOSDO)(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);

#define NDXCHANGEDOHTOSDO_STR  "NDXchangeDOHToSDO"
typedef s32 (_cdecl *FPROCNDXCHANGEDOHTOSDO)(
				_IN const DataObjHeader *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN booln bIncludeBody,
				_IN NDXOptions *pNDXOpts,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);


//---------------------------------------------------------------------
// CObject to XML
//---------------------------------------------------------------------

#define NDXCHANGECOBJTOXML_STR  "NDXchangeCObjToXML"
typedef s32 (_cdecl *FPROCNDXCHANGECOBJTOXML)(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN const astring *pObjName,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);

#define NDXCHANGECOBJTYPEDTOXML_STR  "NDXchangeCObjTypedToXML"
typedef s32 (_cdecl *FPROCNDXCHANGECOBJTYPEDTOXML)(
				_IN const void *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);

#define NDXCHANGEDOHTOXML_STR  "NDXchangeDOHToXML"
typedef s32 (_cdecl *FPROCNDXCHANGEDOHTOXML)(
				_IN const DataObjHeader *pCBuffer,
				_IN u32 sizeCBuffer,
				_IN booln bIncludeBody,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);


//---------------------------------------------------------------------
// SDO to XML
//---------------------------------------------------------------------

#define NDXCHANGESDOTOXML_STR  "NDXchangeSDOToXML"
typedef s32 (_cdecl *FPROCNDXCHANGESDOTOXML)(
				_IN SDOBinary *pSDB,
				_IN u32 sdbSize,
				_IN u16 objType,
				_IN NDXOptions *pNDXOpts,
				_OUT SMXMLStrBuf *pXMLBuf,
				_IN SMXMLStrBuf *pAttrBufOpt
				);

//---------------------------------------------------------------------
// NDXNamespace API
//---------------------------------------------------------------------
						
#define NDXNAMESPACEFREE_STR  "NDXNamespaceFree"
typedef void (_cdecl *FPROCNDXNAMESPACEFREE)(_IN NDXNamespace *pXNS);


#define NDXCHANGEOIDTOONS_STR  "NDXchangeOIDToONS"
typedef NDXNamespace * (_cdecl *FPROCNDXCHANGEOIDTOONS)(_IN u32 oid);


#define NDXCHANGEONSTOOID_STR  "NDXchangeONSToOID"
typedef NDXNamespace * (_cdecl *FPROCNDXCHANGEONSTOOID)(_IN const astring *pNamespace);


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCNDXDEF_H_INCLUDED__
