
//--------------------------------------------------------------------
// 
//             Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
// 
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// 
// Abstract/Purpose:
//
// Header file for Dell SM API support functions. 
// Defines the export/import functions.
// 
// Environment:
// 
// WINNT, Netware and OS2
// 
//--------------------------------------------------------------------

/** @file */

#ifndef __DCSGENAPI_H_INCLUDED__
#define __DCSGENAPI_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Data Engine include files
//---------------------------------------------------------------------
#include <dcsgendef.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// SMIL Generic Interface layer API function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Interface version information */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Gets Major Version of this interface.
 * This API is called to obtain the major version 
 * number. This can be used to perform version control for the 
 * module and to determine which features are supported.
 *
 * @return Major version of the interface.
 *
*/
//---------------------------------------------------------------------
SMINTF u32 _cdecl SMILGetMajorVersion(void);

//---------------------------------------------------------------------
/** Gets Major Version of this interface.
 * This API is called to obtain the minor version 
 * number. This can be used to perform version control for the 
 * module and to determine which features are supported.
 *
 * @return Minor version of the interface.
 *
*/
//---------------------------------------------------------------------
SMINTF u32 _cdecl SMILGetMinorVersion(void);

//---------------------------------------------------------------------
/** Gets Revision Version of this interface.
 * This API is called to obtain the revision version 
 * number. This can be used to perform version control for the 
 * module and to determine which features are supported.
 *
 * @return Revision version of the interface.
 *
*/
//---------------------------------------------------------------------
SMINTF u32 _cdecl SMILGetRevision(void);


//---------------------------------------------------------------------
/** Interface heap memory free funtion.
 * This API is called by to free dynamically allocated memory. This 
 * is used to free memory returned from any API which does NOT have a
 * type set to "const".
 * @note The caller cannot use the C-Lib "free" API method to release 
 * memory returned by this module. 
 * The memory returned from this module may not be compatible with 
 * the C-Lib "free" API method and may lead to undefined execution.
 *
 * @param pO [in] Specifies a pointer to a previously returned memory 
 * object that is to be freed.
 *
*/
//---------------------------------------------------------------------
SMINTF void _cdecl SMILFreeGeneric(_IN void *pO);


//---------------------------------------------------------------------
// SMIL Interface layer API function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Data manager information API function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Gets ready state of Data Manager.
 * This API is called to see if the the Data Manager server is 
 * running and ready.
 *
 * @return If the Data Manager is ready to serve Data Objects this API 
 * method ready, returns TRUE else returns FALSE.
 *
 */
//---------------------------------------------------------------------
SMINTF booln _cdecl SMILIsDataManagerReady(void);



//---------------------------------------------------------------------
/**
 * Request and response allocation API function prototypes
 * use SMILFreeGeneric to free returned memory
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Allocate a largest sized request buffer.
 * This API method will allocate and return a pointer to the largest 
 * request structure that can be used. The request structure 
 * returned should be freed using SMILFreeGeneric. This API will 
 * return NULL if the memory could not be allocated.
 *
 * @param pReqSize [out] Pointer to a variable that receives the
 * size in bytes of the request structure returned.
 *
 * @return pointer to request structure whose memory has 
 * been allocated or NULL if memory could not be allocated.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF void * _cdecl SMILAllocSMReq(_OUT u32 *pReqSize);

//---------------------------------------------------------------------
/** Allocate a largest sized response buffer.
 * This API method will allocate and return a pointer to the largest 
 * response structure that can be used. The response structure 
 * returned should be freed using SMILFreeGeneric. This API will 
 * return NULL if the memory could not be allocated.
 *
 * @param pRspSize [out] Pointer to a variable that receives the
 * size in bytes of the response structure returned.
 *
 * @return pointer to response structure whose memory has 
 * been allocated or NULL if memory could not be allocated.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF void * _cdecl SMILAllocSMRsp(_OUT u32 *pRspSize);



//---------------------------------------------------------------------
// Data Object and Event Allocation API
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Allocate a largest sized data object buffer.
 * This API method will allocate and return a pointer to the largest 
 * data object structure that can be used. The data object structure 
 * returned should be freed using SMILFreeGeneric. This API will 
 * return NULL if the memory could not be allocated.
 *
 * @param pDataObjSize [out] Pointer to a variable that receives the
 * size in bytes of the data object structure returned.
 *
 * @return pointer to data object structure whose memory has 
 * been allocated or NULL if memory could not be allocated.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF DataObjHeader * _cdecl SMILAllocDataObject(_OUT u32 *pDataObjSize);


//---------------------------------------------------------------------
/** Allocate a largest sized event object buffer.
 * This API method will allocate and return a pointer to the largest 
 * event object structure that can be used. The event object structure 
 * returned should be freed using SMILFreeGeneric. This API will 
 * return NULL if the memory could not be allocated.
 *
 * @param pDataEvtSize [out] Pointer to a variable that receives the
 * size in bytes of the event object structure returned.
 *
 * @return pointer to event object structure whose memory has 
 * been allocated or NULL if memory could not be allocated.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF DataEventHeader * _cdecl SMILAllocDataEvent(_OUT u32 *pDataEvtSize);


//---------------------------------------------------------------------
// Object and list Re-allocation API function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Duplicate the required sized data object list.
 * Use these functions if needing to conserve memory buffers returned
 * by the manager and object access API functions described 
 * due to the need for caching lists and objects by the caller
 * returned memory should be freed using SMILFreeGeneric
 *
 * @param pMOL [in] Pointer to a object id list
 *
 * @param pReqListSize [out] Recieves the size of object list returned.
 *
 * @return List of object IDs that are populated in the input list.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILAllocAndCopyRequiredObjList(_IN const ObjList *pMOL, 
														_OUT u32 *pReqListSize);

//---------------------------------------------------------------------
/** Duplicate the required sized data object.
 * Use these functions if needing to conserve memory buffers returned
 * by the manager and object access API functions described 
 * due to the need for caching lists and objects by the caller
 * returned memory should be freed using SMILFreeGeneric
 *
 * @param pMOH [in] Pointer to a data object.
 *
 * @return Data Object within a buffer required to contain it.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF DataObjHeader * _cdecl SMILAllocAndCopyRequiredDataObj(_IN const DataObjHeader *pMOH);

//---------------------------------------------------------------------
/** Duplicate the required sized data event.
 * Use these functions if needing to conserve memory buffers returned
 * by the manager and object access API functions described 
 * due to the need for caching lists and objects by the caller
 * returned memory should be freed using SMILFreeGeneric
 *
 * @param pMEH [in] Pointer to a data event.
 *
 * @return Data Object within a buffer required to contain it.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF DataEventHeader * _cdecl SMILAllocAndCopyRequiredDataEvt(_IN const DataEventHeader *pMEH);


//---------------------------------------------------------------------
// Data Object, Event and Manager access API function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Get a list of parent OIDs for specified OID. 
 * This API is called to retrieve all parent objects (OID's) from a 
 * specific OID of a data object.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 *
 * @return Pointer to a data structure containing parent object 
 * identifier lists returned by the Data Manager for this OID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no parent objects satisfying the search
 * criteria. 
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListParentOID(_IN const ObjID *pOID);

//---------------------------------------------------------------------
/** Get a list of child OIDs for specified OID. 
 * This API is called to retrieve all child objects (OID's) from a 
 * specific OID of a data object.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 *
 * @return Pointer to a data structure containing child object 
 * identifier lists returned by the Data Manager for this OID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no child objects satisfying the search
 * criteria. 
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListChildOID(_IN const ObjID *pOID);

//---------------------------------------------------------------------
/** Get a list of parent/grandparent OIDs for specified OID and type.
 * This API is called to retrieve all parent objects (OID's) from a
 * specific OID and type of a data object.
 * This search routine examines parents and grandparents to satisfy 
 * the search criteria.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 * 
 * @param objType [in] Specifies the type of object to list. 
 * Data Object types are defined in the dcdataobj.h header 
 * (i.e. DATA_OBJ_TYPE_ROOT) 
 *
 * @return Pointer to a data structure containing parent object 
 * identifier lists returned by the Data Manager for this OID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no child objects satisfying the search
 * criteria. 
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListParentOIDByType(_IN const ObjID *pOID, 
												_IN u16 objType);

//---------------------------------------------------------------------
/** Get a list of child/grandchild OIDs for specified OID and type.
 * This API is called to retrieve all child objects (OID's) from a
 * specific OID and type of a data object.
 * This search routine examines children and grandchildren to satisfy 
 * the search criteria.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 * 
 * @param objType [in] Specifies the type of object to list. 
 * Data Object types are defined in the dcdataobj.h header 
 * (i.e. DATA_OBJ_TYPE_ROOT) 
 *
 * @return Pointer to a data structure containing child object 
 * identifier lists returned by the Data Manager for this OID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no child objects satisfying the search
 * criteria. 
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListChildOIDByType(_IN const ObjID *pOID, 
											   _IN u16 objType);

//---------------------------------------------------------------------
/** Get a list of child/grandchild OIDs for specified OID, type 
 * and object status.
 * This API is called to retrieve all child  objects (OID's) from a
 * specific OID and type and status of a data object.
 * This search routine examines children and grandchildren to satisfy 
 * the search criteria.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 * 
 * @param objType [in] Specifies the type of object to list. 
 * Data Object types are defined in the dcdataobj.h header 
 * (i.e. DATA_OBJ_TYPE_ROOT)
 *
 * @param worseThan [in] Specifies the search criteria of the 
 * objStatus field to retrieve (TRUE or FALSE). If TRUE then 
 * object status searched should be equal or worse than the given 
 * status. FALSE if exact match desired.
 *
 * @param objStatus [in] This the object status field to 
 * examine for a match. Specifies the status of the object being 
 * searched for.  Specifies the health status in time associated with 
 * the Data Object and the properties described by it. 
 *
 * @return Pointer to a data structure containing child object 
 * identifier lists returned by the Data Manager for this OID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no child objects satisfying the search
 * criteria. 
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListChildOIDByStatus(_IN const ObjID *pOID, 
												 _IN u16 objType, 
												 _IN booln worseThan, 
												 _IN u8 objStatus);

//---------------------------------------------------------------------
/** This API is called to retrieve the Data Object structure 
 * associated with a specific object identifier (OID).
 * This search is the fastest one of all searches.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param pOID [in] Pointer to OID of Data object.
 *
 * @return Pointer to the specified Data Object. This structure has 
 * a header and specific parameters to describe an object. NULL is 
 * returned if the Data Object could not be found or memory could not 
 * be allocated.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF DataObjHeader * _cdecl SMILGetObjByOID(_IN const ObjID *pOID);

//---------------------------------------------------------------------
/** This API is called to perform set operations on the specified 
 * Data Object. The API is called after setting up the request object. 
 *
 * @param pSReq [in] Pointer to request structure.
 *
 * @param reqSize [in] Specifies the size in bytes of the request 
 * structure.
 *
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SMILAllocSMReq
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMILSetObjByReq(_IN const SMReqHeaderSet *pSReq, 
								  _IN u32 reqSize);

//---------------------------------------------------------------------
/** This API is called to perform pass-through operations on the 
 * specified Data Object. The API is called after setting up the 
 * request object. On completion it returns the completion status 
 * along with the response from the Data Object if any. 
 *
 * @param pOPTReq [in] Pointer to request structure.
 *
 * @param reqSize [in] Specifies the size in bytes of the request 
 * structure.
 *
 * @param pRsp [out] Pointer to response structure.
 *
 * @param rspSize [in] Specifies the size in bytes of the response 
 * structure.
 *
 * @param pBytesReturned [in] Recieves the size in bytes returned in 
 * the response structure.
 *
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SMILAllocSMReq, SMILAllocSMRsp
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMILPassThruObjByReq(_IN const SMReqHeaderPassThru *pOPTReq, 
									   _IN u32 reqSize, 
									   _OUT void *pRsp, 
									   _IN u32 rspSize, 
									   _OUT u32 *pBytesReturned);



//---------------------------------------------------------------------
// Data Event access and listen API function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** This API is called to add an event listener function to handle 
 * event messages from the Data Manager. One must call the API 
 * function SMILRemoveEventListener before terminating the listener. 
 * The event listener is inserted into a LIFO queue which contains a 
 * node per listener.  Note that the event listener function may be 
 * called to handle an event before this API returns to the caller.
 *
 * @param fpEventListener [in] Event Listener Function that is 
 * to receive event messages. The listener function should 
 * return SM_STATUS_SUCCESS if the event should continue to be 
 * passed on to other listeners registered. The listener should 
 * return status other than SM_STATUS_SUCCESS if the event should
 * not be passed on to other listeners. 
 *
 * @return On success returns SM_STATUS_SUCCESS. On error returns 
 * the status code. SM_STATUS_INVALID_PARAMETER implies invalid 
 * argument passed to API function. SM_STATUS_OBJECT_ALREADY_EXISTS 
 * Specified event listener function is already present in queue.
 *
 * @see SMILRemoveEventListener
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMILAddEventListener(_IN const FPROCSMILEVENTLISTENER fpEventListener);

//---------------------------------------------------------------------
/** This API is called to remove an event listener function that 
 * handles event messages from the server Data Manager. 
 * One must call the API function SMILRemoveEventListener 
 * after adding the Event Listener function via SMILAddEventListener 
 * and when it is not necessary to receive event messages anymore 
 * (i.e. before terminating the consumer application to free 
 * resources). 
 *
 * @param fpEventListener [in] Event Listener Function that was 
 * previously added to recieved event messages. 
 *
 * @return On success returns SM_STATUS_SUCCESS. On error returns 
 * the status code. SM_STATUS_INVALID_PARAMETER implies invalid 
 * argument passed to API function. SM_STATUS_NO_SUCH_OBJECT  
 * Specified event listener function is not present in queue.
 *
 * @see SMILAddEventListener
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMILRemoveEventListener(_IN const FPROCSMILEVENTLISTENER fpEventListener);



//---------------------------------------------------------------------
// Data Object String and Support API function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Returns an allocated UTF8 string copy of the UCS2 string contained 
 * within a Data Object at offset determined by offsetStr after 
 * conversion. NULL is returned if memory could not be allocated or
 * converted.
 *
 * @param pDOH [in] Pointer to the DataObjHeader.
 *
 * @param offsetStr [in] offset of the string in the DataObject from the start of pDOH.
 *
 * @return Allocated UTF8 string copy of the UCS2 string supplied.
 * Free returned memory when done with use.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF astring * _cdecl SMILDOGetByOffsetUTF8Str(_IN const DataObjHeader *pDOH, 
												 _IN u32 offsetStr);

//---------------------------------------------------------------------
/** Returns the worst case object status of a sequence of Data 
 * Objects passed to it one at a time. Initially the routine 
 * should be called with a NULL pointer to Data Objects and valid 
 * status field variable. Subsequent to that call with valid Data 
 * Objects to recompute worst case status.
 *
 * @param pDOH [in] Pointer to the DataObjHeader.
 *
 * @param pObjStatus [in/out] Specifies accumulated worst case 
 * status on input and recieves accumulated worst case status 
 * inclusive of input pDOH
 *
 */
//---------------------------------------------------------------------
SMINTF void _cdecl SMILDOComputeObjStatus(_IN const DataObjHeader *pDOH, 
										  _INOUT u8 *pObjStatus);

//---------------------------------------------------------------------
/** Returns the UTF8 time string corresponding to the input UTC UNIX 
 * time from EPOCH as a 64-bit number. This is usefull in outputing 
 * the time stamped into the DataEventHeader or other data objects 
 * like ESM event logs.
 *
 * @param timeVal [in] Number of seconds elapsed since UNIX EPOCH.
 *
 * @return Allocated UTF8 string copy of UNIX time string converted.
 * Free returned memory when done with use.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF astring * _cdecl SMILDOGetObjTimeUTF8Str(_IN s64 timeVal);

//---------------------------------------------------------------------
/** Returns the alias name of the entity (Populator, Manager, other) that 
 * created the Data Object.
 *
 * @param pOID [in] Pointer to OID of Data object.
 * 
 * @return Allocated UTF8 string copy of creator entity name.
 * Free returned memory when done with use.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF astring * _cdecl SMILDOGetCreatorAliasNameByOID(_IN const ObjID *pOID);

//---------------------------------------------------------------------
/** Returns the display name of the entity (Populator, Manager, other) that 
 * created the Data Object.
 *
 * @param pOID [in] Pointer to OID of Data object.
 * 
 * @return Allocated UTF8 string copy of creator entity name.
 * Free returned memory when done with use.
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF astring * _cdecl SMILDOGetCreatorDisplayNameByOID(_IN const ObjID *pOID);

//---------------------------------------------------------------------
/** Get a list of OIDs created by specified creator ID
 * This API is called to retrieve all objects (OID's) that match
 * a specific creator ID.
 * @note These routines return more than required memory on several
 * occassions and need to be combined with Re-allocation API if 
 * memory footprint needs to be minimum while caching objects and 
 * lists.
 *
 * @param creatorID [in] ID used to denote the creator in an OID.
 *
 * @return Pointer to a data structure containing object
 * identifier lists returned by the Data Manager macthed for 
 * this creator ID. 
 * This structure has members defining the number of objects in 
 * the list and the objects ID's. NULL Memory could not be allocated 
 * or initialized or there are no objects satisfying the search
 * criteria. 
 *
 * @see FreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF ObjList * _cdecl SMILListCreatorOIDByID(_IN u8 creatorID);

//---------------------------------------------------------------------
/** Returns a reference/pointer to the body/payload 
 * portion of the object memory pointed to by the parameter pDOH. 
 * It is not a new allocation to prevent unnecessary copies. 
 * The optional parameter pBodySize will be set to the size in 
 * bytes of the body/payload. Return value is NULL if body 
 * absent or is malformed.
 * @note Do NOT call SMILFreeGeneric or any free on the returned memory 
 * since it is just a reference pointer from input pDOH.
 *
 * @param pDOH [in] Pointer to the DataObjHeader.
 *
 * @param pBodySize [out] Recieves the size in bytes of the body/payload.
 *
 * @return Pointer to memory that represent the body/payload of Data 
 * Object.
 *
 */
//---------------------------------------------------------------------
SMINTF const void * _cdecl SMILDOGetObjBody(_IN const DataObjHeader *pDOH,
											_OUT u32 *pBodySize);


//---------------------------------------------------------------------
// Support API functions that may have conterparts in dcsuptXX
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Gets the language identifier applicable to the current 
 * OS installation.
 * This API is used for localization of strings.
 *
 * @return Language identifier example: LID_ASCII_CODE_PAGE
 *
 */
//---------------------------------------------------------------------
SMINTF u32 _cdecl SUPTGetOSLID(void);


//---------------------------------------------------------------------
/** API to allocate memory via this module. 
 * This is used when required to allocate heap memory to work in 
 * conjunction with SMILFreeGeneric.
 * 
 * @param size [in] Size in bytes of heap memory required.
 *
 * @return Language identifier example: LID_ASCII_CODE_PAGE
 *
 * @see SMILFreeGeneric
 *
 */
//---------------------------------------------------------------------
SMINTF void * _cdecl SUPTAllocMemBuf(_IN u32 size);



//---------------------------------------------------------------------
// SGEN Event Filter Configuration (Alert Log Mgmt R/W) APIs
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Returns the configuration settings for the event key string 
 * and the filter tag name. Ideally used by event implementers processing
 * code to identify if a particular event is to be filtered.
 *
 * @param pUTF8SecName [in] Pointer to environmental object section name in the event filter ini file.
 *
 * @param pUTF8KeyStr [in] Pointer to environmental object key name in the event filter ini file.
 *
 * @param pUTF8FilterTagName [in] Pointer to event filter tag name.
 * 
 * @return If the event is set to be filtered out, it returns TRUE. Returns FALSE otherwise, indicating the event needs to be logged.
 *
 * @see SGENSetEventCfgAll, SGENSetEventCfgByKey
 *
 */
//---------------------------------------------------------------------
SMINTF booln _cdecl SGENIsEventFilterSet(_IN const astring *pUTF8SecName, 
										 _IN const astring *pUTF8KeyStr, 
										 _IN const astring *pUTF8FilterTagName);

//---------------------------------------------------------------------
/** Sets the configuration settings for a particular severity log type
 * for the entire system for each of the filter tag names provided.
 * The filter tag names list is a null separated, double null terminated
 * string
 *
 * @param logSeverityType [in] log severity type as defined in dcsupdef.h
 *
 * @param pUTF8FilterTagNameList [in] Pointer to null-separated, double null-terminated event filter tag names.
 * 
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SGENGetEventCfgAll
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SGENSetEventCfgAll(_IN u16 logSeverityType, 
									 _IN const astring *pUTF8FilterTagNameList);

//---------------------------------------------------------------------
/** Gets the configuration settings for a particular severity log type
 * for the entire system. The returned filter tag names list is a null separated, 
 * double null terminated string.
 *
 * @param logSeverityType [in] log severity type as defined in dcsupdef.h
 *
 * @param pUTF8FilterTagNameList [out] Pointer to null-separated, double null-terminated event filter tag names.
 *
 * @param pUTF8BufSize [inout] Pointer to a variable that specifies size in bytes of the structure pointed to by the pUTF8FilterTagNameList parameter. When the method returns, the variable pointed to by pUTF8BufSize contains the number of bytes stored in the buffer.
 * 
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SGENSetEventCfgAll
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SGENGetEventCfgAll(_IN u16 logSeverityType, 
									 _OUT astring *pUTF8FilterTagNameList, 
									 _INOUT u32 *pUTF8BufSize);

//---------------------------------------------------------------------
/** Sets the configuration settings for a particular severity log type
 * for the input key value.
 *
 * @param pUTF8KeyStr [in] Pointer to environmental object key name in the event filter ini file.
 *
 * @param logSeverityType [in] log severity type as defined in dcsupdef.h
 *
 * @param pUTF8FilterTagNameList [out] Pointer to null-separated, double null-terminated event filter tag names.
 * 
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SGENGetEventCfgByKey
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SGENSetEventCfgByKey(_IN const astring *pUTF8KeyStr, 
									   _IN u16 logSeverityType, 
									   _IN const astring *pUTF8FilterTagNameList);

//---------------------------------------------------------------------
/** Gets the configuration settings for a particular severity log type
 * for the input key value. The returned filter tag names list is a null separated, 
 * double null terminated string.
 *
 * @param pUTF8KeyStr [in] Pointer to environmental object key name in the event filter ini file.
 *
 * @param logSeverityType [in] log severity type as defined in dcsupdef.h
 *
 * @param pUTF8FilterTagNameList [out] Pointer to null-separated, double null-terminated event filter tag names.
 *
 * @param pUTF8BufSize [inout] Pointer to a variable that specifies size in bytes of the structure pointed to by the pUTF8FilterTagNameList parameter. When the method returns, the variable pointed to by pUTF8BufSize contains the number of bytes stored in the buffer.
 * 
 * @return On success returns SM_STATUS_SUCCESS. On error returns the status code.
 *
 * @see SGENSetEventCfgByKey
 *
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SGENGetEventCfgByKey(_IN const astring *pUTF8KeyStr, 
									   _IN u16 logSeverityType, 
									   _OUT astring *pUTF8FilterTagNameList, 
									   _INOUT u32 *pUTF8BufSize);



#ifdef __cplusplus
}
#endif


#endif //  __DCSGENAPI_H_INCLUDED__ 



