
//--------------------------------------------------------------------
// 
//             Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
// 
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// 
// Abstract/Purpose:
//
// Header file for Dell SM API support functions. 
// Defines the export/import functions.
// 
// Environment:
// 
// WINNT, Netware and OS2
// 
//--------------------------------------------------------------------

/** @file */

#ifndef __DCSGENDEF_H_INCLUDED__
#define __DCSGENDEF_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------

//--------------------------------------------------------------------
// SDK files
//--------------------------------------------------------------------
#include <dcossdk.h>


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>



//--------------------------------------------------------------------
// Data Engine include files
//--------------------------------------------------------------------
#include <dcsupdef.h>
#include <dcsupapi.h>
#include <dcsmdispatchdef.h>
#include <dcdataevent.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Event log filter section name in file dcip*/
//---------------------------------------------------------------------
#define INI_SEC_EVTLOG_CONFIG			"Event Log Configuration Section"


//---------------------------------------------------------------------
/** Event log generation section and key names */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Key to disable all event (filter all) */
//---------------------------------------------------------------------
#define INI_KEY_EVTLOG_DIS_ALL			"DisableAll"

//---------------------------------------------------------------------
/** Key to disable specific event (filter specified) */
//---------------------------------------------------------------------
#define INI_KEY_EVTLOG_DIS_SPECIFIC		"DisableSpecific"

//---------------------------------------------------------------------
/** Section name and key to disable IPMI R2 Logs(filter specified) */
//---------------------------------------------------------------------
#define INI_SEC_IPMIR2_OS_CONFIG		"IPMI R2 Event Log Configuration Section"
#define INI_KEY_FILTER_IPMIR2_OS_STR	"alert_log_ipmir2_os_off"

//---------------------------------------------------------------------
/** Format string to create a lookup key */
//---------------------------------------------------------------------
#define INI_KEY_FMT_STR_EVTLOG_KEY		"%s.%s"


//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Data structures types and defines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Event Listener Callback signature. 
 * Data Consumers can listen for Data Events by registering a listener.
 * Listeners are inserted in a LIFO order.
 *
 * @param pDEH [in] Data Event that needs to be processed.
 *
 * @return SM_STATUS_SUCCESS if the Data Event should be passed on to
 * the next listener on the queue. Error status if event should not
 * be passed to other listeners in the queue.
 *
 * @see SMILAddEventListener, SMILRemoveEventListener
 */
//---------------------------------------------------------------------
typedef s32 (_cdecl *FPROCSMILEVENTLISTENER)(
				_IN const DataEventHeader *pDEH
				);

	
#ifdef __cplusplus
}
#endif


#endif // __DCSGENDEF_H_INCLUDED__



