
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//   Header file for Dell SMIL Dispatch API request and response
//   structures, defines and macros
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSMDISPATCHDEF_H_INCLUDED__
#define __DCSMDISPATCHDEF_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// SDK files
//--------------------------------------------------------------------
#include <dcossdk.h>


//--------------------------------------------------------------------
// Standard C include files
//--------------------------------------------------------------------
#include <limits.h>


//--------------------------------------------------------------------
// Shared include files
//--------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>
#include <dcdataobj.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Limits
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Maximum size of populator prefix name string */
//---------------------------------------------------------------------
#ifdef SYSTEM_TYPE_LINUX
//---------------------------------------------------------------------
/** The populator module name is a 11.2 format where in LINUX .2 = .so */
//---------------------------------------------------------------------
#define MAX_POP_MODULE_NAME					(14)
#else
//---------------------------------------------------------------------
/** The populator module name is a 8.3 format where in WINDOWS .3 = .DLL and Netware .3 = .NLM */
//---------------------------------------------------------------------
#define MAX_POP_MODULE_NAME					(8)
#endif // SYSTEM_TYPE_LINUX


//---------------------------------------------------------------------
/** 
 * Base Dispatch Request Type classes supported by
 * MODLNAME_SMPopulatorDispatch or SMClientDispatch
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Request type invalid */
//---------------------------------------------------------------------
#define SM_DISP_REQTYPE_INVALID				(0)

//---------------------------------------------------------------------
/** Request type supported by data manager and data populator */
//---------------------------------------------------------------------
#define SM_DISP_REQTYPE_COMMON				(1)

//---------------------------------------------------------------------
/** Request type supported by data populator only */
//---------------------------------------------------------------------
#define SM_DISP_REQTYPE_DATA_POPULATOR		(256)

//---------------------------------------------------------------------
/** Request type supported by data manager only */
//---------------------------------------------------------------------
#define SM_DISP_REQTYPE_DATA_MANAGER		(512)



//---------------------------------------------------------------------
/** 
 * Disptach Request Types (DRT) supported by
 * MODLNAME_SMPopulatorDispatch or SMClientDispatch
 * passed in the reqType field
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Gets the version of the module being called */
//---------------------------------------------------------------------
#define SM_DRT_COM_GET_COMPONENT_VERSION	(SM_DISP_REQTYPE_COMMON + 2)

//---------------------------------------------------------------------
/** Lists information about child tree generated by this module */
//---------------------------------------------------------------------
#define SM_DRT_COM_GET_OBJ_LIST_INFO		(SM_DISP_REQTYPE_COMMON + 4)

//---------------------------------------------------------------------
/** Lists children of an object */
//---------------------------------------------------------------------
#define SM_DRT_COM_LIST_CHILDREN_OID		(SM_DISP_REQTYPE_COMMON + 5)

//---------------------------------------------------------------------
/** Gets entire properties of an object for specified OID */
//---------------------------------------------------------------------
#define SM_DRT_COM_GET_OBJ_BY_OID			(SM_DISP_REQTYPE_COMMON + 6)

//---------------------------------------------------------------------
/** Consumer to hip object populator pass through call */
//---------------------------------------------------------------------
#define SM_DRT_COM_PASS_THRU_BY_OID			(SM_DISP_REQTYPE_COMMON + 7)



//---------------------------------------------------------------------
/** Load and initialize populator */
//---------------------------------------------------------------------
#define SM_DRT_POP_LOAD						(SM_DISP_REQTYPE_DATA_POPULATOR + 0)

//---------------------------------------------------------------------
/** Deinitialize and unload populator */
//---------------------------------------------------------------------
#define SM_DRT_POP_UNLOAD					(SM_DISP_REQTYPE_DATA_POPULATOR + 1)

//---------------------------------------------------------------------
/** Start monitoring device that populator manages */
//---------------------------------------------------------------------
#define SM_DRT_POP_START_MONITOR			(SM_DISP_REQTYPE_DATA_POPULATOR + 2)

//---------------------------------------------------------------------
/** Stop monitoring device that populator manages */
//---------------------------------------------------------------------
#define SM_DRT_POP_STOP_MONITOR				(SM_DISP_REQTYPE_DATA_POPULATOR + 3)

//---------------------------------------------------------------------
/** Refresh the properties of an populator object. @note Data Object will NOT be added to refresh interval list if returned status != SM_STATUS_SUCCESS on dispatch */
//---------------------------------------------------------------------
#define SM_DRT_POP_REFRESH_OBJ				(SM_DISP_REQTYPE_DATA_POPULATOR + 4)

//---------------------------------------------------------------------
/** Data event passed to data populator for consumption */
//---------------------------------------------------------------------
#define SM_DRT_POP_SET_DATA_EVENT			(SM_DISP_REQTYPE_DATA_POPULATOR + 6)

//---------------------------------------------------------------------
/** Set operation on an object given its OID, set request, current obj(if cached) */
//---------------------------------------------------------------------
#define SM_DRT_POP_SET_OBJ_BY_OID			(SM_DISP_REQTYPE_DATA_POPULATOR + 7)



//---------------------------------------------------------------------
/** Set operation on an object given its OID, set request */
//---------------------------------------------------------------------
#define SM_DRT_DMG_SET_OBJ_BY_OID			(SM_DISP_REQTYPE_DATA_MANAGER + 0)

//---------------------------------------------------------------------
/** Lists all children OID of an object matching a given type */
//---------------------------------------------------------------------
#define SM_DRT_DMG_LIST_CHILDREN_OID_BY_TYPE	(SM_DISP_REQTYPE_DATA_MANAGER + 3)

//---------------------------------------------------------------------
/** Get a list of all oids if the status specified in objByStatus */
//---------------------------------------------------------------------
#define SM_DRT_DMG_LIST_CHILDREN_OID_BY_STATUS	(SM_DISP_REQTYPE_DATA_MANAGER + 11)

//---------------------------------------------------------------------
/** Lists all parent and grand parent OID of an object matching a given type */
//---------------------------------------------------------------------
#define SM_DRT_DMG_LIST_PARENT_OID_BY_TYPE	(SM_DISP_REQTYPE_DATA_MANAGER + 12)

//---------------------------------------------------------------------
/** Lists parents of an object */
//---------------------------------------------------------------------
#define SM_DRT_DMG_LIST_PARENT_OID			(SM_DISP_REQTYPE_DATA_MANAGER + 13)

//---------------------------------------------------------------------
/** Obtains alias name of an OID creator when creator ID supplied */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_CREATOR_ALIASNAME_BY_ID	(SM_DISP_REQTYPE_DATA_MANAGER + 14)

//---------------------------------------------------------------------
/** Obtains display name of an OID creator when creator ID supplied */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_CREATOR_DISPLAYNAME_BY_ID	(SM_DISP_REQTYPE_DATA_MANAGER + 15)

//---------------------------------------------------------------------
/** Lists all OID matching (created) by creator ID supplied */
//---------------------------------------------------------------------
#define SM_DRT_DMG_LIST_CREATOR_OID_BY_ID	(SM_DISP_REQTYPE_DATA_MANAGER + 16)



//---------------------------------------------------------------------
/** 
 * @note OBSOLETE (Deprecated): Dispatch Request Types (DRT) supported by 
 * SMClientDispatch passed in the reqType field
 * maintained only for old legacy code and 
 * should not be used in new code
 * refer to HLD for info on how to change code
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** No operation performed on input and output. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_COM_NOP						(SM_DISP_REQTYPE_COMMON + 0)

//---------------------------------------------------------------------
/** Gets the version of the API entry point. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_COM_GET_API_VERSION			(SM_DISP_REQTYPE_COMMON + 1)

//---------------------------------------------------------------------
/** Echo the input buffer into the output buffer. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_COM_ECHO						(SM_DISP_REQTYPE_COMMON + 3)



//---------------------------------------------------------------------
/** Get the next data event in the data poplator's event queue. @note OBSOLETE @see DESubmitSingle */
//---------------------------------------------------------------------
#define SM_DRT_POP_GET_DATA_EVENT			(SM_DISP_REQTYPE_DATA_POPULATOR + 5)



//---------------------------------------------------------------------
/** Gets OID of an object of a given type and instance. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_OID_BY_TYPE			(SM_DISP_REQTYPE_DATA_MANAGER + 1)

//---------------------------------------------------------------------
/** Gets properties of an object of a given type and instance. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_OBJ_BY_TYPE			(SM_DISP_REQTYPE_DATA_MANAGER + 2)

//---------------------------------------------------------------------
/** Gets the parent OID of a given child OID. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_PARENT_OID_BY_OID	(SM_DISP_REQTYPE_DATA_MANAGER + 4)

//---------------------------------------------------------------------
/** Gets properties of a parent object of a given child OID. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_PARENT_OBJ_BY_OID	(SM_DISP_REQTYPE_DATA_MANAGER + 5)

//---------------------------------------------------------------------
/** Gets the parent OID of a given type and child OID. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_PARENT_OID_BY_TYPE	(SM_DISP_REQTYPE_DATA_MANAGER + 6)

//---------------------------------------------------------------------
/** Gets properties of a parent object of a given type and child OID. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_PARENT_OBJ_BY_TYPE	(SM_DISP_REQTYPE_DATA_MANAGER + 7)

//---------------------------------------------------------------------
/** Gets information of a populator given the populator id. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_POP_INFO_BY_ID		(SM_DISP_REQTYPE_DATA_MANAGER + 8)

//---------------------------------------------------------------------
/** Gets OID of an object of a given status and instance. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_OID_BY_STATUS		(SM_DISP_REQTYPE_DATA_MANAGER + 9)

//---------------------------------------------------------------------
/** Gets properties of an object of a given status and instance. @note OBSOLETE */
//---------------------------------------------------------------------
#define SM_DRT_DMG_GET_OBJ_BY_STATUS		(SM_DISP_REQTYPE_DATA_MANAGER + 10)



//---------------------------------------------------------------------
/** 
 * Set type ranges assigned by Data Manager team
 *
 * Set types are used with request type SM_DRT_DMG_SET_OBJ_BY_OID
 * (SMReqHeaderSet.setType)
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 0 is an invalid set type.
 *  No set can contain this type. This is ignored. */
//---------------------------------------------------------------------
#define SM_SET_TYPE_INVALID				(0)


//---------------------------------------------------------------------
/** 1 - 255 are reserved predefined base set types for Data Manager */
//---------------------------------------------------------------------

/** Minimum in predefined base range */
#define SM_SET_TYPE_BASE_LO				(1)

/** Maximum in predefined base range */
#define SM_SET_TYPE_BASE_HI				(255)

	
//---------------------------------------------------------------------
/** 256 - 355 are reserved set types for Disney (HIP) range 1 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 1 */
#define SM_SET_TYPE_HIP_LO_1			(256)

/** Maximum in Disney (HIP) range 1 */
#define SM_SET_TYPE_HIP_HI_1			(355)


//---------------------------------------------------------------------
/** 356 - 367 are reserved set types for DRAC (DRAC 3/4/5) */
//---------------------------------------------------------------------

/** Minimum in DRAC (DRAC 3/4/5) range */
#define SM_SET_TYPE_DRSC_LO_1			(356)

/** Maximum in DRAC (DRAC 3/4/5) range */
#define SM_SET_TYPE_DRSC_HI_1			(367)


//---------------------------------------------------------------------
/** 368 - 2047 are reserved set types for Disney (HIP) range 2 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 2 */
#define SM_SET_TYPE_HIP_LO_2			(368)

/** Maximum in Disney (HIP) range 2 */
#define SM_SET_TYPE_HIP_HI_2			(2047)


//---------------------------------------------------------------------
/** 2048 - 3071 are reserved set types for Storage Services */
//---------------------------------------------------------------------

/** Minimum in Storage Services range */
#define SM_SET_TYPE_STORAGE_SVCS_LO		(2048)

/** Maximum in Storage Services range */
#define SM_SET_TYPE_STORAGE_SVCS_HI		(3071)


//---------------------------------------------------------------------
/** 3072 - 4095 are reserved set types for Generic range */
//---------------------------------------------------------------------

/** Minimum in Generic range */
#define SM_SET_TYPE_GENERIC_LO			(3072)

/** Maximum in Generic range */
#define SM_SET_TYPE_GENERIC_HI			(4095)


//---------------------------------------------------------------------
/** 4096 - 4294967293 are set types available for assignment by Data Manager team.
 *  Should be considered reserved by all other teams. */
//---------------------------------------------------------------------

/** Minimum in next available set type range */
#define SM_SET_TYPE_NEXT_AVAILABLE_LO	(4096)

/** Maximum in next available set type range */
#define SM_SET_TYPE_NEXT_AVAILABLE_HI	(ULONG_MAX - 2)


//---------------------------------------------------------------------
/** 4294967294 is reserved by Data Manager */
//---------------------------------------------------------------------
#define SM_SET_TYPE_RESERVED1			(ULONG_MAX - 1)


//---------------------------------------------------------------------
/** 4294967295 is reserved by Data Manager */
//---------------------------------------------------------------------
#define SM_SET_TYPE_RESERVED2			(ULONG_MAX)



//---------------------------------------------------------------------
/** 
 * Pass thru type ranges assigned by Data Manager team
 *
 * Pass thru types are used with request type SM_DRT_COM_PASS_THRU_BY_OID
 * (SMReqHeaderPassThru.ptCmd)
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 0 is an invalid pass thru type.
 *  No pass thru can contain this type. This is ignored. */
//---------------------------------------------------------------------
#define SM_PASSTHRU_TYPE_INVALID			(0)


//---------------------------------------------------------------------
/** 1-2 are reserved pass thru types for Disney (HIP) range 1 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 1 */
#define SM_PASSTHRU_TYPE_HIP_LO_1			(1)

/** Maximum in Disney (HIP) range 1 */
#define SM_PASSTHRU_TYPE_HIP_HI_1			(2)


//---------------------------------------------------------------------
/** 3 - 255 are reserved pass thru types for DRAC (DRAC 3/4/5) */
//---------------------------------------------------------------------

/** Maximum in DRAC (DRAC 3/4/5) range */
#define SM_PASSTHRU_TYPE_DRSC_LO_1			(3)

/** Maximum in DRAC (DRAC 3/4/5) range */
#define SM_PASSTHRU_TYPE_DRSC_HI_1			(255)


//---------------------------------------------------------------------
/** 256 - 1023 are reserved pass thru types for Disney (HIP) range 2 */
//---------------------------------------------------------------------

/** Minimum in Disney (HIP) range 2 */
#define SM_PASSTHRU_TYPE_HIP_LO_2			(256)

/** Maximum in Disney (HIP) range 2 */
#define SM_PASSTHRU_TYPE_HIP_HI_2			(1023)


//---------------------------------------------------------------------
/** 1024 - 2047 are reserved predefined base pass thru types for Data Manager */
//---------------------------------------------------------------------

/** Minimum in predefined base range */
#define SM_PASSTHRU_TYPE_BASE_LO			(1024)

/** Maximum in predefined base range */
#define SM_PASSTHRU_TYPE_BASE_HI			(2047)


//---------------------------------------------------------------------
/** 2048 - 3071 are reserved pass thru types for Storage Services */
//---------------------------------------------------------------------

/** Minimum in Storage Service ranges */
#define SM_PASSTHRU_TYPE_STORAGE_SVCS_LO	(2048)

/** Maximum in Storage Service ranges */
#define SM_PASSTHRU_TYPE_STORAGE_SVCS_HI	(3071)


//---------------------------------------------------------------------
/** 3072 - 4095 are reserved pass thru types for Generic range */
//---------------------------------------------------------------------

/** Minimum in Generic range */
#define SM_PASSTHRU_TYPE_GENERIC_LO			(3072)

/** Maximum in Generic range */
#define SM_PASSTHRU_TYPE_GENERIC_HI			(4095)


//---------------------------------------------------------------------
/** 4096 - 5119 are reserved pass thru types for Hardware Inventory range */
//---------------------------------------------------------------------

/** Minimum in Hardware Inventory range */
#define SM_PASSTHRU_TYPE_HARDWARE_INV_LO	(4096)

/** Maximum in Hardware Inventory range */
#define SM_PASSTHRU_TYPE_HARDWARE_INV_HI	(5119)


//---------------------------------------------------------------------
/** 5120 - 4294967293 are pass thru types available for assignment by Data Manager team.
 *  Should be considered reserved by all other teams. */
//---------------------------------------------------------------------

/** Minimum in next available pass thru type range */
#define SM_PASSTHRU_TYPE_NEXT_AVAILABLE_LO	(5120)

/** Maximum in next available pass thru type range */
#define SM_PASSTHRU_TYPE_NEXT_AVAILABLE_HI	(ULONG_MAX - 2)


//---------------------------------------------------------------------
/** 4294967294 is reserved by Data Manager */
//---------------------------------------------------------------------
#define SM_PASSTHRU_TYPE_RESERVED1			(ULONG_MAX - 1)


//---------------------------------------------------------------------
/** 4294967295 is reserved by Data Manager */
//---------------------------------------------------------------------
#define SM_PASSTHRU_TYPE_RESERVED2			(ULONG_MAX)



//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Data structures for requests
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 
 * Forward external reference to SMDPDMDispatchTable Table
 * that implements the Data Populator(DP) to Data Manager(DM)
 * push API interface structure
 */
//---------------------------------------------------------------------
typedef struct _SMDPDMDispatchTable SMDPDMDispatchTable;


//---------------------------------------------------------------------
/** 
 * This dispatch environment is used to pass a pointer to a table of
 * an API exposed by the data manager (DM) to data populators (DP) 
 * that allows the data populator to access data objects, events 
 * and services in the data manager similar to a data consumer 
 * additionally other populator specific services to create, 
 * update and destroy data objects and data events are also 
 * implemented via this environment
 * This enviroment pointer (handle) is passed to a populator 
 * on a load request SM_DRT_POP_LOAD
 */
//---------------------------------------------------------------------
typedef struct _SMDPDMDispatchEnv
{
	/** Pointer to DPDM push API table. */
	const SMDPDMDispatchTable *pDPDMDTable;
	/** Pointer to data populator environment context maintained by data manager. */
	const void *pDPDMDCtx;
} SMDPDMDispatchEnv;


//---------------------------------------------------------------------
// NOTE: Need to pack on byte boundary as Legacy Populators expect it
// to be on byte boundaries
//---------------------------------------------------------------------
#pragma pack(1)


//---------------------------------------------------------------------
/** 
 * Parameters passed to data populator on a load request
 * SM_DRT_POP_LOAD to inform the Data Populator of signal (OBSOLSTE), 
 * HAPI (OBSOLETE) and Data Populator to Data Manager interface API (NEW) 
 * handles.
 */
//---------------------------------------------------------------------
typedef struct _SMReqPopLoad
{
	/** Parameters passed to populator which contains a reference to SMDPDMDispatchEnv API environment. */
	const SMDPDMDispatchEnv *pSMDPDMDE;
	/** Handle to be used for access to the HAPI api. @note OBSOLETE */
	HANDLE hHIPApp;
	/** Handle to be used for signaling events to the manager. @note OBSOLETE */
	HANDLE hPopEvent;
	/** Populator ID assigned by the manager. @note OBSOLETE */
	u8 popID;
	/** Alignment for next data element */
	u8 reservedAlign[3];
} SMReqPopLoad;


//---------------------------------------------------------------------
/** 
 * Data structure for set requests
 * SM_DRT_DMG_SET_OBJ_BY_OID
 */
//---------------------------------------------------------------------
typedef struct _SMReqHeaderSet
{
	/** OID of data object to which request applies */
	_IN ObjID objID;
	/** determines the set type to be performed on the object */
	u32 setType;
} SMReqHeaderSet;


//---------------------------------------------------------------------
/** 
 * Data structure for pass thru requests
 * SM_DRT_COM_PASS_THRU_BY_OID
 */
//---------------------------------------------------------------------
typedef struct _SMReqHeaderPassThru
{
	/** OID of data object to which request applies */
	ObjID objID;
	/** Determines the pass thru type to be performed on the object */
	u32 ptCmd;
} SMReqHeaderPassThru;


//---------------------------------------------------------------------
/** 
 * Parameters passed on a request to a get data objects by type
 * SM_DRT_DMG_LIST_CHILDREN_OID_BY_TYPE etc.
 */
//---------------------------------------------------------------------
typedef struct _SMReqObjByType
{
	/** Subsearch qualifier. Specifies parent whose children will be searched to match type and instance. If this is set to DATA_OBJ_OID_ROOT then the search (type, instance) applies to the entire tree */
	ObjID parentObjID;                   
	/** Instance of object being searched for. Set to 0 for SM_DRT_DMG_LIST_CHILDREN_OID_BY_TYPE @note This is Zero(0) indexed. Hence 0 = First instance. */
	u32 instance;
	/** Type of object being searched for */
	u16 objType;
	/** Alignment for next data element */
	u8 reservedAlign[2]; 
} SMReqObjByType;


//---------------------------------------------------------------------
/** 
 * Parameters passed on a request to a get data objects by status
 * SM_DRT_DMG_LIST_CHILDREN_OID_BY_STATUS
 */
//---------------------------------------------------------------------
typedef struct _SMReqObjByStatus
{
	/** Subsearch qualifier. Specifies parent whose children will be searched to match type and instance.If this is set to DATA_OBJ_OID_ROOT then the search (type, instance) applies to the entire tree */
	ObjID parentObjID;
	/** Instance of object being searched for. Set to 0 for SM_DRT_DMG_LIST_CHILDREN_OID_BY_STATUS @note This is Zero(0) indexed. Hence 0 = First instance. */
	u32 instance;
	/** Limit status search to objects of a certain type. For all types set to INVALID_OBJ_TYPE */
	u16 objType;
	/** If TRUE then object status searched should be equal or worse (>=) than the given status. FALSE if exact match desired */
	booln worseThan;
	/** Status of the object being searched for */
	u8 objStatus;
	/** Alignment for next data element */
	u8 reservedAlign[3];
} SMReqObjByStatus;


//---------------------------------------------------------------------
/** 
 * Parameters passed request to a get parent objects by type
 * SM_DRT_DMG_GET_PARENT_OID_BY_OID, SM_DRT_DMG_GET_PARENT_OBJ_BY_OID, 
 * SM_DRT_DMG_GET_PARENT_OID_BY_TYPE, SM_DRT_DMG_GET_PARENT_OBJ_BY_TYPE
 */
//---------------------------------------------------------------------
typedef struct _SMReqObjParent
{
	/** Subsearch qualifier. Specifies child whose parent matching the specified type will be searched */	
	ObjID childObjID;
	/** Type of parent object being searched for. Used only on a search involving type. Ignored otherwise */
	u16 objType;
	/** Alignment for next data element */
	u8 reservedAlign[2];
} SMReqObjParent;



//---------------------------------------------------------------------
//  Data structures for responses
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 
 * Data repository summary information
 * response to a SM_DRT_COM_GET_OBJ_LIST_INFO
 */
//---------------------------------------------------------------------
typedef struct _SMRspObjListInfo
{
	/** Defaullt expected max number of children of a single object */
	u32 maxBreadth;
	/** Default expected max number of generations from a root object */
	u32 maxDepth;
	/** Expected maximum size of data object */
	u32 maxDataObjSize;
} SMRspObjListInfo;


//---------------------------------------------------------------------
/** 
 * Version information about a component command
 * SM_DRT_COM_GET_COMPONENT_VERSION or
 * SM_DRT_COM_GET_API_VERSION
 */
//---------------------------------------------------------------------
typedef struct _SMVersionInfo
{
	/** Major version number of component can be -1 (Unknown) */
	_OUT s8 majorVersion;
	/** Minor version number of component can be -1 (Unknown) */
	_OUT s8 minorVersion;
} SMVersionInfo;


//---------------------------------------------------------------------
/** 
 * Response to a populator information command 
 * SM_DRT_DMG_GET_POP_INFO_BY_ID
 */
//---------------------------------------------------------------------
typedef struct _SMRspPopInfoByID
{
	/** ID assigned to the populator whose info this structure represents */
	u8 popID;
	/** Alignment for next data element */
	u8 reservedAlign[1];
	/** Version information returned by populator to manager on SM_DRT_COM_GET_COMPONENT_VERSION */
	SMVersionInfo vi;
	/** Object list information returned by populator to manager on SM_DRT_COM_GET_OBJ_LIST_INFO */
	SMRspObjListInfo oli;
	/** Accounts for NULL terminator */
	astring moduleName[MAX_POP_MODULE_NAME + 1];
} SMRspPopInfoByID;


#pragma pack()


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCSMDISPATCHDEF_H_INCLUDED__

