
//-------------------------------------------------------------------------
//
//          DELL INCORPORATED PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Incorporated and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 1 $
//
//  Last Modified By / On:
//    $Author: Tushar_Oza $ / $Date: 11/18/03 3:08a $
//
//  Author / Created:
//    Tushar Oza / 2003 Nov 18
//
//  Environment:
//    Linux, Netware, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef _DCSMEPIM_H_INCLUDED_
#define _DCSMEPIM_H_INCLUDED_


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// $(DATAENGINESDK) global include files
//---------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>


#ifdef __cplusplus
extern "C" {
#endif

	
//---------------------------------------------------------------------
//  Structures and definitions
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// DEFINITIONS
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Limits
//--------------------------------------------------------------------


//---------------------------------------------------------------------
/** Maximum size of plugin prefix name string */
//---------------------------------------------------------------------
#ifdef SYSTEM_TYPE_LINUX
//---------------------------------------------------------------------
/** The plugin module name is a 11.2 format where in LINUX .2 = .so */
//---------------------------------------------------------------------
#define MAX_EPI_MODULE_NAME					(14)
#else
//---------------------------------------------------------------------
/** The populator module name is a 8.3 format where in WINDOWS .3 = .DLL and Netware .3 = .NLM */
//---------------------------------------------------------------------
#define MAX_EPI_MODULE_NAME					(8)
#endif // SYSTEM_TYPE_LINUX


//---------------------------------------------------------------------
/** Default maximum number of plug-ins that can be loaded */
//---------------------------------------------------------------------
#define MAX_EVTIMP_PLUGIN_ID				(64)

//--------------------------------------------------------------------
/** Public Event Implementer Plugin ID definition used to set epiIDs */
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** Public plug-in ID (processed by event plug-in manager) */
//--------------------------------------------------------------------
#define EVTIMP_PLUGIN_ID_PUBLIC				(0)

//--------------------------------------------------------------------
/** Reserved plug-in ID (future use by event plug-in manager) */
//--------------------------------------------------------------------
#define EVTIMP_PLUGIN_ID_RESERVED1			(255)


//---------------------------------------------------------------------
/** 
 * Base Event Plug-in Dispatch Request Type classes supported by
 * MODULEXX_SMEventImplementerDispatch
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Request type invalid */
//---------------------------------------------------------------------
#define SM_EPDISP_REQTYPE_INVALID			(0)

//---------------------------------------------------------------------
/** Request type supported by event plug-in manager and event plug-in implementers */
//---------------------------------------------------------------------
#define SM_EPDISP_REQTYPE_COMMON			(1)

//---------------------------------------------------------------------
/** Request type supported by event plug-in implementers only */
//---------------------------------------------------------------------
#define SM_EPDISP_REQTYPE_EPI				(256)

//---------------------------------------------------------------------
/** Request type supported by event plug-in manager only */
//---------------------------------------------------------------------
#define SM_EPDISP_REQTYPE_EPM				(512)


//---------------------------------------------------------------------
/**
 * EP Dispatch Request Types (DRT) supported by
 * MODULEXX_SMEventImplementerDispatch 
 * passed in the reqType field
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Load and initialize event plug-in implementer */
//---------------------------------------------------------------------
#define SM_EPDRT_EPI_LOAD					(SM_EPDISP_REQTYPE_EPI + 0)

//---------------------------------------------------------------------
/** Terminate Event implementer. De-initialize and unload plug-in */
//---------------------------------------------------------------------
#define SM_EPDRT_EPI_UNLOAD					(SM_EPDISP_REQTYPE_EPI + 1)

//---------------------------------------------------------------------
/** Start Event monitoring. Start monitoring event that plug-in manages */
//---------------------------------------------------------------------
#define SM_EPDRT_EPI_START_MONITOR			(SM_EPDISP_REQTYPE_EPI + 2)

//---------------------------------------------------------------------
/** Stop Event monitoring. Stop monitoring event that plug-in manages */
//---------------------------------------------------------------------
#define SM_EPDRT_EPI_STOP_MONITOR			(SM_EPDISP_REQTYPE_EPI + 3)


//--------------------------------------------------------------------
// STRUCTURES
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Data structures for requests
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * type definition of data passed to log callback routines
 */
//---------------------------------------------------------------------
typedef void LogRoutineData;


//---------------------------------------------------------------------
/**
 * SMEPEMDispatchTable
 * forward external reference to SMEPEMDispatchTable Table
 * that implements the Event Plug-in(EP) to Event Plug-in Manager(EM)
 * API interface structure
 */
//---------------------------------------------------------------------
typedef struct _SMEPEMDispatchTable SMEPEMDispatchTable;


//---------------------------------------------------------------------
/** SMEPEMDispatchEnv
 * This dispatch environment is used to pass a pointer to a table of
 * an API exposed by the event plug-in manager (EM) to event plug-in implementerss (EP) 
 * that allows the event plug-in implementers to access 
 * services in the event plug-in manager similar to a data consumer 
 * This enviroment pointer (handle) is passed to a plug-in 
 * on a load request SM_EPDRT_EPI_LOAD
 */
//---------------------------------------------------------------------
typedef struct _SMEPEMDispatchEnv
{
	/** pointer to EPEM push API table */
	const SMEPEMDispatchTable *pEPEMDTable;
	/**  pointer to event plug-in implementers environment context maintained by event plug-in manager */
	const void *pEPEMDCtx;
} SMEPEMDispatchEnv;


//---------------------------------------------------------------------
/** SMEMReqEPILoad
 * Parameters passed to event plug-in implementers on a load request
 * SM_EPDRT_EPI_LOAD
 */
//---------------------------------------------------------------------
typedef struct _SMEMReqEPILoad
{
	/** parameters passed to plug-in which contains a reference to SMEPEMDispatchEnv API environment */
	const SMEPEMDispatchEnv *pSMEPEMDE;
} SMEMReqEPILoad;


//---------------------------------------------------------------------
/** EventMessageData
 * Parameters passed on a request to commit events to OS and XML logs
 */
//---------------------------------------------------------------------
typedef struct _EventMessageData
{
	/** pointer to ASCII substitution description string */
	_OUT astring **ppUTF8DescStr;
	/** number of substitution description strings */
	_OUT u32 numUTF8DescStr;
	/** MC (message compiler) file message ID */
	_OUT u32 mcMsgId;
	/** type of system log (informational, non-critical, error) */
	_OUT u16 logType;
	/** MC (message compiler) file category ID */
	_OUT u16 mcCatId;
	/** pathfileName for log messages */
	_OUT astring *pPathFileName;
	/** optional XML userinfo */
	/** caller needs to allocate and free this */
	_INOUT astring *pXMLUserInfo;
	/** optional flag*/
	/** caller needs to indicate whether */
	/** additional OS Logging on IPMI R2 enabled system */
	/** is needed (FALSE = default; TRUE = disable OS logging) */
	_IN booln bIsIPMIR2OSLogDisabled;
	/** alignment for next data element */
	_OUT u16 reservedAlign;
} EventMessageData;


//---------------------------------------------------------------------
 /** Log Support Function Type Definition
 * Any plugin that provides a logging service, needs to implement
 * a callback log function in this defined format
 * It is responsible for defining the data types it supports
 */
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNLOGROUTINE)(_IN LogRoutineData *pLRData);


//---------------------------------------------------------------------
 /** Event Plug-in Implementer Dispatch Entry Point Function Type Definition
 *
 * This function is used to dispatch requests to and get responses from
 * Event Plug-in Implementers.
 *
 * NOTE: This should only be used as a prototype.
 * The actual exported function  has the form MODULEXX_SMEventImplementerDispatch.
 * Example MODULEXX = SAMEPI32 hence exported function is SAMEPI32_SMEventImplementerDispatch.
 * The way to setup the MODULEXX for implementer compilation in
 * NT: Change preprocessor value of EPIEXPORT_DISPATCHFN to MODULEXX_SMEventImplementerDispatch
 * NW: Change make file value of MODULEXX to the one required.
 * LX: Change make file value of EPIDISP to required MODULEXX_SMEventImplementerDispatch
 */
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNSMEVENTIMPLEMENTERDISPATCH)(_IN u32 epiID, 
													_IN u32 reqType, 
													_IN const void *pInBuf, 
													_IN u32 inBufSize,
													_OUT void *pOutBuf, 
													_IN u32 outBufSize, 
													_OUT u32 *pBytesReturned);


#ifdef __cplusplus
}
#endif


#endif	// _DCSMEPIM_H_INCLUDED_

