
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for SMIL API functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSMILAPI_H_INCLUDED__
#define __DCSMILAPI_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmildef.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Daemon API functions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Data Populator API entry point function
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// FPROCSMPOPULATORDISPATCH
/** 
 * Called by Data Manager to dispatch requests and receive responses
 * from a Data Populator.
 *
 * This API is the Data Populator entry point function. This API method
 * is called by the Data Manager to obtain a response from a Data Populator
 * on execution of an input request. This API method is used to perform
 * operations on Data Objects, Data Events and Data Populators. The
 * specific request types and their corresponding data object structures
 * are outlined in the HLD.
 *
 * @note This should only be used as a prototype. The actual exported
 * function name has the form MODLNMXX_SMPopulatorDispatch. \n
 * Example: If module name is sampop32, the exported function is
 * SAMPOP32_SMPopulatorDispatch. \n
 * MODLNMXX_SMPopulatorDispatch is set up for populator compilation in
 * the following way: \n
 * NT: Set preprocessor value of POPEXPORT_DISPATCHFN to applicable MODLNMXX_SMPopulatorDispatch value. \n
 * NW: Set makefile value of LOC_POP_DISPATCH_NAME to applicable MODLNMXX_SMPopulatorDispatch value. \n
 * LX: Set makefile value of LOC_POP_DISPATCH_NAME to applicable MODLNMXX_SMPopulatorDispatch value.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		obtained previously on a successful call to SMClientAttach.
 *
 * @param reqType [in] Specifies the request type (operation) to be
 *		performed by the Data Populator based on parameters passed in
 *		pInBuf and produces output into pOutBuf.
 *
 * @param pInBuf [in] Pointer to the input request object structure
 *		buffer corresponding to the request type supplied in reqType.
 *		The input request object structure buffer size and contents
 *		correspond to the operation requested.
 *
 * @param inBufSize [in] Specifies the size in bytes of the input
 *		request object structure buffer pointed to by pInBuf.
 *
 * @param pOutBuf [out] Pointer to an output response object structure
 *		buffer which receives the output response from the Data Populator
 *		on completion of the request.
 *
 * @param outBufSize [in] Specifies the size in bytes of the output
 *		response object structure buffer pointed to by pOutBuf.
 *
 * @param pBytesReturned [out] Pointer to a variable that receives
 *		the size in bytes copied to the output object structure buffer.
 *		When the method returns, the variable pointed to by
 *		pBytesReturned contains the number of bytes stored in the
 *		output response object structure buffer.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientAttach, SMClientDispatch
 */
//---------------------------------------------------------------------
typedef s32 (_cdecl *FPROCSMPOPULATORDISPATCH)(_IN u32 clientID,
											   _IN u32 reqType,
											   _IN const void *pInBuf,
											   _IN u32 inBufSize,
											   _OUT void *pOutBuf,
											   _IN u32 outBufSize,
											   _OUT u32 *pBytesReturned);


//---------------------------------------------------------------------
// Data Manager API functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PFNSMSERVERDISPATCH
/** 
 * Called by SMIL to dispatch requests and receive responses from Data Manager.
 *
 * This API is called by SMIL to obtain a response from the Data Manager
 * on execution of a Data Consumer request.  This API method is used
 * to perform operations on Data Objects, Data Events and Data Populators.
 * The specific request types and their corresponding data object
 * structures are outlined in the HLD.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 * 		of the Data Consumer that is making the request.
 *
 * @param reqType [in] Specifies the request type (operation) to be
 *		performed by the Data Manager based on parameters passed in
 *		pInBuf and produces output into pOutBuf.
 *
 * @param pInBuf [in] Pointer to the input request object structure
 *		buffer corresponding to the request type supplied in reqType.
 *		The input request object structure buffer size and contents
 *		correspond to the operation requested.
 *
 * @param inBufSize [in] Specifies the size in bytes of the input request
 *		object structure buffer.
 *
 * @param pOutBuf [out] Pointer to an output response object structure
 *		buffer which receives the output response from the Data Manager
 *		on completion of the request.
 *
 * @param outBufSize [in] Specifies the size in bytes of the output
 *		response object structure buffer.
 *
 * @param pBytesReturned [out] Pointer to a variable that receives
 *		the size in bytes copied to the output response object structure
 *		buffer.  When this API returns SM_STATUS_SUCCESS, the variable
 *		pointed to by pBytesReturned contains the number of bytes
 *		stored in the output response object structure buffer.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMServerAttach
 */
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNSMSERVERDISPATCH)(_IN u32 clientID,
										  _IN u32 reqType,
										  _IN u32 privLevel,
										  _IN const void *pInBuf,
										  _IN u32 inBufSize,
										  _OUT void *pOutBuf,
										  _IN u32 outBufSize,
										  _OUT u32 *pBytesReturned);


//---------------------------------------------------------------------
// SMServerAttach
/** 
 * Called by Data Manager at initialization to register as server with SMIL.
 *
 * This API method is called by the Data Manager (server) to specify 
 * the configuration to be used by the Data Manager and Data Consumer 
 * SMIL IPC Executives. This method is called at load and initialization 
 * time by the Data Manager (server) to register with the SMIL API 
 * layer and specify configuration parameters. These parameters may 
 * be read from an INI file by the Data Manager prior to making the 
 * call. The input SMServerCfg structure contains parameters that are 
 * typically worst case sizes and depths to be used. This method will 
 * fail to initialize successfully if a client has previously 
 * attached and never freed allocated resources by calling API 
 * method SMClientDetach. This occurs as the global IPC resources 
 * are still marked as allocated previously by the OS.
 *
 * @param pfnDispatch [in] Pointer to the Data Manager dispatch
 *		function that is called by SMIL to obtain a response from
 *		the Data Manager on execution of a Data Consumer request.
 *
 * @param pSCBuf [in] Pointer to a SMServerCfg structure buffer
 *		containing configuration information specified by the
 *		Data Manager. The Data Manager IPC Executive enforces this
 *		configuration for communications with the Data Consumer.
 *
 * @param scBufSize [in] Specifies size in bytes of the structure
 *		pointed to by pSCBuf.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMServerDetach, SMClientGetServerCfg
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMServerAttach(_IN PFNSMSERVERDISPATCH pfnDispatch,
								 _IN const SMServerCfg *pSCBuf,
								 _IN u32 scBufSize);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMSERVERATTACH)(_IN PFNSMSERVERDISPATCH pfnDispatch,
										  _IN const SMServerCfg *pSCBuf,
										  _IN u32 scBufSize);

/** Method string definition */
#define FSTRSMSERVERATTACH   "SMServerAttach"


//---------------------------------------------------------------------
// SMServerDetach
/** 
 * Called by Data Manager at unload to de-register as server with SMIL.
 *
 * This API method is called by the Data Manager (server) at unload 
 * and exit time to de-register with the SMIL IPC Executives. 
 * This causes any IPC resources to be freed.  If the Data Manager
 * calls this when the Data Consumer has submitted a request by
 * calling SMClientDispatch or has blocked on a SMClientGetEvent call,
 * then the IPC Executive wakes up the sleeping Data Consumer thread or
 * process and returns status SM_STATUS_DEVICE_NOT_READY to indicate
 * that the Data Manager is exiting.  Thus the Data Consumer can gracefully
 * perform a SMClientDetach and wait for the Data Manager to restart.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMServerAttach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMServerDetach(void);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMSERVERDETACH)(void);

/** Method string definition */
#define FSTRSMSERVERDETACH   "SMServerDetach"


//---------------------------------------------------------------------
// SMServerGenerateEvent
/** 
 * Called by Data Manager to generate an event to specified Data Consumer.
 *
 * This API method is called by the Data Manager (server) to send
 * an event to the specified Data Consumer.  All specified Data Consumers
 * that are currently blocked on a call to SMClientGetEvent will be
 * notified of the event.  If a clientID of SMIL_INVALID_DATA_CONSUMER_ID
 * is specified, this API method will send the event to all connected
 * Data Consumers.  If a specific clientID is specified, the event
 * is sent only to the specified Data Consumer.  The function returns
 * SM_STATUS_SUCCESS if it succeeds.  If the server exits due to a
 * detach sequence, a return status of SM_STATUS_DEVICE_NOT_READY is
 * returned.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		of the Data Consumer that is the target of the event.
 *
 * @param pEvtBuf [in] Pointer to the event structure buffer which
 *		contains the event data from the Data Manager for the event
 *		being generated.  This data is sent to the Data Consumer
 *		identified by clientID.  This Data Consumer must call the
 *		SMClientGetEvent API method to retrieve the event.
 *
 * @param evtBufSize [in] Specifies the size in bytes of the event
 *		structure buffer pointed to by pEvtBuf.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMServerGenerateEvent(_IN u32 clientID,
										_IN void *pEvtBuf,
										_IN u32 evtBufSize);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMSERVERGENERATEEVENT)(_IN u32 clientID,
												 _IN void *pEvtBuf,
												 _IN u32 evtBufSize);

/** Method string definition */
#define FSTRSMSERVERGENERATEEVENT   "SMServerGenerateEvent"


//---------------------------------------------------------------------
// Data Consumer API functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SMClientAttach
/** 
 * Called by Data Consumer at initialization to register with SMIL.
 *
 * This API is called by the Data Consumer to connect to the
 * Data Manager. In order to do this, the API will register this Data 
 * Consumer with the SMIL IPC API layer. On success, it receives 
 * an identifier handle that must be used for all further 
 * communications with the Data Manager. On error, the identifier
 * handle contains SMIL_INVALID_DATA_CONSUMER_ID. If the Data Manager
 * is not ready or not running, then this method returns with
 * a status code of SM_STATUS_DEVICE_NOT_READY which can be used
 * in a well designed loop to wait for the Data Manager to become ready.
 *
 * @param pClientID [out] Pointer to a variable that receives the
 *		Data Consumer identifier handle.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientDetach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMClientAttach(_OUT u32 *pClientID);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMCLIENTATTACH)(_OUT u32 *pClientID);

/** Method string definition */
#define FSTRSMCLIENTATTACH   "SMClientAttach"


//---------------------------------------------------------------------
// SMClientDetach
/** 
 * Called by Data Consumer at unload to de-register with SMIL.
 *
 * This API is called by the Data Consumer at unload to de-register 
 * itself with the SMIL IPC API layer. On success, it disconnects the Data 
 * Consumer from communicating with the Data Manager. This API frees 
 * any event, request and response queues associated with the 
 * identified Data Consumer. The Data Consumer cannot send additional 
 * requests or listen for events on successful return. Additionally 
 * on successful return, the SMIL IPC API layer wakes up any threads 
 * currently blocked on listening for events due to calling 
 * SMClientGetEvent. The SMClientGetEvent API method will return 
 * with a status of SM_STATUS_DEVICE_NOT_READY.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		obtained previously on a successful call to SMClientAttach.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientDetach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMClientDetach(_IN u32 clientID);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMCLIENTDETACH)(_IN u32 clientID);

/** Method string definition */
#define FSTRSMCLIENTDETACH   "SMClientDetach"


//---------------------------------------------------------------------
// SMClientDispatch
/** 
 * Called by Data Consumer to dispatch requests and receive responses
 * from Data Manager.
 *
 * This API is called by the Data Consumer to obtain a response from
 * the Data Manager on execution of an input request. This API method 
 * is used to perform operations on Data Objects, Data Events and Data 
 * Populators. The specific request types and their corresponding data 
 * structures are outlined in the HLD. This is the API used for 
 * dispatching requests to the Data Manager (server). On dispatch,
 * the request is queued to the registered Data Manager (server). 
 * If no request server is registered (not ready), then the API 
 * returns immediately with SM_STATUS_DEVICE_NOT_READY. If a 
 * registered server is present and processing requests, then the 
 * Data Consumer thread or process passes the request to the server 
 * and enters the sleep state until the Data Manager (server) 
 * generates a response to this request. On response generation,
 * the Data Consumer thread or process returns to the ready state
 * and continues processing. If the Data Manager or in-turn the
 * Data Populator does not return a response in time, then the API 
 * method wakes the Data Consumer thread or process and returns 
 * SM_STATUS_TIMEOUT. The API blocks until a response is generated 
 * by the server. The input buffer corresponding to the request type
 * (reqType) is pointed to by pInBuf, and its size is specified in the
 * parameter inBufSize. Similarly the output buffer corresponding to
 * the request type (reqType) is pointed to by pOutBuf, and its size
 * is specified in outBufSize. The actual number of bytes returned by
 * the server is set into a parameter pointed to by pBytesReturned.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		obtained previously on a successful call to SMClientAttach.
 *
 * @param reqType [in] Specifies the request type (operation) to be
 *		performed by the Data Manager based on parameters passed in
 *		pInBuf and produces output into pOutBuf.
 *
 * @param pInBuf [in] Pointer to the input request structure buffer
 *		corresponding to the request type supplied in reqType. The
 *		input request structure buffer size and contents correspond
 *		to the operation requested.
 *
 * @param inBufSize [in] Specifies the size in bytes of the input
 *		request structure buffer pointed to by pInBuf.
 *
 * @param pOutBuf [out] Pointer to an output response structure buffer
 *		which receives the output response from the Data Manager on
 *		completion of the request.
 *
 * @param outBufSize [in] Specifies the size in bytes of the output
 *		response structure buffer pointed to by pOutBuf.
 *
 * @param pBytesReturned [out] Pointer to a variable that receives
 *		the size in bytes copied to the output response structure
 *		buffer. When the method returns, the variable pointed to by
 *		pBytesReturned contains the number of bytes stored in the
 *		output response structure buffer.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientAttach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMClientDispatch(_IN u32 clientID,
								   _IN u32 reqType,
								   _IN const void *pInBuf,
								   _IN u32 inBufSize,
								   _OUT void *pOutBuf,
								   _IN u32 outBufSize,
								   _OUT u32 *pBytesReturned);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMCLIENTDISPATCH)(_IN u32 clientID,
											_IN u32 reqType,
											_IN const void *pInBuf,
											_IN u32 inBufSize,
											_OUT void *pOutBuf,
											_IN u32 outBufSize,
											_OUT u32 *pBytesReturned);

/** Method string definition */
#define FSTRSMCLIENTDISPATCH   "SMClientDispatch"


//---------------------------------------------------------------------
// SMClientGetEvent
/** 
 * Called by Data Consumer to retrieve events from Data Manager.
 *
 * This API is called by the Data Consumer to obtain an event 
 * generated by the Data Manager or a Data Populator through the 
 * Data Manager. The Data Consumer that is interested in listening 
 * for events typically starts a thread and calls this API method 
 * in a structured while loop until it exits. This API method examines 
 * the Data Consumer event queue for events being placed into it. 
 * If there are no events outstanding, it places the calling thread 
 * into a sleep state. When the Data Manager sends an event to 
 * the sleeping Data Consumer, this API is notified of the event.
 * The API awakens, pops the event off the head of the event queue
 * (FIFO queue) and returns the event to the Data Consumer. The event queue
 * is a wrap-around FIFO queue that wraps when SMServerCfg.minEvtQueueDepth
 * events have been placed into the FIFO queue. The event structure
 * and event types are described in the HLD. The specific event types
 * are typically thereafter processed by the Data Consumer in the same
 * thread context using a switch/case structured statement. If no request 
 * server is registered (not ready), then the API returns immediately 
 * with SM_STATUS_DEVICE_NOT_READY. If a registered server is present 
 * and then exits, this event thread is informed by waking up and 
 * receiving a status of SM_STATUS_DEVICE_NOT_READY. Thereafter the 
 * Data Consumer must call API method SMClientDetach to detach and 
 * free any IPC resources allocated and allow the server to exit 
 * gracefully.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		obtained previously on a successful call to SMClientAttach.
 *
 * @param pEvtBuf [out] Pointer to an event structure buffer which
 *		receives the event data from the Data Manager on an event
 *		being generated by the Data Manager or by a Data Populator
 *		through the Data Manager.
 *
 * @param evtBufSize [in] Specifies the size in bytes of the event
 *		structure buffer.
 *
 * @param pBytesReturned [out] Pointer to a variable that receives
 *		the size in bytes copied to the event structure buffer.
 *		When the method returns, the variable pointed to by
 *		pBytesReturned contains the number of bytes stored in the
 *		event structure buffer.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientAttach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMClientGetEvent(_IN u32 clientID,
								   _OUT void *pEvtBuf,
								   _IN u32 evtBufSize,
								   _OUT u32 *pBytesReturned);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMCLIENTGETEVENT)(_IN u32 clientID,
											_OUT void *pEvtBuf,
											_IN u32 evtBufSize,
											_OUT u32 *pBytesReturned);

/** Method string definition */
#define FSTRSMCLIENTGETEVENT   "SMClientGetEvent"


//---------------------------------------------------------------------
// SMClientGetServerCfg
/** 
 * Called by Data Consumer to retrieve vital server side information.
 *
 * This API is called by the Data Consumer to obtain configuration
 * parameters provided to the SMIL IPC API layer by the Data Manager.
 * This API is typically called prior to calling SMClientDispatch 
 * and SMClientGetEvent since it allows the Data Consumer to 
 * allocate worst case data buffers to hold requests, responses, 
 * Data Objects and Data Events.
 *
 * @param clientID [in] Specifies the Data Consumer identifier handle
 *		obtained previously on a successful call to SMClientAttach.
 *
 * @param pSCBuf [out] Pointer to the SMServerCfg structure buffer
 *		that receives information about the Data Manager specified
 *		configuration.
 *
 * @param pSCBufSize [inout] Pointer to a variable that specifies
 *		the size in bytes of the structure buffer pointed to by the
 *		pSCBuf parameter. When the method returns, the variable pointed
 *		to by pSCBufSize contains the number of bytes stored in the
 *		structure buffer.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 *		On error returns another status code defined in dcsmstatus.h.
 *
 * @see SMClientAttach
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMClientGetServerCfg(_IN u32 clientID,
									   _OUT SMServerCfg *pSCBuf,
									   _INOUT u32 *pSCBufSize);

/** Method type definition */
typedef s32 (_cdecl *FPROCSMCLIENTGETSERVERCFG)(_IN u32 clientID,
												_OUT SMServerCfg *pSCBuf,
												_INOUT u32 *pSCBufSize);

/** Method string definition */
#define FSTRSMCLIENTGETSERVERCFG   "SMClientGetServerCfg"


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCSMILAPI_H_INCLUDED__

