
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2011 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//
//  Abstract/Purpose:
//    SM status definitions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

/** @file */

#ifndef __DCSMSTATUS_H_INCLUDED__
#define __DCSMSTATUS_H_INCLUDED__


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
/**
 * Global Status Definitions
 * All negative error codes are considered reserved for future use
 */
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Unsuccessful. */
//---------------------------------------------------------------------
#define SM_STATUS_UNSUCCESSFUL				(-1)

//---------------------------------------------------------------------
/** Success. */
//---------------------------------------------------------------------
#define SM_STATUS_SUCCESS					(0)

//---------------------------------------------------------------------
/** Used for offset on definitions. */
//---------------------------------------------------------------------
#define SM_STATUS_OFFSET					(SM_STATUS_SUCCESS)

//---------------------------------------------------------------------
/** Minimum value of the positive range. */
//---------------------------------------------------------------------
#define SM_STATUS_MIN_POSITIVE_RANGE		(1)

//---------------------------------------------------------------------
/** Maximum value of the positive range. */
//---------------------------------------------------------------------
#define SM_STATUS_MAX_POSITIVE_RANGE		(2147483647)

//---------------------------------------------------------------------
/** Minimum value of the negative range. */
//---------------------------------------------------------------------
#define SM_STATUS_MIN_NEGATIVE_RANGE		(-1)

//---------------------------------------------------------------------
/** Maximum value of the negative range. */
//---------------------------------------------------------------------
#define SM_STATUS_MAX_NEGATIVE_RANGE		(-2147483648)


//---------------------------------------------------------------------
/**
 * Range definitions used for positive error codes
 */
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 1 - 255 is range used by ESM firmware and HIP device drivers
 * IPMI completion code lo range is 0xC0.
 * IPMI completion code hi range is 0xFF. */
//---------------------------------------------------------------------

/** Minimum in ESM firmware and HIP device drivers range */
#define SM_STATUS_DEVICE_DRIVER_LO			(1)

/** Maximum in ESM firmware and HIP device drivers range */
#define SM_STATUS_DEVICE_DRIVER_HI			(255)


//---------------------------------------------------------------------
/** 256 - 1023 is range used by Data Engine and HIP */
//---------------------------------------------------------------------

/** Minimum in Data Engine and HIP range */
#define SM_STATUS_DATA_ENGINE_LO			(256)

/** Maximum in Data Engine and HIP range */
#define SM_STATUS_DATA_ENGINE_HI			(1023)


//---------------------------------------------------------------------
/** 1024 - 2047 is range used by ERA/DRSC */
//---------------------------------------------------------------------

/** Minimum in ERA/DRSC range */
#define SM_STATUS_DRSC_LO					(1024)

/** Maximum in ERA/DRSC range */
#define SM_STATUS_DRSC_HI					(2047)


//---------------------------------------------------------------------
/** 2048 - 3071 is range used by Storage Services */
//---------------------------------------------------------------------

/** Minimum in Storage Services range */
#define SM_STATUS_STORAGE_SVCS_LO			(2048)

/** Maximum in Storage Services range */
#define SM_STATUS_STORAGE_SVCS_HI			(3071)


//---------------------------------------------------------------------
/** 3072 - 3327 is range used by Change Management */
//---------------------------------------------------------------------

/** Minimum in Change Management range */
#define SM_STATUS_CHANGE_MGMT_LO			(3072)

/** Maximum in Change Management range */
#define SM_STATUS_CHANGE_MGMT_HI			(3327)


//---------------------------------------------------------------------
/** 3328 - 3583 is range used by OM Remote */
//---------------------------------------------------------------------

/** Minimum in OM Remote range */
#define SM_STATUS_OM_REMOTE_LO				(3328)

/** Maximum in OM Remote range */
#define SM_STATUS_OM_REMOTE_HI				(3583)


//---------------------------------------------------------------------
/** 4096 - 4351 is range used for IPMI status (with completion code) */
//---------------------------------------------------------------------

/** Minimum in IPMI status range */
#define SM_STATUS_IPMI_STATUS_LO			(4096)

/** Maximum in IPMI status code */
#define SM_STATUS_IPMI_STATUS_HI			(4351)


//---------------------------------------------------------------------
/** 8192 - 9215 is range used by Instrumentation Service */
//---------------------------------------------------------------------

/** Minimum in Instrumentation Service range */
#define SM_STATUS_INST_SVC_LO				(8192)

/** Maximum in Instrumentation Service range */
#define SM_STATUS_INST_SVC_HI				(9215)


//---------------------------------------------------------------------
/**
 * Device Driver error codes
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Command not implemented. */
//---------------------------------------------------------------------
#define SM_STATUS_CMD_NOT_IMPLEMENTED		(SM_STATUS_OFFSET + 1) 

//---------------------------------------------------------------------
/** Command is bad or unknown. */
//---------------------------------------------------------------------
#define SM_STATUS_CMD_BAD					(SM_STATUS_OFFSET + 2) 

//---------------------------------------------------------------------
/** Timeout has occurred. */
//---------------------------------------------------------------------
#define SM_STATUS_TIMEOUT					(SM_STATUS_OFFSET + 3) 

//---------------------------------------------------------------------
/** Host initiated cancelled command. */
//---------------------------------------------------------------------
#define SM_STATUS_CANCELLED					(SM_STATUS_OFFSET + 4) 

//---------------------------------------------------------------------
/** Device initiated cancelled command. */
//---------------------------------------------------------------------
#define SM_STATUS_ABORT						(SM_STATUS_OFFSET + 5) 

//---------------------------------------------------------------------
/** Pending. */
//---------------------------------------------------------------------
#define SM_STATUS_PENDING					(SM_STATUS_OFFSET + 6) 

//---------------------------------------------------------------------
/** No such device. */
//---------------------------------------------------------------------
#define SM_STATUS_NO_SUCH_DEVICE			(SM_STATUS_OFFSET + 7) 

//---------------------------------------------------------------------
/** Device is busy. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_BUSY				(SM_STATUS_OFFSET + 8) 

//---------------------------------------------------------------------
/** Device is bad or unknown. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_BAD				(SM_STATUS_OFFSET + 9) 

//---------------------------------------------------------------------
/** Device spurious interrupt. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_SPURIOUS_INTR		(SM_STATUS_OFFSET + 10) 

//---------------------------------------------------------------------
/** Resource is locked. */
//---------------------------------------------------------------------
#define SM_STATUS_RESOURCE_LOCKED			(SM_STATUS_OFFSET + 11) 

//---------------------------------------------------------------------
/** Invalid index. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_INDEX				(SM_STATUS_OFFSET + 12) 

//---------------------------------------------------------------------
/** SMB error. */
//---------------------------------------------------------------------
#define SM_STATUS_SMB_ERROR					(SM_STATUS_OFFSET + 13) 

//---------------------------------------------------------------------
/** Data requested is not available. */
//---------------------------------------------------------------------
#define SM_STATUS_DATA_NOT_READY			(SM_STATUS_OFFSET + 14) 

//---------------------------------------------------------------------
/** Data requested is > data available. */
//---------------------------------------------------------------------
#define SM_STATUS_DATA_UNDERRUN				(SM_STATUS_OFFSET + 15) 

//---------------------------------------------------------------------
/** Data requested is < data available. */
//---------------------------------------------------------------------
#define SM_STATUS_DATA_OVERRUN				(SM_STATUS_OFFSET + 16) 

//---------------------------------------------------------------------
/** Device not yet ready to accept commands, try later. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_NOT_READY			(SM_STATUS_OFFSET + 17) 

//---------------------------------------------------------------------
/** Device does not have data example disk size = 0 bytes. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_EMPTY				(SM_STATUS_OFFSET + 18) 

//---------------------------------------------------------------------
/** Device full of data example disk full, out of memory. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_FULL				(SM_STATUS_OFFSET + 19) 

//---------------------------------------------------------------------
/** Device already exists. */
//---------------------------------------------------------------------
#define SM_STATUS_DEVICE_ALREADY_EXISTS		(SM_STATUS_OFFSET + 20) 

//---------------------------------------------------------------------
/** Pending DOIO Queue. */
//---------------------------------------------------------------------
#define SM_STATUS_PENDING_DOIO_QUEUE		(SM_STATUS_OFFSET + 21) 

//---------------------------------------------------------------------
/** Pending Busy Queue. */
//---------------------------------------------------------------------
#define SM_STATUS_PENDING_BUSY_QUEUE		(SM_STATUS_OFFSET + 22) 

//---------------------------------------------------------------------
/** BIOS password protection. */
//---------------------------------------------------------------------
#define SM_STATUS_BIOS_PASSWORD_PROTECT		(SM_STATUS_OFFSET + 23) 


//---------------------------------------------------------------------
/**
 * IPMI specific error codes
 */
//---------------------------------------------------------------------

// Below are some values within the IPMI status range.
// Should be considered obselete.
// For backward compatibility support only.

//---------------------------------------------------------------------
/** Invalid command for a given LUN. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_CMD_LUN			(SM_STATUS_IPMI_STATUS_LO + 194)

//---------------------------------------------------------------------
/** Parameter out of range. */
//---------------------------------------------------------------------
#define SM_STATUS_PARAM_OUT_OF_RANGE		(SM_STATUS_IPMI_STATUS_LO + 201)

//---------------------------------------------------------------------
/** Invalid data field in request. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_DATA_FIELD		(SM_STATUS_IPMI_STATUS_LO + 204)


//---------------------------------------------------------------------
/**
 * HAPI specific error codes
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Restore to defaults in progress. */
//---------------------------------------------------------------------
#define SM_STATUS_RESTORE_DEFAULT_IN_PROGRESS	(SM_STATUS_INST_SVC_LO + 23) 

//---------------------------------------------------------------------
/** Invalid value. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_VALUE				(SM_STATUS_INST_SVC_LO + 24) 

//---------------------------------------------------------------------
/** Not absolute file path name. */
//---------------------------------------------------------------------
#define SM_STATUS_NOT_ABSOLUTE_PATH			(SM_STATUS_INST_SVC_LO + 25) 

//---------------------------------------------------------------------
/** Failed to get file info. */
//---------------------------------------------------------------------
#define SM_STATUS_FAILED_FILE_INFO			(SM_STATUS_INST_SVC_LO + 26) 

//---------------------------------------------------------------------
/** Wrong file permission. */
//---------------------------------------------------------------------
#define SM_STATUS_WRONG_FILE_PERM			(SM_STATUS_INST_SVC_LO + 27) 

//---------------------------------------------------------------------
/** FRU not supported. */
//---------------------------------------------------------------------
#define SM_STATUS_FRU_UNSUPPORTED			(SM_STATUS_INST_SVC_LO + 28) 

//---------------------------------------------------------------------
/** FRU not available. */
//---------------------------------------------------------------------
#define SM_STATUS_FRU_UNAVAILABLE			(SM_STATUS_INST_SVC_LO + 29) 

//---------------------------------------------------------------------
/** FRU checksum incorrect. */
//---------------------------------------------------------------------
#define SM_STATUS_FRU_CHECKSUM_INCORRECT	(SM_STATUS_INST_SVC_LO + 30) 

//---------------------------------------------------------------------
/** FRU data corrupted. */
//---------------------------------------------------------------------
#define SM_STATUS_FRU_CORRUPTED				(SM_STATUS_INST_SVC_LO + 31) 

//---------------------------------------------------------------------
/** Indicate duplicate user name */
//--------------------------------------------------------------------
#define SM_STATUS_DUPLICATE_USER_NAME		(SM_STATUS_INST_SVC_LO + 32) 

//---------------------------------------------------------------------
/** Mutex lock not attached. */
//---------------------------------------------------------------------
#define SM_STATUS_MUTEX_LOCK_NOT_ATTACHED	(SM_STATUS_INST_SVC_LO + 33) 



//---------------------------------------------------------------------
/** MASER command success. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_SUCCESS					(SM_STATUS_INST_SVC_LO + 50) 

//---------------------------------------------------------------------
/** MASER command failure. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_FAILURE					(SM_STATUS_INST_SVC_LO + 51) 

//---------------------------------------------------------------------
/** MASER disbaled. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_DISABLED					(SM_STATUS_INST_SVC_LO + 52) 

//---------------------------------------------------------------------
/** MASER session in progress. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_SESSION_INPROGRESS			(SM_STATUS_INST_SVC_LO + 53) 

//---------------------------------------------------------------------
/** MASER partition already attached. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_PARTITIONS_ATTACHED			(SM_STATUS_INST_SVC_LO + 54) 

//---------------------------------------------------------------------
/** MASER not in manufaturing mode. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_NOT_MANUFACTURING_MODE		(SM_STATUS_INST_SVC_LO + 55) 

//---------------------------------------------------------------------
/** MASER dynamic partition exists. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_DYNAMIC_PARTITION_EXISTS	(SM_STATUS_INST_SVC_LO + 56) 

//---------------------------------------------------------------------
/** MASER attached drive could not be mounted. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_MOUNTED_DRIVE_NOT_FOUND		(SM_STATUS_INST_SVC_LO + 57) 

//---------------------------------------------------------------------
/** MASER LCL file not found. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_LCL_FILE_NOT_FOUND			(SM_STATUS_INST_SVC_LO + 58) 

//---------------------------------------------------------------------
/** MASER LCL file not accessible. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_LCL_FILE_NOT_ACCESSIBLE		(SM_STATUS_INST_SVC_LO + 59) 

//---------------------------------------------------------------------
/** MASER cmd out of memory. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_OUT_OF_MEMORY			(SM_STATUS_INST_SVC_LO + 60) 

//---------------------------------------------------------------------
/** MASER cmd pending. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_PENDING					(SM_STATUS_INST_SVC_LO + 61) 

//---------------------------------------------------------------------
/** MASER cmd INVALID handle. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_INVALID_HANDLE			(SM_STATUS_INST_SVC_LO + 62)

//---------------------------------------------------------------------
/** MASER cmd data overflow. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_DATA_OVERFLOW			(SM_STATUS_INST_SVC_LO + 63)

//---------------------------------------------------------------------
/** MASER cmd INVALID bitmap. */
//---------------------------------------------------------------------
#define SM_STATUS_MASER_CMD_INVALID_BITMAP			(SM_STATUS_INST_SVC_LO + 64)



//---------------------------------------------------------------------
/** LCL destination directory size could not be determined. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_FREE_SIZE_NOT_DETERMINED	(SM_STATUS_INST_SVC_LO + 70) 

//---------------------------------------------------------------------
/** LCL destination directory does not have space left. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_DIR_SPACE_NOT_LEFT		(SM_STATUS_INST_SVC_LO + 71) 

//---------------------------------------------------------------------
/** LCL destination directory file copy failed. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_DIR_FILE_COPY_FAILED		(SM_STATUS_INST_SVC_LO + 72) 

//---------------------------------------------------------------------
/** LCL destination is not a directory. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_IS_NOT_A_DIRECTORY		(SM_STATUS_INST_SVC_LO + 73) 

//---------------------------------------------------------------------
/** LCL destination path not found. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_PATH_NOT_FOUND			(SM_STATUS_INST_SVC_LO + 74) 

//---------------------------------------------------------------------
/** LCL destination path access denied. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_PATH_ACCESS_DENIED		(SM_STATUS_INST_SVC_LO + 75) 

//---------------------------------------------------------------------
/** LCL destination too many open files. */
//---------------------------------------------------------------------
#define SM_STATUS_LCL_DST_TOO_MANY_OPEN_FILES		(SM_STATUS_INST_SVC_LO + 76)

//---------------------------------------------------------------------
/** Alt Destination Address format is not correct(IV4/IPV6/FQDN) */
//---------------------------------------------------------------------

#define SM_STATUS_ALT_DEST_FAIL		(SM_STATUS_INST_SVC_LO + 80)


//---------------------------------------------------------------------
/** Invalid command. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_COMMAND			(SM_STATUS_INST_SVC_LO + 193) 


//---------------------------------------------------------------------
/**
 * Data Engine error codes
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Object not found. */
//---------------------------------------------------------------------
#define SM_STATUS_NO_SUCH_OBJECT			(SM_STATUS_OFFSET + 256) 

//---------------------------------------------------------------------
/** Object malformed, corrupt or bad. */
//---------------------------------------------------------------------
#define SM_STATUS_OBJECT_BAD				(SM_STATUS_OFFSET + 257) 

//---------------------------------------------------------------------
/** Object already exists. */
//---------------------------------------------------------------------
#define SM_STATUS_OBJECT_ALREADY_EXISTS		(SM_STATUS_OFFSET + 258) 

//---------------------------------------------------------------------
/** INI file not found. */
//---------------------------------------------------------------------
#define SM_STATUS_NO_SUCH_INI				(SM_STATUS_OFFSET + 260) 

//---------------------------------------------------------------------
/** INI malformed, corrupt or bad. */
//---------------------------------------------------------------------
#define SM_STATUS_INI_BAD					(SM_STATUS_OFFSET + 261) 

//---------------------------------------------------------------------
/** INI section does not exist. */
//---------------------------------------------------------------------
#define SM_STATUS_INI_NO_SUCH_SECTION		(SM_STATUS_OFFSET + 262) 

//---------------------------------------------------------------------
/** INI section does not exist. */
//---------------------------------------------------------------------
#define SM_STATUS_INI_NO_SUCH_KEY			(SM_STATUS_OFFSET + 263) 

//---------------------------------------------------------------------
/** INI value malformed, corrupt or bad. */
//---------------------------------------------------------------------
#define SM_STATUS_INI_VALUE_BAD				(SM_STATUS_OFFSET + 264) 

//---------------------------------------------------------------------
/** Insufficient access rights. */
//---------------------------------------------------------------------
#define SM_STATUS_RIGHTS_INSUFFICIENT		(SM_STATUS_OFFSET + 270) 

//---------------------------------------------------------------------
/** Invalid parameter name or value. */
//---------------------------------------------------------------------
#define SM_STATUS_INVALID_PARAMETER			(SM_STATUS_OFFSET + 271) 

//---------------------------------------------------------------------
/** Out of memory. */
//---------------------------------------------------------------------
#define SM_STATUS_OUT_OF_MEMORY				(SM_STATUS_OFFSET + 272) 

//---------------------------------------------------------------------
/** Resource not available. */
//---------------------------------------------------------------------
#define SM_STATUS_RESOURCE_NOT_AVAILABLE	(SM_STATUS_OFFSET + 273) 

//---------------------------------------------------------------------
/** Disconnection has occurred. */
//---------------------------------------------------------------------
#define SM_STATUS_DISCONNECTED				(SM_STATUS_OFFSET + 274) 

//---------------------------------------------------------------------
/** XML file not found. */
//---------------------------------------------------------------------
#define SM_STATUS_NO_SUCH_XML				(SM_STATUS_OFFSET + 280) 

//---------------------------------------------------------------------
/** XML malformed, corrupt or bad. */
//---------------------------------------------------------------------
#define SM_STATUS_XML_BAD					(SM_STATUS_OFFSET + 281) 

//---------------------------------------------------------------------
/** XML node does not exist. */
//---------------------------------------------------------------------
#define SM_STATUS_XML_NO_SUCH_NODE			(SM_STATUS_OFFSET + 282) 

//---------------------------------------------------------------------
/** XML value malformed, corrupt or bad. */
//---------------------------------------------------------------------
#define SM_STATUS_XML_NODE_VALUE_BAD		(SM_STATUS_OFFSET + 283) 

//---------------------------------------------------------------------
/** Value larger than expected. */
//---------------------------------------------------------------------
#define SM_STATUS_VALUE_TOO_LARGE			(SM_STATUS_OFFSET + 290) 

//---------------------------------------------------------------------
/** Value smaller than expected. */
//---------------------------------------------------------------------
#define SM_STATUS_VALUE_TOO_SMALL			(SM_STATUS_OFFSET + 291) 

//---------------------------------------------------------------------
/** Key not found. */
//---------------------------------------------------------------------
#define SM_STATUS_KEY_NOT_FOUND				(SM_STATUS_OFFSET + 295)

//---------------------------------------------------------------------
/** Key already exists. */
//---------------------------------------------------------------------
#define SM_STATUS_KEY_ALREADY_EXISTS		(SM_STATUS_OFFSET + 296)

//---------------------------------------------------------------------
/** OS SNMP support not found. */
//---------------------------------------------------------------------
#define SM_STATUS_OS_SNMP_NOT_FOUND			(SM_STATUS_OFFSET + 300) 

//---------------------------------------------------------------------
/** Data Engine SNMP support not found. */
//---------------------------------------------------------------------
#define SM_STATUS_DE_SNMP_NOT_FOUND			(SM_STATUS_OFFSET + 301) 

//---------------------------------------------------------------------
/** Indicates an error occurred while processing a script. */
//---------------------------------------------------------------------
#define SM_STATUS_DECMD_SCRIPT_ERROR		(SM_STATUS_OFFSET + 302) 

//---------------------------------------------------------------------
/** IPv4/IPv6 setting not changed due to NIC Disable condition. */
//---------------------------------------------------------------------
#define SM_STATUS_IPV4V6_NOT_SET_NIC_DISABLED	(SM_STATUS_OFFSET + 303)

//---------------------------------------------------------------------
/** No change in state as a result of an action requested */
//---------------------------------------------------------------------
#define SM_STATUS_NO_STATE_CHANGE			(SM_STATUS_OFFSET + 305) 



//---------------------------------------------------------------------
/**
 * DRSC/ESM error codes
 */
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Request Received Too Much Data. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_DATA_OVERFLOW					(SM_STATUS_DRSC_LO + 0) 

//---------------------------------------------------------------------
/** IPMI Request Response Failure. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_IPMI_FAILURE						(SM_STATUS_DRSC_LO + 1) 

//---------------------------------------------------------------------
/** Timeout Waiting For DRSC To Complete FRU Read Write. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_TIMEOUT							(SM_STATUS_DRSC_LO + 2) 

//---------------------------------------------------------------------
/** Received Invalid Message ID from DRSC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_MESSAGE_ID_MISMATCH				(SM_STATUS_DRSC_LO + 3) 

//---------------------------------------------------------------------
/** Unable to Malloc Data Buffer. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_UNABLE_TO_ALLOCATE_BUFFER		(SM_STATUS_DRSC_LO + 4) 

//---------------------------------------------------------------------
/** Unable to Parse Data From DRSC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_UNABLE_TO_PARSE_DATA             (SM_STATUS_DRSC_LO + 5) 

//---------------------------------------------------------------------
/** Received Set Request for Invalid Object. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_SET_REQ_TYPE			    (SM_STATUS_DRSC_LO + 6) 

//---------------------------------------------------------------------
/** Received Set Request for Invalid Field Within Object. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_SET_REQ_SUBTYPE		    (SM_STATUS_DRSC_LO + 7) 

//---------------------------------------------------------------------
/** Failure Code Received from DRSC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_FAILURE_FROM_DRSC			    (SM_STATUS_DRSC_LO + 8) 

//---------------------------------------------------------------------
/** GENERIC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_GENERIC							(SM_STATUS_DRSC_LO + 9)	

//---------------------------------------------------------------------
/** Can't Alloc Msg Buf. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_CANT_ALLOC_MSG_BUF				(SM_STATUS_DRSC_LO + 10) 

//---------------------------------------------------------------------
/** Out of Msg Buffers. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OUT_OF_MSG_BUFFERS				(SM_STATUS_DRSC_LO + 11) 

//---------------------------------------------------------------------
/** Chksum Size Too Big. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_CHKSUM_SIZE_TOO_BIG				(SM_STATUS_DRSC_LO + 12) 

//---------------------------------------------------------------------
/** Chksum Ms gdata Null. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_CHKSUM_MSGDATA_NULL				(SM_STATUS_DRSC_LO + 13) 

//---------------------------------------------------------------------
/** Invalid Pkt Type. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_PKT_TYPE					(SM_STATUS_DRSC_LO + 14) 

//---------------------------------------------------------------------
/** Device ID not Found on Open. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_DEVID_NOT_FOUND_ON_OPEN			(SM_STATUS_DRSC_LO + 15) 

//---------------------------------------------------------------------
/** Device ID not Found on Destroy. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_DEVID_NOT_FOUND_ON_DESTROY		(SM_STATUS_DRSC_LO + 16) 

//---------------------------------------------------------------------
/** Invalid Flags on Open. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_FLAGS_ON_OPEN			(SM_STATUS_DRSC_LO + 17) 

//---------------------------------------------------------------------
/** Write Msg Too Big. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_WRITE_MSG_TOO_BIG				(SM_STATUS_DRSC_LO + 18) 

//---------------------------------------------------------------------
/** Invalid Msg Buf Entry. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_MSG_BUF_ENTRY			(SM_STATUS_DRSC_LO + 19) 

//---------------------------------------------------------------------
/** Invalid Request Msg Type. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_REQUEST_MSG_TYPE			(SM_STATUS_DRSC_LO + 20) 

//---------------------------------------------------------------------
/** Invalid Chksum. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_CHKSUM					(SM_STATUS_DRSC_LO + 21) 

//---------------------------------------------------------------------
/** Invalid Msgsize. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_MSGSIZE					(SM_STATUS_DRSC_LO + 22) 

//---------------------------------------------------------------------
/** Invalid Msgwrite. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_MSGWRITE					(SM_STATUS_DRSC_LO + 23) 

//---------------------------------------------------------------------
/** Invalid Msgread. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_INVALID_MSGREAD					(SM_STATUS_DRSC_LO + 24) 

//---------------------------------------------------------------------
/** Write Msg Too Big RC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_WRITE_MSG_TOO_BIG_RC				(SM_STATUS_DRSC_LO + 25) 

//---------------------------------------------------------------------
/** Read Msg Too Big RC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_READ_MSG_TOO_BIG_RC				(SM_STATUS_DRSC_LO + 26) 

//---------------------------------------------------------------------
/** Read Escmsg Too Big RC. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_READ_ESCMSG_TOO_BIG_RC			(SM_STATUS_DRSC_LO + 27) 

//---------------------------------------------------------------------
/** Db Prop. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_DB_PROP							(SM_STATUS_DRSC_LO + 28) 

//---------------------------------------------------------------------
/** OID Permis Denied Write. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_PERMIS_DENIED_WRITE			(SM_STATUS_DRSC_LO + 29) 

//---------------------------------------------------------------------
/** OID Permis Denied Read. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_PERMIS_DENIED_READ			(SM_STATUS_DRSC_LO + 30) 

//---------------------------------------------------------------------
/** OID Obj Type Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_TYPE_INVALID				(SM_STATUS_DRSC_LO + 31) 

//---------------------------------------------------------------------
/** OID Obj Type Cmdget Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_TYPE_CMDGET_INVALID		(SM_STATUS_DRSC_LO + 32) 

//---------------------------------------------------------------------
/** OID Group Size Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_GROUP_SIZE_INVALID			(SM_STATUS_DRSC_LO + 33) 

//---------------------------------------------------------------------
/** OID Propset Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_PROPSET_ERR					(SM_STATUS_DRSC_LO + 34) 

//---------------------------------------------------------------------
/** OID Propget Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_PROPGET_ERR					(SM_STATUS_DRSC_LO + 35) 

//---------------------------------------------------------------------
/** OID Cmdexec Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_CMDEXEC_ERR					(SM_STATUS_DRSC_LO + 36) 

//---------------------------------------------------------------------
/** OID Get Alloc Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_GET_ALLOC_ERR				(SM_STATUS_DRSC_LO + 37) 

//---------------------------------------------------------------------
/** OID Setcmd Alloc Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_SETCMD_ALLOC_ERR				(SM_STATUS_DRSC_LO + 38) 

//---------------------------------------------------------------------
/** OID Obj Group Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_GROUP_INVALID			(SM_STATUS_DRSC_LO + 39) 

//---------------------------------------------------------------------
/** OID Obj Column Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_COLUMN_INVALID			(SM_STATUS_DRSC_LO + 40) 

//---------------------------------------------------------------------
/** OID Get Output Too Big. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_GET_OUTPUT_TOO_BIG			(SM_STATUS_DRSC_LO + 41) 

//---------------------------------------------------------------------
/** OID Get Mem Overrun. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_GET_MEM_OVERRUN				(SM_STATUS_DRSC_LO + 42) 

//---------------------------------------------------------------------
/** OID Setcmd Mem Overrun. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_SETCMD_MEM_OVERRUN			(SM_STATUS_DRSC_LO + 43) 

//---------------------------------------------------------------------
/** OID Setcmd Cmdline Too Big. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_SETCMD_CMDLINE_TOO_BIG		(SM_STATUS_DRSC_LO + 44) 

//---------------------------------------------------------------------
/** OID Obj Type Stringset Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_TYPE_STRINGSET_INVALID	(SM_STATUS_DRSC_LO + 45) 

//---------------------------------------------------------------------
/** OID Obj Type Indexset Invalid. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_OBJ_TYPE_INDEXSET_INVALID	(SM_STATUS_DRSC_LO + 46) 

//---------------------------------------------------------------------
/** OID Stringget Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_STRINGGET_ERR				(SM_STATUS_DRSC_LO + 47) 

//---------------------------------------------------------------------
/** OID Indexget Err. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_INDEXGET_ERR					(SM_STATUS_DRSC_LO + 48) 

//---------------------------------------------------------------------
/** OID no User Found. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_OID_NO_USER_FOUND				(SM_STATUS_DRSC_LO + 49) 

//---------------------------------------------------------------------
/** Timeout Waiting For Drsc. */
//---------------------------------------------------------------------
#define SM_STATUS_DRSC_TIMEOUT_WAITING_FOR_DRSC			(SM_STATUS_DRSC_LO + 50) 


//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCSMSTATUS_H_INCLUDED__

