
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2009 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Dell SM support API definitions
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSUPDEF_H_INCLUDED__
#define __DCSUPDEF_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// SDK files
//---------------------------------------------------------------------
#include <dcossdk.h>


//---------------------------------------------------------------------
// Shared include files
//---------------------------------------------------------------------
#include <dcdetypes.h>
#include <dcsmstatus.h>


#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(1)


//---------------------------------------------------------------------
//  Defines and types
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  Common String Constants
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** Null string representation for names */
//---------------------------------------------------------------------
#define NULL_NAME_STR						"null"


//---------------------------------------------------------------------
/** Null string representation */
//---------------------------------------------------------------------
#define NULL_STR							"(" NULL_NAME_STR ")"


//---------------------------------------------------------------------
/** Standard true string */
//---------------------------------------------------------------------
#ifndef TRUE_STR
#define TRUE_STR							"true"
#endif

//---------------------------------------------------------------------
/** Standard false string */
//---------------------------------------------------------------------
#ifndef FALSE_STR
#define FALSE_STR							"false"
#endif

//---------------------------------------------------------------------
/** 
 * Boolean to string utility.
 * Convert boolean to standard true/false string.
 */
//---------------------------------------------------------------------
#ifndef BoolnToString
#define BoolnToString(x)					((x == TRUE) ? TRUE_STR : FALSE_STR)
#endif

//---------------------------------------------------------------------
// Compile-time Deprecation warning
//---------------------------------------------------------------------
#ifndef _SUPT_API_DEPRECATE

#ifdef SYSTEM_TYPE_WINNT
#if _MSC_FULL_VER >= 140050320
#define _SUPT_DEPRECATE_TEXT(_Text) __declspec(deprecated(_Text))
#else // _MSC_FULL_VER >= 140050320
#if _MSC_FULL_VER > 12008804
#define _SUPT_DEPRECATE_TEXT(_Text) __declspec(deprecated)
#else // _MSC_FULL_VER > 12008804
#define _SUPT_DEPRECATE_TEXT(_Text) // don't know how to message deprecation
#endif // _MSC_FULL_VER > 12008804
#endif // _MSC_FULL_VER >= 140050320
#define _SUPT_API_DEPRECATE(_NewName) _SUPT_DEPRECATE_TEXT("This name is deprecated. Instead, please use the security enhanced name: " #_NewName ".")
#endif // SYSTEM_TYPE_WINNT

#ifdef SYSTEM_TYPE_LINUX
#if defined (__GNUC__) && (__GNUC__ > 3 || __GNUC__ == 3 && __GNUC_MINOR__ > 0)
#define _SUPT_API_DEPRECATE(_NewName) __attribute__((deprecated))
#else
#define _SUPT_API_DEPRECATE(_NewName)
#warning Cannot define _SUPT_API_DEPRECATE, please use gcc version 3 and later
#endif
#endif // SYSTEM_TYPE_LINUX

#ifdef SYSTEM_TYPE_NETWARE
#define _SUPT_DEPRECATE_TEXT(_Text) // don't know how to message deprecation
#define _SUPT_API_DEPRECATE(_NewName) _SUPT_DEPRECATE_TEXT("This name is deprecated. Instead, please use the security enhanced name: " #_NewName ".")
#endif // SYSTEM_TYPE_NETWARE

#ifdef SYSTEM_TYPE_DOS
#define _SUPT_DEPRECATE_TEXT(_Text) // don't know how to message deprecation
#define _SUPT_API_DEPRECATE(_NewName) _SUPT_DEPRECATE_TEXT("This name is deprecated. Instead, please use the security enhanced name: " #_NewName ".")
#endif // SYSTEM_TYPE_DOS

#endif // _SUPT_API_DEPRECATE


//---------------------------------------------------------------------
// Bit quantities
//---------------------------------------------------------------------

/** Bit quantity: bits per byte */
#define SM_BITS_PER_BYTE					(8)

/** Bit quantity: bits per data type */
#define SM_BITS_PER_TYPE(T)					(sizeof(T) * SM_BITS_PER_BYTE)


//---------------------------------------------------------------------
// BitMap helper macros
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Apply the BitMap */
//---------------------------------------------------------------------
#ifndef SMMBitMapSetOn
#define SMMBitMapSetOn(bitMap, theBit)		(bitMap |= theBit)
#endif

//---------------------------------------------------------------------
/** Not Apply the BitMap */
//---------------------------------------------------------------------
#ifndef SMMBitMapSetOff
#define SMMBitMapSetOff(bitMap, theBit)		(bitMap &= ((-1) ^ theBit))
#endif

//---------------------------------------------------------------------
/** Check if BitMap is on (return true) or off (return false) */
//---------------------------------------------------------------------
#ifndef SMMBitMapIsOn
#define SMMBitMapIsOn(bitMap, theBit)		((bitMap & theBit) == theBit)
#endif

//---------------------------------------------------------------------
/** Check if BitMap is off (return true) or on (return false) */
//---------------------------------------------------------------------
#ifndef SMMBitMapIsOff
#define SMMBitMapIsOff(bitMap, theBit)		((bitMap & theBit) == 0)
#endif


//---------------------------------------------------------------------
// Format for printf and scanf functions
// use these to rid of compiler warnings
//---------------------------------------------------------------------

// u32/U32 in Linux is unsigned int; unsigned long otherwise

// Linux
#ifdef SYSTEM_TYPE_LINUX

//---------------------------------------------------------------------
/** 
 * Print format string: unsigned integer (32-bit), 
 * integer for Linux, long integer for Windows 
 */
//---------------------------------------------------------------------
#define PRT_U32								"%u"

//---------------------------------------------------------------------
/** 
 * Print format string: signed integer (32-bit), 
 * integer for Linux, long integer for Windows
 */
//---------------------------------------------------------------------
#define PRT_S32								"%d"

//---------------------------------------------------------------------
/** 
 * Print format string: hexadecimal integer (32-bit), 
 * integer for Linux, long integer for Windows 
 */
//---------------------------------------------------------------------
#define PRT_HEX_32							"%#x"

//---------------------------------------------------------------------
/** 
 * Print format string: unsigned hexadecimal integer (32-bit), 
 * integer for Linux, long integer for Windows 
 */
//---------------------------------------------------------------------
#define PRT_UHEX_32							"%#X"

//---------------------------------------------------------------------
/** 
 * Print format string: octal integer (32-bit), 
 * integer for Linux, long integer for Windows 
 */
//---------------------------------------------------------------------
#define PRT_OCT_32							"%o"

//---------------------------------------------------------------------
/** Print format string: double floating point (64-bit) */
//---------------------------------------------------------------------
#define PRT_DBL								"%lf"

//---------------------------------------------------------------------
/** 
 * Scan format string: signed hexadecimal integer (32-bit), 
 * integer for Linux, long integer for Windows 
 */
//---------------------------------------------------------------------
#define SCAN_HEX_32							"%x"

// Windows
#else

//---------------------------------------------------------------------
/** Print format string: unsigned integer (32-bit) */
//---------------------------------------------------------------------
#define PRT_U32								"%lu"

//---------------------------------------------------------------------
/** Print format string: signed integer (32-bit) */
//---------------------------------------------------------------------
#define PRT_S32								"%ld"

//---------------------------------------------------------------------
/** Print format string: hexadecimal integer (32-bit) */
//---------------------------------------------------------------------
#define PRT_HEX_32							"%#lx"

//---------------------------------------------------------------------
/** Print format string: unsigned hexadecimal integer (32-bit) */
//---------------------------------------------------------------------
#define PRT_UHEX_32							"%#lX"

//---------------------------------------------------------------------
/** Print format string: octal integer (32-bit) */
//---------------------------------------------------------------------
#define PRT_OCT_32							"%lo"

//---------------------------------------------------------------------
/** Print format string: double floating point (64-bit) */
//---------------------------------------------------------------------
#define PRT_DBL								"%lf"

//---------------------------------------------------------------------
/** Scan format string: signed hexadecimal integer (32-bit) */
//---------------------------------------------------------------------
#define SCAN_HEX_32							"%lx"

#endif


//---------------------------------------------------------------------
// System Management Variable Type Identifier Definitions
// The type identifier occupies bits [0...7]
// The type modifier occupies bits [8...15]
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Reserved - Valid Ranges
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type: valid low range */
//---------------------------------------------------------------------
#define SM_LO_VTYPE							(0x00)

//---------------------------------------------------------------------
/** System Management variable type: valid high range */
//---------------------------------------------------------------------
#define SM_HI_VTYPE							(0xF0)

//---------------------------------------------------------------------
// Reserved - Unspecified Type
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type: unspecified type (reserved) */
//---------------------------------------------------------------------
#define SM_VTYPE_UNDEFINED					(0xFF)

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type: boolean (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_BOOLN						(0x00)

//---------------------------------------------------------------------
// Little Endian integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 8-bit integer 
 * (8-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_s8							(0x01)

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 16-bit integer 
 * (16-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_s16						(0x02)

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 32-bit integer 
 * (32-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_s32						(0x03)

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 64-bit integer 
 * (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_s64						(0x04)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 8-bit integer 
 * (8-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_u8							(0x05)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 16-bit integer 
 * (16-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_u16						(0x06)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 32-bit integer 
 * (32-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_u32						(0x07)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 64-bit integer 
 * (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_u64						(0x08)

//---------------------------------------------------------------------
/** 
 * System Management variable type: double floating point 
 * (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_d64						(0x09)

//---------------------------------------------------------------------
// Big Endian integer types
//---------------------------------------------------------------------

// there is no big endian signed 8-bit integer

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 16-bit integer 
 * (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_S16						(0x21)

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 32-bit integer 
 * (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_S32						(0x22)

//---------------------------------------------------------------------
/** 
 * System Management variable type: signed 64-bit integer
 * (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_S64						(0x23)

// there is no big endian unsigned byte

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 16-bit integer 
 * (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_U16						(0x25)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 32-bit integer 
 * (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_U32						(0x26)

//---------------------------------------------------------------------
/** 
 * System Management variable type: unsigned 64-bit integer
 * (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_U64						(0x27)

//---------------------------------------------------------------------
/** 
 * System Management variable type: double floating point 
 * (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_D64						(0x28)

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type: language ID (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_LANGID						(0x0A)

//---------------------------------------------------------------------
/** System Management variable type: time stamp (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_TIME_T						(0x0B)

//---------------------------------------------------------------------
/** System Management variable type: binaray data string (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_BINARY						(0x0C)

//---------------------------------------------------------------------
/** System Management variable type: ASCII string (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_ASTRING					(0x0D)

//---------------------------------------------------------------------
/** System Management variable type: unicode string (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_USTRING					(0x0E)

//---------------------------------------------------------------------
/** System Management variable type: internet address (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_INETADDR					(0x0F)

//---------------------------------------------------------------------
/** System Management variable type: MAC address (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_MACADDR					(0x10)

//---------------------------------------------------------------------
// Misc types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type: UTF8 string (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_SMSSUTF8STR				(0x30)

//---------------------------------------------------------------------
/** System Management variable type: UCS2 string (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_SMSSUCS2STR				(0x31)

//---------------------------------------------------------------------
/** System Management variable type: XML string buffer (standard) */
//---------------------------------------------------------------------
#define SM_VTYPE_SMXMLSTRBUF				(0x32)


//---------------------------------------------------------------------
//  Standard code pages for unicode string functions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * Language ID: default string code page, 
 * used when no code page can be specified 
 */
//---------------------------------------------------------------------
#define LID_DEFAULT_CODE_PAGE				(0) // Default to this code page when requested not found

//---------------------------------------------------------------------
/** Language ID: ASCII string code page */
//---------------------------------------------------------------------
#define LID_ASCII_CODE_PAGE					(437) // ASCII 

//---------------------------------------------------------------------
/** Language ID: American English string code page */
//---------------------------------------------------------------------
#define LID_EN_US_CODE_PAGE					(0x409) // US English


//---------------------------------------------------------------------
// Event log definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Event log definition: invalid */
//---------------------------------------------------------------------
#define SMLOG_FACILITY_INVALID				(0)

//---------------------------------------------------------------------
/** Event log definition: Operating System event log */
//---------------------------------------------------------------------
#define SMLOG_FACILITY_OS					(BIT(1))

//---------------------------------------------------------------------
/** Event log definition: Unitext event log */
//---------------------------------------------------------------------
#define SMLOG_FACILITY_UNITXT				(BIT(2))

//---------------------------------------------------------------------
/** Event log definition: XML event log */
//---------------------------------------------------------------------
#define SMLOG_FACILITY_XML					(BIT(3))

//---------------------------------------------------------------------
/** Event log definition: all event log */
//---------------------------------------------------------------------
#define SMLOG_FACILITY_ALL					(SMLOG_FACILITY_OS | SMLOG_FACILITY_UNITXT | SMLOG_FACILITY_XML)

//---------------------------------------------------------------------
/** System Management event logging type: success */
//---------------------------------------------------------------------
#define SMLOG_TYPE_SUCCESS					(0x0000)

//---------------------------------------------------------------------
/** System Management event logging type: error */
//---------------------------------------------------------------------
#define SMLOG_TYPE_ERROR					(0x0001)

//---------------------------------------------------------------------
/** System Management event logging type: warning */
//---------------------------------------------------------------------
#define SMLOG_TYPE_WARNING					(0x0002)

//---------------------------------------------------------------------
/** System Management event logging type: information */
//---------------------------------------------------------------------
#define SMLOG_TYPE_INFORMATION				(0x0004)

//---------------------------------------------------------------------
/** System Management event logging type: auditing success */
//---------------------------------------------------------------------
#define SMLOG_TYPE_AUDIT_SUCCESS			(0x0008)

//---------------------------------------------------------------------
/** System Management event logging type: auditing failure */
//---------------------------------------------------------------------
#define SMLOG_TYPE_AUDIT_FAILURE			(0x0010)

//---------------------------------------------------------------------
/** System Management event logging type: all */
//---------------------------------------------------------------------
#define SMLOG_TYPE_ALL						(0xffff)

//---------------------------------------------------------------------
/** System Management event ID: clear event log */
//---------------------------------------------------------------------
#define SMLOG_EVENTID_LOG_CLEARED			(0x00000000)

//---------------------------------------------------------------------
/** System Management event ID: create backup event log */
//---------------------------------------------------------------------
#define SMLOG_EVENTID_LOG_BACKUP_CREATED	(0x00000001)

//---------------------------------------------------------------------
/** System Management event log minimum size */
//---------------------------------------------------------------------
#define SMLOG_MIN_SIZE						((1024 * 1024) * 1)

//---------------------------------------------------------------------
/** System Management event log maximum size */
//---------------------------------------------------------------------
#define SMLOG_MAX_SIZE						((1024 * 1024) * 4)

//---------------------------------------------------------------------
/** System Management event log minimum size on VMware ESXi */
//---------------------------------------------------------------------
#define SMLOG_MIN_SIZE_ESXI					(1024 * 128)

//---------------------------------------------------------------------
/** System Management event log maximum size on VMware ESXi */
//---------------------------------------------------------------------
#define SMLOG_MAX_SIZE_ESXI					((1024 * 1024) * 4)


//---------------------------------------------------------------------
// Common timeout definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Timeout in milliseconds. 0 means no wait. */
//---------------------------------------------------------------------
#define SM_WAIT_MSECS_NONE					(0)

//---------------------------------------------------------------------
/** Timeout in milliseconds. -1 means wait for infinity. */
//---------------------------------------------------------------------
#define SM_WAIT_MSECS_INFINITE				(-1)


//---------------------------------------------------------------------
// Common time definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Time references
//---------------------------------------------------------------------

/** Time reference: no time reference. */
#define SM_TIME_REFERENCE_NONE				(0)

/** Time reference: invalid time reference. */
#define SM_TIME_REFERENCE_INVALID			(-1)

//---------------------------------------------------------------------
// Time quantities
//---------------------------------------------------------------------

/** Time quantity: system clocks per second */
#ifdef SYSTEM_TYPE_LINUX
// 1 millisecond
#define SM_CLOCK_TICK_PER_SEC				(1000)
#else
#define SM_CLOCK_TICK_PER_SEC				(CLOCKS_PER_SEC)
#endif

/** Time quantity: nanoseconds per microsecond */
#define SM_NSECS_PER_USEC					(1000)

/** Time quantity: microseconds per millisecond */
#define SM_USECS_PER_MSEC					(1000)

/** Time quantity: nanoseconds per millisecond */
#define SM_NSECS_PER_MSEC					(SM_NSECS_PER_USEC * SM_USECS_PER_MSEC)

/** Time quantity: milliseconds per second */
#define SM_MSECS_PER_SEC					(1000)

/** Time quantity: microseconds per second */
#define SM_USECS_PER_SEC					(SM_USECS_PER_MSEC * SM_MSECS_PER_SEC)

/** Time quantity: nanoseconds per second */
#define SM_NSECS_PER_SEC					(SM_NSECS_PER_USEC * SM_USECS_PER_SEC)

/** Time quantity: seconds per minute */
#define SM_SECS_PER_MIN						(60)

/** Time quantity: minutes per hour */
#define SM_MINS_PER_HOUR					(60)

/** Time quantity: seconds per hour */
#define SM_SECS_PER_HOUR					(SM_SECS_PER_MIN * SM_MINS_PER_HOUR)

/** Time quantity: hours per day */
#define SM_HOURS_PER_DAY					(24)

/** Time quantity: seconds per day */
#define SM_SECS_PER_DAY						(SM_SECS_PER_HOUR * SM_HOURS_PER_DAY)

/** Time quantity: days per week */
#define SM_DAYS_PER_WEEK					(7)


//---------------------------------------------------------------------
// INI file value definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** INI file value type: none */
//---------------------------------------------------------------------
#define INI_NONE							(0)

//---------------------------------------------------------------------
/** INI file value type: ASCII string */
//---------------------------------------------------------------------
#define INI_ASTRING							(1)

//---------------------------------------------------------------------
/** INI file value type: unicode string */
//---------------------------------------------------------------------
#define INI_USTRING							(2)

//---------------------------------------------------------------------
/** INI file value type: binary string data */
//---------------------------------------------------------------------
#define INI_BINARY							(3)

//---------------------------------------------------------------------
/** INI file value type: boolean */
//---------------------------------------------------------------------
#define INI_BOOLN							(4)

//---------------------------------------------------------------------
/** INI file value type: unsigned 32-bit integer (little endian) */
//---------------------------------------------------------------------
#define INI_UNSIGNED32_LITTLE_ENDIAN		(5)

//---------------------------------------------------------------------
/** INI file value type: signed 32-bit integer (little endian) */
//---------------------------------------------------------------------
#define INI_SIGNED32_LITTLE_ENDIAN			(6)

//---------------------------------------------------------------------
/** INI file value type: unsigned 64-bit integer (little endian) */
//---------------------------------------------------------------------
#define INI_UNSIGNED64_LITTLE_ENDIAN		(7)

//---------------------------------------------------------------------
/** INI file value type: signed 64-bit integer (little endian) */
//---------------------------------------------------------------------
#define INI_SIGNED64_LITTLE_ENDIAN			(8)

//---------------------------------------------------------------------
/** INI file value type: 64-bit double floating point (little endian) */
//---------------------------------------------------------------------
#define INI_DOUBLE64_LITTLE_ENDIAN			(9)


//---------------------------------------------------------------------
// Type Modifier Definitions
// The type identifier occupies bits [0...7]
// The type modifier occupies bits [8...15]
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management variable type mask */
//---------------------------------------------------------------------
#define SM_VTYPE_MASK						(0x00FF)

//---------------------------------------------------------------------
/** 
 * System Managemnt variable type modifier mask: 
 * The type identifier occupies bits [0...7]
 * The type modifier occupies bits [8...15] 
 */
//---------------------------------------------------------------------
#define SM_VTYPE_MODIFIER_MASK				(0xFF00)


//---------------------------------------------------------------------
// Radix definitions 
// used with integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management radix definition: default */
//---------------------------------------------------------------------
#define SM_RADIX_DEFAULT					(0)

//---------------------------------------------------------------------
/** System Management radix definition: binary */
//---------------------------------------------------------------------
#define SM_RADIX_BINARY						(0x0100)

//---------------------------------------------------------------------
/** System Management radix definition: octal */
//---------------------------------------------------------------------
#define SM_RADIX_OCTAL						(0x0200)

//---------------------------------------------------------------------
/** System Management radix definition: decimal */
//---------------------------------------------------------------------
#define SM_RADIX_DECIMAL					(0x0300)

//---------------------------------------------------------------------
/** System Management radix definition: hexadecimal */
//---------------------------------------------------------------------
#define SM_RADIX_HEXADECIMAL				(0x0400)


//---------------------------------------------------------------------
// Binary String format definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management binary string format definition:
 * default
 */
//---------------------------------------------------------------------
#define BINARY_DEFAULT						SM_RADIX_DEFAULT

//---------------------------------------------------------------------
/** 
 * System Management binary string format definition: 
 * hexadecimal value dump 
 */
//---------------------------------------------------------------------
#define BINARY_HEXDUMP						SM_RADIX_BINARY

//---------------------------------------------------------------------
/** 
 * System Management binary string format definition: 
 * hexadecimal list 
 */
//---------------------------------------------------------------------
#define BINARY_HEXLIST						SM_RADIX_HEXADECIMAL


//---------------------------------------------------------------------
// RFC1321 definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** RFC1321 (MD5) definitions: message digest buffer size */
//---------------------------------------------------------------------
#define RFC1321_DIGEST_SIZE					(16)

//---------------------------------------------------------------------
/** RFC1321 (MD5) definitions: size of temporary buffer for append message */
//---------------------------------------------------------------------
#define RFC1321_MSGBLOCK_SIZE				(64)

//---------------------------------------------------------------------
/** RFC1321 (MD5) definitions: message buffer size */
//---------------------------------------------------------------------
#define RFC1321_MSGLEN_SIZE					(8)


//---------------------------------------------------------------------
// Translation type definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Self-conversion
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING				SM_VTYPE_ASTRING

//---------------------------------------------------------------------
// Defines for converting ASTRING to various VTYPEs
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to boolean 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_BOOLN		SM_VTYPE_BOOLN

//---------------------------------------------------------------------
// Little Endian integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 8-bit integer (8-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_s8			SM_VTYPE_s8

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 16-bit integer (16-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_s16			SM_VTYPE_s16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 32-bit integer (32-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_s32			SM_VTYPE_s32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 64-bit integer (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_s64			SM_VTYPE_s64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 8-bit integer (8-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_u8			SM_VTYPE_u8

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 16-bit integer (16-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_u16			SM_VTYPE_u16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 32-bit integer (32-bit, lilttle endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_u32			SM_VTYPE_u32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 64-bit integer (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_u64			SM_VTYPE_u64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to double floating point (64-bit, little endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_d64			SM_VTYPE_d64

//---------------------------------------------------------------------
// Big Endian integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 16-bit integer (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_S16			SM_VTYPE_S16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 32-bit integer (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_S32			SM_VTYPE_S32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to signed 64-bit integer (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_S64			SM_VTYPE_S64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 16-bit integer (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_U16			SM_VTYPE_U16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 32-bit integer (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_U32			SM_VTYPE_U32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unsigned 64-bit integer (64-bit, big endian)
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_U64			SM_VTYPE_U64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to double floating point (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_D64			SM_VTYPE_D64

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to language ID 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_LANGID		SM_VTYPE_LANGID

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to time stamp 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_TIME_T		SM_VTYPE_TIME_T

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to binary string data 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_BINARY		SM_VTYPE_BINARY

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to unicode string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_USTRING		SM_VTYPE_USTRING

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to internet address 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_INETADDR	SM_VTYPE_INETADDR

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * ASCII to MAC address 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_ASTRING_TO_MACADDR		SM_VTYPE_MACADDR

//---------------------------------------------------------------------
// Defines for converting various VTYPEs to ASTRING
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * boolean to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_BOOLN_TO_ASTRING		SM_VTYPE_BOOLN

//---------------------------------------------------------------------
// Little Endian integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 8-bit integer (8-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_s8_TO_ASTRING			SM_VTYPE_s8

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 16-bit integer (16-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_s16_TO_ASTRING			SM_VTYPE_s16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 32-bit integer (32-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_s32_TO_ASTRING			SM_VTYPE_s32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 64-bit integer (64-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_s64_TO_ASTRING			SM_VTYPE_s64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 8-bit integer (8-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_u8_TO_ASTRING			SM_VTYPE_u8

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 16-bit integer (16-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_u16_TO_ASTRING			SM_VTYPE_u16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 32-bit integer (32-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_u32_TO_ASTRING			SM_VTYPE_u32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 64-bit integer (64-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_u64_TO_ASTRING			SM_VTYPE_u64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * double floating point (64-bit, little endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_d64_TO_ASTRING			SM_VTYPE_d64

//---------------------------------------------------------------------
// Big Endian integer types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 16-bit integer (16-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_S16_TO_ASTRING			SM_VTYPE_S16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 32-bit integer (32-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_S32_TO_ASTRING			SM_VTYPE_S32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * signed 64-bit integer(64-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_S64_TO_ASTRING			SM_VTYPE_S64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 16-bit integer (16-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_U16_TO_ASTRING			SM_VTYPE_U16

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 32-bit integer (32-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_U32_TO_ASTRING			SM_VTYPE_U32

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unsigned 64-bit integer (64-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_U64_TO_ASTRING			SM_VTYPE_U64

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * double floating point (64-bit, big endian) to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_D64_TO_ASTRING			SM_VTYPE_D64

//---------------------------------------------------------------------
// Standard Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * language ID to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_LANGID_TO_ASTRING		SM_VTYPE_LANGID

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * time stamp to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_TIME_T_TO_ASTRING		SM_VTYPE_TIME_T

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * binaray string data to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_BINARY_TO_ASTRING		SM_VTYPE_BINARY

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * unicode string to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_USTRING_TO_ASTRING		SM_VTYPE_USTRING

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * internet address to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_INETADDR_TO_ASTRING	SM_VTYPE_INETADDR

//---------------------------------------------------------------------
/** 
 * System Management XLT string conversion definition: 
 * MAC address string to ASCII string 
 */
//---------------------------------------------------------------------
#define SM_XLT_VTYPE_MACADDR_TO_ASTRING		SM_VTYPE_MACADDR


//---------------------------------------------------------------------
// System Management Thread definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management thread stack size */
//---------------------------------------------------------------------
#define SM_THREAD_STACK_SIZE				(128 * 1024) // size in KB


//---------------------------------------------------------------------
// Maximum (worst case) size of encoding from UCS2 to UTF8
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management UCS2 to UTF8 encoding size 
 * (in worst case, one byte is converted to three UTF8 bytes) 
 */
//---------------------------------------------------------------------
#define MAX_UCS2_TO_UTF8_ENCODE_SIZE		(6)


//---------------------------------------------------------------------
// Hash Routines use the following
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management hash code flag: invalid hash code. */
//---------------------------------------------------------------------
#define SM_INVALID_HASH_ID					(0)

//---------------------------------------------------------------------
/** System Management hash code flag: lowest hash code. */
//---------------------------------------------------------------------
#define SM_HASH_TYPE_LOWEST_AVAILABLE		(0)

//---------------------------------------------------------------------
/** System Management hash code flag: highest hash code. */
//---------------------------------------------------------------------
#define SM_HASH_TYPE_HIGHEST_AVAILABLE		(1)


//---------------------------------------------------------------------
// File, directory and environment string seperator definitions
//---------------------------------------------------------------------

// Linux
#ifdef SYSTEM_TYPE_LINUX

//---------------------------------------------------------------------
/** File path seperator (as character): '/' for Linux, '\' for Windows */
//---------------------------------------------------------------------
#define FILE_PATH_DIR_SEPERATOR				'/'

//---------------------------------------------------------------------
/** File path seperator (as string): "/" for Linux, "\" for Windows */
//---------------------------------------------------------------------
#define FILE_PATH_DIR_SEPERATOR_STR			"/"

//---------------------------------------------------------------------
/** 
 * Environment variable path seperator (as string): 
 * ":" for Linux, ";" for Windows 
 */
//---------------------------------------------------------------------
#define ENV_PATH_SEPARATOR					":"

//---------------------------------------------------------------------
/** 
 * File path symbol for current directory (as string): 
 * "." for both Linux and Windows 
 */
//---------------------------------------------------------------------
#define FILE_PATH_CURRENT_DIR				"."

// Windows
#else

//---------------------------------------------------------------------
/** File path seperator (as character): '/' for Linux, '\' for Windows */
//---------------------------------------------------------------------
#define FILE_PATH_DIR_SEPERATOR				'\\'

//---------------------------------------------------------------------
/** File path seperator (as string): "/" for Linux, "\" for Windows */
//---------------------------------------------------------------------
#define FILE_PATH_DIR_SEPERATOR_STR			"\\"

//---------------------------------------------------------------------
/** 
 * Environment variable path seperator (as string): 
 * ":" for Linux, ";" for Windows 
 */
//---------------------------------------------------------------------
#define ENV_PATH_SEPARATOR					";"

//---------------------------------------------------------------------
/** 
 * File path symbol for current directory (as string): 
 * "." for both Linux and Windows 
 */
//---------------------------------------------------------------------
#define FILE_PATH_CURRENT_DIR				"."

#endif


//---------------------------------------------------------------------
// Valid characters for value separator
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// NOTE: '\\0' is used since doxygen mistreats it to be a special 
// command. It actually means '\0' as used in the C language.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** System Management valid value seperator: null ('\\0') */
//---------------------------------------------------------------------
#define SM_VALSEP_NULL						'\0'

//---------------------------------------------------------------------
/** System Management valid value seperator: comma (',') */
//---------------------------------------------------------------------
#define SM_VALSEP_COMMA						','

//---------------------------------------------------------------------
/** System Management valid value seperator: period ('.') */
//---------------------------------------------------------------------
#define SM_VALSEP_PERIOD					'.'

//---------------------------------------------------------------------
/** System Management valid value seperator: forward slash ('/') */
//---------------------------------------------------------------------
#define SM_VALSEP_SLASH						'/'

//---------------------------------------------------------------------
/** System Management valid value seperator: colon (':') */
//---------------------------------------------------------------------
#define SM_VALSEP_COLON						':'

//---------------------------------------------------------------------
/** System Management valid value seperator: semicolon (';') */
//---------------------------------------------------------------------
#define SM_VALSEP_SEMICOLON					';'

//---------------------------------------------------------------------
/** System Management valid value seperator: back slash ('\') */
//---------------------------------------------------------------------
#define SM_VALSEP_BACKSLASH					'\\'

//---------------------------------------------------------------------
/** System Management valid value seperator: pipe ('|') */
//---------------------------------------------------------------------
#define SM_VALSEP_PIPE						'|'	

//---------------------------------------------------------------------
/** 
 * System Management valid value seperator: bit or operator ('|')
 * (same as pipe) 
 */
//---------------------------------------------------------------------
#define SM_VALSEP_BIT_OR					SM_VALSEP_PIPE


//---------------------------------------------------------------------
// OS Abstraction file name templates
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management plugin module file name template: \n
 * Windows: name.dll; \n
 * NetWare: name.nlm; \n
 * Linux: libname.so; \n
 * other unsupported operation system: name.
 */
//---------------------------------------------------------------------
#ifdef SYSTEM_TYPE_WINNT
#define SM_PLUGIN_FILENAME_TEMPLATE			"%s.dll"
#elif defined(SYSTEM_TYPE_NETWARE)
#define SM_PLUGIN_FILENAME_TEMPLATE			"%s.nlm"
#elif defined(SYSTEM_TYPE_LINUX)
#define SM_PLUGIN_FILENAME_TEMPLATE			"lib%s.so"
#else // unsupported system type
#define SM_PLUGIN_FILENAME_TEMPLATE			"%s"
#endif


//---------------------------------------------------------------------
// omreg config file
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management OpenManage regular configuration file name: 
 * omreg.cfg 
 */
//---------------------------------------------------------------------
#define SM_OMREG_FILENAME					"omreg.cfg"

//---------------------------------------------------------------------
/** 
 * System Management OpenManage regular configuration file path: \n
 * Windows: "<none>" \n
 * NetWare: "sys:\etc\omreg.cfg" \n
 * Linux: "/etc/omreg.cfg" \n
 * Other: Unsupported operating system: "<none>"
 */
//---------------------------------------------------------------------
#ifdef SYSTEM_TYPE_WINNT
#define SM_OMREG_PATHFILENAME				""
#elif defined(SYSTEM_TYPE_NETWARE)
#define SM_OMREG_PATHFILENAME				"sys:\\etc\\" SM_OMREG_FILENAME

#elif defined(SYSTEM_TYPE_LINUX)
#ifndef SYSCONFDIR
#define SYSCONFDIR "/etc/"
#endif
#define SM_OMREG_PATHFILENAME				SYSCONFDIR SM_OMREG_FILENAME
#else

#define SM_OMREG_PATHFILENAME				""
#endif


//---------------------------------------------------------------------
// Subdirectory and extension string constants
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management subdirectory definition: 
 * bin for binary files 
 */
//---------------------------------------------------------------------
#define SM_PSUBPATH_BIN						"bin"

//---------------------------------------------------------------------
/** 
 * System Management subdirectory definition: 
 * ini for initialization files 
 */
//---------------------------------------------------------------------
#define SM_PSUBPATH_INI						"ini"

//---------------------------------------------------------------------
/** 
 * System Management subdirectory definition: 
 * log for log files 
 */
//---------------------------------------------------------------------
#define SM_PSUBPATH_LOG						"log"

//---------------------------------------------------------------------
/** System Management INI file extension: .ini */
//---------------------------------------------------------------------
#define SM_EXT_INI							"." SM_PSUBPATH_INI


//---------------------------------------------------------------------
// Custom Install path type string constants
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 1: installation path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_INST					(BIT(1))

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 2: variable data path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_VDAT					(BIT(2))

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 3: product log path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_PLOG					(BIT(3))

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 4: configuration path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_CFGT					(BIT(4))

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 5: log file path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_LOG					(BIT(5))

//---------------------------------------------------------------------
/** 
 * System Management bit mask for path type:
 * Bit 6: ini file path. 
 */
//---------------------------------------------------------------------
#define SM_PATH_TYPE_INI					(BIT(6))


//---------------------------------------------------------------------
// Walk types supported by SMRedBlackTreeDataWalk
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management supported tree parsing method: 
 * walk node only 
 */
//---------------------------------------------------------------------
#define SM_RBT_WALKTYPE_NODE_ONLY			(0)

//---------------------------------------------------------------------
/** 
 * System Management supported tree parsing method: 
 * walk node and subtree in ascending order 
 */
//---------------------------------------------------------------------
#define SM_RBT_WALKTYPE_ASCENDING_ORDER		(1)

//---------------------------------------------------------------------
/** 
 * System Management supported tree parsing method: 
 * walk node and subtree in descending order 
 */
//---------------------------------------------------------------------
#define SM_RBT_WALKTYPE_DESCENDING_ORDER	(2)

//---------------------------------------------------------------------
/** 
 * System Management supported tree parsing method:
 * walk node and subtree until match found to perform the quickest search 
 */
//---------------------------------------------------------------------
#define SM_RBT_WALKTYPE_FIND_DATA			(3)

//---------------------------------------------------------------------
/** 
 * System Management supported tree parsing method:
 * walk node and subtree in the order specified by walk function 
 */
//---------------------------------------------------------------------
#define SM_RBT_WALKTYPE_WALK_ORDER			(4)


//--------------------------------------------------------------------
// Binary Self-Describing Object (SDO) Basic Type Definitions
//--------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management binary object basic type definition:
 * undefined 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_UNDEFINED				(0x0)

//---------------------------------------------------------------------
// Valid Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management binary Self-Describing Object (SDO) basic type definition:
 * boolean 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_BOOLN					(0x1)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * signed 8-bit integer (8-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_S8						(0x2)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * signed 16-bit integer (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_S16						(0x3)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * signed 32-bit integer (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_S32						(0x4)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * signed 64-bit integer (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_S64						(0x5)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * unsigned 8-bit integer (8-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_U8						(0x6)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * unsigned 16-bit integer (16-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_U16						(0x7)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * unsigned 32-bit integer (32-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_U32						(0x8)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * unsigned 64-bit integer (64-bit, big endian) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_U64						(0x9)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * ASCII string 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_ASTRING					(0xa)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * unicode string 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_USTRING					(0xb)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * SDO binary data 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_SDOBINARY				(0xc)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * SDO configuration data 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_SDOCONFIG				(0xd)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * binary data string 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPE_BINARY					(0xe)

//---------------------------------------------------------------------
// Reserved Types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * reserved type 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPELO						(0x1)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type definition:
 * reserved type 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPEHI						(0xe)

//---------------------------------------------------------------------
/** 
 * System Management binary SDO basic type mask: 0xf. 
 * SDO Data Type is stored in 4 bits, thus type limit is (0xf) 
 */
//---------------------------------------------------------------------
#define SM_SDO_TYPEMASK						(0xf)


//---------------------------------------------------------------------
// Binary Self-Describing Object (SDO) Number Format Definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * System Management binary Self-Describing Object (SDO) number format definition: 
 * decimal 
 */
//---------------------------------------------------------------------
#define SM_SDO_NUMBERFORMAT_DECIMAL			(0x0)

//---------------------------------------------------------------------
/** 
 * System Management binary (SDO) number format definition: 
 * hexadecimal 
 */
//---------------------------------------------------------------------
#define SM_SDO_NUMBERFORMAT_HEXADECIMAL		(0x1)

//---------------------------------------------------------------------
/** 
 * System Management binary (SDO) number format definition: 
 * binary 
 */
//---------------------------------------------------------------------
#define SM_SDO_NUMBERFORMAT_BINARY			(0x2)

//---------------------------------------------------------------------
/** 
 * System Management binary (SDO) number format definition: 
 * reserved number format 
 */
//---------------------------------------------------------------------
#define SM_SDO_NUMBERFORMAT_RESERVED		(0x3)
// Maximum is 0x3


//--------------------------------------------------------------------
// Binary Self-Describing Object (SDO) Field ID Definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) Field ID Definitions: Invalid */
//--------------------------------------------------------------------
#define SM_SDO_FIELD_ID_INVALID				(0)

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) Field ID Definitions: reserved 1 */
//--------------------------------------------------------------------
#define SM_SDO_FIELD_ID_RESERVED1			(0xfffe)

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) Field ID Definitions: reserved 2 */
//--------------------------------------------------------------------
#define SM_SDO_FIELD_ID_RESERVED2			(0xffff)


//--------------------------------------------------------------------
// Binary Self-Describing Object (SDO) Limits Definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** 
 * Binary Self-Describing Object (SDO) Limits Definitions:
 * maximum field ID is 0xffff 
 */
//--------------------------------------------------------------------
#define SM_SDO_MAX_FIELD_ID					(0xffff)

//--------------------------------------------------------------------
/** 
 * Binary Self-Describing Object (SDO) Limits Definitions:
 * maximum data count is 0xffff 
 */
//--------------------------------------------------------------------
#define SM_SDO_MAX_DATA_COUNT				(0xffff)

//--------------------------------------------------------------------
/** 
 * Binary Self-Describing Object (SDO) Limits Definitions:
 * minimum binary size is 0x8 
 */
//--------------------------------------------------------------------
#define SM_SDO_MIN_BINARY_SIZE				(0x8)

//--------------------------------------------------------------------
/** 
 * Binary Self-Describing Object (SDO) Limits Definitions:
 * maximum binary size is 0xffff 
 */
//--------------------------------------------------------------------
#define SM_SDO_MAX_BINARY_SIZE				(0xffff)


//--------------------------------------------------------------------
// SDO Field ID Allocation
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** SDO Field ID (FID): Invalid FID */
//--------------------------------------------------------------------
#define SM_SDOFID_INVALID					(0x0000)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): private FID low range.
 * Private FID range typically used for stand-alone or test modules,
 * or for temporary SDO instance. 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_PRIVATE_LO1				(0x0001)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): private FID high range.
 * Private FID range typically used for stand-alone or test modules,
 * or for temporary SDO instance. 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_PRIVATE_HI1				(0x0fff)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): public FID low range.
 * global to all (assigned by DataEngine). 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_PUBLIC_LO1				(0x1000)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): public FID high range.
 * global to all (assigned by DataEngine). 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_PUBLIC_HI1				(0x1fff)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): Unallocated FID low range I
 * (to be assigned by DataEngine) 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_UNALLOCATED_LO1			(0x2000)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): Unallocated FID high range I
 * (to be assigned by DataEngine) 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_UNALLOCATED_HI1			(0x2fff)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID low range for DataEngine */
//--------------------------------------------------------------------
#define SM_SDOFID_DATAENGINE_LO1			(0x3000)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID high range for DataEngine */
//--------------------------------------------------------------------
#define SM_SDOFID_DATAENGINE_HI1			(0x3fff)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID low range for Instrumentation Service (HIP) */
//--------------------------------------------------------------------
#define SM_SDOFID_INST_SVC_LO1				(0x4000)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID high range for Instrumentation Service (HIP) */
//--------------------------------------------------------------------
#define SM_SDOFID_INST_SVC_HI1				(0x4fff)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID low range for Drac */
//--------------------------------------------------------------------
#define SM_SDOFID_DRSC_LO1					(0x5000)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID high range for Drac */
//--------------------------------------------------------------------
#define SM_SDOFID_DRSC_HI1					(0x5fff)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID low range for Storage Services */
//--------------------------------------------------------------------
#define SM_SDOFID_STORAGE_SVC_LO1			(0x6000)

//--------------------------------------------------------------------
/** SDO Field ID (FID): FID high range for Storage Services */
//--------------------------------------------------------------------
#define SM_SDOFID_STORAGE_SVC_HI1			(0x6fff)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): Unallocated FID low range II
 * (to be assigned by DataEngine) 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_UNALLOCATED_LO2			(0x7000)

//--------------------------------------------------------------------
/** 
 * SDO Field ID (FID): Unallocated FID high range II
 * (to be assigned by DataEngine) 
 */
//--------------------------------------------------------------------
#define SM_SDOFID_UNALLOCATED_HI2			(0xefff)

//--------------------------------------------------------------------
// Reserved FID range
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** SDO Field ID (FID): reserved FID low range II */
//--------------------------------------------------------------------
#define SM_SDOFID_RESERVED_LO2				(0xf000)

//--------------------------------------------------------------------
/** SDO Field ID (FID): reserved FID high range II */
//--------------------------------------------------------------------
#define SM_SDOFID_RESERVED_HI2				(0xffff)


//--------------------------------------------------------------------
// Binary Self-Describing Object (SDO) 
// Data Type Generator/Accessor Macro
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** 
 * Binary Self-Describing Object (SDO) Data Type Generator.
 * nf = SDO Number Format (see above for definitions) - 2 bits \n
 * hf = Private/Hidden Flag (0=false, 1=true) - 1 bit \n
 * af = Array Flag (0=false, 1=true) - 1 bit \n
 * bt = Basic Type (see above for definitions) - 4 bits 
 */
//--------------------------------------------------------------------
#ifndef SM_SDO_MAKE_DATATYPE
#define SM_SDO_MAKE_DATATYPE(nf, hf, af, bt) \
	((nf>=0 && nf<=3) && (hf==0 || hf==1) && (af==0 || af==1) && (bt>=SM_SDO_TYPELO && bt<=SM_SDO_TYPEHI)) ? \
	(((nf & 0x3) << 6) | ((hf & 0x1) << 5) | ((af & 0x1) << 4) | (bt & 0xf)) : \
	SM_SDO_TYPE_UNDEFINED
#endif

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) number format accessor */
//--------------------------------------------------------------------
#ifndef SM_SDO_GET_NUMFORMAT
#define SM_SDO_GET_NUMFORMAT(dataType)		((dataType >> 6) & 0x3)
#endif

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) private flag accessor */
//--------------------------------------------------------------------
#ifndef SM_SDO_GET_PRIVFLAG
#define SM_SDO_GET_PRIVFLAG(dataType)		((dataType >> 5) & 0x1)
#endif

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) array flag accessor */
//--------------------------------------------------------------------
#ifndef SM_SDO_GET_ARRAYFLAG
#define SM_SDO_GET_ARRAYFLAG(dataType)		((dataType >> 4) & 0x1)
#endif

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) basic type accessor */
//--------------------------------------------------------------------
#ifndef SM_SDO_GET_BASICTYPE
#define SM_SDO_GET_BASICTYPE(dataType)		(dataType & 0xf)
#endif


//--------------------------------------------------------------------
// Product ID definitions
// These definitions are derived from omdbintf.h maintained by Autry.
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** System Management Product ID: invalid */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_INVALID				(0)

//--------------------------------------------------------------------
/** System Management Product ID: OpenManage root */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_OMROOT				(-1)

//--------------------------------------------------------------------
/** System Management Product ID: Server Administrator */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_OM_SA					(15)

//--------------------------------------------------------------------
/** System Management Product ID: Storage Services */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_STORAGE_SVC			(33)

//--------------------------------------------------------------------
/** System Management Product ID: Data Engine */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_DATA_ENGINE			(34)

//--------------------------------------------------------------------
/** System Management Product ID: Instrumentation Service */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_INST_SVC				(35)

//--------------------------------------------------------------------
/** System Management Product ID: HAPI */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_HAPI					(44)

//--------------------------------------------------------------------
/** System Management Product ID: RAC5 */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_RAC5					(88)

//--------------------------------------------------------------------
/** System Management Product ID: support library */
//--------------------------------------------------------------------
#define SM_PRODUCT_ID_SUPTLIB				(45)


//--------------------------------------------------------------------
// Predefined product install path
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * OpenManage root 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_OMROOT				"osia"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * Instrumentation Service component 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_INST_SVC				"Instrumentation"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * Storage Services component 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_STORAGE_SVC			"omss"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * Data Engine component 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_DATA_ENGINE			"OMDataEngine"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * HAPI component 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_HAPI					"hapi"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path: 
 * RAC5 component 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_RAC5					"rac5"

//--------------------------------------------------------------------
/** 
 * System Management Product Install Path:
 * Support Library 
 */
//--------------------------------------------------------------------
#define SM_PRODUCT_SID_SUPTLIB				"suptlib"


//--------------------------------------------------------------------
/** Enumeration value to UTF8 string mapping */
//--------------------------------------------------------------------
#define SM_ENUMs32_ERROR_VAL				(0x80000000)


//--------------------------------------------------------------------
// XML Generation support definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** 
 * System Management XML template: standard header
 * All System Management data in XML string format will use this standard header.
 */
//--------------------------------------------------------------------
#define SM_XML_STD_HEADER	"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"


//--------------------------------------------------------------------
// Definitions of Field Identifiers (FID) for public use.
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_reserved					(0x1000)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_Reserved					(0x1001)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_reservedAlign				(0x1002)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_reserveAlign				(0x1003)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_reservedAlign1				(0x1004)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_reservedAlign2				(0x1005)

//--------------------------------------------------------------------
/** Definitions of Field Identifiers (FID) for public use. */
//--------------------------------------------------------------------
#define FID_PUB_numUsers					(0x1006)

//--------------------------------------------------------------------
// Security Definitions
//--------------------------------------------------------------------

//-------------------------------------------------------------------------
//  Value format definition for ACL attribute "aclAttribute"
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1      
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 
//  +---------------+---------------+---------------+---------------+
//  |~ ~ ~ ~ X W R L|~ ~ ~ ~ X W R L|~ ~ ~ ~ X W R L|    OPTIONS    |
//  +---------------+---------------+---------------+---------------+
//      access_for_A    access_for_P    access_for_U
//
//  Where 
//    ~ = reserved bit
//    L = set ALL access
//    R = read access
//    W = write access
//    X = execute access
//    A = Administrator: refers to Windows Administrators privilege level 
//        In Linux, refers to "user" access permission
//        Abbreviated as ADMINUSR
//    P = Power User: refers to Windows Power Users privilege level 
//        In Linux, refers to "group" access permission
//        Abbreviated as POWERGRP
//    U = User: refers to Windows Users privilege level
//        In Linux, refers to "others" access permission
//        Abbreviated as USEROTH
//-------------------------------------------------------------------------

// ACL value for least secured setting
// Typical use of this value is for backwards compatibility
// with previous releases which have a least security setting

#define SMACL_VALUE_LEAST_SECURE			(-1)

// ACL value for most secured setting

#define SMACL_VALUE_MOST_SECURE				(0)

// ACL options definition
#define SMACL_OPTION_ADDUSERSID				BIT(1)	// Windows only

// ACL access type definitions

#define SMACL_ACCESS_ALL					BIT(0)
#define SMACL_ACCESS_READ					BIT(1)
#define SMACL_ACCESS_WRITE					BIT(2)
#define SMACL_ACCESS_EXECUTE				BIT(3)
#define SMACL_ACCESS_RESERVED1				BIT(4)
#define SMACL_ACCESS_RESERVED2				BIT(5)
#define SMACL_ACCESS_RESERVED3				BIT(6)

// ACL get macro for specific privilege level

#define SMACL_GET_ADMINUSR(aclValue)		(u8)(aclValue >> 24)
#define SMACL_GET_POWERGRP(aclValue)		(u8)(aclValue >> 16)
#define SMACL_GET_USEROTH(aclValue)			(u8)(aclValue >> 8)

// DO NOT USE: used locally below for other macros
// ACL set macro for specific privilege level

#define SMACL_SET_ADMINUSR(accessValue)		(accessValue << 24)
#define SMACL_SET_POWERGRP(accessValue)		(accessValue << 16)
#define SMACL_SET_USEROTH(accessValue)		(accessValue << 8)

// ACL set access rights for Administrator (root)

#define SMACL_ALIAS_ADMINUSR_ALLACCESS(X) \
	(X | SMACL_SET_ADMINUSR(SMACL_ACCESS_ALL))

#define SMACL_ALIAS_ADMINUSR_READONLY(X) \
	(X | SMACL_SET_ADMINUSR(SMACL_ACCESS_READ))

#define SMACL_ALIAS_ADMINUSR_READWRITE(X) \
	(X | SMACL_SET_ADMINUSR(SMACL_ACCESS_READ) \
	   | SMACL_SET_ADMINUSR(SMACL_ACCESS_WRITE))

#define SMACL_ALIAS_ADMINUSR_READEXECUTE(X) \
	(X | SMACL_SET_ADMINUSR(SMACL_ACCESS_READ) \
	   | SMACL_SET_ADMINUSR(SMACL_ACCESS_EXECUTE))

// ACL set access rights for Power User (user in root group)

#define SMACL_ALIAS_POWERGRP_ALLACCESS(X) \
	(X | SMACL_SET_POWERGRP(SMACL_ACCESS_ALL))

#define SMACL_ALIAS_POWERGRP_READONLY(X) \
	(X | SMACL_SET_POWERGRP(SMACL_ACCESS_READ))

#define SMACL_ALIAS_POWERGRP_READWRITE(X) \
	(X | SMACL_SET_POWERGRP(SMACL_ACCESS_READ) \
	   | SMACL_SET_POWERGRP(SMACL_ACCESS_WRITE))

#define SMACL_ALIAS_POWERGRP_READEXECUTE(X) \
	(X | SMACL_SET_POWERGRP(SMACL_ACCESS_READ) \
	   | SMACL_SET_POWERGRP(SMACL_ACCESS_EXECUTE))

// ACL set access rights for User

#define SMACL_ALIAS_USEROTH_ALLACCESS(X) \
	(X | SMACL_SET_USEROTH(SMACL_ACCESS_ALL))

#define SMACL_ALIAS_USEROTH_READONLY(X) \
	(X | SMACL_SET_USEROTH(SMACL_ACCESS_READ))

#define SMACL_ALIAS_USEROTH_READWRITE(X) \
	(X | SMACL_SET_USEROTH(SMACL_ACCESS_READ) \
	   | SMACL_SET_USEROTH(SMACL_ACCESS_WRITE))

#define SMACL_ALIAS_USEROTH_READEXECUTE(X) \
	(X | SMACL_SET_USEROTH(SMACL_ACCESS_READ) \
	   | SMACL_SET_USEROTH(SMACL_ACCESS_EXECUTE))

// ACL set all access to all privilege level

#define SMACL_ALIAS_ALLLEVEL_ALLACCESS(X) \
	(X | SMACL_ALIAS_ADMINUSR_ALLACCESS(0) \
	   | SMACL_ALIAS_POWERGRP_ALLACCESS(0) \
	   | SMACL_ALIAS_USEROTH_ALLACCESS(0))


//---------------------------------------------------------------------
// Operating System Type
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** Operating System Type: Windows */
//---------------------------------------------------------------------
#define SM_OS_TYPE_WINDOWS					(1)

//---------------------------------------------------------------------
/** Operating System Type: NetWare */
//---------------------------------------------------------------------
#define SM_OS_TYPE_NETWARE					(2)

//---------------------------------------------------------------------
/** Operating System Type: Linux */
//---------------------------------------------------------------------
#define SM_OS_TYPE_LINUX					(3)

//---------------------------------------------------------------------
/** Operating System Type: VMware ESXi */
//---------------------------------------------------------------------
#define SM_OS_TYPE_VMWAREESXI				(4)


//---------------------------------------------------------------------
/** Operating System Type: Citrix XEN */
//---------------------------------------------------------------------
#define SM_OS_TYPE_CITRIXXEN				(5)


//--------------------------------------------------------------------
// Structure definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** Structure for RFC1321 (MD5) context data. Used to perform MD5 Message-Digest algorithm. */
//--------------------------------------------------------------------
typedef struct _RFC1321Data
{
	/** Length of message before padding bits were added. can be 0 */
	_PRIV u64 b;
	
	/** Word A. Word is a 32-bit quantity. */
	_PRIV u32 A;
	
	/** Word B. Word is a 32-bit quantity. */
	_PRIV u32 B;
	
	/** Word C. Word is a 32-bit quantity. */
	_PRIV u32 C;
	
	/** Word D. Word is a 32-bit quantity. */
	_PRIV u32 D;
	
	/** Number of block buffer bytes currently in use. */
	_PRIV u32 bBufInUse;
	
	/** Message digest buffer. RFC1321_DIGEST_SIZE defined as 16.*/
	_PRIV u8 msgDigest[RFC1321_DIGEST_SIZE];
	
	/** Temporary buffer used to append messages. RFC1321_MSGBLOCK_SIZE defined as 64 */
	_PRIV u8 bBuf[RFC1321_MSGBLOCK_SIZE];
} RFC1321Data;


//--------------------------------------------------------------------
/** Thread group switch context data */
//--------------------------------------------------------------------
typedef struct _SMECInfo
{
	/** Used in netware to store and restore the old thread id */
	_PRIV int threadGroupID;
} SMECInfo;


//--------------------------------------------------------------------
/** Fast strcpy structure for astring type */
//--------------------------------------------------------------------
typedef struct _SMSSUTF8Str
{
	/** Points to contents of the UTF-8 string */
	_PRIV astring *pStr;

	/** Size in bytes of buffer pointed to by pStr */
	_PRIV u32 strBufSize;

	/** Size in bytes of buffer occupied by string contents */
	_PRIV u32 strCurLen;
} SMSSUTF8Str;


//--------------------------------------------------------------------
/** Fast strcpy structure for ustring type */
//--------------------------------------------------------------------
typedef struct _SMSSUCS2Str
{
	/** Points to contents of the UCS-2 string */
	_PRIV ustring *pStr;

	/** Size in bytes of buffer pointed to by pStr */
	_PRIV u32 strBufSize;

	/** Size in bytes of buffer occupied by string contents */
	_PRIV u32 strCurLen;
} SMSSUCS2Str;


//--------------------------------------------------------------------
/** 
 * Linked list entry in a singly linked list.
 * Can be used as either a list head, or as link node 
 */
//--------------------------------------------------------------------
typedef struct _SMSLListEntry
{
	/** Pointer to next node in chain */
    _PRIV struct _SMSLListEntry * volatile pNext;

	/** Pointer to node data */
	_PRIV void * volatile pData;
} SMSLListEntry;


//--------------------------------------------------------------------
/** Self contained list container structure for a singly linked list. */
//--------------------------------------------------------------------
typedef struct _SMSLList
{
	/** Node at the head of the list */
    _PRIV SMSLListEntry * volatile pHead;

	/** Node at the tail of the list */
    _PRIV SMSLListEntry * volatile pTail;
} SMSLList;


//--------------------------------------------------------------------
/** 
 * Singly link list walk function.
 * Used to insert node to link list.
 * Uses pWalkData passed in to determine the node 
 * with matching data for insert position in the singly linked list.

 * Returns: \n
 * 0 if pWalkData contents == pSLECur contents (matching node found). \n
 * < 0 if pWalkData contents < pSLECur contents. \n
 * > 0 if pWalkData contents > pSLECur contents. \n
 * pWalkData can be used to return data back. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSLLISTWALK)(_INOUT void *pWalkData, 
									 _IN const SMSLListEntry *pSLECur);


//--------------------------------------------------------------------
/** 
 * Doubly linked list structure. 
 * Can be used as either a list head, or as link words. 
 */
//--------------------------------------------------------------------
typedef struct _SMDLListEntry
{
	/** Pointer to next node in chain */
    _PRIV struct _SMDLListEntry * volatile pNext;

	/** Pointer to previous node in chain */
    _PRIV struct _SMDLListEntry * volatile pPrev;
	
	/** Pointer to node data */
	_PRIV void * volatile pData;
} SMDLListEntry;


//--------------------------------------------------------------------
/** Self contained list container structure for a doubly linked list. */
//--------------------------------------------------------------------
typedef struct _SMDLList
{
	/** node at the head of the list */
    _PRIV SMDLListEntry  * volatile pHead;
	
	/** node at the tail of the list */
    _PRIV SMDLListEntry  * volatile pTail;
} SMDLList;


//--------------------------------------------------------------------
/** 
 * Doubly link list walk function.
 * Used to insert node to link list.
 * Uses pWalkData passed in to determine the node 
 * with matching data for insert position in the singly linked list.
	
 * Returns: \n
 * 0 if pWalkData contents == pDLECur contents (matching node found).\n
 * < 0 if pWalkData contents < pDLECur contents.\n
 * > 0 if pWalkData contents > pDLECur contents.\n
 * pWalkData can be used to return data back. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNDLLISTWALK)(_INOUT void *pWalkData, 
									 _IN const SMDLListEntry *pDLECur);


//--------------------------------------------------------------------
/** Thread callback routine. */
//--------------------------------------------------------------------
typedef void (_cdecl *FPROCSMTHREADROUTINE)(_IN void *pSMThreadData);


//--------------------------------------------------------------------
/** Critical Sections callback routine. */
//--------------------------------------------------------------------
typedef void (_cdecl *FPROCSMCRITSECROUTINE)(_IN void *pSMCritSecData);


//--------------------------------------------------------------------
/** External representation of red black tree node. */
//--------------------------------------------------------------------
typedef struct _SMRedBlackTreeNode SMRedBlackTreeNode;


//--------------------------------------------------------------------
/** External definition of red black tree node data */
//--------------------------------------------------------------------
typedef void SMRedBlackTreeNodeData;


//--------------------------------------------------------------------
/** 
 * Walk function called by tree routines.
 * The pWalkData is passed to the function in determining 
 * the node with matching data to stop walk and return data.
		  
 * Returns: \n
 * 0 if pWalkData contents == pNodeData contents (matching node found). \n
 * < 0 if pWalkData contents < pNodeData contents. \n
 * > 0 if pWalkData contents > pNodeData contents. \n
 * pWalkData can be used to return data back. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSMRBTWALK)(_INOUT void *pWalkData, 
									_IN const SMRedBlackTreeNodeData *pNodeData);


//--------------------------------------------------------------------
/** 
 * Notify the caller routine for delete notification.
 * pNotifyData can be used to return data back. 
 */
//--------------------------------------------------------------------
typedef void (__cdecl *PFNSMRBTNOTIFYDEL)(_INOUT void *pNotifyDelData, 
										  _INOUT SMRedBlackTreeNodeData *pNodeData);


//--------------------------------------------------------------------
// Binary Self-Describing Object (SDO) Definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) Support: Configuration SDO Type */
//--------------------------------------------------------------------
typedef void SDOConfig;

//--------------------------------------------------------------------
/** Binary Self-Describing Object (SDO) Support: Binary SDO Type */
//--------------------------------------------------------------------
typedef void SDOBinary;


//--------------------------------------------------------------------
/** 
 * Callback function prototype used by the SDOBinaryWalk function.
 * This function is executed when the beginning of SDO object is 
 * encountered during the walk.\n

 * The pWalkData is passed to the function in determining 
 * the node with matching data to stop walk and return data. \n
 * pWalkData can be used to return data back. \n
 * This function will return 0 to continue walking. \n
 * Any other return value will cause stop walking. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSDOBINARYWALKBINARYBEGIN)(
						_INOUT void *pWalkData,
						_IN const astring *pBinaryName,
						_IN booln bIsBinaryArray,
						_IN s32 count,
						_IN s32 index,
						_IN const void *pData
						);

//--------------------------------------------------------------------
/** 
 * Callback function prototype used by the SDOBinaryWalk function.
 * This function is executed when the end of the SDO object is
 * encountered during the walk.

 * The pWalkData is passed to the function in determining 
 * the node with matching data to stop walk and return data. \n
 * pWalkData can be used to return data back. \n
 * This function will return 0 to continue walking. \n
 * Any other return value will cause stop walking. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSDOBINARYWALKBINARYEND)(
						_INOUT void *pWalkData,
						_IN const astring *pBinaryName,
						_IN booln bIsBinaryArray
						);

//--------------------------------------------------------------------
/** 
 * Callback function prototype used by the SDOBinaryWalk function.
 * This function is executed when the beginning of the SDO object
 * field is encountered during the walk.

 * The pWalkData is passed to the function in determining 
 * the node with matching data to stop walk and return data. \n
 * pWalkData can be used to return data back. \n
 * This function will return 0 to continue walking. \n
 * Any other return value will cause stop walking. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSDOBINARYWALKFIELDBEGIN)(
						_INOUT void *pWalkData,
						_OUT astring **ppFieldName,
						_IN u16 fieldID,
						_IN u8 numberFormat,
						_IN u8 privateFlag,
						_IN u8 arrayFlag,
						_IN u8 dataType,
						_IN u32 dataSize,
						_IN const void *pData
						);

//--------------------------------------------------------------------
/** 
 * Callback function prototype used by the SDOBinaryWalk function.
 * This function is executed when the beginning of the SDO object
 * field is encountered during the walk.

 * The pWalkData is passed to the function in determining 
 * the node with matching data to stop walk and return data. \n
 * pWalkData can be used to return data back. \n
 * This function will return 0 to continue walking. \n
 * Any other return value will cause stop walking. 
 */
//--------------------------------------------------------------------
typedef s32 (__cdecl *PFNSDOBINARYWALKFIELDEND)(
						_INOUT void *pWalkData,
						_IN astring *pFieldNameToFree
						);


//--------------------------------------------------------------------
// XML Generation support definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
/** XML generation string container/buffer. */
//--------------------------------------------------------------------
typedef SMSSUTF8Str SMXMLStrBuf;


//--------------------------------------------------------------------
/** Object container for value parameter passed to the XML Gen API. */
//--------------------------------------------------------------------
typedef struct _SMXGValOpts
{
	/** 
	 * sm vtype modifier, see SM_RADIX_xxx and BINARY_xxx definitions.
	 * Used to to ensure value safety. May be masked with SM_VTYPE_MODIFIER_MASK. 
	 */
	_IN u16 typeModifier;

	/** Number of XML escape transformation, 0 for no transformation. */
	_IN u8 xmlEscapeCount;

	/** Alignment for next data segment */
	_PRIV u8 reservedAlign[1];
} SMXGValOpts;


//--------------------------------------------------------------------
/** Enumeration value to UTF8 string mapping. */
//--------------------------------------------------------------------
typedef struct _SMEnums32UTF8Map
{
	/** UTF8 string used for mapping */
	_IN const astring *pAsUTF8;
	
	/** Emuveration 32-bit value */
	_IN s32 ass32;

	/** Alignment for next data segment */
	_PRIV u8 reservedAlign[4];
} SMEnums32UTF8Map;


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------

#pragma pack()


#ifdef __cplusplus
}
#endif


#endif //  __DCSUPDEF_H_INCLUDED__ 

