
//-------------------------------------------------------------------------
//
//          Dell Incorporated PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Incorporated and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Incorporated. All Rights Reserved.
//
//  Last Modified By / On:
//    Author: Ryan Brown / Date: 2012 Jul 25
//
//-------------------------------------------------------------------------

#ifndef __MRCACHECOMMON_H_INCLUDED__
#define __MRCACHECOMMON_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------

// local headers
#include "msgregcommon.h"

//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Message Registry
//---------------------------------------------------------------------

// macro definitions

// the number of non-English languages
// NOTE: always sync this number when MSGREG_LOCALE_ changes!
#define MRC_NONENGLANG_COUNT           (5)

// the number of index groups = non-English languages times db type
// NOTE: always sync this number when MRDBTYPE_ changes!
#define MRC_LANG_DBTYPE_COUNT          (3 * MRC_NONENGLANG_COUNT)

// the number of index groups = English language
// NOTE: always sync this number when MRDBTYPE_ changes!
#define MRC_EN_DBTYPE_COUNT            (3)

// this is the count threshold in which the cache manager will
// switch from index-based cache to full-based cache
#define MRC_LANGACCESS_EXPIRECOUNT     (4)

// string definitions

#define MRC_EVENT_SEMAPHORE            "1MRCEventSem"
#define MRC_SHMEM_CONTROLRWL           "2MRCCtrlRWLock"
#define MRC_SHMEM_CONTROLOBJ           "3MRCSmControl"
#define MRC_SHMEM_ENGLISHCACHE         "4MRCSmEnglish"
#define MRC_SHMEM_NONENGCACHE          "5MRCSmNonEnglish"

#define MRC_SIGNATURE_CONTROLOBJ       "MRC1"
#define MRC_SIGNATURE_ENGLISHCACHE     "MRC2"
#define MRC_SIGNATURE_NONENGCACHE      "MRC3"

// type definitions

//---------------------------------------------------------------------
// pack on byte boundary to save space
#pragma pack(1)

// shared memory object to contain control and index information

typedef struct _SmHeaderObj
{
   u32 signature;                            // unique identifier
   u32 objsize;                              // size of the object inclusive of this header
} SmHeaderObj;

typedef struct _SmDataHeaderObj
{
   u32 datasize;                             // size of the data Exclusive of this structure
   // data memory here
} SmDataHeaderObj;

// shared memory object containing control and MrIndex
typedef struct _SmIndexControlObj
{
   SmHeaderObj header;                       // object header
   u16 cachedLangID;                         // cached language ID
   u16 counterLang[MRC_NONENGLANG_COUNT];    // language access counters
   u32 indexOffset[MRC_LANG_DBTYPE_COUNT];   // offset to language-dbtype indices
   // one or more group of SmDataHeaderObj containing MrIndex
} SmIndexControlObj;

// shared memory object containing MrFile
typedef struct _SmCachedLangObj
{
   SmHeaderObj header;                       // object header
   u32 indexOffset[MRC_EN_DBTYPE_COUNT];     // offset to language-dbtype indices
   // one or more group of SmDataHeaderObj containing MrFile
} SmCachedLangObj;

// restore original pack
#pragma pack()
//---------------------------------------------------------------------

typedef struct _MRCACHE
{
   void *pHandle;
   u8 *pDataHeader;
   u8 *pDataPos;
} MRCACHE;

typedef struct _MrDBHandle
{
   union {
      FILE *pFileHdl;
      MRCACHE *pCacheHdl;
   } handle;
   u16 langIndex;
   MrCacheType cachetype;
   MrDBType dbtype;
} MrDBHandle;

//---------------------------------------------------------------------
// Utility/Helper
//---------------------------------------------------------------------

s32 fseekMrDBHandle(_IN MrDBHandle *pmhandle, 
                    _IN long offset, 
                    _IN int origin);

s32 readAndAdvanceMrDBHandle(_INOUT void *pdest,
                             _IN size_t destbytecount,
                             _IN MrDBHandle *pmhandle,
                             _INOUT long *pcurpos);

void closeMrDBHandle(_IN MrDBHandle *pmhandle);

MrDBHandle * openMrDBHandle(_IN u8 mrtype,
                            _IN const astring *pLanguage,
                            _INOUT u32 *psize);

//---------------------------------------------------------------------

void MrCCloseCache(MRCACHE *pcache);

MRCACHE * MrCOpenCache(_IN u16 localeIndex,
                       _IN MrDBType dbtype,
                       _INOUT MrCacheType *pctype);

s32 fseekMrCache(_IN MRCACHE *pcache, 
                 _IN u32 offset, 
                 _IN u16 origin);

s32 readAndAdvanceMrCache(_INOUT void *pdest,
                          _IN size_t destbytecount,
                          _IN MRCACHE *pcache,
                          _INOUT long *pcurpos);

//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __MRCACHECOMMON_H_INCLUDED__
