
//-------------------------------------------------------------------------
//
//          Dell Incorporated PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Incorporated and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Incorporated. All Rights Reserved.
//
//  Last Modified By / On:
//    Author: Ryan Brown / Date: 2012 Jul 25
//
//-------------------------------------------------------------------------

#ifndef __MSGREGCOMMON_H_INCLUDED__
#define __MSGREGCOMMON_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------

// local headers
#include "msgregutil.h"

//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Message Registry
//---------------------------------------------------------------------

// macro definitions

#ifdef SYSTEM_TYPE_LINUX
#define MSGREG_FILELOCATION               "/opt/dell/srvadmin/etc/srvadmin-deng/msgreg"
#else
#define MSGREG_FILELOCATION               "C:\\Program Files (x86)\\Dell\\SysMgt\\dataeng\\msgreg"
#endif

#define MSGREG_FILELOCATIONENV            "MSGREGFILEPATH"

#define MSGREG_FILENAME_SUFFIX            "%s.bin" // TODO: deprecate

#define MSGREG_FILE_VERSION               (1)      // up to 256 only
#define MSGREG_LOCALE_LEN                 (2)      // 2 character ISO 639-1 code
#define MSGREG_LANGUAGE_LEN               (6)      // currently used by locale only, extra for future use

#define MSGREG_ARGUMENT_PREFIX            "arg"
#define MSGREG_ARGUMENT_DELIMITER         '~'

#define NUMMAP_CRCSET(type,context,id)    (((type & 0xFF) << 24) | \
                                          ((context & 0xFF) << 16) | \
                                          (id & 0xFFFF))

#define NUMMAP_CRCGET_TYPE(crc)           ((crc >> 24) & 0xFF)
#define NUMMAP_CRCGET_CONTEXT(crc)        ((crc >> 16) & 0xFF)
#define NUMMAP_CRCGET_ID(crc)             (crc & 0xFFFF)

// type definitions

//---------------------------------------------------------------------
typedef enum _MrDataType
{
   MDT_UNDEFINED = 0,
   MDT_MessageID,
   MDT_DetailDesc,
   MDT_MessageComp,
   MDT_FMCategory,
   MDT_FMSeverity,
   MDT_FMCorrAction
} MrDataType;

#ifdef USE_STRMRDATATYPEG
static const astring * strMrDataTypeG[] = {
   "UNDEFINED",
   "MessageID",
   "DetailDesc",
   "MessageComp",
   "FMCategory",
   "FMSeverity",
   "FMCorrAction"
};
#endif // USE_STRMRDATATYPEG

//---------------------------------------------------------------------
typedef enum _MrRecordType
{
   MRT_UNDEFINED = 0,
   MRT_MSGREG,       // db messages
   MRT_CAT,          // db numeric
   MRT_SUBCAT,       // db string
   MRT_REVNUMSTR,    // db string
   MRT_SEVERITY,     // db numeric
   MRT_CTXCODE       // db numeric
} MrRecordType;

#ifdef USE_STRMRRECORDTYPEG
static const astring * strMrRecordTypeG[] = {
   "UNDEFINED",
   "MSGREG",       // db messages
   "CAT",          // db numeric
   "SUBCAT",       // db string
   "REVNUMSTR",    // db string
   "SEVERITY",     // db numeric
   "CTXCODE"       // db numeric
};
#endif // USE_STRMRRECORDTYPEG

//---------------------------------------------------------------------
typedef enum _MrCacheType
{
   MCACHE_UNDEFINED = 0,
   MCACHE_FILEBASED,
   MCACHE_ENGLISHCACHE,
   MCACHE_NONENGCACHE,
   MCACHE_INDEXED
} MrCacheType;

//---------------------------------------------------------------------
// database types
// must start with 0 and sequential, used to index list of database
typedef enum _MrDBType
{
   MRDBTYPE_MESSAGES = 0,
   MRDBTYPE_STRING,
   MRDBTYPE_NUMERIC
} MrDBType;

//---------------------------------------------------------------------
// pack on byte boundary to save space
#pragma pack(1)

typedef struct _MrData
{
   u16 data_size;       // size of record data (exclusive to value)
   u8 data_type;        // record type
   u8 reserved[1];      // reserved field
   u8 value[1];         // variable length record data
} MrData;

typedef struct _MrRecord
{
   u16 record_size;     // size of entire record (inclusive of this object)
   u8 data_count;       // number of data
   u8 record_type;      // record type
   MrData data[1];      // variable length record data
} MrRecord;

typedef struct _MrIndex
{
   u32 msgid_crc;          // crc representation of message id
   u8 flag;                // internal flag used for crc collisions
   u8 record_offset[3];	   // MrFile offset to the location of the record
} MrIndex;

typedef struct _MrFile
{
   u32 signature;                      // file authenticity signature
   u8 version;                         // file version
   u8 argdelimiter;                    // dynamic element delimiter
   u8 language[MSGREG_LANGUAGE_LEN];   // language string xx_YY + null terminator
   u32 count;                          // number of records
   // one or more MrIndex here
   // one or more MrRecord here
} MrFile;

// restore original pack
#pragma pack()
//---------------------------------------------------------------------

typedef s32 (__cdecl *PFNPROCESSMRFILE)(_IN MrFile *pmrfile,
                                        _INOUT void *pprocessdata);

typedef s32 (__cdecl *PFNPROCESSMRINDEX)(_IN MrIndex *pmrindex,
                                         _IN u32 index,
                                         _IN u32 offset,
                                         _INOUT void *pprocessdata);

typedef s32 (__cdecl *PFNPROCESSMRRECORD)(_IN MrRecord *pmrrecord,
                                          _IN u32 index,
                                          _INOUT void *pprocessdata);

// function prototypes

s32 MrFileParse(_IN const astring *pmrfilename,
                _IN PFNPROCESSMRFILE pfnProcessMrFile,
                _IN PFNPROCESSMRINDEX pfnProcessMrIndex,
                _IN PFNPROCESSMRRECORD pfnProcessMrRecord,
                _INOUT void *pprocessdata,
                _INOUT u32 *pbytesread);

s32 readAndAdvanceMrFile(_INOUT void *pdest,
                         _IN size_t destbytecount,
                         _IN FILE *pfinfile,
                         _INOUT long *pcurpos);

FILE * openRegistryFile(_IN u8 mrtype,
                        _IN const astring *pLanguage,
                        _INOUT u8 *pdbtype,
                        _INOUT u32 *psize);

FILE * openRegistryFile2(_IN const astring *pLanguage,
                         _INOUT u8 dbtype,
                         _INOUT u32 *psize);

//---------------------------------------------------------------------
// Message Registry database type
//---------------------------------------------------------------------

typedef struct _MrFileProp
{
   const astring *pfilenamefmt;
   const astring *psignature;
} MrFileProp;

const u8 * mrtToIndexOffset(void);

u32 mrtToIndexOffsetCount(void);

//---------------------------------------------------------------------
// Debugging
//---------------------------------------------------------------------

// end of memory marker definitions (for debugging memory)
#define EOM_MARKER_BYTECOUNT      1
#define EOM_MARKER_SIGN           0x99

//---------------------------------------------------------------------
// Utility/Helper
//---------------------------------------------------------------------

const MrFileProp * getMrFileStaticProp(void);

u32 getMrFileStaticPropCount(void);

void _NSVOUninit(_INOUT NSVObj *pnsvo);

s32 _NSVOInit(_INOUT NSVObj *pnsvo,
              _IN u32 initialsize);

nsvastring * _NSVOCat(_INOUT NSVObj *pnsvo,
                      _IN const astring *pstrz);

//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __MSGREGCOMMON_H_INCLUDED__
