
//-------------------------------------------------------------------------
//
//          Dell Incorporated PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Incorporated and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Incorporated. All Rights Reserved.
//
//  Last Modified By / On:
//    Author: Ryan Brown / Date: 2012 Jul 25
//
//-------------------------------------------------------------------------

#ifndef __MSGREGUTIL_H_INCLUDED__
#define __MSGREGUTIL_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------

// standard headers
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#ifdef SYSTEM_TYPE_LINUX
#include <stdint.h>
#endif

// DE headers
#include "dcsysdebug.h"
#include "dcossdk.h"
#include "dcdetypes.h"
#include "dcsmstatus.h"
#include "dcsupapi.h"
#include "dcisocppsecurecompat.h"

// local headers
#include "dcmsgregdef.h"

//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Message Registry
//---------------------------------------------------------------------

// macro definitions

#define BUFSIZE_256     (256)
#define BUFSIZE_32      (32)

#define ENGLISH_LOCALE_INDEX     (0)

#define DEFAULT_LOCALE_INDEX     ENGLISH_LOCALE_INDEX

//---------------------------------------------------------------------
#ifndef LIBDELLDLOG_H

#define MRLOG_FILENAME              "mrlog.log"
#define MRLOG_FILELOCATION          ""
#define MRLOG_FILELOCATIONENV       "MRLOGFILEPATH"

#define OUT_NONE                0
#define OUT_CONSOLE             1
#define OUT_RAMFILE             2
#define OUT_FLASHFILE           4

#define DEBUG_LEVEL_NONE            0
#define DEBUG_LEVEL_CRITICAL        1
#define DEBUG_LEVEL_ERROR           2
#define DEBUG_LEVEL_WARNING         3
#define DEBUG_LEVEL_INFO            4
#define DEBUG_LEVEL_DEBUG           5

void mrlogprintf(_IN unsigned char level, 
                 _IN unsigned int source, 
                 _IN const char *format, 
                 _IN ...);

#define DPRINTF_CRIT(SOURCE, ...)         { mrlogprintf(DEBUG_LEVEL_CRITICAL, SOURCE, __VA_ARGS__); }
#define DPRINTF_ERROR(SOURCE, ...)        { mrlogprintf(DEBUG_LEVEL_ERROR, SOURCE, __VA_ARGS__); }
#define DPRINTF_WARNING(SOURCE, ...)      { mrlogprintf(DEBUG_LEVEL_WARNING, SOURCE, __VA_ARGS__); }
#define DPRINTF_INFO(SOURCE, ...)         { mrlogprintf(DEBUG_LEVEL_INFO, SOURCE, __VA_ARGS__); }
#define DPRINTF_DEBUG(SOURCE, ...)        { mrlogprintf(DEBUG_LEVEL_DEBUG, SOURCE, __VA_ARGS__); }

#endif // LIBDELLDLOG_H
//---------------------------------------------------------------------

u16 mrlogcycleoutput(void);
u16 mrlogcyclelevel(void);

void mrlogsetoutput(_IN u16 dbgOutNew);
void mrlogsetlevel(_IN u16 dbgLevelNew);

void mrloggetstrtime(_IN char *sbuf, 
                     _IN int bufsize);

//---------------------------------------------------------------------
// Utility/Helper
//---------------------------------------------------------------------

#define U32to4CHAR(X)      *((char*)&X), \
                           (char)*(((u8*)&X)+1), \
                           (char)*(((u8*)&X)+2), \
                           (char)*(((u8*)&X)+3)

// This is reversed in dcsupt, we define this way since default is Linux
#ifdef SYSTEM_TYPE_WINNT

#ifndef F_OK
#define F_OK   (0)
#endif

#define strcasecmp(X,Y)		stricmp(X,Y)
#define snprintf           _snprintf

#endif // SYSTEM_TYPE_WINNT

//---------------------------------------------------------------------

// function prototypes

const astring * getValidLocale(_IN const astring *plocalestr);

const astring * getLocaleByIndex(_IN u16 index);

u16 getLocaleIndex(_IN const astring *plocalestr);

const astring ** getValidLocaleList(_INOUT u32 *pcount);

u8 atou8(_IN const astring *pstr);

astring * u8dataToStringBit(_IN u8 *pdata,
                            _IN u32 count_byu8);

astring * fbackup(const astring *pfilename);

u32 mr_crc32_str(_IN const astring *buf);

//---------------------------------------------------------------------
// Header Code Ends
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __MSGREGUTIL_H_INCLUDED__
