//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding an EPI a matter of coding
// interfaces defined
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM EPI
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 3 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <epidispatch.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  EventImplementerDispatch
//---------------------------------------------------------------------
s32 EventImplementerDispatch(_IN u32 epiID,
							 _IN u32 reqType,
							 _IN const void *pInBuf, 
							 _IN u32 inBufSize,
							 _OUT void *pOutBuf, 
							 _IN u32 outBufSize,
							 _OUT u32 *pBytesReturned)
{
	s32 status;
	u32 bytesWrittenToOutBuf;


    SysDbgMsg4(("EventImplementerDispatch: entry\n"));

	// default no bytes returned
    *pBytesReturned = 0;
	bytesWrittenToOutBuf = 0;

	// dispatch based on request type
    switch (reqType)
    {
		case SM_EPDRT_EPI_LOAD:
			// sanity check
			if (inBufSize < sizeof(SMEMReqEPILoad))
			{
				status = SM_STATUS_INVALID_PARAMETER;
				break;
			}

			// perform request
			status = EPIDispLoad(((SMEMReqEPILoad *)pInBuf)->pSMEPEMDE);
			// return 0 bytes
		break;

		case SM_EPDRT_EPI_UNLOAD:
			// perform request
			status = EPIDispUnLoad();
			// return 0 bytes
		break;

		case SM_EPDRT_EPI_START_MONITOR:
			// perform request
			status = EPIDispStartMonitor();
			// return 0 bytes
		break;

		case SM_EPDRT_EPI_STOP_MONITOR:
			// perform request
			status = EPIDispStopMonitor();
			// return 0 bytes
		break;

		default:
			status = SM_STATUS_CMD_BAD;
    } // switch (reqType)

	// check if bytesreturned set
	if (status == SM_STATUS_SUCCESS)
	{
		*pBytesReturned = bytesWrittenToOutBuf;
	}

    SysDbgMsg4(("EventImplementerDispatch: exit\n"));

	UNREFERENCED_PARAMETER(epiID);
	UNREFERENCED_PARAMETER(pOutBuf);
	UNREFERENCED_PARAMETER(outBufSize);

    return status;
} // EventImplementerDispatch 


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------
