//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding an EPI a matter of coding
// interfaces defined
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM EPI 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 6 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <epiepemdispatch.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

const SMEPEMDispatchEnv *pEPIEPEMDE = NULL;


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// EPIEPEMDAttach
//---------------------------------------------------------------------
booln EPIEPEMDAttach(_IN const SMEPEMDispatchEnv *pEPEMDE)
{

	SysDbgMsg4(("EPIEPEMDAttach: entry\n"));

	// setup the event plug-in implementer to event plug-in manager interface
	pEPIEPEMDE = pEPEMDE;

	SysDbgMsg4(("EPIEPEMDAttach: exit\n"));

	return TRUE;
} // EPIEPEMDAttach


//---------------------------------------------------------------------
// EPIEPEMDDetach
//---------------------------------------------------------------------
void EPIEPEMDDetach(void)
{

	SysDbgMsg4(("EPIEPEMDDetach: entry\n"));

	// clear the push interface
	pEPIEPEMDE = NULL;

	SysDbgMsg4(("EPIEPEMDDetach: exit\n"));
} // EPIEPEMDDetach


//---------------------------------------------------------------------
// EPIEPEMDGetEPEMDE
//---------------------------------------------------------------------
const SMEPEMDispatchEnv *EPIEPEMDGetEPEMDE(void)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// clear the push interface
	pEPEMDE = pEPIEPEMDE;


	SysDbg3(
		// check for uninitialized state
		if (NULL == pEPEMDE)
		{
			SysDbgMsg3(("EPIEPEMDGetEPEMDE: invalid pEPEMDE == NULL\n"));
		}
	)

	return pEPEMDE;
} // EPIEPEMDGetEPEMDE


//---------------------------------------------------------------------
// EPIEPEMDFreeGeneric
//---------------------------------------------------------------------
void EPIEPEMDFreeGeneric(_IN void *pMem)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return;
	}

	pEPEMDE->pEPEMDTable->FreeGeneric(pMem);
} // EPIEPEMDFreeGeneric


//---------------------------------------------------------------------
// EPIEPEMDGetOSLocaleNameUTF8List
//---------------------------------------------------------------------
astring *EPIEPEMDGetOSLocaleNameUTF8List(_INOUT u32 *pUTF8BufSize)
{
	const SMEPEMDispatchEnv *pEPEMDE;
	
	
	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();
	
	// available?
	if (NULL == pEPEMDE)
	{
		return NULL;
	}
	
	// get the Null separated OS locale name UTF-8
	// The returned string is Double Null terminated
	return pEPEMDE->pEPEMDTable->GetOSLocaleNameUTF8List(pUTF8BufSize);
} // EPIEPEMDGetOSLocaleNameUTF8List


//---------------------------------------------------------------------
// EPIEPEMDAllocEventMessageData
//---------------------------------------------------------------------
EventMessageData *EPIEPEMDAllocEventMessageData(_IN u32 memAllocSize)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return NULL;
	}

	return pEPEMDE->pEPEMDTable->AllocEventMessageData(memAllocSize);
} // EPIEPEMDAllocEventMessageData


//---------------------------------------------------------------------
// EPIEPEMDFreeEventMessageData
//---------------------------------------------------------------------
void EPIEPEMDFreeEventMessageData(_IN EventMessageData *pEMD)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return;
	}

	pEPEMDE->pEPEMDTable->FreeEventMessageData(pEMD);
} // EPIEPEMDFreeEventMessageData


//---------------------------------------------------------------------
// EPIEPEMDLogTagNameInsert
//---------------------------------------------------------------------
s32 EPIEPEMDLogTagNameInsert(_IN const astring *pUTF8LogTag, 
							 _IN const PFNLOGROUTINE pfnLogRoutine)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return SM_STATUS_NO_SUCH_OBJECT;
	} // if (NULL == pEPEMDE)

	return pEPEMDE->pEPEMDTable->LogTagNameInsert(pUTF8LogTag, 
		pfnLogRoutine);
} // EPIEPEMDLogTagNameInsert


//---------------------------------------------------------------------
//  EPIEPEMDLogTagNameGetUTF8List
//---------------------------------------------------------------------
astring *EPIEPEMDLogTagNameGetUTF8List(_INOUT u32 *pUTF8BufSize, 
									   _INOUT u32 *pNumLogTags)
{
	const SMEPEMDispatchEnv *pEPEMDE;
	
	
	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();
	
	// available?
	if (NULL == pEPEMDE)
	{
		return NULL;
	} // if (NULL == pEPEMDE)
	
	return pEPEMDE->pEPEMDTable->LogTagNameGetUTF8List(pUTF8BufSize, 
		pNumLogTags);
} // EPIEPEMDLogTagNameGetUTF8List


//---------------------------------------------------------------------
// EPIEPEMDLogTagNameRemove
//---------------------------------------------------------------------
s32 EPIEPEMDLogTagNameRemove(_IN const astring *pUTF8LogTag, 
							 _IN const PFNLOGROUTINE pfnLogRoutine)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return SM_STATUS_NO_SUCH_OBJECT;
	} // if (NULL == pEPEMDE)

	return pEPEMDE->pEPEMDTable->LogTagNameRemove(pUTF8LogTag, 
		pfnLogRoutine);
} // EPIEPEMDLogTagNameRemove


//---------------------------------------------------------------------
// EPIEPEMDLogEventDataByLogTagName
//---------------------------------------------------------------------
s32 EPIEPEMDLogEventDataByLogTagName(_IN const astring *pUTF8LogTag, 
									 _IN LogRoutineData *pLRData)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return SM_STATUS_NO_SUCH_OBJECT;
	} // if (NULL == pEPEMDE)

	return pEPEMDE->pEPEMDTable->LogEventDataByLogTagName(pUTF8LogTag, 
		pLRData);
} // EPIEPEMDLogEventDataByLogTagName


//---------------------------------------------------------------------
// EPIEPEMDGetepiID
//---------------------------------------------------------------------
u32 EPIEPEMDGetepiID(void)
{
	const SMEPEMDispatchEnv *pEPEMDE;


	// get push environment
	pEPEMDE = EPIEPEMDGetEPEMDE();

	// available?
	if (NULL == pEPEMDE)
	{
		return EVTIMP_PLUGIN_ID_PUBLIC;
	}

	// get the plugin id
	return pEPEMDE->pEPEMDTable->GetepiID(pEPEMDE);
} // EPIEPEMDGetepiID


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------

