//---------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-2003 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
//   epiexport.c
//
// Abstract/Purpose:
//
//   Source file for exporting Event Plug-in Implementer dispatch entry point
//
// Environment:
//
//   All supported operating systems (user mode)
//
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif

// make sure export name for dispatch function has been defined
#ifndef EPIEXPORT_DISPATCHFN
#error EPIEXPORT_DISPATCHFN not defined.
#error Example: EPIEXPORT_DISPATCHFN=SAMEPI32_SMEVENTImplementerDispatch
#error NOTE: Event implementer name should be in uppercase followed by word size.
#error Example: SAMEPI32
#endif


//---------------------------------------------------------------------
// $(DATAENGINESDK) global include files
//---------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebug.h>
#include <dcsupapi.h>


//---------------------------------------------------------------------
// $(DATAENGINESDK) Event Plug-in include files
//---------------------------------------------------------------------
#include <dcsmepim.h>


//---------------------------------------------------------------------
// $(DATAENGINESDK) EPI include files
//---------------------------------------------------------------------
#include <epiexport.h>


//---------------------------------------------------------------------
// NOTE: In modules that export functions, SMINTF must be set up
// for exporting after all other header files are included
//---------------------------------------------------------------------
#define __EXPORTING
#undef SMINTF
#include <dcdetypes.h>


//---------------------------------------------------------------------
// Global data
//---------------------------------------------------------------------
// global code allocator for SysDbg
SYS_DBG_DECLARE_GLOBAL_CODEDATA;

// original module context
SMECInfo EPIexportOrigECI;


//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// ModuleAttach - called when module is loaded by OS
//---------------------------------------------------------------------
booln __lxinit__ ModuleAttach(void)
{
	// get original export context info
	SMGetExportContext(&EPIexportOrigECI);
	
	// first time init for SysDbg
	// send debug trace to console
	// and the file specified
    SysDbgInit((SYSTEM_DEBUG_SCREEN | SYSTEM_DEBUG_FILE), 
		DEFAULT_SYSDBG_FILENAME);

	// tell Event implementer it is being loaded by OS
	if (EventImplementerAttach() != SM_STATUS_SUCCESS)
	{
		return FALSE;
	}

	return TRUE;
} // ModuleAttach


//---------------------------------------------------------------------
// ModuleDetach - called when module is unloaded by OS
//---------------------------------------------------------------------
booln __lxfini__ ModuleDetach(void)
{
	// tell Event implementer it is being unloaded by OS
	EventImplementerDetach();

	return TRUE;
} // ModuleDetach


//---------------------------------------------------------------------
// MODULEXX_SMEVENTImplementerDispatch - exported dispatch function
// called by Event Plug-in Manager
//---------------------------------------------------------------------
SMINTF s32 _cdecl EPIEXPORT_DISPATCHFN(_IN u32 epiID, 
									   _IN u32 reqType, 
									   _IN const void *pInBuf, 
									   _IN u32 inBufSize, 
									   _OUT void *pOutBuf, 
									   _IN u32 outBufSize, 
									   _OUT u32 *pBytesReturned)
{
    SMECInfo eci;
	s32 retStatus;


	// set original export context for Event implementer
    SMSetExportContext(&eci, &EPIexportOrigECI);

	// dispatch the request
	retStatus = EventImplementerDispatch(epiID, 
		reqType, 
		pInBuf, 
		inBufSize, 
		pOutBuf, 
		outBufSize, 
		pBytesReturned);

	// restore export context
    SMResetExportContext(&eci);

    return retStatus;
} // MODULEXX_SMEVENTImplementerDispatch


//---------------------------------------------------------------------
//  Code End
//---------------------------------------------------------------------
