
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a EPI a matter of coding
// interfaces defined
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM EPIs 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 3 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <samepi.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------

// data structure for EPI context
typedef struct _SamEPIGlobalsData
{
	// denotes if module is exiting set to
	// FALSE on load and TRUE on stop monitor
	booln exitingFlag;
	// alignment for next data element
	u8 reservedAlign[6];

	// NOTE: Plugin implementer Developers!
	// Place your EPI specific data here
} SamEPIGlobalsData;


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// pointer to EPI global data
SamEPIGlobalsData *pSEGData = NULL;

void *pSamEPIGlobalsLock = NULL;


//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamEPIGlobalsLock
//---------------------------------------------------------------------
void SamEPIGlobalsLock(void)
{
	// lock the global mutex lock
	SMMutexLock(pSamEPIGlobalsLock, 
		SM_WAIT_MSECS_INFINITE);
} // SamEPIGlobalsLock


//---------------------------------------------------------------------
// SamEPIGlobalsUnLock
//---------------------------------------------------------------------
void SamEPIGlobalsUnLock(void)
{
	// unlock the global mutex lock
	SMMutexUnLock(pSamEPIGlobalsLock);
} // SamEPIGlobalsUnLock


//---------------------------------------------------------------------
// SamEPIGlobalsAlloc
//---------------------------------------------------------------------
s32 SamEPIGlobalsAlloc(void)
{
	void *pLock;


	SysDbgMsg4(("SamEPIGlobalsAlloc: entry\n"));

	// allocate the global mutex lock
	if ((pSamEPIGlobalsLock = SMMutexCreate(NULL)) == NULL)
	{
		SysDbgMsg3(("SamEPIGlobalsAlloc: failed to allocate pSamEPIGlobalsLock\n"));
		return SM_STATUS_OUT_OF_MEMORY;
	}
	
	// lock the context
	SamEPIGlobalsLock();

	// allocate memory
	if ((pSEGData = (SamEPIGlobalsData *)SMAllocMem(sizeof(SamEPIGlobalsData))) == NULL)
	{
		SysDbgMsg3(("SamEPIGlobalsAlloc: failed to allocate static context memory\n"));

		// clear global lock before free
		pLock = pSamEPIGlobalsLock;
		// catch GPFs
		pSamEPIGlobalsLock = NULL;
		
		// free global mutex lock
		SMMutexDestroy(pLock);

		// failed
		return SM_STATUS_OUT_OF_MEMORY;
	}

	// setup important values
	pSEGData->exitingFlag = TRUE;

	// unlock the context
	SamEPIGlobalsUnLock();

	// done
	return SM_STATUS_SUCCESS;
} // SamEPIGlobalsAlloc


//---------------------------------------------------------------------
// SamEPIGlobalsFree
//---------------------------------------------------------------------
void SamEPIGlobalsFree(void)
{
	void *pLock;


	SysDbgMsg4(("SamEPIGlobalsFree: entry\n"));

	// lock the context
	SamEPIGlobalsLock();

	// setup important values
	pSEGData->exitingFlag = TRUE;

	// free the memory
	SMFreeMem(pSEGData);
	pSEGData = NULL;

	// clear global lock before free
	pLock = pSamEPIGlobalsLock;
	// catch GPFs
	pSamEPIGlobalsLock = NULL;

	// free global mutex lock
	SMMutexDestroy(pLock);
} // SamEPIGlobalsFree


//---------------------------------------------------------------------
// SamEPIGlobalsSetExitingFlag
//---------------------------------------------------------------------
booln SamEPIGlobalsSetExitingFlag(_IN booln newFlag)
{
	booln oldFlag;


	// sanity check
	if (pSEGData == NULL)
	{
		// module exiting
		return TRUE;
	}

	// lock the context
	SamEPIGlobalsLock();

	// save old
	oldFlag = pSEGData->exitingFlag;

	// save new
	pSEGData->exitingFlag = newFlag;

	// unlock the context
	SamEPIGlobalsUnLock();

	// done
	return oldFlag;
} // SamEPIGlobalsSetExitingFlag


//---------------------------------------------------------------------
// SamEPIGlobalsGetExitingFlag
//---------------------------------------------------------------------
booln SamEPIGlobalsGetExitingFlag(void)
{
	booln curFlag;


	// sanity check
	if (pSEGData == NULL)
	{
		// module exiting
		return TRUE;
	}

	// lock the context
	SamEPIGlobalsLock();

	// save current
	curFlag = pSEGData->exitingFlag;

	// unlock the context
	SamEPIGlobalsUnLock();

	// done
	return curFlag;
} // SamEPIGlobalsGetExitingFlag


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

