//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding an EPI a matter of coding
// interfaces defined
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM EPI
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 3 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <samepi.h>



//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// EPIDispStartMonitor - Called by epimgrdispatch.c during
// SM_EPDRT_EPI_START_MONITOR to indicate to Event Plug-in module to
// perform any run time procedures to begin monitoring its events
//---------------------------------------------------------------------
s32 EPIDispStartMonitor(void)
{
    SysDbgMsg4(("EPIDispStartMonitor: entry\n"));

	// start the event monitoring thread
	if (FALSE == SamEPISuptStartEventMonitor())
	{
		return SM_STATUS_UNSUCCESSFUL;
	}

    SysDbgMsg4(("EPIDispStartMonitor: exit\n"));

    return SM_STATUS_SUCCESS;
} // EPIDispStartMonitor


//---------------------------------------------------------------------
// EPIDispStopMonitor - Called by epimgrdispatch.c during
// SM_EPDRT_EPI_STOP_MONITOR to indicate to Event Plug-in module to
// perform any run time procedures to end monitoring its events
//---------------------------------------------------------------------
s32 EPIDispStopMonitor(void)
{

    SysDbgMsg4(("EPIDispStopMonitor: entry\n"));

	// set exiting flag
	SamEPIGlobalsSetExitingFlag(TRUE);

	// stop the event monitoring thread
	SamEPISuptStopEventMonitor();

    SysDbgMsg4(("EPIDispStopMonitor: exit\n"));

    return SM_STATUS_SUCCESS;
} // EPIDispStopMonitor


//---------------------------------------------------------------------
// EPIDispLoad - Called by epimgrdispatch.c during SM_EPDRT_EPI_LOAD to
// indicate that the Event Plug-in module should perform load time
// procedures in order to prepare for subsequent requests and return
// success. If error returned then the Event Plug-in is freed from 
// Process or OS memory
//---------------------------------------------------------------------
s32 EPIDispLoad(_IN const SMEPEMDispatchEnv *pEPEMDE)
{
	s32 status;


    SysDbgMsg4(("EPIDispLoad: entry\n"));
	
	// Allocate any resources needed for
	// subsequent request processing
	if (EPIEPEMDAttach(pEPEMDE) == FALSE)
	{
		return SM_STATUS_UNSUCCESSFUL;
	}

	// allocate globals
	status = SamEPIGlobalsAlloc();

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		// detach and return status
		EPIEPEMDDetach();
		// failed
		return status;
	}

	// set exiting flag
	SamEPIGlobalsSetExitingFlag(FALSE);

    SysDbgMsg4(("EPIDispLoad: exit\n"));

	// done
    return SM_STATUS_SUCCESS;
} // EPIDispLoad 


//---------------------------------------------------------------------
// EPIDispUnLoad - Called by epimgrdispatch.c during SM_EPDRT_EPI_UNLOAD
// to indicate to Event Plug-in module to perform any unload time
// procedures and prepare to be freed from Process and OS memory space
//---------------------------------------------------------------------
s32 EPIDispUnLoad(void)
{

    SysDbgMsg4(("EPIDispUnLoad: entry\n"));

	// free the global memory and lock
	SamEPIGlobalsFree();

	// clear the push environment
	EPIEPEMDDetach();

    SysDbgMsg4(("EPIDispUnLoad: exit\n"));

    return SM_STATUS_SUCCESS;
} // EPIDispUnLoad


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------
