
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2010 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Tushar Oza
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Sample EPI support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <samepisupt.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------
#define MAX_DESC_SUBST_STR			(8)
#define MAX_DESC_SUBST_STR_SIZE		(MAX_UNIDAT_STRING_SIZE / sizeof(ustring))
#define XML_SAMPLE_SOURCE_NAME		"Sample Source"
#define XML_SAMPLE_USER_NAME		"Sample User"

#ifdef SYSTEM_TYPE_LINUX
#define LOG_PATH_NAME_FMT_STR				"%s/%s"
#else
#define LOG_PATH_NAME_FMT_STR				"%s\\%s"
#endif // SYSTEM_TYPE_LINUX


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamSuptEventListener
// NOTE: When using event listeners you have to have _cdecl functions
// and compile the executable with multithreading libraries
//---------------------------------------------------------------------
s32 _cdecl SamSuptEventListener(_IN const DataEventHeader *pDEH)
{
	astring *pEvtTimeStampStr;
	astring *pOSLocaleStr;
	astring *pLogTagNameUTF8List;
	astring *pPathFileName;
	void *pEventBody;
	DEObjStatusChange *pDEOSC;
	EventMessageData *pEMD;
	u32 evtBodySize;
	u32 ii;
	u32 size;
	u32 bufSize;
	u32 numLogTags;


	// get the event time stamp as a string
	pEvtTimeStampStr = SMILDOGetObjTimeUTF8Str(pDEH->evtTimeStamp);

	printf("SamSuptEventListener: Recieved event\nType: %u Time: %s Flags: 0x%02X\n", 
		(u32)pDEH->evtType, 
		pEvtTimeStampStr, 
		pDEH->evtFlags);

	// free the string
	SMILFreeGeneric(pEvtTimeStampStr);
	// catch GPFs
	pEvtTimeStampStr = NULL;

	// point to event body
	pEventBody = (void *)((u8 *)pDEH + 
		sizeof(DataEventHeader));

	// set size of body
	evtBodySize = (pDEH->evtSize - 
		sizeof(DataEventHeader));

	// determine event type
	switch (pDEH->evtType)
	{
		case DATA_EVENT_DATA_OBJ_DIRTY_LIST:
			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_DATA_OBJ_DIRTY_LIST\n");
			
			// initialize
			bufSize = 0;

			// get list of locale to log message
			// It is the EPI's responsibility to 
			// form the locale string for each event message
			// and call the logroutine with appropriate logtag names
			// the list returned is a Null-separated-value list
			// with double null termination to mark the end of data buffer
			pOSLocaleStr = EPIEPEMDGetOSLocaleNameUTF8List(&bufSize);

			// failed? should atleast get default locale
			// else punt
			if (NULL == pOSLocaleStr)
			{
				printf("failed to get default locale string\n");
				break;
			} // if (NULL == pOSLocaleStr)

			// calculate size
			size = sizeof(EventMessageData) + 
				(MAX_DESC_SUBST_STR * MAX_DESC_SUBST_STR_SIZE);
			
			// allocate 'size' buffer
			pEMD = EPIEPEMDAllocEventMessageData(size);

			// failed?
			if (pEMD != NULL)
			{
				// initialize
				ii = 0;

				// allocate memory
				pEMD->ppUTF8DescStr[ii] = (astring *)SMAllocMem(MAX_DESC_SUBST_STR_SIZE);

				// failed?
				if (pEMD->ppUTF8DescStr[ii] != NULL)
				{
					strcpy_s(pEMD->ppUTF8DescStr[ii],
						MAX_DESC_SUBST_STR_SIZE,
						"First: Received a Dirty Event\n");

					// increment message string counter
					ii = ii + 1;

					// allocate memory
					pEMD->ppUTF8DescStr[ii] = (astring *)SMAllocMem(MAX_DESC_SUBST_STR_SIZE);
					
					// failed?
					if (pEMD->ppUTF8DescStr[ii] != NULL)
					{
						// copy message
						strcpy_s(pEMD->ppUTF8DescStr[ii],
							MAX_DESC_SUBST_STR_SIZE,
							"Second: Called Log Commit\n");
						// increment message string counter
						ii = ii + 1;
					} // if (pEMD->ppUTF8DescStr[ii] != NULL)

					// continue fault tolerant
					pEMD->numUTF8DescStr = ii;

					// set other event logging parameters
					pEMD->mcMsgId = MSG_DIRTYOBJLIST;
					pEMD->logType = SMLOG_TYPE_INFORMATION;
					pEMD->mcCatId = CATEGORY_DE;

					// get sample log pathfilename
					if ((pPathFileName = SMMakePathFileNameByPIDAndType(
						SM_PRODUCT_ID_DATA_ENGINE,
						SM_PATH_TYPE_PLOG,
						SM_PSUBPATH_LOG,
						SAMPLE_XMLLOG_FILE_NAME)) != NULL)
					{
						// save pathfilename
						strcpy_s(pEMD->pPathFileName,
							MAX_PATH_SIZE,
							pPathFileName);

						// free pathfilename buffer
						SMFreeGeneric(pPathFileName);
						// catch GPFs
						pPathFileName = NULL;

						// make sure the XML file exists
						// good to check return status from the call
						SMForceXMLLogExists(pEMD->pPathFileName, 
							CATEGORY_DE, 
							XML_SAMPLE_USER_NAME, // user info
							XML_SAMPLE_SOURCE_NAME);
					}
					else
					{
						printf("SMGetInstallPathByProductID failed\n");
						// we set it to invalid for now in the sample
						// so that the event plugin manager can pick up the
						// default pathfilename from IS
						pEMD->pPathFileName[0] = '\0';
					}

					// set the userinfo temporarily
					pEMD->pXMLUserInfo = XML_SAMPLE_USER_NAME;
					// initialize
					bufSize = 0;
					numLogTags = 0;

					// get the list of registered log tags
					pLogTagNameUTF8List = EPIEPEMDLogTagNameGetUTF8List(
																&bufSize, 
																&numLogTags);

					// failed?
					if (pLogTagNameUTF8List != NULL)
					{
						// commit to logs by logtag name
						EPIEPEMDLogEventDataByLogTagName(EPM_OS_LOG_TAG, 
							(LogRoutineData *)pEMD);
						
						// free memory
						EPIEPEMDFreeGeneric((void *)pLogTagNameUTF8List);
						// catch GPFs
						pLogTagNameUTF8List = NULL;
					} // if (pLogTagNameUTF8List != NULL)

					// Free memory
					while (ii != 0)
					{
						// decrement
						ii = ii - 1;

						// and free memory
						SMFreeMem(pEMD->ppUTF8DescStr[ii]);
						// catch GPFs
						pEMD->ppUTF8DescStr[ii] = NULL;
					} // while (ii != 0)
				} // if (pEMD->ppUTF8DescStr[ii] != NULL)

				// free EMD memory
				EPIEPEMDFreeEventMessageData(pEMD);
				// catch GPFs
				pEMD = NULL;
			} // if (pEMD != NULL)

			// free locale str
			EPIEPEMDFreeGeneric(pOSLocaleStr);
			// catch GPFs
			pOSLocaleStr = NULL;

		break;

		case DATA_EVENT_OBJ_STATUS_CHG:
			// point to data within event
			pDEOSC = (DEObjStatusChange *)pEventBody;

			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_OBJ_STATUS_CHG\n");

			// print information
			printf("prevObjStatus: %02X prevRedStatus: %02X\n", 
				pDEOSC->prevObjStatus, 
				pDEOSC->prevRedStatus);
		break;

		case DATA_EVENT_DATA_CONSUMER_ATTACHED:
			// print event
			printf("Event Plug-in Implementer attached to Data Manager\nData Manager started\n");
		break;

		case DATA_EVENT_DATA_CONSUMER_DETACHED:
			// print event
			printf("Event Plug-in Implementer detached from Data Manager\nData Manager stopped\n");
		break;

		case DATA_EVENT_DATA_OBJ_CREATE:
			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_DATA_OBJ_CREATE\n");
		break;

		case DATA_EVENT_DATA_OBJ_DESTROY:
			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_DATA_OBJ_DESTROY\n");
		break;

		case DATA_EVENT_DATA_OBJ_BRANCH_CREATE:
			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_DATA_OBJ_BRANCH_CREATE\n");
		break;

		case DATA_EVENT_DATA_OBJ_BRANCH_DESTROY:
			// print event
			printf("Event Plug-in Implementer recieved DATA_EVENT_DATA_OBJ_BRANCH_DESTROY\n");
		break;

		case DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID:
			// print event
			printf("Event Plug-in Implementer recieved obsolete event: DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID\n");
		break;

		case DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID:
			// print event
			printf("Event Plug-in Implementer recieved obsolete event: DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID\n");
		break;

		case DATA_EVENT_DATA_MANAGER_ATTACHED:
			// print event
			printf("Data Manager attached event\n");
		break;

		case DATA_EVENT_DATA_MANAGER_DETACHED:
			// print event
			printf("Data Manager detached event\n");
		break;

		case DATA_EVENT_POWER_SUSPEND:
			printf("Data Manager Power Suspend event\n");
		break;

		case DATA_EVENT_POWER_RESUME:
			printf("Data Manager Power Resume event\n");
		break;

		case DATA_EVENT_DEVICE_ADD:
		{
			DESvcDeviceEvtData *pDevInfo;

			printf("Data Manager Device Add event\n");

			if (evtBodySize >= sizeof(DESvcDeviceEvtData))
			{
				pDevInfo = (DESvcDeviceEvtData*)pEventBody;
				switch (pDevInfo->devType)
				{
					case DESVC_DEVTYPE_PCI:
						printf("PCI Device Added\nVendorID: %04X DeviceID: %04X\nSubvendorID: %04X SubdeviceID: %04X\n", 
							pDevInfo->Info.PCIInfo.VendorID,
							pDevInfo->Info.PCIInfo.DeviceID,
							pDevInfo->Info.PCIInfo.SubVendorID,
							pDevInfo->Info.PCIInfo.SubDeviceID);
						break;

					case DESVC_DEVTYPE_USB:
						printf("USB Device Added\nVid: %04X Did: %04X\n", 
							pDevInfo->Info.USBInfo.VendorID, 
							pDevInfo->Info.USBInfo.ProductID);
						break;

					default:
						printf("Unknown device added\n");
						break;
				}
			}
			else
			{
				printf("DESvcDeviceEvtData buffer size underrun\n");
			}
		}
		break;

		case DATA_EVENT_DEVICE_REMOVE:
			printf("Data Manager Device Remove event\n");
			// NOTE same parameters available in DESvcDeviceEvtData as DEVICE_ADD event
		break;

		default:
			// print event
			printf("Event Plug-in Implementer recieved non-base event body: \n");
			printf("\n");
		break;
	} // switch (pDEH->evtType)

	// add a line between events
	printf("\n");

	// continue sending event to 
	// next listner in the chain
	return SM_STATUS_SUCCESS;
} // SamSuptEventListener


//---------------------------------------------------------------------
//  SamEPISuptStartEventMonitor
//---------------------------------------------------------------------
booln SamEPISuptStartEventMonitor(void)
{
	s32 status;


	printf("Sample Event Plug-in Implementer connecting to Data Manager...\r");

	// insert event listener
	status = SMILAddEventListener(SamSuptEventListener);

	// dummy print
	printf(" ");

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		printf("Sample Event Plug-in Implementer failed to add event listener status: %d\n", 
			status);
		return FALSE;
	}

	// check if server ready
	if (SMILIsDataManagerReady() == FALSE)
	{
		printf("Sample Event Plug-in Implementer failed to connect to Data Manager\nTry later...\n");
	}
	else
	{
		// ready so do work
		printf("Sample Event Plug-in Implementer connected to Data Manager\n");
	}
	
	return TRUE;
} // SamEPISuptStartEventMonitor


//---------------------------------------------------------------------
//  SamEPISuptStopEventMonitor
//---------------------------------------------------------------------
void SamEPISuptStopEventMonitor(void)
{
	SMILRemoveEventListener(SamSuptEventListener);
} // SamEPISuptStopEventMonitor


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------

