
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    DataEngine Team
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Event listener support
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// SDK include files
//---------------------------------------------------------------------
#include <dcossdk.h>


//---------------------------------------------------------------------
// Standard C include files
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Local includes
//---------------------------------------------------------------------
#include <dcdataobj.h>
#include <dcsgenapi.h>
#include <dcsupapi.h>


//---------------------------------------------------------------------
// Local defines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  SamConPrintOID
//---------------------------------------------------------------------
void SamConPrintOID(_IN const ObjID *pOID, 
					_IN const astring *pOIDMsg)
{
	astring *pOIDCreatorName;


	// get creator name
	pOIDCreatorName = SMILDOGetCreatorAliasNameByOID(pOID);

	// print information
	printf("%s%08X created by: %s\n", 
		(pOIDMsg != NULL) ? pOIDMsg:"", 
		pOID->ObjIDUnion.asu32, 
		(pOIDCreatorName != NULL) ? pOIDCreatorName:"");

	// free creator name
	SMILFreeGeneric(pOIDCreatorName);
} // SamConPrintOID


//---------------------------------------------------------------------
//  SamConPrintObjList
//---------------------------------------------------------------------
void SamConPrintObjList(_IN const ObjList *pOL)
{
	astring *pOIDCreatorName;
	u32 ii;


	// print information
	printf("objCount: " PRT_U32 "\n", 
		pOL->objCount);

	// go through all OIDS
	for (ii = 0; ii < pOL->objCount; ii++)
	{
		// get creator name
		pOIDCreatorName = SMILDOGetCreatorAliasNameByOID(&pOL->objID[ii]);

		// print information
		printf("objID[" PRT_U32 "]: %08X created by: %s\n", 
			ii, 
			pOL->objID[ii].ObjIDUnion.asu32, 
			(pOIDCreatorName != NULL) ? pOIDCreatorName:"");

		// free creator name
		SMILFreeGeneric(pOIDCreatorName);
	} // for (ii...
} // 


//---------------------------------------------------------------------
//  SamConPrintMemBuf
//---------------------------------------------------------------------
void SamConPrintMemBuf(_IN const void *pBuf, 
					   _IN u32 bufSize)
{
	astring *pABufStr;
	u32 size;


	if (bufSize == 0)
	{
		printf("None");
		return;
	}

	// allocate temporary buffer
	size = (((bufSize / 16) + 1) * 84);
	if ((pABufStr = (astring *)SMAllocMem(size)) == NULL)
	{
		return;
	}

	// convert buffer to string
	if ((SMXLTTypeValueToUTF8(pBuf, 
		bufSize, 
		pABufStr, 
		&size, 
		(SM_XLT_VTYPE_BINARY_TO_ASTRING | BINARY_HEXDUMP)) == SM_STATUS_SUCCESS) && 
		(size > 0))
	{
		printf("%s", 
			pABufStr);
	}

	// free temporary buffer
	SMFreeMem(pABufStr);
} // SamConPrintMemBuf


//---------------------------------------------------------------------
// GetEventSourceString
//---------------------------------------------------------------------
const astring *GetEventSourceString(_IN u16 evtType)
{

	if ((evtType >= DATA_EVENT_TYPE_BASE_LO) &&
		(evtType <= DATA_EVENT_TYPE_BASE_HI))
	{
		return "DataManager";
	}

	if ((evtType >= DATA_EVENT_TYPE_HIP_LO) &&
		(evtType <= DATA_EVENT_TYPE_HIP_HI))
	{
		return "InstSvcs";
	}

	if ((evtType >= DATA_EVENT_TYPE_STORAGE_SVCS_LO) &&
		(evtType <= DATA_EVENT_TYPE_STORAGE_SVCS_HI))
	{
		return "StorageSvcs";
	}

	return "";
} // GetEventSourceString


//---------------------------------------------------------------------
// SamConEventListener
// NOTE: When using event listeners you have to have _cdecl functions
// and compile the executable will multithreading libraries
//---------------------------------------------------------------------
s32 _cdecl SamConEventListener(_IN const DataEventHeader *pDEH)
{
	astring *pEvtTimeStampStr;
	void *pEventBody;
	DESingleParentCreateObj *pDESPCO;
	DEObjStatusChange *pDEOSC;
	u32 evtBodySize;


	// get the event time stamp as a string
	pEvtTimeStampStr = SMILDOGetObjTimeUTF8Str(pDEH->evtTimeStamp);

	printf("SamConEventListener: Recieved event\nType: %u Time: %s Flags: 0x%02X Source: %s\n", 
		pDEH->evtType, 
		pEvtTimeStampStr, 
		pDEH->evtFlags,
		GetEventSourceString(pDEH->evtType));

	// free the string
	SMILFreeGeneric(pEvtTimeStampStr);
	// catch GPFs
	pEvtTimeStampStr = NULL;

	// point to event body
	pEventBody = (void *)((u8 *)pDEH + 
		sizeof(DataEventHeader));

	// set size of body
	evtBodySize = (pDEH->evtSize - 
		sizeof(DataEventHeader));

	// determine event type
	switch (pDEH->evtType)
	{
		case DATA_EVENT_DATA_OBJ_DIRTY_LIST:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DATA_OBJ_DIRTY_LIST\n");
			
			// print obj list
			SamConPrintObjList((ObjList *)pEventBody);
		break;
			
		case DATA_EVENT_OBJ_STATUS_CHG:
			// point to data within event
			pDEOSC = (DEObjStatusChange *)pEventBody;
			
			// print event
			printf("Data Consumer recieved DATA_EVENT_OBJ_STATUS_CHG\n");
			
			// print information
			printf("prevObjStatus: %02X prevRedStatus: %02X\n", 
				pDEOSC->prevObjStatus, 
				pDEOSC->prevRedStatus);
			
			// print OID
			SamConPrintOID(&pDEOSC->objHeader.objID, 
				"objID: ");
		break;
			
		case DATA_EVENT_DATA_CONSUMER_ATTACHED:
			// print event
			printf("Data Consumer attached to Data Manager\nData Manager started\n");
		break;
			
		case DATA_EVENT_DATA_CONSUMER_DETACHED:
			// print event
			printf("Data Consumer detached from Data Manager\nData Manager stopped\n");
		break;
			
		case DATA_EVENT_DATA_OBJ_CREATE:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DATA_OBJ_CREATE\n");
			
			// print obj list
			SamConPrintObjList((ObjList *)pEventBody);
		break;
			
		case DATA_EVENT_DATA_OBJ_DESTROY:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DATA_OBJ_DESTROY\n");
			
			// print obj list
			SamConPrintObjList((ObjList *)pEventBody);
		break;
			
		case DATA_EVENT_DATA_OBJ_BRANCH_CREATE:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DATA_OBJ_BRANCH_CREATE\n");
			
			// print obj list
			SamConPrintObjList((ObjList *)pEventBody);
		break;
			
		case DATA_EVENT_DATA_OBJ_BRANCH_DESTROY:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DATA_OBJ_BRANCH_DESTROY\n");
			
			// print obj list
			SamConPrintObjList((ObjList *)pEventBody);
		break;
			
		case DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID:
			// point to OID within event
			pDESPCO = (DESingleParentCreateObj *)pEventBody;
			
			// print event
			printf("Data Consumer recieved obsolete event: DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID\n");
			
			// print OID
			SamConPrintOID(&pDESPCO->parentID, 
				"parentID: ");
			
			// print OID
			SamConPrintOID(&pDESPCO->objID, 
				"objID: ");
		break;
			
		case DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID:
			// print event
			printf("Data Consumer recieved obsolete event: DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID\n");
			
			// print OID
			SamConPrintOID((ObjID *)pEventBody, 
				"objID: ");
		break;

		case DATA_EVENT_DATA_MANAGER_ATTACHED:
			// print event
			printf("Data Manager attached event\n");
		break;

		case DATA_EVENT_DATA_MANAGER_DETACHED:
			// print event
			printf("Data Manager detached event\n");
		break;
			
		case DATA_EVENT_POWER_SUSPEND:
			// print event
			printf("Data Consumer recieved DATA_EVENT_POWER_SUSPEND\n");
		break;
			
		case DATA_EVENT_POWER_RESUME:
			// print event
			printf("Data Consumer recieved DATA_EVENT_POWER_RESUME\n");
		break;

		case DATA_EVENT_DEVICE_ADD:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DEVICE_ADD\n");
			
			// dump the event data in hex
			SamConPrintMemBuf(pEventBody, 
				evtBodySize);
		break;

		case DATA_EVENT_DEVICE_REMOVE:
			// print event
			printf("Data Consumer recieved DATA_EVENT_DEVICE_REMOVE\n");
			
			// dump the event data in hex
			SamConPrintMemBuf(pEventBody, 
				evtBodySize);
		break;
			
		default:
			// print event
			printf("Data Consumer recieved non-base event body: \n");
			
			// dump the event data in hex
			SamConPrintMemBuf(pEventBody, 
				evtBodySize);
		break;
	} // switch (pDEH->evtType)
	
	// add a line between events
	printf("\n");
	
	// continue sending event to 
	// next listner in the chain
	return SM_STATUS_SUCCESS;
} // SamConEventListener


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

