
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Store test
//
// Abstract/Purpose:
//
// This module contains routines to test
// dcsmil and dsm_sa_datamgr interfaces
//
// Environment:
//
// WINNT, Netware, Linux and OS2
//
// Created/Updated Version:
// $Date: 7/22/03 5:07p $ $Revision: 4 $
//
// Last Modified by:
// $Author: Tushar_oza $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Local headers
//--------------------------------------------------------------------
#include <samcon.h>



//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  SamConGetControllerNames
//---------------------------------------------------------------------
s32 SamConGetControllerNames(void)
{
	ObjList *pOLC;
	ObjList *pOLS;
	SAMPStorageDataObj *pSSDO;
	SAMPControllerDataObj *pSCDO;
	astring *pDeviceName;
	astring *pVendorName;
	ObjID tempOID;
	u32 ii;
	u32 jj;


	tempOID.ObjIDUnion.asu32 = DATA_OBJ_OID_ROOT;
	pOLC = SMILListChildOIDByType(&tempOID, 
		DATA_OBJ_TYPE_SAMP_CONTROLLER);

	// failed?
	if (pOLC == NULL)
	{
		return SM_STATUS_NO_SUCH_OBJECT;
	}

	// get all objects
	for (ii = 0; ii < pOLC->objCount; ii++)
	{
		// get the object
		pSCDO = (SAMPControllerDataObj *)SMILGetObjByOID(&pOLC->objID[ii]);

		// failed?
		if (pSCDO == NULL)
		{
			printf("SamConGetControllerNames: failed to find object with OID: %08X\n", 
				pOLC->objID[ii].ObjIDUnion.asu32);
			// skip this
			continue;
		}

		// display the controller name and vendor
		pDeviceName = SMILDOGetByOffsetUTF8Str(&pSCDO->doh, 
			pSCDO->offsetDeviceName);

		pVendorName = SMILDOGetByOffsetUTF8Str(&pSCDO->doh, 
			pSCDO->offsetVendorName);

		printf("Controller Device:  %s Vendor: %s\n", 
			pDeviceName, 
			pVendorName);

		// free the strings
		SMILFreeGeneric(pVendorName);
		SMILFreeGeneric(pDeviceName);

		// free the object
		SMILFreeGeneric(pSCDO);

		// get the storage object version to which they belong
		tempOID = pOLC->objID[ii];
		pOLS = SMILListParentOIDByType(&tempOID, 
			DATA_OBJ_TYPE_SAMP_STORAGE);

		// failed?
		if (pOLS == NULL)
		{
			// skip this
			continue;
		}

		// get all objects
		for (jj = 0; jj < pOLS->objCount; jj++)
		{
			// get the object
			pSSDO = (SAMPStorageDataObj *)SMILGetObjByOID(&pOLS->objID[jj]);

			// failed?
			if (pSSDO == NULL)
			{
				printf("SamConGetStorageObjVersion: failed to find object with OID: %08X\n", 
					pOLS->objID[jj].ObjIDUnion.asu32);
				// skip this
				continue;
			}

			// display the version
			printf("SamConGetControllerNames: Storage Object Version: %u.%u.%u\n", 
				pSSDO->majorVersion, 
				pSSDO->minorVersion, 
				pSSDO->maintanenceVersion);

			// free the object
			SMILFreeGeneric(pSSDO);
		} // for (ii...

		// free the list
		SMILFreeGeneric(pOLS);
	} // for (ii...

	// free the list
	SMILFreeGeneric(pOLC);

	return SM_STATUS_SUCCESS;
} // SamConGetControllerNames


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------






