
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Store test
//
// Abstract/Purpose:
//
// This module contains routines to test
// dcsmil and dsm_sa_datamgr interfaces
//
// Environment:
//
// WINNT, Netware, Linux and OS2
//
// Created/Updated Version:
// $Date: 7/22/03 5:07p $ $Revision: 9 $
//
// Last Modified by:
// $Author: Tushar_oza $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Local headers
//--------------------------------------------------------------------
#include <samcon.h>
#include <dcsupapi.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// Event handle for termination signal
void *pGTerminateHandle = NULL;


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  ModSignalHandler
//---------------------------------------------------------------------
void ModSignalHandler(int sigval)
{

	// signal blocked threads for termination
	SMEventSet(pGTerminateHandle);

	// main has been shutdown by OS
	UNREFERENCED_PARAMETER(sigval);
} // ModSignalHandler


//---------------------------------------------------------------------
//  ModuleAttach
//---------------------------------------------------------------------
booln ModuleAttach(void)
{
	s32 status;


	// create event handle for termination signal
	if ((pGTerminateHandle = SMEventCreate(NULL, 
		FALSE, 
		FALSE)) == NULL)
	{
		printf("Sample Consumer failed to allocate event resource\n");

		// just leave
		return FALSE;
	}

	// install signal handlers
	signal(SIGINT, 
		ModSignalHandler);
	signal(SIGTERM, 
		ModSignalHandler);

	printf("Press Ctrl-C to exit or unload to terminate...\n");

	printf("Sample Consumer connecting to Data Manager...\r");

	// insert event listener
	status = SMILAddEventListener(SamConEventListener);

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		printf("Sample Consumer failed to add event listener status: %d\n", 
			status);

		return FALSE;
	}

	// check if server ready
	if (SMILIsDataManagerReady() == FALSE)
	{
		SMILRemoveEventListener(SamConEventListener);

		printf("Sample Consumer failed to connect to Data Manager\nTry later.\n");

		return FALSE;
	}

	// ready so do work
	printf("Sample Consumer connected to Data Manager...   \n");

	return TRUE;
} // ModuleAttach


//---------------------------------------------------------------------
//  ModuleDetach
//---------------------------------------------------------------------
void ModuleDetach(void)
{

	// wait until termination signal arrives
	SMEventWait(pGTerminateHandle, 
		SM_WAIT_MSECS_INFINITE);

	// ok sastst is done
	SMILRemoveEventListener(SamConEventListener);

	// release event
	SMEventDestroy(pGTerminateHandle);
} // ModuleDetach


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------






