
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
//
// Abstract/Purpose:
//
// Header file for popdpdmdispatch.c support module
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 4 $
//
//
//  Last Modified by/on:
//
//--------------------------------------------------------------------


#ifndef __POPPRIVATEDATA_H_INCLUDED__
#define __POPPRIVATEDATA_H_INCLUDED__



//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// $(DATAENGINESDK) include files
//--------------------------------------------------------------------
#include <dcsysdebug.h>
#include <dcsupapi.h>
#include <dcdataobj.h>
#include <popdatasync.h>



#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Function prototypes 
//---------------------------------------------------------------------

// define the function pointer to notify callback function
typedef void (_cdecl *PFNNOTIFYPOPPRIVDATA)(
	_IN const ObjID *pOID, 
	_IN void *pPrivateData, 
	_IN void *pNotifyData
	);

	
//---------------------------------------------------------------------
// PopPrivateDataAttach
// call this function to initialize this module typically on PopDispLoad
// If initialNumElements set to 0 then default initial table elements
// will be allocated
// Call this function before using any PopPrivateDataXXX functions
// NOTE: Call PopDataSyncAttach and PopDPDMDAttach 
// prior to calling this attach function
//---------------------------------------------------------------------
s32 PopPrivateDataAttach(_IN u8 popID, 
						 _IN u32 initialNumElements, 
						 _IN PFNNOTIFYPOPPRIVDATA pfnNotifyInsert, 
						 _IN PFNNOTIFYPOPPRIVDATA pfnNotifyDelete);


//---------------------------------------------------------------------
// PopPrivateDataDetach
// call this function after finshed using the popprivate data functions
// typically called on PopDispUnload
//---------------------------------------------------------------------
s32 PopPrivateDataDetach(_IN void *pNotifyDataDelete);


//---------------------------------------------------------------------
// PopPrivateDataInsert
// If allocateOID is set to FALSE this function will associate the 
// private data provided to the given OID (input) in a OID sorted table 
// If allocateOID is set to TRUE this function will generate an OID
// (output) and then associate the private data in a OID sorted table
// It will also call a insert notification callback if specified
//---------------------------------------------------------------------
s32 PopPrivateDataInsert(_INOUT ObjID *pOID, 
						 _IN void *pPrivateData, 
						 _IN void *pNotifyDataInsert, 
						 _IN booln allocateOID);

//---------------------------------------------------------------------
// PopPrivateDataGet
// This function will return the private data given an OID 
// assigned to the data when inserted
//---------------------------------------------------------------------
void *PopPrivateDataGet(_IN const ObjID *pOID);


//---------------------------------------------------------------------
// PopPrivateDataDelete
// This function will delete private data element associated with
// given an OID
// It will also call a delete notification callback if specified
//---------------------------------------------------------------------
s32 PopPrivateDataDelete(_IN const ObjID *pOID, 
						 _IN void *pNotifyDataDelete);




#ifdef __cplusplus
}
#endif

#endif //  __POPPRIVATEDATA_H_INCLUDED__ 


