
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 2 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 8/12/03 4:04p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popdatasync.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------
void *p_gPopDataSyncLock = NULL;


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopDataSyncReadLock
//---------------------------------------------------------------------
void PopDataSyncReadLock(void)
{
	// lock the reader mutex
	SMMutexLock(p_gPopDataSyncLock, 
		SM_WAIT_MSECS_INFINITE);
} // PopDataSyncReadLock


//---------------------------------------------------------------------
// PopDataSyncReadUnLock
//---------------------------------------------------------------------
void PopDataSyncReadUnLock(void)
{
	// unlock the reader mutex
	SMMutexUnLock(p_gPopDataSyncLock);
} // PopDataSyncReadUnLock


//---------------------------------------------------------------------
// PopDataSyncWriteLock
//---------------------------------------------------------------------
void PopDataSyncWriteLock(void)
{
	// lock the write mutex
	SMMutexLock(p_gPopDataSyncLock, 
		SM_WAIT_MSECS_INFINITE);
} // PopDataSyncWriteLock


//---------------------------------------------------------------------
// PopDataSyncWriteUnLock
//---------------------------------------------------------------------
void PopDataSyncWriteUnLock(void)
{
	// unlock the write mutex
	SMMutexUnLock(p_gPopDataSyncLock);
} // PopDataSyncWriteUnLock


//---------------------------------------------------------------------
// PopDataSyncAttach
//---------------------------------------------------------------------
s32 PopDataSyncAttach(void)
{
	s32 status = SM_STATUS_SUCCESS;


	// allocate reader lock
	p_gPopDataSyncLock = SMMutexCreate(NULL);

	// failed?
	if (NULL == p_gPopDataSyncLock)
	{
		SysDbgMsg3(("PopDataSyncAttach: failed to allocate p_gPopDataSyncLock\n"));
		status = SM_STATUS_OUT_OF_MEMORY;
		_throw error1;
	} // if (NULL == p_gPopDataSyncLock)

	// success return

error1:
	return status;
} // PopDataSyncAttach


//---------------------------------------------------------------------
// PopDataSyncDetach
//---------------------------------------------------------------------
void PopDataSyncDetach(void)
{
	void *pLock = NULL;


	// lock the context
	PopDataSyncWriteLock();

	// clear reader lock before free
	pLock = p_gPopDataSyncLock;
	// catch GPFs
	p_gPopDataSyncLock = NULL;
	
	// free global lock
	SMMutexDestroy(pLock);
} // PopDataSyncDetach


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------

