
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 19 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popdpdmdispatch.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

const SMDPDMDispatchEnv *pPopDPDMDE = NULL;


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopDPDMDAttach
//---------------------------------------------------------------------
booln PopDPDMDAttach(_IN const SMDPDMDispatchEnv *pDPDMDE)
{

	SysDbgMsg4(("PopDPDMDAttach: entry\n"));

	// setup the data populator to data manager push interface
	pPopDPDMDE = pDPDMDE;

	SysDbgMsg4(("PopDPDMDAttach: exit\n"));

	return TRUE;
} // PopDPDMDAttach


//---------------------------------------------------------------------
// PopDPDMDDetach
//---------------------------------------------------------------------
void PopDPDMDDetach(void)
{

	SysDbgMsg4(("PopDPDMDDetach: entry\n"));

	// clear the push interface
	pPopDPDMDE = NULL;

	SysDbgMsg4(("PopDPDMDDetach: exit\n"));
} // PopDPDMDDetach


//---------------------------------------------------------------------
// PopDPDMDGetDPDMDE
//---------------------------------------------------------------------
const SMDPDMDispatchEnv *PopDPDMDGetDPDMDE(void)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// clear the push interface
	pDPDMDE = pPopDPDMDE;


	SysDbg3(
		// check for uninitialized state
		if (NULL == pDPDMDE)
		{
			SysDbgMsg3(("PopDPDMDGetDPDMDE: invalid pDPDMDE == NULL\n"));
		}
	)

	return pDPDMDE;
} // PopDPDMDGetDPDMDE


//---------------------------------------------------------------------
// PopDPDMDFreeGeneric
//---------------------------------------------------------------------
void PopDPDMDFreeGeneric(_IN void *pMem)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return;
	}

	pDPDMDE->pDPDMDTable->FreeGeneric(pMem);
} // PopDPDMDFreeGeneric


//---------------------------------------------------------------------
// PopDPDMDIsDataManagerReady
//---------------------------------------------------------------------
booln PopDPDMDIsDataManagerReady(void)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return FALSE;
	}

	return pDPDMDE->pDPDMDTable->IsDataManagerReady();
} // PopDPDMDIsDataManagerReady


//---------------------------------------------------------------------
// PopDPDMDGetPopulatorID
//---------------------------------------------------------------------
u8 PopDPDMDGetPopulatorID(void)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return DATA_CREATOR_ID_PUBLIC;
	}

	// get the populator id
	return pDPDMDE->pDPDMDTable->GetPopulatorID(pDPDMDE);
} // PopDPDMDGetPopulatorID


//---------------------------------------------------------------------
// PopDPDMDAllocSMReq
//---------------------------------------------------------------------
void *PopDPDMDAllocSMReq(_OUT u32 *pReqSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocSMReq(pReqSize);
} // PopDPDMDAllocSMReq


//---------------------------------------------------------------------
// PopDPDMDAllocSMRsp
//---------------------------------------------------------------------
void *PopDPDMDAllocSMRsp(_OUT u32 *pRspSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocSMRsp(pRspSize);
} // PopDPDMDAllocSMRsp


//---------------------------------------------------------------------
// PopDPDMDAllocDataObject
//---------------------------------------------------------------------
DataObjHeader *PopDPDMDAllocDataObject(_OUT u32 *pDataObjSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocDataObject(pDataObjSize);
} // PopDPDMDAllocDataObject


//---------------------------------------------------------------------
// PopDPDMDAllocDataEvent
//---------------------------------------------------------------------
DataEventHeader *PopDPDMDAllocDataEvent(_OUT u32 *pDataEvtSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocDataEvent(pDataEvtSize);
} // PopDPDMDAllocDataEvent


//---------------------------------------------------------------------
// PopDPDMDAllocAndCopyRequiredObjList
//---------------------------------------------------------------------
ObjList *PopDPDMDAllocAndCopyRequiredObjList(_IN const ObjList *pMOL, 
											 _OUT u32 *pReqListSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocAndCopyRequiredObjList(pMOL, 
		pReqListSize);
} // PopDPDMDAllocAndCopyRequiredObjList


//---------------------------------------------------------------------
// PopDPDMDAllocAndCopyRequiredDataObj
//---------------------------------------------------------------------
DataObjHeader *PopDPDMDAllocAndCopyRequiredDataObj(_IN const DataObjHeader *pMOH)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocAndCopyRequiredDataObj(pMOH);
} // PopDPDMDAllocAndCopyRequiredDataObj


//---------------------------------------------------------------------
// PopDPDMDAllocAndCopyRequiredDataEvt
//---------------------------------------------------------------------
DataEventHeader *PopDPDMDAllocAndCopyRequiredDataEvt(_IN const DataEventHeader *pMEH)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->AllocAndCopyRequiredDataEvt(pMEH);
} // PopDPDMDAllocAndCopyRequiredDataEvt


//---------------------------------------------------------------------
// PopDPDMDListParentOID
//---------------------------------------------------------------------
ObjList *PopDPDMDListParentOID(_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->ListParentOID(pOID);
} // PopDPDMDListParentOID

	
//---------------------------------------------------------------------
// PopDPDMDListChildOID
//---------------------------------------------------------------------
ObjList *PopDPDMDListChildOID(_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->ListChildOID(pOID);
} // PopDPDMDListChildOID


//---------------------------------------------------------------------
// PopDPDMDListParentOIDByType
//---------------------------------------------------------------------
ObjList *PopDPDMDListParentOIDByType(_IN const ObjID *pOID, 
									 _IN u16 objType)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->ListParentOIDByType(pOID, 
		objType);
} // PopDPDMDListParentOIDByType


//---------------------------------------------------------------------
// PopDPDMDListChildOIDByType
//---------------------------------------------------------------------
ObjList *PopDPDMDListChildOIDByType(_IN const ObjID *pOID, 
									_IN u16 objType)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->ListChildOIDByType(pOID, 
		objType);
} // PopDPDMDListChildOIDByType


//---------------------------------------------------------------------
// PopDPDMDListChildOIDByStatus
//---------------------------------------------------------------------
ObjList *PopDPDMDListChildOIDByStatus(_IN const ObjID *pOID, 
									  _IN u16 objType, 
									  _IN booln worseThan, 
									  _IN u8 objStatus)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->ListChildOIDByStatus(pOID, 
		objType,
		worseThan, 
		objStatus);
} // PopDPDMDListChildOIDByStatus


//---------------------------------------------------------------------
// PopDPDMDGetDataObjByOID
//---------------------------------------------------------------------
DataObjHeader *PopDPDMDGetDataObjByOID(_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->GetDataObjByOID(pOID);
} // PopDPDMDGetDataObjByOID


//---------------------------------------------------------------------
// PopDPDMDPassThruObjByReq
//---------------------------------------------------------------------
s32 PopDPDMDPassThruObjByReq(_IN const SMReqHeaderPassThru *pOPTReq, 
							 _IN u32 reqSize, 
							 _OUT void *pRsp, 
							 _IN u32 rspSize, 
							 _OUT u32 *pBytesReturned)
{
	const SMDPDMDispatchEnv *pDPDMDE;
	
	
	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();
	
	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}
	
	return pDPDMDE->pDPDMDTable->PassThruObjByReq(pOPTReq, 
		reqSize, 
		pRsp, 
		rspSize, 
		pBytesReturned);
} // PopDPDMDPassThruObjByReq


//---------------------------------------------------------------------
// PopDPDMDSetObjByReq
//---------------------------------------------------------------------
s32 PopDPDMDSetObjByReq(_IN const SMReqHeaderSet *pSReq,
						_IN u32 reqSize)
{
	const SMDPDMDispatchEnv *pDPDMDE;
	
	
	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();
	
	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}
	
	return pDPDMDE->pDPDMDTable->SetObjByReq(pSReq,
		reqSize);
} // PopDPDMDSetObjByReq


//---------------------------------------------------------------------
// PopDPDMDDataObjCreateSingle
//---------------------------------------------------------------------
s32 PopDPDMDDataObjCreateSingle(_IN const DataObjHeader *pDOH, 
								_IN const ObjID *pParentOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjCreateSingle(pDPDMDE, 
		pDOH, 
		pParentOID);
} // PopDPDMDDataObjCreateSingle


//---------------------------------------------------------------------
// PopDPDMDDataObjCreateMultiple
//---------------------------------------------------------------------
s32 PopDPDMDDataObjCreateMultiple(_IN const DataObjHeader **ppDOH,
								  _IN const u32 *pNumInDOHList,
								  _IN const ObjList *pParentOL)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjCreateMultiple(pDPDMDE, 
		ppDOH,
		pNumInDOHList,
		pParentOL);
} // PopDPDMDDataObjCreateMultiple


//---------------------------------------------------------------------
// PopDPDMDDataObjRefreshSingle
//---------------------------------------------------------------------
s32 PopDPDMDDataObjRefreshSingle(_IN const DataObjHeader *pDOH)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjRefreshSingle(pDPDMDE, 
		pDOH);
} // PopDPDMDDataObjRefreshSingle


//---------------------------------------------------------------------
// PopDPDMDDataObjRefreshMultiple
//---------------------------------------------------------------------
s32 PopDPDMDDataObjRefreshMultiple(_IN const DataObjHeader **ppDOH, 
								   _IN const u32 *pNumInDOHList)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjRefreshMultiple(pDPDMDE, 
		ppDOH,
		pNumInDOHList);
} // PopDPDMDDataObjRefreshMultiple


//---------------------------------------------------------------------
// PopDPDMDDataObjDestroySingle
//---------------------------------------------------------------------
s32 PopDPDMDDataObjDestroySingle(_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjDestroySingle(pDPDMDE, 
		pOID);
} // PopDPDMDDataObjDestroySingle


//---------------------------------------------------------------------
// PopDPDMDDataObjDestroyMultiple
//---------------------------------------------------------------------
s32 PopDPDMDDataObjDestroyMultiple(_IN const ObjList *pOL)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DataObjDestroyMultiple(pDPDMDE, 
		pOL);
} // PopDPDMDDataObjDestroyMultiple


//---------------------------------------------------------------------
// PopDPDMDDOBranchCreateMultiple
//---------------------------------------------------------------------
s32 PopDPDMDDOBranchCreateMultiple(_IN const ObjList *pOL, 
								   _IN const s32 *pListType, 
								   _IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// create multiple branch
	return pDPDMDE->pDPDMDTable->DOBranchCreateMultiple(pDPDMDE, 
		pOL,
		pListType,
		pOID);
} // PopDPDMDDOBranchCreateMultiple


//---------------------------------------------------------------------
// PopDPDMDDOBranchDestroyMultiple
//---------------------------------------------------------------------
s32 PopDPDMDDOBranchDestroyMultiple(_IN const ObjList *pOL, 
									_IN const s32 *pListType, 
									_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// destroy multiple branch
	return pDPDMDE->pDPDMDTable->DOBranchDestroyMultiple(pDPDMDE, 
		pOL,
		pListType,
		pOID);
} // PopDPDMDDOBranchDestroyMultiple


//---------------------------------------------------------------------
// PopDPDMDDESubmitSingle
//---------------------------------------------------------------------
s32 PopDPDMDDESubmitSingle(_INOUT DataEventHeader *pDEH)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DESubmitSingle(pDPDMDE, 
		pDEH);
} // PopDPDMDDESubmitSingle


//---------------------------------------------------------------------
// PopDPDMDDOGetObjTimeUTF8Str
//---------------------------------------------------------------------
astring *PopDPDMDDOGetObjTimeUTF8Str(_IN s64 timeStampVal)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DOGetObjTimeUTF8Str(timeStampVal);
} // PopDPDMDDOGetObjTimeUTF8Str


//---------------------------------------------------------------------
// PopDPDMDObjListClear
//---------------------------------------------------------------------
s32 PopDPDMDObjListClear(_OUT ObjList *pOL, 
						 _IN const u32 *pOLSize, 
						 _OUT u32 *pBytesReturned)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DOObjListClear(pOL, 
		pOLSize, 
		pBytesReturned);
} // PopDPDMDObjListClear


//---------------------------------------------------------------------
// PopDPDMDObjListAppendOID
//---------------------------------------------------------------------
s32 PopDPDMDObjListAppendOID(_IN const ObjID *pOIDToAppend, 
							 _INOUT ObjList *pOL, 
							 _IN const u32 *pOLSize, 
							 _OUT u32 *pBytesReturned)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DOObjListAppendOID(pOIDToAppend, 
		pOL, 
		pOLSize, 
		pBytesReturned);
} // PopDPDMDObjListAppendOID


//---------------------------------------------------------------------
// PopDPDMDDOAppendUTF8Str
//---------------------------------------------------------------------
s32 PopDPDMDDOAppendUTF8Str(_INOUT DataObjHeader *pDOH, 
							_IN const u32 *pDOHBufSize, 
							_INOUT u32 *pOffsetStrVar, 
							_IN const astring *pUTF8Str)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DOAppendUTF8Str(pDOH, 
			pDOHBufSize, 
			pOffsetStrVar, 
			pUTF8Str);
} // PopDPDMDDOAppendUTF8Str


//---------------------------------------------------------------------
// PopDPDMDDOAppendUCS2Str
//---------------------------------------------------------------------
s32 PopDPDMDDOAppendUCS2Str(_INOUT DataObjHeader *pDOH, 
							_IN const u32 *pDOHBufSize, 
							_INOUT u32 *pOffsetStrVar, 
							_IN const ustring *pUCS2Str)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return SM_STATUS_DEVICE_NOT_READY;
	}

	// append the string
	return pDPDMDE->pDPDMDTable->DOAppendUCS2Str(pDOH, 
			pDOHBufSize, 
			pOffsetStrVar, 
			pUCS2Str);
} // PopDPDMDDOAppendUCS2Str


//---------------------------------------------------------------------
// PopDPDMDDOGetCreatorAliasNameByOID
//---------------------------------------------------------------------
astring *PopDPDMDDOGetCreatorAliasNameByOID(_IN const ObjID *pOID)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->DOGetCreatorAliasNameByOID(pOID);
} // PopDPDMDDOGetCreatorAliasNameByOID


//---------------------------------------------------------------------
// PopDPDMDDOGetObjBody
//---------------------------------------------------------------------
const void *PopDPDMDDOGetObjBody(_IN const DataObjHeader *pDOH, 
								 _IN u32 *pBodySize)
{
	const SMDPDMDispatchEnv *pDPDMDE;


	// get push environment
	pDPDMDE = PopDPDMDGetDPDMDE();

	// available?
	if (NULL == pDPDMDE)
	{
		return NULL;
	}

	return pDPDMDE->pDPDMDTable->DOGetObjBody(pDOH, 
		pBodySize);
} // PopDPDMDDOGetObjBody


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------





