
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 3 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>
#include <popexport.h>


//---------------------------------------------------------------------
// Export definitions
// NOTE: In modules that export functions, SMINTF must be set up
// for exporting after all other header files are included
//---------------------------------------------------------------------
#define __EXPORTING
#undef SMINTF
#include <dcdetypes.h>


//--------------------------------------------------------------------
// Local Defines 
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Function prototypes for exports from populators
//---------------------------------------------------------------------

// make sure defines are in place
#ifndef POPEXPORT_DISPATCHFN
#error POPEXPORT_DISPATCHFN not defined. Example: POPEXPORT_DISPATCHFN=SAMPOP32_SMPopulatorDispatch
#error NOTE: Populator name should be in uppercase followed by word size. Example: DCSPOP32
#endif // POPEXPORT_DISPATCHFN

// determine the debug file to be
// used by this populator
#ifndef POPEXPORT_SYSDBG_FILE
// use default file name
#define POPEXPORT_SYSDBG_FILE		DEFAULT_SYSDBG_FILENAME	
#endif // POPEXPORT_SYSDBG_FILE


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// global code allocator for SysDbg
SYS_DBG_DECLARE_GLOBAL_CODEDATA;

// original module context
SMECInfo popexportOrigECI;


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  ModuleAttach
//---------------------------------------------------------------------
booln __lxinit__ ModuleAttach(void)
{

	// get original export context info
	SMGetExportContext(&popexportOrigECI);

    // first time init for SysDbg
	// send debug trace to console
	// and the file specified
    SysDbgInit((SYSTEM_DEBUG_SCREEN | SYSTEM_DEBUG_FILE), 
		POPEXPORT_SYSDBG_FILE);

	// inform the populator the intent to load
	// into process or OS memory
	if (PopulatorAttach() != SM_STATUS_SUCCESS)
	{
		return FALSE;
	}

	return TRUE;
} // ModuleAttach


//---------------------------------------------------------------------
//  ModuleDetach
//---------------------------------------------------------------------
booln __lxfini__ ModuleDetach(void)
{

	// inform the populator the intent to unload
	// from process or OS memory
	PopulatorDetach();

	return TRUE;
} // ModuleDetach


//---------------------------------------------------------------------
//  MODULEXX_SMPopulatorDispatch
//---------------------------------------------------------------------
SMINTF s32 _cdecl POPEXPORT_DISPATCHFN(_IN u32 clientID,
									   _IN u32 reqType,
									   _IN const void *pInBuf,
									   _IN u32 inBufSize,
									   _OUT void *pOutBuf,
									   _IN u32 outBufSize,
									   _OUT u32 *pBytesReturned)
{
    SMECInfo eci;
	s32 retStatus;


    SMSetExportContext(&eci, 
		&popexportOrigECI);

	retStatus = PopulatorDispatch(clientID, 
		reqType, 
		pInBuf, 
		inBufSize, 
		pOutBuf, 
		outBufSize, 
		pBytesReturned);

    SMResetExportContext(&eci);

    return retStatus;
} // MODULEXX_SMPopulatorDispatch


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------
