
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2009 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Populator INI read support
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popiniread.h>


//---------------------------------------------------------------------
// Local Defines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopINIFreeGeneric - frees non constant memory returned by PopINI
// routines
//---------------------------------------------------------------------
void PopINIFreeGeneric(_IN void *pMem)
{
	SMFreeGeneric(pMem);
} // PopINIFreeGeneric


//---------------------------------------------------------------------
// PopINIGetINIPathFileName
// constructs a standard path to a INI filename based on the Autry
// productID (SM_PRODUCT_ID_XXX)
//---------------------------------------------------------------------
astring *PopINIGetINIPathFileName(_IN s32 productID, 
								  _IN const astring *pFileName)
{
	// prepare and return path to ini
	return SMMakePathFileNameByPIDAndType(productID, 
		SM_PATH_TYPE_INI,
		SM_PSUBPATH_INI, 
		pFileName);
} // PopINIGetINIPathFileName


//---------------------------------------------------------------------
// PopINIGetKeyValueMultiUTF8 - Called by populator to get a NSV 
// list of values assigned to an INI key of type UTF8 (astring)
// which will be in the CSV format
// NOTE: Use PopINIFreeGeneric to free returned memory
//---------------------------------------------------------------------
astring *PopINIGetKeyValueMultiUTF8(_IN const astring *pINIPFName, 
									_IN const astring *pSecName, 
									_IN const astring *pKeyName, 
									_IN const astring *pDefaultVal, 
									_OUT u32 *pValSize)
{
	astring *pTBuf1;
	astring *pRetVal = NULL;


	// read value from INI
	pTBuf1 = SMReadINISectionKeyUTF8Value(pSecName,
		pKeyName,
		pDefaultVal,
		pValSize,
		pINIPFName,
		TRUE);

	// passed?
	if (pTBuf1 != NULL)
	{
		// convert the value CSV to NSV
		pRetVal = SMUTF8ConvertXSVToYSV(pTBuf1, 
			SM_VALSEP_COMMA, 
			SM_VALSEP_NULL, 
			NULL,
			pValSize);

		// free temporary memory
		SMFreeGeneric(pTBuf1);
		// prevent GPFs
		pTBuf1 = NULL;
	} // if (pTBuf1 != NULL)

	// done
	return pRetVal;
} // PopINIGetKeyValueMultiUTF8


//---------------------------------------------------------------------
// PopINIGetKeyValueUTF8 - Called when populator to get a key 
// of type UTF8 (astring). This routine can also be used to list
// sections (if pSecName and pKeyName is NULL) and 
// keys if (pKeyName is NULL)
// NOTE: Use PopINIFreeGeneric to free returned memory
//---------------------------------------------------------------------
astring *PopINIGetKeyValueUTF8(_IN const astring *pINIPFName, 
							   _IN const astring *pSecName, 
							   _IN const astring *pKeyName, 
							   _IN const astring *pDefaultVal, 
							   _OUT u32 *pValSize)
{
	// read value from INI
	return SMReadINISectionKeyUTF8Value(pSecName, 
		pKeyName, 
		pDefaultVal, 
		pValSize, 
		pINIPFName, 
		TRUE);
} // PopINIGetKeyValueUTF8


//---------------------------------------------------------------------
//  PopINIGetKeyValueUnSigned32
//---------------------------------------------------------------------
u32 PopINIGetKeyValueUnSigned32(_IN const astring *pINIPFName, 
								_IN const astring *pSecName, 
								_IN const astring *pKeyName, 
								_IN u32 defaultVal)
{
	u32 retVal;
	u32 tsize1;


	// read key value
	// NOTE: on failure the default value
	// will be stored into returned value
	tsize1 = sizeof(retVal);
	SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_UNSIGNED32_LITTLE_ENDIAN, 
		&retVal, 
		&tsize1, 
		&defaultVal, 
		sizeof(defaultVal), 
		pINIPFName, 
		TRUE);

	// return data
	return retVal;
} // PopINIGetKeyValueUnSigned32


//---------------------------------------------------------------------
//  PopINIGetKeyValueSigned32
//---------------------------------------------------------------------
s32 PopINIGetKeyValueSigned32(_IN const astring *pINIPFName, 
							  _IN const astring *pSecName, 
							  _IN const astring *pKeyName, 
							  _IN s32 defaultVal)
{
	s32 retVal;
	u32 tsize1;


	// read key value
	// NOTE: on failure the default value
	// will be stored into returned value
	tsize1 = sizeof(retVal);
	SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_SIGNED32_LITTLE_ENDIAN, 
		&retVal, 
		&tsize1, 
		&defaultVal, 
		sizeof(defaultVal), 
		pINIPFName, 
		TRUE);

	// return data
	return retVal;
} // PopINIGetKeyValueSigned32


//---------------------------------------------------------------------
//  PopINIGetKeyValueUnSigned64
//---------------------------------------------------------------------
u64 PopINIGetKeyValueUnSigned64(_IN const astring *pINIPFName, 
								_IN const astring *pSecName, 
								_IN const astring *pKeyName, 
								_IN u64 defaultVal)
{
	u64 retVal;
	u32 tsize1;


	// read key value
	// NOTE: on failure the default value
	// will be stored into returned value
	tsize1 = sizeof(retVal);
	SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_UNSIGNED64_LITTLE_ENDIAN, 
		&retVal, 
		&tsize1, 
		&defaultVal, 
		sizeof(defaultVal), 
		pINIPFName, 
		TRUE);

	// return data
	return retVal;
} // PopINIGetKeyValueUnSigned64


//---------------------------------------------------------------------
//  PopINIGetKeyValueSigned64
//---------------------------------------------------------------------
s64 PopINIGetKeyValueSigned64(_IN const astring *pINIPFName, 
							  _IN const astring *pSecName, 
							  _IN const astring *pKeyName, 
							  _IN s64 defaultVal)
{
	s64 retVal;
	u32 tsize1;


	// read key value
	// NOTE: on failure the default value
	// will be stored into returned value
	tsize1 = sizeof(retVal);
	SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_SIGNED64_LITTLE_ENDIAN, 
		&retVal, 
		&tsize1, 
		&defaultVal, 
		sizeof(defaultVal), 
		pINIPFName, 
		TRUE);

	// return data
	return retVal;
} // PopINIGetKeyValueSigned64


//---------------------------------------------------------------------
//  PopINIGetKeyValueBooln
//---------------------------------------------------------------------
booln PopINIGetKeyValueBooln(_IN const astring *pINIPFName, 
							 _IN const astring *pSecName, 
							 _IN const astring *pKeyName, 
							 _IN booln defaultVal)
{
	booln retVal;
	u32 tsize1;


	// read key value
	// NOTE: on failure the default value
	// will be stored into returned value
	tsize1 = sizeof(retVal);
	SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_BOOLN, 
		&retVal, 
		&tsize1, 
		&defaultVal, 
		sizeof(defaultVal), 
		pINIPFName, 
		TRUE);

	// return data
	return retVal;
} // PopINIGetKeyValueBooln


//---------------------------------------------------------------------
//  PopINIGetKeyValueEnum
//---------------------------------------------------------------------
s32 PopINIGetKeyValueEnum(_IN const astring *pINIPFName, 
						  _IN const astring *pSecName, 
						  _IN const astring *pKeyName, 
						  _IN const SMEnums32UTF8Map *pEnumMap, 
						  _IN s32 enumMapCount)
{
	// read key value and return data
	return SMReadINIEnums32Value(pSecName, 
		pKeyName, 
		pEnumMap, 
		enumMapCount, 
		FALSE, 
		pINIPFName, 
		TRUE);
} // PopINIGetKeyValueEnum


//---------------------------------------------------------------------
//  PopINIGetKeyValueBitMap
//---------------------------------------------------------------------
u32 PopINIGetKeyValueBitMap(_IN const astring *pINIPFName, 
							_IN const astring *pSecName, 
							_IN const astring *pKeyName, 
							_IN const SMEnums32UTF8Map *pEnumMap, 
							_IN s32 enumMapCount)
{
	// read key value and return data
	return SMReadINIEnums32Value(pSecName, 
		pKeyName, 
		pEnumMap, 
		enumMapCount, 
		TRUE, 
		pINIPFName, 
		TRUE);
} // PopINIGetKeyValueBitMap


//---------------------------------------------------------------------
// PopINIGetKeyValueBinary - Called when populator to get a key 
// of type binary u8*).  need to alloc own memory for pValue
//---------------------------------------------------------------------
s32 PopINIGetKeyValueBinary(_IN const astring *pINIPFName, 
						   _IN const astring *pSecName, 
						   _IN const astring *pKeyName, 
						   _OUT u8 *pValue,
						   _OUT u32 *pValSize)
{
	s32 status;

	// read value from INI
	status = SMReadINIPathFileValue(pSecName, 
		pKeyName, 
		INI_BINARY,
		pValue,
		pValSize,
		NULL,
		0,
		pINIPFName, 
		TRUE);

	return status;
} // PopINIGetKeyValueBinary


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

