
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 5 $
//
//
//  Last Modified by/on:
//  $Author: Anil_rao $ / $Date: 8/12/03 11:17a $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <popiniwrite.h>



//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------



//---------------------------------------------------------------------
// PopINISetKeyValueUTF8
// NOTE: If pValSize is set to NULL then this routine will compute the
// size of pValue
//---------------------------------------------------------------------
s32 PopINISetKeyValueUTF8(_IN const astring *pINIPFName, 
						  _IN const astring *pSecName, 
						  _IN const astring *pKeyName, 
						  _IN const astring *pValue, 
						  _IN const u32 *pValSize)
{
	u32 size;


	// compute size
	if (pValSize == NULL)
	{
		size = (u32)(strlen(pValue) + 1);
	}
	else
	{
		size = *pValSize;
	} // if (pValSize == NULL)

	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_ASTRING, 
		pValue, 
		size, 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueUTF8


//---------------------------------------------------------------------
//  PopINISetKeyValueUnSigned32
//---------------------------------------------------------------------
s32 PopINISetKeyValueUnSigned32(_IN const astring *pINIPFName, 
								_IN const astring *pSecName, 
								_IN const astring *pKeyName, 
								_IN u32 value)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_UNSIGNED32_LITTLE_ENDIAN, 
		&value, 
		sizeof(value), 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueUnSigned32


//---------------------------------------------------------------------
//  PopINISetKeyValueSigned32
//---------------------------------------------------------------------
s32 PopINISetKeyValueSigned32(_IN const astring *pINIPFName, 
							  _IN const astring *pSecName, 
							  _IN const astring *pKeyName, 
							  _IN s32 value)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_SIGNED32_LITTLE_ENDIAN, 
		&value, 
		sizeof(value), 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueSigned32


//---------------------------------------------------------------------
//  PopINISetKeyValueUnSigned64
//---------------------------------------------------------------------
s32 PopINISetKeyValueUnSigned64(_IN const astring *pINIPFName, 
								_IN const astring *pSecName, 
								_IN const astring *pKeyName, 
								_IN u64 value)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_UNSIGNED64_LITTLE_ENDIAN, 
		&value, 
		sizeof(value), 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueUnSigned64


//---------------------------------------------------------------------
//  PopINISetKeyValueSigned64
//---------------------------------------------------------------------
s32 PopINISetKeyValueSigned64(_IN const astring *pINIPFName, 
							  _IN const astring *pSecName, 
							  _IN const astring *pKeyName, 
							  _IN s64 value)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_SIGNED64_LITTLE_ENDIAN, 
		&value, 
		sizeof(value), 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueSigned64


//---------------------------------------------------------------------
//  PopINISetKeyValueBooln
//---------------------------------------------------------------------
s32 PopINISetKeyValueBooln(_IN const astring *pINIPFName, 
						   _IN const astring *pSecName, 
						   _IN const astring *pKeyName, 
						   _IN booln value)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_BOOLN, 
		&value, 
		sizeof(value), 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueBooln


//---------------------------------------------------------------------
// PopINISetKeyValueBinary
// NOTE: this routine will calculate the size based on how many bytes
// it copied to string
//---------------------------------------------------------------------
s32 PopINISetKeyValueBinary(_IN const astring *pINIPFName, 
						  _IN const astring *pSecName, 
						  _IN const astring *pKeyName, 
						  _IN const u8 *pValue, 
						  _IN const u32 ValSize)
{
	// write key value and return status
	return SMWriteINIPathFileValue(pSecName, 
		pKeyName, 
		INI_BINARY, 
		pValue, 
		ValSize, 
		pINIPFName, 
		TRUE);
} // PopINISetKeyValueBinary


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------
