
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2001 Dell Inc. All Rights Reserved.
// 
//  Abstract/Purpose:
//
//
//  Environment:
//    Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 4 $
//
//
//  Last Modified by/on:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//-------------------------------------------------------------------------


#ifndef __SAMPOPOBJDEF_H_INCLUDED__
#define __SAMPOPOBJDEF_H_INCLUDED__


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcdataobj.h>



#ifdef __cplusplus
extern "C" {
#endif



//---------------------------------------------------------------------
//  Defines and types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//  Sample Object Type Base
//---------------------------------------------------------------------
// object type base used
// NOTE: Do not use the same base in your data populator
// request a range from the Data Engine team if not already assigned
#define DATA_OBJ_TYPE_SAMP_BASE			(DATA_OBJ_TYPE_NEXT_AVAILABLE_LO_1)

//---------------------------------------------------------------------
//  Sample Data Object Types
//---------------------------------------------------------------------
#define DATA_OBJ_TYPE_SAMP_STORAGE			(DATA_OBJ_TYPE_SAMP_BASE + 0)
#define DATA_OBJ_TYPE_SAMP_CONTROLLER		(DATA_OBJ_TYPE_SAMP_BASE + 1)
#define DATA_OBJ_TYPE_SAMP_ENCLOSURE		(DATA_OBJ_TYPE_SAMP_BASE + 2)
#define DATA_OBJ_TYPE_SAMP_TIMER			(DATA_OBJ_TYPE_SAMP_BASE + 3)
#define DATA_OBJ_TYPE_SAMP_ARRAYDISK		(DATA_OBJ_TYPE_SAMP_BASE + 4)
#define DATA_OBJ_TYPE_SAMP_VIRTDISK			(DATA_OBJ_TYPE_SAMP_BASE + 5)
// SDO Version
#define DATA_OBJ_TYPE_SAMP_BASE_SDO			(DATA_OBJ_TYPE_SAMP_BASE + 100)
#define DATA_OBJ_TYPE_SAMP_SDO_STORAGE		(DATA_OBJ_TYPE_SAMP_BASE_SDO + 0)
#define DATA_OBJ_TYPE_SAMP_SDO_CONTROLLER	(DATA_OBJ_TYPE_SAMP_BASE_SDO + 1)
#define DATA_OBJ_TYPE_SAMP_SDO_ENCLOSURE	(DATA_OBJ_TYPE_SAMP_BASE_SDO + 2)
#define DATA_OBJ_TYPE_SAMP_SDO_TIMER		(DATA_OBJ_TYPE_SAMP_BASE_SDO + 3)
#define DATA_OBJ_TYPE_SAMP_SDO_ARRAYDISK	(DATA_OBJ_TYPE_SAMP_BASE_SDO + 4)
#define DATA_OBJ_TYPE_SAMP_SDO_VIRTDISK		(DATA_OBJ_TYPE_SAMP_BASE_SDO + 5)


//---------------------------------------------------------------------
//  Sample Object Defines for DATA_OBJ_TYPE_SAMP_CONTROLLER state
//---------------------------------------------------------------------
#define DOTSC_STATE_UNKNOWN				(0)
#define DOTSC_STATE_OTHER				(1)
#define DOTSC_STATE_ALIVE				(2)
#define DOTSC_STATE_FRIED				(3)
#define DOTSC_STATE_DEAD				(4)


//---------------------------------------------------------------------
//  Sample Object Defines for DATA_OBJ_TYPE_SAMP_ENCLOSURE state
//---------------------------------------------------------------------
#define DOTSE_STATE_UNKNOWN				(0)
#define DOTSE_STATE_OTHER				(1)
#define DOTSE_STATE_CRAWLING			(2)
#define DOTSE_STATE_WALKING				(3)
#define DOTSE_STATE_RUNNING				(4)
#define DOTSE_STATE_MAX					(4)


//---------------------------------------------------------------------
//  Sample Object Defines for DATA_OBJ_TYPE_SAMP_ARRAYDISK state
//---------------------------------------------------------------------
#define DOTSAD_STATE_UNKNOWN			(0)
#define DOTSAD_STATE_OTHER				(1)
#define DOTSAD_STATE_FAILED				(2)
#define DOTSAD_STATE_DEGRADED			(3)
#define DOTSAD_STATE_OK					(4)
#define DOTSAD_STATE_MAX				(4)

//---------------------------------------------------------------------
//  Sample Object Defines for DATA_OBJ_TYPE_SAMP_VIRTDISK state
//---------------------------------------------------------------------
#define DOTSVD_STATE_UNKNOWN			(0)
#define DOTSVD_STATE_OTHER				(1)
#define DOTSVD_STATE_OK					(2)
#define DOTSVD_STATE_DEGRADED			(3)
#define DOTSVD_STATE_FAILED				(4)


//---------------------------------------------------------------------
//  Structure definitions
//---------------------------------------------------------------------

// definition of storage data object created under root
typedef struct _SAMPStorageDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// release major version
	s32 majorVersion;
	// release minor version
	s32 minorVersion;
	// release maintanence version
	s32 maintanenceVersion;
} SAMPStorageDataObj;


// definition of controller data object created under SAMPStorageDataObj
typedef struct _SAMPControllerDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// offset to Vendor Name
	u32 offsetVendorName;
	// offset to Device Name
	u32 offsetDeviceName;
	// controller state search DOTSC_STATE_
	u32 state;
} SAMPControllerDataObj;


// definition of a channel data object created under SAMPControllerDataObj
typedef struct _SAMPEnclosureDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// offset to Vendor Name
	u32 offsetEnclName;
	// channel on which it is present
	u32 channelNum;
	// number of array disks
	u32 numArrays;
	// major firmware version
	s32 firmwareMajorVersion;
	// minor firmware version
	s32 firmwareMinorVersion;
	// release firmware version
	s32 firmwareMaintanenceVersion;
	// state
	u32 state;
} SAMPEnclosureDataObj;


// definition of an array data object created under SAMPEnclosureDataObj
typedef struct _SAMPArrayDiskDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// offset to Array Disk Name
	u32 offsetArrayDiskName;
	// offset to Vendor Name
	u32 offsetVendorName;
	// state
	u32 state;
	// number of partitions on this physical disk
	u32 partitionCount;
} SAMPArrayDiskDataObj;

// definition of an array data object created under SAMPEnclosureDataObj
typedef struct _SAMPVirtDiskDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// offset to Virtual Disk Name
	u32 offsetVirtDiskName;
	// state
	u32 state;
} SAMPVirtDiskDataObj;


// definition of sample time elapsed count
typedef struct _SAMPTimerDataObj
{
	// object header needed for all objects
	DataObjHeader doh;
	// value of ctime at start in seconds from EPOCH
	s64 startTimeFromEPOCH;
	// number of 60 seconds increments since startTime
	s32 numMinutesFromStart;
	// temporary storage for tick count
	s32 tickCount;
} SAMPTimerDataObj;


#ifdef __cplusplus
}
#endif

#endif //  __SAMPOPOBJDEF_H_INCLUDED__


