
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 6 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//---------------------------------------------------------------------
//  Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  SamPopPrintOID
//---------------------------------------------------------------------
void SamPopPrintOID(_IN const ObjID *pOID, 
					_IN const astring *pOIDMsg)
{
	astring *pOIDCreatorName;


	// get creator name
	pOIDCreatorName = PopDPDMDDOGetCreatorAliasNameByOID(pOID);

	// print information
	SysDbgMsg4(("%s%08X created by: %s\n", 
		(pOIDMsg != NULL) ? pOIDMsg:"", 
		pOID->ObjIDUnion.asu32, 
		(pOIDCreatorName != NULL) ? pOIDCreatorName:""));

	// free creator name
	PopDPDMDFreeGeneric(pOIDCreatorName);

	// prevent netware compiler error
	UNREFERENCED_PARAMETER(pOIDMsg);
} // SamPopPrintOID


//---------------------------------------------------------------------
//  SamPopPrintObjList
//---------------------------------------------------------------------
void SamPopPrintObjList(_IN const ObjList *pOL)
{
	astring *pOIDCreatorName;
	u32 ii;


	// print information
	SysDbgMsg4(("objCount: %lu\n", 
		pOL->objCount));

	// go through all OIDS
	for (ii = 0; ii < pOL->objCount; ii++)
	{
		// get creator name
		pOIDCreatorName = PopDPDMDDOGetCreatorAliasNameByOID(&pOL->objID[ii]);

		// print information
		SysDbgMsg4(("objID[%lu]: %08X created by: %s\n", 
			ii, 
			pOL->objID[ii].ObjIDUnion.asu32, 
			(pOIDCreatorName != NULL) ? pOIDCreatorName:""));

		// free creator name
		PopDPDMDFreeGeneric(pOIDCreatorName);
	} // for (ii...
} // SamPopPrintObjList


//---------------------------------------------------------------------
//  SamPopPrintMemBuf
//---------------------------------------------------------------------
void SamPopPrintMemBuf(_IN const void *pBuf, 
					   _IN u32 bufSize)
{
	astring *pABufStr;
	u32 size;


	if (bufSize == 0)
	{
		SysDbgMsg4(("None"));
		return;
	}

	// allocate temporary buffer
	size = (((bufSize / 16) + 1) * 84);
	if ((pABufStr = (astring *)SMAllocMem(size)) == NULL)
	{
		return;
	}

	// convert buffer to string
	if ((SMXLTTypeValueToUTF8(pBuf, 
		bufSize, 
		pABufStr, 
		&size, 
		(SM_XLT_VTYPE_BINARY_TO_ASTRING | BINARY_HEXDUMP)) == SM_STATUS_SUCCESS) && 
		(size > 0))
	{
		SysDbgMsg4(("%s", 
			pABufStr));
	}

	// free temporary buffer
	SMFreeMem(pABufStr);
} // SamPopPrintMemBuf


//---------------------------------------------------------------------
// SamPopEventListener
// NOTE: When using event listeners you have to have _cdecl functions
// and compile the executable will multithreading libraries
//---------------------------------------------------------------------
s32 SamPopEventListener(_IN const DataEventHeader *pDEH)
{
	astring *pEvtTimeStampStr;
	void *pEventBody;
	DESingleParentCreateObj *pDESPCO;
	DEObjStatusChange *pDEOSC;
	u32 evtBodySize;


	// get the event time stamp as a string
	pEvtTimeStampStr = PopDPDMDDOGetObjTimeUTF8Str(pDEH->evtTimeStamp);

	SysDbgMsg4(("SamPopEventListener: Recieved event\nType: %lu Time: %s Flags: 0x%02X\n", 
		(u32)pDEH->evtType, 
		pEvtTimeStampStr, 
		pDEH->evtFlags));

	// free the string
	PopDPDMDFreeGeneric(pEvtTimeStampStr);
	// catch GPFs
	pEvtTimeStampStr = NULL;

	// point to event body
	pEventBody = (void *)((u8 *)pDEH + 
		sizeof(DataEventHeader));

	// set size of body
	evtBodySize = (pDEH->evtSize - 
		sizeof(DataEventHeader));

	// determine event type
	switch (pDEH->evtType)
	{
		case DATA_EVENT_DATA_OBJ_DIRTY_LIST:
			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_DATA_OBJ_DIRTY_LIST\n"));

			// print obj list
			SamPopPrintObjList((ObjList *)pEventBody);
		break;

		case DATA_EVENT_OBJ_STATUS_CHG:
			// point to data within event
			pDEOSC = (DEObjStatusChange *)pEventBody;

			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_OBJ_STATUS_CHG\n"));

			// print information
			SysDbgMsg4(("prevObjStatus: %02X prevRedStatus: %02X\n", 
				pDEOSC->prevObjStatus, 
				pDEOSC->prevRedStatus));

			// print OID
			SamPopPrintOID(&pDEOSC->objHeader.objID, 
				"objID: ");
		break;

		case DATA_EVENT_DATA_CONSUMER_ATTACHED:
			// print event
			SysDbgMsg3(("SamPopEventListener: invalid Data Consumer attached to Data Manager event\n"));
		break;

		case DATA_EVENT_DATA_CONSUMER_DETACHED:
			// print event
			SysDbgMsg3(("SamPopEventListener: invalid Data Consumer detached from Data Manager\n"));
		break;

		case DATA_EVENT_DATA_OBJ_CREATE:
			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_DATA_OBJ_CREATE\n"));

			// print obj list
			SamPopPrintObjList((ObjList *)pEventBody);
		break;

		case DATA_EVENT_DATA_OBJ_DESTROY:
			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_DATA_OBJ_DESTROY\n"));

			// print obj list
			SamPopPrintObjList((ObjList *)pEventBody);
		break;

		case DATA_EVENT_DATA_OBJ_BRANCH_CREATE:
			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_DATA_OBJ_BRANCH_CREATE\n"));

			// print obj list
			SamPopPrintObjList((ObjList *)pEventBody);
		break;

		case DATA_EVENT_DATA_OBJ_BRANCH_DESTROY:
			// print event
			SysDbgMsg4(("Data Populator recieved DATA_EVENT_DATA_OBJ_BRANCH_DESTROY\n"));

			// print obj list
			SamPopPrintObjList((ObjList *)pEventBody);
		break;

		case DATA_EVENT_DATA_MANAGER_ATTACHED:
			// print event
			SysDbgMsg4(("Data Manager attached event\n"));
		break;

		case DATA_EVENT_DATA_MANAGER_DETACHED:
			// print event
			SysDbgMsg4(("Data Manager detached event\n"));
		break;

		case DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID:
			// point to OID within event
			pDESPCO = (DESingleParentCreateObj *)pEventBody;

			// print event
			SysDbgMsg4(("Data Populator recieved obsolete event: DATA_EVENT_SINGLEPARENT_CREATE_OBJ_BY_OID\n"));

			// print OID
			SamPopPrintOID(&pDESPCO->parentID, 
				"parentID: ");

			// print OID
			SamPopPrintOID(&pDESPCO->objID, 
				"objID: ");
		break;

		case DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID:
			// print event
			SysDbgMsg4(("Data Populator recieved obsolete event: DATA_EVENT_SINGLE_DESTROY_OBJTREE_BY_OID\n"));

			// print OID
			SamPopPrintOID((ObjID *)pEventBody, 
				"objID: ");
		break;

		default:
			// print event
			SysDbgMsg4(("Data Populator recieved non-base event body: \n"));

			// dump the event data in hex
			SamPopPrintMemBuf(pEventBody, 
				evtBodySize);

			SysDbgMsg4(("\n"));
		break;
	} // switch (pDEH->evtType)

	// add a line between events
	SysDbgMsg4(("\n"));

	// continue sending event to 
	// next listner in the chain
	return SM_STATUS_SUCCESS;
} // SamPopEventListener


//---------------------------------------------------------------------
//  Code Ends
//---------------------------------------------------------------------

