
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 7 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>



//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------



//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopDispStartMonitor - Called by popdisp.c during SM_DRT_POP_START_MONITOR
// to indicate to populator module to perform any run time procedures
// to begin monitoring its or other data objects and generate events
//---------------------------------------------------------------------
s32 PopDispStartMonitor(void)
{
	s32 status;


    SysDbgMsg4(("PopDispStartMonitor: entry\n"));

	// start all worker threads
	status = SamPopAttachThreads();

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("PopDispStartMonitor: failed SamPopAttachThreads with status: %ld\n", 
			status));
		return status;
	} // if (status != SM_STATUS_SUCCESS)

	// test to get DEV_PCI_OBJ_TYPE
	status = SamPopGetDevPCIObjType();

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("PopDispStartMonitor: failed SamPopGetDevPCIObjType with status: %ld\n", 
			status));
		_throw error1;
	} // if (status != SM_STATUS_SUCCESS)

    SysDbgMsg4(("PopDispStartMonitor: exit\n"));

	// success exit
	return SM_STATUS_SUCCESS;

error1:
	// detach threads
	SamPopDetachThreads();
	// failed exit codeS
    return status;
} // PopDispStartMonitor


//---------------------------------------------------------------------
// PopDispStopMonitor - Called by popdisp.c during SM_DRT_POP_STOP_MONITOR
// to indicate to populator module to perform any run time procedures
// to end monitoring its or other objects and generation of events
//---------------------------------------------------------------------
s32 PopDispStopMonitor(void)
{
	s32 status;


    SysDbgMsg4(("PopDispStopMonitor: entry\n"));

	// start all worker threads
	status = SamPopDetachThreads();

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("PopDispStopMonitor: failed SamPopDetachThreads with status: %ld\n", 
			status));
		return status;
	} // if (status != SM_STATUS_SUCCESS)

    SysDbgMsg4(("PopDispStopMonitor: exit\n"));

    return SM_STATUS_SUCCESS;
} // PopDispStopMonitor


//---------------------------------------------------------------------
// PopDispLoad - Called by popdisp.c during SM_DRT_POP_LOAD to indicate
// that the Data Populator module should perform load time procedures
// in order to prepare for subsequent requests and return success. 
// If error returned then the Data Populator is freed from 
// Process or OS memory
//---------------------------------------------------------------------
s32 PopDispLoad(_IN const SMDPDMDispatchEnv *pDPDMDE)
{
	s32 status;


    SysDbgMsg4(("PopDispLoad: entry\n"));

	// the push interface environmet is valid
	// at this stage however real object data
	// can only be found at start monitor

	// In this routine the most important
	// steps are to do two main things:

	// 1. Allocate any resources needed for
	// subsequent request processing
	PopDPDMDAttach(pDPDMDE);

	// allocate data populator globals
	status = SamPopGlobalsAlloc();

	// failed?
	if (status != SM_STATUS_SUCCESS)
	{
		return status;
	}

	// 2. Determine if the hardware or software
	// being managed is present on this box and
	// there is a reason for this Data Populator
	// to be loaded in process or OS memory
	if (SamPopIsManagedComponentPresent() == FALSE)
	{
		// free the global memory and lock
		SamPopGlobalsFree();
		// clear the push environment
		PopDPDMDDetach();
		return SM_STATUS_NO_SUCH_DEVICE;
	} // if (SamPopIsManagedComponentPresent() == FALSE)

    SysDbgMsg4(("PopDispLoad: exit\n"));

	// done
    return SM_STATUS_SUCCESS;
} // PopDispLoad 


//---------------------------------------------------------------------
// PopDispUnLoad - Called by popdisp.c during SM_DRT_POP_UNLOAD to indicate
// to populator module to perform any unload time procedures and
// prepare to be freed from Process and OS memory space
//---------------------------------------------------------------------
s32 PopDispUnLoad(void)
{

    SysDbgMsg4(("PopDispUnLoad: entry\n"));

	// we have to make sure that all threads
	// are really stopped by now as the 
	// push interface environment will
	// be no longer valid (freed by data manager) when
	// this routine returns

	// make sure we have stopped threads
	SamPopDetachThreads();

	// free the global memory and lock
	SamPopGlobalsFree();

	// clear the push environment
	PopDPDMDDetach();

    SysDbgMsg4(("PopDispUnLoad: exit\n"));

    return SM_STATUS_SUCCESS;
} // PopDispUnLoad




