
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2002 Dell Inc. All Rights Reserved.
// 
//  Abstract/Purpose:
//
//
//  Environment:
//    Linux, Netware, Windows NT/2000
//
//  Author/Version:
//    Chris Poblete / $Revision: 8 $
//
//  Last Modified By/On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//-------------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>
#include <sampopini.h>
#include <popdpdmdispatch.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------

#define DEFAULT_REQ_TEST_NUMTHREADS		(4)

// data structure for populator worker thread data
typedef struct _SamPopThreadWorkerData
{
	// denotes number of threads requested
	s32 numThreads;

	// points to numThreads thread handles
	void **ppReqTestThreads;

} SamPopThreadWorkerData;


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//  SamPopDMReqPrintOID
//---------------------------------------------------------------------
void SamPopDMReqPrintOID(_IN const ObjID *pOID, 
				   _IN const astring *pOIDMsg)
{
	astring *pOIDCreatorName;


	// get creator name
	pOIDCreatorName = PopDPDMDDOGetCreatorAliasNameByOID(pOID);

	// print information
	SysDbgMsg3(("%s%08X created by: %s\n", 
		(pOIDMsg != NULL) ? pOIDMsg:"", 
		pOID->ObjIDUnion.asu32, 
		(pOIDCreatorName != NULL) ? pOIDCreatorName:""));

	// free creator name
	PopDPDMDFreeGeneric(pOIDCreatorName);

	// prevent netware compiler error
	UNREFERENCED_PARAMETER(pOIDMsg);
} // SamPopDMReqPrintOID


//---------------------------------------------------------------------
// SamPopDMReqTest
//---------------------------------------------------------------------
s32 SamPopDMReqTest(_IN s32 rLevel,
					_IN ObjID *pOID)
{
	DataObjHeader *pDOH;
	ObjList *pOL1;
	ObjList *pOL2;
	astring *pCreatorName;
	SMReqHeaderPassThru rhpu;
	SMReqHeaderSet rhs;
	s32 status;
	u32 sizeT;
	u32 ii;
	u32 jj;


	// inits
	status = SM_STATUS_UNSUCCESSFUL;
	pOL2 = NULL;
	pOL1 = NULL;
	pCreatorName = NULL;
	pDOH = NULL;

	// check for invalid OID
	//----------------------------------------------------------------------
	if (pOID->ObjIDUnion.asu32 == DATA_OBJ_OID_INVALID)
	{
		SysDbgMsg3(("SamPopDMReqTest: Invalid OID(0x%08X) encountered", 
			pOID->ObjIDUnion.asu32));
		status = SM_STATUS_UNSUCCESSFUL;
		_throw error1;
	}

	// get the object given the OID and verify
	//----------------------------------------------------------------------
	SysDbgMsg4(("SamPopDMReqTest: Processing object from OID(0x%08X)\n",
		pOID->ObjIDUnion.asu32));
	pDOH = PopDPDMDGetDataObjByOID(pOID);
	if (pDOH == NULL)
	{
		SysDbgMsg4(("SamPopDMReqTest: PopDPDMDGetObjByOID " \
					"returned NULL on OID(0x%08X)\n", 
			pOID->ObjIDUnion.asu32));
		status = SM_STATUS_UNSUCCESSFUL;
		_throw error1;
	}
	// verify size
	if (pDOH->objSize < sizeof(DataObjHeader))
	{
		SysDbgMsg3(("SamPopDMReqTest: PopDPDMDGetObjByOID " \
					"obj size smaller than MIN for OID(0x%08X)\n", 
			pOID->ObjIDUnion.asu32));
		status = SM_STATUS_UNSUCCESSFUL;
		_throw error2;
	}

	// verify that we can identify the populator that created this object
	//----------------------------------------------------------------------
	SysDbgMsg4(("SamPopDMReqTest: Verifying populator name for given OID\n"));
	pCreatorName = PopDPDMDDOGetCreatorAliasNameByOID(pOID);
	if (pCreatorName == NULL)
	{
		SysDbgMsg3(("SamPopDMReqTest: Cannot get populator name for OID(0x%08X)\n", 
			pOID->ObjIDUnion.asu32));
		status = SM_STATUS_UNSUCCESSFUL;
		_throw error2;
	}
	SysDbgMsg4(("SamPopDMReqTest: populator name(%s)\n", pCreatorName));

	if (pOID->ObjIDUnion.InnerObjIDStruct.popID != DATA_CREATOR_ID_PUBLIC)
	{
		// verify we can get to the parent root
		//----------------------------------------------------------------------
		SysDbgMsg4(("SamPopDMReqTest: Verifying parents of ROOT type\n"));
		pOL1 = PopDPDMDListParentOIDByType(pOID, DATA_OBJ_TYPE_ROOT);
		if (pOL1 == NULL)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOIDByType " \
						"returned NULL for OID(0x%08X)\n", 
				pOID->ObjIDUnion.asu32));
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error3;
		}
		// list should have at least 1 entry
		if (pOL1->objCount < 1)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOIDByType " \
						"list is empty for OID(0x%08X)\n", 
				pOID->ObjIDUnion.asu32));
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error4;
		}
		// entries in list should have OID of ROOT
		for (ii = 0; ii < pOL1->objCount; ii++)
		{
			if (pOL1->objID[ii].ObjIDUnion.asu32 != DATA_OBJ_OID_ROOT)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOIDByType " \
							"entry(0x%08X) is not ROOT for OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error4;
			}
		}
		
		// verify that the current OID shows up in
		// its parent's list of child by type and status
		//----------------------------------------------------------------------
		SysDbgMsg4(("SamPopDMReqTest: Verifying ROOT parents query by objtype and status\n"));
		for (ii = 0; ii < pOL1->objCount; ii++)
		{
			// If TRUE then object status searched should be equal or
			// worse (>=) than the given status. FALSE if exact match desired
			pOL2 = PopDPDMDListChildOIDByStatus(&pOL1->objID[ii],
				pDOH->objType,
				TRUE,
				DATA_OBJ_STATUS_OTHER);
			if (pOL2 == NULL)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByStatus " \
							"parent(0x%08X) returned NULL for OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			// list should have at least 1 entry
			if (pOL2->objCount < 1)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByStatus " \
							"parent(0x%08X) list is empty for OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			// current OID should show up in the list
			for (jj = 0; jj < pOL2->objCount; jj++)
			{
				if (pOL2->objID[jj].ObjIDUnion.asu32 == pOID->ObjIDUnion.asu32)
				{
					break;
				}
			}
			// verify
			if (jj == pOL2->objCount)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByStatus " \
							"parent(0x%08X) list does not contain OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			
			// free local memory
			PopDPDMDFreeGeneric(pOL2);
			pOL2 = NULL;
		}

		// verify that the current OID shows up in
		// its parent's list of child by type
		//----------------------------------------------------------------------
		SysDbgMsg4(("SamPopDMReqTest: Verifying ROOT parents query by objtype only\n"));
		for (ii = 0; ii < pOL1->objCount; ii++)
		{
			pOL2 = PopDPDMDListChildOIDByType(&pOL1->objID[ii],
				pDOH->objType);
			if (pOL2 == NULL)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByType " \
							"parent(0x%08X) returned NULL for OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			// list should have at least 1 entry
			if (pOL2->objCount < 1)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByType " \
							"parent(0x%08X) list is empty for OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			// current OID should show up in the list
			for (jj = 0; jj < pOL2->objCount; jj++)
			{
				if (pOL2->objID[jj].ObjIDUnion.asu32 == pOID->ObjIDUnion.asu32)
				{
					break;
				}
			}
			// verify
			if (jj == pOL2->objCount)
			{
				SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOIDByType " \
							"parent(0x%08X) list does not contain OID(0x%08X)\n", 
					pOL1->objID[ii].ObjIDUnion.asu32,
					pOID->ObjIDUnion.asu32));
				status = SM_STATUS_UNSUCCESSFUL;
				_throw error5;
			}
			
			// free local memory
			PopDPDMDFreeGeneric(pOL2);
			pOL2 = NULL;
		}

		// perform pass thru test
		//----------------------------------------------------------------------
		SysDbgMsg4(("SamPopDMReqTest: Performing pass thru test\n"));
		
		// init request object
		rhpu.objID = *pOID;
		rhpu.ptCmd = SM_PASSTHRU_TYPE_INVALID;
		
		// call the API
		sizeT = 0;
		status = PopDPDMDPassThruObjByReq(&rhpu, 
			sizeof(rhpu), 
			NULL, 
			0, 
			&sizeT);
		if ((status == SM_STATUS_SUCCESS) || 
			(sizeT != 0))
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDPassThruObjByReq " \
						"size(%lu) stat(%ld) unexpected success\n", 
				sizeT,
				status));
			SamPopDMReqPrintOID(pOID, "OID: ");
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error5;
		}
		
		// perform invalid set test
		//----------------------------------------------------------------------
		SysDbgMsg4(("SamPopDMReqTest: Performing invalid set test\n"));
		
		// init request object
		rhs.objID = *pOID;
		rhs.setType = SM_SET_TYPE_INVALID;
		
		// call the API
		status = PopDPDMDSetObjByReq(&rhs, 
			sizeof(rhs));
		if (status == SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDSetObjByReq " \
						"unexpected success\n"));
			SamPopDMReqPrintOID(pOID, "OID: ");
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error5;
		}
		
	} // if (pOID->ObjIDUnion.asu32 != DATA_OBJ_OID_ROOT)
		
	// get current OID children and check that current OID is one of its parent
	//----------------------------------------------------------------------
	SysDbgMsg4(("SamPopDMReqTest: Checking that each child of curOID has its parent the curOID\n"));

	// pOL1 is not needed at this point
	// pOL2 should be free at this point
	PopDPDMDFreeGeneric(pOL1);
	pOL1 = NULL;
	
	// get list of children
	pOL1 = PopDPDMDListChildOID(pOID);
	if (pOL1 == NULL)
	{
		SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListChildOID returned NULL for OID(0x%08X)\n", 
			pOID->ObjIDUnion.asu32));
		status = SM_STATUS_UNSUCCESSFUL;
		_throw error5;
	}
	// list should have at least 1 entry
	if (pOL1->objCount < 1)
	{
		// perhaps current OID is leaf, so just leave with success
		status = SM_STATUS_SUCCESS;
		_throw error5;
	}
	// entries in list should have OID of ROOT
	for (ii = 0; ii < pOL1->objCount; ii++)
	{
		pOL2 = PopDPDMDListParentOID(&pOL1->objID[ii]);
		if (pOL2 == NULL)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOID " \
						"returned NULL on child(0x%08X) of OID(0x%08X)\n", 
				pOL1->objID[ii].ObjIDUnion.asu32,
				pOID->ObjIDUnion.asu32));
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error5;
		}
		// list should have at least 1 entry
		if (pOL2->objCount < 1)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOID " \
						"list is empty on child(0x%08X) of OID(0x%08X)\n", 
				pOL1->objID[ii].ObjIDUnion.asu32,
				pOID->ObjIDUnion.asu32));
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error5;
		}
		// verify that the current OID is one of the child's parent
		for (jj = 0; jj < pOL2->objCount; jj++)
		{
			if (pOL2->objID[jj].ObjIDUnion.asu32 == pOID->ObjIDUnion.asu32)
			{
				break;
			}
		}
		// verify
		if (jj == pOL2->objCount)
		{
			SysDbgMsg3(("SamPopDMReqTest: PopDPDMDListParentOID " \
						"current OID is not a parent of child(0x%08X) of OID(0x%08X)\n", 
				pOL1->objID[ii].ObjIDUnion.asu32,
				pOID->ObjIDUnion.asu32));
			status = SM_STATUS_UNSUCCESSFUL;
			_throw error5;
		}

		// free local memory
		PopDPDMDFreeGeneric(pOL2);
		pOL2 = NULL;
	}

	// Now recursing through the children
	//----------------------------------------------------------------------
	SysDbgMsg4(("SamPopDMReqTest: Now recursing through children of curOID\n"));
	for (ii = 0; ii < pOL1->objCount; ii++)
	{
		status = SamPopDMReqTest((rLevel + 1), &pOL1->objID[ii]);
		if (status != SM_STATUS_SUCCESS)
		{
			status = SM_STATUS_UNSUCCESSFUL;
			break;
		}
	}

 error5:
	if (pOL2 != NULL)
	{
		PopDPDMDFreeGeneric(pOL2);
		pOL2 = NULL;
	}
 error4:
	if (pOL1 != NULL)
	{
		PopDPDMDFreeGeneric(pOL1);
		pOL1 = NULL;
	}
 error3:
	if (pCreatorName != NULL)
	{
		PopDPDMDFreeGeneric(pCreatorName);
		pCreatorName = NULL;
	}
 error2:
	if (pDOH != NULL)
	{
		PopDPDMDFreeGeneric(pDOH);
		pDOH = NULL;
	}
 error1:
	return status;
} // SamPopDMReqTest


//---------------------------------------------------------------------
/**
 * Behavior for sampop req threads.
 *
 * This function is called by SamPopReqTestThreadStart() and is execute
 * by multi-threads. All it does is recursively test ROOT OID (two 
 * request a second.)
 *
 * @param pSMThreadData [in] Parameters to the thread, only contain the
 *		index for current thread.
 *
 * @return no return.
 *
 * @sa SamPopReqTestThreadStart()
 *
 */
//---------------------------------------------------------------------
void _cdecl SamPopReqTestThreadWorker(_IN void *pSMThreadData)
{
	ObjID oid;
	s32 ii;


	// get thread index
	ii = (s32)pSMThreadData;

	// display
	SysDbgMsg4(("SamPopReqTestThreadWorker: entry thread (%d)\n", 
		ii));

	// while the module is not exiting
	// do something every half second
	while (SamPopGlobalsGetExitingFlag() == FALSE)
	{
		// start recursive test from ROOT OID
		oid.ObjIDUnion.asu32 = DATA_OBJ_OID_ROOT;
		if (SamPopDMReqTest(1, &oid) != SM_STATUS_SUCCESS)
		{
			SysDbgMsg4(("SamPopReqTestThreadWorker: thread (%d): \
				DM Req stress test failed!\n", 
				ii));
		}

		SysDbgMsg4(("SamPopReqTestThreadWorker: thread (%d): \
			DM Req stress test success!\n", 
			ii));

		// sleep 0.5 second
		Sleep(500);
	} // while

	SysDbgMsg4(("SamPopReqTestThreadWorker: exit thread (%d)\n", 
		ii));

	UNREFERENCED_PARAMETER(pSMThreadData);
} // SamPopReqTestThreadWorker


//---------------------------------------------------------------------
/**
 * Thread management function for req test threads.
 *
 * This function is called by SamPopAttachThreads(), which is called by
 * PopDispStartMonitor after sampop is loaded.
 *
 * It then reads from sampst32.ini for number of threads defined
 * (default 4) and start these threads. Thread function is defined in
 * SamPopReqTestThreadWorker()
 * 
 * @param
 *
 * @return SM_STATUS_SUCCESS when success, error code when failed.
 *
 * @sa SamPopReqTestThreadWorker()
 *
 */
//---------------------------------------------------------------------
s32 SamPopReqTestThreadStart(void)
{
	SamPopThreadWorkerData *pSPTWD = NULL;
	u32 size;
	s32 rstat = SM_STATUS_SUCCESS;
	s32 ii = 0;


	SysDbgMsg4(("SamPopReqTestThreadStart: entry\n"));

	// allocate thread memory
	if ((pSPTWD = SMAllocMem(sizeof(SamPopThreadWorkerData))) == NULL)
	{
		rstat = SM_STATUS_OUT_OF_MEMORY;
		_throw error1;
	}

	// read from INI file for thread number
	pSPTWD->numThreads = DEFAULT_REQ_TEST_NUMTHREADS;
	size = sizeof(pSPTWD->numThreads);
	SMReadINIFileValue(INI_SEC_REQ_TEST, // [Req Test]
		INI_KEY_REQ_NUMTHEADS, // req.numthreads
		INI_SIGNED32_LITTLE_ENDIAN,	// a signed integer
		&pSPTWD->numThreads, // output buffer
		&size, // output size
		&pSPTWD->numThreads, // default
		size, // default size
		INI_NAME_SAMPOP_STATIC, // sampopst32.ini
		TRUE); // wait

	// see if default selected
	if (pSPTWD->numThreads <= 0)
	{
		// use default
		pSPTWD->numThreads = DEFAULT_REQ_TEST_NUMTHREADS;
	} // if (pSPTWD->numThreads <= 0)

	// display
	SysDbgMsg4(("SamPopReqTestThreadStart: ini numthreads is (%d)\n", 
				pSPTWD->numThreads));

	// allocate memory for threads
	// freed globally since SamPopGlobalsSetReqTestThreadPtr called
	pSPTWD->ppReqTestThreads = (void **)SMAllocMem((sizeof(void *) * pSPTWD->numThreads));

	// failed?
	if (pSPTWD->ppReqTestThreads != NULL)
	{
		// start all threads to monitor the data base
		for (ii = 0; ii < pSPTWD->numThreads; ii++ )
		{
			// start a thread
			pSPTWD->ppReqTestThreads[ii] = SMThreadStart(SamPopReqTestThreadWorker, 
						(void *)ii);

			// failed ?
			if (pSPTWD->ppReqTestThreads[ii] == NULL)
			{
				SysDbgMsg2(("SamPopReqTestThreadStart: start thread (%d) failed\n", 
					ii));
			}
		} // for (...
	} // if (ppReqTestThreads != NULL)

	// save thread pointers in global memory
	SamPopGlobalsSetReqTestThreadPtr(pSPTWD);

	// done
	rstat = SM_STATUS_SUCCESS;

error1:
	SysDbgMsg4(("SamPopReqTestThreadStart: exit\n"));
	return rstat;
} // SamPopReqTestThreadStart


//---------------------------------------------------------------------
// SamPopReqTestThreadStop
//---------------------------------------------------------------------
void SamPopReqTestThreadStop(void)
{
	SamPopThreadWorkerData *pSPTWD = NULL;
	s32 ii;


	SysDbgMsg4(("SamPopReqTestThreadStop: entry\n"));

	// get and clear the thread pointer in global memory
	pSPTWD = (SamPopThreadWorkerData *)SamPopGlobalsSetReqTestThreadPtr(NULL);

	// check if thread worker data returned
	// (will be NULL if already stopped)
	if (pSPTWD != NULL)
	{
		// stop all threads from monitoring the data base
		for (ii = 0; ii < pSPTWD->numThreads; ii++ )
		{
			// stop all worker threads
			if (pSPTWD->ppReqTestThreads[ii] != NULL)
			{
				// stop the thread
				SMThreadStop(pSPTWD->ppReqTestThreads[ii]);
				// catch GPFs
				pSPTWD->ppReqTestThreads[ii] = NULL;
			}
		} // for (...

		// free thread memory
		SMFreeMem(pSPTWD);
		// catch GPFs
		pSPTWD = NULL;
	}

	SysDbgMsg4(("SamPopReqTestThreadStop: exit\n"));
} // SamPopReqTestThreadStop


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

