
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
// 
//  Abstract/Purpose:
//
//
//  Environment:
//    Linux, Netware, Windows NT/2000
//
//  Author/Version:
//    Chris Poblete / $Revision: 5 $
//
//  Last Modified By/On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// SDK include files
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Standard C include files
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Common Module include
//---------------------------------------------------------------------
#include <sampop.h>
#include <sampopdef.h>


//---------------------------------------------------------------------
// Local includes
//---------------------------------------------------------------------
#include <sampop.h>


//---------------------------------------------------------------------
// Local defines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// External Definitions/Functions
//---------------------------------------------------------------------

// simulated controller data
extern const SAMPControllerSimData simController[SAMP_NUM_CONTROLLERS];
// simulated enclosure data
extern const SAMPEnclosureSimData simEnclosure[SAMP_NUM_ENCLOSURES];
// simulated array disk data
extern const SAMPArrayDiskSimData simArray[];
// simulated virtual disk data
extern const SAMPVirtDiskSimData simVirt[];


extern ObjID SamPopGetStorageObjChildDOGetOID(_IN u8 popID, 
											  _IN u16 objType, 
											  _IN u8 instance);

extern void SAMPopSetupObjDefaultHeader(_IN const ObjID *pOID, 
										_OUT DataObjHeader *pDOH);


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SPSInsertSDOToDOH
//---------------------------------------------------------------------
s32 SPSInsertSDOToDOH(_IN SDOConfig *pSDConfig,
					  _INOUT DataObjHeader *pDOH,
					  _IN u32 sizeDOH)
{
	SDOBinary *pSDBinary;
	u32 sizeBinary;
	u32 sizeTemp;
	s32 rstat;


	// sanity check
	if ((pSDConfig == NULL) ||
		(pDOH == NULL))
	{
		SysDbgMsg3(("SPSInsertSDOToDOH: invalid parameter\n"));
	}
	else
	{
		// get size binary size
		sizeBinary = SMSDOConfigGetBinarySize(pSDConfig);
		
		// check binary size
		if (sizeBinary < SM_SDO_MIN_BINARY_SIZE)
		{
			SysDbgMsg3(("SPSInsertSDOToDOH: " \
				"size(%lu) is less than minimum SDO binary size\n", 
				sizeBinary));
		}
		else
		{
			// calculate minimum size
			sizeTemp = (sizeof(DataObjHeader) + SM_SDO_MIN_BINARY_SIZE);
			// check binary size
			if (sizeBinary > (MAX_DATA_OBJECT_SIZE - sizeTemp))
			{
				SysDbgMsg3(("SPSInsertSDOToDOH: " \
					"size(%lu) exceeds maximum SDO binary size to fit in DOH\n", 
					sizeBinary));
			}
			else
			{
				// get reference to target memory
				sizeTemp = sizeof(DataObjHeader);
				pSDBinary = (SDOBinary *)(((u8 *)pDOH) + sizeTemp);
				
				// save the calculated binary size for comparison
				sizeTemp = sizeBinary;
				
				// convert to binary
				rstat = SMSDOConfigToBinary(pSDConfig,
					pSDBinary,
					&sizeTemp);
				if (rstat == SM_STATUS_SUCCESS)
				{
					// check converted size
					if (sizeTemp != sizeBinary)
					{
						SysDbgMsg3(("SPSInsertSDOToDOH: " \
							"converted size mismatch pre(%lu) post(%lu)\n",
							sizeBinary,
							sizeTemp));
					}
					else
					{
						// now set the DOH size
						pDOH->objSize = sizeof(DataObjHeader) + sizeBinary;
						
						// good at this point
						return SM_STATUS_SUCCESS;
					}
				}
			}
		}
	}

	UNREFERENCED_PARAMETER(sizeDOH);

	return SM_STATUS_UNSUCCESSFUL;
} // SPSInsertSDOToDOH


//---------------------------------------------------------------------
// SPSAddSDOSAMPControllerSimData
//---------------------------------------------------------------------
s32 SPSAddSDOSAMPControllerSimData(_IN SDOConfig *pSDConfig,
								   _IN u32 dataIndex)
{
	s32 rstat = SM_STATUS_INVALID_PARAMETER;
	u16 currentFID = 0;


	// sanity check
	if ((pSDConfig != NULL) &&
		(dataIndex < SAMP_NUM_CONTROLLERS))
	{
		// add vendor name
		currentFID = SDOFID_CSD_VENDORNAME;
		rstat = SMSDOConfigAddData(pSDConfig,
			currentFID,
			SM_SDO_TYPE_ASTRING,
			simController[dataIndex].pVendorName,
			(u32)(strlen(simController[dataIndex].pVendorName) + 1),
			TRUE);
		if (rstat == SM_STATUS_SUCCESS)
		{
			// add device name
			currentFID = SDOFID_CSD_DEVICENAME;
			rstat = SMSDOConfigAddData(pSDConfig,
				currentFID,
				SM_SDO_TYPE_ASTRING,
				simController[dataIndex].pDeviceName,
				(u32)(strlen(simController[dataIndex].pDeviceName) + 1),
				TRUE);
			if (rstat == SM_STATUS_SUCCESS)
			{
				// add state
				currentFID = SDOFID_CSD_STATE;
				rstat = SMSDOConfigAddData(pSDConfig,
					currentFID,
					SM_SDO_TYPE_U32,
					&simController[dataIndex].state,
					sizeof(u32),
					TRUE);
				if (rstat == SM_STATUS_SUCCESS)
				{
					// add numChannels
					currentFID = SDOFID_CSD_NUMCHANNELS;
					rstat = SMSDOConfigAddData(pSDConfig,
						currentFID,
						SM_SDO_TYPE_U32,
						&simController[dataIndex].numChannels,
						sizeof(u32),
						TRUE);
					if (rstat == SM_STATUS_SUCCESS)
					{
						// add state
						currentFID = SDOFID_CSD_NUMVIRTDISKS;
						rstat = SMSDOConfigAddData(pSDConfig,
							currentFID,
							SM_SDO_TYPE_U32,
							&simController[dataIndex].numVirtDisks,
							sizeof(u32),
							TRUE);
					}
				}
			}
		}
	}

	if (rstat != SM_STATUS_SUCCESS)
	{
		SysDbgMsg4(("SPSAddSDOSAMPControllerSimData: " \
			"Add failed on FID(%lu) IDX(%lu) STAT(%ld)\n",
			currentFID,
			dataIndex,
			rstat));
	}

	return rstat;
} // SPSAddSDOSAMPControllerSimData


//---------------------------------------------------------------------
// SPSAddSDOSAMPEnclosureSimData
//---------------------------------------------------------------------
s32 SPSAddSDOSAMPEnclosureSimData(_IN SDOConfig *pSDConfig,
								  _IN u32 dataIndex)
{
	s32 rstat = SM_STATUS_INVALID_PARAMETER;
	u16 currentFID = 0;


	// sanity check
	if ((pSDConfig != NULL) &&
		(dataIndex < SAMP_NUM_ENCLOSURES))
	{
		// add pEnclName
		currentFID = SDOFID_ESD_ENCLNAME;
		rstat = SMSDOConfigAddData(pSDConfig,
			currentFID,
			SM_SDO_TYPE_ASTRING,
			simEnclosure[dataIndex].pEnclName,
			(u32)(strlen(simEnclosure[dataIndex].pEnclName) + 1),
			TRUE);
		if (rstat == SM_STATUS_SUCCESS)
		{
			// add state
			currentFID = SDOFID_ESD_STATE;
			//
			// set init state to DOTSE_STATE_RUNNING
			//
			rstat = SMSDOConfigAddData(pSDConfig,
				currentFID,
				SM_SDO_TYPE_U32,
				&simEnclosure[dataIndex].state,
				sizeof(u32),
				TRUE);
			if (rstat == SM_STATUS_SUCCESS)
			{
				// add channelNum
				currentFID = SDOFID_ESD_CHANNELNUM;
				rstat = SMSDOConfigAddData(pSDConfig,
					currentFID,
					SM_SDO_TYPE_U32,
					&simEnclosure[dataIndex].channelNum,
					sizeof(u32),
					TRUE);
				if (rstat == SM_STATUS_SUCCESS)
				{
					// add numArrays
					currentFID = SDOFID_ESD_NUMARRAYS;
					rstat = SMSDOConfigAddData(pSDConfig,
						currentFID,
						SM_SDO_TYPE_U32,
						&simEnclosure[dataIndex].numArrays,
						sizeof(u32),
						TRUE);
					if (rstat == SM_STATUS_SUCCESS)
					{
						// add firmwareMajorVersion
						currentFID = SDOFID_ESD_FIRMWAREMAJORVERSION;
						rstat = SMSDOConfigAddData(pSDConfig,
							currentFID,
							SM_SDO_TYPE_S32,
							&simEnclosure[dataIndex].firmwareMajorVersion,
							sizeof(s32),
							TRUE);
						if (rstat == SM_STATUS_SUCCESS)
						{
							// add firmwareMinorVersion
							currentFID = SDOFID_ESD_FIRMWAREMINORVERSION;
							rstat = SMSDOConfigAddData(pSDConfig,
								currentFID,
								SM_SDO_TYPE_S32,
								&simEnclosure[dataIndex].firmwareMinorVersion,
								sizeof(s32),
								TRUE);
							if (rstat == SM_STATUS_SUCCESS)
							{
								// add firmwareMaintanenceVersion
								currentFID = SDOFID_ESD_FIRMWAREMAINTANENCEVERSION;
								rstat = SMSDOConfigAddData(pSDConfig,
									currentFID,
									SM_SDO_TYPE_S32,
									&simEnclosure[dataIndex].firmwareMaintanenceVersion,
									sizeof(s32),
									TRUE);
							}
						}
					}
				}
			}
		}
	}

	if (rstat != SM_STATUS_SUCCESS)
	{
		SysDbgMsg4(("SPSAddSDOSAMPEnclosureSimData: " \
			"Add failed on FID(%lu) IDX(%lu) STAT(%ld)\n",
			currentFID,
			dataIndex,
			rstat));
	}

	return rstat;
} // SPSAddSDOSAMPEnclosureSimData


//---------------------------------------------------------------------
// SPSAddSDOSAMPArrayDiskSimData
//---------------------------------------------------------------------
s32 SPSAddSDOSAMPArrayDiskSimData(_IN SDOConfig *pSDConfig,
								  _IN u32 dataIndex,
								  _IN const astring *pPhysDiskName)
{
	s32 rstat = SM_STATUS_INVALID_PARAMETER;
	u16 currentFID = 0;


	// sanity check
	if ((pSDConfig != NULL) &&
		(dataIndex < SAMP_NUM_ENCLOSURES))
	{
		if (pPhysDiskName != NULL)
		{
			// add pArrayDiskName
			currentFID = SDOFID_ADSD_ARRAYDISKNAME;
			rstat = SMSDOConfigAddData(pSDConfig,
				currentFID,
				SM_SDO_TYPE_ASTRING,
				pPhysDiskName,
				(u32)(strlen(pPhysDiskName) + 1),
				TRUE);
		}
		else
		{
			// add pArrayDiskName
			currentFID = SDOFID_ADSD_ARRAYDISKNAME;
			rstat = SMSDOConfigAddData(pSDConfig,
				currentFID,
				SM_SDO_TYPE_ASTRING,
				simArray[dataIndex].pArrayDiskName,
				(u32)(strlen(simArray[dataIndex].pArrayDiskName) + 1),
				TRUE);
		}
		if (rstat == SM_STATUS_SUCCESS)
		{
			// add pVendorName
			currentFID = SDOFID_ADSD_VENDORNAME;
			rstat = SMSDOConfigAddData(pSDConfig,
				currentFID,
				SM_SDO_TYPE_ASTRING,
				simArray[dataIndex].pVendorName,
				(u32)(strlen(simArray[dataIndex].pVendorName) + 1),
				TRUE);
			if (rstat == SM_STATUS_SUCCESS)
			{
				// add state
				currentFID = SDOFID_ADSD_STATE;
				rstat = SMSDOConfigAddData(pSDConfig,
					currentFID,
					SM_SDO_TYPE_U32,
					&simArray[dataIndex].state,
					sizeof(u32),
					TRUE);
				if (rstat == SM_STATUS_SUCCESS)
				{
					// add partitionCount
					currentFID = SDOFID_ADSD_PARTITIONCOUNT;
					rstat = SMSDOConfigAddData(pSDConfig,
						currentFID,
						SM_SDO_TYPE_U32,
						&simArray[dataIndex].partitionCount,
						sizeof(u32),
						TRUE);
				}
			}
		}
	}

	if (rstat != SM_STATUS_SUCCESS)
	{
		SysDbgMsg4(("SPSAddSDOSAMPArrayDiskSimData: " \
			"Add failed on FID(%lu) IDX(%lu) STAT(%ld)\n",
			currentFID,
			dataIndex,
			rstat));
	}

	return rstat;
} // SPSAddSDOSAMPArrayDiskSimData


//---------------------------------------------------------------------
// SamPopStorageObjChildSDODOCreate
//---------------------------------------------------------------------
void SamPopStorageObjChildSDODOCreate(void)
{
	SDOConfig *pSDConfig = NULL;
	DataObjHeader *pDOH = NULL;
	astring *pPhysDiskName = NULL;
	ObjID parentOID;
	ObjID ctlrOID;
	ObjID enclOID;
	ObjID arryOID;
	const u32 sizeDOH = MAX_DATA_OBJECT_SIZE;
	const u32 sizePhysDiskName = MAX_PATH_SIZE;
	u32 ii;
	u32 jj;
	u32 kk;
	u32 ll;
	u32 physDiskCount;
	u32 physDiskIndex;
	u32 virtDiskIndex;
	s32 rstat;
	u8 popID;


	// initialize data
	pPhysDiskName = NULL;

	// allocate room for physical disk name ("PhysicalDisk%d:%d" 
	// where first %d = Enclosure/channel number
	// and  second %d = Index of the array disk (aka physical disk)
	if ((pPhysDiskName = (astring *)SMAllocMem(sizePhysDiskName)) == NULL)
	{
		// out of memory, so just exit
		_throw _error1;
	}

	// allocate room for largest object
	if ((pDOH = (DataObjHeader *)SMAllocMem(sizeDOH)) == NULL)
	{
		// out of memory, so just exit
		_throw _error2;
	}

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// insert all objects under the storage OID
	parentOID = SamPopDOStorageGetOID(popID);

	// counter for simulated enclosures
	kk = 0;
	// counter for physical disks
	physDiskIndex = 0;
	// counter for virtual disks
	virtDiskIndex = 0;

	// insert all controllers into the system
	for (ii = 0; ii < SAMP_NUM_CONTROLLERS; ii++)
	{
		// good multithreaded programs should be
		// aware of multithreaded nature
		if (SamPopGlobalsGetExitingFlag() == TRUE)
		{
			break;
		}

		// generate OID for controller
		ctlrOID = SamPopGetStorageObjChildDOGetOID(popID, 
			DATA_OBJ_TYPE_SAMP_SDO_CONTROLLER, 
			(u8)ii);

		// setup header
		memset(pDOH, 0, sizeDOH);
		SAMPopSetupObjDefaultHeader(&ctlrOID, 
			pDOH);

		// free the old SDO Config object
		if (pSDConfig != NULL)
		{
			SMSDOConfigFree(pSDConfig);
		}
		// allocate new SDO Config object
		if ((pSDConfig = SMSDOConfigAlloc()) == NULL)
		{
			// out of memory, so just exit
			_throw _error3;
		}
		
		// populate SDO with SAMPControllerSimData
		rstat = SPSAddSDOSAMPControllerSimData(pSDConfig, ii);
		if (rstat != SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
				"SPSAddSDOSAMPControllerSimData failed status(%ld)\n", 
				rstat));

			// skip this controller
			continue;
		}

		// insert controller SDO into DOH
		rstat = SPSInsertSDOToDOH(pSDConfig, pDOH, sizeDOH);
		if (rstat != SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
				"SPSInsertSDOToDOH failed status(%ld)\n", 
				rstat));

			// skip this controller
			continue;
		}

		// insert data object into data manager
		if (PopDPDMDDataObjCreateSingle(pDOH, 
			&parentOID) != SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
				"failed to create controller OID: %08X\n", 
				ctlrOID.ObjIDUnion.asu32));

			// skip this controller
			continue;
		}

		// create its channels
		for (jj = 0; jj < simController[ii].numChannels; jj++)
		{
			// good multithreaded programs should be
			// aware of multithreaded nature
			if (SamPopGlobalsGetExitingFlag() == TRUE)
			{
				break;
			}

			// generate OID for controller
			enclOID = SamPopGetStorageObjChildDOGetOID(popID, 
				DATA_OBJ_TYPE_SAMP_SDO_ENCLOSURE, 
				(u8)kk);
			
			// setup header
			memset(pDOH, 0, sizeDOH);
			SAMPopSetupObjDefaultHeader(&enclOID, 
				pDOH);
			
			// free the old SDO Config object
			if (pSDConfig != NULL)
			{
				SMSDOConfigFree(pSDConfig);
			}
			// allocate new SDO Config object
			if ((pSDConfig = SMSDOConfigAlloc()) == NULL)
			{
				// out of memory, so just exit
				_throw _error3;
			}
			
			// populate SDO with SAMPEnclosureSimData
			rstat = SPSAddSDOSAMPEnclosureSimData(pSDConfig, ii);
			if (rstat != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
					"SPSAddSDOSAMPEnclosureSimData failed status(%ld)\n", 
					rstat));
				
				// skip this controller
				continue;
			}
			
			// insert controller SDO into DOH
			rstat = SPSInsertSDOToDOH(pSDConfig, pDOH, sizeDOH);
			if (rstat != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
					"SPSInsertSDOToDOH failed status(%ld)\n", 
					rstat));
				
				// skip this controller
				continue;
			}
			
			// insert data object into data manager
			if (PopDPDMDDataObjCreateSingle(pDOH, 
				&ctlrOID) != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
					"failed to create enclosure OID: %08X\n", 
					enclOID.ObjIDUnion.asu32));
				
				// skip this controller
				continue;
			}			

			// reset data
			physDiskCount = 0;

			// create array disks for this enclosure
			for (ll = 0; ll < simEnclosure[kk].numArrays; ll++)
			{
				// good multithreaded programs should be
				// aware of multithreaded nature
				if (SamPopGlobalsGetExitingFlag() == TRUE)
				{
					break;
				}
				
				// generate OID for controller
				arryOID = SamPopGetStorageObjChildDOGetOID(popID, 
					DATA_OBJ_TYPE_SAMP_SDO_ARRAYDISK, 
					(u8)physDiskIndex);
				
				// setup header
				memset(pDOH, 0, sizeDOH);
				SAMPopSetupObjDefaultHeader(&arryOID, 
					pDOH);
				
				// object is assumed to be cached and a leaf object
				pDOH->objFlags = DATA_OBJ_FLAG_STATUS_POPMNGD;

				// construct physical disk name
				SMsnprintf(pPhysDiskName,
					sizePhysDiskName,
					"%s%d:%d",
					"PhysicalDisk",
					jj,
					physDiskCount);

				// free the old SDO Config object
				if (pSDConfig != NULL)
				{
					SMSDOConfigFree(pSDConfig);
				}
				// allocate new SDO Config object
				if ((pSDConfig = SMSDOConfigAlloc()) == NULL)
				{
					// out of memory, so just exit
					_throw _error3;
				}
				
				// populate SDO with SAMPArrayDiskSimData
				rstat = SPSAddSDOSAMPArrayDiskSimData(pSDConfig, ii, pPhysDiskName);
				if (rstat != SM_STATUS_SUCCESS)
				{
					SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
						"SPSAddSDOSAMPArrayDiskSimData failed status(%ld)\n", 
						rstat));
					
					// skip this controller
					continue;
				}
				
				// insert controller SDO into DOH
				rstat = SPSInsertSDOToDOH(pSDConfig, pDOH, sizeDOH);
				if (rstat != SM_STATUS_SUCCESS)
				{
					SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
						"SPSInsertSDOToDOH failed status(%ld)\n", 
						rstat));
					
					// skip this controller
					continue;
				}
				
				// insert data object into data manager
				if (PopDPDMDDataObjCreateSingle(pDOH, 
					&enclOID) != SM_STATUS_SUCCESS)
				{
					SysDbgMsg3(("SamPopStorageObjChildSDODOCreate: " \
						"failed to create arrayDisk OID: %08X\n", 
						arryOID.ObjIDUnion.asu32));
					
					// skip this controller
					continue;
				}			

				// bump the physical disk count for this enclosure
				physDiskCount++;

				// bump the physical disk index
				physDiskIndex++;
			}// for (ll...

			// bump the enclosure index
			kk++;
		} // for (jj...
	} // for (ii...


 _error3:
	if (pSDConfig != NULL)
	{
		SMSDOConfigFree(pSDConfig);
		pSDConfig = NULL;
	}
	SMFreeMem(pDOH);
	pDOH = NULL;
 _error2:
	SMFreeMem(pPhysDiskName);
	pPhysDiskName = NULL;
 _error1:
	return;
} // SamPopStorageObjChildSDODOCreate


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

