
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 2 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamPopStorageObjGet - Fill into a data object buffer a 
// Chasis Object type given that the Data Object buffer OID is filled
// by the caller
//---------------------------------------------------------------------
s32 SamPopStorageObjGet(_INOUT DataObjHeader *pDOH, 
						_INOUT u32 *pDOHBufSize)
{
	u32 sizeRequired;


	// compute size required
	sizeRequired = (sizeof(ObjHeader));

	// sanity check
	if (*pDOHBufSize < sizeRequired)
	{
		*pDOHBufSize = 0;
		return SM_STATUS_DATA_OVERRUN;
	}

	// NOTE: assume OID is setup by caller

	// set object size
	pDOH->objSize = sizeRequired;

	// set object type
	pDOH->objType = DATA_OBJ_TYPE_STORAGE_SVCS_LO;

	// there is no status initally but its
	// children will propagate status into it
	pDOH->objStatus = OBJ_STATUS_UNKNOWN;

	// object is cached
	pDOH->objFlags = OBJ_FLAG_NONE;

	// this object is not refreshed
	pDOH->refreshInterval = OBJ_REFRESH_NONE;

	// reserved aligns to zero
	memset(pDOH->reservedAlign, 
		0, 
		sizeof(pDOH->reservedAlign));

	// set returned size
	*pDOHBufSize = sizeRequired;

	return SM_STATUS_SUCCESS;
} // SamPopStorageObjGet


//---------------------------------------------------------------------
// PopRefreshChassisHipObj - Implements any refresh required on 
// Chasis Object type given the hip object
//---------------------------------------------------------------------
s32 PopRefreshChassisHipObj(_IN const DataObjHeader *pIHO, 
							_OUT DataObjHeader *pOHO, 
							_IN u32 objSize, 
							_OUT u32 *pBytesReturned)
{

	SysDbgMsg3(("PopRefreshChassisHipObj: no refresh applicable on OID: %08X\n", 
		pIHO->objHeader.objID.ObjIDUnion.asu32));

	// nothing returned
	*pBytesReturned = 0;

	return SM_STATUS_CMD_NOT_IMPLEMENTED;
} // PopRefreshChassisHipObj


//---------------------------------------------------------------------
// PopSetChassisHipObj - Implements any sets required on chassis obj
//---------------------------------------------------------------------
s32 PopSetChassisHipObj(_IN const SetReq *pSR, 
						_IN u32 setSize, 
						_INOUT DataObjHeader *pDOH, 
						_IN u32 objSize, 
						_OUT u32 *pBytesReturned)
{

	SysDbgMsg3(("PopSetChassisHipObj: set type: %lu inapplicable on OID: %08X\n", 
		pSR->type, 
		pSR->objID.ObjIDUnion.asu32));

	// nothing returned
	*pBytesReturned = 0;

	return SM_STATUS_CMD_NOT_IMPLEMENTED;
} // PopSetChassisHipObj


//---------------------------------------------------------------------
// PopPassThruChassisHipObj - Implements any passthru required on chassis obj
//---------------------------------------------------------------------
s32 PopPassThruChassisHipObj(_IN const SMApiReq *pI, 
							 _IN u32 inBufSize, 
							 _OUT SMApiResp *pO, 
							 _IN u32 outBufSize, 
							 _OUT u32 *pBytesReturned)
{

	SysDbgMsg3(("PopPassThruChassisHipObj: passthru ptCmd: %lu inapplicable on OID: %08X\n", 
		pI->CmdData.objPassThru.ptCmd, 
		pI->CmdData.objPassThru.objID.ObjIDUnion.asu32));

	// nothing returned
	*pBytesReturned = 0;

	return SM_STATUS_CMD_NOT_IMPLEMENTED;
} // PopPassThruChassisHipObj

