
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 4 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------



//---------------------------------------------------------------------
//  Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamPopDOStorageGetOID
//---------------------------------------------------------------------
ObjID SamPopDOStorageGetOID(_IN u8 popID)
{
	ObjID oidTemp;


	// To represent the storage OID
	// object we use the lowest OID
	// number that this Data Populator
	// can generate which is the popID ORed with 0

	oidTemp.ObjIDUnion.asu32 = DATA_OBJ_OID_INVALID;
	oidTemp.ObjIDUnion.InnerObjIDStruct.popID = popID;

	// done
	return oidTemp;
} // SamPopDOStorageGetOID


//---------------------------------------------------------------------
// SamPopDOStorageGetObj - Fill into a data object buffer a 
// Chasis Object type given that the Data Object buffer OID is filled
// by the caller
//---------------------------------------------------------------------
s32 SamPopDOStorageGetObj(_IN const ObjID *pOID, 
						  _OUT DataObjHeader *pDOH, 
						  _INOUT u32 *pDOHBufSize)
{
	SAMPStorageDataObj *pSDO;
	u32 sizeRequired;


	// compute size required
	sizeRequired = (sizeof(SAMPStorageDataObj));

	// sanity check
	if (*pDOHBufSize < sizeRequired)
	{
		*pDOHBufSize = 0;
		return SM_STATUS_DATA_OVERRUN;
	}

	// cast
	pSDO = (SAMPStorageDataObj *)pDOH;

	// setup header

	// set OID
	pSDO->doh.objID = *pOID;

	// set object size
	pDOH->objSize = sizeRequired;

	// set object type
	pDOH->objType = DATA_OBJ_TYPE_SAMP_STORAGE;

	// there is no status initally but its
	// children will propagate status into it
	pDOH->objStatus = DATA_OBJ_STATUS_UNKNOWN;

	// object is cached
	pDOH->objFlags = DATA_OBJ_FLAG_NONE;

	// this object is not refreshed
	pDOH->refreshInterval = DATA_OBJ_REFRESH_NONE;

	// reserved aligns to zero
	memset(pDOH->reservedAlign, 
		0, 
		sizeof(pDOH->reservedAlign));

	// setup body

	// setup versions
	pSDO->majorVersion = PRD_VER_PRODUCTVERSION;
	pSDO->minorVersion = PRD_VER_PRODUCTREVISION;
	pSDO->maintanenceVersion = PRD_VER_PRODUCTMAINTAIN;

	// set returned size
	*pDOHBufSize = sizeRequired;

	return SM_STATUS_SUCCESS;
} // SamPopDOStorageGetObj



