
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 17 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>
#include <sampopdef.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------
#define DEV_PCI_OBJ_TYPE				(0xE6) // DevPCI - PCI Device


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------

// simulated controller data
const SAMPControllerSimData simController[SAMP_NUM_CONTROLLERS] = 
{
	{ "TinyWidget Technologies", "BasicAdapter", DOTSC_STATE_ALIVE, 1, 2 }, 
	{ "BigWidget Corporation", "AdvancedAdapter", DOTSC_STATE_DEAD, 4, 4 }, 
};

// simulated enclosure data
const SAMPEnclosureSimData simEnclosure[SAMP_NUM_ENCLOSURES] = 
{
	{ "Enclosure 1", 0, 3, 3, 0, 1 }, 
	{ "Enclosure 2", 0, 2, 2, 3, 0 }, 
	{ "Enclosure 3", 1, 2, 2, 3, 0 }, 
	{ "Enclosure 4", 2, 5, 3, 1, 1 }, 
	{ "Enclosure 5", 2, 1, 3, 1, 1 }, 
};

// simulated array disk data
const SAMPArrayDiskSimData simArray[] = 
{
	{ "", "Dell Disk Mfr",	 3 }, 
	{ "", "Dell RR5 Mfr",	 2 }, 
	{ "", "Dell India Mfr",	 5 }, 
	{ "", "Dell Austin Mfr", 1 }, 
	{ "", "Dell Brazil Mfr", 4 }, 
};

// simulated virtual disk data
const SAMPVirtDiskSimData simVirt[] = 
{
	{ "VirtDisk 1" }, 
	{ "VirtDisk 2" }, 
	{ "VirtDisk 3" }, 
	{ "VirtDisk 4" }, 
	{ "VirtDisk 5" }, 
};


//---------------------------------------------------------------------
// Code Start
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// SamPopIsManagedComponentPresent
//---------------------------------------------------------------------
booln SamPopIsManagedComponentPresent(void)
{
	// since there is no specific hardware adpater (or controller)
	// that we manage in this sample return TRUE
	// so that we remain loaded to perform sample routines
	// however this is where we would determine if any
	// controller that we support is present in this system
	// and return TRUE if present
	return TRUE;
} // SamPopIsManagedComponentPresent


//---------------------------------------------------------------------
// SamPopAttachThreads
//---------------------------------------------------------------------
s32 SamPopAttachThreads(void)
{
	s32 status;


	SysDbgMsg4(("SamPopAttachThreads: entry\n"));

	// set exiting flag
	SamPopGlobalsSetExitingFlag(FALSE);

	// At this point all load time objects have
	// been loaded into the data manager by
	// all populators and it this data populator
	// can begin monitoring the managed component
	// additionally the Data Populator can lookup
	// data objects within the data manager and
	// generate events based on managed component
	// status and other state changes
	if ((status = SamPopThreadStart()) != SM_STATUS_SUCCESS)
	{
		return status;
	}

	// start the STOR Req Stress Testing Thread
	if ((status = SamPopReqTestThreadStart()) != SM_STATUS_SUCCESS)
	{
		return status;
	}

	// done
	SysDbgMsg4(("SamPopAttachThreads: exit\n"));
	return SM_STATUS_SUCCESS;
} // SamPopAttachThreads


//---------------------------------------------------------------------
// SamPopDetachThreads
//---------------------------------------------------------------------
s32 SamPopDetachThreads(void)
{

	SysDbgMsg4(("SamPopDetachThreads: entry\n"));

	// we have to make sure that all threads
	// are really stopped by now as the 
	// push interface environment is
	// no longer functional

	// set exiting flag
	SamPopGlobalsSetExitingFlag(TRUE);

	// stop all worker threads
	SamPopThreadStop();

	// stop the STOR Req Stress Testing Thread
	SamPopReqTestThreadStop();

	// done
	SysDbgMsg4(("SamPopDetachThreads: exit\n"));
	return SM_STATUS_SUCCESS;
} // SamPopDetachThreads


//---------------------------------------------------------------------
// SamPopGetStorageObjChildDOGetOID
//---------------------------------------------------------------------
ObjID SamPopGetStorageObjChildDOGetOID(_IN u8 popID, 
									   _IN u16 objType, 
									   _IN u8 instance)
{
	ObjID oidTemp;


	// clear OID
	oidTemp.ObjIDUnion.asu32 = DATA_OBJ_OID_INVALID;

	// create the OID using fixed mapping
	// where the object type, instance and populator id
	// are used to provide unique numbers
	oidTemp.ObjIDUnion.ObjIDTypeInstStruct.objType = objType;

	oidTemp.ObjIDUnion.ObjIDTypeInstStruct.objInst = instance;

	oidTemp.ObjIDUnion.ObjIDTypeInstStruct.popID = popID;

	return oidTemp;
} // SamPopGetStorageObjChildDOGetOID


//---------------------------------------------------------------------
// SAMPopSetupObjDefaultHeader
//---------------------------------------------------------------------
void SAMPopSetupObjDefaultHeader(_IN const ObjID *pOID, 
								_OUT DataObjHeader *pDOH)
{

	// set OID
	pDOH->objID = *pOID;
	
	// set object size
	pDOH->objSize = sizeof(DataObjHeader);

	// setup object type
	// here we use the fact that
	// we embeded the object type in the OID
	pDOH->objType = pOID->ObjIDUnion.ObjIDTypeInstStruct.objType;

	// setup status
	pDOH->objStatus = DATA_OBJ_STATUS_OK;

	// object is assumed to be not a leaf object
	pDOH->objFlags = DATA_OBJ_FLAG_NONE;

	// this object is not refreshed
	pDOH->refreshInterval = DATA_OBJ_REFRESH_NONE;

	// reserved aligns to zero
	memset(pDOH->reservedAlign, 
		0, 
		sizeof(pDOH->reservedAlign));
} // SAMPopSetupObjDefaultHeader


//---------------------------------------------------------------------
// SAMPopSetupEvtDefaultHeader
//---------------------------------------------------------------------
void SAMPopSetupEvtDefaultHeader(_OUT DataEventHeader *pDEH, 
								 _IN u32 evtSize, 
								 _IN u16 evtType, 
								 _IN u8 evtFlags)
{

	
	// set event size
	if (evtSize == 0)
	{
		// default required
		pDEH->evtSize = MIN_DATA_EVENT_SIZE;
	}
	else
	{
		pDEH->evtSize = evtSize;
	}

	// setup event type
	pDEH->evtType = evtType;

	// setup status
	pDEH->evtFlags = (DATA_EVENT_FLAG_NONE | evtFlags);
} // SAMPopSetupEvtDefaultHeader


//---------------------------------------------------------------------
// SamPopStorageObjChildDOCreate
//---------------------------------------------------------------------
void SamPopStorageObjChildDOCreate(void)
{
	DataObjHeader *pDOH;
	SAMPControllerDataObj *pSCDO;
	SAMPEnclosureDataObj *pSEDO = NULL;
	SAMPArrayDiskDataObj *pSADDO;
	SAMPVirtDiskDataObj *pSVDDO;
	astring *pPhysDiskName;
	ObjID parentOID;
	ObjID ctlrOID;
	ObjID enclOID;
	ObjID arryOID;
	ObjID virtOID;
	ObjList *pOL;
	u32 doBufSize;
	u32 ii;
	u32 jj;
	u32 kk;
	u32 ll;
	u32 mm;
	u32 physDiskCount;
	u32 physDiskIndex;
	u32 virtDiskIndex;
	u32 size;
	u32 sizeObjList;
	u32 bytesWrittenToOutBuf;
	s32 status;
	s32 listType;
	u8 popID;

	// initialize data
	pPhysDiskName = NULL;
	pOL = NULL;

	// allocate room for physical disk name ("PhysicalDisk%d:%d" 
	// where first %d = Enclosure/channel number
	// and  second %d = Index of the array disk (aka physical disk)
	size = MAX_PATH_SIZE;
	if ((pPhysDiskName = (astring *)SMAllocMem(size)) == NULL)
	{
		// this can happen!
		return;
	}

	// allocate room for largest object
	doBufSize = MAX_DATA_OBJECT_SIZE;
	if ((pDOH = (DataObjHeader *)SMAllocMem(doBufSize)) == NULL)
	{
		// this can happen!
		return;
	}

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// insert all objects under the storage OID
	parentOID = SamPopDOStorageGetOID(popID);

	// counter for simulated enclosures
	kk = 0;
	// counter for physical disks
	physDiskIndex = 0;
	// counter for virtual disks
	virtDiskIndex = 0;


	// insert all controllers into the system
	for (ii = 0; ii < SAMP_NUM_CONTROLLERS; ii++)
	{
		// good multithreaded programs should be
		// aware of multithreaded nature
		if (SamPopGlobalsGetExitingFlag() == TRUE)
		{
			break;
		}

		// generate OID
		ctlrOID = SamPopGetStorageObjChildDOGetOID(popID, 
			DATA_OBJ_TYPE_SAMP_CONTROLLER, 
			(u8)ii);

		// setup a controller object
		pSCDO = (SAMPControllerDataObj *)pDOH;

		// setup header
		SAMPopSetupObjDefaultHeader(&ctlrOID, 
			&pSCDO->doh);

		// setup initial body size
		pSCDO->doh.objSize = sizeof(SAMPControllerDataObj);

		// insert vendor name
		PopDPDMDDOAppendUTF8Str(pDOH, 
			&doBufSize, 
			&pSCDO->offsetVendorName, 
			simController[ii].pVendorName);

		// insert device name
		PopDPDMDDOAppendUTF8Str(pDOH, 
			&doBufSize, 
			&pSCDO->offsetDeviceName, 
			simController[ii].pDeviceName);

		// setup state
		pSCDO->state = simController[ii].state;

		// insert data object into data manager
		if (PopDPDMDDataObjCreateSingle(pDOH, 
			&parentOID) != SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopStorageObjChildDOCreate: failed to create controller OID: %08X\n", 
				ctlrOID.ObjIDUnion.asu32));

			// skip this controller
			continue;
		}

		// clear the object list before
		// creating channels and array disks for
		// this controller
		if (pOL != NULL)
		{
			// free Object List
			SMFreeMem(pOL);
			// catch GPFs
			pOL = NULL;
		}


		// create its channels
		for (jj = 0; jj < simController[ii].numChannels; jj++)
		{
			// good multithreaded programs should be
			// aware of multithreaded nature
			if (SamPopGlobalsGetExitingFlag() == TRUE)
			{
				break;
			}

			// generate OID
			enclOID = SamPopGetStorageObjChildDOGetOID(popID, 
				DATA_OBJ_TYPE_SAMP_ENCLOSURE, 
				(u8)kk);

			// setup enclosure
			pSEDO = (SAMPEnclosureDataObj *)pDOH;

			// setup header
			SAMPopSetupObjDefaultHeader(&enclOID, 
				&pSEDO->doh);

			// setup initial body size
			pSEDO->doh.objSize = sizeof(SAMPEnclosureDataObj);

			// insert vendor name
			PopDPDMDDOAppendUTF8Str(pDOH, 
				&doBufSize, 
				&pSEDO->offsetEnclName, 
				simEnclosure[kk].pEnclName);

			// setup other data
			pSEDO->channelNum = jj;
			pSEDO->firmwareMajorVersion = 
				simEnclosure[kk].firmwareMajorVersion;
			pSEDO->firmwareMinorVersion = 
				simEnclosure[kk].firmwareMinorVersion;
			pSEDO->firmwareMaintanenceVersion = 
				simEnclosure[kk].firmwareMaintanenceVersion;
			pSEDO->state = DOTSE_STATE_RUNNING;

			// insert data object into data manager
			if (PopDPDMDDataObjCreateSingle(pDOH, 
				&ctlrOID) != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("SamPopStorageObjChildDOCreate: failed to create enclosure OID: %08X\n", 
					enclOID.ObjIDUnion.asu32));

				// skip this enclosure
				continue;
			}

			// reset data
			physDiskCount = 0;
			sizeObjList = 0;

			// create array disks for this enclosure
			for (ll = 0; ll < simEnclosure[kk].numArrays; ll++)
			{
				// good multithreaded programs should be
				// aware of multithreaded nature.
				if (SamPopGlobalsGetExitingFlag() == TRUE)
				{
					break;
				}
				
				// generate OID
				arryOID = SamPopGetStorageObjChildDOGetOID(popID, 
					DATA_OBJ_TYPE_SAMP_ARRAYDISK, 
					(u8)physDiskIndex);
				
				// setup array disk
				pSADDO = (SAMPArrayDiskDataObj *)pDOH;
				
				// setup header
				SAMPopSetupObjDefaultHeader(&arryOID, 
					&pSADDO->doh);
				
				// object is assumed to be cached and a leaf object
				pSADDO->doh.objFlags = DATA_OBJ_FLAG_STATUS_POPMNGD;

				// setup initial body size
				pSADDO->doh.objSize = sizeof(SAMPArrayDiskDataObj);

				// set pPhysDiskName to empty
				pPhysDiskName[0] = '\0';

				// construct physical disk name
				SMsnprintf(pPhysDiskName,
					size,
					"%s%d:%d",
					"PhysicalDisk",
					jj,
					physDiskCount);
				
				
				// insert physical disk name/array disk name
				PopDPDMDDOAppendUTF8Str(pDOH, 
					&doBufSize, 
					&pSADDO->offsetArrayDiskName, 
					pPhysDiskName);
				
				// insert array disk vendor
				PopDPDMDDOAppendUTF8Str(pDOH, 
					&doBufSize, 
					&pSADDO->offsetVendorName, 
					simArray[ll].pVendorName);
				
				
				// setup other data
				pSADDO->state = DOTSAD_STATE_OK;
				pSADDO->partitionCount = simArray[ll].partitionCount;
				
				// insert data object into data manager
				if (PopDPDMDDataObjCreateSingle(pDOH, 
					&enclOID) != SM_STATUS_SUCCESS)
				{
					SysDbgMsg3(("SamPopStorageObjChildDOCreate: failed to create arrayDisk OID: %08X\n", 
						arryOID.ObjIDUnion.asu32));
					
					// skip this array disk
					continue;
				}
				

				// bump the physical disk count for this enclosure
				physDiskCount++;

				if (pOL == NULL)
				{
					// first time
					sizeObjList = sizeof(ObjList);

					pOL = (ObjList *)SMAllocMem(sizeObjList);
					// clear the list contents
					status = PopDPDMDObjListClear(pOL, 
						&sizeObjList, 
						&bytesWrittenToOutBuf);
				}
				else
				{
					sizeObjList = sizeof(pOL->objCount) + 
						((pOL->objCount + 1) * sizeof(pOL->objID[0]));
					pOL = (ObjList *)SMReAllocMem(pOL, sizeObjList);
				}

				// sanity check
				if (pOL != NULL)
				{
					// capture a list of array disks created
					// This will be used later for Virtual Disk Obj creation
					// In practical implementation, the user will specify
					// a list of Disk Objects to be used for a Virtual Object
					// We will have to check if the list belongs to the same Ctrl Num?
					status = PopDPDMDObjListAppendOID(&arryOID,
						pOL,
						&sizeObjList,
						&bytesWrittenToOutBuf);

					if (status == SM_STATUS_DATA_OVERRUN)
					{
						sizeObjList = bytesWrittenToOutBuf;
						pOL = (ObjList *)SMReAllocMem(pOL, sizeObjList);

						// try again
						status = PopDPDMDObjListAppendOID(&arryOID,
							pOL,
							&sizeObjList,
							&bytesWrittenToOutBuf);
					}
				}

				// bump the physical disk index
				physDiskIndex++;
			}// for (ll...

			// bump the enclosure index
			kk++;
		} // for (jj...

		// create the virtual disks
		// under a controller
		for (mm = 0; mm < simController[ii].numVirtDisks; mm++)
		{
			// good multithreaded programs should be
			// aware of multithreaded nature
			if (SamPopGlobalsGetExitingFlag() == TRUE)
			{
				break;
			}

			// generate OID
			virtOID = SamPopGetStorageObjChildDOGetOID(popID, 
				DATA_OBJ_TYPE_SAMP_VIRTDISK,
				(u8)virtDiskIndex);

			// setup virtual disk
			pSVDDO = (SAMPVirtDiskDataObj *)pDOH;

			// setup header
			SAMPopSetupObjDefaultHeader(&virtOID, 
				&pSVDDO->doh);

			// setup initial body size
			pSVDDO->doh.objSize = sizeof(SAMPVirtDiskDataObj);

			// insert virtual disk name
			PopDPDMDDOAppendUTF8Str(pDOH, 
				&doBufSize, 
				&pSVDDO->offsetVirtDiskName,
				simVirt[mm].pVirtDiskName);

			// setup other data

			// NOTE:: state should be cumulative of all array disks
			// at this stage itself?
			if(pSEDO != NULL)
			{
				pSEDO->state = DOTSVD_STATE_OK;
			}

			// insert data object into data manager
			if (PopDPDMDDataObjCreateSingle(pDOH, 
				&ctlrOID) != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("SamPopStorageObjChildDOCreate: failed to create virtual OID: %08X\n", 
					virtOID.ObjIDUnion.asu32));

				// skip this virtual disk
				continue;
			}

			// Create MPMC relationship.
			// For simplicity reasons, we will consider each virtual disk obj
			// containing all possible Array disks under ctrl(ii).
			
			listType = DPDMD_OID_LIST_TYPE_CHILD;

			// check status??
			status = PopDPDMDDOBranchCreateMultiple(pOL,
				&listType,
				&virtOID);

			// bump up the virtual disk index
			virtDiskIndex++;
		} // for (mm...
	} // for (ii...

	// free the memory
	SMFreeMem(pDOH);
	// catch GPFs
	pDOH = NULL;
	
	if (pPhysDiskName != NULL)
	{
		// free physicalDiskName memory
		SMFreeMem(pPhysDiskName);
		// catch GPFs
		pPhysDiskName = NULL;
	}

	if (pOL != NULL)
	{
		// free Object List
		SMFreeMem(pOL);
		// catch GPFs
		pOL = NULL;
	}
} // SamPopStorageObjChildDOCreate


//---------------------------------------------------------------------
// SamPopTimerDOCreate
//---------------------------------------------------------------------
void SamPopTimerDOCreate(void)
{
	DataObjHeader *pDOH;
	SAMPTimerDataObj *pSTDO;
	ObjID parentOID;
	ObjID tmrOID;
	time_t curUTCTime;
	u32 doBufSize;
	u8 popID;


	// allocate room for largest object
	doBufSize = MAX_DATA_OBJECT_SIZE;
	if ((pDOH = (DataObjHeader *)SMAllocMem(doBufSize)) == NULL)
	{
		// this can happen!
		return;
	}

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// insert all objects under the storage OID
	parentOID = SamPopDOStorageGetOID(popID);

	// generate OID
	tmrOID = SamPopGetStorageObjChildDOGetOID(popID, 
		DATA_OBJ_TYPE_SAMP_TIMER, 
		0);

	// get start time
	time(&curUTCTime);

	// setup a timer object
	pSTDO = (SAMPTimerDataObj *)pDOH;

	// setup header
	SAMPopSetupObjDefaultHeader(&tmrOID, 
		&pSTDO->doh);

	// setup initial body size
	pSTDO->doh.objSize = sizeof(SAMPTimerDataObj);

	// NOTE: We would like this object to 
	// be refreshed approximately every 5 seconds
	pSTDO->doh.refreshInterval = DATA_OBJ_REFRESH_T1;

	// setup start time
	pSTDO->startTimeFromEPOCH = (s64)curUTCTime;

	// setup minute count
	pSTDO->numMinutesFromStart = 0;

	// setup tick count
	pSTDO->tickCount = 0;

	// insert data object into data manager
	if (PopDPDMDDataObjCreateSingle(pDOH, 
		&parentOID) != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("SamPopTimerDOCreate: failed to create timer OID: %08X\n", 
			pDOH->objID.ObjIDUnion.asu32));

		SMFreeMem(pDOH);

		// failed
		return;
	}

	// free the memory
	SMFreeMem(pDOH);
	// catch GPFs
	pDOH = NULL;
} // SamPopTimerDOCreate


//---------------------------------------------------------------------
// SamPopSubmitPopToConsEvent
//---------------------------------------------------------------------
void SamPopSubmitPopToConsEvent(_IN SAMPTimerDataObj *pSTDO)
{
	SAMPPopToConDataEvt *pSPTCDE;
	u32 maxDESize;


	// allocate memory for this type of event
	if ((pSPTCDE = (SAMPPopToConDataEvt *)PopDPDMDAllocDataEvent(&maxDESize)) == NULL)
	{
		// this can happen!
		return;
	}

	// setup the pop to consumer event

	// setup header
	SAMPopSetupEvtDefaultHeader(&pSPTCDE->deh, 
		sizeof(SAMPPopToConDataEvt), 
		DATA_EVT_TYPE_SAMP_POP_TO_CON, 
		DATA_EVENT_FLAG_POP_TO_CONSUMER);

	// setup body
	pSPTCDE->numMinutesFromStart = pSTDO->numMinutesFromStart;
	pSPTCDE->startTimeFromEPOCH = pSTDO->startTimeFromEPOCH;

	// submit event to data manager
	if (PopDPDMDDESubmitSingle(&pSPTCDE->deh) != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("SamPopSubmitPopToConsEvent: failed to submit event\n"));
		// free the memory
		PopDPDMDFreeGeneric(pSPTCDE);
		// catch GPFs
		pSPTCDE = NULL;
		return;
	} // if (PopDPDMDDESubmitSingle(&pSPTCDE->deh) != SM_STATUS_SUCCESS)

	// free the memory
	PopDPDMDFreeGeneric(pSPTCDE);
	// catch GPFs
	pSPTCDE = NULL;
} // SamPopSubmitPopToConsEvent


//---------------------------------------------------------------------
// SamPopStorageObjChildDORefresh
//---------------------------------------------------------------------
s32 SamPopStorageObjChildDORefresh(_INOUT DataObjHeader *pDOH, 
								   _INOUT u32 *pDOHBufSize)
{
	SAMPTimerDataObj *pSTDO;


	// we support refresh only on 
	// certain types of data objects
	switch (pDOH->objType)
	{
		case DATA_OBJ_TYPE_SAMP_TIMER:
			// setup a timer object
			pSTDO = (SAMPTimerDataObj *)pDOH;

			// increment the tick count
			pSTDO->tickCount++;

			// reached a minute?
			if (pSTDO->tickCount >= 12)
			{
				// bump minute count
				pSTDO->numMinutesFromStart++;

				// bump tick count
				pSTDO->tickCount = 0;

				// also generate an populator to consumer
				// event to demostrate sample events
				// and embeded the start time and
				// numMinutesFromStart
				SamPopSubmitPopToConsEvent(pSTDO);
			} // if (pSTDO->tickCount >= 12)
		break;

		default:
			SysDbgMsg3(("SamPopStorageObjChildDORefresh: invalid refreshInterval setting on OID: %08X\n",
				pDOH->objID.ObjIDUnion.asu32));
			// nothing returned
			*pDOHBufSize = 0;
			// failed
			return SM_STATUS_OBJECT_BAD;
	} // switch (pDOH->objType)

	// set returned size
	*pDOHBufSize = pDOH->objSize;

	return SM_STATUS_SUCCESS;	
} // SamPopStorageObjChildDORefresh


//---------------------------------------------------------------------
// SamPopProcessTimer
//---------------------------------------------------------------------
void SamPopProcessTimer(void)
{
	SAMPArrayDiskDataObj *pSADDO;
	ObjList *pOL;
	ObjID parentOID;
	u32 ii;
	u8 popID;


	// every time we get called here
	// we lookup a enclosure object and
	// flip its status and state

	// get populator ID
	popID = PopDPDMDGetPopulatorID();

	// get enclosure objects under the storage OID
	parentOID = SamPopDOStorageGetOID(popID);

	// remember to free the memory
	pOL = PopDPDMDListChildOIDByType(&parentOID, 
		DATA_OBJ_TYPE_SAMP_ARRAYDISK);

	// not present?
	if ((pOL == NULL) || 
		(pOL->objCount == 0))
	{
		SysDbgMsg3(("SamPopProcessTimer: failed to find physical disks\n"));

		// recieved memory?
		if (pOL != NULL)
		{
			PopDPDMDFreeGeneric(pOL);
		}
		return;
	}

	// run through list
	// we do not check for
	// data manager exiting here since
	for (ii = 0; ii < pOL->objCount; ii++)
	{
		// good multithreaded programs should be
		// aware of multithreaded nature
		if (SamPopGlobalsGetExitingFlag() == TRUE)
		{
			break;
		}

		// get the object and remember to free
		pSADDO = (SAMPArrayDiskDataObj *)PopDPDMDGetDataObjByOID(&pOL->objID[ii]);

		// failed?
		if (pSADDO == NULL)
		{
			// skip this at it may 
			// have been deleted
			continue;
		}

		// flip status and state and refresh
		pSADDO->state++;
		if (pSADDO->state > DOTSAD_STATE_MAX)
		{
			pSADDO->state = DOTSAD_STATE_UNKNOWN;
		}

		// set status based on state
		switch (pSADDO->state)
		{
			case DOTSAD_STATE_FAILED:
				pSADDO->doh.objStatus = DATA_OBJ_STATUS_CRITICAL;
			break;

			case DOTSAD_STATE_DEGRADED:
				pSADDO->doh.objStatus = DATA_OBJ_STATUS_NON_CRITICAL;
			break;

			case DOTSAD_STATE_OK:
				pSADDO->doh.objStatus = DATA_OBJ_STATUS_OK;
			break;

			default:
				pSADDO->doh.objStatus = DATA_OBJ_STATUS_UNKNOWN;
			break;
		} // switch (pSADDO->state)

		// refresh the object in the data manager
		if (PopDPDMDDataObjRefreshSingle(&pSADDO->doh) 
			!= SM_STATUS_SUCCESS)
		{
			SysDbgMsg3(("SamPopProcessTimer: failed to refresh OID: %08X\n",
				pSADDO->doh.objID.ObjIDUnion.asu32));
		}
		
		// free object
		PopDPDMDFreeGeneric(pSADDO);
	} // for (ii...

	// free list
	PopDPDMDFreeGeneric(pOL);
} // SamPopProcessTimer


//---------------------------------------------------------------------
// SamPopGetDevPCIObjTypeFromMainChassis
//---------------------------------------------------------------------
s32 SamPopGetDevPCIObjTypeFromMainChassis(void)
{
	ObjID oid;
	ObjList *pObjList;
	s32 status;


	// set the oid to main chassis so we search the tree beneath it
	oid.ObjIDUnion.asu32 = DATA_OBJ_OID_MAIN_CHASSIS;
	
	// attempt to list DEV_PCI_OBJ_TYPE
	pObjList = PopDPDMDListChildOIDByType(&oid, 
		DEV_PCI_OBJ_TYPE);

	if (NULL == pObjList)
	{
		SysDbgMsg3(("SamPopGetDevPCIObjTypeFromMainChassis: failed to get list (pObjList == NULL)\n"));
		status = SM_STATUS_NO_SUCH_OBJECT;
	}
	else
	{
		if (0 == pObjList->objCount)
		{
			SysDbgMsg3(("SamPopGetDevPCIObjTypeFromMainChassis: failed to get objects (pObjList->objCount == 0)\n"));
			status = SM_STATUS_DEVICE_EMPTY;
		}
		else
		{
			// print the number of DEV_PCI_OBJ_TYPEs and leave
			SysDbgMsg3(("SamPopGetDevPCIObjTypeFromMainChassis: Number of DEV_PCI_OBJ_TYPEs objects: %d\n", 
				pObjList->objCount));
			// indicate success
			status = SM_STATUS_SUCCESS;
		}

		// free list
		PopDPDMDFreeGeneric(pObjList);
		// catch GFPs
		pObjList = NULL;
	}

	// return status
	return status;
} // SamPopGetDevPCIObjTypeFromMainChassis


//---------------------------------------------------------------------
// SamPopGetDevPCIObjTypeFromRoot
//---------------------------------------------------------------------
s32 SamPopGetDevPCIObjTypeFromRoot(void)
{
	ObjID oid;
	ObjList *pObjList;
	s32 status;


	// set the oid to root so we search the tree beneath it
	oid.ObjIDUnion.asu32 = DATA_OBJ_OID_ROOT;
	
	// attempt to list DEV_PCI_OBJ_TYPE
	pObjList = PopDPDMDListChildOIDByType(&oid, 
		DEV_PCI_OBJ_TYPE);

	if (NULL == pObjList)
	{
		SysDbgMsg3(("SamPopGetDevPCIObjTypeFromRoot: failed to get list (pObjList == NULL)\n"));
		status = SM_STATUS_NO_SUCH_OBJECT;
	}
	else
	{
		if (0 == pObjList->objCount)
		{
			SysDbgMsg3(("SamPopGetDevPCIObjTypeFromRoot: failed to get objects (pObjList->objCount == 0)\n"));
			status = SM_STATUS_DEVICE_EMPTY;
		}
		else
		{
			// print the number of DEV_PCI_OBJ_TYPEs and leave
			SysDbgMsg3(("SamPopGetDevPCIObjTypeFromRoot: Number of DEV_PCI_OBJ_TYPEs objects: %d\n", 
				pObjList->objCount));
			// indicate success
			status = SM_STATUS_SUCCESS;
		}

		// free list
		PopDPDMDFreeGeneric(pObjList);
		// catch GFPs
		pObjList = NULL;
	}

	// return status
	return status;
} // SamPopGetDevPCIObjTypeFromRoot


//---------------------------------------------------------------------
// SamPopGetDevPCIObjType
//---------------------------------------------------------------------
s32 SamPopGetDevPCIObjType(void)
{
	s32 status;
	s32 retCode;


	status = SM_STATUS_SUCCESS;
	retCode = SM_STATUS_SUCCESS;

	// check from Main Chassis down
	retCode = SamPopGetDevPCIObjTypeFromMainChassis();
	if (retCode != SM_STATUS_SUCCESS)
	{
		status = retCode;
	}

	// check from root down
	retCode = SamPopGetDevPCIObjTypeFromRoot();
	if (retCode != SM_STATUS_SUCCESS)
	{
		status = retCode;
	}

	// return status
	return status;
} // SamPopGetDevPCIObjType


//---------------------------------------------------------------------
// Code End
//---------------------------------------------------------------------

