
//--------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Inc. and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-98 Dell Inc.. All Rights Reserved.
// 
// Module Name:
//
// Support module to make coding a populator a matter of coding
// interfaces defined in popdisp.h
//
// Abstract/Purpose:
//
// This module contains common OS Independent code for SM Populators 
//
//
//  Environment:
//  Windows NT/2000, Linux, Netware
//
//  Version:
//  $Revision: 7 $
//
//
//  Last Modified by/on:
//  $Author: Tushar_oza $ / $Date: 7/22/03 5:12p $
//
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------


//---------------------------------------------------------------------
// Includes this file is dependent on
//---------------------------------------------------------------------
#include <sampop.h>


//--------------------------------------------------------------------
// Local Defines
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//---------------------------------------------------------------------
// SamPopThreadWorker
//---------------------------------------------------------------------
void _cdecl SamPopThreadWorker(_IN void *pSMThreadData)
{

	SysDbgMsg4(("SamPopThreadWorker: entry pSMThreadData: %p\n", 
		pSMThreadData));

	// create timer object
	// that will cause SM_DRT_POP_REFRESH_OBJ
	SamPopTimerDOCreate();

	// create all controller objects in this system
	SamPopStorageObjChildDOCreate();

	// create all controller objects in this system as SDO objects
	SamPopStorageObjChildSDODOCreate();

	// while the module or data manager not exiting
	// do something every second
	while ((SamPopGlobalsGetExitingFlag() == FALSE) && 
		(PopDPDMDIsDataManagerReady() == TRUE))
	{
		// process timer event
		SamPopProcessTimer();

		// sleep 1 second
		Sleep(1000);
	} // while

	SysDbgMsg4(("SamPopThreadWorker: exit\n"));

	UNREFERENCED_PARAMETER(pSMThreadData);
} // SamPopThreadWorker


//---------------------------------------------------------------------
// SamPopThreadStart
//---------------------------------------------------------------------
s32 SamPopThreadStart(void)
{
	void *pWorkerThread;


	SysDbgMsg4(("SamPopThreadStart: entry\n"));

	// start a thread to monitor the data base
	pWorkerThread = SMThreadStart(SamPopThreadWorker, 
		NULL);

	// failed ?
	if (pWorkerThread == NULL)
	{
		return SM_STATUS_OUT_OF_MEMORY;
	}

	// save thread pointer in global memory
	SamPopGlobalsSetWorkerThreadPtr(pWorkerThread);

	SysDbgMsg4(("SamPopThreadStart: exit\n"));

	return SM_STATUS_SUCCESS;
} // SamPopThreadStart


//---------------------------------------------------------------------
// SamPopThreadStop
//---------------------------------------------------------------------
void SamPopThreadStop(void)
{
	void *pWorkerThread;


	SysDbgMsg4(("SamPopThreadStop: entry\n"));

	// get and clear the thread pointer in global memory
	pWorkerThread = SamPopGlobalsSetWorkerThreadPtr(NULL);

	// threads running?
	if (pWorkerThread != NULL)
	{
		// stop all worker threads
		SMThreadStop(pWorkerThread);
	}

	SysDbgMsg4(("SamPopThreadStop: exit\n"));
} // SamPopThreadStop


