
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

#ifndef __DECMD_H_INCLUDED__
#define __DECMD_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------
#include <decmddef.h>
#include <prodversioninfo.h>
#include <dcsupapi.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

#define NVPGetUTF8Value				SMNVPGetUTF8ParamValueByUTF8Name


//---------------------------------------------------------------------
// Function Prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
//
//
//
// Public Functions used by Command Implementers
//
//
//
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Functions using global resources
//---------------------------------------------------------------------

SMECInfo *DECmdModGetOrigECI(void);


// Returns true if the module is exiting or not yet initialized
// Threads should use this to determine exit process
booln DECmdIsModuleExiting(void);


// The currently running thread pauses and blocks until time given is 
// consumed or module exit event is signaled. This guarantees threads 
// wake up when module is exiting. This should be used with 
// DECmdIsModuleExiting
void DECmdModExitListenerPause(s32 pauseMilliSeconds);


// The currently running thread blocks and wait indefinitely until
// module exit event is signaled. This should be used with
// DECmdIsModuleExiting.
void DECmdModExitListenerWait(void);


s32 ModExitSignalListenerAdd(_IN PFNDECMODEXITSIGNALLISTENER pfnListener,
							 _IN void *pListenerData);


//---------------------------------------------------------------------
// IO functions support using the cmd control object
//---------------------------------------------------------------------

// Version of vfprintf using user specified options
// Returns -1 if vfprintf fails, -2 if input parameters are invalid.
s32 DECmdOutErrf(_IN const DECmdControl *pDCC, 
				 _IN booln bIsError,
				 _IN u32 policyBitMap,
				 _IN u32 optionBitMap,
				 _IN const astring *pFmtStr, 
				 _IN va_list vap);

// Version of fprintf using user specified options
// see DECmdOutErrf for info
s32 DECmdOutf(_IN const DECmdControl *pDCC, 
			  _IN const astring *pFmtStr, 
			  _IN ...);

// Version of fprintf using user specified options
// see DECmdOutErrf for info
s32 DECmdErrf(_IN const DECmdControl *pDCC, 
			  _IN const astring *pFmtStr, 
			  _IN ...);

// Version of fprintf using user specified options
// with optional bit map
// see DECmdOutErrf for info
s32 DECmdOutfv(_IN const DECmdControl *pDCC, 
			   _IN u32 policyBitMap,
			   _IN u32 optionBitMap,
			   _IN const astring *pFmtStr, 
			   _IN ...);

// Version of fprintf using user specified options
// with optional bit map
// see DECmdOutErrf for info
s32 DECmdErrfv(_IN const DECmdControl *pDCC, 
			   _IN u32 policyBitMap,
			   _IN u32 optionBitMap,
			   _IN const astring *pFmtStr, 
			   _IN ...);

// Print hex dump of specified buffer
void DECmdDumpBufOutf(_IN const DECmdControl *pDCC,
					  _IN const void *pBuf,
					  _IN u32 bufSize);


//---------------------------------------------------------------------
// Help display support
//---------------------------------------------------------------------

// Returns TRUE if command help is requested. This function is used
// within the command implementation to process command paramater
// help requests.  The pHelpObj is required to generate help for the
// request. When this returns TRUE, typically the command implementation
// will return SM_STATUS_SUCCESS indicating that the help request
// is processed and successful.
booln DECmdPrintHelpCmdParamList(_IN const DECmdControl *pDCC,
								 _IN const DECmdHelpObject *pHelpObj);

// Returns FALSE when at least one parameter is not valid. The
// pHelpObj defines the parameter names and its type for the command
// in check.  The command implementation typically returns 
// SM_STATUS_INVALID_PARAMETER when this function returns FALSE.
booln DECmdValidateInputParameter(_IN const DECmdControl *pDCC,
								  _IN s32 argc, 
								  _IN astring **argv,
								  _IN const DECmdHelpObject *pHelpObj);


//---------------------------------------------------------------------
// Utility functions
//---------------------------------------------------------------------

// Iteratively/Repeatedly executes the named iterator function
// This function exits when the number of iteration reaches the 
// requested value; or when the module exit flag is detected;
// or when the iterator function returns a failure status.
// Returns SM_STATUS_INVALID_PARAMETER if pDCC or pfnIterateCode
// is NULL; returns SM_STATUS_SUCCESS upon normal termination; or
// the status returned by pfnIterateCode.
void DECmdFunctionIterator(_IN const DECmdControl *pDCC,
						   _IN s32 numNVPair, 
						   _IN astring **ppNVPair,
						   _IN PFNDECITERATORFUNC pfnIterateCode,
						   _INOUT void *pIterateData,
						   _OUT s32 *pStatus,
						   _IN s32 numIterateMin,
						   _IN s32 numIterateMax,
						   _IN s32 numIterateDefault);


// Same as DECmdFunctionIterator except that the number of iteration
// and pausemsecs is provided
void DECmdFunctionIteratorNum(_IN const DECmdControl *pDCC,
							  _IN PFNDECITERATORFUNC pfnIterateCode,
							  _INOUT void *pIterateData,
							  _OUT s32 *pStatus,
							  _IN s32 numiterate,
							  _IN u32 pausemsecs);


// Iterates through N threads and calls thread kill to make sure
// ran threads stops. Frees pNTHandle allocated by DECmdNThreadCreate
void DECmdNThreadDestroy(_IN void *pNTHandle);


// Blocks and wait until all N threads have terminated
// If threadExitTimeOutMSecs is > 0, every threadExitTimeOutMSecs
// module exit is checked.
void DECmdNThreadJoin(_IN void *pNTHandle,
					  _IN s32 threadExitTimeOutMSecs);


// Creates and runs N number of threads using the named thread function
// Returns a handle for use with joining all the ran threads
// Use the matching destroy function to free the handle
void *DECmdNThreadCreate(_IN const DECmdControl *pDCC,
						 _IN s32 numNVPair,
						 _IN astring **ppNVPair,
						 _IN PFNDECNTHREADFUNC pfnNThreadCode,
						 _INOUT void *pNThreadData,
						 _OUT s32 *pStatus,
						 _IN s32 numThreadMin,
						 _IN s32 numThreadMax,
						 _IN s32 numThreadDefault);


// Same as DECmdNThreadCreate except that the number of threads is provided
void * DECmdNThreadCreateNum(_IN const DECmdControl *pDCC,
							 _IN PFNDECNTHREADFUNC pfnNThreadCode,
							 _INOUT void *pNThreadData,
							 _OUT s32 *pStatus,
							 _IN s32 numThread);


//---------------------------------------------------------------------
// Common utility functions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Common Hello function provided by the common tool routines
//---------------------------------------------------------------------

s32 CMDHello(_IN const DECmdControl *pDCC, 
			 _IN s32 numNVPair, 
			 _IN astring **ppNVPair);


/*******************************************************************************
 * This function takes a pointer to a char* array and a string and parses the
 * string into parameters.  The structure is similar the the argv array. Each
 * pointer points to a single parameter.
 *
 * Parses a line from the script file into replacement arguments for argc & argv
 * a null char*** pointer is passed to recieve the new vector. This memory
 * should be freed by calling SMFreeMem(&strv[0]);
 ******************************************************************************/
s32 CmdScriptParseLine(_IN astring* argv0, 
					   _IN astring* pScriptLine, 
					   _INOUT s32* pCount, 
					   _INOUT astring*** strv);


void DECmdOutNVLinef(_IN const DECmdControl *pDCC,
					 _IN const astring *pName,
					 _IN s32 nameRegionLen,
					 _IN const astring *pValueFmt,
					 _IN ...);


booln NNSVContainsString(_IN astring *pNNSV,
						 _IN const astring *pString,
						 _IN booln bIgnoreCase);


//---------------------------------------------------------------------
//
//
//
// Private Functions used by decmdmodmain.c
// !!!!!! Command implementers should not use these !!!!!!
//
//
//
//---------------------------------------------------------------------

// SendCmd main entry point
s32 DECmdSendCmd(_IN const DECmdControl *pDCC,
				 _IN s32 numNVPair, 
				 _IN astring **ppNVPair);

// Initializes CMDOptions object with necessary values
// NOTE: Use DECmdControlDetach at end of calling block
s32 DECmdControlAttach(_IN s32 argc,
					   _IN astring **argv,
					   _INOUT DECmdControl *pDCC,
					   _OUT u32 *pNonNVPCount);

// Frees the resources acquired in call to DECmdControlAttach
void DECmdControlDetach(_INOUT DECmdControl *pDCC);

// Sort the Command Dispatch table
s32 DECmdSortCmdDispatchTable(void);

// Print the contents of the error message buffer
void DECmdPrintParamErrMssg(_IN const DECmdControl *pDCC);

// Print the standard help
s32 DECmdPrintHelp(_IN const DECmdControl *pDCC,
				   _IN s32 argc, 
				   _IN astring **argv);

// Print the standard help header and usage prepended
// with contents of error messages if any
void DECmdPrintHelpHeaderUsage(_IN const DECmdControl *pDCC,
							   _IN s32 argc, 
							   _IN astring **argv);


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DECMD_H_INCLUDED__

