
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 27 $
//
//  Last Modified By / On:
//    $Author: Chris_poblete $ / $Date: 8/18/03 9:09a $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

#ifndef __DECMDDEF_H_INCLUDED__
#define __DECMDDEF_H_INCLUDED__


//---------------------------------------------------------------------
// Include files this header is dependent on
//---------------------------------------------------------------------

//--------------------------------------------------------------------
// Data Engine core includes
//--------------------------------------------------------------------
#include <dcossdk.h>
#include <dcsupdef.h>


//---------------------------------------------------------------------
// Includes end
//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Defines and types
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Command syntax definitions
//---------------------------------------------------------------------

#define DECMD_CMD_KEY					"command"

#define DECMD_OPT_SCRIPT				"-script"
#define DECMD_OPT_MAX_SCRIPT_ERRORS		"-maxerrors"
#define DECMD_OPT_OUT					"-outc"
#define DECMD_OPT_OUTA					"-outa"
#define DECMD_OPT_ERR					"-errc"
#define DECMD_OPT_ERRA					"-erra"
#define DECMD_OPT_TS					"-ts"

#define DECMD_HELPSTR_TAB				"  "

#define DECMD_ERR_STR					"Error! "
#define DECMD_WARN_STR					"Warning: "

#define DECMD_OPT_STR					"(opt.)"
#define DECMD_REQ_STR					"(req.)"


#define DECMD_HELPSTR_HEADER_FMT \
	"\n" PRD_VER_PRODUCTNAME_STR " " PRD_VER_FILEDESCRIPTION_STR "  " \
		PRD_VER_PRODUCTVERSION_STR "\n" PRD_VER_LEGALCOPYRIGHT_STR "\n\n"

#define DECMD_HELPSTR_USAGE_FMT \
	"Usage: %s " DECMD_CMD_KEY "=COMMAND [PARAMETERS...] [OPTIONS...]\n"

#define DECMD_HELPSTR_PARAMS_FMT \
	"\n" \
	"PARAMETERS:\n"

#define DECMD_HELPSTR_COMMANDS_FMT \
	"\n" \
	"COMMANDS:\n"

#define DECMD_HELPSTR_COMMAND_DESC_FMT \
	"\n" \
	"COMMAND:\n"

#define DECMD_HELPSTR_ERR_MSSG_FMT \
	DECMD_ERR_STR "%s\n"

#define DECMD_HELPSTR_STDOPT_FMT \
	"\n" \
	"OPTIONS:\n" \
	DECMD_HELPSTR_TAB "-?                  Display help\n" \
	DECMD_HELPSTR_TAB DECMD_OPT_TS " LEVEL           Prepend output with timestamp\n" \
	DECMD_HELPSTR_TAB DECMD_OPT_OUT " FILE          Redirect stdout to FILE, use " DECMD_OPT_OUTA " to append\n" \
	DECMD_HELPSTR_TAB DECMD_OPT_ERR " FILE          Redirect stderr to FILE, use " DECMD_OPT_ERRA " to append\n" \
	DECMD_HELPSTR_TAB DECMD_OPT_SCRIPT " FILE        Read commands from FILE\n" \
	DECMD_HELPSTR_TAB DECMD_OPT_MAX_SCRIPT_ERRORS " N        Max number of script errors tolerated before exit\n" \
	DECMD_HELPSTR_TAB "                    Default is (0) continue on all errors\n" \
	"\nNOTES:\n" \
	"If \"" DECMD_OPT_TS "\" is specified, output is prepended with timestamp depending on given\n" \
	"priority level: 4=low(all output), 3=medium, 2=high, 1=reserved(default).\n" \
	"Type \"" DECMD_CMD_KEY " -?\" or \"" DECMD_CMD_KEY "=?\" to get a list of available commands.\n" \
	"Type \"" DECMD_CMD_KEY "=COMMAND -?\" to get a list of available parameters for COMMAND.\n" \
	"Enclosing quotes must be in Windows/Linux format: param=\"some string\".\n" \
	"A script file should contain a valid command as displayed by the command=?\n" \
	"and its parameters and options on each line. If errors occur while\n" \
	"running the script, return value is SM_STATUS_DECMD_SCRIPT_ERROR.\n"
    


//---------------------------------------------------------------------
// Reserved command parameter names
//---------------------------------------------------------------------

// common definitions
#define DECMDPRM_VERBOSE					"verbose"
#define DECMDPRM_VERBOSE_TYPE				SM_VTYPE_u32
#define DECMDPRM_VERBOSE_DESC				DECMD_OPT_STR " verbose level; default=1"
#define DECMDPRM_VERBOSE_DESC_DETAILS		"Verbose level: 1=minimal 2=busy 3=verybusy 4=maximum."

// initialized and used by DECmdFunctionIterator
#define DECMDPRM_NUMITERATE					"numiterate"
#define DECMDPRM_NUMITERATE_TYPE			SM_VTYPE_s32
#define DECMDPRM_NUMITERATE_DESC			DECMD_OPT_STR " number of iterations"

// initialized and used by DECmdFunctionIterator
#define DECMDPRM_PAUSEMSECS					"pausemsecs"
#define DECMDPRM_PAUSEMSECS_TYPE			SM_VTYPE_u32
#define DECMDPRM_PAUSEMSECS_DESC			DECMD_OPT_STR " pause in milliseconds"

// initialized and used by DECmdNThreadCreate
#define DECMDPRM_NUMTHREADS					"numthreads"
#define DECMDPRM_NUMTHREADS_TYPE			SM_VTYPE_s32
#define DECMDPRM_NUMTHREADS_DESC			DECMD_OPT_STR " number of threads to run"


// default values for numiterate
#define DECMD_MIN_NUMITERATE				(1)
#define DECMD_MIN_NUMITERATE_STR			"1"
#define DECMD_MAX_NUMITERATE				(-1)		// infinite
#define DECMD_MAX_NUMITERATE_STR			"-1"		// infinite
#define DECMD_DEFAULT_NUMITERATE			DECMD_MIN_NUMITERATE
#define DECMD_DEFAULT_NUMITERATE_STR		DECMD_MIN_NUMITERATE_STR

// default values for numthread
#define DECMD_MIN_NUMTHREADS				(1)
#define DECMD_MIN_NUMTHREADS_STR			"1"
#define DECMD_MAX_NUMTHREADS				(64)
#define DECMD_MAX_NUMTHREADS_STR			"64"
#define DECMD_DEFAULT_NUMTHREADS			DECMD_MIN_NUMTHREADS
#define DECMD_DEFAULT_NUMTHREADS_STR		DECMD_MIN_NUMTHREADS_STR


//---------------------------------------------------------------------
// Output control bitmap
//---------------------------------------------------------------------
#define DECMDOUT_UNDEFINED					(0)

// time stamp level
#define DECMDOUT_TS_LEVEL1					BIT(4) // reserved
#define DECMDOUT_TS_LEVEL2					BIT(5) // high
#define DECMDOUT_TS_LEVEL3					BIT(6) // medium
#define DECMDOUT_TS_LEVEL4					BIT(7) // low

// verbose level
#define DECMDOUT_VERBOSE_LEVEL1				BIT(8) // minimum
#define DECMDOUT_VERBOSE_LEVEL2				BIT(9) // busy
#define DECMDOUT_VERBOSE_LEVEL3				BIT(10) // very busy
#define DECMDOUT_VERBOSE_LEVEL4				BIT(11) // maximum


//---------------------------------------------------------------------
// Command control options bitmap
// MAX=BIT(31)
//---------------------------------------------------------------------
#define DECMDCTRL_OPT_UNDEFINED				(0)
// help options
#define DECMDCTRL_OPT_HELP_CMD				BIT(0)
#define DECMDCTRL_OPT_HELP_PARAM			BIT(1)
// output options
#define DECMDCTRL_OPT_STDERR_APPEND			BIT(2)
#define DECMDCTRL_OPT_STDOUT_APPEND			BIT(3)
// timestamp options
#define DECMDCTRL_OPT_TS_LEVEL1				DECMDOUT_TS_LEVEL1
#define DECMDCTRL_OPT_TS_LEVEL2				(DECMDOUT_TS_LEVEL2 | DECMDCTRL_OPT_TS_LEVEL1)
#define DECMDCTRL_OPT_TS_LEVEL3				(DECMDOUT_TS_LEVEL3 | DECMDCTRL_OPT_TS_LEVEL2)
#define DECMDCTRL_OPT_TS_LEVEL4				(DECMDOUT_TS_LEVEL4 | DECMDCTRL_OPT_TS_LEVEL3)
// verbose options
#define DECMDCTRL_OPT_VERBOSE_LEVEL1		DECMDOUT_VERBOSE_LEVEL1
#define DECMDCTRL_OPT_VERBOSE_LEVEL2		(DECMDOUT_VERBOSE_LEVEL2 | DECMDCTRL_OPT_VERBOSE_LEVEL1)
#define DECMDCTRL_OPT_VERBOSE_LEVEL3		(DECMDOUT_VERBOSE_LEVEL3 | DECMDCTRL_OPT_VERBOSE_LEVEL2)
#define DECMDCTRL_OPT_VERBOSE_LEVEL4		(DECMDOUT_VERBOSE_LEVEL4 | DECMDCTRL_OPT_VERBOSE_LEVEL3)


//---------------------------------------------------------------------
// MASK definitions
//---------------------------------------------------------------------
#define DECMDMASK_TS_LEVEL					(DECMDOUT_TS_LEVEL1 | \
											 DECMDOUT_TS_LEVEL2 | \
											 DECMDOUT_TS_LEVEL3 | \
											 DECMDOUT_TS_LEVEL4)

#define DECMDMASK_VERBOSE_LEVEL				(DECMDOUT_VERBOSE_LEVEL1 | \
											 DECMDOUT_VERBOSE_LEVEL2 | \
											 DECMDOUT_VERBOSE_LEVEL3 | \
											 DECMDOUT_VERBOSE_LEVEL4)


//---------------------------------------------------------------------
// Limits
//---------------------------------------------------------------------

// maximum single string length from command line
#define DECMD_MAX_PARAM_SIZE				MAX_PATH_SIZE

// maximum pause waiting for threads to finish
#define DECMD_WAIT_MSECS_DEFAULT			(60000) // 1 minute


//---------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// The command control object
//---------------------------------------------------------------------
typedef struct _DECmdControl
{
	FILE *pOutStream;						// output stream opened once during module init
	FILE *pErrStream;						// error stream opened once during module init
	SMSSUTF8Str errMssg;					// error messages
	u32 optionsMap;							// bitmap of options
	astring outFile[DECMD_MAX_PARAM_SIZE];	// user specified output filename
	astring errFile[DECMD_MAX_PARAM_SIZE];	// user specified error output filename
	astring cmdName[DECMD_MAX_PARAM_SIZE];	// DE command name
} DECmdControl;


//---------------------------------------------------------------------
// The command marshal function prototype
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNDECMDSENDCMD)(
						_IN const DECmdControl *pDCC, 
						_IN s32 numNVPair, 
						_IN astring **ppNVPair
						);


//---------------------------------------------------------------------
// The command marshal database definition
//---------------------------------------------------------------------
typedef struct _DECmdDispatchTable
{
	astring *pCmdName;
	PFNDECMDSENDCMD pfnDECmdSendCmd;
} DECmdDispatchTable;


//---------------------------------------------------------------------
// Command parameter object definition
// NOTE: In order to align the display on console, it is up to the 
// code implementor to restrict the param name to 12 characters
// and the param description to 52 characters not including the 
// NULL terminator.
//---------------------------------------------------------------------
typedef struct _DECmdParameterObject
{
	const astring *pParamName;			// (restrict to 12 chars) name of the parameter
	s16 paramType;						// type of parameter, see SM_VTYPE_...
	u8 reservedAlign[6];				// alignment for next data element
	const astring *pParamDescription;	// (restrict to 52 chars) description of the parameter
} DECmdParameterObject;


//---------------------------------------------------------------------
// Command help object definition
// NOTE: In order to align the display on console, it is up to the 
// code implementor to restrict the command description to 72
// characters not including the NULL terminator.
//---------------------------------------------------------------------
typedef struct _DECmdHelpObject
{
	const astring *pCmdDescription;		// (restrict to 72 chars) description of the command
	const astring *pComment;			// command comments (may be NULL)
	const DECmdParameterObject *pParamList;	// the parameter list definition (may be NULL)
	u32 countParamList;					// parameter list count
} DECmdHelpObject;


//---------------------------------------------------------------------
// Function Iterator prototype
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNDECITERATORFUNC)(
						_INOUT void *pIteratorData,
						_IN s32 iteration,
						_IN s32 maxIteration
						);

//---------------------------------------------------------------------
// N-Thread runner prototype
//---------------------------------------------------------------------
typedef s32 (_cdecl *PFNDECNTHREADFUNC)(
						_IN void *pNThreadData,
						_IN s32 threadIndex,
						_IN s32 maxThreadIndex
						);


//---------------------------------------------------------------------
// Module Exit Listener prototype
//---------------------------------------------------------------------
typedef void (_cdecl *PFNDECMODEXITSIGNALLISTENER)(
						_IN void *pData
						);


//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif




#endif // __DECMDDEF_H_INCLUDED__
