
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Include Files
//---------------------------------------------------------------------

#include <prodbuildnumberinfo.h>
#include <decmd.h>
#include <dcsysdebug.h>


//---------------------------------------------------------------------
// Local Defines
//---------------------------------------------------------------------

// maximum number of threads this tool allows to run
#define ULIMIT_NUM_THREAD					(64)

// upper limit of iteration where it wraps to 0
#define ULIMIT_NUM_ITERATE					(0x7fffffff)


//---------------------------------------------------------------------
// Thread runner object definitions
//---------------------------------------------------------------------

typedef struct _NThreadControlData
{
	struct _NThreadControl *pThreadControl;
	PFNDECNTHREADFUNC pfnNThreadCode;
	void *pNThreadData;
	void *pNThreadHandle;
	s32 threadIndex;
	u32 maxThreadIndex;
} NThreadControlData;


typedef struct _NThreadControl
{
	const DECmdControl *pDCC;
	void *pObjectLock;
	void *pThreadCtrlExitEvt;
	u32 startedThreadCount;
	u32 threadCount;
	NThreadControlData ntcd[1];
} NThreadControl;


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Function prototypes defined elsewhere
//---------------------------------------------------------------------

extern DECmdDispatchTable * NVPCmdGetCDT(_OUT s32 *pNumElements);


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// DECmdGetArgKeyValue
//---------------------------------------------------------------------
const astring * DECmdGetArgKeyValue(_IN s32 argc, 
									_IN astring **argv,
									_IN const astring *pKey,
									_IN booln bJustKey)
{
	s32 ii = 1;

	while (ii < argc)
	{
		if (stricmp(argv[ii], pKey) == 0)
		{
			if (bJustKey)
			{
				return argv[ii];
			}
			if ((++ii) < argc)
			{
				return argv[ii];
			}
		}
		ii++;
	}

	return NULL;
} // DECmdGetArgKeyValue


//--------------------------------------------------------------------
// DECmdOutErrf
// Returns -1 if vfprintf fails, -2 if input parameters are invalid.
//--------------------------------------------------------------------
s32 DECmdOutErrf(_IN const DECmdControl *pDCC, 
				 _IN booln bIsError,
				 _IN u32 policyBitMap,
				 _IN u32 optionBitMap,
				 _IN const astring *pFmtStr, 
				 _IN va_list vap)
{
	FILE *pIOStream;
	SMECInfo currECI;
	time_t currTime;
	astring strTime[32];
	s32 rstat;
	booln bIsLocalOpen;


	// prevent compiler warnings, remove when used
	UNREFERENCED_PARAMETER(policyBitMap);

	// make sure we are in the primary context
	SMSetExportContext(&currECI,
		DECmdModGetOrigECI());

	// sanity check
	if ((pDCC == NULL) ||
		(pFmtStr == NULL))
	{
		// signal that parameter is invalid
		rstat = -2;
	}
	else
	{
		// init
		bIsLocalOpen = FALSE;
		
		if (bIsError == TRUE)
		{
			// use user specified stream
			pIOStream = pDCC->pErrStream;
			// validate
			if (pIOStream == NULL)
			{
				// if user has specified stream file use it, but always append
				if (pDCC->errFile[0] != '\0')
				{
					pIOStream = fopen(pDCC->errFile, "a");
					// flag that stream is opened locally
					bIsLocalOpen = TRUE;
				}
				
				// validate again
				if (pIOStream == NULL)
				{
					// use default
					pIOStream = stderr;
				}
			}
		}
		else
		{
			// use user specified stream
			pIOStream = pDCC->pOutStream;
			// validate
			if (pIOStream == NULL)
			{
				// if user has specified stream file use it, but always append
				if (pDCC->outFile[0] != '\0')
				{
					pIOStream = fopen(pDCC->outFile, "a");
					// flag that stream is opened locally
					bIsLocalOpen = TRUE;
				}
				
				// validate again
				if (pIOStream == NULL)
				{
					// use default
					pIOStream = stdout;
				}
			}
		}
		
		// prepend with timestamp?
		if (pDCC->optionsMap & DECMDMASK_TS_LEVEL & optionBitMap)
		{
			// get current time
			time(&currTime);
			// get string time
			// length of string from ctime is always constant
			// "Wed Mar 21 15:58:27 1990\n\0"
			strncpy(strTime, ctime(&currTime), 24);
			// terminate it
			strTime[24] = '\0';
			// stream it
			fprintf(pIOStream, "[%s] ", strTime);
		}

		// stream it
		rstat = vfprintf(pIOStream, pFmtStr, vap);
		
		// always flush the stream
		fflush(pIOStream);

		if (bIsLocalOpen)
		{
			// close the handle
			fclose(pIOStream);
			// prevent GPFs
			pIOStream = NULL;
		}
	}

	// reset context to previous
	SMResetExportContext(&currECI);

	return rstat;
} // DECmdOutErrf


//--------------------------------------------------------------------
// DECmdOutf
// see DECmdOutErrf for info
//--------------------------------------------------------------------
s32 DECmdOutf(_IN const DECmdControl *pDCC, 
			  _IN const astring *pFmtStr, 
			  _IN ...)
{
	va_list vap;
	s32 retVal = -2;


	va_start(vap, pFmtStr);
	retVal = DECmdOutErrf(pDCC, 
		FALSE, 
		(DECMDOUT_TS_LEVEL4), 
		(DECMDOUT_TS_LEVEL4), 
		pFmtStr, 
		vap);
	va_end(vap);

	return retVal;
} // DECmdOutf


//--------------------------------------------------------------------
// DECmdErrf
// see DECmdOutErrf for info
//--------------------------------------------------------------------
s32 DECmdErrf(_IN const DECmdControl *pDCC, 
			  _IN const astring *pFmtStr, 
			  _IN ...)
{
	va_list vap;
	s32 retVal = -2;


	va_start(vap, pFmtStr);
	retVal = DECmdOutErrf(pDCC, 
		TRUE, 
		(DECMDOUT_TS_LEVEL4),
		(DECMDOUT_TS_LEVEL4),
		pFmtStr, 
		vap);
	va_end(vap);

	return retVal;
} // DECmdErrf


//--------------------------------------------------------------------
// DECmdOutfv
// see DECmdOutErrf for info
//--------------------------------------------------------------------
s32 DECmdOutfv(_IN const DECmdControl *pDCC, 
			   _IN u32 policyBitMap,
			   _IN u32 optionBitMap,
			   _IN const astring *pFmtStr, 
			   _IN ...)
{
	va_list vap;
	s32 retVal = -2;


	// check verbose level
	if (((DECMDMASK_VERBOSE_LEVEL & policyBitMap) & optionBitMap) == 
		(DECMDMASK_VERBOSE_LEVEL & policyBitMap))
	{
		va_start(vap, pFmtStr);
		retVal = DECmdOutErrf(pDCC, 
			FALSE, 
			policyBitMap,
			optionBitMap, 
			pFmtStr, 
			vap);
		va_end(vap);
	}

	return retVal;
} // DECmdOutfv


//--------------------------------------------------------------------
// DECmdErrfv
// see DECmdOutErrf for info
//--------------------------------------------------------------------
s32 DECmdErrfv(_IN const DECmdControl *pDCC, 
			   _IN u32 policyBitMap,
			   _IN u32 optionBitMap,
			   _IN const astring *pFmtStr, 
			   _IN ...)
{
	va_list vap;
	s32 retVal = -2;


	if (((DECMDMASK_VERBOSE_LEVEL & policyBitMap) & optionBitMap) == 
		(DECMDMASK_VERBOSE_LEVEL & policyBitMap))
	{
		va_start(vap, pFmtStr);
		retVal = DECmdOutErrf(pDCC, 
			TRUE, 
			policyBitMap,
			optionBitMap,
			pFmtStr, 
			vap);
		va_end(vap);
	}

	return retVal;
} // DECmdErrfv


//---------------------------------------------------------------------
// DECmdDumpBufOutf
//---------------------------------------------------------------------
void DECmdDumpBufOutf(_IN const DECmdControl *pDCC,
					  _IN const void *pBuf,
					  _IN u32 bufSize)
{
	astring *pAStr;
	s32 status;
	u32 size;
	u32 vType;


	// set conversion type
	vType = (SM_XLT_VTYPE_BINARY_TO_ASTRING | BINARY_HEXDUMP);

	// get string buffer size needed
	size = 0;
	if ((status = SMXLTTypeValueToUTF8(pBuf,
		bufSize,
		NULL,
		&size,
		vType)) != SM_STATUS_DATA_OVERRUN)
	{
		SysDbgMsg3(("DECmdDumpBufOutf: failed to get buffer size: status: %d\n",
			status));
		_throw error1;
	}

	// allocate string buffer
	if ((pAStr = SMAllocMem(size)) == NULL)
	{
		SysDbgMsg3(("DECmdDumpBufOutf: failed to allocate buffer\n"));
		_throw error1;
	}

	// convert to hexdump format
	if ((status = SMXLTTypeValueToUTF8(pBuf,
		bufSize,
		pAStr,
		&size,
		vType)) != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("DECmdDumpBufOutf: failed to convert data: status: %d\n",
			status));
		_throw error2;
	}

	// display hexdump string
	DECmdOutf(pDCC, pAStr);

error2:
	// free string buffer
	SMFreeMem(pAStr);
	pAStr = NULL;

error1:
	return;
} // DECmdDumpBufOutf


//---------------------------------------------------------------------
// DECmdControlAddErrorMssgf
//---------------------------------------------------------------------
void DECmdControlAddErrorMssgf(_IN const DECmdControl *pDCC,
							   _IN const astring *pErrorMssgFormat,
							   ...)
{
	astring strBuf[MAX_PATH_SIZE];
	va_list vap;
	u32 sizeStr;
	s32 sizeWritten;


	// sanity check
	if ((pDCC != NULL) &&
		(pErrorMssgFormat != NULL))
	{
		// check for string length in range first
		// on the safe side, upper range is 1/2 the max size
		// to account for the arguments
		sizeStr = (u32)(strlen(pErrorMssgFormat) + 1);
		if (sizeStr > (MAX_PATH_SIZE / 2))
		{
			SysDbgMsg3(("DECmdControlAddErrorMssgf: " \
				"pErrorMssgFormat size is too long\n"));
		}
		else
		{
			// init buffer
			strBuf[0] = '\0';
			
			// start VA
			va_start(vap, pErrorMssgFormat);
			
			// generate the message
			sizeWritten = vsprintf(strBuf, 
				pErrorMssgFormat, 
				vap);
			if (sizeWritten > 0)
			{
				// check for buffer overflow since we could not
				// use vsnprintf
				if (sizeWritten > (MAX_PATH_SIZE - 1))
				{
					DECmdErrf(pDCC,
						"DECmdControlAddErrorMssgf: ERROR: " \
						"buffer overflow with size (" PRT_S32 ")\n" \
						"Message format is (%s)\n", 
						sizeWritten);
				}
				else
				{
					// get the size of the string
					// add 1 to include the NULL terminator
					sizeStr = (u32)(strlen(strBuf) + 1);
					
					// now add it to the error message buffer
					SMSSUTF8StrNCatUTF8Str(&((DECmdControl *)pDCC)->errMssg,
						strBuf,
						sizeStr);
				}
			}
			
			// stop VA
			va_end(vap);
		}
	}
} // DECmdControlAddErrorMssgf


//---------------------------------------------------------------------
// DECmdPrintHelpHeader
//---------------------------------------------------------------------
void DECmdPrintHelpHeader(_IN const DECmdControl *pDCC)
{

	// sanity check
	if (pDCC != NULL)
	{
		DECmdOutf(pDCC, DECMD_HELPSTR_HEADER_FMT);
	}
} // DECmdPrintHelpHeader


//---------------------------------------------------------------------
// DECmdPrintHelpUsage
//---------------------------------------------------------------------
void DECmdPrintHelpUsage(_IN const DECmdControl *pDCC,
						 _IN s32 argc,
						 _IN astring **argv)
{
	astring *pProgName;


	// sanity check
	if (pDCC != NULL)
	{
		// check for argc < 1 for systems not supporting argv[0]
		if (argc > 0)
		{
			// find the last occurrence of the file separator
			// so that we can exclude the path portion and 
			// just return the program name
			pProgName = strrchr(argv[0], FILE_PATH_DIR_SEPERATOR);
			
			if (pProgName == NULL)
			{
				// search failed, just get the whole string
				pProgName = argv[0];
			}
			else
			{
				// get the string without the path portion
				pProgName++;
			}
			
			DECmdOutf(pDCC, DECMD_HELPSTR_USAGE_FMT, pProgName);

		}
	}
} // DECmdPrintHelpUsage


//---------------------------------------------------------------------
// DECmdPrintHelpFooter
//---------------------------------------------------------------------
void DECmdPrintHelpFooter(_IN const DECmdControl *pDCC)
{

	// sanity check
	if (pDCC != NULL)
	{
		DECmdOutf(pDCC, DECMD_HELPSTR_STDOPT_FMT);
	}
} // DECmdPrintHelpFooter


//---------------------------------------------------------------------
// DECmdPrintParamErrMssg
//---------------------------------------------------------------------
void DECmdPrintParamErrMssg(_IN const DECmdControl *pDCC)
{
	astring *pStrC;


	// is the error message buffer not emtpy
	if (pDCC != NULL)
	{
		// check count
		if (pDCC->errMssg.strCurLen > 0) 
		{
			pStrC = pDCC->errMssg.pStr;
			while (*pStrC != '\0')
			{
				// display the error entry
				DECmdOutf(pDCC, DECMD_HELPSTR_ERR_MSSG_FMT, pStrC);

				// go to the next entry
				pStrC += (u32)(strlen(pStrC) + 1);

				// make sure we do not go out of bounds
				if ((u32)(pStrC - pDCC->errMssg.pStr) > pDCC->errMssg.strCurLen)
				{
					break;
				}
			}
		}
	}
} // DECmdPrintParamErrMssg


//---------------------------------------------------------------------
// DECmdPrintHelpHeaderUsage
//---------------------------------------------------------------------
void DECmdPrintHelpHeaderUsage(_IN const DECmdControl *pDCC,
							   _IN s32 argc, 
							   _IN astring **argv)
{

	// print argument error messages
	DECmdPrintParamErrMssg(pDCC);

	// print the help header
	DECmdPrintHelpHeader(pDCC);
	// print the usage
	DECmdPrintHelpUsage(pDCC, argc, argv);
} // DECmdPrintHelp


//---------------------------------------------------------------------
// DECmdPrintHelp
//---------------------------------------------------------------------
s32 DECmdPrintHelp(_IN const DECmdControl *pDCC,
				   _IN s32 argc, 
				   _IN astring **argv)
{

	// print the header usage and error messages
	DECmdPrintHelpHeaderUsage(pDCC, argc, argv);

	// print the help footer
	DECmdPrintHelpFooter(pDCC);

	return SM_STATUS_INVALID_PARAMETER;
} // DECmdPrintHelp


//---------------------------------------------------------------------
// DECmdSMNativeVTypeToStr
//---------------------------------------------------------------------
const astring * DECmdSMNativeVTypeToStr(_IN s16 smVType)
{
	const astring *pTStr = "???";


	// get string version of sm vtype
	switch(smVType)
	{
		case SM_VTYPE_BOOLN:
			pTStr = "BOOLN";
			break;
		case SM_VTYPE_s8:
			pTStr = "s8";
			break;
		case SM_VTYPE_s16:
			pTStr = "s16";
			break;
		case SM_VTYPE_s32:
			pTStr = "s32";
			break;
		case SM_VTYPE_s64:
			pTStr = "s64";
			break;
		case SM_VTYPE_u8:
			pTStr = "u8";
			break;
		case SM_VTYPE_u16:
			pTStr = "u16";
			break;
		case SM_VTYPE_u32:
			pTStr = "u32";
			break;
		case SM_VTYPE_u64:
			pTStr = "u64";
			break;
		case SM_VTYPE_d64:
			pTStr = "d64";
			break;
		case SM_VTYPE_ASTRING:
			pTStr = "STRING";
			break;
		case SM_VTYPE_USTRING:
			pTStr = "USTRING";
			break;
		case SM_VTYPE_S16:
			pTStr = "S16";
			break;
		case SM_VTYPE_S32:
			pTStr = "S32";
			break;
		case SM_VTYPE_S64:
			pTStr = "S64";
			break;
		case SM_VTYPE_U16:
			pTStr = "U16";
			break;
		case SM_VTYPE_U32:
			pTStr = "U32";
			break;
		case SM_VTYPE_U64:
			pTStr = "U64";
			break;
		case SM_VTYPE_D64:
			pTStr = "D64";
			break;
		default:
			break;
	}

	return pTStr;
} // DECmdSMNativeVTypeToStr


//---------------------------------------------------------------------
// DECmdPrintHelpCmdParamListForce
//---------------------------------------------------------------------
booln DECmdPrintHelpCmdParamListForce(_IN const DECmdControl *pDCC,
									  _IN const DECmdHelpObject *pHelpObj,
									  _IN booln bForceHelpProcessing)
{
	u32 ii;
	u32 jj;


	// sanity check
	if ((pDCC == NULL) ||
		(pHelpObj == NULL))
	{
		SysDbgMsg3(("DECmdPrintHelpCmdParamListForce: " \
			"NULL required parameter(s)\n"));
	}
	else
	{
		// only process if cmd param help is requested
		if (SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_PARAM) ||
			bForceHelpProcessing)
		{
			// print out the command header
			DECmdOutf(pDCC, DECMD_HELPSTR_COMMAND_DESC_FMT);

			// print the command name
			DECmdOutf(pDCC, DECMD_HELPSTR_TAB "%s",
				pDCC->cmdName);

			// print the spacer
			// transfer value to variable
			// since we are about to use a macro
			jj = (20 - (u32)strlen(pDCC->cmdName));
			ii = (u32)MAX(0, jj);
			while (ii-- > 0)
			{
				DECmdOutf(pDCC, "%c", ' ');
			}

			// now print the description
			DECmdOutf(pDCC, "%s\n",
				pHelpObj->pCmdDescription);
			
			DECmdOutf(pDCC, DECMD_HELPSTR_PARAMS_FMT);
			
			// print out the parameter list if there is one
			if (pHelpObj->pParamList == NULL)
			{
				DECmdOutf(pDCC, DECMD_HELPSTR_TAB "NONE\n");
			}
			else
			{
				for (ii = 0; ii < pHelpObj->countParamList; ii++)
				{
					// print the param name and type
					DECmdOutf(pDCC, 
						"%s%s=%s",
						DECMD_HELPSTR_TAB,
						pHelpObj->pParamList[ii].pParamName,
						DECmdSMNativeVTypeToStr(pHelpObj->pParamList[ii].paramType));
					
					// print the space formatter
					// tab(2) name(12) equal(1) type(5) spacer(?) = 22
					jj = (19 - 
						(u32)strlen(pHelpObj->pParamList[ii].pParamName) - 
						(u32)strlen(DECmdSMNativeVTypeToStr(pHelpObj->pParamList[ii].paramType)));
					jj = (u32)MIN(22, 
						jj);
					while (jj-- != 0)
					{
						DECmdOutf(pDCC,
							"%c",
							' ');
					}
					
					// print the description
					DECmdOutf(pDCC,
						"%s\n",
						pHelpObj->pParamList[ii].pParamDescription);
					
				}
			}

			// print out the comment
			if (pHelpObj->pComment != NULL)
			{
				DECmdOutf(pDCC, "\nCOMMENTS:\n%s\n", 
					pHelpObj->pComment);
			}
			
			// help is requested so return true
			return TRUE;

		} // if (SMMBitMapIsOn(pDCC->optionsMap...
	} // if ((pDCC != NULL)...

	// help is not requested or parameters are invalid
	return FALSE;
} // DECmdPrintHelpCmdParamListForce


//---------------------------------------------------------------------
// DECmdPrintHelpCmdParamList
//---------------------------------------------------------------------
booln DECmdPrintHelpCmdParamList(_IN const DECmdControl *pDCC,
								 _IN const DECmdHelpObject *pHelpObj)
{

	return DECmdPrintHelpCmdParamListForce(pDCC, 
		pHelpObj, 
		FALSE);
} // DECmdPrintHelpCmdParamList


//---------------------------------------------------------------------
// DECmdValidateInputParameter
//---------------------------------------------------------------------
booln DECmdValidateInputParameter(_IN const DECmdControl *pDCC,
								  _IN s32 argc, 
								  _IN astring **argv,
								  _IN const DECmdHelpObject *pHelpObj)
{
	astring *pStrCursor;
	booln bIsValid;
	u32 strSize;
	u32 ii;

	
	// assume all input parameters are valid until proven otherwise
	bIsValid = TRUE;

	// sanity check
	if (pDCC == NULL)
	{
		SysDbgMsg3(("DECmdValidateInputParameter: pDCC == NULL\n"));
		bIsValid = FALSE;
		_throw error1;
	}

	// is there any input to process
	if ((argc > 0) &&
		(argv != NULL))
	{		
		// browse the input parameters
		while(argc != 0)
		{
			// set str cursor
			pStrCursor = argv[argc - 1];
			
			// skip the command string
			if (strncmp(DECMD_CMD_KEY, 
				pStrCursor, 
				(u32)strlen(DECMD_CMD_KEY)) != 0)
			{
				// browse the parameter list
				for (ii = 0; ii < pHelpObj->countParamList; ii++)
				{
					// get the size of the param name to compare
					strSize = (u32)strlen(pHelpObj->pParamList[ii].pParamName);
					
					if (strncmp(pHelpObj->pParamList[ii].pParamName,
						pStrCursor,
						strSize) == 0)
					{
						// make sure that input string has an equal sign
						// prevent matches like "foo" and "foobar=xxx"
						if (pStrCursor[strSize] == '=')
						{
							// make sure that the input string has a valid value
							// prevent matches like "foo" and "foo==xxxx"
							if (pStrCursor[strSize + 1] != '=')
							{
								// match found so break the for loop
								break;
							}
						}
					}
				}
				
				// was there a match for the current input parameter
				if (ii == pHelpObj->countParamList)
				{
					// invalid input parameter found
					bIsValid = FALSE;
					
					// add error to error buffer
					DECmdControlAddErrorMssgf(pDCC,
						"invalid parameter: %s",
						pStrCursor);
					
					// keep going to validate the rest
				}
			}
			
			// go to next element
			argc--;
		} // while(argc != 0)...
	} // if (argc > 0)...

 error1:
	return bIsValid;
} // DECmdValidateInputParameter


//---------------------------------------------------------------------
// DECmdNVPIsUTF8ParamPresent
//---------------------------------------------------------------------
booln DECmdNVPIsUTF8ParamPresent(_IN s32 numNVPair, 
								 _IN astring **ppNVPair, 
								 _IN const astring *pName, 
								 _IN booln ignoreCase)
{
	u32 sizeQueryName;


	// sanity check
	if (pName != NULL)
	{
		// get the length of the query name including null terminator
		sizeQueryName = (u32)(strlen(pName) + 1);

		while (numNVPair-- > 0)
		{
			if (ignoreCase)
			{
				if (strnicmp(pName, 
					ppNVPair[numNVPair], 
					sizeQueryName) == 0)
				{
					// found it
					return TRUE;
				}
			}
			else
			{
				if (strncmp(pName, 
					ppNVPair[numNVPair], 
					sizeQueryName) == 0)
				{
					// found it
					return TRUE;
				}
			}
		}
	}

	return FALSE;
} // DECmdNVPIsUTF8ParamPresent


//---------------------------------------------------------------------
// DECmdControlDetach
//---------------------------------------------------------------------
void DECmdControlDetach(_INOUT DECmdControl *pDCC)
{

	// sanity check
	if (pDCC != NULL)
	{
		// close cmd help request
		if (SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD) ||
			SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_PARAM))
		{
			// print the help footer
			DECmdPrintHelpFooter(pDCC);
		}

		// close file handle
		if (pDCC->pOutStream != NULL)
		{
			// flush it first
			fflush(pDCC->pOutStream);
			if (pDCC->pOutStream != stdout)
			{
				// close it
				fclose(pDCC->pOutStream);
			}
			// prevent GPFs
			pDCC->pOutStream = NULL;
		}

		// close file handle
		if (pDCC->pErrStream != NULL)
		{
			// flush it first
			fflush(pDCC->pErrStream);
			if (pDCC->pErrStream != stderr)
			{
				// close it
				fclose(pDCC->pErrStream);
			}
			// prevent GPFs
			pDCC->pErrStream = NULL;
		}

		// clear the string buffers
		pDCC->cmdName[0] = '\0';
		pDCC->outFile[0] = '\0';
		pDCC->errFile[0] = '\0';

		// clear the options map
		pDCC->optionsMap = DECMDCTRL_OPT_UNDEFINED;

		// free the error message buffer
		if (pDCC->errMssg.pStr != NULL)
		{
			// free it
			SMSSUTF8StrFree(&pDCC->errMssg);
			// prevent GPFs
			pDCC->errMssg.pStr = NULL;
			// set length
			pDCC->errMssg.strCurLen = 0;
		}

	}
} // DECmdControlDetach


//---------------------------------------------------------------------
// DECmdControlAttach
//---------------------------------------------------------------------
s32 DECmdControlAttach(_IN s32 argc,
					   _IN astring **argv,
					   _INOUT DECmdControl *pDCC,
					   _OUT u32 *pNonNVPCount)
{
	const astring *pVal;
	astring *pStrEntry;
	u32 sizeStr;
	s32 idxParam;
	s32 idxOption;
	s32 rstat;


	SysDbgMsg4(("DECmdControlAttach: entry\n"));

	// sanity check
	if ((argv == NULL) ||
		(pDCC == NULL) ||
		(pNonNVPCount == NULL))
	{
		SysDbgMsg3(("DECmdControlAttach: required parameter is NULL\n"));
		rstat = SM_STATUS_UNSUCCESSFUL;
		_throw error1;
	}

	// init control object
	pDCC->pOutStream = stdout;
	pDCC->pErrStream = stderr;
	pDCC->cmdName[0] = '\0';
	pDCC->outFile[0] = '\0';
	pDCC->errFile[0] = '\0';
	pDCC->optionsMap = DECMDCTRL_OPT_UNDEFINED;
	pDCC->errMssg.pStr = NULL;
	pDCC->errMssg.strCurLen = 0;

	// set default value
	SMMBitMapSetOn(pDCC->optionsMap, 
		DECMDCTRL_OPT_TS_LEVEL1);

	// set the status to print the help
	rstat = SM_STATUS_INVALID_PARAMETER;

	// need at least the command 
	if (argc < 2)
	{
		_throw error1;
	}

	// allocate error message buffer
	SMSSUTF8StrAlloc(&pDCC->errMssg, DECMD_MAX_PARAM_SIZE);

	// always start at 1 to exclude argv 0.
	*pNonNVPCount = 1;

	// second user argument must be the command
	if (strncmp(DECMD_CMD_KEY, 
		argv[1], 
		(u32)strlen(DECMD_CMD_KEY)) == 0)
	{
		// account for this option
		(*pNonNVPCount)++;

		// then get the command name
		pVal = SMNVPGetUTF8ParamValueByUTF8Name(argc, 
			argv, 
			DECMD_CMD_KEY, 
			TRUE);
		if (pVal == NULL)
		{
			// perhaps requesting command listing
			SMMBitMapSetOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD);
		}
		else
		{
			// get the size of the command value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if ((sizeStr <= 1) ||
				(sizeStr > DECMD_MAX_PARAM_SIZE))
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"command name length out of range [1..." PRT_U32 "]", 
					DECMD_MAX_PARAM_SIZE);
			}
			else
			{
				// perhaps requesting command listing
				// check for 'command=?' exclusive where chars after ? invalidates.
				if (strncmp("?", pVal, 2) == 0)
				{
					SMMBitMapSetOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD);
				}
				else
				{
					// copy it to the control object
					strncpy(pDCC->cmdName, pVal, sizeStr);
				}
			}
		}
	}

	// now find the first option
	for (idxParam = *pNonNVPCount; idxParam < argc; idxParam++)
	{
		if (strncmp(argv[idxParam], "-", 1) == 0)
		{
			// found the first option
			break;
		}
	}

	// update non-NVP count
	*pNonNVPCount += (argc - idxParam);

	// check if we have no options and command name is not specified
	// then the user input is not recognized, print help
	if ((*pNonNVPCount == 1) &&
		(pDCC->cmdName[0] == '\0'))
	{
		_throw error1;
	}

	// now browse the options
	for (idxOption = idxParam; idxOption < argc; idxOption++)
	{
		// get reference to current arg
		pStrEntry = argv[idxOption];

		// help request
		// check for '-?' exclusive where chars after ? invalidates.
		if (strncmp(pStrEntry, "-?", 3) == 0)
		{
			SMMBitMapSetOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_PARAM);
			continue;
		}

		// get stdout option
		idxParam = (u32)(strlen(DECMD_OPT_TS) + 1);
		if (strncmp(pStrEntry, DECMD_OPT_TS, idxParam) == 0)
		{
			// check range
			if (++idxOption >= argc)
			{
				break;
			}

			// get the value
			pVal = argv[idxOption];

			// get the size of the value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if (sizeStr != 2)
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"invalid timestamp level");
			}
			else
			{
				switch(*pVal)
				{
					case '4':
						SMMBitMapSetOn(pDCC->optionsMap, 
							DECMDCTRL_OPT_TS_LEVEL4);
						break;
					case '3':
						SMMBitMapSetOn(pDCC->optionsMap, 
							DECMDCTRL_OPT_TS_LEVEL3);
						break;
					case '2':
						SMMBitMapSetOn(pDCC->optionsMap, 
							DECMDCTRL_OPT_TS_LEVEL2);
						break;
					case '1':
						SMMBitMapSetOn(pDCC->optionsMap, 
							DECMDCTRL_OPT_TS_LEVEL1);
						break;
					default:
						// add error to error buffer
						DECmdControlAddErrorMssgf(pDCC,
							"invalid timestamp level");
						break;
				}
			}

			// stop compare and go to next argument
			continue;
		}

		// get stdout option
		idxParam = (u32)strlen(DECMD_OPT_OUT);
		if (strncmp(pStrEntry, DECMD_OPT_OUT, idxParam) == 0)
		{
			// check range
			if (++idxOption >= argc)
			{
				break;
			}

			// get the value
			pVal = argv[idxOption];

			// get the size of the value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if ((sizeStr <= 1) ||
				(sizeStr > DECMD_MAX_PARAM_SIZE))
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"file name length out of range [1..." PRT_U32 "]", 
					DECMD_MAX_PARAM_SIZE);
			}
			else
			{
				// try opening the file
				if ((pDCC->pOutStream = fopen(pVal, "w")) == NULL)
				{
					// add error to error buffer
					DECmdControlAddErrorMssgf(pDCC,
						"unable to open file %s for writing",
						pVal);
				}
				else
				{
					// copy it to the control object
					strncpy(pDCC->outFile, pVal, sizeStr);
				}
			}

			// stop compare and go to next argument
			continue;
		}

		// try the append version
		idxParam = (u32)strlen(DECMD_OPT_OUTA);
		if (strncmp(pStrEntry, DECMD_OPT_OUTA, idxParam) == 0)
		{
			// check range
			if (++idxOption >= argc)
			{
				break;
			}
			
			// get the value
			pVal = argv[idxOption];
			
			// get the size of the value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if ((sizeStr <= 1) ||
				(sizeStr > DECMD_MAX_PARAM_SIZE))
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"file name length out of range [1..." PRT_U32 "]", 
					DECMD_MAX_PARAM_SIZE);
			}
			else
			{
				// try opening the file
				if ((pDCC->pOutStream = fopen(pVal, "a")) == NULL)
				{
					// add error to error buffer
					DECmdControlAddErrorMssgf(pDCC,
						"unable to open file %s for appending",
						pVal);
				}
				else
				{
					// set the append flag
					SMMBitMapSetOn(pDCC->optionsMap, 
						DECMDCTRL_OPT_STDOUT_APPEND);
					
					// copy it to the control object
					strncpy(pDCC->outFile, pVal, sizeStr);
				}
			}

			// stop compare and go to next argument
			continue;
		}
		
		// get stdout option
		idxParam = (u32)strlen(DECMD_OPT_ERR);
		if (strncmp(pStrEntry, DECMD_OPT_ERR, idxParam) == 0)
		{
			// check range
			if (++idxOption >= argc)
			{
				break;
			}

			// get the value
			pVal = argv[idxOption];

			// get the size of the value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if ((sizeStr <= 1) ||
				(sizeStr > DECMD_MAX_PARAM_SIZE))
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"file name length out of range [1..." PRT_U32 "]", 
					DECMD_MAX_PARAM_SIZE);
			}
			else
			{
				// try opening the file
				if ((pDCC->pErrStream = fopen(pVal, "w")) == NULL)
				{
					// add error to error buffer
					DECmdControlAddErrorMssgf(pDCC,
						"unable to open file %s for writing",
						pVal);
				}
				else
				{
					// copy it to the control object
					strncpy(pDCC->errFile, pVal, sizeStr);
				}
			}

			// stop compare and go to next argument
			continue;
		}

		// try the append version
		idxParam = (u32)strlen(DECMD_OPT_ERRA);
		if (strncmp(pStrEntry, DECMD_OPT_ERRA, idxParam) == 0)
		{
			// check range
			if (++idxOption >= argc)
			{
				break;
			}
			
			// get the value
			pVal = argv[idxOption];
			
			// get the size of the value
			sizeStr = (u32)(strlen(pVal) + 1);
			
			// validate it
			if ((sizeStr <= 1) ||
				(sizeStr > DECMD_MAX_PARAM_SIZE))
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC,
					"file name length out of range [1..." PRT_U32 "]", 
					DECMD_MAX_PARAM_SIZE);
			}
			else
			{
				// try opening the file
				if ((pDCC->pErrStream = fopen(pVal, "a")) == NULL)
				{
					// add error to error buffer
					DECmdControlAddErrorMssgf(pDCC,
						"unable to open file %s for appending",
						pVal);
				}
				else
				{
					// set the append flag
					SMMBitMapSetOn(pDCC->optionsMap, 
						DECMDCTRL_OPT_STDERR_APPEND);
					
					// copy it to the control object
					strncpy(pDCC->errFile, pVal, sizeStr);
					
					// account for this option
					*pNonNVPCount += 2;
				}
			}

			// stop compare and go to next argument
			continue;
		}

		// no known option matched
		DECmdControlAddErrorMssgf(pDCC,
			"invalid option %s",
			pStrEntry);
	}

	// check if we have error messages in the buffer
	if (pDCC->errMssg.strCurLen == 0)
	{
		rstat = SM_STATUS_SUCCESS;
		
		// cmd help requested
		if (SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD) ||
			SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_PARAM))
		{
			// print the help header
			DECmdPrintHelpHeader(pDCC);
			// print the usage
			DECmdPrintHelpUsage(pDCC, argc, argv);
		}
	}
	else
	{
		// turn off option help flag since error message exist
		SMMBitMapSetOff(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD);
		SMMBitMapSetOff(pDCC->optionsMap, DECMDCTRL_OPT_HELP_PARAM);
	}

 error1:
	SysDbgMsg4(("DECmdControlAttach: exit status(" PRT_S32 ")\n", rstat));
	return rstat;
} // DECmdControlAttach


//---------------------------------------------------------------------
// DECmdCDTEntryCmdNameCompare
//---------------------------------------------------------------------
int __cdecl DECmdCDTEntryCmdNameCompare(const void *pElem1,
										const void *pElem2)
{
    return strcmp(((DECmdDispatchTable *)pElem1)->pCmdName, 
				  ((DECmdDispatchTable *)pElem2)->pCmdName);
} // DECmdCDTEntryCmdNameCompare


//---------------------------------------------------------------------
// DECmdCDTCmdNameCompare
//---------------------------------------------------------------------
int __cdecl DECmdCDTCmdNameCompare(const void *pElem1, 
								   const void *pElem2)
{
    return strcmp((astring *)pElem1, 
				  ((DECmdDispatchTable *)pElem2)->pCmdName);
} // DECmdCDTCmdNameCompare


//---------------------------------------------------------------------
// DECmdSortCmdDispatchTable
//---------------------------------------------------------------------
s32 DECmdSortCmdDispatchTable(void)
{
	DECmdDispatchTable *pCDT;
	s32 cdtCount;


	if ((pCDT = NVPCmdGetCDT(&cdtCount)) != NULL)
	{
		// sort it alphabetically
		qsort(pCDT, 
			cdtCount, 
			sizeof(*pCDT), 
			DECmdCDTEntryCmdNameCompare);

		return SM_STATUS_SUCCESS;
	}

	return SM_STATUS_UNSUCCESSFUL;
} // DECmdSortCmdDispatchTable


//---------------------------------------------------------------------
// DECmdSendCmd
//---------------------------------------------------------------------
s32 DECmdSendCmd(_IN const DECmdControl *pDCC,
				 _IN s32 numNVPair, 
				 _IN astring **ppNVPair)
{
	DECmdDispatchTable *pCDTable;
	DECmdDispatchTable *pCDEntry;
	s32 numElements;
	s32 rstat;
	s32 idx;


	SysDbgMsg4(("DECmdSendCmd: entry\n"));

	// get Dispatch table
	if ((pCDTable = NVPCmdGetCDT(&numElements)) == NULL)
	{
		// add error to error buffer
		DECmdControlAddErrorMssgf(pDCC,
			"cannot get dispatch table");
		
		rstat = SM_STATUS_UNSUCCESSFUL;
	}	
	else
	{
		// cmd help request
		if (SMMBitMapIsOn(pDCC->optionsMap, DECMDCTRL_OPT_HELP_CMD))
		{
			// display command help intro
			DECmdOutf(pDCC, DECMD_HELPSTR_COMMANDS_FMT);
			
			for (idx = 0; idx < numElements; idx++)
			{
				// add to command options
				DECmdOutf(pDCC, "  " DECMD_CMD_KEY "=%s\n", 
					pCDTable[idx].pCmdName);
			}			

			rstat = SM_STATUS_SUCCESS;
		}
		else
		{
			// search the table for cmd match
			pCDEntry = (DECmdDispatchTable *)bsearch(pDCC->cmdName, 
				pCDTable, 
				numElements, 
				sizeof(*pCDTable), 
				DECmdCDTCmdNameCompare);
			if (pCDEntry == NULL)
			{
				// add error to error buffer
				DECmdControlAddErrorMssgf(pDCC, "command %s not found\n",
					pDCC->cmdName);
				
				rstat = SM_STATUS_INVALID_PARAMETER;
			}
			else
			{
				// found it, send the command!
				rstat = pCDEntry->pfnDECmdSendCmd(pDCC, 
					numNVPair, 
					ppNVPair);
			}
		}
	}

	SysDbgMsg4(("DECmdSendCmd: exit status(" PRT_S32 ")\n", rstat));
	return rstat;
} // DECmdSendCmd


//---------------------------------------------------------------------
// DECmdFunctionIteratorNum
//---------------------------------------------------------------------
void DECmdFunctionIteratorNum(_IN const DECmdControl *pDCC,
							  _IN PFNDECITERATORFUNC pfnIterateCode,
							  _INOUT void *pIterateData,
							  _OUT s32 *pStatus,
							  _IN s32 numiterate,
							  _IN u32 pausemsecs)
{
	s32 rstat;
	s32 iteration;
	s32 maxIteration;


	// prevent netware warning, remove when in use
	UNREFERENCED_PARAMETER(pDCC);

	// init status
	rstat = SM_STATUS_SUCCESS;

	// init iteration
	iteration = 0;

	maxIteration = numiterate;
	if (numiterate < 0)
	{
		maxIteration = ULIMIT_NUM_ITERATE;
	}

	// start the work
	while (!DECmdIsModuleExiting())
	{
		// infinite iteration?
		if (numiterate < 0)
		{
			// reset to 0 when max positive value reached
			if (iteration == ULIMIT_NUM_ITERATE)
			{
				iteration = 0;
			}
		}
		else
		{
			if (iteration >= numiterate)
			{
				break;
			}
		}

		// do the work
		rstat = pfnIterateCode(pIterateData,
			iteration,
			maxIteration);

		// exit if fail status
		if (rstat != SM_STATUS_SUCCESS)
		{
			_throw error1;
		}

		SMThreadYield();

		// pause current thread as requested
		DECmdModExitListenerPause(pausemsecs);

		// loop
		iteration++;
	}

 error1:
	SysDbgMsg4(("DECmdFunctionIteratorNum: exit status(" PRT_S32 ")\n",
		rstat));
	if (pStatus != NULL)
	{
		*pStatus = rstat;
	}
	return;
} // DECmdFunctionIteratorNum


//---------------------------------------------------------------------
// DECmdFunctionIterator
//---------------------------------------------------------------------
void DECmdFunctionIterator(_IN const DECmdControl *pDCC,
						   _IN s32 numNVPair, 
						   _IN astring **ppNVPair,
						   _IN PFNDECITERATORFUNC pfnIterateCode,
						   _INOUT void *pIterateData,
						   _OUT s32 *pStatus,
						   _IN s32 numIterateMin,
						   _IN s32 numIterateMax,
						   _IN s32 numIterateDefault)
{
	u32 tSize;
	u32 pausemsecs;
	s32 rstat;
	s32 numiterate;


	SysDbgMsg4(("DECmdFunctionIterator: entry\n"));

	// sanity check
	if ((pDCC == NULL) ||
		(pfnIterateCode == NULL))
	{
		SysDbgMsg3(("DECmdFunctionIterator: NULL required input parameter\n"));
		rstat = SM_STATUS_INVALID_PARAMETER;
		_throw error1;
	}

	// get user parameter numiterate
	tSize = sizeof(numiterate);
	rstat = SMNVPUTF8ToTypeValue(numNVPair, 
		ppNVPair,
		DECMDPRM_NUMITERATE, 
		TRUE, 
		&numiterate, 
		&tSize,
		SM_XLT_VTYPE_ASTRING_TO_s32);
	if (rstat != SM_STATUS_SUCCESS)
	{
		// set to default
		numiterate = numIterateDefault;
	}
	else
	{
		if (numiterate >= 0)
		{
			if ((numiterate < numIterateMin) ||
				((numIterateMax > 0) && 
				(numiterate > numIterateMax)))
			{
				DECmdErrf(pDCC,
					"Parameter %s exceeds limits min: " PRT_S32 " max: " PRT_S32 "\n",
					DECMDPRM_NUMITERATE,
					numIterateMin,
					numIterateMax);
				rstat = SM_STATUS_INVALID_PARAMETER;
				_throw error1;
			}
		}
	}

	// get user parameter pausemsecs
	tSize = sizeof(pausemsecs);
	rstat = SMNVPUTF8ToTypeValue(numNVPair, 
		ppNVPair,
		DECMDPRM_PAUSEMSECS, 
		TRUE, 
		&pausemsecs, 
		&tSize,
		SM_XLT_VTYPE_ASTRING_TO_s32);
	if (rstat != SM_STATUS_SUCCESS)
	{
		// set to default
		pausemsecs = 0;
	}

	// execute it
	DECmdFunctionIteratorNum(pDCC,
		pfnIterateCode,
		pIterateData,
		pStatus,
		numiterate,
		pausemsecs);

	// good at this point
	SysDbgMsg4(("DECmdFunctionIterator: exit\n"));
	return;

 error1:
	SysDbgMsg4(("DECmdFunctionIterator: exit\n"));
	if (pStatus != NULL)
	{
		*pStatus = rstat;
	}
	return;
} // DECmdFunctionIterator


//---------------------------------------------------------------------
// DECmdNThreadDestroy
//---------------------------------------------------------------------
void DECmdNThreadDestroy(_IN void *pNTHandle)
{
	NThreadControl *pNTC = NULL;
	void *pObjectLock;
	u32 ii;


	SysDbgMsg4(("DECmdNThreadDestroy: entry\n"));

	// sanity check
	if (pNTHandle != NULL)
	{
		// cast to internal 
		pNTC = (NThreadControl *)pNTHandle;

		// lock the object
		pObjectLock = pNTC->pObjectLock;
		SMMutexLock(pObjectLock,
			SM_WAIT_MSECS_INFINITE);

		// NOTE: assumes that only 1 thread will call this function
		// invalidate lock, so that others cannot use it
		pNTC->pObjectLock = NULL;

		// walk the thread table to stop running threads
		for (ii = 0; ii < pNTC->threadCount; ii++)
		{
			if (pNTC->ntcd[ii].pNThreadHandle != NULL)
			{
				// Threads should have exited by this time
				// call thread stop to free resource allocated
				// during thread start.
				SMThreadStop(pNTC->ntcd[ii].pNThreadHandle);

				// prevent GPFs
				pNTC->ntcd[ii].pfnNThreadCode = NULL;
				pNTC->ntcd[ii].pNThreadData = NULL;
				pNTC->ntcd[ii].pNThreadHandle = NULL;
				pNTC->ntcd[ii].pThreadControl = NULL;
			}
		}

		// clear the count
		pNTC->threadCount = 0;
		// clear the DCC
		pNTC->pDCC = NULL;

		// free exit thread event
		if (pNTC->pThreadCtrlExitEvt != NULL)
		{
			// free it
			SMSemaphoreDestroy(pNTC->pThreadCtrlExitEvt);
			// prevent GPFs
			pNTC->pThreadCtrlExitEvt = NULL;
		}

		SMMutexUnLock(pObjectLock);
		SMMutexDestroy(pObjectLock);
		pObjectLock = NULL;

		// finally free this handle
		SMFreeMem(pNTC);
		// prevent GPFs
		pNTC = NULL;
		pNTHandle = NULL;
	}

	SysDbgMsg4(("DECmdNThreadDestroy: exit\n"));
} // DECmdNThreadDestroy


//---------------------------------------------------------------------
// DECmdNThreadJoin
//---------------------------------------------------------------------
void DECmdNThreadJoin(_IN void *pNTHandle,
					  _IN s32 threadExitTimeOutMSecs)
{
	NThreadControl *pNTC;


	SysDbgMsg4(("DECmdNThreadJoin: entry\n"));


	// caller guarantees handle is valid
	// otherwise result is undefined
	if (pNTHandle != NULL)
	{
		// get internal object
		pNTC = (NThreadControl *)pNTHandle;

		// lock the object
		SMMutexLock(pNTC->pObjectLock,
			SM_WAIT_MSECS_INFINITE);

		// wait for all threads to signal exit
		while (pNTC->startedThreadCount > 0)
		{
			SysDbgMsg4(("DECmdNThreadJoin: count: " PRT_U32 "\n",
				pNTC->startedThreadCount));

			// unlock the object
			SMMutexUnLock(pNTC->pObjectLock);

			// wait for a thread to signal
			if (SMSemaphoreAcquireWait(pNTC->pThreadCtrlExitEvt, 
				threadExitTimeOutMSecs) != SM_STATUS_SUCCESS)
			{
				SysDbgMsg3(("DECmdNThreadJoin: " \
					"WARNING: timeout waiting for threads to exit, active threads: " PRT_U32 "\n",
					pNTC->startedThreadCount));

				// check if module is exiting perhaps due to user break
				if (DECmdIsModuleExiting() == TRUE)
				{
					SysDbgMsg3(("DECmdNThreadJoin: " \
						"module exit is flagged, will not wait for threads to signal\n"));
					_throw error1;
				}
			}

			// Yield Control. Since Netware misbehaves
			SMThreadYield();

			// lock the object
			SMMutexLock(pNTC->pObjectLock,
				SM_WAIT_MSECS_INFINITE);
		}

		// unlock the object
		SMMutexUnLock(pNTC->pObjectLock);
	}

 error1:
	SysDbgMsg4(("DECmdNThreadJoin: exit\n"));

	return;
} // DECmdNThreadJoin


//---------------------------------------------------------------------
// DECmdNThreadRunner
//---------------------------------------------------------------------
void DECmdNThreadRunner(_IN void *pSMThreadData)
{
	NThreadControlData *pNTCD;


	// cast to internal object
	pNTCD = (NThreadControlData *)pSMThreadData;

	SysDbgMsg4(("DECmdNThreadRunner: entry thread handle: %p, index: " PRT_S32 "\n",
		pNTCD->pNThreadHandle,
		pNTCD->threadIndex));

	// execute it
	pNTCD->pfnNThreadCode(pNTCD->pNThreadData,
		pNTCD->threadIndex,
		pNTCD->maxThreadIndex);

	SysDbgMsg4(("DECmdNThreadRunner: exit signaling thread control, index: " PRT_S32 "\n",
		pNTCD->threadIndex));

	// lock the object
	SMMutexLock(pNTCD->pThreadControl->pObjectLock,
		SM_WAIT_MSECS_INFINITE);

	// decrement started thread count
	pNTCD->pThreadControl->startedThreadCount--;

	// unlock the object
	SMMutexUnLock(pNTCD->pThreadControl->pObjectLock);

	// signal this thread is exiting
	SMSemaphoreRelease(pNTCD->pThreadControl->pThreadCtrlExitEvt,
		1,
		NULL);

	// NOTE: Do not add any more code after semaphore release
	// since thread data may have been freed during the signal
	return;
} // DECmdNThreadRunner


//---------------------------------------------------------------------
// DECmdNThreadCreateNum
//---------------------------------------------------------------------
void * DECmdNThreadCreateNum(_IN const DECmdControl *pDCC,
							 _IN PFNDECNTHREADFUNC pfnNThreadCode,
							 _INOUT void *pNThreadData,
							 _OUT s32 *pStatus,
							 _IN s32 numThread)
{
	NThreadControl *pNTC = NULL;
	u32 tSize;
	s32 rstat;
	s32 ii;


	SysDbgMsg4(("DECmdNThreadCreateNum: entry\n"));

	// caller guarantees input is valid

	// set status
	rstat = SM_STATUS_UNSUCCESSFUL;

	// set size of control object
	tSize = (u32)(sizeof(NThreadControl));
	tSize += (u32)(sizeof(NThreadControlData) * (numThread - 1));

	// allocate the control object
	pNTC = (NThreadControl *)SMAllocMem(tSize);
	if (pNTC == NULL)
	{
		// out of memory, just exit
		rstat = SM_STATUS_OUT_OF_MEMORY;
		_throw error1;
	}

	// init thread control
	pNTC->pDCC = pDCC;
	pNTC->threadCount = 0;
	pNTC->startedThreadCount = 0;
	pNTC->pThreadCtrlExitEvt = NULL;
	pNTC->pObjectLock = NULL;

	// create the object lock
	pNTC->pObjectLock = SMMutexCreate(NULL);
	if (pNTC->pObjectLock == NULL)
	{
		_throw error2;
	}

	// lock the object
	SMMutexLock(pNTC->pObjectLock,
		SM_WAIT_MSECS_INFINITE);

	// create and run the threads
	for (ii = 0; ii < numThread; ii++)
	{
		// init thread parameter
		pNTC->ntcd[ii].pfnNThreadCode = pfnNThreadCode;
		pNTC->ntcd[ii].pNThreadData = pNThreadData;
		pNTC->ntcd[ii].threadIndex = ii;
		pNTC->ntcd[ii].maxThreadIndex = numThread;
		pNTC->ntcd[ii].pThreadControl = pNTC;

		// execute the thread
		pNTC->ntcd[ii].pNThreadHandle = SMThreadStart(DECmdNThreadRunner,
			&pNTC->ntcd[ii]);

		// validate
		if (pNTC->ntcd[ii].pNThreadHandle != NULL)
		{
			// increment number thread successfully started
			pNTC->startedThreadCount++;
		}
		else
		{
			DECmdErrf(pDCC, "Failed to create thread index: " PRT_S32 "\n",
				ii);
		}

		// increment thread count
		pNTC->threadCount++;
	}

	// create the exit thread event
	// init semaphore with 0 
	pNTC->pThreadCtrlExitEvt = SMSemaphoreCreate(NULL, 0);

	// validate
	if (pNTC->pThreadCtrlExitEvt == NULL)
	{
		DECmdErrf(pDCC, "SMSemaphoreCreate failed on pNTC->pThreadCtrlExitEvt\n");
		_throw error3;
	}

	// unlock the object
	SMMutexUnLock(pNTC->pObjectLock);

	// success at this point
	SysDbgMsg4(("DECmdNThreadCreateNum: exit, threads: " PRT_U32 ", started: " PRT_U32 "\n",
		numThread,
		pNTC->startedThreadCount));
	// update user status
	if (pStatus != NULL)
	{
		*pStatus = SM_STATUS_SUCCESS;
	}

	return pNTC;

 error3:
	// unlock the object
	SMMutexUnLock(pNTC->pObjectLock);
 error2:
	// stop running threads and free handle
	DECmdNThreadDestroy(pNTC);
	// prevent GPFs
	pNTC = NULL;
 error1:
	SysDbgMsg4(("DECmdNThreadCreateNum: exit with errors\n"));
	// update user status
	if (pStatus != NULL)
	{
		*pStatus = rstat;
	}
	return NULL;
} // DECmdNThreadCreateNum


//---------------------------------------------------------------------
// DECmdNThreadCreate
//---------------------------------------------------------------------
void * DECmdNThreadCreate(_IN const DECmdControl *pDCC,
						  _IN s32 numNVPair,
						  _IN astring **ppNVPair,
						  _IN PFNDECNTHREADFUNC pfnNThreadCode,
						  _INOUT void *pNThreadData,
						  _OUT s32 *pStatus,
						  _IN s32 numThreadMin,
						  _IN s32 numThreadMax,
						  _IN s32 numThreadDefault)
{
	NThreadControl *pNTC = NULL;
	s32 numthread;
	s32 rstat;
	u32 tSize;


	SysDbgMsg4(("DECmdNThreadCreate: entry\n"));

	// sanity check
	if ((pDCC == NULL) ||
		(pfnNThreadCode == NULL))
	{
		SysDbgMsg3(("DECmdNThreadCreate: NULL required input parameter\n"));
		rstat = SM_STATUS_UNSUCCESSFUL;
		_throw error1;
	}

	// get user parameter numthread
	tSize = sizeof(numthread);
	if (SMNVPUTF8ToTypeValue(numNVPair, 
		ppNVPair,
		DECMDPRM_NUMTHREADS, 
		TRUE, 
		&numthread, 
		&tSize,
		SM_XLT_VTYPE_ASTRING_TO_s32) != SM_STATUS_SUCCESS)
	{
		// set to default
		numthread = numThreadDefault;
	}
	else
	{
		if (numthread < 0)
		{
			// set to maximum
			numthread = numThreadMax;
		}
		else
		{
			if (numthread >= 0)
			{
				if ((numthread < numThreadMin) ||
					((numThreadMax > 0) &&
					(numthread > numThreadMax)))
				{
					DECmdErrf(pDCC,
						"Parameter %s exceeds limits min: " PRT_S32 " max: " PRT_S32 "\n",
						DECMDPRM_NUMTHREADS,
						numThreadMin,
						numThreadMax);
					rstat = SM_STATUS_INVALID_PARAMETER;
					_throw error1;
				}
			} // if (numthread >= 0)
		} // if (numthread < 0)
	} // if (rstat != SM_STATUS_SUCCESS)

	pNTC = DECmdNThreadCreateNum(pDCC,
		pfnNThreadCode,
		pNThreadData,
		pStatus,
		numthread);

	SysDbgMsg4(("DECmdNThreadCreate: exit\n"));
	return pNTC;

 error1:
	SysDbgMsg4(("DECmdNThreadCreate: exit\n"));
	// update status as requested
	if (pStatus != NULL)
	{
		*pStatus = rstat;
	}
	return NULL;
} // DECmdNThreadCreate


//---------------------------------------------------------------------
// CMDHello
//---------------------------------------------------------------------
s32 CMDHello(_IN const DECmdControl *pDCC, 
			 _IN s32 numNVPair, 
			 _IN astring **ppNVPair)
{
	astring *pDot;
	astring moduleName[64];
	static DECmdHelpObject helpObj = 
	{
		"",
		NULL,
		NULL,
		0
	};


	// prevent netware warning, remove when in use
	UNREFERENCED_PARAMETER(numNVPair);
	UNREFERENCED_PARAMETER(ppNVPair);

	// process cmd help request
	DECmdPrintHelpCmdParamList(pDCC, 
		&helpObj);

	// init the buffer
	moduleName[0] = '\0';
	// copy it locally
	strncpy(moduleName, PRD_VER_ORIGINALFILENAME_STR, 64);
	// remove the file suffix
	if ((pDot = strstr(moduleName, ".")) != NULL)
	{
		*pDot = '\0';
	}

	// display a message
	DECmdOutf(pDCC, "Hello from %s\n", moduleName);

	return SM_STATUS_SUCCESS;
} // CMDHello


//---------------------------------------------------------------------
// Scripting Code Additions
//---------------------------------------------------------------------
// The following would probably be better located in a different module,
// but that would meam modifying a bunch of make files for all OSs.

#define NULCHAR    '\0'
#define SPACECHAR  ' '
#define TABCHAR    '\t'
#define DQUOTECHAR '\"'
#define SLASHCHAR  '\\'


/*******************************************************************************
 *static void parse_cmdline(pCmdStart, argv, pArgs, pNumArgs, pNumChars)
 *
 *Purpose: Parses the line from a script and creates a facsimile of 
 *		  the argv array to pass back into the program. If argv and args
 *		  are NULL, only a count is performed and the correct results 
 *		  are returned in numargs and pNumChars. The count returned by numargs 
 *		  includes the terminating null pointer and the count returned by
 *		  pNumChars includes the terminating null char.
 *		  This parse is very similar to MS CRT and Linux, but double quotes
 *		  are handled differently in NetWare. NetWare style quoting should not
 *		  be used in DECMD based script files.
 *
 *Entry:
 *       astring* pCmdStart - pointer to command line of the form
 *           <progname><nul><args><nul>
 *       astring** argv - where to build argv array; NULL means don't
 *                       build array, only count
 *       astring* pArgs - where to place argument text; NULL means don't
 *                       store text, only count
 *
 *Exit:
 *       no return value
 *       int *numargs - returns number of argv entries created
 *       int *pNumChars - number of characters used in args buffer
 *
 *Exceptions:
 *
 ******************************************************************************/
static void parse_cmdline(_IN astring* pCmdStart,
						  _INOUT astring** argv,
						  _INOUT astring* pArgs,
						  _OUT s32* pNumArgs,
						  _OUT s32* pNumChars)
{
        astring* p;
        astring c;
        s32 inquote;                    // 1 = inside quotes
        s32 copychar;                   // 1 = copy char to *pArgs
        unsigned numslash;              // num of backslashes seen

        *pNumChars = 0;
        *pNumArgs = 1;                   // the program name at least 

        // first scan the program name, copy it, and count the bytes
        p = pCmdStart;
        if (argv)
		{
            *argv++ = pArgs;
		}

        // the processing of the program name is simple because it has to be 
		// a valid file name and may not have any enclosed double quotes.
		// double quotes are stripped and not counted.
        if (*p == DQUOTECHAR) 
		{
            // scan from just past the first double-quote through the next
            // double-quote, or up to a null, whichever comes first
            while ((*(++p) != DQUOTECHAR) && (*p != NULCHAR)) 
			{
                ++*pNumChars;
                if (pArgs)
				{
                    *pArgs++ = *p;
				}
            }
            // append the terminating null
            ++*pNumChars;
            if (pArgs)
			{
                *pArgs++ = NULCHAR;
			}

            // if we stopped on a double-quote (usual case), skip over it
            if (*p == DQUOTECHAR)
			{
                p++;
			}
        }
        else 
		{
            // Not a quoted program name
            do 
			{
                ++*pNumChars;
                if (pArgs)
				{
                    *pArgs++ = *p;
				}

                c = (astring)*p++;

            } while (c != SPACECHAR && c != NULCHAR && c != TABCHAR);

            if (c == NULCHAR) 
			{
                p--;
            } 
			else 
			{
                if (pArgs)
				{
                    *(pArgs-1) = NULCHAR;
				}
            }
        }

		p++;
        inquote = 0;

        // loop on each argument
        for(;;) 
		{
            if (*p) 
			{
                while (*p == SPACECHAR || *p == TABCHAR) // eat up whitespace
				{
                    ++p;
				}
            }

            if (*p == NULCHAR)		// we hit the end 
			{
                break;
			}

            // scan an argument
            if (argv)
			{
                *argv++ = pArgs;     // store ptr to arg
			}
            ++*pNumArgs;


        // now scan one argument
        for (;;) 
		{
            copychar = 1;
            // Rules: 2N backslashes + " ==> N backslashes and begin/end quote
            // 2N+1 backslashes + " ==> N backslashes + literal "
            // N backslashes ==> N backslashes
            numslash = 0;
            while (*p == SLASHCHAR) 
			{
                // count number of backslashes for use below
                ++p;
                ++numslash;
            }
            if (*p == DQUOTECHAR) 
			{
                // if 2N backslashes before, start/end quote, otherwise copy literally
                if (numslash % 2 == 0) 
				{
                    if (inquote) 
					{
                        if (p[1] == DQUOTECHAR)
						{
                            p++;    // Double quote inside quoted string
						}
                        else        // skip first quote char and copy second
						{
                            copychar = 0;
						}
                    } 
					else
					{
                        copychar = 0;       // don't copy quote
					}

                    inquote = !inquote;
                }
                numslash /= 2;          // divide numslash by two
            }

            // copy slashes
            while (numslash--) 
			{
                if (pArgs)
				{
                    *pArgs++ = SLASHCHAR;
				}
                ++*pNumChars;
            }

            // if at end of arg, break loop
            if (*p == NULCHAR || (!inquote && (*p == SPACECHAR || *p == TABCHAR)))
			{
                break;
			}

            // copy character into argument
            if (copychar) 
			{
                if (pArgs)
				{
                    *pArgs++ = *p;
				}
                ++*pNumChars;
            }
            ++p;
		}

        // null-terminate the argument

        if (pArgs)
		{
            *pArgs++ = NULCHAR;          // terminate string
		}
        ++*pNumChars;
    }

    // null-terminate the array of char pointers
    if (argv)
	{
        *argv++ = NULL;
	}
    ++*pNumArgs;
} // parse_cmdline


/*******************************************************************************
 * This function takes a pointer to a char* array and a string and parses the
 * string into parameters.  The structure is similar the the argv array. Each
 * pointer points to a single parameter.
 *
 * Parses a line from the script file into replacement arguments for argc & argv
 * a null char*** pointer is passed to recieve the new vector. This memory
 * should be freed by calling SMFreeMem(&strv[0]);
 ******************************************************************************/
s32 CmdScriptParseLine(_IN astring* argv0, 
					   _IN astring* pScriptLine, 
					   _INOUT s32* pCount, 
					   _INOUT astring*** strv)
{
	s32 retval = SM_STATUS_SUCCESS;
	s32 paramLen = 0;
	s32 paramCount = 0;
	u32 cmdStartSize = 0;
	astring* pParamStrings = 0;
	astring** ppArgs = 0;
	astring* pCmdStart = 0;

	
	SysDbgMsg4(("CmdScriptParseLine(%s, %s, %d, strv)\n", 
		argv0, 
		pScriptLine, 
		*pCount));

	if(!pScriptLine)
	{
		// pScriptLine is a null ptr, this is bad(tm)
		retval = SM_STATUS_INVALID_PARAMETER;
		_throw exit;
	}
	
	cmdStartSize = (u32)(strlen(argv0) + strlen(pScriptLine) + 4);
	pCmdStart = SMAllocMem(cmdStartSize);
	if(NULL == pCmdStart)
	{
		retval = SM_STATUS_OUT_OF_MEMORY;
		_throw exit;
	}

	sprintf_s(pCmdStart, cmdStartSize, "\"%s\"", argv0);
	strncpy(pCmdStart + strlen(pCmdStart) + 1,
				pScriptLine,
				cmdStartSize - strlen(pCmdStart) - 1);

	parse_cmdline(pCmdStart, 
		0, 
		0, 
		&paramCount, 
		&paramLen);

	// get a memory chunk big enough to hold the array and the param strings
	// array will include a ptr to argv[0] and a null ptr to terminate the array, thus + 2
	ppArgs = SMAllocMem((paramCount)*sizeof(char*) + paramLen);

	if(!ppArgs)
	{
		// handle no memory
		retval = SM_STATUS_OUT_OF_MEMORY;
		_throw exit;
	}

	memset(&ppArgs[0], 
		0, 
		(paramCount)*sizeof(char*) + paramLen);
	// ppArgs[0] = argv0;
	pParamStrings = (char*)(&(*ppArgs) + paramCount);
	parse_cmdline(pCmdStart, 
		ppArgs, 
		pParamStrings, 
		&paramCount, 
		&paramLen);
	*strv = ppArgs;
	*pCount = paramCount-1;

	if(NULL != pCmdStart)
	{
		SMFreeMem(pCmdStart);
		pCmdStart = NULL;
	}

exit:

	return retval;
} // CmdScriptParseLine


//---------------------------------------------------------------------
// DECmdOutNVLinef
//---------------------------------------------------------------------
void DECmdOutNVLinef(_IN const DECmdControl *pDCC,
					 _IN const astring *pName,
					 _IN s32 nameRegionLen,
					 _IN const astring *pValueFmt,
					 _IN ...)
{
	astring *pCursor;
	va_list vap;
	astring lineBuf[64];
	s32 nameSize;
	s32 ii;

	// no sanity check
	// assume all arguments are valid

	// copy name to buffer
	nameSize = SMsnprintf(lineBuf, 
		63,
		"%s",
		pName);
	// determine nameregionlen
	if (64 <= nameRegionLen)
	{
		nameRegionLen = 63;
	}
	// fill with spaces
	pCursor = &lineBuf[nameSize];
	for (ii = nameSize; nameSize < nameRegionLen; nameSize++)
	{
		*pCursor = ' ';
		pCursor++;
	}
	// terminate it
	*pCursor = '\0';
	DECmdOutf(pDCC,
		"%s",
		lineBuf);
	// display value
	va_start(vap, pValueFmt);
	DECmdOutErrf(pDCC, 
		FALSE,
		(DECMDOUT_TS_LEVEL4), 
		(DECMDOUT_TS_LEVEL4), 
		pValueFmt, 
		vap);
	va_end(vap);

	return;
} // DECmdOutNVLinef


//---------------------------------------------------------------------
// NNSVContainsString
//---------------------------------------------------------------------
booln NNSVContainsString(_IN astring *pNNSV,
						 _IN const astring *pString,
						 _IN booln bIgnoreCase)
{

	if (NULL != pNNSV)
	{
		while (*pNNSV != '\0')
		{
			if (bIgnoreCase == TRUE)
			{
				if (stricmp(pNNSV, pString) == 0)
				{
					return TRUE;
				}
			}
			else
			{
				if (strcmp(pNNSV, pString) == 0)
				{
					return TRUE;
				}
			}
			// go to next list element
			pNNSV += (strlen(pNNSV) + 1);
		}
	}

	return FALSE;
} // NNSVContainsString


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

