
//-------------------------------------------------------------------------
//
//                 Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author / Created:
//    Chris Poblete / 2004 May 05
//
//  Environment:
//    Linux, Netware, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// SDK includes
//---------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsgenapi.h>


//---------------------------------------------------------------------
// Standard C includes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Local includes
//---------------------------------------------------------------------
#include <decmd.h>


//---------------------------------------------------------------------
// Local definitions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Global data
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// DECmdIsDataManagerReady
//
// if waitTimeOutMilliSecs != 0, then wait
// if waitTimeOutMilliSecs  < 0, then infinite wait
// if waitTimeOutMilliSecs == 0, then no wait
// if pingTimeOutMilliSecs <= waitTimeOutMilliSecs and
//    pingTimeOutMilliSecs != 0, then ping while wait
//
//---------------------------------------------------------------------
s32 DECmdIsDataManagerReady(_IN const DECmdControl *pDCC,
							_IN s32 numNVPair, 
							_IN astring **ppNVPair)
{
	s32 rstat;
	s32 sleepTimeMilliSecs;
	s32 counter;
	s32 waitTimeOutMilliSecs;
	s32 pingTimeOutMilliSecs;
	booln bIsReady;


	// init
	rstat = SM_STATUS_SUCCESS;

	// get input
	counter = (s32)sizeof(waitTimeOutMilliSecs);
	if (SMNVPUTF8ToTypeValue(numNVPair, 
		ppNVPair,
		"smiltimeout", 
		TRUE, 
		&waitTimeOutMilliSecs, 
		(u32 *)&counter,
		SM_XLT_VTYPE_ASTRING_TO_s32) != SM_STATUS_SUCCESS)
	{
		// set to default
		waitTimeOutMilliSecs = 0;
	}

	// get input
	counter = (s32)sizeof(pingTimeOutMilliSecs);
	if (SMNVPUTF8ToTypeValue(numNVPair, 
		ppNVPair,
		"smilping", 
		TRUE, 
		&pingTimeOutMilliSecs, 
		(u32 *)&counter,
		SM_XLT_VTYPE_ASTRING_TO_s32) != SM_STATUS_SUCCESS)
	{
		// set to default
		pingTimeOutMilliSecs = 0;
	}

	// check Data Manager
	bIsReady = SMILIsDataManagerReady();

	// wait?
	if ((bIsReady == FALSE) &&
		(waitTimeOutMilliSecs != 0))
	{
		// ping while waiting?
		if ((pingTimeOutMilliSecs != 0) && 
			(pingTimeOutMilliSecs < waitTimeOutMilliSecs))
		{
			// set sleep time to ping time
			sleepTimeMilliSecs = pingTimeOutMilliSecs;
		}
		else
		{
			// is the infinite flag on
			if (waitTimeOutMilliSecs < 0)
			{
				// set predefined sleep time
				sleepTimeMilliSecs = 3000;
				// set predefined ping time
				pingTimeOutMilliSecs = 0;
				// reset waitTimeOutMilliSecs so we loop infinitely
				waitTimeOutMilliSecs = sleepTimeMilliSecs + 1;
			}
			else
			{
				// set sleep time to wait time
				sleepTimeMilliSecs = waitTimeOutMilliSecs;
				// set ping time equal to wait time
				pingTimeOutMilliSecs = waitTimeOutMilliSecs;
			}
		}

		DECmdErrf(pDCC, "Trying to connect to Data Manager...");

		// set counter time
		counter = sleepTimeMilliSecs;

		do
		{
			// Using module exit event to make sure we do not block
			// for a long time when user termination is requested,
			// pause as requested by command.
			DECmdModExitListenerPause(sleepTimeMilliSecs);

			// check Data Manager
			bIsReady = SMILIsDataManagerReady();
			if (bIsReady)
			{
				break;
			}

			// increment sleep time
			counter += pingTimeOutMilliSecs;

		} 
		while(!DECmdIsModuleExiting() &&
			(counter <= waitTimeOutMilliSecs));

		DECmdErrf(pDCC, "\n");
	}

	if (bIsReady != TRUE)
	{
		rstat = SM_STATUS_TIMEOUT;
		DECmdErrf(pDCC, "Failed to connect to Data Manager, try later\n");
	}

	return rstat;
} // DECmdIsDataManagerReady


//---------------------------------------------------------------------
// ProcHelpRequest_ValidateInput_CheckDataManager
//---------------------------------------------------------------------
booln ProcHelpRequest_ValidateInput_CheckDataManager(
						_IN const DECmdControl *pDCC,
						_IN s32 numNVPair, 
						_IN astring **ppNVPair,
						_IN const DECmdHelpObject *pHelpObj,
						_OUT s32 *pStatus
						)
{

	// process help request
	if (DECmdPrintHelpCmdParamList(pDCC, 
		pHelpObj) == TRUE)
	{
		// help requested
		*pStatus = SM_STATUS_SUCCESS;
	}
	else
	{
		// validate input parameter
		if (DECmdValidateInputParameter(pDCC,
			numNVPair,
			ppNVPair,
			pHelpObj) == FALSE)
		{
			// invalid input found
			*pStatus = SM_STATUS_INVALID_PARAMETER;
		}
		else
		{
			// check if we can have connection to data manager
			*pStatus = DECmdIsDataManagerReady(pDCC,
				numNVPair, 
				ppNVPair);

			if (*pStatus == SM_STATUS_SUCCESS)
			{
				return TRUE;
			}
		}
	}

	return FALSE;
} // ProcHelpRequest_ValidateInput_CheckDataManager



//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

