
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 8 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:15p $
//
//  Author:
//    Chris Poblete
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// SDK include files
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Standard C include files
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Local includes
//---------------------------------------------------------------------
#include <fmttest.h>
#include <decmd.h>


//---------------------------------------------------------------------
// Local defines
//---------------------------------------------------------------------

#define FTEST_MAX_TEXT_FILL					(65)
#define FTEST_MAX_LINE_SIZE					(80)
#define FTEST_MAX_STRBUF_SIZE				(MAX_UNIDAT_STRING_SIZE)

#define FTEST_TRAILER_CHAR					'.'
#define FTEST_PASS_STR						"[  OK  ]\n"
#define FTEST_FAIL_STR						"[FAILED]\n"
#define FTEST_WARN_STR						"[ WARN ]\n"


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// GetCurrentRawTime
//---------------------------------------------------------------------
s32 GetCurrentRawTime(_OUT s64 *pRawTime)
{
	time_t ltime;


	if (pRawTime != NULL)
	{
		time(&ltime);
		*pRawTime = (s64)ltime;

		return 0;
	}

	return -1;
} // GetCurrentRawTime


//---------------------------------------------------------------------
// GetCurrentRawTimeDiff
//---------------------------------------------------------------------
s32 GetCurrentRawTimeDiff(_IN s64 previousRawTime, 
						  _OUT s64 *pRawTimeDiff)
{
	time_t ltime;


	if (pRawTimeDiff != NULL)
	{
		time(&ltime);

		*pRawTimeDiff = ((s64)ltime - previousRawTime);
		return 0;
	}

	return -1;
} // GetCurrentRawTimeDiff


//---------------------------------------------------------------------
// FTestMakeTimeLapseStr
//---------------------------------------------------------------------
booln FTestMakeTimeLapseStr(_INOUT FTestData *pFTD,
							_INOUT astring *pStrBuf,
							_INOUT s32 *pStrLen,
							_IN s32 strBufSize)
{
	astring sbuf[64];
	s32 lsize;


	// calculate response time
	SMRefTicksDiffCheckTimer(
		pFTD->beginTime,
		SMRefTicksGet(), 
		-1, 
		&pFTD->beginTime, 
		NULL, 
		NULL);
	
	sbuf[0] = '\0';
	lsize = 64;
	if (SMXLTTypeValueToUTF8(&pFTD->beginTime,
		sizeof(pFTD->beginTime),
		sbuf,
		(u32 *)&lsize,
		SM_XLT_VTYPE_s64_TO_ASTRING) != SM_STATUS_SUCCESS)
	{
		pFTD->beginTime = -1;
		_throw error1;
	}

	pFTD->beginTime = -1;
	if ((*pStrLen + lsize + 1) > strBufSize)
	{
		_throw error1;
	}

	if ((pFTD->fillCounter + lsize) > FTEST_MAX_TEXT_FILL)
	{
		strncat(pStrBuf,
			"\n",
			1);
		pFTD->fillCounter = lsize;
		(*pStrLen)++;
	}
	else
	{
		pFTD->fillCounter += lsize;
	}

	strncat(pStrBuf,
		sbuf,
		lsize);

	*pStrLen += lsize;

	return TRUE;

 error1:
	return FALSE;
} // FTestMakeTimeLapseStr


//---------------------------------------------------------------------
// FTestMakeNormalizeStr
//---------------------------------------------------------------------
booln FTestMakeNormalizeStr(_INOUT FTestData *pFTD,
							_INOUT astring *pStrBuf,
							_INOUT s32 *pStrLen,
							_IN s32 strBufSize)
{
	astring *pStr1;
	astring *pStr2;
	s32 lsize;


	lsize = *pStrLen +
		pFTD->fillCounter;
	lsize += (lsize / 68);
	if (lsize > strBufSize)
	{
		// buffer underrrun
		_throw error1;
	}

	pStr1 = SMUTF8Strdup(pStrBuf);
	if (pStr1 == NULL)
	{
		// out of memory
		_throw error1;
	}

	pStr2 = pStr1;
	*pStrLen = 0;
	while (*pStr2 != '\0')
	{
		if (*pStr2 == '\n')
		{
			// reset fillcounter
			pFTD->fillCounter = 0;
		}
		else
		{
			// check if we need to wrap
			if (pFTD->fillCounter > FTEST_MAX_TEXT_FILL)
			{
				*pStrBuf++ = '\n';
				pFTD->fillCounter = 0;
				(*pStrLen)++;
			}
			// account for the current character
			pFTD->fillCounter++;
		}
		*pStrBuf++ = *pStr2++;
		(*pStrLen)++;
	}
	*pStrBuf = '\0';
	(*pStrLen)++;

	SMFreeGeneric(pStr1);
	pStr1 = NULL;

	return TRUE;

 error1:
	return FALSE;
} // FTestMakeNormalizeStr


//---------------------------------------------------------------------
// FTestDataInit
//---------------------------------------------------------------------
void FTestDataInit(_INOUT FTestData *pFTD,
				   _IN const DECmdControl *pDCC)
{

	// sanity check
	if (ARGUMENT_PRESENT(pFTD))
	{
		// init
		pFTD->beginTime = -1;
		pFTD->fillCounter = 0;
		pFTD->optionsBitMap = 0;
		if (ARGUMENT_PRESENT(pDCC))
		{
			pFTD->pOutStream = pDCC->pOutStream;
		}
		else
		{
			pFTD->pOutStream = stdout;
		}
	}

	return;
} // FTestDataInit


//---------------------------------------------------------------------
// FTestOutfv
//---------------------------------------------------------------------
booln FTestOutfv(_INOUT FTestData *pFTD,
				 _IN booln bIsEnd,
				 _IN const astring *fmt,
				 _IN va_list vap)
{
	astring *pStr1;
	s32 lsize;
	booln rstat = FALSE;

	
	// allocate buffer
	if ((pStr1 = (astring *)SMAllocMem(FTEST_MAX_STRBUF_SIZE)) == NULL)
	{
		_throw error1;
	}

	// init
	*pStr1 = '\0';
	lsize = 1;

	// anything to process?
	if (ARGUMENT_PRESENT(fmt))
	{
		// resolve the output string
		lsize += SMvsnprintf(pStr1,
			FTEST_MAX_STRBUF_SIZE,
			fmt,
			vap);
		if (lsize < 2)
		{
			_throw error2;
		}
		
		if (SMMBitMapIsOn(pFTD->optionsBitMap,
			FTEST_NORMALIZE_DISABLE_TEMP) == TRUE)
		{
			// reset
			pFTD->fillCounter = 0;
			SMMBitMapSetOff(pFTD->optionsBitMap,
				FTEST_NORMALIZE_DISABLE_TEMP);
		}
		else
		{
			if (FTestMakeNormalizeStr(pFTD,
				pStr1,
				&lsize,
				FTEST_MAX_STRBUF_SIZE) == FALSE)
			{
				_throw error2;
			}
		}
	}

	if (bIsEnd == TRUE)
	{
		if (pFTD->beginTime != -1)
		{
			if (FTestMakeTimeLapseStr(pFTD,
				pStr1,
				&lsize,
				FTEST_MAX_STRBUF_SIZE) == FALSE)
			{
				_throw error2;
			}
		}
	}

	if (lsize > 1)
	{
		fprintf(pFTD->pOutStream, 
			pStr1, 
			NULL);
	}

	// good at this point
	rstat = TRUE;

 error2:
	SMFreeMem(pStr1);
	pStr1 = NULL;
 error1:
	return rstat;
} // FTestOutfv


//---------------------------------------------------------------------
// FTestBeginOutf
//---------------------------------------------------------------------
booln FTestBeginOutf(_INOUT FTestData *pFTD,
					 _IN const astring *fmt,
					 ...)
{
	va_list vap;
	booln bStatus;


	va_start(vap, fmt);
	bStatus = FTestOutfv(pFTD,
		FALSE,
		fmt,
		vap);
	va_end(vap);

	if (SMMBitMapIsOn(pFTD->optionsBitMap, FTEST_DISPLAY_TIMELAPSE) == TRUE)
	{
		pFTD->beginTime = SMRefTicksGet();
	}
	else
	{
		pFTD->beginTime = -1;
	}

	return bStatus;
} // FTestBeginOutf


//---------------------------------------------------------------------
// FTestOutf
//---------------------------------------------------------------------
booln FTestOutf(_INOUT FTestData *pFTD,
				_IN const astring *fmt,
				...)
{
	va_list vap;
	booln bStatus;


	va_start(vap, fmt);
	bStatus = FTestOutfv(pFTD,
		FALSE,
		fmt,
		vap);
	va_end(vap);

	return bStatus;
} // FTestOutf


//---------------------------------------------------------------------
// FTestEndOutf
//---------------------------------------------------------------------
booln FTestEndOutf(_INOUT FTestData *pFTD,
				   _IN s32 status)
{
	astring *pStr1;
	astring *pStr2;
	u32 ii;
	booln rstat = FALSE;


	// allocate buffer
	if ((pStr1 = (astring *)SMAllocMem(MAX_PATH_SIZE)) == NULL)
	{
		_throw error1;
	}

	// init
	*pStr1 = '\0';
	pStr2 = pStr1;

	for (ii = 0; ii <= (FTEST_MAX_LINE_SIZE - 12 - pFTD->fillCounter); ii++)
	{
		*pStr2++ = FTEST_TRAILER_CHAR;
	}
	if (status == SM_STATUS_SUCCESS)
	{
		sprintf(pStr2,
			FTEST_PASS_STR);
	}
	else
	{
		if (status == SM_STATUS_DECMD_SCRIPT_ERROR)
		{
			sprintf(pStr2,
				FTEST_WARN_STR);
		}
		else
		{
			sprintf(pStr2,
				FTEST_FAIL_STR);
		}
	}

	fprintf(pFTD->pOutStream, 
		pStr1,
		NULL);

	pFTD->fillCounter = 0;

	rstat = TRUE;

	SMFreeMem(pStr1);
	pStr1 = NULL;

 error1:
	return rstat;
} // FTestEndOutf


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

