
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2009 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    <Put author's name here>
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------

#include <modincs.h>


//---------------------------------------------------------------------
// Local Defines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Local Structures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------

// alias ordinal state map
_PRIV u8 gINIAliasOrdinalMap[INIALIAS_ORDINAL_MAX_VAL];

_PRIV booln gInitFlag = FALSE;


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// PopAliasInit
//---------------------------------------------------------------------
void PopAliasInit(void)
{

	if (gInitFlag == FALSE)
	{
		memset(gINIAliasOrdinalMap, 
			0,
			INIALIAS_ORDINAL_MAX_VAL);

		gInitFlag = TRUE;
	}
} // PopAliasInit


//---------------------------------------------------------------------
// FindNextOrdinal
//---------------------------------------------------------------------
s32 FindNextOrdinal(_IN const DECmdControl *pDCC,
					_IN const astring *pAliasName,
					_IN u32 optionsMap,
					_IN const astring *pINIPathFile,
					_IN const astring *pSectionName)
{
	astring *pSINIKeys;
	astring *pSINIVal;
	astring *pStrItem;
	astring *pStrProbe;
	u32 strSize;
	u32 ii;
	u8 currOrd;
	s32 cstat;
	s32 rOrdVal;


	SysDbgMsg4(("FindNextOrdinal: entry\n"));

	// init the map table if not already done so
	PopAliasInit();

	// init return value
	rOrdVal = INIALIAS_ORDINAL_INVALID;

	// sanity check
	if ((pDCC == NULL) ||
		(pINIPathFile == NULL) ||
		(pAliasName == NULL))
	{
		SysDbgMsg3(("FindNextOrdinal: required input parameter is NULL\n"));
		_throw error1;
	}

	// use maximum ini buffer size
	strSize = MAX_INI_LINE_SIZE;
	// allocate buffer
	if ((pSINIKeys = (astring *)SMAllocMem(strSize)) == NULL)
	{
		// just exit
		_throw error1;
	}
	// init
	pSINIKeys[0] = '\0';

	// use maximum ini buffer size
	strSize = MAX_INI_VALUE_SIZE;
	// allocate buffer
	if ((pSINIVal = (astring *)SMAllocMem(strSize)) == NULL)
	{
		// just exit
		_throw error2;
	}

	// init
	pSINIVal[0] = '\0';

	// get data populator list
	strSize = MAX_INI_LINE_SIZE;
	cstat = SMReadINIPathFileValue(pSectionName,
		NULL,
		INI_ASTRING,
		pSINIKeys,
		&strSize,
		NULL,
		0,
		pINIPathFile,
		TRUE);
	if (cstat != SM_STATUS_SUCCESS)
	{
		SysDbgMsg4(("FindNextOrdinal: failed to get data from %s\n", 
			pINIPathFile));
	}

	// browse the list to find next available ordinal
	currOrd = 0;
	pStrItem = pSINIKeys;
	while (*pStrItem != '\0')
	{
		SysDbgMsg4(("FindNextOrdinal: processing %s\n", 
			pStrItem));

		// remember item size
		ii = (u32)(strlen(pStrItem) + 1);

		// get ordvalue
		pStrProbe = strrchr(pStrItem, '.');
		if (pStrProbe != NULL)
		{
			// point to the value
			pStrProbe++;
			
			// convert to number
			strSize = sizeof(currOrd);
			cstat = SMXLTUTF8ToTypeValue(pStrProbe,
				&currOrd,
				&strSize,
				(SM_XLT_VTYPE_ASTRING_TO_u8 | SM_RADIX_HEXADECIMAL));
			if (cstat != SM_STATUS_SUCCESS)
			{
				// try decimal format
				strSize = sizeof(currOrd);
				cstat = SMXLTUTF8ToTypeValue(pStrProbe,
					&currOrd,
					&strSize,
					SM_XLT_VTYPE_ASTRING_TO_u8);
			}

			if (cstat == SM_STATUS_SUCCESS)
			{
				// make sure current ordinal is in range
				if (currOrd >= INIALIAS_ORDINAL_MAX_VAL)
				{
					DECmdErrf(pDCC,
						"Alias ordinal read: %u is out of bounds, max: %u\n",
						currOrd,
						INIALIAS_ORDINAL_MAX_VAL);

					rOrdVal = INIALIAS_ORDINAL_INVALID;
					_throw error3;
				}

				// record used ordinal
				gINIAliasOrdinalMap[currOrd] = 1;
			}
		}

		// check for pop duplicates
		strSize = MAX_INI_VALUE_SIZE;
		cstat = SMReadINIPathFileValue(pSectionName,
			pStrItem,
			INI_ASTRING,
			pSINIVal,
			&strSize,
			NULL,
			0,
			pINIPathFile,
			TRUE);
		if (cstat == SM_STATUS_SUCCESS)
		{
			if (strcmp(pSINIVal, pAliasName) == 0)
			{
				// error if update is false and duplicate is present
				if (SMMBitMapIsOff(optionsMap, INIALIAS_OPTS_UPDATE))
				{
					DECmdErrf(pDCC, "Error! Duplicate alias name (%s) found\n", 
						pAliasName);

					rOrdVal = INIALIAS_ORDINAL_DUPLICATE;
					_throw error3;
				}
				else
				{
					// use its ordinal
					rOrdVal = (s32)currOrd;
					_throw error3;
				}
			}
		}

		// go to next item
		pStrItem += ii;

	} // while (*pStrItem

	// now find the next free ordinal and return its index
	for (ii = 0; ii < INIALIAS_ORDINAL_MAX_VAL; ii++)
	{
		if (gINIAliasOrdinalMap[ii] == 0)
		{
			// save its index as return ordinal value
			rOrdVal = (s32)ii;

			// done
			break;
		}
	}

	SysDbgMsg4(("FindNextOrdinal: ordinal found " PRT_S32 "\n", 
		rOrdVal));

 error3:
	// free local mem
	SMFreeMem(pSINIVal);
	pSINIVal = NULL;
 error2:
	// free local mem
	SMFreeMem(pSINIKeys);
	pSINIKeys = NULL;
 error1:
	SysDbgMsg4(("FindNextOrdinal: exit\n"));
	return rOrdVal;
} // FindNextOrdinal


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

