
//-------------------------------------------------------------------------
//
//                  Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2008 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows IA64
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

//--------------------------------------------------------------------
// This file is not for Windows x86 and x64 environment
//--------------------------------------------------------------------
#if !defined(SYSTEM_TYPE_WINNT) || defined(_IA64_)


//--------------------------------------------------------------------
// Defines to customize headers to follow
//--------------------------------------------------------------------
#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//--------------------------------------------------------------------
// Dell specific headers
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Local include files
// Function prototypes and other local includes
//--------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcisocppsecurecompat.h>


//--------------------------------------------------------------------
// Local Defines and structures
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Global Data Area
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// Code Begins
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// strcpy_s
//--------------------------------------------------------------------
errno_t _cdecl strcpy_s(_OUT char *strDest,
						_IN size_t sizeInBytes,
   						_IN const char *strSource
						)
{

	if ((strDest == NULL) || (strSource == NULL))
	{
		return EINVAL;
	}

	if ((sizeInBytes <= 0) || (sizeInBytes < (strlen(strSource) + 1)))
	{
		return ERANGE;
	}
	
	strcpy(strDest, strSource);
			
	return 0;
} // strcpy_s

//--------------------------------------------------------------------
// wcscpy_s
//--------------------------------------------------------------------
errno_t _cdecl wcscpy_s(_OUT wchar_t *strDestination,
						_IN size_t sizeInWords,
						_IN const wchar_t *strSource
						)
{

	if ((strDestination == NULL) || (strSource == NULL))
	{
		return EINVAL;
	}

	if ((sizeInWords <= 0) || (sizeInWords < (wcslen(strSource) + 1)))
	{
		return ERANGE;
	}
	
	wcscpy(strDestination, strSource);
	
	return 0;
} // wcscpy_s

//--------------------------------------------------------------------
// strcat_s
//--------------------------------------------------------------------
errno_t _cdecl strcat_s(_OUT char *strDestination,
						_IN size_t sizeInBytes,
   						_IN const char *strSource
						)
{

	if ((strDestination == NULL) || (strSource == NULL))
	{
		return EINVAL;
	}

	if ((sizeInBytes <= 0) || (sizeInBytes < (strlen(strDestination) + strlen(strSource) + 1)))
	{
		return ERANGE;
	}
	
	strcat(strDestination, strSource);
	
	return 0;
} // strcat_s

//--------------------------------------------------------------------
// wcscat_s
//--------------------------------------------------------------------
errno_t _cdecl wcscat_s(_OUT wchar_t *strDestination,
						_IN size_t sizeInWords,
						_IN const wchar_t *strSource
						)
{

	if ((strDestination == NULL) || (strSource == NULL))
	{
		return EINVAL;
	}

	if ((sizeInWords <= 0) || (sizeInWords < (wcslen(strDestination) + wcslen(strSource) + 1)))
	{
		return ERANGE;
	}
	
	wcscat(strDestination, strSource);
	
	return 0;
} // wcscat_s


//--------------------------------------------------------------------
// strncat_s
//--------------------------------------------------------------------
errno_t _cdecl strncat_s(_OUT char *strDestination,
						 _IN size_t sizeInBytes,
   						 _IN const char *strSource,
						 _IN size_t count)
{

	if ((strDestination == NULL) ||
		(strSource == NULL))
	{
		return EINVAL;
	}

	if ((sizeInBytes <= 0) ||
		(sizeInBytes < (strlen(strDestination) + count + 1)))
	{
		return ERANGE;
	}
	
	strncat(strDestination, strSource, count);
	
	return 0;
} // strncat_s


//--------------------------------------------------------------------
// strncpy_s
//--------------------------------------------------------------------
errno_t _cdecl strncpy_s(_OUT char *strDest,
						 _IN size_t sizeInBytes,
						 _IN const char *strSource,
						 _IN size_t count
						 )
{
	
	if ((strDest == NULL) || (strSource == NULL) || (sizeInBytes <= 0))
	{
		return EINVAL;
	}
	
	strncpy(strDest, strSource, count);

	return 0;
} // strncpy_s

//--------------------------------------------------------------------
// wcsncpy_s
//--------------------------------------------------------------------
errno_t _cdecl wcsncpy_s(_OUT wchar_t *strDest,
						 _IN size_t sizeInWords,
						 _IN const wchar_t *strSource,
						 _IN size_t count
						 )
{
	
	if ((strDest == NULL) || (strSource == NULL) || (sizeInWords <= 0))
	{
		return EINVAL;
	}
	
	wcsncpy(strDest, strSource, count);

	return 0;
} // wcsncpy_s

//--------------------------------------------------------------------
// wcsncat_s
//--------------------------------------------------------------------
errno_t _cdecl wcsncat_s(_OUT wchar_t *strDest,
						 _IN size_t sizeInWords,
						 _IN const wchar_t *strSource,
						 _IN size_t count
						 )
{
	
	if ((strDest == NULL) || (strSource == NULL) || (sizeInWords <= 0))
	{
		return EINVAL;
	}

	wcsncat(strDest, strSource, count);

	return 0;
} // wcsncat_s

//--------------------------------------------------------------------
// strtok_s
// NOTE: This is probably not thread safe like the real strtok_s
//--------------------------------------------------------------------
char* _cdecl strtok_s(_INOUT char *strToken,
					  _IN const char *strDelimit,
					  _INOUT char **context
					  )
{
	
	// let strtok validate arguments
#ifndef SYSTEM_TYPE_WINNT
	return strtok_r(strToken, strDelimit, context);
#else
	return strtok(strToken, strDelimit);
	UNREFERENCED_PARAMETER(context);
#endif
} // strtok_s

// NOTE: This is probably not thread safe like the real wcstok_s
//--------------------------------------------------------------------
// wcstok_s
//--------------------------------------------------------------------
wchar_t* _cdecl wcstok_s(_INOUT wchar_t *strToken,
						 _IN const wchar_t *strDelimit,
						 _INOUT wchar_t **context
						 )
{
	
#if defined(SYSTEM_TYPE_LINUX)
	return wcstok(strToken, strDelimit, context);
#elif defined(SYSTEM_TYPE_NETWARE)
	return unitok((punicode)strToken, (punicode)strDelimit);
	UNREFERENCED_PARAMETER(context);
#else
	if ((context == NULL) || (strDelimit == NULL))
	{
		return NULL;
	}
	
	if ((*context != NULL) || (strToken != NULL))
	{
		return NULL;
	}

	*context = wcstok(strToken, strDelimit);

	return *context;
#endif
} // wcstok_s

//--------------------------------------------------------------------
// _wcsset_s
//--------------------------------------------------------------------
errno_t _cdecl _wcsset_s(_OUT wchar_t *str,
						 _IN size_t sizeInWords,
						 _IN wchar_t c
						 )
{
	
	if ((str == NULL) || (sizeInWords <= 0))
	{
		return EINVAL;
	}

#if defined(SYSTEM_TYPE_LINUX)
	str = wmemset(str, c, wcslen(str));
#elif defined(SYSTEM_TYPE_NETWARE)
	str = uniset((punicode)str, c);
	UNREFERENCED_PARAMETER(sizeInWords);
#else
	str = _wcsset(str, c);
#endif

	return 0;
} // _wcsset_s

//--------------------------------------------------------------------
// _wcsnset_s
//--------------------------------------------------------------------
errno_t _cdecl _wcsnset_s(_OUT wchar_t *str,
						  _IN size_t sizeInWords,
						  _IN wchar_t c,
						  _IN size_t count
						  ) 
{
	
	if ((str == NULL) || (sizeInWords <= 0))
	{
		return EINVAL;
	}

#if defined(SYSTEM_TYPE_LINUX)
	str = wmemset(str, c, count);
#elif defined(SYSTEM_TYPE_NETWARE)
	str = uninset((punicode)str, c, count);
	UNREFERENCED_PARAMETER(sizeInWords);
#else
	str = _wcsnset(str, c, count);
#endif

	return 0;
}

//---------------------------------------------------------------------
// __dec2bin - converts number from decimal to binary
//---------------------------------------------------------------------
static void __dec2bin(char *buf, 
					  int num)
{
	int index = 0;
	unsigned int mask;


	// initialize
	mask = 0;
	// most significant bit is set;
	mask = ~((~mask) >> 1);

	while (mask)
	{
		if (mask & num)
		{
			buf[index] = '1';
			index++;
		}
		else
		{
			if (index > 0)
			{
				buf[index] = '0';
				index++;
			}
		} // if (mask & num)

		mask = (mask >> 1);
	} // while (mask)

	buf[index] = '\0';
} // __dec2bin

//---------------------------------------------------------------------
// __ltoa - converts an integer to a string
//---------------------------------------------------------------------
static char * __ltoa(int value, 
					 char *string, 
					 int radix)
{
	
#ifndef SYSTEM_TYPE_LINUX
	return ltoa(value, string, radix);
#else
	switch (radix)
	{
		case 16:
			sprintf(string, "%x", value);
			break;

		case 10:
			sprintf(string, "%d", value);
			break;

		case 8:
			sprintf(string, "%o", value);
			break;

		case 2:
			__dec2bin(string, value);
			break;

		default:
			sprintf(string, "%d", value);
			break;
	}

	return string;
#endif // SYSTEM_TYPE_LINUX
} // __ltoa

//---------------------------------------------------------------------
// __ultoa - converts an integer to a string
//---------------------------------------------------------------------
static char * __ultoa(_IN unsigned int value, 
					  _OUT char *string, 
					  _IN int radix)
{
#ifndef SYSTEM_TYPE_LINUX
	return ultoa(value, string, radix);
#else
	switch (radix)
	{
		case 16:
			sprintf(string, "%x", value);
			break;

		case 10:
			sprintf(string, "%u", value);
			break;

		case 8:
			sprintf(string, "%o", value);
			break;

		case 2:
			__dec2bin(string, value);
			break;

		default:
			sprintf(string, "%u", value);
			break;
	}

	return string;
#endif // SYSTEM_TYPE_LINUX
} // __ultoa

//--------------------------------------------------------------------
// _ultoa_s
//--------------------------------------------------------------------
errno_t _cdecl _ultoa_s(_IN unsigned long value,
						_OUT char *str,
						_IN size_t sizeOfstr,
						_IN int radix
						) 
{
	
	char tmpStr[64];

	if ((str == NULL) || (sizeOfstr <= 0))
	{
		return EINVAL;
	}

	tmpStr[0] = '\0';
	if (__ultoa(value, tmpStr, radix) != NULL)
	{
		if (strlen(tmpStr) < sizeOfstr)
		{
			strcpy(str, tmpStr);
			return 0;
		}
	}
	return ERANGE;
} // _ultoa_s

//--------------------------------------------------------------------
// _ltoa_s
//--------------------------------------------------------------------
errno_t _cdecl _ltoa_s(_IN long value,
					   _OUT char *str,
					   _IN size_t sizeOfstr,
					   _IN int radix
					   )
{
	
	char tmpStr[64];

	if ((str == NULL) || (sizeOfstr <= 0))
	{
		return EINVAL;
	}
	 
	tmpStr[0] = '\0';
	if (__ltoa(value, tmpStr, radix) != NULL)
	{
		if (strlen(tmpStr) < sizeOfstr)
		{
			strcpy(str, tmpStr);
			return 0;
		}
	}
	return ERANGE;
} // _ltoa_s


//---------------------------------------------------------------------
// _snprintf_s
//---------------------------------------------------------------------
int _cdecl _snprintf_s(_OUT char *pBuffer,
					   _IN size_t sizeOfBuffer,
					   _IN size_t count,
					   _IN const char *pFormat,
					   _IN ... )
{
	va_list vl;
	int retVal;

	
	if ((pBuffer == NULL) ||
		(sizeOfBuffer <= 0) ||
		(pFormat == NULL))
	{
		retVal = -1;
	}
	else
	{
		va_start(vl, pFormat);
#ifdef WIN64
		retVal = _vsnprintf(pBuffer, count, pFormat, vl);
#else
		retVal = vsnprintf(pBuffer, count, pFormat, vl);
#endif
		va_end(vl);
	}

	return retVal;
} // _snprintf_s


//---------------------------------------------------------------------
// sprintf_s
//---------------------------------------------------------------------
int _cdecl sprintf_s(_OUT char *pBuffer,
					 _IN size_t sizeInBytes,
					 _IN const char *pFormat,
					 _IN ... )
{
	va_list vl;
	int retVal;
	

	if ((pBuffer == NULL) ||
		(sizeInBytes <= 0) ||
		(pFormat == NULL))
	{
		retVal = -1;
	}
	else
	{
		va_start(vl, pFormat);
#ifdef WIN64
		retVal = _vsnprintf(pBuffer, sizeInBytes, pFormat, vl);
#else
		retVal = vsnprintf(pBuffer, sizeInBytes, pFormat, vl);
#endif
		va_end(vl);

		// check if buffer too small
		// (see MSDN for sprintf_s behavior)
		if (retVal >= sizeInBytes)
		{
			pBuffer[0] = '\0';
			retVal = -1;
		}
	}

	return retVal;
} // sprintf_s


#ifdef WIN64

//---------------------------------------------------------------------
// swprintf_s
//---------------------------------------------------------------------
int _cdecl swprintf_s(_OUT wchar_t *pBuffer,
					  _IN size_t sizeInWords,
					  _IN const wchar_t *pFormat,
					  _IN ... )
{
	va_list vl;
	int retVal;
	

	if ((pBuffer == NULL) ||
		(sizeInWords <= 0) ||
		(pFormat == NULL))
	{
		retVal = -1;
	}
	else
	{
		va_start(vl, pFormat);
		retVal = _vsnwprintf(pBuffer, sizeInWords, pFormat, vl);
		va_end(vl);

		// check if buffer too small
		// (see MSDN for sprintf_s behavior)
		if (retVal >= sizeInWords)
		{
			pBuffer[0] = 0;
			retVal = -1;
		}
	}

	return retVal;
} // swprintf_s

#endif // WIN64


//---------------------------------------------------------------------
// tmpnam_s
//---------------------------------------------------------------------
errno_t _cdecl tmpnam_s(_OUT char * pBuffer, 
						_IN size_t sizeInBytes
						)
{

	if ((pBuffer == NULL) || (sizeInBytes == 0))
	{
		return EINVAL;
	}

	pBuffer = tmpnam(pBuffer);

	return 0;
} // tmpnam_s

//---------------------------------------------------------------------
// is_tm_valid
//---------------------------------------------------------------------
static int is_tm_valid (_IN const struct tm* ptm)
{

	if ((0 <= ptm->tm_sec && ptm->tm_sec <=59) &&		/* seconds after the minute - [0,59] */
		(0 <= ptm->tm_min && ptm->tm_min <=59) &&		/* minutes after the hour - [0,59] */
		(0 <= ptm->tm_hour && ptm->tm_hour <= 23) &&    /* hours since midnight - [0,23] */
		(1 <= ptm->tm_mday && ptm->tm_mday <= 31) &&    /* day of the month - [1,31] */
		(0 <= ptm->tm_mon && ptm->tm_mon <= 11) &&     	/* months since January - [0,11] */
		(0 <= ptm->tm_year) &&							/* years since 1900 */
		(0 <= ptm->tm_wday && ptm->tm_wday <= 6) &&		/* days since Sunday - [0,6] */
		(0 <= ptm->tm_yday && ptm->tm_yday <= 365))		/* days since January 1 - [0,365] */
	{
		return TRUE;
	}

	return FALSE;
} // is_tm_valid

//---------------------------------------------------------------------
// fopen_s
//---------------------------------------------------------------------
errno_t _cdecl fopen_s(_OUT FILE** ppFile, 
					   _IN const char* pFilename, 
					   _IN const char* pMode
					   ) 
{

	if ((ppFile == NULL) ||
		(pFilename == NULL) ||
		(pMode == NULL))
	{
		return EINVAL;
	}	

	*ppFile = fopen(pFilename, pMode);
	if (*ppFile == NULL)
	{
		return errno;
	}

	return 0;
} // fopen_s


//---------------------------------------------------------------------
// getenv_s
//---------------------------------------------------------------------
errno_t _cdecl getenv_s(_OUT size_t *pReturnValue,
						_OUT char *pBuffer,
						_IN size_t sizeInBytes,
						_IN const char *pVarName)
{
	char *pVarVal;
	errno_t retVal;


	// check parameters
	if ((pReturnValue == NULL) ||
		((pBuffer == NULL) && (sizeInBytes != 0)) ||
		((pBuffer != NULL) && (sizeInBytes == 0)) ||
		(pVarName == NULL))
	{
		// invalid parameter
		retVal = EINVAL;
	}
	else
	{
		// get variable value
		if ((pVarVal = getenv(pVarName)) != NULL)
		{
			// get number of characters in value
			*pReturnValue = (strlen(pVarVal) + 1);

			// check if caller buffer large enough
			if (sizeInBytes == 0)
			{
				// caller is requesting number of characters
				retVal = 0;
			}
			else if (*pReturnValue <= sizeInBytes)
			{
				// copy value to caller buffer
				retVal = strcpy_s(pBuffer, sizeInBytes, pVarVal);
			}
			else
			{
				// buffer too small
				retVal = ERANGE;
			}
		}
		else
		{
			// variable not found
			*pReturnValue = 0;
			retVal = 0;

			// set to empty string
			if (pBuffer != NULL)
			{
				pBuffer[0] = '\0';
			}
		}
	}

	return retVal;
} // getenv_s


#ifdef WIN64

//---------------------------------------------------------------------
// _wgetenv_s
//---------------------------------------------------------------------
errno_t _cdecl _wgetenv_s(_OUT size_t *pReturnValue,
						  _OUT wchar_t *pBuffer,
						  _IN size_t sizeInWords,
						  _IN const wchar_t *pVarName)
{
	wchar_t *pVarVal;
	errno_t retVal;


	// check parameters
	if ((pReturnValue == NULL) ||
		((pBuffer == NULL) && (sizeInWords > 0)) ||
		(pVarName == NULL))
	{
		// invalid parameter
		retVal = EINVAL;
	}
	else
	{
		// get variable value
		if ((pVarVal = _wgetenv(pVarName)) != NULL)
		{
			// get number of characters in value
			*pReturnValue = (wcslen(pVarVal) + 1);

			// check if caller buffer large enough
			if (*pReturnValue <= sizeInWords)
			{
				// copy value to caller buffer
				retVal = wcscpy_s(pBuffer, sizeInWords, pVarVal);
			}
			else
			{
				// buffer too small
				retVal = ERANGE;
			}
		}
		else
		{
			// variable not found
			*pReturnValue = 0;
			retVal = 0;
		}
	}

	return retVal;
} // _wgetenv_s

#endif // WIN64


//---------------------------------------------------------------------
// _dupenv_s
// NOTE: Varname not found not an error. Buffer and size cleared, returns zero
//---------------------------------------------------------------------
errno_t _cdecl _dupenv_s(_OUT char** ppBuffer, 
						 _OUT size_t* pSizeInBytes, 
						 _IN const char* pVarName
						 )
{
	
	errno_t retval = EINVAL;

	if ((ppBuffer == NULL) || (pVarName == NULL))
	{
		// no op
	}
	else
	{	
		// clear buffer and size in bytes		
		*ppBuffer = NULL;
		*pSizeInBytes = 0;
		getenv_s(pSizeInBytes, NULL, 0, pVarName);
		if (*pSizeInBytes > 0)
		{
			*ppBuffer = (char*)malloc(*pSizeInBytes);
			if (*ppBuffer == NULL)
			{
				retval = ENOMEM;
			}
			else
			{
				retval = getenv_s(pSizeInBytes, *ppBuffer, *pSizeInBytes, pVarName);
			}
		}
	}

	return retval;
} // _dupenv_s

//---------------------------------------------------------------------
// _sopen_s
//---------------------------------------------------------------------
errno_t _cdecl _sopen_s(_OUT int* pfh, 
						_IN const char* pFilename, 
						_IN int oflag, 
						_IN int shflag, 
						_IN int pmode
						)
{

	if ((pfh == NULL) || (pFilename == NULL))
	{
		return EINVAL;
	}
	
	*pfh = open(pFilename, oflag);
	if (*pfh == -1)
	{
		return errno;
	}

	return 0;
	UNREFERENCED_PARAMETER(shflag);
	UNREFERENCED_PARAMETER(pmode);
} // _sopen_s


//---------------------------------------------------------------------
// ctime_s
//---------------------------------------------------------------------
errno_t _cdecl ctime_s(_OUT char *pBuffer,
					   _IN size_t sizeInBytes,
					   _IN const time_t *pTime)
{
#ifndef SYSTEM_TYPE_LINUX
	const char *pTimeStr;
#endif
	errno_t retVal;


	if ((pBuffer == NULL) ||
		(sizeInBytes <= 0))
	{
		retVal = EINVAL;
	}
	else
	{
		pBuffer[0] = '\0';

		if ((sizeInBytes < TIME_STRING_BUF_LEN) ||
			(pTime == NULL) ||
			(*pTime < 0))
		{
			retVal = EINVAL;
		}
		else
		{
#ifdef SYSTEM_TYPE_LINUX
			if (ctime_r(pTime, pBuffer) != NULL)
			{
				retVal = 0;
			}
			else
			{
				retVal = -1;
			}
#else
			if ((pTimeStr = ctime(pTime)) != NULL)
			{
				retVal = strcpy_s(pBuffer, sizeInBytes, pTimeStr);
			}
			else
			{
				retVal = -1;
			}
#endif
		}
	}

	return retVal;
} // ctime_s


//---------------------------------------------------------------------
// asctime_s
//---------------------------------------------------------------------
errno_t _cdecl asctime_s(_OUT char *pBuffer,
						 _IN size_t sizeInBytes,
						 _IN const struct tm *pTM)
{
#ifndef SYSTEM_TYPE_LINUX
	const char *pTimeStr;
#endif
	errno_t retVal;


	if ((pBuffer == NULL) ||
		(sizeInBytes <= 0))
	{
		retVal = EINVAL;
	}
	else
	{
		pBuffer[0] = '\0';

		if ((sizeInBytes < TIME_STRING_BUF_LEN) ||
			(pTM == NULL) ||
			(is_tm_valid(pTM) != TRUE))
		{
			retVal = EINVAL;
		}
		else
		{
#ifdef SYSTEM_TYPE_LINUX
			if (asctime_r(pTM, pBuffer) != NULL)
			{
				retVal = 0;
			}
			else
			{
				retVal = -1;
			}
#else
			if ((pTimeStr = asctime(pTM)) != NULL)
			{
				retVal = strcpy_s(pBuffer, sizeInBytes, pTimeStr);
			}
			else
			{
				retVal = -1;
			}
#endif
		}
	}

	return retVal;
} // asctime_s


//---------------------------------------------------------------------
// localtime_s
//---------------------------------------------------------------------
errno_t _cdecl localtime_s(_OUT struct tm *pTM,
						   _IN const time_t *pTime)
{
#ifndef SYSTEM_TYPE_LINUX
	const struct tm *pTMTmp;
#endif
	errno_t retVal;


	if ((pTM == NULL) ||
		(pTime == NULL))
	{
		retVal = EINVAL;
	}
	else
	{
#ifdef SYSTEM_TYPE_LINUX
			if (localtime_r(pTime, pTM) != NULL)
			{
				retVal = 0;
			}
			else
			{
				retVal = -1;
			}
#else
			if ((pTMTmp = localtime(pTime)) != NULL)
			{
				*pTM = *pTMTmp;
				retVal = 0;
			}
			else
			{
				retVal = -1;
			}
#endif
	}

	return retVal;
} // localtime_s


//---------------------------------------------------------------------
// _get_timezone
//---------------------------------------------------------------------
errno_t _cdecl _get_timezone(_OUT long *pSeconds)
{
	errno_t retVal;


	if (pSeconds == NULL)
	{
		retVal = EINVAL;
	}
	else
	{
		*pSeconds = timezone;
		retVal = 0;
	}

	return retVal;
} // _get_timezone


//--------------------------------------------------------------------
// This file is not for Windows x86 and x64 environment
//--------------------------------------------------------------------
#endif // #if !defined(SYSTEM_TYPE_WINNT) || defined(_IA64_)


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

