
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 6 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:07p $
//
//  Author:
//    Anil Rao
//
//  Environment:
//    Linux, Netware, Windows NT/2000/.NET
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    <Put a sentence or two describing the contents of this file>
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//--------------------------------------------------------------------
// Dependent headers
//--------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------


//--------------------------------------------------------------------
// Function prototypes for functions defined in other modules
//--------------------------------------------------------------------
extern s32 ModuleMain(s32 argc, astring **argv);


//---------------------------------------------------------------------
// Code Start
//---------------------------------------------------------------------


#if defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE) && defined(SYSTEM_TYPE_WINNT)

//---------------------------------------------------------------------
//  SysDbgMemUsageReportHook
//  CRT (C-Language Run Time Report Hook)
//---------------------------------------------------------------------
int SysDbgMemUsageReportHook(_IN int reportType, 
							 _IN char *pMessage, 
							 _OUT int *pRetVal)
{
	FILE *fp;
	char *pEnv;
	char logEPFName[MAX_PATH_SIZE];


	// get system path
	pEnv = getenv("SystemRoot");

	// failed?
	if (NULL == pEnv)
	{
		pEnv = ".";
	}

	// generate file name
	sprintf(logEPFName, 
		"%s\\dcmleak%s.log", 
		pEnv, 
		NATIVE_DATA_WIDTH_STR);

	// open the file for append
	fp = fopen(logEPFName, 
		"a");

	// failed?
	if (NULL != fp)
	{
		// output message to file
		fprintf(fp, 
			"%s", 
			pMessage);

		// close file
		fclose(fp);
	}

	// return default value since we are about
	// to return unhandled status anyway
	*pRetVal = *pRetVal;

	// return a value that will cause the 
	// CRT to report to all other setup destinations
	return FALSE;
} // SysDbgMemUsageReportHook

#endif // defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE) && defined(SYSTEM_TYPE_WINNT)


//---------------------------------------------------------------------
//  main
//---------------------------------------------------------------------
s32 __cdecl main(s32 argc, astring **argv)
{

#ifdef SYSTEM_TYPE_LINUX
#if (NATIVE_DATA_WIDTH == 64)
	// AVR(09/10/01): turn off printing of "unaligned access" messages
	prctl(PR_SET_UNALIGN, PR_UNALIGN_NOPRINT);
#endif
#endif
	
#ifdef WIN64
	// AVR(11/16/00): I discovered that on Itanium the process
	// will cause a excpetion on a misaligned transfer
	// The SDK suggets each process to turn off the misaligned
	// transfer option so as not to cause the exception and the
	// misaligment will be fixed by the default system handler
	// This will cause a performance degredation.
	SetErrorMode(SEM_NOALIGNMENTFAULTEXCEPT);
#endif

	// Turn on reporting errors on debug builds if requested
#if defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE) && defined(SYSTEM_TYPE_WINNT)
	{
		int tmpFlag;

		// Get current flag
		tmpFlag = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
		// Turn on features
		tmpFlag |= (_CRTDBG_ALLOC_MEM_DF | 
			_CRTDBG_CHECK_ALWAYS_DF | 
			_CRTDBG_DELAY_FREE_MEM_DF | 
			_CRTDBG_LEAK_CHECK_DF);
		// Turn off features
		tmpFlag &= ~(_CRTDBG_CHECK_CRT_DF);
		// Set flag to the new value
		_CrtSetDbgFlag(tmpFlag);

	   // send assert reports to debugger, stderr and dialog box
	   _CrtSetReportMode(_CRT_ASSERT, (_CRTDBG_MODE_DEBUG | _CRTDBG_MODE_FILE | _CRTDBG_MODE_WNDW));
	   _CrtSetReportFile(_CRT_ASSERT, _CRTDBG_FILE_STDERR);

	   // send warning reports to debugger and stderr
	   _CrtSetReportMode(_CRT_WARN, (_CRTDBG_MODE_DEBUG | _CRTDBG_MODE_FILE));
	   _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDERR);

	   // send error reports to debugger and stderr
	   _CrtSetReportMode(_CRT_ERROR, (_CRTDBG_MODE_DEBUG | _CRTDBG_MODE_FILE));
	   _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);

	   // use report hook to send to our log file
	   // the reporting function is called first on
	   // a _CrtDbgReport call
	   _CrtSetReportHook(SysDbgMemUsageReportHook);
	}
#endif // defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE) && defined(SYSTEM_TYPE_WINNT)

	// sanity check
	if (argc < 1)
	{
		return 1;
	}

	// call module defined main routine
	// and exit with return code from routine
	return ModuleMain(argc, argv);
} // main


//---------------------------------------------------------------------
//  Code End
//---------------------------------------------------------------------

