
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Service library exported APIs called by the app that loads the library.
//    Dependent external functions must be implemented by the host module.
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------


//---------------------------------------------------------------------
// Defines to customize headers to follow
//---------------------------------------------------------------------

#if (!defined SYSTEM_DEBUG_LEVEL) && (defined _DEBUG)
#define SYSTEM_DEBUG_LEVEL      3
#endif


//---------------------------------------------------------------------
// Include Files
//---------------------------------------------------------------------

#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcdesvc.h>
#include <dcsupapi.h>

// Export definitions
// NOTE: In modules that export functions, SMINTF must be set up
// for exporting after all other header files are included
#define __EXPORTING
#undef SMINTF
#include <dcdetypes.h>


//---------------------------------------------------------------------
// Check for required defines
//---------------------------------------------------------------------

#ifndef SVCLIBEXPORT_ATTACHFN
#error SVCLIBEXPORT_ATTACHFN not defined. Example: SVCLIBEXPORT_ATTACHFN=DSM_SA_DATAMGR32_SMSvcLibAttach
#endif

#ifndef SVCLIBEXPORT_DETACHFN
#error SVCLIBEXPORT_DETACHFN not defined. Example: SVCLIBEXPORT_DETACHFN=DSM_SA_DATAMGR32_SMSvcLibDetach
#endif


//---------------------------------------------------------------------
// Local Defines 
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Local Structures
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Global Data Area
//---------------------------------------------------------------------

// original module context
static SMECInfo gSvcLibExportOrigECI;


//---------------------------------------------------------------------
// Code Begins
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// ModuleAttach - Called when library loaded by the OS
// into process memory
//---------------------------------------------------------------------
booln __lxinit__ ModuleAttach(void)
{

	// get original export context info
	SMGetExportContext(&gSvcLibExportOrigECI);

	return TRUE;
} // ModuleAttach


//---------------------------------------------------------------------
// ModuleDetach - Called when library unloaded by OS
// from process memory
//---------------------------------------------------------------------
booln __lxfini__ ModuleDetach(void)
{

	return TRUE;
} // ModuleDetach


//---------------------------------------------------------------------
// MODULEXX_SMSvcLibAttach - Called by module responsible for
// loading service library
//---------------------------------------------------------------------
SMINTF booln _cdecl SVCLIBEXPORT_ATTACHFN(void)
{
	SMECInfo eci;
	booln retVal;


	SMSetExportContext(&eci,
		&gSvcLibExportOrigECI);

	// call service attach function of host module
	retVal = ModuleSvcAttach(0,
		NULL);

	SMResetExportContext(&eci);

	return retVal;
} // MODULEXX_SMSvcLibAttach


//---------------------------------------------------------------------
// MODULEXX_SMSvcLibDetach - Called by module responsible for
// loading service library
//---------------------------------------------------------------------
SMINTF booln _cdecl SVCLIBEXPORT_DETACHFN(void)
{
	SMECInfo eci;
	booln retVal;


	SMSetExportContext(&eci,
		&gSvcLibExportOrigECI);

	// call service detach function of host module
	retVal = ModuleSvcDetach();

	SMResetExportContext(&eci);

	return retVal;
} // MODULEXX_SMSvcLibDetach


//---------------------------------------------------------------------
// Code Ends
//---------------------------------------------------------------------

