/*
----------------------------------------------------------------------------

	(C) Copyright 1991 Novell, Inc.	All Rights Reserved.

	This program is an unpublished copyrighted work which is proprietary
	to Novell, Inc. and contains confidential information that is not
	to be reproduced or disclosed to any other person or entity without
	prior written consent from Novell, Inc. in each and every instance.

	WARNING:  Unauthorized reproduction of this program as well as
	unauthorized preparation of derivative works based upon the
	program or distribution of copies by sale, rental, lease or
	lending are violations of federal copyright laws and state trade
	secret laws, punishable by civil and criminal penalties.

----------------------------------------------------------------------------
*/

#ifndef _AGENT_H_
#define _AGENT_H_

#include <stdio.h>

/* =============
 * Definition of NULL value, if not allready defined
 *
 */

#ifndef NULL
#define NULL 0
#endif /* NULL */

/* =============
 * Resource Tag IDs for the SNMP agent interface resources.
 *
 * define our resource tag name.  this is an arbitrary number, in theory,
 * but the standard is to use a four character string (defined as a
 * 32 bit number, but so that the string displays itself correctly when
 * seen in memory).
 *
 * rather than try to get our tag into the OS header file resource.h,
 * i'll just invent a standard: all uppercase reserved to NetWare
 * development, some uppercase reserved for Novell, all lower case
 * free for third parties (except, of course, they don't know about
 * this stuff because it isn't documented....).  Maybe some day i'll
 * remember to ask Provo to standardize something like this.
 */
#define	SAI_MIB_RTAG				0x6d4d4e53	/* 'SNMm', low byte first */
#define	SAI_TRAP_HAND_RTAG		0x744d4e53	/* 'SNMt' */
#define	SAI_IFINDEX_RTAG			0x694d4e53	/* 'SNMi', low byte first */
#define  SAI_TRAP_RECEIVE_RTAG 	0x724d4e53 /* 'SNMr' */
#define  SAI_TRAP_FILTER_RTAG 	0x664d4e53 /* 'SNMf' */

/* =============
 * forward declaration of ObjControl
 *
 */

struct ObjControl;

/* =============
 * Macro Definitions
 *
 */

/* Often in an API such as SNMP's, client routines wind up having a
 * particular set of parameters for no other reason than to agree
 * with a hook provided by the API.  In such cases, some parameters
 * will not be referenced in the routine.  Some compilers gripe about
 * such things.  Here is a macro that can be placed in the routine
 * after the declarations to insure the compiler that you know this
 * parameter is not referenced.
 */
#define	SA_NO_REFERENCE( parameterName ) parameterName = parameterName

/* =====
 * Attribute Value Buffer Maximum Size
 *
 */

#define ATTR_VAL_BUFF_MAX              256



/* Data structures */
/* --------------- */

/*
 *	SAI_PROCESSOR_INDEPENDENCE
 *	--------------------------
 *
 * Originally, the basic array in sa_gener.h was treated as an
 * array of unsigned longs and pointers were stuffed into those
 * values.  Although this works fine in the NetWare environment,
 * is not processor independent.  This API has been adjusted to
 * be PI, but because we use a union where there was previously
 * just a value, THIS MIGHT CAUSE TROUBLE for some modules that
 * previously compiled fine with the older version of sa_gener.h.
 *
 * This actually isn't as much of a problem as it seems: the basic
 * use of the unsigned long array is hidden from this module because
 * it is passed through the ocb_context_pt field of the ObjControl
 * structure.  Since ocb_context_pt is of type void *, it allows
 * the intrumented code to use the old form while these routines use
 * the new form.  The two forms look the same in memory (on NetWare)
 * so the fact that they are coded differently is unimportant.
 *
 * There is only a problem with code using SAGGetFromArray and
 * SAGSetToArray (which were never documented and aren't very
 * useful anyway) and code that defines a OCBLookupInst routine
 * (which might possibly be used more, but I don't know).
 *
 * It turns out that none of that should be common which is why we
 * choose to go with the new as the default.  Places where these arrays
 * are actually defined need to be updated from unsigned long arrays
 * to ValueOrPtr arrays and, unfortunately, that means the arrays can
 * no longer be initialized at compile time (initializing unions is
 * not possible for all compilers) so the arrays will need to be
 * initialized at run time.  Such is the cost portability.  If
 * portability is not desired, the old practice of using unsigned long
 * will still work fine in a NetWare 386 environment.
 *
 * To make a long story short, unless it's already been defined
 * otherwise, compile this file for processor independence.
 * (This was originally all defined in sa_gener.h, but I had
 * to move it here as the ValueOrPtr value became more widely used.)
 */
#ifndef	SAI_PROCESSOR_INDEPENDENCE
#define	SAI_PROCESSOR_INDEPENDENCE	1
#endif

/* split code between processor independent code and old yucky code.
 * normally we go with processor independence.
 */
#if	SAI_PROCESSOR_INDEPENDENCE

/* a union that can take on either a 32 bit value or a pointer:
 * a common concept for processor independence.  (In a 80386
 * 32-bit environment, this is equivalent at the binary level
 * with stuffing pointers into unsigned longs....)
 */
typedef	union ValueOrPtr
{
	unsigned int	vop_value;	/* 32 bit value */
	void		*vop_pointer;	/* arbitrary pointer */
} ValueOrPtr;

#else	/* else of #if SAI_PROCESSOR_INDEPENDENCE */

/* in the old NetWare 32-bit environment, pointers and values
 * were handled by stuffing pointers into unsigned longs.
 * for code written for that environment (which would compile
 * using the old header file), we dummy up the ValueOrPtr
 * definition.  This should be the default if we want those
 * old files to compile correctly.
 */
#define	ValueOrPtr	unsigned int

#endif	/* end of #else on #ifdef SAI_PROCESSOR_INDEPENDENCE */


/*
 * ============================================================================
 *
 * AttrInfo - Attribute Information Block
 *
 * This block describes a particular attribute's characteristics.  The
 * Attribute Value Block points to this block to provide the access
 * routines with information about the attribute being accessed.  The
 * information in this block is derived from the compiled MIB.
 *
 * Types for aib_asn_type are defined in sa_asn1.h.  Valid types are:
 *
 *    ASN_BOOLEAN
 *    ASN_INTEGER
 *    ASN_BIT_STRING
 *    ASN_OCTET_STRING
 *    ASN_NULL
 *    ASN_OBJECT_ID
 *    ASN_IPADDRESS
 *    ASN_COUNTER
 *    ASN_GAUGE
 *    ASN_TIMETICKS
 *    ASN_OPAQUE
 *
 */

typedef struct AttrInfo
{
    int    aib_id;         /* attribute ID for this attribute */
    int    aib_type;       /* data type of this attribute */
    int    aib_asn_type;   /* asn.1 specific type (see sa_asn1.h for types) */
    int    aib_min;        /* minimum legal value (or min length) */
    int    aib_max;        /* maximum legal value (or max length) */
    int    aib_flags;      /* flags */
} AttrInfo;

#define    AIT_UNKNOWN        0    /* should never be set */
#define    AIT_INTEGER        1    /* integer, gauge, counter, etc. */
#define    AIT_TIMETICKS      2    /* 1/100 of a second times */
#define    AIT_SHORTOCTET     3    /* octet strings 4 bytes of less */
#define    AIT_LONGOCTET      4    /* longer octet strings */
#define    AIT_DISPLAYSTRING  5    /* NVT ascii strings */
#define    AIT_OBJECTID       6    /* object ID */
#define    AIT_CHOICE         7    /* ASN.1 choice value (i.e., a union) */
#define    AIT_HEADER         8    /* type and length without any value
												* (not used in actual attribute blocks)
												*/
#define    AIT_NULL           9    /* null type and zero length without any
												* value.
												* (not used in actual attribute blocks)
												*/
#define    AIT_MAX            9    /* highest defined type */

#define    AIF_READ         ( 1 << 0 )    /* item can be read */
#define    AIF_WRITE        ( 1 << 1 )    /* item can be written */
#define    AIF_IMMEDIATE    ( 1 << 2 )    /* item fits in an unsigned long */
#define    AIF_LIMITS       ( 1 << 3 )    /* item is an enumerated type
                                           * or has size limits */


/*
 * ============================================================================
 *
 * AttrValue - Attribute Value Block
 *
 * The SNMP agent passes a list of these blocks to each access routine
 * to provide them with the attribute being accessed.  The access
 * routines either extracts each attribute's value from this block (for
 * SET) or writes the value into this block (for GET and GET NEXT).
 * typedef struct AttrValue
 *
 */

typedef struct AttrValue
{
    struct AttrValue    *avb_next;    /* pointer to next in list */
    AttrInfo            *avb_info;    /* pointer to info block */
    unsigned int        avb_length;   /* length, or value if immediate */
    void                *avb_value;   /* pointer to value or value buffer */
} AttrValue;


/*
 * ============================================================================
 *
 * TLV - Structure containing an ASN.1 type/length/value
 *
 * This structure describes an arbitrary value by providing the value's
 * identification, type, length, and location.  It is also used to
 * describe a buffer for receiving such a value.
 *
 * tlv_id      Contains an identifier indicating what the value actually
 *             means.  In some cases this field is not used.
 * tlv_type    Contains the appropriate ASN.1 type of the value, such as
 *             ASN_INTEGER, if the type is known.
 * tlv_length  Total number of bytes in the value, or the number of bytes
 * 	         available in the buffer to receive the value.
 * tlv_value   Pointer to the value or to a buffer for receiving the value.
 *
 * The SNMP agent copies information into and out of buffers described
 * by this structure.  The structures themselves are defined and provided
 * by the entity accessing the SNMP agent's database.
 *
 * IMPORTANT IMPLEMENTATION NOTE
 *
 *  In order to use this structure, you must first include "sa_asn1.h"
 *  This file defines the types to go into the tlv_type field.  An immediate
 *  type has the value stored in tlv_length & tlv_value is NULL;  Other values
 *  are stored in tlv_value, with tlv_length indicating the size of the value.
 *  Valid types are:
 *
 *       ASN_BOOLEAN                Immediate
 *       ASN_INTEGER                Immediate
 *       ASN_OCTET_STRING
 *       ASN_NULL                   Immediate
 *       ASN_OBJECT_ID
 *       ASN_IPADDRESS              Immediate
 *       ASN_COUNTER                Immediate
 *       ASN_GAUGE                  Immediate
 *       ASN_TIMETICKS              Immediate
 *       ASN_OPAQUE
 *
 */

typedef struct TLV
{
    int		tlv_id;		/* ID of value, when appropriate */
    int		tlv_type;	/* type of value */
    int		tlv_length;	/* number of bytes in the value */
    void	*tlv_value;	/* pointer to value or buffer */
} TLV;


/*
 * ============================================================================
 *
 * VarBind - structure containing a single SNMP variable binding.
 *
 * Contains a variable binding, including its object ID, its encoding
 * rule, its ASN.1 type, and, of course, the value itself.
 *
 * vb_id_pt		pointer to array of unsigned longs containing
 *			the var bind's object ID.
 * vb_id_ln		number of sub-IDs in the object id.
 * vb_encoding		AIT_* code indicating the appropriate encoding
 *			mechanism for handling the value.
 * vb_asn_type		Value to put in the "type" field of the encoded
 *			ASN.1 type/length/value.
 * vb_length		Number of bytes in the value, or, for AIT_* types
 *			that pass the value as an immediate value, the
 *			immediate value of the var bind.
 * vb_value		pointer to the value, if not an immediate value.
 *
 * Used by some of the client interface routines when an SNMP variable
 * binding is passed across the interface.
 *
 */
typedef struct VarBind
{
	unsigned int	*vb_id_pt;	/* pointer to object ID */
	int		vb_id_ln;	/* number of objects in ID */
	int		vb_encoding;	/* AIT_* encoding mechanism */
	int		vb_asn_type;	/* ASN type or label */
	unsigned int	vb_length;	/* number of bytes in value
					 * (or immediate value)
					 */
	void		*vb_value;	/* pointer to non-immediate value */
} VarBind;

/*
 * ============================================================================
 *
 * ObjInfo - Object Information Block
 *
 * This block describes the object to the SNMP agent.
 * This Block will be generated by the MIB COMPILER mibcomp.exe
 *
 */

typedef struct ObjInfo
{
    struct ObjInfo      *oib_next;       /* next in list */
    struct ObjControl   *oib_control;    /* pointer to object control block */
    unsigned long       oib_rtag;        /* Resource Tag */
    unsigned int        oib_id_ln;       /* number of sub-IDs in object ID */
    unsigned int        *oib_id_pt;      /* pointer to object ID */
    unsigned int        oib_index_ln;    /* number of items in the index */
    AttrInfo            *oib_index_pt;   /* pointer to list of index items */
    unsigned int        oib_attr_mx;     /* number of attributes */
    AttrInfo            *oib_attr_pt;    /* pointer to array of attributes */
} ObjInfo;

/*
 * ============================================================================
 *
 * OCB FUNCTION RETURN CODES
 *
 * For all OCB Functions, OCBAbortSet, OCBCheckSet, OCBCommitSet, OCBGet,
 * OCBGetNext, if the function is successful, return SNMP_ERR_NOERROR.
 *
 * Otherwise
 *   1. If possible, mark the AttrValue struct that causes the error.
 *      It may not be possible to mark any structures.  Mark this struct by:
 *      A. set AttrValue->avb_info = (AttrInfo *)NULL;
 *      B. Set avb_length to one of the SNMP error codes.
 *  2. Return the snmp error code as an indication of the type of error.
 *
 * Error Codes are listed below.
 *
 */

#define SNMP_ERR_NOERROR    (0x0)      /* Function Success */
#define SNMP_ERR_TOOBIG	    (0x1)      /* Response exceeds space  */
#define SNMP_ERR_NOSUCHNAME (0x2)      /* Attribute Not Found */
#define SNMP_ERR_BADVALUE   (0x3)      /* Limits Error */
#define SNMP_ERR_READONLY   (0x4)      /* Read Only Violation */
#define SNMP_ERR_GENERR	    (0x5)      /* General Errors */

/*  The following two error conditions are used for the sparse table handling 
	While travesing  the sparse table, the instrumentation agent needs to populate the attribute list the following 
	error codes
	
        set AttrValue->avb_info = (AttrInfo *)NULL;
        Set avb_length to one of the  SNMP_ERR_UNPOPULATEDCELL
        Return the snmp error code SNMP_ERR_SPARSETABLEROW_UNPOPULATEDENTRIESMARKED 
        as an indication of the type of error.  */
	
#define SNMP_ERR_SPARSETABLEROW_UNPOPULATEDENTRIESMARKED  100
#define SNMP_ERR_UNPOPULATEDCELL  101


/* SACTrapHandlerPROC - trap handler */
typedef int	SACTrapHandlerPROC(
	struct TLV	*target_addr,	/* target address to receive trap */
	int		trap_ln,	/* number of bytes in SNMP packet */
	unsigned char	*trap_pt	/* pointer to the SNMP trap packet */
);

/* SACTrapReceiverPROC - trap receiver */
typedef int	SACTrapReceiverPROC
(
	unsigned long *enterpriseID,
	int 	genericType,
	int	specificType,
	int	varBindCount,
	struct VarBind	*varBinds,
	char	*communityString,
	void *securityControl,
	char 	*xportName,
	struct TLV	*sourceAddr,
	void 	*context_ptr
);


/*
 * ============================================================================
 *							
 * OCBAbortSet - Routine to abort an previously checked set
 *
 * OCBAbortSet is a typedef routine definition, defining a routine to be
 * called to abort a set which was previously checked via OCBCheckSet,
 * but which will not be committed because of an unrelated problem in
 * the SNMP request being processed.  The exporting module implements a
 * routine of this type and places its address in the ocb_abort_set
 * field of the ObjControl control structure for each abstract object.
 *
 * ocb_pt     Pointer to the object control block which led the SNMP agent to
 *            this routine.
 * inst_list  Pointer to a NULL terminated list of instance values which
 *            identify the instance for this object.  If this object has
 *            no instance information, inst_list will be NULL.
 * handle_pt  Pointer to handle returned by OCBCheckSet.
 *
 * The SNMP agent calls the exporting module's implementation of this
 * routine to abort an SNMP SET operation which has been
 * partially processed.  This call is made only if OCBCheckSet succeeded
 * for this object instance, but OCBCheckSet failed for some other
 * object instance in the SNMP request being processed.
 *
 * OCBAbortSet should free up any resources which were allocated by
 * OCBCheckSet in preparation for handling the SET operation.
 *
 * The handle_pt parameter provides context to OCBAbortSet from
 * OCBCheckSet.
 *
 */

typedef void OCBAbortSet(
	struct ObjControl  *ocb_pt,             /* control block */
	AttrValue          *inst_list,          /* list of instance ID info */
	void               *handle_pt           /* handle from OCBCheckSet */
);


/*
 * ============================================================================
 *
 * OCBCheckSet - Routine to check a set operation for legality
 *
 * OCBCheckSet is a typedef routine definition, defining a routine to be
 * called to check to see the exporting module wants to allow the
 * setting of a set of attributes in an specific object instance.  The
 * exporting module implements a routine of this type and places its
 * address in the ocb_check_set field of the ObjControl control
 * structure for each abstract object.
 *
 * ocb_pt      Pointer to the object control block which led the SNMP agent
 *             to this routine.
 * inst_list   Pointer to a NULL terminated list of instance values which
 *             identify the instance for this object.  If this object has
 *             no instance information, inst_list will be NULL.
 * attr_list   Pointer to a NULL terminated list of the attribute value blocks
 *             for the attributes being set by the request.
 * handle_pp   Pointer to a location into which OCBCheckSet can return a
 *             handle to be passed to OCBCommitSet or OCBAbortSet.
 *
 * Returns zero if successful, otherwise an error code indicating why
 * the values cannot be set.
 *
 * The SNMP agent calls the exporting module's implementation of this
 * routine to check for the legality of a set operation.  This allows
 * the SNMP agent to check to see if all the requested attributes of all
 * the requested objects can be legally set before actually changing the
 * database.  Once the agent has received a successful return from all
 * object instances in the request, it then goes back and calls
 * OCBCommitSet for each object instance to actually effect the changes.
 *
 * The routine should confirm that the set of attributes of the specific
 * object instance can legally be set to the given values, but it should
 * make this check without modifying the database.  This routine may
 * block, but it should only block if absolutely necessary.
 *
 * The handle_pp parameter can be used to pass all the resources
 * required for the SET from OCBCheckSet to OCBCommitSet.  This
 * insures that OCBCommitSet will not fail because it cannot get
 * required resources.
 *
 */

typedef int OCBCheckSet(
   struct ObjControl  *ocb_pt,                 /* control block */
   AttrValue          *inst_list,              /* list of instance ID info */
   AttrValue          *attr_list,              /* list of attributes to set */
   void               **handle_pp              /* place to return handle */
);


/*
 * ============================================================================
 *
 * OCBCommitSet - Routine to set a list of attributes in an object
 *
 * OCBCommitSet is a typedef routine definition, defining a routine to
 * be called to set a list of attributes in an specific object instance.
 * The exporting module implements a routine of this type and places
 * its address in the ocb_commit_set field of the ObjControl control
 * structure for each abstract object.
 *
 * ocb_pt      Pointer to the object control block which led the SNMP
 *             agent to this routine.
 * inst_list   Pointer to a NULL terminated list of instance values which
 *             identify the instance for this object.  If this object has
 *             no instance information, inst_list will be NULL.
 * attr_list   Pointer to a NULL terminated list of the attribute value
 *             blocks for the attributes being set by the request.
 * handle_pt   Pointer to handle returned by OCBCheckSet.
 *
 * Returns zero if successful, otherwise an error code indicating why
 * the values cannot be set.  Since OCBCheckSet returns successfully
 * before OCBCommitSet is called, OCBCommitSet should only fail under
 * extreme conditions.
 *
 * The SNMP agent calls the exporting module's implementation of this
 * routine to accomplish an SNMP SET operation.  This call is
 * made only after the SNMP agent has called OCBCheckSet for all object
 * instances to insure that the set can be legally accomplished.
 *
 * OCBCommitSet must use the attribute list pointed to by inst_list to
 * find the object instance being address.  It should then update that
 * specific object instance to reflect the values in the list of
 * attribute values pointed to by attr_list.
 *
 * The handle_pt parameter provides context to OCBCommitSet from
 * OCBCheckSet.  OCBCheckSet should pass OCBCommitSet all the resources
 * required for the SET.  This insures that OCBCommitSet will
 * not fail because it cannot get required resources.
 *
 */

typedef int OCBCommitSet(
   struct ObjControl  *ocb_pt,                 /* control block */
   AttrValue          *inst_list,              /* list of instance ID info */
   AttrValue          *attr_list,              /* list of attributes to set */
   void               *handle_pt               /* handle from OCBCheckSet */
);


/*
 * ============================================================================
 *
 * OCBGet - Routine to read attributes from a specific object
 *
 * OCBGet is a typedef routine definition, defining a routine to be
 * called to read a list of attributes from an specific object.  The
 * exporting module implements a routine of this type and places its
 * address in the ocb_get field of the ObjControl control structure for
 * each abstract object.
 *
 * ocb_pt     Pointer to the object control block which led the SNMP agent
 *            to this routine.
 * inst_list  Pointer to a NULL terminated list of instance values which
 *            identify the instance for this object.  If this object has
 *            no instance information, inst_list will be NULL.
 * attr_list  Pointer to a NULL terminated list of the attribute value
 *            blocks for the attributes being read by the request.
 *
 * Returns zero if successful, otherwise an error code indicating why
 * the read failed.
 *
 * The SNMP agent calls the exporting module's implementation of this
 * routine to read a list of attributes from an object instance.  If
 * this routine fails for any SNMP object in the request, the entire
 * request fails.
 *
 * OCBGet must first locate the correct instance of this abstract object
 * by using the index information in the NULL terminated list pointed to
 * by inst_list.  Once the correct instance has been found, OCBGet
 * should go through the NULL terminated attribute list pointed to by
 * attr_list and copy the value of each attribute into its value block.
 *
 */

typedef int OCBGet(
   struct ObjControl  *ocb_pt,                /* control block */
   AttrValue          *inst_list,             /* list of instance ID info */
   AttrValue          *attr_list              /* list of attributes to get */
);


/*
 * ============================================================================
 *
 * OCBGetNext - Routine to read attributes from the following object instance
 *
 * OCBGetNext is a typedef routine definition, defining a routine to be
 * called to read a list of attributes from the object instance which
 * follows the specified object instance.  The exporting module
 * implements a routine of this type and places its address in the
 * ocb_get_next field of the ObjControl control structure for each
 * abstract object.
 *
 * ocb_pt      Pointer to the object control block which led the SNMP agent
 *             to this routine.
 * inst_list   Pointer to a NULL terminated list of instance values which
 *             identify the previous instance of this object.
 * next_list   Pointer to a NULL terminated list of attribute values blocks.
 *             OCBGetNext should fill in the blocks with the instance
 *             information of the instance actually read.
 * attr_list   Pointer to a NULL terminated list of the attribute value
 *             blocks for the attributes being read by the request.
 *
 * Returns zero if successful, otherwise an error code indicating why
 * the read failed.
 *
 * The SNMP agent calls the exporting module's implementation of this
 * routine to read a list of attributes from the next object in a list
 * of objects.  If this routine fails for any SNMP object in the
 * request, the entire request fails.
 *
 * OCBGetNext must first locate the previous instance of this abstract
 * object by using the index information in the NULL terminated list
 * pointed to by inst_list.  It then must locate the next object
 * instance.  The meaning of next depends on the MIB definition of
 * the object.
 *
 * Once the correct instance has been found, OCBGetNext should first go
 * through the list of attribute blocks pointed to by next_list to store
 * the instance attribute values which identify the object instance
 * which OCBGetNext will actually return.  Finally, OCBGetNext should go
 * through attribute list pointed to by attr_list and use the next
 * instance of the object to fill in the correct value of each attribute
 * in its value block.
 *
 * The inst_list and next_list AttrValue lists will allways be "fully
 * qualified".  That is, if the object has one index attribute, both
 * inst_list and next list will have exactly one element.  If the object
 * has two (2) index attributes, both inst_list and next list will have
 * exactly two (2) elements, and so on.
 *
 */

typedef int OCBGetNext(
   struct ObjControl  *ocb_pt,              /* control block */
   AttrValue          *inst_list,           /* instance ID for this object */
   AttrValue          *next_list,           /* instance ID for next object */
   AttrValue          *attr_list            /* list of attributes to get */
);


/*
 * ============================================================================
 *
 * ObjControl - Object Control Block
 *
 * Defines Abstract Objects and callback functions used by a single
 * managing entity.
 *
 */

typedef struct ObjControl
{
    unsigned int    ocb_flags;             /* flags */
    void            *ocb_context_pt;       /* pointer context */
    OCBGet          *ocb_get;              /* get values from object */
    OCBGetNext      *ocb_get_next;         /* get values from next object */
    OCBCheckSet     *ocb_check_set;        /* check set for legality */
    OCBCommitSet    *ocb_commit_set;       /* set values into object */
    OCBAbortSet     *ocb_abort_set;        /* abort checked set */
} ObjControl;

/* no flags are defined yet. */
#define    OCBF_NONE    0            /* no flags here */



/*
 * Structure Name:	IfIndexBlock
 *
 * Purpose:
 *  Contains the necessary information for allocating and maintaining
 *  an ifIndex entry.
 *
 * Fields:
 *  iib_structLength
 *	Number of bytes in this structure at the time the module
 *	defining it was compiled. I.e., always set to the value of
 *	sizeof (IfIndexBlock).
 *
 *  iib_ifIndex
 *	ifIndex value assigned to this interface. Should be
 *	initialized to SAI_IFN_INVALID to avoid any chance of it
 *	accidentally being used as a valid ifIndex if the index
 *	allocation fails.
 *
 *  iib_boardNumber
 *	ODI logical board of the driver behind this interface.
 *	At this time, there *must* be a logical board, although
 *	one logical board can be used by multiple ifIndexes.
 *
 *  iib_ownerIndex
 *	Values supplied by caller to distinquish this interface
 *	from other interfaces using the same ODI logical board.
 *	The value zero (a.k.a., SAI_IFN_OWNER_PRIMARY) has a
 *	special meaning of the primary ifIndex, which is the
 *	ifIndex that maps directly to the ODI board number. (The
 *	value zero is not coincidence: it's zero so ifIndex has
 *	exactly the same value as the ODI board number.)
 *	Another special value for this field is SAI_IFN_OWNER_AUTOMATIC.
 *	Setting iib_ownerIndex to this value before calling
 *	SAIAllocateIfIndex will result in the lowest unallocated
 *	owner index to be discovered and used for this ifIndex.
 *
 *  iib_ifEntry
 *	Pointer to this interface's ifEntry array, as per ifentry.h.
 *	May be NULL.
 *
 *  iib_ifEntryMask
 *	Pointer to the ifEntry's mask. see ifentry.h.
 */
typedef struct IfIndexBlock
{
	unsigned long	 iib_structLength;	/* sizeof IfIndexBlock */
	unsigned long	 iib_ifIndex;		/* assigned ifIndex */
	unsigned long	 iib_boardNumber;	/* ODI logical board */
	unsigned long	 iib_ownerIndex;	/* caller assigned value */
	ValueOrPtr	*iib_ifEntry;		/* see ifentry.h */
	unsigned long   *iib_ifEntryMask;	/* see ifentry.h */
} IfIndexBlock;

/*
 * ============================================================================
 *
 * Function Prototypes
 *  For more information, please see "SNMP Agent Instrumentation Interface
 *  for NetWare v3.2", Don Provan; and "SNMP Agent Client Interface for
 *  NetWare v3.2", Don Provan.
 *
 */

/*
 *	MIB registration.
 */

extern void SAIDeregisterMIB  (ObjInfo **mib_pp);

extern int  SAIRegisterMIB    ( ObjInfo **mib_pp, unsigned long rtag );

extern void SAIDeregisterMIBInst  (ObjInfo **mib_pp, AttrValue *inst_list);

extern int  SAIRegisterMIBInst    ( ObjInfo **mib_pp, AttrValue *inst_list, unsigned long rtag );

#define	SAI_REGISTER_OK		0	/* Registration success */
#define	SAI_REGISTER_FAIL_DUP	-1	/* Duplicate Obj Id */
#define	SAI_REGISTER_BAD_RTAG	-2	/* RTag wasn't for a MIB */
#define	SAI_REGISTER_FAIL_GEN	-3	/* Couldn't get memory for Multiple MIB
                                       instance registration */




/*
 *	ifIndex allocation and control.
 */

/* error codes */
#define	SAI_IFNE_OK		 0	/* no error */
#define	SAI_IFNE_NOMEM		-1	/* out of memory */
#define	SAI_IFNE_BADTAG		-2	/* not an ifIndex resource tag */
#define	SAI_IFNE_NOTFOUND	-3	/* indicated entity not found */
#define	SAI_IFNE_unused		-4	/* not used at this time */
#define	SAI_IFNE_DUPLICATE	-5	/* duplicate entry detected */

/* Special value for an invalid ifIndex or ODI logical board. We use
 * this as the return value when a mapping is requested which does
 * not exist.
 */
#define	SAI_IFN_INVALID	0xffffffff

/* Symbol for use as an ownerIndex to flag the primary ifIndex, which is
 * always the one that maps directly to the ODI board number. This is
 * into SAIAllocateIfIndex in the iib_ownerIndex field, for example.
 */
#define	SAI_IFN_OWNER_PRIMARY	0

/* Set the iib_ownerIndex field to SAI_IFN_OWNER_AUTOMATIC if you want
 * SAI_AllocateIfIndex to allocate the next unallocated secondary owner
 * index for the ODI board indicated by iib_boardNumber. Note that
 * setting this value will *never* allocate the primary ifIndex, but
 * it can be used willy-nilly for secondaries.
 */
#define	SAI_IFN_OWNER_AUTOMATIC	0xffffffff


/* Name:	SAIAllocateIfIndex
 *
 * Purpose:
 *		Allocates an ifIndex value for a new MIB-II interface.
 *
 * Arguments:
 *  ifBlock	Pointer to an IfIndexBlock structure which
 *		describes the relevant points of this interface.
 *		iib_ifIndex is ignored on input. On successful return,
 *		iib_ifIndex is set to the allocated ifIndex value. All
 *		other values should be set appropriately before calling
 *		and should not be changed until SAIDeallocateIfIndex is
 *		called to free the ifIndex.
 *
 *		NOTE: Allocating an ifIndex passes control of this
 *		block to the ifIndex support code. It may not be
 *		modified or deleted while the ifIndex is allocated.
 *		The ifIndex support code may copy some values in this
 *		structure to other data structures, so changing the
 *		values in this block while the ifIndex is allocated
 *		may or may not have any effect on the support.
 *		Short point: DO NOT MODIFY.
 *
 *  resourceTag	Resource tag allocated by passing AllocateResourceTag
 *		a resource type of SAI_IFINDEX_RTAG. The allocated
 *		ifIndex will be charged to this resource tag. The
 *		resource tag is required. (Originally we discussed
 *		automatically allocating a resource tag, but this
 *		will not happen. Passing a zero resource will result
 *		in an SAI_BADTAG return code.)
 *
 * Return codes:
 *  SAI_IFNE_OK		A new ifIndex was successfully allocated and written
 *			into the iib_ifIndex field of the ifBlock.
 *  SAI_IFNE_NOMEM	Insufficient memory to allocate a new ifIndex.
 *  SAI_IFNE_BADTAG	The value in resourceTag does not represent an
 *			resource tag for the resource SAI_IFINDEX_RTAG.
 *  SAI_IFNE_NOTFOUND	The ODI logical board in iib_boardNumber does
 *			not exist.
 *  SAI_IFNE_DUPLICATE	An ifIndex has already been allocated with the
 *			indicated ownerIndex for the indicate ODI logical
 *			board.
 */
extern int	SAIAllocateIfIndex(	IfIndexBlock	*ifBlock,
					unsigned long	resourceTag
				  );

/* Name:	SAIDeallocateIfIndex
 *
 * Purpose:
 *	The obvious mate for SAIAllocateIfIndex: frees up an ifIndex.
 *
 * Special Notes:
 * 	Although possibly not actually required by the implementation,
 *	I hereby declare that this IfIndexBlock *MUST* be the exact
 *	same block used in the SAIAllocateIfIndex call that allocated
 *	this ifIndex. (This rather conveniently requires that the
 *	original block be kept around until the deallocation, something
 *	which almost certainly *will* be required for the implementation.)
 */
extern void	SAIDeallocateIfIndex( IfIndexBlock *ifBlock );

/* Name:	SAIStackIfIndexes
 *
 * Purpose:	Creates a mapping between two ifIndex values. The higher
 *		interface is the one closer to network layer (i.e., IPX).
 *		The lower interface is the one closer to the hardware.
 *		(Note that "higher" and "lower" are merely semantic
 *		conventions of RFC-1573; they have no bearing on the
 *		order of allocation or the owner indexes used when
 *		allocating ifIndexes via SAIAllocateIfIndex.)
 *
 * RFC-1573 allows any arbitrary mappings -- one-to-one, many-to-one,
 * none-to-one -- in all directions.
 *
 * RFC-1573 calls for using zero for the higher or lower layer in a
 * stack to indicate the extremes: the sub-layers which are highest
 * or lowest. I'm not sure under what conditions these extremes are
 * to be reported, but the agent will provide them automatically, so
 * do not specify them explicitly.
 *
 * Return codes:
 *  SAI_IFNE_OK		The indicate stacking was successful.
 *  SAI_IFNE_NOMEM	Insufficient memory to stack these layers.
 *  SAI_IFNE_NOTFOUND	One of the ifIndexes was not found in the ifIndex
 *			database.
 *  SAI_IFNE_DUPLICATE	The indicating stacking already exists. (I doubt
 *			there will be any crime in ignoring this return code,
 *			and I considered never returning it.)
 */
extern int	SAIStackIfIndexes(	unsigned long	higherLayer,
					unsigned long	lowerLayer
			         );

/* Name:	SAIUnstackIfIndexes
 *
 * Purpose:	Destroy a stacking created with SAIStackIfIndexes.
 *
 * Note:	SAIDeallocateIfIndex automatically deletes all related
 *		stackings, so this call isn't normally needed.
 */
extern void	SAIUnstackIfIndexes(	unsigned long	higherLayer,
					unsigned long	lowerLayer
				   );

/* Name:	SAIMapBoardToPrimaryIfIndex
 *
 * Purpose:	Takes an ODI logical board number and returns the primary
 *		ifIndex allocated for that board. Protocol stacks will use
 *		this to map from a logical board they use to send packets
 *		into the ifIndex value they should use in their MIBs.
 *
 * Returns:
 *	Normally returns the ifIndex, but returns SAI_IFN_INVALID if no
 *	mapping exists for the logical board. SAI_IFN_INVALID could simply
 *	mean that the logical board doesn't exist, but assuming that case
 *	is handled in other ways by the caller, it would normally indicate
 *	a logical board that did not have an ifIndex assigned for it.
 *
 * Note:	since, at least initially, we're mapping ODI board numbers
 *		one-to-one into the ifIndex range, the initial implementation of
 *		this routine may simply return odiBoard. In particular,
 *		it may *not* detect an invalid board number and, if so, will
 *		not return SAI_IFN_INVALID at this time.
 */
extern unsigned long SAIMapBoardToPrimaryIfIndex( unsigned long odiBoard );

/* Name:	SAIMapBoardToIfIndex
 *
 * Pupose:	Takes an ODI logical board number and an "owner index"
 *		used when an ifIndex was allocated via SAIAllocateIfIndex,
 *		and returns the ifIndex assigned to that layer.
 *
 * Returns:
 *	Normally returns the ifIndex, but returns SAI_IFN_INVALID if no
 *	mapping exists for the board/index pair. SAI_IFN_INVALID could
 *	simply mean that the logical board doesn't exist, but assuming
 *	that case is handled in other ways by the caller, it would
 *	normally indicate no ifIndex was allocated for that ownerIndex.
 */
extern unsigned long SAIMapBoardToIfIndex(	unsigned long	odiBoard,
						unsigned long	ownerIndex
				         );

/* Name:	SAIMapIfIndexToBoard
 *
 * Purpose:	Takes an ifIndex and returns the ODI logical board that
 *		supports that interface. returns SAI_IFN_INVALID if the given
 *		ifIndex is not present.
 */
extern unsigned long SAIMapIfIndexToBoard( unsigned long ifIndex );



/*
 *	SNMP Communications module support.
 */

extern int  SACProcessRequest (TLV *src_addr,
				int request_ln, unsigned char *request_pt,
	                        int reply_buffer_ln, unsigned char *reply_buffer,
	                        int bypass_authentication);

#define    PROCESS_REQ_FAIL_PARSE         -1       /* Parse Failure */
#define    PROCESS_REQ_FAIL_VERSION       -2       /* Wrong SNMP Version # */
#define    PROCESS_REQ_FAIL_AUTH          -3       /* Authentication Failure */
#define    PROCESS_REQ_FAIL_BAD_PDU       -4       /* Bad PDU */
#define    PROCESS_REQ_FAIL_NO_MEM        -5       /* Agent is Out of Memory */
#define    PROCESS_REQ_FAIL_GENERAL       -6       /* General Misc. Error */

extern int SACReadSNMPObject  (int object_id_ln, unsigned long *object_id_pt,
	                        TLV *value_buffer);
extern int SACReadAttributes  (unsigned long *object_id_pt,
				int inst_ct, TLV *inst_pt,
	                        int attr_ct, TLV *attr_pt);
extern int SACSetAttributes   (unsigned long *object_id_pt,
				int inst_ct, TLV *inst_pt,
	                        int attr_ct, TLV *attr_pt);
extern int SACReadNextAttributes(unsigned long *object_id_pt,
			 	int inst_ct, struct TLV *inst_pt,
				int next_ct, struct TLV *next_pt,
				int attr_ct, struct TLV *attr_pt);

/*
 * ============================================================================
 *
 * SACTrap - send a trap to all configured trap targets.
 *
 * PAREMETERS
 *
 * enterpriseId - the object id to be placed inthe Enterprise ID part of
 *                the trap PDU.
 *
 * generictype  - one of the six generic trap types:
 *
 *                0 - cold start,
 *                1 - warm start,
 *                2 - link down,
 *                3 - link up,
 *                4 - authentication failure
 *                5 - egp neighbor loss,
 *                6 - enterprise specific
 *
 * specific type - If genericType is enterprise specific (6), this parameter
 *                 identifies the trap within enterpriseId.
 *
 * varBindCount - the number of var binds in the array varBinds.
 *
 * varBinds - array of var binds to be encoded into the trap PDU.
 *
 * communityString - the community string to be encoded into the trap PDU.
 *                   NULL indicates that SACTrap() should use the load time
 *                   configured trap community ("public" by default).
 *
 * RETURNS:
 *    nothing
 */

extern void SACTrap(	unsigned long	*enterpriseId,
			int		genericType,
			int		specificType,
			int		varBindCount,
			struct VarBind	*varBinds,
			char		*communityString
		   );
extern void SACGenericTrap(	int		genericType,
				int		varBindCount,
				struct VarBind	*varBinds,
				char		*communityString
			  );

/*
	constants defining the generic and enterprise specific traps from
	rfc 1157.
*/

#define SNMP_TRAP_COLDSTART		(0x0)
#define SNMP_TRAP_WARMSTART		(0x1)
#define SNMP_TRAP_LINKDOWN		(0x2)
#define SNMP_TRAP_LINKUP		(0x3)
#define SNMP_TRAP_AUTHFAIL		(0x4)
#define SNMP_TRAP_EGPNEIGHBORLOSS	(0x5)
#define SNMP_TRAP_ENTERPRISESPECIFIC	(0x6)


/*
	constant strings used for the xport_name parameter in SACRegTrapHandler(),
	SACDeregTrapHandler(), SACTrapToTarget(), and the xport_protocol parameter
	in SACAddTrapTarget() and SACDeleteTrapTarget
*/

#define SACTRAP_XPORT_NAME_IPX	"IPX"
#define SACTRAP_XPORT_NAME_UDP	"UDP"

extern void SACDeregTrapHandler( char *xport_name );

extern int SACRegTrapHandler(	SACTrapHandlerPROC	*handler,
				char			*xport_name,
				struct TLV		*local_addr,
				unsigned long		rtag
			    );

/*
	constants defining the returns codes of SACRegTrapHandler()
*/

#define	SAC_REGISTER_OK		    0	/* Registration successful */
#define	SAC_REGISTER_FAIL_DUP	   -1	/* transport already registered */
#define	SAC_REGISTER_BAD_RTAG	   -2	/* resource tag was bogus */
#define	SAC_REGISTER_NO_MEM	   -3	/* couldn't get memory */
#define	SAC_REGISTER_FAIL_BAD_ADDR -4	/* local address type not supported */





/*
	constants defining the returns codes of SACRegTrapReceiver()
*/

#define	SAC_RECEIVE_REGISTER_OK		    0	/* Registration successful */
#define	SAC_RECEIVE_REGISTER_BAD_RTAG	   -1	/* resource tag was bogus */
#define	SAC_RECEIVE_REGISTER_NO_MEM	   -2	/* couldn't get memory */



/*
	constants defining the returns codes of SACRegTrapFilter()
*/

#define	SAC_FILTER_REGISTER_OK		    0	/* Registration successful */
#define	SAC_FILTER_REGISTER_BAD_RTAG	   -1	/* resource tag was bogus */
#define	SAC_FILTER_REGISTER_NO_MEM	   -2	/* couldn't get memory */

/*
 * ============================================================================
 *
 * SACTrapToTarget - send a trap to a specific network address instead of
 *                   all configured trap targets.
 *
 * PAREMETERS
 *
 * The first six arguments are the same as for SACTrap.
 *
 * securityControl - a place holder for future security specifications.
 *                   It must be NULL for now.
 *
 * xportName - the case insensitive name of the transport on which the trap
 *             should be sent. The currently defined transports are "UDP"
 *             and "IPX" and should be specified using one of the
 *             SACTRAP_XPORT_NAME* constants defined in agent.h. (It is
 *             not a coincidence that these are the names of the existing
 *             registered trap handlers.)
 *
 * targetAddress is defined differently depending on the transport:
 *
 * 	For UDP, in this first version the fields of targetAddress
 *      should be as follows:
 *
 *       tlv_value points to a four-byte buffer containing the IP address in
 *                 network order.  For example if the target address is
 *                 127.0.0.1, the first byte should be decimal 127, the
 *                 second byte 0, the third byte 0 and fourth byte 1.
 *
 *       tlv_length should be 4.
 *
 *       tlv_type should be zero.
 *
 *       tlv_id should be zero.
 *
 * 	For IPX, the fields of targetAddress should be as follows:
 *
 *       tlv_value points to an IPX_ADDR block as defined in
 *                 sjf-ipd\vol1:include\nwsdk\v0400\tispxipx.h.
 *
 *       tlv_length should be sizeof(IPX_ADDR).
 *
 *       tlv_type should be set to ASN_IPADDRESS (which is defined in agent.h,
 *       and no that's not a typo, there is no such constant ASN_IPXADDRESS).
 *
 *       tlv_id should be zero.
 *
 * The future plan for UDP is to have tlv_value point to a sockaddr_in block
 * as defined in sjf-ipd\vol1:include\nwsdk\v0400\netinet\in.h. This will
 * allow the SACTrapToTarget() caller to specify a different port which will
 * allow the caller to direct a trap to an application onther than the one
 * listening on the well-defined trap port. To provide this feature though
 * requires a change to TCPIP.NLM, which is something we won't be able to do
 * for MPR 3.1. Of course, the plan is also to be backward compatible with
 * the current implementation, i.e. tlv_value points to the four-byte IP
 * address.
 *
 * So, in any case, tlv_value points to the address or address structure and
 * tlv_length is set to the number of bytes in the address or address
 * structure.
 */

extern void SACTrapToTarget(
		unsigned long	*enterpriseId,
		int		genericType,
		int		specificType,
		int		varBindCount,
		struct VarBind	*varBinds,
		char		*communityString,
		void		*securityControl,	/* must be NULL */
		char		*xportName,
		struct TLV	*targetAddress
	   );

/* special hooks to allow protocol stacks to register there
 * own traps, to allow command line backwards compatibility
 * with circa 3.11 utilities.
 */
extern int SACAddTrapTarget(	char		*xport_protocol,
				struct TLV	*target_addr
			   );
extern int SACDeleteTrapTarget(	char		*xport_protocol,
				struct TLV	*target_addr
			      );

/*
	constants defining the return codes of SACAddTrapTarget() and
	SACDeleteTrapTarget().
*/

#define	SAC_TARGET_OK		0	/* target operation successful */
#define	SAC_TARGET_BAD_XPORT	-1	/* transport protocol not found */
#define	SAC_TARGET_NO_MEM	-2	/* couldn't get necessary memory */
#define	SAC_TARGET_NOT_FOUND	-3	/* target being deleted not found */

/* file access routines */

extern int SACNextConfigLine(	FILE		**fileHandle,
				char		*fileName,
				char		*signPostToken,
				char		*typeToken,
				char		**lineBuffer,
				int		bufferLength
			    );
extern int SACNextTrapLine(	FILE		**fileHandle,
				char		*xportName,
				char		**lineBuffer,
				int		bufferLength
			  );
extern int SACInitFromConfig(	char		*signPost,
				char		*sysBuffer,
				unsigned	bufferLength
			    );


extern int SACRegTrapReceiver(
		unsigned long	*enterpriseId,
		SACTrapReceiverPROC * trapReceiver,
		void 					  * context_ptr,
		unsigned long 			trapReceiveRtag
		);

extern int SACDeregTrapReceiver(
		unsigned long	*enterpriseId,
		SACTrapReceiverPROC * trapReceiver
		);



extern int SACRegOutgoingTrapReceiver(
		SACTrapReceiverPROC * trapReceiver,
		void 					  * context_ptr,
		unsigned long 			trapReceiveRtag
		);


extern void SACDeregOutgoingTrapReceiver(
	SACTrapReceiverPROC	*trapReceiver);


extern void SACDeliverTrap( char *xportName,
							struct TLV *sourceAddr,
							int 		packetLength,
							unsigned char *packetPtr
						 );

#define	SAI_FILTER_TRAP_HANDLED_OK 0
#define SAI_FILTER_TRAP_DISREGARD 1
#define SAI_FILTER_TRAP_SEND_TO_OTHERS 2
#define SAI_FILTER_TRAP_ERROR 		-1


typedef int SACTrapFilterPROC
		(
		unsigned long	*enterpriseId,
		int		genericType,
		int		specificType,
		int		varBindCount,
		struct VarBind	*varBinds,
		char		*communityString,
		void  	* context_ptr
	   );

extern int SACRegisterTrapFilter(
		unsigned long		*enterpriseId,
		SACTrapFilterPROC	*filterProc,
		void					* context_ptr,
		unsigned long 		trapReceiveRtag
	 );


extern void SACDeregisterTrapFilter(
		SACTrapFilterPROC	*filterProc
	 );

#endif /* #ifndef _AGENT_H_ */

