
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//
//  Copyright (c) 1995-2004 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for MIB Plug-in Implementer API
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

/** @file */

#ifndef __DCSMSNMP_H_INCLUDED__
#define __DCSMSNMP_H_INCLUDED__


//-----------------------------------------------------------------------------
// Begin Dependent Includes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core header files
//-----------------------------------------------------------------------------
#include <dcossdk.h>
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP header files
//-----------------------------------------------------------------------------
#include <agent.h>						// provides ObjInfo data structure


//-----------------------------------------------------------------------------
// End Dependent Includes
//-----------------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// Begin Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// MIB Plug-in Implementer API Version
//-----------------------------------------------------------------------------

/** MIB Plug-in Implementer API Version Info: \n
 *  Major version supported */
#define SM_SNMP_API_MAJOR_VER				(1)

/** MIB Plug-in Implementer API Version Info: \n
 *  Minor version supported */
#define SM_SNMP_API_MINOR_VER				(0)

/** MIB Plug-in Implementer API Version Info: \n
 *  Revision supported */
#define SM_SNMP_API_REVISION				(0)


//-----------------------------------------------------------------------------
// MIB Plug-in Implementer API Request Types
// (for Pull API of MIB Plug-in Implementer API)
//-----------------------------------------------------------------------------

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to return major version
 *  of MIB Plug-in Implementer API that it supports */
#define SM_SNMP_API_MPI_GET_API_MAJOR_VER	(1)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to return minor version
 *  of MIB Plug-in Implementer API that it supports */
#define SM_SNMP_API_MPI_GET_API_MINOR_VER	(2)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to return revision
 *  of MIB Plug-in Implementer API that it supports */
#define SM_SNMP_API_MPI_GET_API_REVISION	(3)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to initialize because it has been loaded */
#define SM_SNMP_API_MPI_LOAD				(4)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to deinitialize because it is being unloaded */
#define SM_SNMP_API_MPI_UNLOAD				(5)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to list the MIBs that it implements */
#define SM_SNMP_API_MPI_LIST_MIB			(6)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to start monitoring for status changes */
#define SM_SNMP_API_MPI_START_MONITOR		(7)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to stop monitoring for status changes */
#define SM_SNMP_API_MPI_STOP_MONITOR		(8)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to get value for specified MIB variable */
#define SM_SNMP_API_MPI_GET					(9)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to get value for next MIB variable
 *  after specified MIB variable */
#define SM_SNMP_API_MPI_GET_NEXT			(10)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to test ability to set value
 *  for specified MIB variable */
#define SM_SNMP_API_MPI_SET_TEST			(11)

/** MIB Plug-in Implementer Pull API Request Type: \n
 *  Request for MIB implementer to set value for specified MIB variable */
#define SM_SNMP_API_MPI_SET_COMMIT			(12)


//-----------------------------------------------------------------------------
// MIB Plug-in Implementer API Status Codes
//-----------------------------------------------------------------------------

// SNMPv1 status codes

/** MIB Plug-in Implementer API Status: \n
 *  No error */
#define SM_SNMP_STATUS_NO_ERROR				(0)

/** MIB Plug-in Implementer API Status: \n
 *  Data too big */
#define SM_SNMP_STATUS_TOO_BIG				(1)

/** MIB Plug-in Implementer API Status: \n
 *  No such name */
#define SM_SNMP_STATUS_NO_SUCH_NAME			(2)

/** MIB Plug-in Implementer API Status: \n
 *  Bad value */
#define SM_SNMP_STATUS_BAD_VALUE			(3)

/** MIB Plug-in Implementer API Status: \n
 *  Read only */
#define SM_SNMP_STATUS_READ_ONLY			(4)

/** MIB Plug-in Implementer API Status: \n
 *  General error */
#define SM_SNMP_STATUS_GEN_ERR				(5)

// Extended status codes (not for use with SNMP requests)

/** MIB Plug-in Implementer API Status: \n
 *  Duplicate registration (not for use with SNMP requests) */
#define SM_SNMP_STATUS_EX_DUPLICATE_REG		(263)


//-----------------------------------------------------------------------------
// MIB Plug-in Implementer API Value Types (SNMPv1 value types)
//-----------------------------------------------------------------------------

/** MIB Plug-in Implementer API Value Type: INTEGER \n
 *  SMSnmpValue Usage: 'val32' contains 32-bit integer value */
#define SM_SNMP_TYPE_INTEGER				(0x02)

/** MIB Plug-in Implementer API Value Type: OCTET STRING \n
 *  SMSnmpValue Usage: 'valptr' points to octet string and
 *  'val32' contains length of octet string */
#define SM_SNMP_TYPE_OCTET_STRING			(0x04)

/** MIB Plug-in Implementer API Value Type: NULL \n
 *  SMSnmpValue Usage: no value */
#define SM_SNMP_TYPE_NULL					(0x05)

/** MIB Plug-in Implementer API Value Type: OBJECT IDENTIFIER \n
 *  SMSnmpValue Usage: 'valptr' points to array of 32-bit integer IDs and
 *  'val32' contains number of IDs */
#define SM_SNMP_TYPE_OBJ_ID					(0x06)

/** MIB Plug-in Implementer API Value Type: IP ADDRESS \n
 *  SMSnmpValue Usage: 'valptr' points to 4-byte octet string and
 *  'val32' contains length of octet string */
#define SM_SNMP_TYPE_IP_ADDRESS				(0x40)

/** MIB Plug-in Implementer API Value Type: COUNTER \n
 *  SMSnmpValue Usage: 'val32' contains 32-bit integer value */
#define SM_SNMP_TYPE_COUNTER				(0x41)

/** MIB Plug-in Implementer API Value Type: GAUGE \n
 *  SMSnmpValue Usage: 'val32' contains 32-bit integer value */
#define SM_SNMP_TYPE_GAUGE					(0x42)

/** MIB Plug-in Implementer API Value Type: TIMETICKS \n
 *  SMSnmpValue Usage: 'val32' contains 32-bit integer value */
#define SM_SNMP_TYPE_TIMETICKS				(0x43)

/** MIB Plug-in Implementer API Value Type: OPAQUE \n
 *  SMSnmpValue Usage: 'valptr' points to octet string and
 *  'val32' contains length of octet string */
#define SM_SNMP_TYPE_OPAQUE					(0x44)


//-----------------------------------------------------------------------------
// SNMP Trap Types
//-----------------------------------------------------------------------------

/** Generic trap ID for enterprise specific trap */
#define SM_SNMP_GENTRAP_ENTERPRISE_SPECIFIC	(6)


//-----------------------------------------------------------------------------
// SNMP Version Bit Masks
//-----------------------------------------------------------------------------

/** SNMP Version Info: \n
 *  Value that represents no SNMP version */
#define SM_SNMP_SNMPVER_NONE				(0)

/** SNMP Version Info: \n
 *  Bit mask that represents SNMPv1 */
#define SM_SNMP_SNMPVER_V1					BIT(0)

/** SNMP Version Info: \n
 *  Bit mask that represents SNMPv2C */
#define SM_SNMP_SNMPVER_V2C					BIT(1)

/** SNMP Version Info: \n
 *  Bit mask that represents SNMPv3 */
#define SM_SNMP_SNMPVER_V3					BIT(2)


//-----------------------------------------------------------------------------
// SNMP OID Definitions and Utility Macros
//-----------------------------------------------------------------------------

/** Zero-based index for position of enterprise ID in SNMP OID */
#define SM_SNMP_OID_ENTERPRISE_ID_INDEX		(6)

/** Minimum number of IDs in SNMP OID */
#define SM_SNMP_OID_MIN_NUMIDS				(2)

/** Maximum number of IDs in SNMP OID */
#define SM_SNMP_OID_MAX_NUMIDS				(128)

/** Macro to calculate number of IDs in IDs array (X) */
#define SM_SNMP_OID_NUMIDS(X)				(sizeof(X) / sizeof((X)[0]))

/** Macro to initialize variable of type SMSnmpOid where number of IDs
 *  is automatically calculated from IDs array (X) \n
 *  Example: \n
 *  u32 myIds[] = {1, 3, 6, 1, 4, 1, 674, 99999, 1}; \n
 *  SMSnmpOid myOid = SM_SNMP_OID_INITIALIZER(myIds); */
#define SM_SNMP_OID_INITIALIZER(X)			{SM_SNMP_OID_NUMIDS(X), {0}, (X)}

/** Macro to initialize variable of type SMSnmpOid where number of IDs (Y)
 *  is specified by user \n
 *  Example: \n
 *  u32 myIds[] = {1, 3, 6, 1, 4, 1, 674, 99999, 1}; \n
 *  SMSnmpOid myOid = SM_SNMP_OID_INITIALIZER_NUMIDS(myIds, 9); */
#define SM_SNMP_OID_INITIALIZER_NUMIDS(X,Y)	{(Y), {0}, (X)}

/** Macro to convert number of IDs (X) in SNMP OID to size in bytes */
#define SM_SNMP_OID_NUMIDS2SIZE(X)			((X) * sizeof(u32))

/** Macro to convert size in bytes (X) to number of IDs in SNMP OID */
#define SM_SNMP_OID_SIZE2NUMIDS(X)			((X) / sizeof(u32))


//-----------------------------------------------------------------------------
// Common Data Structures
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpOid represents a SNMP object identifier (OID) in the
 * MIB Plug-in Implementer API.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpOid
{
	/** Number of IDs in OID. */
	u32 numIds;

	/** Alignment for next data element. Reserved for future use. */
	u8 reservedAlign[4];

	/** Pointer to array of IDs (32-bit integers) that comprise OID. */
	u32 * ids;
} SMSnmpOid;


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpValue represents a SNMP value in the MIB Plug-in Implementer API.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpValue
{
	/** Value type. */
	u32 type;

	/** Integer value OR length of value pointed to by valptr. */
	u32 val32;

	/**
	 * Pointer to value for value types:
	 * SM_SNMP_TYPE_OCTET_STRING,
	 * SM_SNMP_TYPE_OBJ_ID,
	 * SM_SNMP_TYPE_IP_ADDRESS,
	 * SM_SNMP_TYPE_OPAQUE.
	 */
	void * valptr;
} SMSnmpValue;


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpVarBind represents a SNMP variable binding (VarBind) in the
 * MIB Plug-in Implementer API.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpVarBind
{
	/** Variable name. */
	SMSnmpOid name;

	/** Variable value. */
	SMSnmpValue value;
} SMSnmpVarBind;


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpMPIMPMDispatchTable defines the dispatch table for the Push API
 * of the MIB Plug-in Implementer API.  The functions in the Push API are
 * MIB Plug-in Manager functions called by the MIB Plug-in Implementer.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpMPIMPMDispatchTable
{
	//-------------------------------------------------------------------------
	// Interface version information API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a GetMajorVersion is called to obtain the major version of the
	 * MIB Plug-in Implementer API supported by the MIB Plug-in Manager.
	 * This API can be used to perform version control for the 
	 * module and to determine which features are supported.
	 * 
	 * @param None.
	 *
	 * @return Major version of the MIB Plug-in Implementer API
	 * that the MIB Plug-in Manager supports.
	 */
	//-------------------------------------------------------------------------
    u32 (_cdecl *GetMajorVersion)(void);

	//-------------------------------------------------------------------------
	/**
	 * \a GetMinorVersion is called to obtain the minor version of the
	 * MIB Plug-in Implementer API supported by the MIB Plug-in Manager.
	 * This API can be used to perform version control for the 
	 * module and to determine which features are supported.
	 * 
	 * @param None.
	 *
	 * @return Minor version of the MIB Plug-in Implementer API
	 * that the MIB Plug-in Manager supports.
	 */
	//-------------------------------------------------------------------------
    u32 (_cdecl *GetMinorVersion)(void);

	//-------------------------------------------------------------------------
	/**
	 * \a GetRevision is called to obtain the revision of the
	 * MIB Plug-in Implementer API supported by the MIB Plug-in Manager.
	 * This API can be used to perform version control for the 
	 * module and to determine which features are supported.
	 * 
	 * @param None.
	 *
	 * @return Revision of the MIB Plug-in Implementer API
	 * that the MIB Plug-in Manager supports.
	 */
	//-------------------------------------------------------------------------
    u32 (_cdecl *GetRevision)(void);


	//-------------------------------------------------------------------------
	// Interface heap API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a FreeGeneric is called to free dynamically allocated memory returned
	 * from any Push API which does NOT have a type set to "const".
	 *
	 * @note The caller cannot use the C Lib "free" API method to release 
	 * memory returned by the Push API.  The memory returned by the Push API
	 * may not be compatible with the C Lib "free" API method and may lead
	 * to undefined execution.
	 *
	 * @param pMem [in] Pointer to previously returned memory 
	 * that is to be freed.
	 *
	 * @return None.
	 */
	//-------------------------------------------------------------------------
    void (_cdecl *FreeGeneric)(_IN void * pMem);


	//-------------------------------------------------------------------------
	// MIB Plug-in Manager information API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a IsMIBManagerReady is called to get the ready status of the
	 * MIB Plug-in Manager.  This API can be used to see if the
	 * MIB Plug-in Manager is ready to dispatch SNMP requests and
	 * send SNMP traps.
	 * 
	 * @param None.
	 *
	 * @return TRUE if the MIB Plug-in Manager is ready; else FALSE.
	 */
	//-------------------------------------------------------------------------
	booln (_cdecl *IsMIBManagerReady)(void);

	//-------------------------------------------------------------------------
	/**
	 * \a GetMIBEnterpriseID is called to determine if the MIB Plug-in Manager
	 * has been configured with an alternate MIB enterprise ID to use.
	 * This API can be used by the MIB Plug-in Implementer to obtain
	 * a possible alternate MIB enterprise ID for the MIB Plug-in Implementer
	 * to use in the SNMP OIDs for all MIB objects that the MIB Plug-in
	 * Implementer supports.  If the value returned by this API is zero,
	 * the MIB Plug-in Implementer should use the default MIB enterprise ID
	 * for the MIB that it implements.  If the value returned by this API
	 * is non-zero, the MIB Plug-in Implementer should use this value
	 * for the MIB enterprise ID and update all variables used to represent
	 * the SNMP OIDs for all MIB objects that it supports.  In SNMP OIDs,
	 * the MIB enterprise ID is the identifier after the enterprises identifier.
	 * In the SNMP OID 1.3.6.1.4.1.674, the MIB enterprise ID is 674.
	 * 
	 * This feature is intended for possible use by OEMs.  It provides
	 * a single point of configuration in the MIB Plug-in Manager for
	 * changing the MIB enterprise ID supported by the MIB Plug-in Implementers
	 * at runtime.  See a sample MIB Plug-in Implementer for an example
	 * of supporting this feature.
	 * 
	 * @param None.
	 *
	 * @return Zero means use default MIB enterprise ID defined in MIB
	 * implemented by MIB Plug-in Implementer.  Non-zero means MIB Plug-in
	 * Manager has been configured with alternate MIB enterprise ID to use.
	 */
	//-------------------------------------------------------------------------
	u32 (_cdecl *GetMIBEnterpriseID)(void);

	//-------------------------------------------------------------------------
	/**
	 * \a IsSNMPSetsEnabled is called to determine whether the MIB Plug-in Manager
	 * will dispatch SNMP Set requests (SM_SNMP_API_MPI_SET_TEST and
	 * SM_SNMP_API_MPI_SET_COMMIT) to the MIB Plug-in Implementer.
	 * 
	 * @param None.
	 *
	 * @return TRUE if the MIB Plug-in Manager will dispatch SNMP Set requests
	 * to the MIB Plug-in Implementer; else FALSE.
	 */
	//-------------------------------------------------------------------------
	booln (_cdecl *IsSNMPSetsEnabled)(void);


	//-------------------------------------------------------------------------
	// SNMP trap API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a SendTrap is called to send a SNMP trap.
	 *
	 * @param mpitoken [in] Token that was passed to the MIB Plug-in Implementer
	 * on the SM_SNMP_API_MPI_LOAD request.
	 *
	 * @param pEnterpriseOid [in] Pointer to enterprise OID for trap.
	 *
	 * @param genericTrapId [in] Generic trap ID.  Should be
	 * SM_SNMP_GENTRAP_ENTERPRISE_SPECIFIC.
	 *
	 * @param specificTrapId [in] Enterprise specific trap ID.
	 *
	 * @param numVarBinds [in] Number of variable bindings in trap.
	 *
	 * @param pVarBinds [in] Pointer to array of variable bindings for trap.
	 * Size of array is specified by numVarBinds parameter.
	 * The MIB Plug-in Manager does not free memory for this parameter.
	 * If memory for this parameter needs to be freed, the MIB Plug-in
	 * Implementer must free the memory after this API returns.
	 *
	 * @return On success returns SM_SNMP_STATUS_NO_ERROR.  On error returns
	 * one of the other MIB Plug-in Implementer API status codes.
	 */
	//-------------------------------------------------------------------------
	s32 (_cdecl *SendTrap)(_IN const void * mpiToken,
						   _IN SMSnmpOid * pEnterpriseOid,
						   _IN u32 genericTrapId,
						   _IN u32 specificTrapId,
						   _IN u32 numVarBinds,
						   _IN SMSnmpVarBind * pVarBinds);


	//-------------------------------------------------------------------------
	// Shared MIB table API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a RegisterMIBTableIndex is called to register an index for a row in a
	 * table that it is sharing with another MIB Plug-in Implementer.
	 * If a MIB Plug-in Implementer is sharing a MIB table, it must register
	 * the indexes for the rows in the table that it supports so that the
	 * MIB Plug-in Manager can route requests for those rows to the
	 * MIB Plug-in Implementer properly.  If a MIB Plug-in Implementer is not
	 * sharing a MIB table, it does not need to use this API to register
	 * table indexes.
	 *
	 * If this API returns SM_SNMP_STATUS_EX_DUPLICATE_REG, the registration
	 * failed because the index is already registered.  If the MIB Plug-in
	 * Implementer uses arbitrary index values (i.e. does not need a specific
	 * index), the MIB Plug-in Implementer may try to register another index.
	 * This could be as simple as incrementing the index by one and calling
	 * the API again.
	 *
	 * @note This API should not be called until the MIB registration cycle
	 * is complete which means this API should not be called until the
	 * SM_SNMP_API_MPI_START_MONITOR request is received.
	 * 
	 * @param mpiToken [in] Token that was passed to the MIB Plug-in Implementer
	 * on the SM_SNMP_API_MPI_LOAD request.
	 *
	 * @param pTableEntryObjInfo [in] Points to the ObjInfo structure for the
	 * shared table's Table Entry abstract object.  This pointer must be one
	 * of the pointers in the MIB information returned by the MIB Plug-in
	 * Implementer on the SM_SNMP_API_MPI_LIST_MIB request.  This information
	 * is required so that the MIB Plug-in Manager can update its MIB database
	 * in order to properly route requests for the table row being registered.
	 *
	 * @param pIndexOid [in] Points to a partial SNMP OID that specifies the
	 * index for the table row that the MIB Plug-in Implementer wants to
	 * register to support.  For example, if the index for a table is a single
	 * integer value, the number of IDs in the index OID would be one.
	 * 
	 * @return On success returns SM_SNMP_STATUS_NO_ERROR.
	 * If index is already registered, returns SM_SNMP_STATUS_EX_DUPLICATE_REG.
	 * On other error returns one of the other MIB Plug-in Implementer API
	 * status codes.
	 */
	//-------------------------------------------------------------------------
	s32 (_cdecl *RegisterMIBTableIndex)(_IN const void * mpiToken,
										_IN ObjInfo * pTableEntryObjInfo,
										_IN SMSnmpOid * pIndexOid);

	//-------------------------------------------------------------------------
	/**
	 * \a UnregisterMIBTableIndex is called to unregister an index for a row
	 * in a table that it is sharing with another MIB Plug-in Implementer.
	 * The MIB Plug-in Implementer uses this API to unregister a table index
	 * that it registered using the \a RegisterMIBTableIndex API.
	 * 
	 * @note This API should not be called until the MIB registration cycle
	 * is complete which means this API should not be called until the
	 * SM_SNMP_API_MPI_START_MONITOR request is received.
	 * 
	 * @param mpiToken [in] Token that was passed to the MIB Plug-in Implementer
	 * on the SM_SNMP_API_MPI_LOAD request.
	 *
	 * @param pTableEntryObjInfo [in] Points to the ObjInfo structure for the
	 * shared table's Table Entry abstract object.  This pointer must be one
	 * of the pointers in the MIB information returned by the MIB Plug-in
	 * Implementer on the SM_SNMP_API_MPI_LIST_MIB request.  This information
	 * is required so that the MIB Plug-in Manager can update its MIB database
	 * in order to remove the table row being unregistered.
	 *
	 * @param pIndexOid [in] Points to a partial SNMP OID that specifies the
	 * index for the table row that the MIB Plug-in Implementer wants to
	 * unregister.  For example, if the index for a table is a single
	 * integer value, the number of IDs in the index OID would be one.
	 * 
	 * @return On success returns SM_SNMP_STATUS_NO_ERROR.
	 * On error returns one of the other MIB Plug-in Implementer API
	 * status codes.
	 */
	//-------------------------------------------------------------------------
	s32 (_cdecl *UnregisterMIBTableIndex)(_IN const void * mpiToken,
										  _IN ObjInfo * pTableEntryObjInfo,
										  _IN SMSnmpOid * pIndexOid);


	//-------------------------------------------------------------------------
	// OS SNMP Agent information API
	//-------------------------------------------------------------------------

	//-------------------------------------------------------------------------
	/**
	 * \a SNMPAgentIsSparseTableSupported is called to determine whether the
	 * OS SNMP Agent properly supports sparse MIB tables.  The OS SNMP Agents
	 * on Windows and Linux properly support sparse MIB tables.  The OS SNMP
	 * Agent on earlier versions of NetWare does not properly support sparse
	 * MIB tables.  If a SNMP management application sends a SNMP GetNext
	 * request for multiple SNMP OIDs that are part of a row in a sparse MIB
	 * table, it may receive incorrect values in the response if the request
	 * is processed by a version of the OS SNMP Agent for NetWare that does
	 * not properly support sparse MIB tables.  This problem was fixed
	 * recently by Novell.  The MIB Plug-in Manager will detect if the
	 * OS SNMP Agent properly supports sparse MIB tables and will indicate
	 * that via this API.
	 * 
	 * @param None.
	 *
	 * @return TRUE if the OS SNMP Agent properly supports sparse MIB tables;
	 * else FALSE.
	 */
	//-------------------------------------------------------------------------
	booln (_cdecl *SNMPAgentIsSparseTableSupported)(void);

	//-------------------------------------------------------------------------
	/**
	 * \a SNMPAgentGetSupportedSNMPVersions is called to determine the SNMP
	 * versions supported by the OS SNMP Agent.  The return value is a
	 * bitfield composed of the ORed SM_SNMP_SNMPVER_* bitmask values defined
	 * in dcsmsnmp.h.
	 * 
	 * @param None.
	 *
	 * @return Bitfield that indicates the SNMP versions supported by the
	 * OS SNMP Agent.
	 */
	//-------------------------------------------------------------------------
	u8 (_cdecl *SNMPAgentGetSupportedSNMPVersions)(void);

} SMSnmpMPIMPMDispatchTable;


//-----------------------------------------------------------------------------
// Request Data Structures
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpLoadReq defines the request data for a SM_SNMP_API_MPI_LOAD request.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpLoadReq
{
	/**
	 * Pointer to MIB Plug-in Implementer to MIB Plug-in Manager
	 * push API dispatch table.
	 */
	const SMSnmpMPIMPMDispatchTable * pMPIMPMDT;

	/**
	 * Token assigned to MIB Plug-in Implementer and
	 * passed by MIB Plug-in Implementer when calling certain push APIs.
	 */
	const void * mpiToken;
} SMSnmpLoadReq;


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpApiReq defines the request data for a MIB Plug-in Implementer API
 * request.
 */
//-----------------------------------------------------------------------------
typedef struct _IN _SMSnmpApiReq
{
	/** Choose the representation that fits the request. */
	union _ReqData
	{
		/**
		 * Request data for:
		 * SM_SNMP_API_MPI_LOAD.
		 */
		SMSnmpLoadReq loadReq;

		/**
		 * Request data for:
		 * SM_SNMP_API_MPI_GET,
		 * SM_SNMP_API_MPI_GET_NEXT,
		 * SM_SNMP_API_MPI_SET_TEST,
		 * SM_SNMP_API_MPI_SET_COMMIT.
		 */
		SMSnmpVarBind varBind;
	} ReqData;
} SMSnmpApiReq;


//-----------------------------------------------------------------------------
// Response Data Structures
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpMibList defines the response data for a response to a
 * SM_SNMP_API_MPI_LIST_MIB request.
 */
//-----------------------------------------------------------------------------
typedef struct _SMSnmpMibList
{
	/** Number of pointers to MIBs in list. */
	u32 mibCount;

	/** Alignment for next data element. Reserved for future use. */
	u8 reservedAlign[4];

	/** Pointer to first MIB followed by variable number of pointers. */
	ObjInfo ** mibList[1];
} SMSnmpMibList;


//-----------------------------------------------------------------------------
/** 
 * \a SMSnmpApiRsp defines the response data for a response to a
 * MIB Plug-in Implementer API request.
 */
//-----------------------------------------------------------------------------
typedef struct _OUT _SMSnmpApiRsp
{
	/** Choose the representation that fits the response. */
	union _RspData
	{
		/**
		 * Response data for:
		 * SM_SNMP_API_MPI_GET_API_MAJOR_VER.
		 */
		u32 majorVer;

		/**
		 * Response data for:
		 * SM_SNMP_API_MPI_GET_API_MINOR_VER.
		 */
		u32 minorVer;

		/**
		 * Response data for:
		 * SM_SNMP_API_MPI_GET_API_REVISION.
		 */
		u32 revision;

		/**
		 * Response data for:
		 * SM_SNMP_API_MPI_LIST_MIB.
		 */
		SMSnmpMibList mibList;

		/**
		 * Response data for:
		 * SM_SNMP_API_MPI_GET,
		 * SM_SNMP_API_MPI_GET_NEXT.
		 */
		SMSnmpVarBind varBind;
	} RspData;
} SMSnmpApiRsp;


//-----------------------------------------------------------------------------
/** 
 * MIB Plug-in Implementer Dispatch Entry Point Function Type Definition
 *
 * This function is part of the Pull API of the MIB Plug-in Implementer API
 * and is called by the MIB Plug-in Manager to dispatch requests to and get
 * responses from MIB Plug-in Implementers.
 *
 * @note This should only be used as a prototype.
 *
 * The actual exported function name has the form
 * MODULEXX_SMMIBImplementerDispatch.
 *
 * Example: If the name of the MIB Plug-in Implementer (MODULEXX) is SAMMPI32,
 * the exported function name is SAMMPI32_SMMIBImplementerDispatch.
 *
 * Using the makefiles from the Data Engine SDK, the way to set up
 * the exported function name for MIB Plug-in Implementer compilation is: \n
 * \b NT: Change preprocessor value of MPIEXPORT_DISPATCHFN to
 *     MODULEXX_SMMIBImplementerDispatch. \n
 * \b NW: Change makefile value of LOC_MPI_DISPATCH_NAME to required
 *     MODULEXX_SMMIBImplementerDispatch. \n
 * \b LX: Change makefile value of LOC_MPI_DISPATCH_NAME to required
 *     MODULEXX_SMMIBImplementerDispatch.
 */
//-----------------------------------------------------------------------------
typedef s32 (_cdecl *PFNSMMIBIMPLEMENTERDISPATCH)(_IN u32 reqType,
												  _IN const void *pInBuf, 
												  _IN u32 inBufSize,
												  _OUT void *pOutBuf, 
												  _IN u32 outBufSize,
												  _OUT u32 *pBytesReturned
												  );


//-----------------------------------------------------------------------------
// End Definitions
//-----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif	// __DCSMSNMP_H_INCLUDED__

