/*
----------------------------------------------------------------------------

	(C) Copyright 1991 Novell, Inc.	All Rights Reserved.

	This program is an unpublished copyrighted work which is proprietary
	to Novell, Inc. and contains confidential information that is not
	to be reproduced or disclosed to any other person or entity without
	prior written consent from Novell, Inc. in each and every instance.

	WARNING:  Unauthorized reproduction of this program as well as
	unauthorized preparation of derivative works based upon the
	program or distribution of copies by sale, rental, lease or
	lending are violations of federal copyright laws and state trade
	secret laws, punishable by civil and criminal penalties.

----------------------------------------------------------------------------
*/


#ifndef _SA_ASN1_H_
#define _SA_ASN1_H_

/* --------------------
 * MACRO Definitions
 * -------------------- */

/* Return Codes */
#define ASN_NO_ERROR		0		/* ASN function worked ok */
#define ASN_BAD_LENGTH	1		/* length is not in range */
#define ASN_NO_SPACE		2		/* encode buffer is too small */
#define ASN_WRONG_ID		3		/* id field doesn't match decode request */

/* ASN.1 BER Types  */
#define ASN_BOOLEAN          0x01
#define ASN_INTEGER          0x02
#define ASN_BIT_STRING       0x03
#define ASN_OCTET_STRING     0x04
#define ASN_NULL             0x05
#define ASN_OBJECT_ID        0x06
#define ASN_SEQUENCE         0x10
                             
#define ASN_UNIVERSAL        0x00
#define ASN_APPLICATION      0x40
#define ASN_CONTEXT          0x80
#define ASN_PRIVATE          0xC0
                             
#define ASN_PRIMITIVE        0x00
#define ASN_CONSTRUCTED      0x20
                             
#define ASN_IPADDRESS        (ASN_APPLICATION | 0x0)
#define ASN_COUNTER          (ASN_APPLICATION | 0x1)
#define ASN_GAUGE            (ASN_APPLICATION | 0x2)
#define ASN_TIMETICKS        (ASN_APPLICATION | 0x3)
#define ASN_OPAQUE           (ASN_APPLICATION | 0x4)

#define ASN_SNMP_MSG         (ASN_SEQUENCE | ASN_CONSTRUCTED)                             
#define ASN_GET_REQUEST      (ASN_CONTEXT | ASN_CONSTRUCTED | 0x0)
#define ASN_GETNEXT_REQUEST  (ASN_CONTEXT | ASN_CONSTRUCTED | 0x1)
#define ASN_GET_RESPONSE     (ASN_CONTEXT | ASN_CONSTRUCTED | 0x2)
#define ASN_SET_REQUEST      (ASN_CONTEXT | ASN_CONSTRUCTED | 0x3)
#define ASN_TRAP_MSG         (ASN_CONTEXT | ASN_CONSTRUCTED | 0x4)

#define ASN_SNMP_VARBIND     (ASN_SEQUENCE | ASN_CONSTRUCTED)                             
#define ASN_SNMP_VARBINDLIST (ASN_SEQUENCE | ASN_CONSTRUCTED)                             

#define ASN_LONG_LEN	        (0x80)

/* --------------------
 * Function Prototypes
 * -------------------- */
int asn_decode_header(unsigned char **p_p_buffer, unsigned char *p_id,
                      unsigned long *p_length);
int asn_encode_header(unsigned char **p_p_buffer, unsigned long *p_buff_length,
                      unsigned char id, unsigned long length);
int asn_decode_integer(unsigned char **p_p_buffer, unsigned char id,
                       unsigned long *p_long);
int asn_encode_integer(unsigned char **p_p_buffer,
                       unsigned long *p_buff_length, unsigned char id,
                       long long_int);
int asn_decode_octet_string(unsigned char **p_p_buffer, unsigned char id,
                            unsigned char *p_string,
                            unsigned long *p_string_size);
int asn_encode_octet_string(unsigned char **p_p_buffer,
                            unsigned long *p_buff_length, unsigned char id,
                            unsigned char *p_string,
                            unsigned long string_size);
int asn_decode_null(unsigned char **p_p_buffer);
int asn_encode_null(unsigned char **p_p_buffer, unsigned long *p_buff_length);
int asn_decode_object_id(unsigned char **p_p_buffer,
                         unsigned long *p_id_buffer,
                         unsigned long *p_number_ids);
int asn_encode_object_id(unsigned char **p_p_buffer,
                         unsigned long *p_buff_length,
                         unsigned long *p_id_buffer,
                         unsigned long number_ids);
int asn_skip(unsigned char **p_p_buffer);

#endif /* _SA_ASN1_H_ */


