
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for SNMP OID support functions that use secure
//    string functions
//
//  See Also:
//    <Put document references here>
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

#ifndef __SNMPOID_H_INCLUDED__
#define __SNMPOID_H_INCLUDED__


//-----------------------------------------------------------------------------
// Begin Dependent Includes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core header files
//-----------------------------------------------------------------------------
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP header files
//-----------------------------------------------------------------------------
#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// End Dependent Includes
//-----------------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// Begin Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------

// SnmpOidAlloc - allocate SNMP OID struct
SMSnmpOid * SnmpOidAlloc(void);

// SnmpOidCopy - copy SNMP OID (by allocating memory for and copying IDs)
s32 SnmpOidCopy(_IN const SMSnmpOid * pSrcOid,
				_OUT SMSnmpOid * pDstOid);

// SnmpOidAllocAndCopy - allocate and copy SNMP OID
SMSnmpOid * SnmpOidAllocAndCopy(_IN const SMSnmpOid * pSrcOid);

// SnmpOidFree - free memory for SNMP OID
void SnmpOidFree(_INOUT SMSnmpOid * pOid,
				 _IN booln freeStruct);

// SnmpOidCmp - compare two SNMP OIDs
s32 SnmpOidCmp(_IN const SMSnmpOid * pOid1,
			   _IN const SMSnmpOid * pOid2);

// SnmpOidNCmp - compare specified number of IDs of two SNMP OIDs
s32 SnmpOidNCmp(_IN const SMSnmpOid * pOid1,
				_IN const SMSnmpOid * pOid2,
				_IN u32 numIdsCmp);

// SnmpOidAStrToIds - convert SNMP OID from dotted string to binary format
s32 SnmpOidAStrToIds(_OUT u32 * ids,
					 _INOUT u32 * pNumIds,
					 _IN const astring * pAStr);

// SnmpOidToAStr - convert SNMP OID from binary to string format
s32 SnmpOidToAStr(_IN const SMSnmpOid * pOid,
				  _OUT astring * pABuf,
				  _IN u32 aBufSize);

// SnmpIdsToAStr - convert SNMP IDs from binary to string format
s32 SnmpIdsToAStr(_IN u32 * ids,
				  _IN u32 numIds,
				  _OUT astring * pABuf,
				  _IN u32 aBufSize);


//-----------------------------------------------------------------------------
// End Definitions
//-----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif	// __SNMPOID_H_INCLUDED__

