
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for mpidispatch.c support module
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

#ifndef __MPIDISPATCH_H_INCLUDED__
#define __MPIDISPATCH_H_INCLUDED__


//-----------------------------------------------------------------------------
// Begin Dependent Includes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core header files
//-----------------------------------------------------------------------------
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP header files
//-----------------------------------------------------------------------------
#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// End Dependent Includes
//-----------------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// Begin Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function defined in mpidispatch.c to dispatch requests from
// MIB Plug-in Manager
//-----------------------------------------------------------------------------

s32 MIBImplementerDispatch(_IN u32 reqType,
						   _IN const void * pInBuf, 
						   _IN u32 inBufSize,
						   _OUT void * pOutBuf, 
						   _IN u32 outBufSize,
						   _OUT u32 * pBytesReturned);


//-----------------------------------------------------------------------------
// Functions to be defined by MIB implementer if it uses mpidispatch.c
// to dispatch requests
// (these functions are called by MIBImplementerDispatch)
//-----------------------------------------------------------------------------

// MIBImplementerLoad - called to handle SM_SNMP_API_MPI_LOAD
s32 MIBImplementerLoad(void);

// MIBImplementerUnload - called to handle SM_SNMP_API_MPI_UNLOAD
s32 MIBImplementerUnload(void);

// MIBImplementerListMib - called to handle SM_SNMP_API_MPI_LIST_MIB
s32 MIBImplementerListMib(_OUT SMSnmpMibList * pMibList,
						  _IN u32 outBufSize,
						  _OUT u32 * pBytesReturned);

// MIBImplementerStartMonitor - called to handle SM_SNMP_API_MPI_START_MONITOR
s32 MIBImplementerStartMonitor(void);

// MIBImplementerStopMonitor - called to handle SM_SNMP_API_MPI_STOP_MONITOR
s32 MIBImplementerStopMonitor(void);

// MIBImplementerGet - called to handle SM_SNMP_API_MPI_GET
s32 MIBImplementerGet(_IN const SMSnmpVarBind * pIVB,
					  _OUT SMSnmpVarBind * pOVB);

// MIBImplementerGetNext - called to handle SM_SNMP_API_MPI_GET_NEXT
s32 MIBImplementerGetNext(_IN const SMSnmpVarBind * pIVB,
						  _OUT SMSnmpVarBind * pOVB);

// MIBImplementerSetTest - called to handle SM_SNMP_API_MPI_SET_TEST
s32 MIBImplementerSetTest(_IN const SMSnmpVarBind * pIVB);

// MIBImplementerSetCommit - called to handle SM_SNMP_API_MPI_SET_COMMIT
s32 MIBImplementerSetCommit(_IN const SMSnmpVarBind * pIVB);


//-----------------------------------------------------------------------------
// Functions defined in mpidispatch.c to provide interface to
// MIB Plug-in Manager callback functions
//-----------------------------------------------------------------------------

u32 MIBImpMPIMPMGetMajorVersion(void);

u32 MIBImpMPIMPMGetMinorVersion(void);

u32 MIBImpMPIMPMGetRevision(void);

void MIBImpMPIMPMFreeGeneric(_IN void * pMem);

booln MIBImpMPIMPMIsMIBManagerReady(void);

u32 MIBImpMPIMPMGetMIBEnterpriseID(void);

booln MIBImpMPIMPMIsSNMPSetsEnabled(void);

s32 MIBImpMPIMPMSendTrap(_IN SMSnmpOid * pEnterpriseOid,
						 _IN u32 genericTrapId,
						 _IN u32 specificTrapId,
						 _IN u32 numVarBinds,
						 _IN SMSnmpVarBind * pVarBinds);

s32 MIBImpMPIMPMRegisterMIBTableIndex(_IN ObjInfo * pTableEntryObjInfo,
									  _IN SMSnmpOid * pIndexOid);

s32 MIBImpMPIMPMUnregisterMIBTableIndex(_IN ObjInfo * pTableEntryObjInfo,
										_IN SMSnmpOid * pIndexOid);

booln MIBImpMPIMPMSNMPAgentIsSparseTableSupported(void);

u8 MIBImpMPIMPMSNMPAgentGetSupportedSNMPVersions(void);


//-----------------------------------------------------------------------------
// End Definitions
//-----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __MPIDISPATCH_H_INCLUDED__

