
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 6 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:14p $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for MIB Plug-in Implementer VarBind support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

#ifndef __MPIVARBIND_H_INCLUDED__
#define __MPIVARBIND_H_INCLUDED__


//-----------------------------------------------------------------------------
// Begin Dependent Includes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core header files
//-----------------------------------------------------------------------------
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP header files
//-----------------------------------------------------------------------------
#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// End Dependent Includes
//-----------------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// Begin Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------

// MPIVarBindValidateNameScalar - validate name for scalar variable and
// return pointer to AttrInfo struct for attribute if name is valid
s32 MPIVarBindValidateNameScalar(_IN const SMSnmpVarBind * pVB,
								 _IN ObjInfo * pObjInfo,
								 _OUT AttrInfo ** ppAttrInfo);

// MPIVarBindValidateNameTable1Idx - validate name for table variable with
// one index and return index and pointer to AttrInfo for attribute if name
// is valid
s32 MPIVarBindValidateNameTable1Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex);

// MPIVarBindValidateNameTable2Idx - validate name for table variable with
// two indexex and return indexex and pointer to AttrInfo for attribute if name
// is valid
s32 MPIVarBindValidateNameTable2Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex1,
									_OUT u32 * pIndex2);

// MPIVarBindValidateNameTable3Idx - validate name for table variable with
// three indexes and return indexes and pointer to AttrInfo for attribute
// if name is valid
s32 MPIVarBindValidateNameTable3Idx(_IN const SMSnmpVarBind * pVB,
									_IN ObjInfo * pObjInfo,
									_OUT AttrInfo ** ppAttrInfo,
									_OUT u32 * pIndex1,
									_OUT u32 * pIndex2,
									_OUT u32 * pIndex3);

// MPIVarBindValidateSetCommon - perform common validation for Set request
s32 MPIVarBindValidateSetCommon(_IN const SMSnmpVarBind * pVB,
								_IN AttrInfo * pAttrInfo);

// MPIVarBindGenerateNameScalar - generate name for scalar variable
// (should only be used to generate scalar names for trap variables)
s32 MPIVarBindGenerateNameScalar(_INOUT SMSnmpVarBind * pVB,
								 _IN ObjInfo * pObjInfo,
								 _IN AttrInfo * pAttrInfo);

// MPIVarBindSetValueInt32 - set 32-bit integer value into VarBind
s32 MPIVarBindSetValueInt32(_INOUT SMSnmpVarBind * pVB,
							_IN AttrInfo * pAttrInfo,
							_IN u32 val32);

// MPIVarBindSetValueOctStr - set octet string value into VarBind
s32 MPIVarBindSetValueOctStr(_INOUT SMSnmpVarBind * pVB,
							 _IN AttrInfo * pAttrInfo,
							 _IN void * pOctetString,
							 _IN u32 size,
							 _IN booln allocateBuffer);

// MPIVarBindSetValueOid - set object identifier value into VarBind
s32 MPIVarBindSetValueOid(_INOUT SMSnmpVarBind * pVB,
						  _IN AttrInfo * pAttrInfo,
						  _IN SMSnmpOid * pOid,
						  _IN booln allocateBuffer);

// MPIVarBindSetValueUCS2ToUTF8Str - convert UCS2 string to UTF8 and
// set UTF8 string value into VarBind
s32 MPIVarBindSetValueUCS2ToUTF8Str(_INOUT SMSnmpVarBind * pVB,
									_IN AttrInfo * pAttrInfo,
									_IN ustring * pUStr);

// MPIVarBindClean - clean a VarBind by freeing any memory allocated
// for the name and value buffers and by resetting the name and value info
// (should only be used to clean trap variables)
s32 MPIVarBindClean(_INOUT SMSnmpVarBind * pVB);

// MPIVarBindComputeValueChecksum - compute checksum for VarBind value
// as required for OpenManage Set security
s32 MPIVarBindComputeValueChecksum(_IN const SMSnmpVarBind * pVB,
								   _OUT u32 * pChecksum);


//-----------------------------------------------------------------------------
// End Definitions
//-----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __MPIVARBIND_H_INCLUDED__

