
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Source file for MIB Plug-in Implementer dispatch function
//    called by mpiexport.c
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core include files
//-----------------------------------------------------------------------------

#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebugsecure.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP include files
//-----------------------------------------------------------------------------

#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP MPI include files
//-----------------------------------------------------------------------------

#include <mpidispatch.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------

SysDbg4(

const astring * MIBImplementerGetReqTypeStr(_IN u32 reqType);

const astring * MIBImplementerGetSMSnmpStatusStr(_IN u32 status);

void MIBImplementerDumpVarBindName(_IN const SMSnmpVarBind * pVB);

void MIBImplementerDumpVarBindValue(_IN const SMSnmpVarBind * pVB);

void MIBImplementerSnmpOidToStr(_IN const SMSnmpOid * pOid,
								_OUT astring * pABuf,
								_IN u32 aBufSize);

) // SysDbg4


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

// pointer to MIB Plug-in Implementer to MIB Plug-in Manager
// callback function dispatch table
const SMSnmpMPIMPMDispatchTable * g_pMPIMPMDT = NULL;

// token assigned to MIB Plug-in Implementer and
// passed by MIB Plug-in Implementer on certain callback functions
const void * g_mpiToken = NULL;


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// MIBImplementerDispatch - called by MIB implementer dispatch entry point
// in mpiexport.c to handle requests from MIB Plug-in Manager
//=============================================================================
s32 MIBImplementerDispatch(_IN u32 reqType,
						   _IN const void * pInBuf,
						   _IN u32 inBufSize,
						   _OUT void * pOutBuf,
						   _IN u32 outBufSize,
						   _OUT u32 * pBytesReturned)
{
    const SMSnmpApiReq * pI = pInBuf;
    SMSnmpApiRsp * pO = pOutBuf;
	s32 status;


	SysDbgMsg4(("+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - + \n"));
    SysDbgMsg4(("MIBImplementerDispatch: entry: %s (%u)\n",
		MIBImplementerGetReqTypeStr(reqType),
		reqType));

	// default to no bytes returned
    *pBytesReturned = 0;

	// dispatch based on request type
	switch (reqType)
	{
		case SM_SNMP_API_MPI_GET:
		{
			// perform size checks
			if (inBufSize < sizeof(pI->ReqData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}
			if (outBufSize < sizeof(pO->RspData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			SysDbg4( MIBImplementerDumpVarBindName(&pI->ReqData.varBind); )

			// perform request
			status = MIBImplementerGet(&pI->ReqData.varBind,
									   &pO->RspData.varBind);

			// check request status
			if (status == SM_SNMP_STATUS_NO_ERROR)
			{
				SysDbg4( MIBImplementerDumpVarBindName(&pI->ReqData.varBind); )
				SysDbg4( MIBImplementerDumpVarBindValue(&pO->RspData.varBind); )

				// VarBind being returned
				*pBytesReturned = sizeof(pO->RspData.varBind);
			}

			break;
		} // case SM_SNMP_API_MPI_GET

		case SM_SNMP_API_MPI_GET_NEXT:
		{
			// perform size checks
			if (inBufSize < sizeof(pI->ReqData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}
			if (outBufSize < sizeof(pO->RspData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			SysDbg4( MIBImplementerDumpVarBindName(&pI->ReqData.varBind); )

			// perform request
			status = MIBImplementerGetNext(&pI->ReqData.varBind,
										   &pO->RspData.varBind);

			// check request status
			if (status == SM_SNMP_STATUS_NO_ERROR)
			{
				SysDbg4( MIBImplementerDumpVarBindName(&pO->RspData.varBind); )
				SysDbg4( MIBImplementerDumpVarBindValue(&pO->RspData.varBind); )

				// VarBind being returned
				*pBytesReturned = sizeof(pO->RspData.varBind);
			}

			break;
		} // case SM_SNMP_API_MPI_GET_NEXT

		case SM_SNMP_API_MPI_SET_TEST:
		{
			// perform size checks
			if (inBufSize < sizeof(pI->ReqData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			SysDbg4( MIBImplementerDumpVarBindName(&pI->ReqData.varBind); )
			SysDbg4( MIBImplementerDumpVarBindValue(&pI->ReqData.varBind); )

			// perform request
			status = MIBImplementerSetTest(&pI->ReqData.varBind);

			break;
		} // case SM_SNMP_API_MPI_SET_TEST

		case SM_SNMP_API_MPI_SET_COMMIT:
		{
			// perform size checks
			if (inBufSize < sizeof(pI->ReqData.varBind))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			SysDbg4( MIBImplementerDumpVarBindName(&pI->ReqData.varBind); )
			SysDbg4( MIBImplementerDumpVarBindValue(&pI->ReqData.varBind); )

			// perform request
			status = MIBImplementerSetCommit(&pI->ReqData.varBind);

			break;
		} // case SM_SNMP_API_MPI_SET_COMMIT

		case SM_SNMP_API_MPI_GET_API_MAJOR_VER:
		{
			// perform size checks
			if (outBufSize < sizeof(pO->RspData.majorVer))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			// return API major version
			pO->RspData.majorVer = SM_SNMP_API_MAJOR_VER;
			*pBytesReturned = sizeof(pO->RspData.majorVer);
			status = SM_SNMP_STATUS_NO_ERROR;

			SysDbgMsg4(("MIBImplementerDispatch: API MAJOR VER: %u\n",
				pO->RspData.majorVer));

			break;
		} // case SM_SNMP_API_MPI_GET_API_MAJOR_VER

		case SM_SNMP_API_MPI_GET_API_MINOR_VER:
		{
			// perform size checks
			if (outBufSize < sizeof(pO->RspData.minorVer))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			// return API minor version
			pO->RspData.minorVer = SM_SNMP_API_MINOR_VER;
			*pBytesReturned = sizeof(pO->RspData.minorVer);
			status = SM_SNMP_STATUS_NO_ERROR;

			SysDbgMsg4(("MIBImplementerDispatch: API MINOR VER: %u\n",
				pO->RspData.minorVer));

			break;
		} // case SM_SNMP_API_MPI_GET_API_MINOR_VER

		case SM_SNMP_API_MPI_GET_API_REVISION:
		{
			// perform size checks
			if (outBufSize < sizeof(pO->RspData.revision))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			// return API revision
			pO->RspData.revision = SM_SNMP_API_REVISION;
			*pBytesReturned = sizeof(pO->RspData.revision);
			status = SM_SNMP_STATUS_NO_ERROR;

			SysDbgMsg4(("MIBImplementerDispatch: API REVISION: %u\n",
				pO->RspData.revision));

			break;
		} // case SM_SNMP_API_MPI_GET_API_REVISION

		case SM_SNMP_API_MPI_LOAD:
		{
			// perform size checks
			if (inBufSize < sizeof(pI->ReqData.loadReq))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			// save needed data from request data
			g_pMPIMPMDT = pI->ReqData.loadReq.pMPIMPMDT;
			g_mpiToken = pI->ReqData.loadReq.mpiToken;

			// perform request
			status = MIBImplementerLoad();

			break;
		} // case SM_SNMP_API_MPI_LOAD

		case SM_SNMP_API_MPI_UNLOAD:
		{
			// perform request
			status = MIBImplementerUnload();

			// clear variables set on Load request
			g_pMPIMPMDT = NULL;
			g_mpiToken = NULL;

			break;
		} // case SM_SNMP_API_MPI_UNLOAD

		case SM_SNMP_API_MPI_LIST_MIB:
		{
			// perform size checks
			if (outBufSize < sizeof(SMSnmpMibList))
			{
				status = SM_SNMP_STATUS_GEN_ERR;
				break;
			}

			// perform request
			status = MIBImplementerListMib(&pO->RspData.mibList,
										   outBufSize,
										   pBytesReturned);

			break;
		} // case SM_SNMP_API_MPI_LIST_MIB

		case SM_SNMP_API_MPI_START_MONITOR:
		{
			// perform request
			status = MIBImplementerStartMonitor();

			break;
		} // case SM_SNMP_API_MPI_START_MONITOR

		case SM_SNMP_API_MPI_STOP_MONITOR:
		{
			// perform request
			status = MIBImplementerStopMonitor();

			break;
		} // case SM_SNMP_API_MPI_STOP_MONITOR

		default:
		{
			SysDbgMsg3(("MIBImplementerDispatch: unknown request type: %u\n",
				reqType));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
		} // default
	} // switch (reqType)

    SysDbgMsg4(("MIBImplementerDispatch: exit:  %s (%u)\n",
		MIBImplementerGetSMSnmpStatusStr(status),
		status));
	SysDbgMsg4(("- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - \n"));

	return status;
} // MIBImplementerDispatch


SysDbg4(


//=============================================================================
// MIBImplementerGetReqTypeStr - return string for request type
//=============================================================================
const astring * MIBImplementerGetReqTypeStr(_IN u32 reqType)
{

	switch (reqType)
	{
		case SM_SNMP_API_MPI_GET_API_MAJOR_VER:
			return "SM_SNMP_API_MPI_GET_API_MAJOR_VER";

		case SM_SNMP_API_MPI_GET_API_MINOR_VER:
			return "SM_SNMP_API_MPI_GET_API_MINOR_VER";

		case SM_SNMP_API_MPI_GET_API_REVISION:
			return "SM_SNMP_API_MPI_GET_API_REVISION";

		case SM_SNMP_API_MPI_LOAD:
			return "SM_SNMP_API_MPI_LOAD";

		case SM_SNMP_API_MPI_LIST_MIB:
			return "SM_SNMP_API_MPI_LIST_MIB";

		case SM_SNMP_API_MPI_UNLOAD:
			return "SM_SNMP_API_MPI_UNLOAD";

		case SM_SNMP_API_MPI_START_MONITOR:
			return "SM_SNMP_API_MPI_START_MONITOR";

		case SM_SNMP_API_MPI_STOP_MONITOR:
			return "SM_SNMP_API_MPI_STOP_MONITOR";

		case SM_SNMP_API_MPI_GET:
			return "SM_SNMP_API_MPI_GET";

		case SM_SNMP_API_MPI_GET_NEXT:
			return "SM_SNMP_API_MPI_GET_NEXT";

		case SM_SNMP_API_MPI_SET_TEST:
			return "SM_SNMP_API_MPI_SET_TEST";

		case SM_SNMP_API_MPI_SET_COMMIT:
			return "SM_SNMP_API_MPI_SET_COMMIT";

		default:
			return "UNKNOWN REQUEST TYPE";
	} // switch (reqType)
} // MIBImplementerGetReqTypeStr


//=============================================================================
// MIBImplementerGetSMSnmpStatusStr - return string for SM SNMP status value
//=============================================================================
const astring * MIBImplementerGetSMSnmpStatusStr(_IN u32 status)
{

	switch (status)
	{
		case SM_SNMP_STATUS_NO_ERROR:
			return "SM_SNMP_STATUS_NO_ERROR";

		case SM_SNMP_STATUS_TOO_BIG:
			return "SM_SNMP_STATUS_TOO_BIG";

		case SM_SNMP_STATUS_NO_SUCH_NAME:
			return "SM_SNMP_STATUS_NO_SUCH_NAME";

		case SM_SNMP_STATUS_BAD_VALUE:
			return "SM_SNMP_STATUS_BAD_VALUE";

		case SM_SNMP_STATUS_READ_ONLY:
			return "SM_SNMP_STATUS_READ_ONLY";

		case SM_SNMP_STATUS_GEN_ERR:
			return "SM_SNMP_STATUS_GEN_ERR";

		case SM_SNMP_STATUS_EX_DUPLICATE_REG:
			return "SM_SNMP_STATUS_EX_DUPLICATE_REG";

		default:
			return "UNKNOWN STATUS";
	} // switch (status)
} // MIBImplementerGetSMSnmpStatusStr


//=============================================================================
// MIBImplementerDumpVarBindName - dump contents of VarBind name using SysDbg
//=============================================================================
void MIBImplementerDumpVarBindName(_IN const SMSnmpVarBind * pVB)
{
	astring aBuf[512];


	MIBImplementerSnmpOidToStr(&pVB->name, aBuf, sizeof(aBuf));

	SysDbgMsg4(("  OID:    %s\n", aBuf));
} // MIBImplementerDumpVarBindName


//=============================================================================
// MIBImplementerDumpVarBindValue - dump contents of VarBind value using SysDbg
//=============================================================================
void MIBImplementerDumpVarBindValue(_IN const SMSnmpVarBind * pVB)
{
	astring * pTypeStr;
	SMSnmpOid oid;
	astring aBuf[512];


	// dump the value type
	switch (pVB->value.type)
	{
		case SM_SNMP_TYPE_INTEGER:
			pTypeStr = "INTEGER";
			break;
		case SM_SNMP_TYPE_OCTET_STRING:
			pTypeStr = "OCTET STRING";
			break;
		case SM_SNMP_TYPE_NULL:
			pTypeStr = "NULL";
			break;
		case SM_SNMP_TYPE_OBJ_ID:
			pTypeStr = "OBJECT IDENTIFIER";
			break;
		case SM_SNMP_TYPE_IP_ADDRESS:
			pTypeStr = "IP ADDRESS";
			break;
		case SM_SNMP_TYPE_COUNTER:
			pTypeStr = "COUNTER";
			break;
		case SM_SNMP_TYPE_GAUGE:
			pTypeStr = "GAUGE";
			break;
		case SM_SNMP_TYPE_TIMETICKS:
			pTypeStr = "TIMETICKS";
			break;
		case SM_SNMP_TYPE_OPAQUE:
			pTypeStr = "OPAQUE";
			break;
		default:
			pTypeStr = "UNKNOWN";
			break;
	} // switch (pVB->value.type)

	SysDbgMsg4(("  Type:   %s (%02Xh)\n",
		pTypeStr,
		pVB->value.type));

	// dump the value
	switch (pVB->value.type)
	{
		case SM_SNMP_TYPE_INTEGER:
		case SM_SNMP_TYPE_COUNTER:
		case SM_SNMP_TYPE_GAUGE:
		case SM_SNMP_TYPE_TIMETICKS:
			SysDbgMsg4(("  Value:  %u (%02Xh)\n",
				pVB->value.val32,
				pVB->value.val32));
			break;

		case SM_SNMP_TYPE_OCTET_STRING:
		case SM_SNMP_TYPE_IP_ADDRESS:
		case SM_SNMP_TYPE_OPAQUE:
			SysDbgMsg4(("  Length: %u\n",
				pVB->value.val32));
			if ((pVB->value.val32 != 0) && (pVB->value.valptr != NULL))
			{
				SysDbgMsg4(("  Value:  (below in hex)\n"));
				SysDbgDumpBuf4((pVB->value.valptr, pVB->value.val32));
			}
			break;

		case SM_SNMP_TYPE_OBJ_ID:
			oid.numIds = pVB->value.val32;
			oid.ids = pVB->value.valptr;
			MIBImplementerSnmpOidToStr(&oid, aBuf, sizeof(aBuf));
			SysDbgMsg4(("  NumIds: %u\n",
				pVB->value.val32));
			SysDbgMsg4(("  Value:  %s\n",
				aBuf));
			break;
	} // switch (pVB->value.type)

} // MIBImplementerDumpVarBindValue


//=============================================================================
// MIBImplementerSnmpOidToStr - convert SNMP OID to dotted string format
//=============================================================================
void MIBImplementerSnmpOidToStr(_IN const SMSnmpOid * pOid,
								_OUT astring * pABuf,
								_IN u32 aBufSize)
{
	u32 i;
	u32 len;


	if (pOid->ids != NULL)
	{
		len = 0;
		pABuf[0] = 0;
		for (i = 0; i < pOid->numIds; i++)
		{
			// convert and append the next ID
			len += sprintf_s(&pABuf[len],
				(aBufSize - len),
				"%u.",
				pOid->ids[i]);
		}
		if (len != 0)
		{
			// remove the last dot
			pABuf[len - 1] = 0;
		}
	}
	else
	{
		// somebody has a problem
		strcpy_s(pABuf, aBufSize, "<null>");
	}
} // MIBImplementerSnmpOidToStr


) // SysDbg4


//-----------------------------------------------------------------------------
// Functions defined in mpidispatch.c to provide interface to
// MIB Plug-in Manager callback functions
//-----------------------------------------------------------------------------


//=============================================================================
// MIBImpMPIMPMGetMajorVersion - get MIB Plug-in Implementer API major version
// supported by MIB Plug-in Manager
//=============================================================================
u32 MIBImpMPIMPMGetMajorVersion(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return 0;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->GetMajorVersion();
} // MIBImpMPIMPMGetMajorVersion


//=============================================================================
// MIBImpMPIMPMGetMinorVersion - get MIB Plug-in Implementer API minor version
// supported by MIB Plug-in Manager
//=============================================================================
u32 MIBImpMPIMPMGetMinorVersion(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return 0;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->GetMinorVersion();
} // MIBImpMPIMPMGetMinorVersion


//=============================================================================
// MIBImpMPIMPMGetRevision - get MIB Plug-in Implementer API revision
// supported by MIB Plug-in Manager
//=============================================================================
u32 MIBImpMPIMPMGetRevision(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return 0;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->GetRevision();
} // MIBImpMPIMPMGetRevision


//=============================================================================
// MIBImpMPIMPMFreeGeneric - free non constant memory returned by
// MIB Plug-in Manager on previous callback
//=============================================================================
void MIBImpMPIMPMFreeGeneric(_IN void * pMem)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return;
	}

	// call MIB Plug-in Manager
	g_pMPIMPMDT->FreeGeneric(pMem);
} // MIBImpMPIMPMFreeGeneric


//=============================================================================
// MIBImpMPIMPMIsMIBManagerReady - get MIB Plug-in Manager ready status
// from MIB Plug-in Manager
//=============================================================================
booln MIBImpMPIMPMIsMIBManagerReady(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return FALSE;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->IsMIBManagerReady();
} // MIBImpMPIMPMIsMIBManagerReady


//=============================================================================
// MIBImpMPIMPMGetMIBEnterpriseID - get possible alternate MIB enterprise ID
// to use
// (return value of zero means use default MIB enterprise ID)
//=============================================================================
u32 MIBImpMPIMPMGetMIBEnterpriseID(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return 0;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->GetMIBEnterpriseID();
} // MIBImpMPIMPMGetMIBEnterpriseID


//=============================================================================
// MIBImpMPIMPMIsSNMPSetsEnabled - determine if SNMP Set requests are enabled
// in MIB Plug-in Manager
//=============================================================================
booln MIBImpMPIMPMIsSNMPSetsEnabled(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return FALSE;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->IsSNMPSetsEnabled();
} // MIBImpMPIMPMIsSNMPSetsEnabled


//=============================================================================
// MIBImpMPIMPMSendTrap - call MIB Plug-in Manager to send a trap
//=============================================================================
s32 MIBImpMPIMPMSendTrap(_IN SMSnmpOid * pEnterpriseOid,
						 _IN u32 genericTrapId,
						 _IN u32 specificTrapId,
						 _IN u32 numVarBinds,
						 _IN SMSnmpVarBind * pVarBinds)
{
	SysDbg4( u32 ii; )
	SysDbg4( astring aBuf[512]; )


	// dump trap info using SysDbg
	SysDbgAssert4(
		MIBImplementerSnmpOidToStr(pEnterpriseOid, aBuf, sizeof(aBuf));
		SysDbgMsg4(("MIBImpMPIMPMSendTrap: EnterpriseOID: %s\n",
			aBuf));
		SysDbgMsg4(("MIBImpMPIMPMSendTrap: GenericTrapID: %u, SpecificTrapID: %u, NumVariables: %u\n",
			genericTrapId,
			specificTrapId,
			numVarBinds));
		for (ii = 0; ii < numVarBinds; ii++)
		{
			SysDbgMsg4(("Variable: %u\n", ii + 1));
			MIBImplementerDumpVarBindName(&pVarBinds[ii]);
			MIBImplementerDumpVarBindValue(&pVarBinds[ii]);
			SysDbgMsg4(("- - - - -\n"));
		}
	); // SysDbgAssert4

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->SendTrap(g_mpiToken,
								 pEnterpriseOid,
								 genericTrapId,
								 specificTrapId,
								 numVarBinds,
								 pVarBinds);
} // MIBImpMPIMPMSendTrap


//=============================================================================
// MIBImpMPIMPMRegisterMIBTableIndex - register index for shared MIB
// table with MIB Plug-in Manager
//=============================================================================
s32 MIBImpMPIMPMRegisterMIBTableIndex(_IN ObjInfo * pTableEntryObjInfo,
									  _IN SMSnmpOid * pIndexOid)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->RegisterMIBTableIndex(g_mpiToken,
											  pTableEntryObjInfo,
											  pIndexOid);
} // MIBImpMPIMPMRegisterMIBTableIndex


//=============================================================================
// MIBImpMPIMPMUnregisterMIBTableIndex - unregister index for shared MIB
// table with MIB Plug-in Manager
//=============================================================================
s32 MIBImpMPIMPMUnregisterMIBTableIndex(_IN ObjInfo * pTableEntryObjInfo,
										_IN SMSnmpOid * pIndexOid)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->UnregisterMIBTableIndex(g_mpiToken,
												pTableEntryObjInfo,
												pIndexOid);
} // MIBImpMPIMPMUnregisterMIBTableIndex


//=============================================================================
// MIBImpMPIMPMSNMPAgentIsSparseTableSupported - determine if SNMP agent
// supports sparse tables
//=============================================================================
booln MIBImpMPIMPMSNMPAgentIsSparseTableSupported(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return FALSE;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->SNMPAgentIsSparseTableSupported();
} // MIBImpMPIMPMSNMPAgentIsSparseTableSupported


//=============================================================================
// MIBImpMPIMPMSNMPAgentGetSupportedSNMPVersions - get SNMP versions supported
// by SNMP agent
// (return value is bitfield composed of ORed SM_SNMP_SNMPVER_* values)
//=============================================================================
u8 MIBImpMPIMPMSNMPAgentGetSupportedSNMPVersions(void)
{

	// check for dispatch table
	if (g_pMPIMPMDT == NULL)
	{
		return SM_SNMP_SNMPVER_NONE;
	}

	// call MIB Plug-in Manager
	return g_pMPIMPMDT->SNMPAgentGetSupportedSNMPVersions();
} // MIBImpMPIMPMSNMPAgentGetSupportedSNMPVersions


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

