
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 5 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:14p $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Source file for exporting MIB Plug-in Implementer dispatch entry point
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Make sure export name for dispatch function has been defined
//-----------------------------------------------------------------------------

#ifndef MPIEXPORT_DISPATCHFN
#error MPIEXPORT_DISPATCHFN not defined.
#error Example: MPIEXPORT_DISPATCHFN=SAMMPI32_SMMIBImplementerDispatch
#error NOTE: MIB implementer name should be in uppercase followed by word size.
#error Example: SAMMPI32
#endif


//-----------------------------------------------------------------------------
// Data Engine core include files
//-----------------------------------------------------------------------------

#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebug.h>
#include <dcsupapi.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP include files
//-----------------------------------------------------------------------------

#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP MPI include files
//-----------------------------------------------------------------------------

#include <mpiexport.h>


//-----------------------------------------------------------------------------
// NOTE: In modules that export functions, SMINTF must be set up
// for exporting after all other header files are included
//-----------------------------------------------------------------------------

#define __EXPORTING
#undef SMINTF
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

// original module context
SMECInfo mpiexportOrigECI;

// global code and data allocator for SysDbg
SYS_DBG_DECLARE_GLOBAL_CODEDATA;


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// ModuleAttach - called when module is loaded by OS
//=============================================================================
booln __lxinit__ ModuleAttach(void)
{

	// get original export context info
	SMGetExportContext(&mpiexportOrigECI);

	// tell MIB implementer it is being loaded by OS
	if (MIBImplementerAttach() != SM_SNMP_STATUS_NO_ERROR)
	{
		return FALSE;
	}

	return TRUE;
} // ModuleAttach


//=============================================================================
// ModuleDetach - called when module is unloaded by OS
//=============================================================================
booln __lxfini__ ModuleDetach(void)
{

	// tell MIB implementer it is being unloaded by OS
	if (MIBImplementerDetach() != SM_SNMP_STATUS_NO_ERROR)
	{
		return FALSE;
	}

	return TRUE;
} // ModuleDetach


//=============================================================================
// MODULEXX_SMMIBImplementerDispatch - exported dispatch function called by
// MIB Plug-in Manager
//=============================================================================
SMINTF s32 _cdecl MPIEXPORT_DISPATCHFN(_IN u32 reqType,
									   _IN const void *pInBuf,
									   _IN u32 inBufSize,
									   _OUT void *pOutBuf,
									   _IN u32 outBufSize,
									   _OUT u32 *pBytesReturned)
{
    SMECInfo eci;
	s32 retStatus;


	// set original export context for MIB implementer
    SMSetExportContext(&eci, &mpiexportOrigECI);

	// dispatch the request
	retStatus = MIBImplementerDispatch(reqType, 
									   pInBuf, 
									   inBufSize, 
									   pOutBuf, 
									   outBufSize, 
									   pBytesReturned);

	// restore export context
    SMResetExportContext(&eci);

    return retStatus;
} // MODULEXX_SMMIBImplementerDispatch


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

