
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    MIB Plug-in Implementer MIB List support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core include files
//-----------------------------------------------------------------------------

#include <dcossdk.h>
#include <dcdetypes.h>
#include <dcsysdebug.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP include files
//-----------------------------------------------------------------------------

#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP MPI include files
//-----------------------------------------------------------------------------

#include <mpimiblist.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// MPIMibListInit - initialize MIB list
//=============================================================================
s32 MPIMibListInit(_OUT SMSnmpMibList * pMibList,
				   _IN u32 outBufSize,
				   _OUT u32 * pBytesReturned)
{

	// initialize bytes returned to minimum size
	*pBytesReturned = sizeof(SMSnmpMibList) - sizeof(pMibList->mibList);

	// check output buffer size
	if (outBufSize < *pBytesReturned)
	{
		*pBytesReturned = 0;
		return SM_SNMP_STATUS_TOO_BIG;
	}

	// clear output buffer
	memset(pMibList, 0, outBufSize);

	// initialize count
	pMibList->mibCount = 0;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIMibListInit


//=============================================================================
// MPIMibListAdd - add MIB to MIB list
//=============================================================================
s32 MPIMibListAdd(_IN ObjInfo ** pMib,
				  _INOUT SMSnmpMibList * pMibList,
				  _IN u32 outBufSize,
				  _INOUT u32 * pBytesReturned)
{

	SysDbgMsg4(("MPIMibListAdd: pMib: %p\n", pMib));

	// update bytes returned to size needed to add MIB
	*pBytesReturned += sizeof(pMibList->mibList[0]);

	// check output buffer size
	if (outBufSize < *pBytesReturned)
	{
		*pBytesReturned -= sizeof(pMibList->mibList[0]);
		return SM_SNMP_STATUS_TOO_BIG;
	}

	// add MIB to MIB list
	pMibList->mibList[pMibList->mibCount++] = pMib;

	return SM_SNMP_STATUS_NO_ERROR;
} // MPIMibListAdd


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

