
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 4 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:14p $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Header file for Sample Shared MIB support
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------

#ifndef __SAMSHARESUPT_H_INCLUDED__
#define __SAMSHARESUPT_H_INCLUDED__


//-----------------------------------------------------------------------------
// Begin Dependent Includes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Data Engine core header files
//-----------------------------------------------------------------------------
#include <dcdetypes.h>


//-----------------------------------------------------------------------------
// Data Engine SNMP header files
//-----------------------------------------------------------------------------
#include <dcsmsnmp.h>


//-----------------------------------------------------------------------------
// End Dependent Includes
//-----------------------------------------------------------------------------


#ifdef __cplusplus
extern "C" {
#endif


//-----------------------------------------------------------------------------
// Begin Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// MIB definitions
//-----------------------------------------------------------------------------

#define SAMSHARE_MESSAGE_GROUP						(100)
#define SAMSHARE_MESSAGE_AGENT_TABLE				(10)
#define SAMSHARE_MESSAGE_TABLE						(20)

#define SAMSHARE_TABLE_ENTRY						(1)


//-----------------------------------------------------------------------------
// Message definitions
//-----------------------------------------------------------------------------

#define MESSAGE_TABLE_MIN_MESSAGE_SIZE				(1)
#define MESSAGE_TABLE_MAX_MESSAGE_SIZE				(255)
#define MESSAGE_TABLE_MAX_DATETIME_SIZE				(63)

#define MESSAGE_CONTROL_LOWERCASE					(1)
#define MESSAGE_CONTROL_UPPERCASE					(2)
#define MESSAGE_CONTROL_REVERSE						(3)
#define MESSAGE_CONTROL_DELETE						(99)

// information for entry in message table
typedef struct _MessageInfo
{
	u32 index;
	astring * pMessage;
	astring * pDateTime;
} MessageInfo;


//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------


// samsharemessage.c

s32 SamshareMessageGroupStartMonitor(void);

s32 SamshareMessageGroupStopMonitor(void);

s32 SamshareMessageAgentTableGet(_IN const SMSnmpVarBind * pIVB,
								 _OUT SMSnmpVarBind * pOVB);

s32 SamshareMessageAgentTableGetNext(_IN const SMSnmpVarBind * pIVB,
									 _OUT SMSnmpVarBind * pOVB,
									 _IN booln fellThrough);

s32 SamshareMessageAgentTableSet(_IN const SMSnmpVarBind * pIVB,
								 _IN u32 reqType);

s32 SamshareMessageTableGet(_IN const SMSnmpVarBind * pIVB,
							_OUT SMSnmpVarBind * pOVB);

s32 SamshareMessageTableGetNext(_IN const SMSnmpVarBind * pIVB,
								_OUT SMSnmpVarBind * pOVB,
								_IN booln fellThrough);

s32 SamshareMessageTableSet(_IN const SMSnmpVarBind * pIVB,
							_IN u32 reqType);

s32 SamshareMessageTableAddMessage(_IN const astring * pMessage,
								   _IN u32 messageLen);

s32 SamshareMessageTableDeleteMessage(_IN u32 index);

MessageInfo * SamshareMessageTableFindMessage(_IN u32 index);

MessageInfo * SamshareMessageTableFindNextMessage(_IN u32 index);

s32 __cdecl SamshareMessageTableCompareExact(_INOUT void * pWalkData, 
											 _IN const SMDLListEntry * pDLECur);

s32 __cdecl SamshareMessageTableCompareNext(_INOUT void * pWalkData, 
											_IN const SMDLListEntry * pDLECur);


// samsharesupt.c

s32 SamshareAttach(void);

s32 SamshareDetach(void);

s32 SamshareListMib(_INOUT SMSnmpMibList * pMibList,
					_IN u32 outBufSize,
					_OUT u32 * pBytesReturned);

s32 SamshareStartMonitor(void);

s32 SamshareStopMonitor(void);

s32 SamshareGet(_IN const SMSnmpVarBind * pIVB,
				_OUT SMSnmpVarBind * pOVB);

s32 SamshareGetNext(_IN const SMSnmpVarBind * pIVB,
					_OUT SMSnmpVarBind * pOVB);

s32 SamshareSet(_IN const SMSnmpVarBind * pIVB,
				_IN u32 reqType);

s32 SamshareCheckPrefix(_IN const SMSnmpVarBind * pVB);

s32 SamshareGetGroupID(_IN const SMSnmpVarBind * pVB,
					   _OUT u32 * pGroupId);

s32 SamshareGetTableID(_IN const SMSnmpVarBind * pVB,
					   _OUT u32 * pTableId);

s32 SamshareGetTableEntryID(_IN const SMSnmpVarBind * pVB,
							_OUT u32 * pTableEntryId);

s32 SamshareGetNextTableIdx1Int(_IN ObjInfo * pObjInfo,
								_OUT u32 * pIndex);

s32 SamshareFreeTableIdx1Int(_IN ObjInfo * pObjInfo,
							 _IN u32 index);

s32 SamshareScalarGroupGetNext(_IN const SMSnmpVarBind * pIVB,
							   _OUT SMSnmpVarBind * pOVB,
							   _IN ObjInfo * pObjInfo,
							   _IN booln fellThrough);


//-----------------------------------------------------------------------------
// End Definitions
//-----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __SAMSHARESUPT_H_INCLUDED__

