
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    MIB Implementer functions called by mpiexport.c and mpidispatch.c
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// MIBImplementerAttach - called by ModuleAttach when MIB implementer
// is loaded by OS
//=============================================================================
s32 MIBImplementerAttach(void)
{
	SysDbg3( time_t currTime; )
	SysDbg3( astring timeStr[26]; )


	// initialize debug trace subsystem
	SysDbgInit((SYSTEM_DEBUG_SCREEN | SYSTEM_DEBUG_FILE),
		DEFAULT_SYSDBG_FILENAME);

	SysDbgAssert3(
		time(&currTime);
		ctime_s(timeStr, sizeof(timeStr), &currTime);
		SysDbgMsg3(("MIBImplementerAttach: %s: %s",
			ModuleConfigGetSvcName(),
			timeStr));
	); // SysDbgAssert3

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerAttach


//=============================================================================
// MIBImplementerDetach - called by ModuleDetach when MIB implementer
// is unloaded by OS
//=============================================================================
s32 MIBImplementerDetach(void)
{
	SysDbg3( time_t currTime; )
	SysDbg3( astring timeStr[26]; )


	//-------------------------------------------------------------------------
	// Normally, cleanup is done on the SM_SNMP_API_MPI_UNLOAD request.
	// However, due to limitations on Windows NT, we will not receive
	// SM_SNMP_API_MPI_UNLOAD when running on Windows NT.  If necessary,
	// call any cleanup functions from here to make sure cleanup is done on
	// Windows NT.  For this reason, it's best to avoid doing anything other
	// than SysDbg work in MIBImplementerAttach and MIBImplementerDetach.
	//-------------------------------------------------------------------------

	SysDbgAssert3(
		time(&currTime);
		ctime_s(timeStr, sizeof(timeStr), &currTime);
		SysDbgMsg3(("MIBImplementerDetach: %s: %s",
			ModuleConfigGetSvcName(),
			timeStr));
	); // SysDbgAssert3

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerDetach


//=============================================================================
// MIBImplementerLoad - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_LOAD
//=============================================================================
s32 MIBImplementerLoad(void)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerLoad: %s\n",
		ModuleConfigGetSvcName()));

	// trace certain MIB Plug-in Manager callback functions for test purposes
    SysDbgMsg4(("MIBImplementerLoad: MIBImpMPIMPMIsMIBManagerReady: %u\n",
		MIBImpMPIMPMIsMIBManagerReady()));

	// attach Sample MIB 1
	status = Sammib1Attach();
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// attach Sample Shared MIB
	status = SamshareAttach();
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		// detach Sample MIB 1
		Sammib1Detach();

		return status;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerLoad


//=============================================================================
// MIBImplementerUnload - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_UNLOAD
//=============================================================================
s32 MIBImplementerUnload(void)
{

    SysDbgMsg4(("MIBImplementerUnload: %s\n",
		ModuleConfigGetSvcName()));

	// trace certain MIB Plug-in Manager callback functions for test purposes
    SysDbgMsg4(("MIBImplementerUnload: MIBImpMPIMPMIsMIBManagerReady: %u\n",
		MIBImpMPIMPMIsMIBManagerReady()));

	// detach Sample MIB 1
	Sammib1Detach();

	// detach Sample Shared MIB
	SamshareDetach();

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerUnload


//=============================================================================
// MIBImplementerListMib - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_LIST_MIB
//=============================================================================
s32 MIBImplementerListMib(_OUT SMSnmpMibList * pMibList,
						  _IN u32 outBufSize,
						  _OUT u32 * pBytesReturned)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerListMib: %s\n",
		ModuleConfigGetSvcName()));

	// initialize MIB list
	status = MPIMibListInit(pMibList, outBufSize, pBytesReturned);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// add Sample MIB 1
	status = Sammib1ListMib(pMibList, outBufSize, pBytesReturned);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// add Sample Shared MIB
	status = SamshareListMib(pMibList, outBufSize, pBytesReturned);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerListMib


//=============================================================================
// MIBImplementerStartMonitor - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_START_MONITOR
//=============================================================================
s32 MIBImplementerStartMonitor(void)
{

    SysDbgMsg4(("MIBImplementerStartMonitor: %s\n",
		ModuleConfigGetSvcName()));

	// trace certain MIB Plug-in Manager callback functions for test purposes
    SysDbgMsg4(("MIBImplementerStartMonitor: MIBImpMPIMPMIsMIBManagerReady: %u\n",
		MIBImpMPIMPMIsMIBManagerReady()));

	// start monitor for Sample MIB 1
	Sammib1StartMonitor();

	// start monitor for Sample Shared MIB
	SamshareStartMonitor();

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerStartMonitor


//=============================================================================
// MIBImplementerStopMonitor - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_STOP_MONITOR
//=============================================================================
s32 MIBImplementerStopMonitor(void)
{

    SysDbgMsg4(("MIBImplementerStopMonitor: %s\n",
		ModuleConfigGetSvcName()));

	// trace certain MIB Plug-in Manager callback functions for test purposes
    SysDbgMsg4(("MIBImplementerStopMonitor: MIBImpMPIMPMIsMIBManagerReady: %u\n",
		MIBImpMPIMPMIsMIBManagerReady()));

	// stop monitor for Sample MIB 1
	Sammib1StopMonitor();

	// stop monitor for Sample Shared MIB
	SamshareStopMonitor();

	return SM_SNMP_STATUS_NO_ERROR;
} // MIBImplementerStopMonitor


//=============================================================================
// MIBImplementerGet - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_GET
//=============================================================================
s32 MIBImplementerGet(_IN const SMSnmpVarBind * pIVB,
					  _OUT SMSnmpVarBind * pOVB)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerGet: entry: %s\n",
		ModuleConfigGetSvcName()));

	// determine which MIB should handle the request
	if (Sammib1CheckPrefix(pIVB) == 0)
	{
		// Sample MIB 1 should handle the request
		status = Sammib1Get(pIVB, pOVB);
	}
	else if (SamshareCheckPrefix(pIVB) == 0)
	{
		// Sample Shared MIB should handle the request
		status = SamshareGet(pIVB, pOVB);
	}
	else
	{
		// target variable is not part of any supported MIB
		status = SM_SNMP_STATUS_NO_SUCH_NAME;
	}

    SysDbgMsg4(("MIBImplementerGet: exit: %s\n",
		ModuleConfigGetSvcName()));

	return status;
} // MIBImplementerGet


//=============================================================================
// MIBImplementerGetNext - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_GET_NEXT
//=============================================================================
s32 MIBImplementerGetNext(_IN const SMSnmpVarBind * pIVB,
						  _OUT SMSnmpVarBind * pOVB)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerGetNext: entry: %s\n",
		ModuleConfigGetSvcName()));

	// determine which MIB should handle the request
	if (Sammib1CheckPrefix(pIVB) <= 0)
	{
		// Sample MIB 1 should handle the request
		status = Sammib1GetNext(pIVB, pOVB);
	}
	else if (SamshareCheckPrefix(pIVB) <= 0)
	{
		// Sample Shared MIB should handle the request
		status = SamshareGetNext(pIVB, pOVB);
	}
	else
	{
		// target variable comes after all supported MIBs
		status = SM_SNMP_STATUS_NO_SUCH_NAME;
	}

    SysDbgMsg4(("MIBImplementerGetNext: exit: %s\n",
		ModuleConfigGetSvcName()));

	return status;
} // MIBImplementerGetNext


//=============================================================================
// MIBImplementerSetTest - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_SET_TEST
//=============================================================================
s32 MIBImplementerSetTest(_IN const SMSnmpVarBind * pIVB)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerSetTest: entry: %s\n",
		ModuleConfigGetSvcName()));

	// determine which MIB should handle the request
	if (Sammib1CheckPrefix(pIVB) == 0)
	{
		// Sample MIB 1 should handle the request
		status = Sammib1Set(pIVB, SM_SNMP_API_MPI_SET_TEST);
	}
	else if (SamshareCheckPrefix(pIVB) == 0)
	{
		// Sample Shared MIB should handle the request
		status = SamshareSet(pIVB, SM_SNMP_API_MPI_SET_TEST);
	}
	else
	{
		// target variable is not part of any supported MIB
		status = SM_SNMP_STATUS_NO_SUCH_NAME;
	}

    SysDbgMsg4(("MIBImplementerSetTest: exit: %s\n",
		ModuleConfigGetSvcName()));

	return status;
} // MIBImplementerSetTest


//=============================================================================
// MIBImplementerSetCommit - called by MIBImplementerDispatch to handle
// SM_SNMP_API_MPI_SET_COMMIT
//=============================================================================
s32 MIBImplementerSetCommit(_IN const SMSnmpVarBind * pIVB)
{
	s32 status;


    SysDbgMsg4(("MIBImplementerSetCommit: entry: %s\n",
		ModuleConfigGetSvcName()));

	// determine which MIB should handle the request
	if (Sammib1CheckPrefix(pIVB) == 0)
	{
		// Sample MIB 1 should handle the request
		status = Sammib1Set(pIVB, SM_SNMP_API_MPI_SET_COMMIT);
	}
	else if (SamshareCheckPrefix(pIVB) == 0)
	{
		// Sample Shared MIB should handle the request
		status = SamshareSet(pIVB, SM_SNMP_API_MPI_SET_COMMIT);
	}
	else
	{
		// target variable is not part of any supported MIB
		status = SM_SNMP_STATUS_NO_SUCH_NAME;
	}

    SysDbgMsg4(("MIBImplementerSetCommit: exit: %s\n",
		ModuleConfigGetSvcName()));

	return status;
} // MIBImplementerSetCommit


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

