
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Functions to implement Sample MIB 1 Data Type Group
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

u32 g_dataTypeInteger = 0;

astring g_dataTypeOctetStringBuf[SAMMIB1_DATATYPE_OCTET_STRING_MAX_SIZE + 1] =
	{0};

u32 g_dataTypeOIDIds[SM_SNMP_OID_MAX_NUMIDS] = {0, 0};
SMSnmpOid g_dataTypeOID = SM_SNMP_OID_INITIALIZER_NUMIDS(g_dataTypeOIDIds, 2);

u32 g_dataTypeIpAddr = 0;

u32 g_dataTypeCounter = 0;

u32 g_dataTypeGauge = 0;

u32 g_dataTypeTimeTicks = 0;


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// Sammib1DataTypeGroupGet - process Get requests for Data Type Group
//=============================================================================
s32 Sammib1DataTypeGroupGet(_IN const SMSnmpVarBind * pIVB,
							_OUT SMSnmpVarBind * pOVB)
{
	AttrInfo * pAttrInfo;
	s32 status;


    SysDbgMsg4(("Sammib1DataTypeGroupGet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameScalar(pIVB,
										  &dataTypeGroup_ObjInfo,
										  &pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// get value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case dataTypeInteger_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 g_dataTypeInteger);
			break;

		case dataTypeOctetString_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  g_dataTypeOctetStringBuf,
											  (u32)strlen(g_dataTypeOctetStringBuf),
											  FALSE);
			break;

		case dataTypeObjectIdentifier_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueOid(pOVB,
										   pAttrInfo,
										   &g_dataTypeOID,
										   FALSE);
			break;

		case dataTypeIpAddress_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  &g_dataTypeIpAddr,
											  sizeof(g_dataTypeIpAddr),
											  FALSE);
			break;

		case dataTypeCounter_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 g_dataTypeCounter);
			break;

		case dataTypeGauge_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 g_dataTypeGauge);
			break;

		case dataTypeTimeTicks_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 g_dataTypeTimeTicks);
			break;

		default:
			SysDbgMsg3(("Sammib1DataTypeGroupGet: attribute %u: Get not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // Sammib1DataTypeGroupGet


//=============================================================================
// Sammib1DataTypeGroupSet - process Set requests for Data Type Group
//=============================================================================
s32 Sammib1DataTypeGroupSet(_IN const SMSnmpVarBind * pIVB,
							_IN u32 reqType)
{
	AttrInfo * pAttrInfo;
	s32 status;


    SysDbgMsg4(("Sammib1DataTypeGroupSet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameScalar(pIVB,
										  &dataTypeGroup_ObjInfo,
										  &pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// perform common Set validation for target attribute
	status = MPIVarBindValidateSetCommon(pIVB, pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// set value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case dataTypeInteger_ATTRIBUTE:
			// no more validation needed for this attribute

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			g_dataTypeInteger = pIVB->value.val32;
			break;

		case dataTypeOctetString_ATTRIBUTE:
			// check length of new value
			if (pIVB->value.val32 > SAMMIB1_DATATYPE_OCTET_STRING_MAX_SIZE)
			{
				SysDbgMsg3(("Sammib1DataTypeGroupSet: attribute %u: new value too big\n",
					pAttrInfo->aib_id));
				status = SM_SNMP_STATUS_BAD_VALUE;
				break;
			}

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			memcpy(g_dataTypeOctetStringBuf,
				   pIVB->value.valptr,
				   pIVB->value.val32);

			// make it null terminated
			g_dataTypeOctetStringBuf[pIVB->value.val32] = 0;
			break;

		case dataTypeObjectIdentifier_ATTRIBUTE:
			// check number of IDs in new value
			// (val32 = number of IDs in new value)
			if (pIVB->value.val32 > SM_SNMP_OID_MAX_NUMIDS)
			{
				SysDbgMsg3(("Sammib1DataTypeGroupSet: attribute %u: new value too big\n",
					pAttrInfo->aib_id));
				status = SM_SNMP_STATUS_BAD_VALUE;
				break;
			}

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			memcpy(g_dataTypeOID.ids,
				   pIVB->value.valptr,
				   SM_SNMP_OID_NUMIDS2SIZE(pIVB->value.val32));
			g_dataTypeOID.numIds = pIVB->value.val32;
			break;

		case dataTypeIpAddress_ATTRIBUTE:
			// check length of new value
			if (pIVB->value.val32 != sizeof(g_dataTypeIpAddr))
			{
				SysDbgMsg3(("Sammib1DataTypeGroupSet: attribute %u: invalid value size\n",
					pAttrInfo->aib_id));
				status = SM_SNMP_STATUS_BAD_VALUE;
				break;
			}

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			memcpy(&g_dataTypeIpAddr, pIVB->value.valptr, pIVB->value.val32);
			break;

		case dataTypeCounter_ATTRIBUTE:
			// no more validation needed for this attribute

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			g_dataTypeCounter = pIVB->value.val32;
			break;

		case dataTypeGauge_ATTRIBUTE:
			// no more validation needed for this attribute

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			g_dataTypeGauge = pIVB->value.val32;
			break;

		case dataTypeTimeTicks_ATTRIBUTE:
			// no more validation needed for this attribute

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit the value
			g_dataTypeTimeTicks = pIVB->value.val32;
			break;

		default:
			SysDbgMsg3(("Sammib1DataTypeGroupSet: attribute %u: Set not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	// if value successfully set, send data type trap
	if ((reqType == SM_SNMP_API_MPI_SET_COMMIT) && 
		(status == SM_SNMP_STATUS_NO_ERROR))
	{
		Sammib1DataTypeSendTrap(pAttrInfo->aib_id);
	}

	return status;
} // Sammib1DataTypeGroupSet


//=============================================================================
// Sammib1DataTypeSendTrap - send data type trap
//=============================================================================
s32 Sammib1DataTypeSendTrap(u32 attrId)
{
	SMSnmpVarBind * pVarBinds;
	SMSnmpVarBind * pVB;
	AttrInfo * pAttrInfo;
	s32 status;
	u32 ii;
	u32 size;


    SysDbgMsg4(("Sammib1DataTypeSendTrap: entry\n"));

	//-------------------------------------------------------------------------
	// allocate memory for trap variables
	//-------------------------------------------------------------------------
	size = SAMMIB1_DATATYPE_TRAP_NUM_VARS * sizeof(SMSnmpVarBind);
	if ((pVarBinds = SMAllocMem(size)) == NULL)
	{
		SysDbgMsg3(("Sammib1DataTypeSendTrap: ERROR: no memory for VarBinds\n"));
		status = SM_SNMP_STATUS_GEN_ERR;
		_throw exit1;
	}
	memset(pVarBinds, 0, size);

	//-------------------------------------------------------------------------
	// set up trap variable 1 (Integer)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[0];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeInteger_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueInt32(pVB,
									 pAttrInfo,
									 g_dataTypeInteger);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 2 (OctetString)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[1];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeOctetString_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueOctStr(pVB,
									  pAttrInfo,
									  g_dataTypeOctetStringBuf,
									  (u32)strlen(g_dataTypeOctetStringBuf),
									  TRUE);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 3 (ObjectIdentifier)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[2];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeObjectIdentifier_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueOid(pVB,
								   pAttrInfo,
								   &g_dataTypeOID,
								   TRUE);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 4 (IpAddress)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[3];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeIpAddress_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueOctStr(pVB,
									  pAttrInfo,
									  &g_dataTypeIpAddr,
									  sizeof(g_dataTypeIpAddr),
									  TRUE);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 5 (Counter)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[4];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeCounter_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueInt32(pVB,
									 pAttrInfo,
									 g_dataTypeCounter);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 6 (Gauge)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[5];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeGauge_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueInt32(pVB,
									 pAttrInfo,
									 g_dataTypeGauge);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// set up trap variable 7 (TimeTicks)
	//-------------------------------------------------------------------------
	pVB = &pVarBinds[6];
	pAttrInfo = &dataTypeGroup_AttrInfo[dataTypeTimeTicks_ATTRIBUTE];

	// trap variable name
	status = MPIVarBindGenerateNameScalar(pVB,
										  &dataTypeGroup_ObjInfo,
										  pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	// trap variable value
	status = MPIVarBindSetValueInt32(pVB,
									 pAttrInfo,
									 g_dataTypeTimeTicks);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw exit2;
	}

	//-------------------------------------------------------------------------
	// call MIB Plug-in Manager to send the trap
	// (MIBImpMPIMPMSendTrap is defined in mpidispatch.c)
	//-------------------------------------------------------------------------
	status = MIBImpMPIMPMSendTrap(&g_sammib1Oid,
								  SM_SNMP_GENTRAP_ENTERPRISE_SPECIFIC,
								  SAMMIB1_DATATYPE_TRAP_BASE_ID + attrId,
								  SAMMIB1_DATATYPE_TRAP_NUM_VARS,
								  pVarBinds);

exit2:
	// clean trap variables (to free any dynamically allocated memory)
	for (ii = 0; ii < SAMMIB1_DATATYPE_TRAP_NUM_VARS; ii++)
	{
		MPIVarBindClean(&pVarBinds[ii]);
	}
	// free memory for trap variables
	SMFreeMem(pVarBinds);
exit1:
	return status;
} // Sammib1DataTypeSendTrap


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

