
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2003 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: 4 $
//
//  Last Modified By / On:
//    $Author: Tushar_oza $ / $Date: 7/22/03 5:14p $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Functions to implement Sample MIB 1 MIB Implementer Group
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//============================================================================
// Sammib1MIBImplementerGroupGet - process Get requests for MIB Implementer
// Group
//=============================================================================
s32 Sammib1MIBImplementerGroupGet(_IN const SMSnmpVarBind * pIVB,
								  _OUT SMSnmpVarBind * pOVB)
{
	AttrInfo * pAttrInfo;
	astring * pAStr;
	s32 status;


    SysDbgMsg4(("Sammib1MIBImplementerGroupGet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameScalar(pIVB,
										  &mibImplementerGroup_ObjInfo,
										  &pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// get value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case mibImplementerDescriptionName_ATTRIBUTE:
			// get value
			pAStr = (astring *)ModuleConfigGetDescriptionStr();

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		case mibImplementerVersionName_ATTRIBUTE:
			// get value
			pAStr = (astring *)ModuleConfigGetVersionStr();

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		case mibImplementerBuildDateTimeName_ATTRIBUTE:
			// get value
			pAStr = (astring *)ModuleConfigGetBuildDateTimeStr();

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		case mibImplementerOSTypeName_ATTRIBUTE:
			// get value
			pAStr = (astring *)ModuleConfigGetOSTypeStr();

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		default:
			SysDbgMsg3(("Sammib1MIBImplementerGroupGet: attribute %u: Get not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // Sammib1MIBImplementerGroupGet


//=============================================================================
// Sammib1MIBImplementerGroupSet - process Set requests for MIB Implementer
// Group
//=============================================================================
s32 Sammib1MIBImplementerGroupSet(_IN const SMSnmpVarBind * pIVB,
						_IN u32 reqType)
{
	AttrInfo * pAttrInfo;
	s32 status;


	UNREFERENCED_PARAMETER(reqType);

    SysDbgMsg4(("Sammib1MIBImplementerGroupSet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameScalar(pIVB,
										  &dataTypeGroup_ObjInfo,
										  &pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// perform common Set validation for target attribute
	status = MPIVarBindValidateSetCommon(pIVB, pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// set value of target attribute
	switch (pAttrInfo->aib_id)
	{
		// currently no read-write attributes

		default:
			SysDbgMsg3(("Sammib1MIBImplementerGroupSet: attribute %u: Set not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // Sammib1MIBImplementerGroupSet


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

