
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Sample MIB 1 support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

// Sample MIB 1 SNMP OID used for prefix check and traps
u32 g_sammib1Ids[] = { 1, 3, 6, 1, 4, 1, 674, 55555, 1 };
SMSnmpOid g_sammib1Oid = SM_SNMP_OID_INITIALIZER(g_sammib1Ids);


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// Sammib1Attach - attach Sample MIB 1
//=============================================================================
s32 Sammib1Attach(void)
{
	ObjInfo ** pMIB;
	ObjInfo * pObjInfo;
	u32 enterpriseID;


    SysDbgMsg4(("Sammib1Attach: entry\n"));

	// check for possible alternate MIB enterprise ID
	enterpriseID = ModuleConfigGetEnterpriseID();

	// was alternate MIB enterprise ID specified?
	if (enterpriseID != 0)
	{
		// yes, change MIB enterprise ID used for Sample MIB 1
		SysDbgMsg4(("Sammib1Attach: changing MIB enterprise ID to: %u\n",
			enterpriseID));

		// the base SNMP OID is being changed from:
		//   1.3.6.1.4.1.674.55555.1
		// to:
		//   1.3.6.1.4.1.<enterpriseID>.55555.1

		// update SNMP OID used for prefix check and traps
		g_sammib1Ids[SM_SNMP_OID_ENTERPRISE_ID_INDEX] = enterpriseID;

		// update SNMP OIDs generated by Novell MIB compiler
		for (pMIB = SAMMIB1_MIB; *pMIB != NULL; pMIB++)
		{
			pObjInfo = *pMIB;
			pObjInfo->oib_id_pt[SM_SNMP_OID_ENTERPRISE_ID_INDEX] = enterpriseID;
		}
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1Attach


//=============================================================================
// Sammib1Detach - detach Sample MIB 1
//=============================================================================
s32 Sammib1Detach(void)
{

    SysDbgMsg4(("Sammib1Detach: entry\n"));

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1Attach


//=============================================================================
// Sammib1ListMib - list MIBs for Sample MIB 1
//=============================================================================
s32 Sammib1ListMib(_INOUT SMSnmpMibList * pMibList,
				   _IN u32 outBufSize,
				   _OUT u32 * pBytesReturned)
{
	s32 status;


    SysDbgMsg4(("Sammib1ListMib: entry\n"));

	// add Sample MIB 1
	// (SAMMIB1_MIB is defined in sammib1.c which is generated
	//  by Novell MIB compiler)
	status = MPIMibListAdd(SAMMIB1_MIB, pMibList, outBufSize, pBytesReturned);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1ListMib


//=============================================================================
// Sammib1StartMonitor - start monitor for Sample MIB 1
//=============================================================================
s32 Sammib1StartMonitor(void)
{

    SysDbgMsg4(("Sammib1StartMonitor: entry\n"));

	// call MIB Plug-in Manager to send ready trap
	// (MIBImpMPIMPMSendTrap is defined in mpidispatch.c)
	MIBImpMPIMPMSendTrap(&g_sammib1Oid,
						 SM_SNMP_GENTRAP_ENTERPRISE_SPECIFIC,
						 SAMMIB1_READY_TRAP_ID,
						 0,
						 NULL);

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1StartMonitor


//=============================================================================
// Sammib1StopMonitor - stop monitor for Sample MIB 1
//=============================================================================
s32 Sammib1StopMonitor(void)
{

    SysDbgMsg4(("Sammib1StopMonitor: entry\n"));

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1StopMonitor


//=============================================================================
// Sammib1Get - process Get requests for Sample MIB 1
//=============================================================================
s32 Sammib1Get(_IN const SMSnmpVarBind * pIVB,
			   _OUT SMSnmpVarBind * pOVB)
{
	s32 status;
	u32 groupId;


    SysDbgMsg4(("Sammib1Get: entry\n"));

	// get MIB group ID
	status = Sammib1GetGroupID(pIVB, &groupId);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		// not enough IDs for MIB group ID;
		// we can't proceed without the MIB group ID
		return status;
	}

	// process request based on group ID
	switch (groupId)
	{
		case SAMMIB1_MIBIMPLEMENTER_GROUP:
			status = Sammib1MIBImplementerGroupGet(pIVB, pOVB);
			break;

		case SAMMIB1_DATATYPE_GROUP:
			status = Sammib1DataTypeGroupGet(pIVB, pOVB);
			break;

		default:
			SysDbgMsg3(("Sammib1Get: ERROR: invalid group ID: %u\n",
				groupId));
			status = SM_SNMP_STATUS_NO_SUCH_NAME;
			break;
	} // switch (groupId)

	return status;
} // Sammib1Get


//=============================================================================
// Sammib1GetNext - process GetNext requests for Sample MIB 1
//=============================================================================
s32 Sammib1GetNext(_IN const SMSnmpVarBind * pIVB,
				   _OUT SMSnmpVarBind * pOVB)
{
	s32 prefixStatus;
	u32 groupID;
	booln fellThrough = FALSE;


    SysDbgMsg4(("Sammib1GetNext: entry\n"));

	// check input variable OID
	prefixStatus = Sammib1CheckPrefix(pIVB);
	if (prefixStatus == 0)
	{
		// input variable OID is in our MIB subtree;
		// get MIB group ID
		if (Sammib1GetGroupID(pIVB, &groupID) != SM_SNMP_STATUS_NO_ERROR)
		{
			// not enough IDs for MIB group ID;
			// return first attribute instance in our MIB
			groupID = 0;
		}
	}
	else if (prefixStatus < 0)
	{
		// input variable OID comes before our MIB subtree;
		// return first attribute instance in our MIB
		SysDbgMsg3(("Sammib1GetNext: before our MIB subtree\n"));
		groupID = 0;
	}
	else
	{
		// input variable OID comes after our MIB subtree
		// (we should never make it here)
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// if valid group ID for non-indexed group, do nothing here;
	// if valid group ID for indexed group, get Table ID;
	// if invalid group ID, map to valid group ID
	switch (groupID)
	{
		case SAMMIB1_MIBIMPLEMENTER_GROUP:
			// valid group ID for non-indexed group;
			// no mapping needed
			break;

		case SAMMIB1_DATATYPE_GROUP:
			// valid group ID for non-indexed group;
			// no mapping needed
			break;

		// currently no indexed groups;
		// if one is added, we should try to get table ID here

		default:
			// invalid group ID;
			// map it to the next group ID numerically
			fellThrough = TRUE;
			if (groupID < SAMMIB1_MIBIMPLEMENTER_GROUP)
			{
				groupID = SAMMIB1_MIBIMPLEMENTER_GROUP;
			}
			else if (groupID < SAMMIB1_DATATYPE_GROUP)
			{
				groupID = SAMMIB1_DATATYPE_GROUP;
			}
			else
			{
				// group ID is greater than our last supported group ID;
				// it's past end of our MIB subtree
				SysDbgMsg4(("Sammib1GetNext: group ID after last supported group ID\n"));
				return SM_SNMP_STATUS_NO_SUCH_NAME;
			}
			SysDbgMsg4(("Sammib1GetNext: mapped group ID: %u\n", groupID));
			break;
	} // switch (groupID)

	// process GetNext command based on group ID
	switch (groupID)
	{
		case SAMMIB1_MIBIMPLEMENTER_GROUP:
		{
			return Sammib1ScalarGroupGetNext(
							pIVB,
							pOVB,
							&mibImplementerGroup_ObjInfo,
							fellThrough,
							mibImplementerDescriptionName_ATTRIBUTE,
							mibImplementerGroup_ATTRIB_MAX);
		}

		case SAMMIB1_DATATYPE_GROUP:
		{
			return Sammib1ScalarGroupGetNext(
							pIVB,
							pOVB,
							&dataTypeGroup_ObjInfo,
							fellThrough,
							dataTypeInteger_ATTRIBUTE,
							dataTypeGroup_ATTRIB_MAX);
		}

		default:
		{
			break;
		} //  default
	} // switch (groupID)

	// end of Sample MIB 1
	SysDbgMsg4(("Sammib1GetNext: end of MIB\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // Sammib1GetNext


//=============================================================================
// Sammib1Set - process Set requests for Sample MIB 1
//=============================================================================
s32 Sammib1Set(_IN const SMSnmpVarBind * pIVB,
			   _IN u32 reqType)
{
	s32 status;
	u32 groupID;


    SysDbgMsg4(("Sammib1Set: entry\n"));

	// get MIB group ID
	status = Sammib1GetGroupID(pIVB, &groupID);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		// not enough IDs for MIB group ID;
		// we can't proceed without the MIB group ID
		return status;
	}

	// process request based on group ID
	switch (groupID)
	{
		case SAMMIB1_MIBIMPLEMENTER_GROUP:
			status = Sammib1MIBImplementerGroupSet(pIVB, reqType);
			break;

		case SAMMIB1_DATATYPE_GROUP:
			status = Sammib1DataTypeGroupSet(pIVB, reqType);
			break;

		default:
			SysDbgMsg3(("Sammib1Set: ERROR: invalid group ID: %u\n",
				groupID));
			status = SM_SNMP_STATUS_NO_SUCH_NAME;
			break;
	} // switch (groupID)

	return status;
} // Sammib1Set


//=============================================================================
// Sammib1CheckPrefix - determine where the specified variable falls in the
// MIB tree relative to the Sample MIB 1 MIB subtree
//
// returns <0 if variable precedes Sample MIB 1
//          0 if variable is in    Sample MIB 1
//         >0 if variable follows  Sample MIB 1
//=============================================================================
s32 Sammib1CheckPrefix(_IN const SMSnmpVarBind * pVB)
{
	u32 numIdsToCompare;


	// number of IDs to compare is number of IDs in shorter of the two OIDs
	if (pVB->name.numIds < g_sammib1Oid.numIds)
	{
		numIdsToCompare = pVB->name.numIds;
	}
	else
	{
		numIdsToCompare = g_sammib1Oid.numIds;
	}

	// compare the variable OID to the MIB subtree OID
	return SnmpOidNCmp(&pVB->name, &g_sammib1Oid, numIdsToCompare);
} // Sammib1CheckPrefix


//=============================================================================
// Sammib1GetGroupID - get group ID from variable name
//=============================================================================
s32 Sammib1GetGroupID(_IN const SMSnmpVarBind * pVB,
					  _OUT u32 * pGroupId)
{

	// Sammib1CheckPrefix must be called before calling this function;
	// variable OID must be at least one ID longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_sammib1Oid.numIds + 1))
	{
		// variable OID is too short to contain the group ID
		SysDbgMsg4(("Sammib1GetGroupID: not enough IDs for group ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the first ID after MIB subtree base OID
	*pGroupId = pVB->name.ids[g_sammib1Oid.numIds];

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1GetGroupID


//=============================================================================
// Sammib1GetTableID - get table ID from variable name
//=============================================================================
s32 Sammib1GetTableID(_IN const SMSnmpVarBind * pVB,
					  _OUT u32 * pTableId)
{

	// Sammib1CheckPrefix must be called before calling this function;
	// variable OID must be at least two IDs longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_sammib1Oid.numIds + 2))
	{
		// variable OID is too short to contain the table ID
		SysDbgMsg4(("Sammib1GetTableID: not enough IDs for table ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the second ID after MIB subtree base OID
	*pTableId = pVB->name.ids[g_sammib1Oid.numIds + 1];

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1GetTableID


//=============================================================================
// Sammib1GetTableEntryID - get table entry ID from variable name
//=============================================================================
s32 Sammib1GetTableEntryID(_IN const SMSnmpVarBind * pVB,
						   _OUT u32 * pTableEntryId)
{

	// Sammib1CheckPrefix must be called before calling this function;
	// variable OID must be at least three IDs longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_sammib1Oid.numIds + 3))
	{
		// variable OID is too short to contain the table entry ID
		SysDbgMsg4(("Sammib1VerifyTableEntryID: not enough IDs for table entry ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the third ID after MIB subtree base OID
	*pTableEntryId = pVB->name.ids[g_sammib1Oid.numIds + 2];

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1GetTableEntryID


//=============================================================================
// Sammib1GetNextTableIdx1Int - get next available table index for table
// with 1 integer index
//=============================================================================
s32 Sammib1GetNextTableIdx1Int(_IN ObjInfo * pObjInfo,
							   _OUT u32 * pIndex)
{
	SMSnmpOid indexOid;
	s32 status;
	u32 index;


	// initialize index OID
	indexOid.numIds = 1;
	indexOid.ids = &index;

	// find available table index; start at 1 and keep incrementing index
	// until an index is successfully registered
	index = 1;
	while (TRUE)
	{
		// register index with MIB Plug-in Manager
		status = MIBImpMPIMPMRegisterMIBTableIndex(pObjInfo, &indexOid);
		if (status == SM_SNMP_STATUS_NO_ERROR)
		{
			// table index was successfully registered
			break;
		}
		else if (status != SM_SNMP_STATUS_EX_DUPLICATE_REG)
		{
			// fatal error
			SysDbgMsg3(("Sammib1GetNextTableIdx1Int: MIBImpMPIMPMRegisterMIBTableIndex failed: %d\n",
				status));
			return status;
		}

		// index in use; try another index
		index++;
	}

	// return index
	*pIndex = index;

	return SM_SNMP_STATUS_NO_ERROR;
} // Sammib1GetNextTableIdx1Int


//=============================================================================
// Sammib1FreeTableIdx1Int - free table index for table with 1 integer index
//=============================================================================
s32 Sammib1FreeTableIdx1Int(_IN ObjInfo * pObjInfo,
							_IN u32 index)
{
	SMSnmpOid indexOid;
	s32 status;


	// initialize index OID
	indexOid.numIds = 1;
	indexOid.ids = &index;

	// unregister table index with MIB Plug-in Manager
	status = MIBImpMPIMPMUnregisterMIBTableIndex(pObjInfo, &indexOid);

	SysDbg3(
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		SysDbgMsg3(("Sammib1FreeTableIdx1Int: MIBImpMPIMPMUnregisterTableIndex failed: %d\n",
			status));
	}
	) // SysDbg3

	return status;
} // Sammib1GetNextTableIdx1Int


//=============================================================================
// Sammib1ScalarGroupGetNext - process GetNext requests for scalar groups
//=============================================================================
s32 Sammib1ScalarGroupGetNext(_IN const SMSnmpVarBind * pIVB,
							  _OUT SMSnmpVarBind * pOVB,
							  _IN ObjInfo * pObjInfo,
							  _IN booln fellThrough,
							  _IN u32 startAttributeID,
							  _IN u32 maxAttributeID)
{
	s32 status;
	u32 attributeID;
	u32 tempID;
	SysDbg4( astring aBuf[512]; )


	SysDbgMsg4(("Sammib1ScalarGroupGetNext: fellThrough: %u, startAttributeID: %u, maxAttributeID: %u\n",
		fellThrough,
		startAttributeID,
		maxAttributeID));

	// initialize to return start attribute instance specified by caller
	attributeID = startAttributeID;

	// was this a fall through from a previous group?
	if (fellThrough == FALSE)
	{
		// no, do we have the attribute ID?
		if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 1)
		{
			// yes, get the attribute ID
			tempID = pIVB->name.ids[pObjInfo->oib_id_ln];

			// is passed attribute ID before first attribute ID for group?
			if (tempID >= 1)
			{
				// no, use the passed attribute ID
				attributeID = tempID;

				// do we have the suffix?
				if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 2)
				{
					// yes, at this point, the input OID is the correct length
					// or is too long; since the suffix for a scalar is zero,
					// it doesn't matter if the OID is too long;  we should
					// just try to return the next attribute in this group
					// for both cases
					attributeID++;
				}
			}
		}
	}

	// attribute loop
	for ( ; attributeID <= maxAttributeID; attributeID++)
	{
		// is this a null attribute?
		if (pObjInfo->oib_attr_pt[attributeID].aib_id == 0)
		{
			// yes, skip null attributes
			// (null attributes can occur for an abstract object
			// if there are gaps in the attribute numbers)
			continue;
		}

		// set up OID for next attribute instance
		memcpy(pOVB->name.ids,
			   pObjInfo->oib_id_pt,
			   SM_SNMP_OID_NUMIDS2SIZE(pObjInfo->oib_id_ln));
		pOVB->name.ids[pObjInfo->oib_id_ln] = attributeID;
		pOVB->name.ids[pObjInfo->oib_id_ln + 1] = 0;
		pOVB->name.numIds = pObjInfo->oib_id_ln + 2;

		SysDbgAssert4(
			SnmpOidToAStr(&pOVB->name, aBuf, sizeof(aBuf));
			SysDbgMsg4(("Sammib1ScalarGroupGetNext: trying SNMP OID: %s\n",
				aBuf));
		);

		// get value for next attribute instance
		status = Sammib1Get(pOVB, pOVB);
		if (status != SM_SNMP_STATUS_NO_SUCH_NAME)
		{
			// success or fatal error
			return status;
		}
	} // for ( ; attributeID <= maxAttributeID; attributeID++)

	// no more attribute instances for this group
	SysDbgMsg4(("Sammib1ScalarGroupGetNext: end of group\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // Sammib1ScalarGroupGetNext


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

