
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Functions to implement Sample Shared MIB Message Group
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

// Message Agent Table index used by this MIB Plug-in Implementer
u32 g_messageAgentIndex = 0;

// value returned for messageAgentAddMessageName MIB attribute
astring * g_messageAgentAddMessageName = "<set to add message to Message Table>";

// messages populated in Message Table by this MIB Plug-in Implementer
SMDLList g_messageList;


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Message Group functions
//-----------------------------------------------------------------------------


//=============================================================================
// SamshareMessageGroupStartMonitor - start monitor for Message Group
//=============================================================================
s32 SamshareMessageGroupStartMonitor(void)
{
	s32 status;


    SysDbgMsg4(("SamshareMessageGroupStartMonitor: entry\n"));

	// find first available index in Message Agent Table
	// for this message agent to use
	status = SamshareGetNextTableIdx1Int(&messageAgentTableEntry_ObjInfo,
										 &g_messageAgentIndex);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

    SysDbgMsg4(("SamshareMessageGroupStartMonitor: message agent index: %u\n",
		g_messageAgentIndex));

	// initialize message list
	SMDLListInitNoAlloc(&g_messageList);

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareMessageGroupStartMonitor


//=============================================================================
// SamshareMessageGroupStopMonitor - stop monitor for Message Group
//=============================================================================
s32 SamshareMessageGroupStopMonitor(void)
{
	s32 status;
	u32 index;
	MessageInfo * pMI;


    SysDbgMsg4(("SamshareMessageGroupStopMonitor: entry\n"));

	// delete all messages in message list
	while ((pMI = SamshareMessageTableFindNextMessage(0)) != NULL)
	{
		// get message index
		index = pMI->index;
		pMI = NULL;

		// delete message
		SamshareMessageTableDeleteMessage(index);
	}

	// free index for Message Agent Table
	status = SamshareFreeTableIdx1Int(&messageAgentTableEntry_ObjInfo,
									  g_messageAgentIndex);

	// clear message agent index
	g_messageAgentIndex = 0;

	return status;
} // SamshareMessageGroupStopMonitor


//-----------------------------------------------------------------------------
// Message Agent Table functions
//-----------------------------------------------------------------------------


//=============================================================================
// SamshareMessageAgentTableGet - process Get requests for
// Message Agent Table
//=============================================================================
s32 SamshareMessageAgentTableGet(_IN const SMSnmpVarBind * pIVB,
								 _OUT SMSnmpVarBind * pOVB)
{
	AttrInfo * pAttrInfo;
	astring * pAStr;
	s32 status;
	u32 index;


    SysDbgMsg4(("SamshareMessageAgentTableGet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameTable1Idx(pIVB,
											 &messageAgentTableEntry_ObjInfo,
											 &pAttrInfo,
											 &index);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// validate index;
	// only one index is valid for this message agent
	if (index != g_messageAgentIndex)
	{
		SysDbgMsg3(("SamshareMessageAgentTableGet: ERROR: invalid index\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// get value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case messageAgentIndex_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 index);
			break;

		case messageAgentName_ATTRIBUTE:
			// get value
			pAStr = (astring *)ModuleConfigGetDescriptionStr();

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		case messageAgentAddMessageName_ATTRIBUTE:
			// get value
			pAStr = g_messageAgentAddMessageName;

			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pAStr,
											  (u32)strlen(pAStr),
											  FALSE);
			break;

		default:
			SysDbgMsg3(("SamshareMessageAgentTableGet: attribute %u: Get not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // SamshareMessageAgentTableGet


//=============================================================================
// SamshareMessageAgentTableGetNext - process GetNext requests for
// Message Agent Table
//=============================================================================
s32 SamshareMessageAgentTableGetNext(_IN const SMSnmpVarBind * pIVB,
									 _OUT SMSnmpVarBind * pOVB,
									 _IN booln fellThrough)
{
	ObjInfo * pObjInfo;
	s32 status;
	u32 attributeID;
	u32 tempID;
	u32 index;


    SysDbgMsg4(("SamshareMessageAgentTableGetNext: fellThrough: %u\n",
		fellThrough));

	pObjInfo = &messageAgentTableEntry_ObjInfo;

	// initialize to return our first attribute instance in table
	attributeID = messageAgentIndex_ATTRIBUTE;
	index = g_messageAgentIndex;

	// was this a fall through from a previous group or table?
	if (fellThrough == FALSE)
	{
		// no, do we have the attribute ID?
		if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 1)
		{
			// yes, get the attribute ID
			tempID = pIVB->name.ids[pObjInfo->oib_id_ln];

			// is passed attribute ID before first attribute ID for table?
			if (tempID >= messageAgentIndex_ATTRIBUTE)
			{
				// no, use the passed attribute ID
				attributeID = tempID;

				// do we have the index?
				if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 2)
				{
					// yes, get the index
					index = pIVB->name.ids[pObjInfo->oib_id_ln + 1];

					// increment to return next instance of specified attribute
					index++;
				}
			}
		}
	}

	SysDbgMsg4(("SamshareMessageAgentTableGetNext: attributeID: %u, index: %u\n",
		attributeID, index));

	// attribute loop
	for ( ; attributeID <= pObjInfo->oib_attr_mx; attributeID++)
	{
		// index loop
		for ( ; index == g_messageAgentIndex; index++)
		{
			// set up OID for next attribute instance
			memcpy(pOVB->name.ids,
				   pObjInfo->oib_id_pt,
				   SM_SNMP_OID_NUMIDS2SIZE(pObjInfo->oib_id_ln));
			pOVB->name.ids[pObjInfo->oib_id_ln] = attributeID;
			pOVB->name.ids[pObjInfo->oib_id_ln + 1] = index;
			pOVB->name.numIds = pObjInfo->oib_id_ln + 2;

			// get value for next attribute instance
			status = SamshareMessageAgentTableGet(pOVB, pOVB);
			if (status != SM_SNMP_STATUS_NO_SUCH_NAME)
			{
				// success or fatal error
				return status;
			}
		} // index loop

		// reset index and try next attribute
		index = g_messageAgentIndex;
	} //  attribute loop

	// no more attribute instances for this table
	SysDbgMsg4(("SamshareMessageAgentTableGetNext: end of table\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // SamshareMessageAgentTableGetNext


//=============================================================================
// SamshareMessageAgentTableSet - process Set requests for
// Message Agent Table
//=============================================================================
s32 SamshareMessageAgentTableSet(_IN const SMSnmpVarBind * pIVB,
								 _IN u32 reqType)
{
	AttrInfo * pAttrInfo;
	s32 status;
	u32 index;


    SysDbgMsg4(("SamshareMessageAgentTableSet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameTable1Idx(pIVB,
											 &messageAgentTableEntry_ObjInfo,
											 &pAttrInfo,
											 &index);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// validate index;
	// only one index is valid for this message agent
	if (index != g_messageAgentIndex)
	{
		SysDbgMsg3(("SamshareMessageAgentTableSet: ERROR: invalid index\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// perform common Set validation for target attribute
	status = MPIVarBindValidateSetCommon(pIVB, pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// set value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case messageAgentAddMessageName_ATTRIBUTE:
			// validate value
			if ((pIVB->value.val32 < MESSAGE_TABLE_MIN_MESSAGE_SIZE) ||
				(pIVB->value.val32 > MESSAGE_TABLE_MAX_MESSAGE_SIZE))
			{
				SysDbgMsg3(("SamshareMessageAgentTableSet: ERROR: invalid message length\n"));
				status = SM_SNMP_STATUS_BAD_VALUE;
				break;
			}

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit value
			status = SamshareMessageTableAddMessage(pIVB->value.valptr,
													pIVB->value.val32);
			break;

		default:
			SysDbgMsg3(("SamshareMessageAgentTableSet: attribute %u: Set not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // SamshareMessageAgentTableSet


//-----------------------------------------------------------------------------
// Message Table functions
//-----------------------------------------------------------------------------


//=============================================================================
// SamshareMessageTableGet - process Get requests for Message Table
//=============================================================================
s32 SamshareMessageTableGet(_IN const SMSnmpVarBind * pIVB,
							_OUT SMSnmpVarBind * pOVB)
{
	AttrInfo * pAttrInfo;
	MessageInfo * pMI;
	s32 status;
	u32 index;


    SysDbgMsg4(("SamshareMessageTableGet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameTable1Idx(pIVB,
											 &messageTableEntry_ObjInfo,
											 &pAttrInfo,
											 &index);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// validate index by finding message with specified index
	pMI = SamshareMessageTableFindMessage(index);
	if (pMI == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableGet: ERROR: invalid index\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// get value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case messageIndex_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 index);
			break;

		case messageName_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pMI->pMessage,
											  (u32)strlen(pMI->pMessage),
											  FALSE);
			break;

		case messageDateTimeName_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueOctStr(pOVB,
											  pAttrInfo,
											  pMI->pDateTime,
											  (u32)strlen(pMI->pDateTime),
											  FALSE);
			break;

		case messageMessageAgentIndex_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 g_messageAgentIndex);
			break;

		case messageControl_ATTRIBUTE:
			// put value in output Varbind
			status = MPIVarBindSetValueInt32(pOVB,
											 pAttrInfo,
											 0);
			break;

		default:
			SysDbgMsg3(("SamshareMessageTableGet: attribute %u: Get not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // SamshareMessageTableGet


//=============================================================================
// SamshareMessageTableGetNext - process GetNext requests for Message Table
//=============================================================================
s32 SamshareMessageTableGetNext(_IN const SMSnmpVarBind * pIVB,
								_OUT SMSnmpVarBind * pOVB,
								_IN booln fellThrough)
{
	ObjInfo * pObjInfo;
	MessageInfo * pMI;
	s32 status;
	u32 attributeID;
	u32 tempID;
	u32 index;
	u32 firstIndex;
	u32 tempIndex;


    SysDbgMsg4(("SamshareMessageTableGetNext: fellThrough: %u\n",
		fellThrough));

	pObjInfo = &messageTableEntry_ObjInfo;

	// initialize to return our first attribute instance in table
	attributeID = messageIndex_ATTRIBUTE;

	// get first index that we support
	pMI = SamshareMessageTableFindNextMessage(0);
	if (pMI == NULL)
	{
		SysDbgMsg4(("SamshareMessageTableGetNext: end of table: no messages\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}
	firstIndex = pMI->index;
	index = pMI->index;

	// was this a fall through from a previous group or table?
	if (fellThrough == FALSE)
	{
		// no, do we have the attribute ID?
		if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 1)
		{
			// yes, get the attribute ID
			tempID = pIVB->name.ids[pObjInfo->oib_id_ln];

			// is passed attribute ID before first attribute ID for table?
			if (tempID >= messageIndex_ATTRIBUTE)
			{
				// no, use the passed attribute ID
				attributeID = tempID;

				// do we have the index?
				if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 2)
				{
					// yes, get the index
					tempIndex = pIVB->name.ids[pObjInfo->oib_id_ln + 1];

					// get next index that we support after specified index
					pMI = SamshareMessageTableFindNextMessage(tempIndex);
					if (pMI == NULL)
					{
						// return first instance of next attribute
						attributeID++;
					}
					else
					{
						// return next instance of specified attribute
						index = pMI->index;
					}
				}
			}
		}
	}

	SysDbgMsg4(("SamshareMessageTableGetNext: attributeID: %u, index: %u\n",
		attributeID, index));

	// attribute loop
	for ( ; attributeID <= pObjInfo->oib_attr_mx; attributeID++)
	{
		// index loop
		while (TRUE)
		{
			// set up OID for next attribute instance
			memcpy(pOVB->name.ids,
				   pObjInfo->oib_id_pt,
				   SM_SNMP_OID_NUMIDS2SIZE(pObjInfo->oib_id_ln));
			pOVB->name.ids[pObjInfo->oib_id_ln] = attributeID;
			pOVB->name.ids[pObjInfo->oib_id_ln + 1] = index;
			pOVB->name.numIds = pObjInfo->oib_id_ln + 2;

			// get value for next attribute instance
			status = SamshareMessageTableGet(pOVB, pOVB);
			if (status != SM_SNMP_STATUS_NO_SUCH_NAME)
			{
				// success or fatal error
				return status;
			}

			// try next index
			pMI = SamshareMessageTableFindNextMessage(index);
			if (pMI == NULL)
			{
				// no next index
				break;
			}
			index = pMI->index;
		} // index loop

		// get first index and try next attribute
		index = firstIndex;
	} //  attribute loop

	// no more attribute instances for this table
	SysDbgMsg4(("SamshareMessageTableGetNext: end of table\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // SamshareMessageTableGetNext


//=============================================================================
// SamshareMessageTableSet - process Set requests for Message Table
//=============================================================================
s32 SamshareMessageTableSet(_IN const SMSnmpVarBind * pIVB,
							_IN u32 reqType)
{
	AttrInfo * pAttrInfo;
	MessageInfo * pMI;
	s32 status;
	u32 index;


    SysDbgMsg4(("SamshareMessageTableSet: entry\n"));

	// validate name of target attribute
	status = MPIVarBindValidateNameTable1Idx(pIVB,
											 &messageTableEntry_ObjInfo,
											 &pAttrInfo,
											 &index);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// validate index by finding message with specified index
	pMI = SamshareMessageTableFindMessage(index);
	if (pMI == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableGet: ERROR: invalid index\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// perform common Set validation for target attribute
	status = MPIVarBindValidateSetCommon(pIVB, pAttrInfo);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	// set value of target attribute
	switch (pAttrInfo->aib_id)
	{
		case messageControl_ATTRIBUTE:
			// validate value
			switch (pIVB->value.val32)
			{
				case MESSAGE_CONTROL_LOWERCASE:
				case MESSAGE_CONTROL_UPPERCASE:
				case MESSAGE_CONTROL_REVERSE:
				case MESSAGE_CONTROL_DELETE:
					status = SM_SNMP_STATUS_NO_ERROR;
					break;

				default:
					SysDbgMsg3(("SamshareMessageTableSet: ERROR: invalid message control value\n"));
					status = SM_SNMP_STATUS_BAD_VALUE;
					break;
			} // switch (pIVB->value.val32)

			// is value valid?
			if (status != SM_SNMP_STATUS_NO_ERROR)
			{
				// no
				break;
			}

			// test or commit?
			if (reqType == SM_SNMP_API_MPI_SET_TEST)
			{
				status = SM_SNMP_STATUS_NO_ERROR;
				break;
			}

			// commit value
			switch (pIVB->value.val32)
			{
				case MESSAGE_CONTROL_LOWERCASE:
					SMUTF8StrLowerCase(pMI->pMessage);
					break;

				case MESSAGE_CONTROL_UPPERCASE:
					SMUTF8StrUpperCase(pMI->pMessage);
					break;

				case MESSAGE_CONTROL_REVERSE:
					SMUTF8Strrev(pMI->pMessage);
					break;

				case MESSAGE_CONTROL_DELETE:
					status = SamshareMessageTableDeleteMessage(index);
					break;
			} // switch (pIVB->value.val32)
			break;

		default:
			SysDbgMsg3(("SamshareMessageTableSet: attribute %u: Set not implemented\n",
				pAttrInfo->aib_id));
			status = SM_SNMP_STATUS_GEN_ERR;
			break;
	} // switch (pAttrInfo->aib_id)

	return status;
} // SamshareMessageTableSet


//=============================================================================
// SamshareMessageTableAddMessage - add message to Message Table
//=============================================================================
s32 SamshareMessageTableAddMessage(_IN const astring * pMessage,
								   _IN u32 messageLen)
{
	SMDLListEntry * pDLE;
	MessageInfo * pMI;
	time_t currTime;
	s32 status;
	u32 index;
	u32 dateTimeBufSize;


	// find first available index in Message Table
	status = SamshareGetNextTableIdx1Int(&messageTableEntry_ObjInfo,
										 &index);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		_throw error1;
	}

    SysDbgMsg4(("SamshareMessageTableAddMessage: index: %u\n", index));

	// allocate message list entry
	pDLE = SMDLListEntryAlloc(sizeof(MessageInfo));
	if (pDLE == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableAddMessage: ERROR: no memory\n"));
		status = SM_SNMP_STATUS_GEN_ERR;
		_throw error2;
	}

	// put message info in message list entry
	pMI = pDLE->pData;
	pMI->index = index;

	// allocate buffer for message
	pMI->pMessage = SMAllocMem(messageLen + 1);
	if (pMI->pMessage == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableAddMessage: ERROR: no memory\n"));
		status = SM_SNMP_STATUS_GEN_ERR;
		_throw error3;
	}

	// copy message to buffer and make it null terminated
	memcpy(pMI->pMessage, pMessage, messageLen);
	pMI->pMessage[messageLen] = 0;

	// allocate buffer for date and time
	dateTimeBufSize = (MESSAGE_TABLE_MAX_DATETIME_SIZE + 1);
	pMI->pDateTime = SMAllocMem(dateTimeBufSize);
	if (pMI->pDateTime == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableAddMessage: ERROR: no memory\n"));
		status = SM_SNMP_STATUS_GEN_ERR;
		_throw error4;
	}

	// format date and time in buffer
	time(&currTime);
	ctime_s(pMI->pDateTime, dateTimeBufSize, &currTime);
	pMI->pDateTime[strlen(pMI->pDateTime) - 1] = 0;

	// insert message list entry into message list;
	// message list is sorted in ascending order based on index
	status = SMDLListInsertEntry(&g_messageList,
								 pDLE,
								 (void *)&index,
								 SamshareMessageTableCompareExact);
	if (status != SM_STATUS_SUCCESS)
	{
		SysDbgMsg3(("SamshareMessageTableAddMessage: SMDLListInsertEntry failed: %d\n",
			status));
		status = SM_SNMP_STATUS_GEN_ERR;
		_throw error5;
	}

	// success
	return SM_SNMP_STATUS_NO_ERROR;


	// error handling
error5:
	// free buffer for date and time
	SMFreeMem(pMI->pDateTime);
error4:
	// free buffer for message
	SMFreeMem(pMI->pMessage);
error3:
	// free message list entry
	SMDLListEntryFree(pDLE);
error2:
	// free table index
	SamshareFreeTableIdx1Int(&messageTableEntry_ObjInfo, index);
error1:
	// return error
	return status;
} // SamshareMessageTableAddMessage


//=============================================================================
// SamshareMessageTableDeleteMessage - delete message from Message Table
//=============================================================================
s32 SamshareMessageTableDeleteMessage(_IN u32 index)
{
	SMDLListEntry * pDLE;
	MessageInfo * pMI;
	s32 status;


    SysDbgMsg4(("SamshareMessageTableDeleteMessage: index: %u\n", index));

	// find message to be deleted
	pDLE = SMDLListWalkAtHead(&g_messageList,
							  (void *)&index,
							  SamshareMessageTableCompareExact);
	if (pDLE == NULL)
	{
		SysDbgMsg3(("SamshareMessageTableDeleteMessage: ERROR: message not found\n"));
		return SM_SNMP_STATUS_GEN_ERR;
	}

	// get access to message info
	pMI = pDLE->pData;

	// remove message from message list
	SMDLListDeleteEntry(&g_messageList, pDLE);

	// free buffer for date and time
	SMFreeMem(pMI->pDateTime);

	// free buffer for message
	SMFreeMem(pMI->pMessage);

	// free message table entry
	SMDLListEntryFree(pDLE);

	// free table index
	status = SamshareFreeTableIdx1Int(&messageTableEntry_ObjInfo, index);
SysDbg3(
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		SysDbgMsg3(("SamshareMessageTableDeleteMessage: MIBImplementerUnregTableIndex failed: %d\n",
			status));
	}
) // SysDbg3

	return status;
} // SamshareMessageTableDeleteMessage


//=============================================================================
// SamshareMessageTableFindMessage - find message with specified index
//=============================================================================
MessageInfo * SamshareMessageTableFindMessage(_IN u32 index)
{
	SMDLListEntry * pDLE;


	// search message list for message with specified index
	pDLE = SMDLListWalkAtHead(&g_messageList,
							  (void *)&index,
							  SamshareMessageTableCompareExact);
	if (pDLE == NULL)
	{
		// message not found
		return NULL;
	}

	// return message info
	return (MessageInfo *)pDLE->pData;
} // SamshareMessageTableFindMessage


//=============================================================================
// SamshareMessageTableFindNextMessage - find message after message with
// specified index
//=============================================================================
MessageInfo * SamshareMessageTableFindNextMessage(_IN u32 index)
{
	SMDLListEntry * pDLE;


	// search message list for message after message with specified index
	pDLE = SMDLListWalkAtHead(&g_messageList,
							  (void *)&index,
							  SamshareMessageTableCompareNext);
	if (pDLE == NULL)
	{
		// next message not found
		return NULL;
	}

	// return next message info
	return (MessageInfo *)pDLE->pData;
} // SamshareMessageTableFindNextMessage


//=============================================================================
// SamshareMessageTableCompareExact - perform exact comparison of walk data to
// specified list entry
//=============================================================================
s32 __cdecl SamshareMessageTableCompareExact(_INOUT void * pWalkData, 
											 _IN const SMDLListEntry * pDLECur)
{
	MessageInfo * pMI;
	u32 index;


	// get target index
	index = *((u32 *)pWalkData);

	// get message info from current list entry
	pMI = (MessageInfo *)pDLECur->pData;

	// return:  0 if pWalkData contents == pDLECur contents
	//        < 0 if pWalkData contents <  pDLECur contents
	//        > 0 if pWalkData contents >  pDLECur contents
	return (index - pMI->index);
} // SamshareMessageTableCompareExact


//=============================================================================
// SamshareMessageTableCompareNext - perform comparison of walk data to
// specified list entry to determine if list entry comes after walk data
//=============================================================================
s32 __cdecl SamshareMessageTableCompareNext(_INOUT void * pWalkData, 
											_IN const SMDLListEntry * pDLECur)
{
	MessageInfo * pMI;
	u32 index;


	// get target index
	index = *((u32 *)pWalkData);

	// get message info from current list entry
	pMI = (MessageInfo *)pDLECur->pData;

	// return:  0 if pDLECur contents >  pWalkData contents
	//        < 0 if pLDECur contents <= pWalkData contents
	if (pMI->index > index)
	{
		return 0;
	}
	else
	{
		return -1;
	}
} // SamshareMessageTableCompareNext


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

