
//-----------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2007 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Doug Warzecha
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/LH
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Sample Shared MIB support functions
//
//  See Also:
//    Data Engine High Level Design Specification
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include <moduleincs.h>


//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Local function prototypes
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

// Sample Shared MIB SNMP OID used for prefix check and traps
u32 g_samshareIds[] = { 1, 3, 6, 1, 4, 1, 674, 55555, 9 };
SMSnmpOid g_samshareOid = SM_SNMP_OID_INITIALIZER(g_samshareIds);


//-----------------------------------------------------------------------------
// Begin Code
//-----------------------------------------------------------------------------


//=============================================================================
// SamshareAttach - attach Sample Shared MIB
//=============================================================================
s32 SamshareAttach(void)
{
	ObjInfo ** pMIB;
	ObjInfo * pObjInfo;
	u32 enterpriseID;


    SysDbgMsg4(("SamshareAttach: entry\n"));

	// check for possible alternate MIB enterprise ID
	enterpriseID = ModuleConfigGetEnterpriseID();

	// was alternate MIB enterprise ID specified?
	if (enterpriseID != 0)
	{
		// yes, change MIB enterprise ID used for Sample MIB 1
		SysDbgMsg4(("SamshareAttach: changing MIB enterprise ID to: %u\n",
			enterpriseID));

		// the base SNMP OID is being changed from:
		//   1.3.6.1.4.1.674.55555.9
		// to:
		//   1.3.6.1.4.1.<enterpriseID>.55555.9

		// update SNMP OID used for prefix check and traps
		g_samshareIds[SM_SNMP_OID_ENTERPRISE_ID_INDEX] = enterpriseID;

		// update SNMP OIDs generated by Novell MIB compiler
		for (pMIB = SAMSHARE_MIB; *pMIB != NULL; pMIB++)
		{
			pObjInfo = *pMIB;
			pObjInfo->oib_id_pt[SM_SNMP_OID_ENTERPRISE_ID_INDEX] = enterpriseID;
		}
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareAttach


//=============================================================================
// SamshareDetach - detach Sample Shared MIB
//=============================================================================
s32 SamshareDetach(void)
{

    SysDbgMsg4(("SamshareDetach: entry\n"));

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareAttach


//=============================================================================
// SamshareListMib - list MIBs for Sample Shared MIB
//=============================================================================
s32 SamshareListMib(_INOUT SMSnmpMibList * pMibList,
					_IN u32 outBufSize,
					_OUT u32 * pBytesReturned)
{
	s32 status;


    SysDbgMsg4(("SamshareListMib: entry\n"));

	// add Sample Shared MIB
	// (SAMSHARE_MIB is defined in samshare.c which is generated
	//  by Novell MIB compiler)
	status = MPIMibListAdd(SAMSHARE_MIB, pMibList, outBufSize, pBytesReturned);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		return status;
	}

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareListMib


//=============================================================================
// SamshareStartMonitor - start monitor for Sample Shared MIB
//=============================================================================
s32 SamshareStartMonitor(void)
{
	s32 status;


    SysDbgMsg4(("SamshareStartMonitor: entry\n"));

	// start monitor for Message Group
	status = SamshareMessageGroupStartMonitor();

	return status;
} // SamshareStartMonitor


//=============================================================================
// SamshareStopMonitor - stop monitor for Sample Shared MIB
//=============================================================================
s32 SamshareStopMonitor(void)
{
	s32 status;


    SysDbgMsg4(("SamshareStopMonitor: entry\n"));

	// stop monitor for Message Group
	status = SamshareMessageGroupStopMonitor();

	return status;
} // SamshareStopMonitor


//=============================================================================
// SamshareGet - process Get requests for Sample Shared MIB
//=============================================================================
s32 SamshareGet(_IN const SMSnmpVarBind * pIVB,
				_OUT SMSnmpVarBind * pOVB)
{
	s32 status;
	u32 groupId;
	u32 tableId;


    SysDbgMsg4(("SamshareGet: entry\n"));

	// get MIB group ID
	status = SamshareGetGroupID(pIVB, &groupId);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		// not enough IDs for MIB group ID;
		// we can't proceed without the MIB group ID
		return status;
	}

	// process request based on group ID
	switch (groupId)
	{
		case SAMSHARE_MESSAGE_GROUP:
			// get MIB table ID
			status = SamshareGetTableID(pIVB, &tableId);
			if (status != SM_SNMP_STATUS_NO_ERROR)
			{
				// not enough IDs for MIB table ID;
				// we can't proceed without the MIB table ID
				return status;
			}

			// process request based on table ID
			switch (tableId)
			{
				case SAMSHARE_MESSAGE_AGENT_TABLE:
					status = SamshareMessageAgentTableGet(pIVB, pOVB);
					break;

				case SAMSHARE_MESSAGE_TABLE:
					status = SamshareMessageTableGet(pIVB, pOVB);
					break;

				default:
					SysDbgMsg3(("SamshareGet: ERROR: invalid table ID: %u\n",
						tableId));
					status = SM_SNMP_STATUS_NO_SUCH_NAME;
					break;
			}
			break;

		default:
			SysDbgMsg3(("SamshareGet: ERROR: invalid group ID: %u\n",
				groupId));
			status = SM_SNMP_STATUS_NO_SUCH_NAME;
			break;
	} // switch (groupId)

	return status;
} // SamshareGet


//=============================================================================
// SamshareGetNext - process GetNext requests for Sample Shared MIB
//=============================================================================
s32 SamshareGetNext(_IN const SMSnmpVarBind * pIVB,
					_OUT SMSnmpVarBind * pOVB)
{
	s32 prefixStatus;
	u32 groupID;
	u32 tableID;
	u32 tableEntryID;
	booln fellThrough = FALSE;


    SysDbgMsg4(("SamshareGetNext: entry\n"));

	// check input variable OID
	prefixStatus = SamshareCheckPrefix(pIVB);
	if (prefixStatus == 0)
	{
		// input variable OID is in our MIB subtree;
		// get MIB group ID
		if (SamshareGetGroupID(pIVB, &groupID) != SM_SNMP_STATUS_NO_ERROR)
		{
			// not enough IDs for MIB group ID;
			// return first attribute instance in our MIB
			groupID = 0;
		}
	}
	else if (prefixStatus < 0)
	{
		// input variable OID comes before our MIB subtree;
		// return first attribute instance in our MIB
		SysDbgMsg3(("SamshareGetNext: before our MIB subtree\n"));
		groupID = 0;
	}
	else
	{
		// input variable OID comes after our MIB subtree
		// (we should never make it here)
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// if valid group ID for non-indexed group, do nothing here;
	// if valid group ID for indexed group, get Table ID;
	// if invalid group ID, map to valid group ID
	switch (groupID)
	{
		// currently no non-indexed groups;
		// if one is added, a case should be added here to do nothing

		case SAMSHARE_MESSAGE_GROUP:
			// valid group ID for indexed group;
			// no mapping needed

			// get MIB table ID
			if (SamshareGetTableID(pIVB, &tableID) != SM_SNMP_STATUS_NO_ERROR)
			{
				// not enough IDs for MIB table ID;
				// return first attribute instance in group
				fellThrough = TRUE;
				break;
			}
			break;

		default:
			// invalid group ID;
			// map it to the next group ID numerically
			fellThrough = TRUE;
			if (groupID < SAMSHARE_MESSAGE_GROUP)
			{
				groupID = SAMSHARE_MESSAGE_GROUP;
			}
			else
			{
				// group ID is greater than our last supported group ID;
				// it's past end of our MIB subtree
				SysDbgMsg4(("SamshareGetNext: group ID after last supported group ID\n"));
				return SM_SNMP_STATUS_NO_SUCH_NAME;
			}
			SysDbgMsg4(("SamshareGetNext: mapped group ID: %u\n", groupID));
			break;
	} // switch (groupID)

	// process GetNext command based on group ID
	switch (groupID)
	{
		case SAMSHARE_MESSAGE_GROUP:
		{
			if (fellThrough == TRUE)
			{
				// return first attribute instance in group
				tableID = 0;
			}

			// if valid table ID, check Table Entry ID;
			// if invalid table ID, map to valid table ID
			switch (tableID)
			{
				case SAMSHARE_MESSAGE_AGENT_TABLE:
				case SAMSHARE_MESSAGE_TABLE:
					// get Table Entry ID
					if (SamshareGetTableEntryID(pIVB, &tableEntryID) !=
						SM_SNMP_STATUS_NO_ERROR)
					{
						// not enough IDs for Table Entry ID;
						// return first attribute instance in table
						fellThrough = TRUE;
						break;
					}

					// is Table Entry ID is valid?
					if (tableEntryID == SAMSHARE_TABLE_ENTRY)
					{
						// yes
						break;
					}

					// is Table Entry ID before required table entry ID?
					if (tableEntryID < SAMSHARE_TABLE_ENTRY)
					{
						// yes, return first attribute instance in table
						fellThrough = TRUE;
						break;
					}

					// Table Entry ID is after required Table Entry ID;
					// fall through to next table or group
					tableID++;

				default:
					// map to next table ID;
					// if table ID is greater than last table ID in group,
					// fall through to next group
					fellThrough = TRUE;
					if (tableID < SAMSHARE_MESSAGE_AGENT_TABLE)
					{
						tableID = SAMSHARE_MESSAGE_AGENT_TABLE;
					}
					else if (tableID < SAMSHARE_MESSAGE_TABLE)
					{
						tableID = SAMSHARE_MESSAGE_TABLE;
					}
					SysDbgMsg4(("SamshareGetNext: mapped table ID: %u\n",
						tableID));
					break;
			} // switch (tableID)

			// process GetNext command based on table ID
			switch (tableID)
			{
				case SAMSHARE_MESSAGE_AGENT_TABLE:
					return SamshareMessageAgentTableGetNext(pIVB,
															pOVB,
															fellThrough);

				case SAMSHARE_MESSAGE_TABLE:
					return SamshareMessageTableGetNext(pIVB,
													   pOVB,
													   fellThrough);
			} // switch (tableID)

			SysDbgMsg4(("SamshareGetNext: fall through to next group\n"));
			fellThrough = TRUE;
		} // case SAMSHARE_MESSAGE_GROUP


		default:
		{
			break;
		} //  default
	} // switch (groupID)

	// end of Sample Shared MIB
	SysDbgMsg4(("SamshareGetNext: end of MIB\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // SamshareGetNext


//=============================================================================
// SamshareSet - process Set requests for Sample Shared MIB
//=============================================================================
s32 SamshareSet(_IN const SMSnmpVarBind * pIVB,
				_IN u32 reqType)
{
	s32 status;
	u32 groupID;
	u32 tableID;


    SysDbgMsg4(("SamshareSet: entry\n"));

	// get MIB group ID
	status = SamshareGetGroupID(pIVB, &groupID);
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		// not enough IDs for MIB group ID;
		// we can't proceed without the MIB group ID
		return status;
	}

	// process request based on group ID
	switch (groupID)
	{
		case SAMSHARE_MESSAGE_GROUP:
			// get MIB table ID
			status = SamshareGetTableID(pIVB, &tableID);
			if (status != SM_SNMP_STATUS_NO_ERROR)
			{
				// not enough IDs for MIB table ID;
				// we can't proceed without the MIB table ID
				return status;
			}

			// process request based on table ID
			switch (tableID)
			{
				case SAMSHARE_MESSAGE_AGENT_TABLE:
					status = SamshareMessageAgentTableSet(pIVB, reqType);
					break;

				case SAMSHARE_MESSAGE_TABLE:
					status = SamshareMessageTableSet(pIVB, reqType);
					break;

				default:
					SysDbgMsg3(("SamshareSet: ERROR: invalid table ID: %u\n",
						tableID));
					status = SM_SNMP_STATUS_NO_SUCH_NAME;
					break;
			}
			break;

		default:
			SysDbgMsg3(("SamshareSet: ERROR: invalid group ID: %u\n",
				groupID));
			status = SM_SNMP_STATUS_NO_SUCH_NAME;
			break;
	} // switch (groupID)

	return status;
} // SamshareSet


//=============================================================================
// SamshareCheckPrefix - determine where the specified variable falls in the
// MIB tree relative to the Sample Shared MIB subtree
//
// returns <0 if variable precedes Sample Shared MIB
//          0 if variable is in    Sample Shared MIB
//         >0 if variable follows  Sample Shared MIB
//=============================================================================
s32 SamshareCheckPrefix(_IN const SMSnmpVarBind * pVB)
{
	u32 numIdsToCompare;


	// number of IDs to compare is number of IDs in shorter of the two OIDs
	if (pVB->name.numIds < g_samshareOid.numIds)
	{
		numIdsToCompare = pVB->name.numIds;
	}
	else
	{
		numIdsToCompare = g_samshareOid.numIds;
	}

	// compare the variable OID to the MIB subtree OID
	return SnmpOidNCmp(&pVB->name, &g_samshareOid, numIdsToCompare);
} // SamshareCheckPrefix


//=============================================================================
// SamshareGetGroupID - get group ID from variable name
//=============================================================================
s32 SamshareGetGroupID(_IN const SMSnmpVarBind * pVB,
					   _OUT u32 * pGroupId)
{

	// SamshareCheckPrefix must be called before calling this function;
	// variable OID must be at least one ID longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_samshareOid.numIds + 1))
	{
		// variable OID is too short to contain the group ID
		SysDbgMsg4(("SamshareGetGroupID: not enough IDs for group ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the first ID after MIB subtree base OID
	*pGroupId = pVB->name.ids[g_samshareOid.numIds];

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareGetGroupID


//=============================================================================
// SamshareGetTableID - get table ID from variable name
//=============================================================================
s32 SamshareGetTableID(_IN const SMSnmpVarBind * pVB,
					   _OUT u32 * pTableId)
{

	// SamshareCheckPrefix must be called before calling this function;
	// variable OID must be at least two IDs longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_samshareOid.numIds + 2))
	{
		// variable OID is too short to contain the table ID
		SysDbgMsg4(("SamshareGetTableID: not enough IDs for table ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the second ID after MIB subtree base OID
	*pTableId = pVB->name.ids[g_samshareOid.numIds + 1];

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareGetTableID


//=============================================================================
// SamshareGetTableEntryID - get table entry ID from variable name
//=============================================================================
s32 SamshareGetTableEntryID(_IN const SMSnmpVarBind * pVB,
							_OUT u32 * pTableEntryId)
{

	// SamshareCheckPrefix must be called before calling this function;
	// variable OID must be at least three IDs longer than our MIB subtree OID

	// check length of variable OID
	if (pVB->name.numIds < (g_samshareOid.numIds + 3))
	{
		// variable OID is too short to contain the table entry ID
		SysDbgMsg4(("SamshareVerifyTableEntryID: not enough IDs for table entry ID\n"));
		return SM_SNMP_STATUS_NO_SUCH_NAME;
	}

	// return the third ID after MIB subtree base OID
	*pTableEntryId = pVB->name.ids[g_samshareOid.numIds + 2];

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareGetTableEntryID


//=============================================================================
// SamshareGetNextTableIdx1Int - get next available table index for table
// with 1 integer index
//=============================================================================
s32 SamshareGetNextTableIdx1Int(_IN ObjInfo * pObjInfo,
								_OUT u32 * pIndex)
{
	SMSnmpOid indexOid;
	s32 status;
	u32 index;


	// initialize index OID
	indexOid.numIds = 1;
	indexOid.ids = &index;

	// find available table index; start at 1 and keep incrementing index
	// until an index is successfully registered
	index = 1;
	while (TRUE)
	{
		// register index with MIB Plug-in Manager
		status = MIBImpMPIMPMRegisterMIBTableIndex(pObjInfo, &indexOid);
		if (status == SM_SNMP_STATUS_NO_ERROR)
		{
			// table index was successfully registered
			break;
		}
		else if (status != SM_SNMP_STATUS_EX_DUPLICATE_REG)
		{
			// fatal error
			SysDbgMsg3(("SamshareGetNextTableIdx1Int: MIBImpMPIMPMRegisterMIBTableIndex failed: %d\n",
				status));
			return status;
		}

		// index in use; try another index
		index++;
	} // while (TRUE)

	// return index
	*pIndex = index;

	return SM_SNMP_STATUS_NO_ERROR;
} // SamshareGetNextTableIdx1Int


//=============================================================================
// SamshareFreeTableIdx1Int - free table index for table with 1 integer index
//=============================================================================
s32 SamshareFreeTableIdx1Int(_IN ObjInfo * pObjInfo,
							 _IN u32 index)
{
	SMSnmpOid indexOid;
	s32 status;


	// initialize index OID
	indexOid.numIds = 1;
	indexOid.ids = &index;

	// unregister table index with MIB Plug-in Manager
	status = MIBImpMPIMPMUnregisterMIBTableIndex(pObjInfo, &indexOid);

	SysDbg3(
	if (status != SM_SNMP_STATUS_NO_ERROR)
	{
		SysDbgMsg3(("SamshareFreeTableIdx1Int: MIBImpMPIMPMUnregisterMIBTableIndex failed: %d\n",
			status));
	}
	) // SysDbg3

	return status;
} // SamshareGetNextTableIdx1Int


//=============================================================================
// SamshareScalarGroupGetNext - process GetNext requests for scalar groups
//=============================================================================
s32 SamshareScalarGroupGetNext(_IN const SMSnmpVarBind * pIVB,
							   _OUT SMSnmpVarBind * pOVB,
							   _IN ObjInfo * pObjInfo,
							   _IN booln fellThrough)
{
	s32 status;
	u32 attributeID;
	u32 tempID;
	SysDbg4( astring aBuf[512]; )


	SysDbgMsg4(("SamshareScalarGroupGetNext: fellThrough: %u\n",
		fellThrough));

	// initialize to return first attribute instance in group
	attributeID = 1;

	// was this a fall through from a previous group?
	if (fellThrough == FALSE)
	{
		// no, do we have the attribute ID?
		if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 1)
		{
			// yes, get the attribute ID
			tempID = pIVB->name.ids[pObjInfo->oib_id_ln];

			// is passed attribute ID before first attribute ID for group?
			if (tempID >= 1)
			{
				// no, use the passed attribute ID
				attributeID = tempID;

				// do we have the suffix?
				if (pIVB->name.numIds >= pObjInfo->oib_id_ln + 2)
				{
					// yes, at this point, the input OID is the correct length
					// or is too long; since the suffix for a scalar is zero,
					// it doesn't matter if the OID is too long;  we should
					// just try to return the next attribute in this group
					// for both cases
					attributeID++;
				}
			}
		}
	}

	// attribute loop
	for ( ; attributeID <= pObjInfo->oib_attr_mx; attributeID++)
	{
		// set up OID for next attribute instance
		memcpy(pOVB->name.ids,
			   pObjInfo->oib_id_pt,
			   SM_SNMP_OID_NUMIDS2SIZE(pObjInfo->oib_id_ln));
		pOVB->name.ids[pObjInfo->oib_id_ln] = attributeID;
		pOVB->name.ids[pObjInfo->oib_id_ln + 1] = 0;
		pOVB->name.numIds = pObjInfo->oib_id_ln + 2;

		SysDbgAssert4(
			SnmpOidToAStr(&pOVB->name, aBuf, sizeof(aBuf));
			SysDbgMsg4(("SamshareScalarGroupGetNext: trying SNMP OID: %s\n",
				aBuf));
		);

		// get value for next attribute instance
		status = SamshareGet(pOVB, pOVB);
		if (status != SM_SNMP_STATUS_NO_SUCH_NAME)
		{
			// success or fatal error
			return status;
		}
	} // attribute loop

	// no more attribute instances for this group
	SysDbgMsg4(("SamshareScalarGroupGetNext: end of group\n"));

	return SM_SNMP_STATUS_NO_SUCH_NAME;
} // SamshareScalarGroupGetNext


//-----------------------------------------------------------------------------
// End Code
//-----------------------------------------------------------------------------

