
//-------------------------------------------------------------------------
//
//          Dell Inc. PROPRIETARY INFORMATION
//
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Dell Inc. and may not
//  be copied or disclosed except in accordance with the terms of that
//  agreement.
//
//  Copyright (c) 1995-2012 Dell Inc. All Rights Reserved.
//
//
//  Revision:
//    $Revision: $
//
//  Last Modified By / On:
//    $Author: $ / $Date: $
//
//  Author:
//    Data Engine
//
//  Environment:
//    Linux, NetWare, Windows NT/2000/2003/2008
//
//  User/Kernel Mode:
//    User
//  
//  Abstract/Purpose:
//    Dell SM support API functions
//
//  See Also:
//    Data Engine High Level Design document
//
//  Notes:
//    <Put notes here that you want to be read before code reading/modifying>
//
//-------------------------------------------------------------------------

/** @file */

#ifndef __DCSUPAPI_H_INCLUDED__
#define __DCSUPAPI_H_INCLUDED__


//---------------------------------------------------------------------
//  Includes this file is dependent on
//---------------------------------------------------------------------
#include <dcsupdef.h>


#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------
// Miscellaneous API function prototypes applied to all.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Miscellaneous API function prototypes: get major product version.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SUPTGetMajorVersion(void);


//---------------------------------------------------------------------
/**
 * Miscellaneous API function prototypes: get Minor product version.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SUPTGetMinorVersion(void);


//---------------------------------------------------------------------
/**
 * Miscellaneous API function prototypes: get product revision.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SUPTGetRevision(void);


#ifdef SM_LIB_STATIC
//---------------------------------------------------------------------
/**
 * Function to support static version of this library.
 * These functions are not exported.
 */
//---------------------------------------------------------------------

booln DCSUPTDetach(void);


//---------------------------------------------------------------------
/**
 * Function to support static version of this library.
 * These functions are not exported.
 */
//---------------------------------------------------------------------

booln DCSUPTAttach(void);
#endif // SM_LIB_STATIC



//---------------------------------------------------------------------
// Memory management routines.
//---------------------------------------------------------------------

// setup marcos as needed
#if defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE)

// redefine support routines as needed
// if compiling DCSUPT with SYSTEM_DEBUG_MEM_USAGE
#ifndef MODULE_IS_DCSUPT
// Module being compiled is not DCSUPT
#define SMAllocMem			malloc
#define SMReAllocMem		realloc
#define SMFreeMem			free
#endif // MODULE_IS_DCSUPT

#endif // defined(_DEBUG) && defined(SYSTEM_DEBUG_MEM_USAGE)


//---------------------------------------------------------------------
/**
 * Allocates the memory amount of memory requested. 
 * 
 * SMAllocMem() function allocates a memory block of at least size 
 * bytes. The block may be larger than size bytes because of space 
 * required for alignment and maintenance information. 
 *
 * If size is 0, SMAllocMem() allocates a zero-length item in the heap 
 * and returns a valid pointer to that item. 
 *
 * Always check the return from SMAllocMem(), even if the amount of 
 * memory requested is small.
 *
 * @note SM wraps memory allocation routines around preprocessor macros 
 *		so that they can be redefined for memory leak check purposes.
 *
 * @note Use SMFreeMem() call to release returned memory.
 *
 * @param size [in] size of memory to be allocated.
 *
 * @return Return a void pointer to the allocated memory or NULL if 
 *		there is insufficient memory available.
 *
 * @sa SMReAllocMem();
 *
 * @sa SMFreeMem();
 *
 * @sa SMFreeGeneric();
 */
//---------------------------------------------------------------------

// If not mapped to CRT debug routines by dcsossdk.h
#ifndef	SMAllocMem
SMINTF void * _cdecl SMAllocMem(_IN u32 size);
#endif // SMAllocMem


//---------------------------------------------------------------------
/**
 * Reallocates the memory amount of memory requested. 
 *
 * SMReAllocMem() changes the size of an allocated memory block. The 
 * pMem argument points to the beginning of the memory block. 
 *
 * If pMem is NULL, SMReAllocMem() behaves the same way as SMAllocMem()
 * and allocates a new block of size bytes. If pMem is not NULL, it 
 * should be a pointer returned by a previous call to SMAllocMem(), or 
 * SMReAllocMem().
 *
 * The size argument gives the new size of the block, in bytes. The 
 * contents of the block are unchanged up to the shorter of the new and 
 * old sizes, although the new block can be in a different location. 
 * Because the new block can be in a new memory location, the pointer 
 * returned by SMReAllocMem() is not guaranteed to be the pointer passed 
 * through the pMem argument.
 *
 * @note SM wraps memory allocation routines around preprocessor macros 
 *		so that they can be redefined for memory leak check purposes.
 *
 * @note Use SMFreeMem() call to release returned memory.
 *
 * @param pMem [in] Pointer to previously allocated memory block.
 *
 * @param size [in] new size in bytes.
 *
 * @return Returns a void pointer to the reallocated (and possibly 
 *		moved) memory block. 
 * 
 * @return The return value is NULL if the size is zero and the buffer 
 *		argument is not NULL, or if there is not enough available memory 
 *		to expand the block to the given size. In the first case, the 
 *		original block is freed. In the second, the original block is 
 *		unchanged. 
 *
 * @note The return value points to a storage space that is guaranteed 
 * to be suitably aligned for storage of any type of object. To get a 
 * pointer to a type other than void, use a type cast on the return 
 * value.
 *
 * @sa SMAllocMem();
 *
 * @sa SMFreeMem();
 *
 * @sa SMFreeGeneric();
 */
//---------------------------------------------------------------------

// If not mapped to CRT debug routines by dcsossdk.h
#ifndef SMReAllocMem
SMINTF void * _cdecl SMReAllocMem(_IN void *pMem, 
								  _IN u32 size);
#endif // SMReAllocMem


//---------------------------------------------------------------------
/**
 * Deallocate or frees a memory block. 
 * 
 * The SMFreeMem() function deallocates a memory block (pMem) that was 
 * previously allocated by a call to SMAllocMem(), or SMReAllocMem(). 
 * The number of freed bytes is equivalent to the number of bytes 
 * requested when the block was allocated (or reallocated, in the case 
 * of SMReAllocMem()). 
 * 
 * If pMem is NULL, the pointer is ignored and free immediately 
 * returns. 
 *
 * Attempting to free an invalid pointer (a pointer to a memory block 
 * that was not allocated by SMAllocMem(), or SMReAllocMem()) may affect 
 * subsequent allocation requests and cause errors.
 *
 * @param pMem [in] pointer to memory to be freed.
 *
 * @return None.
 *
 * @sa SMAllocMem()
 * 
 * @sa SMReAllocMem()
 * 
 * @sa SMFreeGeneric()
 */
//---------------------------------------------------------------------

// If not mapped to CRT debug routines by dcsossdk.h
#ifndef SMFreeMem
SMINTF void _cdecl SMFreeMem(_IN void *pMem);
#endif // SMFreeMem


//---------------------------------------------------------------------
/**
 * SM memory management routine: free memory (generic).
 *
 * SMFreeGeneric should be used to free memory returned from APIs 
 * in this module that have to be freed after use, with the exception of 
 * SMAllocMem, SMReAllocMem, and those routines that return memory but 
 * have a corresponding free API. (Example: SMSLListFree)
 *
 * @param pMem [in] pointer to memory to be freed.
 *
 * @return This function does not return any value.
 *
 * @sa SMAllocMem()
 * 
 * @sa SMReAllocMem()
 * 
 * @sa SMFreeMem()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMFreeGeneric(_IN void *pMem);



//---------------------------------------------------------------------
// UNICODE support function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * This function can be used to append a Unicode string to another 
 * Unicode string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strcat. The SMUCS2Strcat() function appends pSrc to pDest and 
 * terminates the resulting string with a null character. 
 *
 * No overflow checking is performed when strings are copied or 
 * appended. The behavior of SMUCS2Strcat is undefined if the source 
 * and destination strings overlap.
 *
 * @param pDest	[inout] pointer to a null-terminated destination 
 *		Unicode string.
 *
 * @param pSrc [in] pointer to a null-terminated source Unicode string.
 *
 * @return This function returns the pointer pDest. No other values 
 *		are returned.
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strcat_s) \
SMINTF ustring * _cdecl SMUCS2Strcat(_OUT ustring *pDest,
									 _IN const ustring *pSrc);

SMINTF ustring * _cdecl SMUCS2Strcat_s(_OUT ustring *pDest,
									   _IN u32 szDestInWords,
									   _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * This function can be used to find a character in a string. 
 * 
 * The SMUCS2Strchr() function finds the first occurrence of c in pStr, 
 * or it returns NULL if c is not found. The null-terminating character 
 * is included in the search.
 * 
 * @param pStr [in] pointer to a Unicode string.
 * 
 * @param c	[in] character to find in the string pStr.
 * 
 * @return This function returns a pointer to the first occurrence of 
 *		c in pStr and NULL pointer if character c is not found.
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMUCS2Strchr(_IN const ustring *pStr,
									 _IN ustring c);


//---------------------------------------------------------------------
/** 
 * This function can be used to copy a Unicode string to a given buffer. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strcpy. The SMUCS2Strcpy() function copies pSrc, including the 
 * terminating null character, to the location specified by pDest. 
 * 
 * No overflow checking is performed when string is copied. 
 *
 * @param pDest	[out] pointer to a buffer where the string needs to be 
 *		copied.
 * @param pSrc [in] pointer to a null-terminated Unicode string which 
 *		is to be copied.
 *
 * @return This function returns the pointer pDest. No other values are 
 *		returned.
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strcpy_s) \
SMINTF ustring * _cdecl SMUCS2Strcpy(_OUT ustring *pDest,
									 _IN const ustring *pSrc);

SMINTF ustring * _cdecl SMUCS2Strcpy_s(_OUT ustring *pDest,
									   _IN u32 szDestInWords,
									   _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * This function can be used to find the first substring in a given 
 * string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strcspn. The SMUCS2Strcspn() function returns the index of the first 
 * occurrence of a character in pStr that belongs to the set of 
 * characters in pStrCharSet. Terminating null characters are included 
 * in the search.
 * 
 * @param pStr [in] pointer to a null terminated string to search.
 *
 * @param pStrCharSet [in] pointer to a null-terminated character set.
 *
 * @return Return an integer value specifying the length of the 
 *		substring in pStr that consists entirely of characters in 
 *		pStrCharSet. If pStr begins with a character not in pStrCharSet, 
 *		the function returns 0. 
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMUCS2Strcspn(_IN const ustring *pStr,
								_IN const ustring *pStrCharSet);


//---------------------------------------------------------------------
/** 
 * This function can be used Get the length of a Unicode string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strlen. The SMUCS2Strlen() function returns the number of characters 
 * in pStr, not including the terminating null character. 
 *
 * @param pStr [in] pointer to a null terminated Unicode string.
 *
 * @return 	Return the number of characters in pStr, excluding the 
 *		terminating null character.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMUCS2Strlen(_IN const ustring *pStr);


//---------------------------------------------------------------------
/** 
 * This function can be used to append a Unicode string to another 
 * Unicode string. 
 *
 * This is Unicode equivalent function for its ASCII counter part 
 * strncat. The SMUCS2Strncat function appends, at most, the first 
 * count characters of pSrc to pDest. The initial character of pSrc 
 * overwrites the terminating null character of pDest. 
 * 
 * If a null character appears in pSrc before count characters are 
 * appended, SMUCS2Strncat() appends all characters from pSrc, up to 
 * the null character. 
 * 
 * If count is greater than the length of pSrc, the length of pSrc is 
 * used in place of count. The resulting string is terminated with a 
 * null character. 
 * 
 * If copying takes place between strings that overlap, the behavior is 
 * undefined.
 *
 * @param pDest	[inout] pointer to a null-terminated destination 
 *		Unicode string.
 *
 * @param pSrc [in] pointer to a null-terminated source Unicode string.
 *
 * @param count	[in] Number of characters to append.
 *
 * @return This function returns pDest. No other values are returned.
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strncat_s) \
SMINTF ustring * _cdecl SMUCS2Strncat(_OUT ustring *pDest,
									  _IN const ustring *pSrc,
									  _INOUT u32 count);

SMINTF ustring * _cdecl SMUCS2Strncat_s(_OUT ustring *pDest,
										_IN u32 szDestInWords,
										_IN const ustring *pSrc,
										_INOUT u32 count);


//---------------------------------------------------------------------
/** 
 * This function can be used to Copy characters of one Unicode string 
 * to another. 
 *
 * This is Unicode equivalent function for its ASCII counter part 
 * strncpy. The SMUCS2Strncpy() function copies the initial count 
 * characters of pSrc to pDest and returns pDest. 
 *
 * If count is less than or equal to the length of pSrc, a null 
 * character is not appended automatically to the copied string. 
 *
 * If count is greater than the length of pSrc, the destination string 
 * is padded with null characters up to length count. 
 *
 * The behavior of SMUCS2Strncpy is undefined if the source and 
 * destination strings overlap.
 *
 * @param pDest	[out] pointer to a buffer where the string needs to be 
 *		copied.
 *
 * @param pSrc [in] pointer to a null-terminated Unicode string which 
 *		is to be copied.
 *
 * @param count	[in] number of characters to be copied.
 *
 * @return This function returns pDest. No other values are returned.
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strncpy_s) \
SMINTF ustring * _cdecl SMUCS2Strncpy(_OUT ustring *pDest,
									  _IN const ustring *pSrc,
									  _INOUT u32 count);

SMINTF ustring * _cdecl SMUCS2Strncpy_s(_OUT ustring *pDest,
										_IN u32 szDestInWords,
										_IN const ustring *pSrc,
										_INOUT u32 count);

//---------------------------------------------------------------------
/** 
 * This function can be used to find the first substring in a given 
 * string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strspn. The SMUCS2Strspn() function returns the index of the first 
 * occurrence of a character in pStr that does not belongs to the set 
 * of characters in pStrCharSet. Terminating null characters are not 
 * included in the search. 
 *
 * @param pStr [in] pointer to a null terminated string to search.
 *
 * @param pStrCharSet [in] pointer to a null-terminated character set.
 *
 * @return Return an integer value specifying the length of the 
 *		substring in pStr that consists entirely of characters in 
 *		pStrCharSet. If pStr begins with a character not in pStrCharSet, 
 *		the function returns 0. 
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMUCS2Strspn(_IN const ustring *pStr,
							   _IN const ustring *pStrCharSet);


//---------------------------------------------------------------------
/** 
 * This function can be used to find a substring. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strstr. The SMUCS2Strstr() function returns a pointer to the first 
 * occurrence of pStrCharSet in pStr. 
 *
 * The search does not include terminating null characters.
 *
 * @param pSrc [in] pointer to a null-terminated string to search.
 * 
 * @param pStrCharSet [in] pointer to a null-terminated string to search for.
 *
 * @return Return a pointer to the first occurrence of pStrCharSet in 
 *		pStr, or NULL if pStrCharSet does not appear in string. If 
 *		pStrCharSet points to a string of zero length, the function 
 *		returns pStr.
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMUCS2Strstr(_IN const ustring *pSrc,
									 _IN const ustring *pStrCharSet);


//---------------------------------------------------------------------
/** 
 * This function can be used to find the next token in a string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strtok. The SMUCS2Strtok() function finds the next token in pToken. 
 * The set of characters in pDelim specifies possible delimiters of 
 * the token to be found in pToken on the current call. 
 *
 * On the first call to SMUCS2Strtok(), the function skips leading 
 * delimiters and returns a pointer to the first token in pToken, 
 * terminating the token with a null character. More tokens can be 
 * broken out of the remainder of pToken by a series of calls to 
 * SMUCS2Strtok(). Each call to SMUCS2Strtok() modifies pToken by 
 * inserting a null character after the token returned by that call. 
 *
 * To read the next token from pToken, call SMUCS2Strtok() with a NULL 
 * value for the pToken argument. The NULL pToken argument causes 
 * SMUCS2Strtok() to search for the next token in the modified pToken. 
 * 
 * The pDelim argument can take any value from one call to the next so 
 * that the set of delimiters may vary.
 *
 * @param pToken [inout] pointer to string containing token(s).
 * 
 * @param pDelim [in] pointer to a set of delimiter characters.
 *
 * @return This function returns a pointer to the next token found in 
 *		pToken. It returns NULL when no more tokens are found. Each 
 *		call modifies pToken by substituting a NULL character for each 
 *		delimiter that is encountered.
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strtok_s) \
SMINTF ustring * _cdecl SMUCS2Strtok(_IN ustring *pToken,
									 _IN const ustring *pDelim);

SMINTF ustring * _cdecl SMUCS2Strtok_s(_IN ustring *pToken,
									   _IN const ustring *pDelim,
									   _IN ustring **ppTokenContext);

//---------------------------------------------------------------------
/** 
 * This function sets the first "count" bytes of the memory area 
 * pointed to by pSrc with the constant byte 'c'. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * memset(). The terminating null character remains in place.
 * 
 * @param pSrc [in] pointer to a null-terminated string to set to 
 *		character 'c'.
 * 
 * @param c [in] constant Unicode character.
 *
 * @param count [in] number of bytes of characters to be set to 'c'.
 * 
 * @return This function returns a pointer to the altered string. 
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strnset_s) \
SMINTF ustring * _cdecl SMUCS2Strnset(_IN ustring *pSrc,
									  _IN ustring c,
									  _INOUT u32 count);

SMINTF ustring * _cdecl SMUCS2Strnset_s(_IN ustring *pSrc,
										_IN u32 szSrcInWords,
										_IN ustring c,
										_INOUT u32 count);


//---------------------------------------------------------------------
/** 
 * This function sets the memory area pointed to by pSrc with the 
 * constant byte 'c'. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * memset. The terminating null character remains in place.
 *
 * @param pSrc [in] pointer to a null-terminated string to set to 
 *		character 'c'.
 * 
 * @param c [in] constant Unicode character.
 *
 * @return This function returns a pointer to the altered string. 
 */
//---------------------------------------------------------------------

_SUPT_API_DEPRECATE(SMUCS2Strset_s) \
SMINTF ustring * _cdecl SMUCS2Strset(_IN ustring *pSrc,
									 _IN ustring c);

SMINTF ustring * _cdecl SMUCS2Strset_s(_IN ustring *pSrc,
									 _IN u32 szSrcInWords,
									 _IN ustring c);


//---------------------------------------------------------------------
/** 
 * This function can be used reverse characters of a string. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strrev. The SMUCS2Strrev() function reverses the order of the 
 * characters in string. 
 * 
 * The terminating null character remains in place.
 * 
 * @param pStrToBeRev [in] pointer to a null-terminated string to reverse.
 * 
 * @return This function returns a pointer to the altered string. 
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMUCS2Strrev(_IN ustring *pStrToBeRev);


//---------------------------------------------------------------------
/** 
 * This function can be used to compare strings. 
 * 
 * This is Unicode equivalent function for its ASCII counter part 
 * strcmp. The SMUCS2Strcmp() function compares pStr1 and pStr2 
 * lexicographically and returns a value indicating their relationship.
 *
 * @param pStr1 [in] null terminated strings to compare.
 * 
 * @param pStr2 [in] null terminated strings to compare.
 *
 * @return 	The return value for each of these functions indicates the 
 *		lexicographic relation of pStr1 to pStr2:\n
 *		Value:		Meaning \n
 *		< 0			pStr1 less than pStr2 \n
 *		0			pStr 1 identical to pStr 2 \n
 *		> 0			pStr 1 greater than pStr 2 \n
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUCS2Strcmp(_IN const ustring *pStr1,
							   _IN const ustring *pStr2);



//---------------------------------------------------------------------
// localization support function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * This function retrieves the language identifier of the system locale. 
 *
 * The function SMGetLocalLanguageID() returns the local language 
 * identifier of the system.
 * 
 * @return The return value is the language identifier of the system 
 *		locale.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGetLocalLanguageID(void);


//---------------------------------------------------------------------
/** 
 * This function retrieves the path name given the file name and the 
 * application name.
 *
 * @param pRootPath	[in] Product installation root path. If it is NULL, 
 *		the OS default root path is used.
 * 
 * @param pApp [in] The Application name string. Cannot be NULL.
 * 
 * @param pLocaleStr [in] The locale name string. If NULL, the OS 
 *		default locale name is used.
 * 
 * @param pType	[in] The type of component display or registration. 
 *		This can be NULL.
 * 
 * @param pFileName	[in] the file name string to be looked up for. 
 *		This cannot be NULL.
 *
 * @return The return value is the absolute path name for the locale 
 *		file, if found.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMGetLocalizedPathFileName(_IN const astring *pRootPath, 
												   _IN const astring *pApp, 
												   _IN const astring *pLocaleStr, 
												   _IN const astring *pType, 
												   _IN const astring *pFileName);


//---------------------------------------------------------------------
/** 
 * Converts a given UCS2 String to a UTF8 string.
 * 
 * This function converts the given UCS-2 encoded string to a UTF-8 
 * encoded string. 
 * 
 * If this function is called with the destination buffer as null, then 
 * it will return the size needed in the location pointed by pDestSize. 
 * But if the function is called with insufficient buffer then it will 
 * return SM_DATA_OVERRUN status. 
 * 
 * @param pDest [out] Pointer to a buffer to store the converted UTF-8 
 *		string.
 * 
 * @param pDestSize [inout] pointer to a u32 to store the size of the 
 *		destination buffer.
 * 
 * @param pSrc [in] pointer to UCS-2 string that is to be converted.
 * 
 * @return On success returns SM_STATUS_SUCCESS.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 * @sa SMUTF8StrToUCS2Str()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUCS2StrToUTF8Str(_OUT astring *pDest, 
									 _INOUT u32 *pDestSize, 
									 _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * Converts a given UTF8 String to a UCS2 string.
 * 
 * This function converts the given UTF-8 encoded string to a UCS-2 
 * encoded string. 
 * 
 * If this function is called with the destination buffer as null, then 
 * it will return the size needed in the location pointed by pDestSize. 
 * But if the function is called with insufficient buffer then it will 
 * return SM_DATA_OVERRUN status. 
 *
 * @param pDest [out] Pointer to a buffer to store the converted UCS-2 
 *		string.
 * 
 * @param pDestSize [inout] pointer to a u32 to store the size in bytes 
 *		of the destination buffer on input and the size in bytes of the 
 *		UCS-2 encoded string on output.
 *
 * @param pSrc [in] pointer to UTF-8 string that is to be converted.
 *
 * @return On success returns SM_STATUS_SUCCESS.
 * 
 * @return On error returns the status code defined in dcsmstatus.h.
 *
 * @sa SMUCS2StrToUTF8Str()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUTF8StrToUCS2Str(_OUT ustring *pDest, 
									 _INOUT u32 *pDestSize, 
									 _IN const astring *pSrc);


//---------------------------------------------------------------------
/** 
 * Converts a given UCS2 String to XML escape format.
 * 
 * This function converts the given UCS-2 encoded string to a UTF-8 
 * encoded string. 
 * 
 * If this function is called with the destination buffer as null, then 
 * it will return the size needed in the location pointed by pDestSize. 
 * But if the function is called with insufficient buffer then it will 
 * return SM_DATA_OVERRUN status. 
 * 
 * @param pDest [out] Pointer to a buffer to store the converted string 
 *		in XML escape format.
 * 
 * @param pDestSize [inout] pointer to a u32 to store the size of the 
 *		destination buffer.
 * 
 * @param pSrc [in] pointer to UCS-2 string that is to be converted.
 * 
 * @return On success returns SM_STATUS_SUCCESS.
 *
 * @return On error returns the status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUCS2ToEscape(_OUT astring *pDest,
								 _INOUT u32 *pDestSize,
								 _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * Allocate the memory to store a Unicode string.
 *
 * This function allocates the number of bytes specified in the 
 * initialSize parameter type casts it to an ustring pointer and placed 
 * in the pStr parameter of passed SMSSUCS2Str structure and returns 
 * that pointer. It also initializes the strBufSize as initialSize and 
 * strCurLen to 0. If the function failed to allocate the memory then 
 * it will return a NULL pointer.
 *
 * This allocated structure can be used either in SMSSUCS2StrCatUCS2Str()
 * or SMSSUCS2StrCatUTF8Str().
 *
 * Use SMSSUCS2StrFree() function to free this buffer. 
 *
 * @param pSS [out] pointer to a SMSSUCS2Str structure.
 *
 * @param initialSize [in] size to be allocated.
 *
 * @return If the function succeeds it will return the pointer to the 
 * buffer allocated. On error NULL pointer is returned.
 * 
 * @sa SMSSUCS2StrFree()
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMSSUCS2StrAlloc(_IN SMSSUCS2Str *pSS,
										 _IN u32 initialSize);


//---------------------------------------------------------------------
/** 
 * Allocate the memory to store an ASCII string.
 *
 * This function allocates the number of bytes specified in the 
 * initialSize parameter type casts it to an astring pointer and placed 
 * in the pStr parameter of passed SMSSUTF8Str structure and returns 
 * that pointer. It also initializes the strBufSize as initialSize and 
 * strCurLen to 0. If the function failed to allocate the memory then 
 * it will return a NULL pointer. This allocated structure can be used 
 * either in SMSSUTF8StrCatUCS2Str or SMSSUTF8StrCatUTF8Str.
 *
 * Use SMSSUTF8StrFree() function to free this buffer.
 *
 * @param pSS [out] pointer to a SMSSUTF8Str structure.
 *
 * @param initialSize [in] size to be allocated.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer allocated. On error NULL pointer is returned.
 *
 * @sa SMSSUTF8StrFree()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMSSUTF8StrAlloc(_IN SMSSUTF8Str *pSS,
										 _IN u32 initialSize);


//---------------------------------------------------------------------
/** 
 * Append one Unicode string to another Unicode string.
 * 
 * This function appends one Unicode string to another. The parameter 
 * pSSDest contains a Unicode string which will be appended with the 
 * one that is pointed to by pSrc. If the pSSDest do not have enough 
 * memory to put the final string then it will allocate a new buffer 
 * and frees the old one to accommodate the final string. 
 *
 * Using this function has two advantages one, performance and the other 
 * is ease of use i.e. there is no need to worry about the size of the 
 * destination buffer. 
 *
 * This function returns a null pointer only either the pSSDest has 
 * null pointer in it or memory allocation is failed.
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to Unicode string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated Unicode string. NULL 
 *		pointer will be returned for failure cases.
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMSSUCS2StrCatUCS2Str(_IN SMSSUCS2Str *pSSDest,
											  _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * Append one Unicode string to another Unicode string, up to a number
 * of characters.
 * 
 * This function is similar to function SMSSUCS2StrCatUCS2Str(), but
 * only append a number of characters. 
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to Unicode string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @param count [in] number of characters to append.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated Unicode string. NULL 
 *		pointer will be returned for failure cases.
 *
 * @sa SMSSUCS2StrCatUCS2Str()
 */

//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMSSUCS2StrNCatUCS2Str(_IN SMSSUCS2Str *pSSDest,
											   _IN const ustring *pSrc,
											   _IN u32 count);


//---------------------------------------------------------------------
/** 
 * This function can be used to append an ASCII string to a Unicode 
 * string by converting the ASCII string to Unicode string.
 *
 * The parameter pSSDest contains a Unicode string which will be 
 * appended with the one that is pointed to by pSrc. 
 *
 * This function returns a null pointer for the following conditions:\n 
 * - pSSDest has null pointer in it; \n
 * - pSrc is a null pointer; \n
 * - conversion from ASCII to Unicode is failed; \n
 * - memory allocation is failed.
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to ASCII string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated Unicode string. NULL 
 *		pointer will be returned for failure cases.
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMSSUCS2StrCatUTF8Str(_IN SMSSUCS2Str *pSSDest,
											  _IN const astring *pSrc);


//---------------------------------------------------------------------
/** 
 * This function can be used to append an ASCII string to a Unicode 
 * string, up to a certain number of characters, by converting the 
 * ASCII string to Unicode string.
 *
 * This function is similar to function SMSSUCS2StrCatUTF8Str(), but
 * only append a number of characters. 
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to ASCII string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @param count [in] number of characters to append.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated Unicode string. NULL 
 *		pointer will be returned for failure cases.
 *
 * @sa SMSSUCS2StrCatUTF8Str()
 */
//---------------------------------------------------------------------

SMINTF ustring * _cdecl SMSSUCS2StrNCatUTF8Str(_IN SMSSUCS2Str *pSSDest,
											   _IN const astring *pSrc,
											   _IN u32 count);


//---------------------------------------------------------------------
/** 
 * This function can be used to append one ASCII string to another 
 * ASCII string.
 * 
 * This function appends one ASCII string to another. The parameter 
 * pSSDest contains an ASCII string which will be appended with the 
 * one that is pointed to by pSrc. If the pSSDest do not have enough 
 * memory to put the final string then it will allocate a new buffer 
 * and frees the old one to accommodate the final string. Using this 
 * function has two advantages one, performance and the other is ease 
 * of use i.e. there is no need to worry about the size of the 
 * destination buffer. 
 *
 * This function returns a null pointer only either the pSSDest has 
 * null pointer in it or memory allocation is failed.
 * 
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to ASCII string that needs to be appended 
 *		to the string the in PSSDest structure.
 * 
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated ASCII string. NULL 
 *		pointer will be returned for failure cases.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMSSUTF8StrCatUTF8Str(_IN SMSSUTF8Str *pSSDest,
											  _IN const astring *pSrc);


//---------------------------------------------------------------------
/** 
 * This function can be used to append one ASCII string to another 
 * ASCII string, up to a certain number of characters.
 * 
 * This function is similar to function SMSSUTF8StrCatUTF8Str(), but
 * only append a certain number of characters.
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to ASCII string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @param count [in] number of characters to append.
 * 
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated ASCII string. NULL 
 *		pointer will be returned for failure cases.
 *
 * @sa SMSSUTF8StrCatUTF8Str()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMSSUTF8StrNCatUTF8Str(_IN SMSSUTF8Str *pSSDest,
											   _IN const astring *pSrc,
											   _IN u32 count);


//---------------------------------------------------------------------
/** 
 * This function can be used to append a Unicode string to an ASCII 
 * string by converting the Unicode string to ASCII string.
 * 
 * The parameter pSSDest contains an ASCII string which will be appended 
 * with the one that is pointed to by pSrc. 
 *
 * This function returns a null pointer for the following conditions: \n
 * - pSSDest has null pointer in it; \n
 * - pSrc is a null pointer; \n
 * - conversion from ASCII to Unicode is failed; \n
 * - memory allocation is failed.
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 *
 * @param pSrc [in] pointer to Unicode string that needs to be appended 
 *		to the string in the PSSDest structure.
 *
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated ASCII string. NULL 
 *		pointer will be returned for failure cases.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMSSUTF8StrCatUCS2Str(_IN SMSSUTF8Str *pSSDest,
											  _IN const ustring *pSrc);


//---------------------------------------------------------------------
/** 
 * This function can be used to append a Unicode string to an ASCII 
 * string, up to a certain number of characters, by converting the 
 * Unicode string to ASCII string.
 * 
 * This function is similar to function SMSSUTF8StrCatUCS2Str(), but
 * only append a certain number of characters.
 *
 * @param pSSDest [inout] pointer to a SMSSUTF8Str structure which will 
 *		be updated.
 * 
 * @param pSrc [in] pointer to Unicode string that needs to be appended 
 *		to the string the in PSSDest structure.
 *
 * @param count [in] number of characters to append.
 * 
 * @return If the function succeeds it will return the pointer to the 
 *		buffer which will point to concatenated ASCII string. NULL 
 *		pointer will be returned for failure cases.
 *
 * @sa SMSSUTF8StrCatUCS2Str()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMSSUTF8StrNCatUCS2Str(_IN SMSSUTF8Str *pSSDest,
											   _IN const ustring *pSrc,
											   _IN u32 count);


//---------------------------------------------------------------------
/** 
 * Free the memory allocated for a SMSSUCS2Str.
 *
 * This function can be used to free the memory that may have been 
 * allocated by any of the functions: SMSSUCS2StrAlloc(), 
 * SMSSUCS2StrCatUCS2Str(), or SMSSUCS2StrCatUTF8Str. 
 * 
 * It will free the memory allocated and reinitializes the parameters 
 * in the SMSSUCS2Str structure.
 * 
 * @param pSS [in] pointer to a SMSSUCS2Str structure .
 *
 * @return This function does not return any values.
 *
 * @sa SMSSUCS2StrAlloc()
 *
 * @sa SMSSUCS2StrAlloc()
 *
 * @sa SMSSUCS2StrAlloc()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSSUCS2StrFree(_IN SMSSUCS2Str *pSS);


//---------------------------------------------------------------------
/** 
 * Free the memory allocated for a SMSSUTF8Str.
 * 
 * This function can be used to free the memory that may have been 
 * allocated by any of the functions: SMSSUTF8StrAlloc(), 
 * SMSSUTF8StrCatUCS2Str(), or SMSSUTF8StrCatUTF8Str().
 * 
 * It will free the memory allocated and reinitializes the parameters 
 * in the SMSSUTF8Str structure.
 * 
 * @param pSS [in] pointer to a SMSSUTF8Str structure.
 *
 * @return This function does not return any values.
 * 
 * @sa SMSSUTF8StrAlloc()
 *
 * @sa SMSSUTF8StrCatUCS2Str()
 * 
 * @sa SMSSUTF8StrCatUTF8Str()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSSUTF8StrFree(_IN SMSSUTF8Str *pSS);


//---------------------------------------------------------------------
// OS Abstraction support for UTF8 manipulations
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/** 
 * Returns pointer to the input string without leading whitespace.
 *
 * @param pAstr [in] pointer to UTF8 (ASCII) string.
 *
 * @return the new UTF8 string without leading whitespace.
 *
 * @sa SMUTF8rtrim()
 *
 * @sa SMUTF8trim()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8ltrim(_IN astring *pAstr);


//---------------------------------------------------------------------
/** 
 * Returns pointer to the input string without trailing whitespace.
 *
 * @param pAstr [in] pointer to UTF8 (ASCII) string.
 *
 * @return the new UTF8 string without trailing whitespace.
 *
 * @sa SMUTF8ltrim()
 *
 * @sa SMUTF8trim()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8rtrim(_IN astring *pAstr);


//---------------------------------------------------------------------
/** 
 * Returns pointer to the input string without leading and trailing 
 * whitespace.
 *
 * @param pAstr [in] pointer to UTF8 (ASCII) string.
 *
 * @return the new UTF8 string without leading and trailing whitespace.
 *
 * @sa SMUTF8ltrim()
 *
 * @sa SMUTF8rtrim()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8trim(_IN astring *pAstr);


//---------------------------------------------------------------------
/** 
 * Converts all characters to upper case.
 *
 * @param pAStr	[in] pointer to a null-terminated string to be 
 *		converted to Upper Case.
 *
 * @return This function returns a pointer to the altered string.
 *
 * @sa SMUTF8StrLowerCase()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8StrUpperCase(_IN astring *pAStr);


//---------------------------------------------------------------------
/** 
 * Converts all characters to lower case.
 *
 * @param pAStr	[in] pointer to a null-terminated string to be 
 *		converted to Lower Case.
 *
 * @return This function returns a pointer to the altered string.
 *
 * @sa SMUTF8StrLowerCase()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8StrLowerCase(_IN astring *pAStr);


//---------------------------------------------------------------------
/** 
 * Reverses the order of characters in a string.
 *
 * @param pAStr	[in] pointer to a null-terminated string.
 *
 * @return This function returns a pointer to the altered string.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8Strrev(_IN astring *pAStr);


//---------------------------------------------------------------------
/** 
 * Duplicate string.
 * 
 * @note User should use SMFreeGeneric() to free returned memory.
 *
 * @param pAStr	[in] pointer to a null-terminated string.
 *
 * @return This function returns a pointer to the duplicate string.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8Strdup(_IN const astring *pAStr);


//---------------------------------------------------------------------
/** 
 * UTF8 string comparison, ignore case, up to a number of characters.
 *
 * The SMUTF8Strnicmp function lexicographically compares, at most, the 
 * first count characters of pStr1 and pStr2. The comparison is 
 * performed without regard to case.
 *
 * @param pStr1 [in] Null-terminated strings to compare.
 *
 * @param pStr2 [in] Null-terminated strings to compare.
 *
 * @param count [in] Number of characters to compare
 *
 * @return The return value for each of these functions indicates the 
 *		lexicographic relation of pStr1 to pStr2: \n
 *		Value		Meaning
 *		< 0			pStr1 less than pStr2
 *		0			pStr1 identical to pStr2
 *		> 0			pStr1 greater than pStr2
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUTF8Strnicmp(_IN const astring *pStr1,
								 _IN const astring *pStr2,
								 _IN u32 count);


//---------------------------------------------------------------------
/** 
 * Search for substring in UTF8 string using case insensitive search.
 *
 * The SMUTF8Strstri function searchs pSrcStr for pSubStr.
 * The search is performed without regard to case.
 *
 * @param pSrcStr [in] Null-terminated string to search.
 *
 * @param pSubStr [in] Null-terminated string to search for.
 *
 * @return The function returns a pointer to the beginning of the 
 *		substring in the source string or NULL if the substring is
 *		not found.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8Strstri(_IN const astring *pSrcStr,
									  _IN const astring *pSubStr);


//---------------------------------------------------------------------
/** 
 * This function converts the passed ASCII string to its type equivalent 
 * as determined by the vType parameter.
 *
 * @param pValStr [in] pointer to buffer containing ASCII string to be converted.
 *
 * @param pValBuf [out] pointer to return value buffer. 
 *		If NULL, the function simply tests for validity of the type of data passed. 
 *		(pVBufSize should be zero in this case.)
 *
 * @param pVBufSize	[inout] pointer to size in bytes of out buffer on input and 
 *		size in bytes of converted value on output.
 *		Should be large enough to hold the return data.
 * 
 * @param vType	[in] type of value data to be converted.
 *		Should be one of the values defined in dcsupdef.h.
 *		These vTypes can be used to convert passed ASCII strings to a value data type.\n\n
 *		The following table contains the types that are supported by this API.\n
 *		(Symbol:						Value:		Meaning)\n
 *		SM_XLT_VTYPE_ASTRING_TO_BOOLN:	0:			Boolean\n
 *		SM_XLT_VTYPE_ASTRING_TO_s8:		1:			Convert to s8\n
 *		SM_XLT_VTYPE_ASTRING_TO_s16:	2:			Convert to s16\n
 *		SM_XLT_VTYPE_ASTRING_TO_s32:	3:			Convert to s32\n
 *		SM_XLT_VTYPE_ASTRING_TO_s64:	4:			Convert to s64\n
 *		SM_XLT_VTYPE_ASTRING_TO_u8:		5:			Convert to u8\n
 *		SM_XLT_VTYPE_ASTRING_TO_u16:	6:			Convert to u16\n
 *		SM_XLT_VTYPE_ASTRING_TO_u32:	7:			Convert to u32\n
 *		SM_XLT_VTYPE_ASTRING_TO_u64:	8:			Convert to u64\n
 *		SM_XLT_VTYPE_ASTRING_TO_DOUBLE:	9:			Convert to double\n
 *		SM_XLT_VTYPE_ASTRING_TO_LANGID:	10:			Convert to language ID\n\n
 *
 * @return SM_STATUS_SUCCESS is returned upon successful completion.
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if any of the required fields are NULL.
 *
 * @return SM_STATUS_DATA_OVERRUN is returned if the buffer size is less than the required size.
 *		The return value contains the minimum size required to copy the data over.
 *
 * @return SM_STATUS_VALUE_TOO_LARGE is returned if the return value is larger than expected.
 *
 * @return SM_STATUS_VALUE_TOO_SMALL is returned if the return value is smaller than expected.
 *
 * @return SM_STATUS_CMD_BAD if vType is not one of the types as defined in dcsupdef.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXLTUTF8ToTypeValue(_IN const astring *pValStr,
									   _OUT void *pValBuf,
									   _INOUT u32 *pVBufSize,
									   _IN u32 vType);


//---------------------------------------------------------------------
/**
 * This function converts the passed value type as sepcified by the 
 * vType parameter to it's equivalent ASCII string.
 *
 * @param pValBuf [in] pointer to value buffer.
 *
 * @param valBufSize [in] pointer to size of out buffer value.
 *
 * @param pValStr [out] pointer to return buffer containing converted string.
 * 
 * @param pValStrSize [inout] pointer to size of out buffer value. 
 *		Should be large enough to hold the return data.
 *
 * @param vType	[in] type of value data to be converted.
 *		Should be one of the values as defined in dcsupdef.h.
 *		These vTypes can be used to convert passed data type value to ASCII string.\n\n
 *		The following table contains the types that are supported by this API.\n
 *		(Symbol: Value: Meaning)
 *		SM_XLT_VTYPE_BOOLN_TO_ASTRING:	0:	Boolean
 *		SM_XLT_VTYPE_s8_TO_ASTRING:		1:	Convert s8
 *		SM_XLT_VTYPE_s16_TO_ASTRING:	2:	Convert s16
 *		SM_XLT_VTYPE_s32_TO_ASTRING:	3:	Convert s32
 *		SM_XLT_VTYPE_s64_TO_ASTRING:	4:	Convert s64
 *		SM_XLT_VTYPE_u8_TO_ASTRING:		5:	Convert u8
 *		SM_XLT_VTYPE_u16_TO_ASTRING:	6:	Convert u16
 *		SM_XLT_VTYPE_u32_TO_ASTRING:	7:	Convert u32
 *		SM_XLT_VTYPE_u64_TO_ASTRING:	8:	Convert u64
 *		SM_XLT_VTYPE_DOUBLE_TO_ASTRING:	9:	Convert double
 *		SM_XLT_VTYPE_LANGID_TO_ASTRING:	10:	Convert language id to locale String
 *		SM_XLT_VTYPE_TIME_T_TO_ASTRING:	11:	Convert time_t value to ASCII\n
 *
 * @return SM_STATUS_SUCCESS is returned upon successful completion.
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if any of the required fields are NULL.
 *
 * @return SM_STATUS_DATA_OVERRUN is returned if the buffer size is less than the required size.\n 
 *		The return value contains the minimum size required to copy the data over.
 *
 * @return SM_STATUS_CMD_BAD if vType is not one of the types as defined in dcsupdef.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXLTTypeValueToUTF8(_IN const void *pValBuf,
									   _IN u32 valBufSize,
									   _OUT astring *pValStr,
									   _INOUT u32 *pValStrSize,
									   _IN u32 vType);


//---------------------------------------------------------------------
/**
 * This function converts an X-separated-value (XSV) string to
 * a Y-separated-value (YSV) string.
 *
 * @param pCSVString [in] Pointer to X-separated-value string.
 *
 * @param valSeparatorCharIn [in] X separator character (SM_VALSEP_XXX).
 *
 * @param valSeparatorCharOut [in] Y separator character (SM_VALSEP_XXX).
 *
 * @param pCount [out] Pointer to variable to receive number of Y-separated
 *		values in returned string.  This parameter is optional and may be NULL.
 * 
 * @param pSize [out] Pointer to variable to receive size in bytes
 *		of returned string.  This parameter is optional and may be NULL.
 *
 * @return Pointer to Y-separated-value string on success or NULL on error.
 *
 * @note Use SMFreeGeneric() to free returned memory.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMUTF8ConvertXSVToYSV(_IN const astring *pCSVString,
											  _IN astring valSeparatorCharIn,
											  _IN astring valSeparatorCharOut,
											  _OUT s32 *pCount,
											  _OUT u32 *pSize);


//---------------------------------------------------------------------
/**
 * This function returns a UTF8 string from the given path file.
 * 
 * @param stringID [in] id of the string to be converted.
 *
 * @param pLanguageId [in] pointer to the language id.
 *
 * @param pPathFileName [in] pointer to the ASCII path filename.
 * 
 * @param pUTF8Dest [out] pointer to the destination buffer.
 *
 * @param pUTF8DestSize [inout] pointer to the size of the number of bytes 
 *		written to the destination buffer.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS.
 * 
 * @return If the function fails, the return value is 
 *		SM_STATUS_UNSUCCESSFUL.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGetUTF8StrFromIDPathFile(_IN const u32 stringID, 
											 _INOUT u32 *pLanguageId, 
											 _IN const astring *pPathFileName, 
											 _OUT astring *pUTF8Dest, 
											 _INOUT u32 *pUTF8DestSize);


//---------------------------------------------------------------------
/** SMReadPCIDevFile
 * Reads the value from a pcidevs file given vendor and device IDs.
 *
 * This function can be used to read a value from a specified pcidevs file, 
 * given file name with path, vendor ID, Device ID, Subvendor ID and subdevice ID. 
 * 
 * If the function could not find the string in the specified Vendor/device id 
 * in the file, then it will not return SM_STATUS_SUCCESS. 
 *
 * @param vendorID [in] specifies the vendor ID to lookup for the string in the pcidevs file.
 *
 * @param deviceID [in] specifies the device ID for the corresponding vendor ID to lookup for the string in the pcidevs file.
 *
 * @param subSystemVendorID [in] specifies the sub system vendor ID to lookup for the string in the pcidevs file.
 *
 * @param subSystemDeviceID [in] specifies the sub system device ID to lookup for the string in the pcidevs file.
 *
 * @param pPathFileName [in] pcidevs path and file name to read from.
 *
 * @param pManufacturer [out] pointer to a buffer contains manufacture name string value
 *		in case of success to locate vendor/device ID.
 *
 * @param pMfrBufSize	[in/out] size of manufacture string data buffer.
 * 
 * @param pDeviceDesc [out] pointer to a buffer contains device description string value
 *		in case of success to locate vendor/device ID.
 *
 * @param pDevDescBufSize [in/out] size of manufacture string data buffer.
 *
 * @return Upon success return SM_STATUS_SUCCESS.
 *		On error returns the status code defined in dcsmstatus.h
 */ 
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMReadPCIDevFile(_IN u16 vendorID, 
								   _IN u16 deviceID, 
								   _IN u16 subSystemVendorID, 
								   _IN u16 subSystemDeviceID, 
								   _IN astring *pPathFileName, 
								   _OUT astring *pManufacturer, 
								   _INOUT u32 *pMfrBufSize, 
								   _OUT astring *pDeviceDesc, 
								   _INOUT u32 *pDevDescBufSize);


//---------------------------------------------------------------------
// INI file support function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Reads the value from a INI file given section and key names.
 *
 * This function can be used to read a value from an INI file, 
 * given file name with path, section name and the key value. 
 *
 * The value can be of any type. 
 * For example it could be an u32 value, or could be an array of bytes. 
 * 
 * If the function could not find the key in the specified section 
 * in the file, then it will copy the default value to the buffer provided. 
 * The canBlock parameter can be used to specify whether to block or 
 * return immediately if the file is locked by some other thread/process. 
 * 
 * This function can also be used to get all the Keys for a section by 
 * passing the pKey parameter as NULL. Similarly if the Section name is 
 * NULL then the function will return the all the sections in the file. 
 *
 * To get the all keys/sections the vType parameter should be INI_ASTRING.
 *
 * @param pSection [in] Name of the section in the file. 
 *		If NULL supplied then all section names are read into the supplied 
 *		pValue buffer. Each section name is '\\0' character delimited, 
 *		with the last section name terminated with an additional '\\0'.
 *
 * @param pKey [in] Key to be searched in the section. 
 *		If NULL supplied then all key names are read into the supplied 
 *		pValue buffer.  Each key name is '\\0' character delimited,
 *		with the last key name terminated with an additional '\\0'.
 *
 * @param vType [in] Value type to be read. Type is INI_ASTRING if NULL 
 *		supplied to pSection or pKey.
 *
 * @param pValue [out] pointer to value data buffer to be read into.
 *
 * @param pvSize [inout] size of value data buffer.
 *
 * @param pDefault [in] pointer to a buffer contains default value to 
 *		return in case of failure to locate key.
 *
 * @param dSize	[in] size of default value data buffer.
 * 
 * @param pPathFileName [in] Path and file name to read.
 *
 * @param canBlock [in] Boolean value to enable or disable blocking 
 *		in the function.
 * 
 * @return Upon success return SM_STATUS_SUCCESS. 
 *		On error returns the status code defined in dcsmstatus.h.
 * 
 * @sa SMReadINISectionKeyUTF8Value, SMReadINIEnums32Value
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMReadINIPathFileValue(
				_IN const astring *pSection, // Section Name
				_IN const astring *pKey, // Key
				_IN u32 vType, // type of value data to be written INI_XXX
				_OUT void *pValue, // pointer to value data buffer to be read into
				_INOUT u32 *pvSize, // size of value data buffer
				_IN const void *pDefault, // default value to return incase of failure to locate key
				_IN u32 dSize, // size of default value data buffer
				_IN const astring *pPathFileName, // Path and File Name
				_IN booln canBlock // True (block on completion), FALSE (don't block)
				);

//---------------------------------------------------------------------
/**
 * Writes a value to an INI file.
 *
 * This function can be used to write a value to an INI file, 
 * given file name with path, section name and key name. 
 * The value can be of any type; for example, it could be an u32 value 
 * or could be an array of bytes.
 *
 * The canBlock parameter can be used to specify whether to block or 
 * return immediately if the file is locked by some other thread/process.
 *
 * @param pSection [in] Name of the section in the file.
 *
 * @param pKey [in] Name of the key in the section.
 *
 * @param vType	[in] Type of value to be written (INI_XXX).
 *
 * @param pValue [in] Pointer to value data buffer to be written from.
 *
 * @param vSize [in] Size of value in value data buffer.
 *
 * @param pPathFileName [in] Path and file name of INI file.
 *
 * @param canBlock [in] Boolean value to enable or disable blocking 
 *		in the function.
 * 
 * @return On success returns SM_STATUS_SUCCESS, 
 *		on error returns a status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMWriteINIPathFileValue(
				_IN const astring *pSection, // Section Name
				_IN const astring *pKey, // Key
				_IN u32 vType, // type of value data to be written INI_XXX
				_IN const void *pValue, // pointer to value data buffer to be written from
				_IN u32 vSize, // size of value data buffer
				_IN const astring *pPathFileName, // Path and File Name
				_IN booln canBlock // True (block on completion), FALSE (don't block)
				);

//---------------------------------------------------------------------
/**
 * Reads a list of sections, a list of keys or a string value
 * from an INI file.
 *
 * This function can be used to read a list of sections, a list of keys 
 * in a section or a string value associated with a key in an INI file, 
 * given file name with path, section name and key name. 
 *
 * If pSection and pKey are both NULL, a list of section names in the
 * file is returned.
 *
 * If pSection is specified and pKey is NULL, a list of key names in the
 * section is returned.
 *
 * If pSection and pKey are specified, a pointer to a string value is
 * returned.
 *
 * If an error is encountered, NULL is returned.
 *
 * If a list of section or key names is returned, the list returned is a
 * NULL-separated-value (NSV) list that is double NULL terminated.
 *
 * The canBlock parameter can be used to specify whether to block or 
 * return immediately if the file is locked by some other thread/process.
 *
 * @note Use SMFreeGeneric to release returned memory.
 *
 * @param pSection [in] Name of the section in the file.
 *
 * @param pKey [in] Name of the key in the section.
 *
 * @param pDefaultValue [in] Pointer to buffer that contains default
 *		value to return in case of failure to locate key.
 *
 * @param pValueSize [out] Size of value returned.
 *
 * @param pPathFileName [in] Path and file name of INI file.
 *
 * @param canBlock [in] Boolean value to enable or disable blocking 
 *		in the function.
 * 
 * @return
 *		1) List of sections if pSection and pKey are both NULL. \n
 *		2) List of keys in section if pSection is specified and pKey is NULL. \n
 *		3) Pointer to a string value if pSection and pKey are specified. \n
 *		4) NULL if an error is encountered.
 * 
 * @sa SMReadINIPathFileValue, SMReadINIEnums32Value
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMReadINISectionKeyUTF8Value(
				_IN const astring *pSection,
				_IN const astring *pKey,
				_IN const astring *pDefaultValue,
				_OUT u32 *pValueSize,
				_IN const astring *pPathFileName,
				_IN booln canBlock
				);

//---------------------------------------------------------------------
/** 
 * Reads value under INI section and key, compares the string in the enum
 * map provided and returns the corresponding enum value.  
 * 
 * The enum map must have the first element as the invalid value and the 
 * second element as the default value. Set isBitMask if the enum is a 
 * bit mask type.
 * 
 * The returned value is the "OR" of the read value in the INI. The value 
 * must be in a comma-separated format. 
 * 
 * On error, SM_ENUMs32_ERROR_VAL is returned.
 * 
 * @return Returns default enum value if INI value does not exist or 
 *		invalid enum value if INI value does not match any enum string 
 *		in map or value 0 if no match is found for bitmaps.
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMReadINIEnums32Value(
				_IN const astring *pSection,
				_IN const astring *pKey,
				_IN const SMEnums32UTF8Map *pEnumMap,
				_IN s32 enumMapCount,
				_IN booln isBitMask,
				_IN const astring *pPathFileName,
				_IN booln canBlock
				);



//---------------------------------------------------------------------
// File Property routines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Allocate memory for properties file.
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMFilePropAlloc(_IN const astring *pPathFileName);


//---------------------------------------------------------------------
/**
 * Modified the properties file structure.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMFilePropIsModified(_INOUT void *pFileProp);


//---------------------------------------------------------------------
/**
 * De-allocate the momery allocated by SMFilePropAlloc().
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMFilePropFree(_IN void *pFileProp);



//---------------------------------------------------------------------
// Internet Protocol (IP) support function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Returns the standard host name for the local machine. 
 * 
 * SMGetIPHostName() function returns the name of the local host into 
 * the buffer specified by the pBufHostName parameter. The host name is 
 * returned as a null-terminated string. 
 *
 * WINDOWS: The format of the host name is dependent on the Windows 
 * Sockets provider.
 *
 * @param pBufHostName [out] pointer to a buffer to store the returned 
 *		string.
 * 
 * @param pBufSize [inout] pointer to a variable that has the size of 
 *		the buffer before call and will have the size of the string if 
 *		the call is succeeded.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetIPHostName(
				_OUT astring *pBufHostName, // host name buffer
				_INOUT u32 *pBufSize // size of host name buffer
				);


//---------------------------------------------------------------------
/**
 * Returns the standard host name for the local machine. 
 *
 * SMUCS2GetIPHostName() function returns the name of the local host 
 * into the buffer specified by the pBufHostName parameter. The host 
 * name is returned as a null-terminated Unicode string. 
 *
 * WINDOWS: The form of the host name is dependent on the Windows 
 * Sockets provider.
 *
 * @param pBufHostName [out] pointer to a buffer to store the returned 
 *		Unicode string.
 * 
 * @param pBufSize [inout] pointer to a variable that has the size in 
 *		bytes of the host name buffer on input and the size in bytes 
 *		of the host name string on output.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUCS2GetIPHostName(
				_OUT ustring *pBufHostName, // host name buffer
				_INOUT u32 *pBufSize // size of host name buffer
				);


//---------------------------------------------------------------------
/**
 * Returns the standard host name for the local machine. 
 * 
 * SMGetIPFQDN() function returns the fully qualified domain name of the
 * local host into the buffer specified by the pBufHostName parameter.  
 * The fully qualified domain name is returned as a null-terminated string. 
 *
 * WINDOWS: The format of the fully qualified domain name is dependent on  
 * the Windows Sockets provider.
 *
 * @param pBufHostName [out] pointer to a buffer to store the returned 
 *		string.
 * 
 * @param pBufSize [inout] pointer to a variable that has the size of 
 *		the buffer before call and will have the size of the string if 
 *		the call is succeeded.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetIPFQDN(
				_OUT astring *pBufHostName, // host name buffer
				_INOUT u32 *pBufSize // size of host name buffer
				);


//---------------------------------------------------------------------
/**
 * Returns the standard host name for the local machine. 
 * 
 * SMUCS2GetFQDN() function returns the fully qualified domain name of the
 * local host into the buffer specified by the pBufHostName parameter.  
 * The fully qualified domain name is returned as a null-terminated string. 
 *
 * WINDOWS: The format of the fully qualified domain name is dependent on  
 * the Windows Sockets provider.
 *
 * @param pBufHostName [out] pointer to a buffer to store the returned 
 *		string.
 * 
 * @param pBufSize [inout] pointer to a variable that has the size of 
 *		the buffer before call and will have the size of the string if 
 *		the call is succeeded.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMUCS2GetFQDN(
				_OUT ustring *pBufHostName, // host name buffer
				_INOUT u32 *pBufSize // size of host name buffer
				);


//---------------------------------------------------------------------
/**
 * Returns list of IP Addresses associated with the provided host name.
 * 
 * SMGetIPAddrStrList() function returns list of IP Addresses associated 
 * with the provided host name.
 *
 * LINUX: IP addresses are returned for the local host only.
 *
 * @param pBufHostName [in] pointer to a null terminated host name 
 *		string.
 *
 * @param pIPASCount [inout] pointer to a variable that has the size of 
 *		the list provided in pIPASList parameter.
 * 
 * @param pIPASList [out] pointer to a list of IPAddrStr structures.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 *
 * @return SM_STATUS_DATA_OVERRUN is returned if there was not 
 *		sufficient buffer space provided for all IP addresses.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetIPAddrStrList(
				_IN const astring *pBufHostName, // host name buffer
				_INOUT u32 *pIPASCount, // On input the number of IPAddrStr allocated pointed by pIPASList. On output the number copied
				_OUT IPAddrStr *pIPASList // IPAddrStr list buffer. If NULL then pIPASCount will contain count needed on _OUT.
				);


//---------------------------------------------------------------------
/**
 * Returns the DNS suffix of the host if one exists. 
 *
 * SMGetDNSSuffixNameName() function returns the DNS suffix if setup 
 * for the local host. The suffix is returned as a null-terminated 
 * string. If the provided size pointed by pBufSize is not sufficient 
 * then this will return SM_STATUS_DATA_OVERRUN and updates the size 
 * needed in pBufSize.
 *
 * @param pBufDNSSuffix	[out] pointer to a buffer to store the returned 
 *		string.
 *
 * @param pBufSize [inout] pointer to a variable that has the size of 
 *		the buffer before call and will have the size of the string if 
 *		the call is succeeded.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetDNSSuffixNameName(
				_OUT astring *pBufDNSSuffix, // returned DNS suffix shown in ipconfig
				_INOUT u32 *pBufSize  // size of DNS suffix buffer
				);


//---------------------------------------------------------------------
/**
 * Convert IP address from dotted ASCII string format to network 
 * integer format. 
 *
 * The SMIPAddrASCIIToNetwork() function interprets the character string 
 * specified by the pSrc parameter. This string represents a numeric 
 * Internet address expressed in the Internet standard ".'' notation. 
 * The value returned is a number suitable for use as an Internet 
 * address. All Internet addresses are returned in IP's network order 
 * (bytes ordered from left to right). 
 *
 * If you pass in " " (a space) to the SMIPAddrASCIIToNetwork() 
 * function, SMIPAddrASCIIToNetwork() returns SM_STATUS_UNSUCCESSFUL.
 *
 * @param pSrc [in] pointer to a null-terminated character string 
 *		representing a number expressed in the Internet standard "." 
 *		(Dotted) notation.
 *
 * @param pDest [out] pointer to a variable that will be filled with 
 *		the resulting network number.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.
 * 
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded.
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned for internal provider 
 *		returned errors.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMIPAddrASCIIToNetwork(
				_IN const astring *pSrc,	// IP address ASCII string buffer
				_OUT u32 *pDest				// IP address integer buffer
				);


//---------------------------------------------------------------------
/**
 * Convert IP address from network integer format to dotted ASCII 
 * string format.
 *
 * The SMIPAddrNetworkToASCII() function takes an Internet address 
 * structure specified by the nSrc parameter and returns an ASCII 
 * string representing the address in "." (Dot) notation as in a.b.c.d.
 *
 * @param nSrc [in] Network number to be converted.
 *
 * @param pDest	[out] pointer to a buffer to store a null-terminated 
 *		character string representing a number expressed in the Internet 
 *		standard "."(dotted) notation.
 *
 * @param pDestSize [inout] pointer to a variable that has the size of 
 *		the buffer before call and will have the size of the string if 
 *		the call is succeeded.
 *
 * @return If no error occurs, returns SM_STATUS_SUCCESS.  
 *
 * @return SM_STATUS_DEVICE_NOT_READY is returned if required components 
 *		are not loaded. 
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned for internal provider 
 *		returned errors.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMIPAddrNetworkToASCII(
				_IN u32 nSrc,				// IP address integer
				_OUT astring *pDest,		// IP address ASCII string buffer
				_INOUT u32 *pDestSize		// size of IP address string buffer
				);


//---------------------------------------------------------------------
// SMInetAddrUTF8ToNetwork
/**
 * Convert Internet address from UTF8 character string format to
 * binary network format.
 *
 * SMInetAddrUTF8ToNetwork interprets the specified Internet address
 * in UTF8 character string format according to the specified address type
 * and converts the string to binary format in network byte order.
 *
 * @param addrType [in] type of address pointed to by pSrc.
 *		The following address types are supported currently. \n
 *		\n
 *		Value                   Meaning \n
 *		-----                   ------- \n
 *		SM_INETADDR_TYPE_IPV4   IPv4 address \n
 *		SM_INETADDR_TYPE_IPV6   IPv6 address \n
 *
 * @param pSrc [in] pointer to a null-terminated UTF8 character string 
 *		representing an Internet address.  The type of the address
 *		is interpreted according to addrType.
 *
 * @param pDest [out] pointer to buffer that will be filled with 
 *		the resulting Internet address in binary network format.
 *
 * @param pDestSize [inout] pointer to a variable that has the size of 
 *		the buffer provided in pDest parameter.  On input, contains
 *		the size in bytes of the buffer pointed to by pDest.
 *		On output, contains the number of bytes written to pDest.
 *		When addrType is SM_INETADDR_TYPE_IPV4, size should be at least
 *		4 on input.
 *		When addrType is SM_INETADDR_TYPE_IPV6, size should be at least
 *		16 on input.
 *
 * @return SM_STATUS_SUCCESS returned if no error occurs.
 *
 * @return SM_STATUS_CMD_BAD returned if a required parameter
 *		is not provided.
 *
 * @return SM_STATUS_INVALID_PARAMETER returned if an invalid parameter
 *		is provided.
 *
 * @return SM_STATUS_DATA_OVERRUN returned if dest buffer is too small.
 *
 * @return Status codes from dcsmstatus.h returned for other errors. 
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMInetAddrUTF8ToNetwork(_IN u32 addrType,
										  _IN const astring *pSrc,
										  _OUT u8 *pDest,
										  _INOUT u32 *pDestSize);


//---------------------------------------------------------------------
// SMInetAddrNetworkToUTF8
/**
 * Convert Internet address from binary network format to UTF8
 * character string format.
 *
 * SMInetAddrNetworkToUTF8 interprets the specified Internet address
 * in binary network format according to the specified address type
 * and converts it to UTF8 character string format.
 *
 * @param addrType [in] type of address pointed to by pSrc.
 *		The following address types are supported currently. \n
 *		\n
 *		Value                   Meaning \n
 *		-----                   ------- \n
 *		SM_INETADDR_TYPE_IPV4   IPv4 address \n
 *		SM_INETADDR_TYPE_IPV6   IPv6 address \n
 *
 * @param pSrc [in] pointer to Internet address in binary network
 *		format.  The type of the address is interpreted according
 *		to addrType.
 *
 * @param srcSize [in] size in bytes of the address pointed to by pSrc.
 *		When addrType is SM_INETADDR_TYPE_IPV4, srcSize should be 4.
 *		When addrType is SM_INETADDR_TYPE_IPV6, srcSize should be 16.
 *
 * @param pDest [out] pointer to buffer that will be filled with 
 *		the resulting Internet address in UTF8 character string format.
 *		IPv4 addresses are converted to dotted-decimal format.
 *		IPv6 addresses are converted to the most appropriate IPv6
 *		network address format for the address.
 *
 * @param pDestSize [inout] pointer to a variable that has the size of 
 *		the buffer provided in pDest parameter.  On input, contains
 *		the size in bytes of the buffer pointed to by pDest.
 *		On output, contains the number of bytes written to pDest.
 *
 * @return SM_STATUS_SUCCESS returned if no error occurs.
 *
 * @return SM_STATUS_CMD_BAD returned if a required parameter
 *		is not provided.
 *
 * @return SM_STATUS_INVALID_PARAMETER returned if an invalid parameter
 *		is provided.
 *
 * @return SM_STATUS_DATA_OVERRUN returned if dest buffer is too small.
 *
 * @return Status codes from dcsmstatus.h returned for other errors. 
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMInetAddrNetworkToUTF8(_IN u32 addrType,
										  _IN const u8 *pSrc,
										  _IN u32 srcSize,
										  _OUT astring *pDest,
										  _INOUT u32 *pDestSize);


//---------------------------------------------------------------------
// SMGetInetAddrStrList
/**
 * Get list of Internet addresses associated with the local host.
 * 
 * SMGetInetAddrStrList returns a list of Internet addresses of the
 * specified address type for the local host.
 *
 * @param addrType [in] type of addresses to retrieve.
 *		The following address types are supported currently. \n
 *		\n
 *		Value                   Meaning \n
 *		-----                   ------- \n
 *		SM_INETADDR_TYPE_UNSPEC Return IPv4 and IPv6 addresses \n
 *		SM_INETADDR_TYPE_IPV4   Return IPv4 addresses \n
 *		SM_INETADDR_TYPE_IPV6   Return IPv6 addresses \n
 *
 * @param pIASCount [inout] pointer to a variable that has the size of 
 *		the list provided in the pIASList parameter.  On input, contains
 *		the number of SMInetAddrStr structures pointed to by pIASList.
 *		On output, contains the number of Internet addresses returned.
 * 
 * @param pIASList [out] pointer to a list of SMInetAddrStr structures
 *		that will be filled with Internet addresses.  If NULL is specified,
 *		pIASCount will contain the number of SMInetAddrStr structures needed
 *		to hold all available Internet addressess on output.
 *
 * @return SM_STATUS_SUCCESS returned if no error occurs.
 *
 * @return SM_STATUS_CMD_BAD returned if a required parameter
 *		is not provided.
 *
 * @return SM_STATUS_INVALID_PARAMETER returned if an invalid parameter
 *		is provided.
 *
 * @return SM_STATUS_DATA_OVERRUN returned if buffer is provided but
 *		is too small for all Internet addresses.  The number of
 *		SMInetAddrStr structures needed to hold all Internet addresses
 *		is written to pIASCount.
 *
 * @return Status codes from dcsmstatus.h returned for other errors. 
 */
//---------------------------------------------------------------------
SMINTF s32 _cdecl SMGetInetAddrStrList(_IN u32 addrType,
									   _INOUT u32 *pIASCount,
									   _OUT SMInetAddrStr *pIASList);



//---------------------------------------------------------------------
// RFC1321 support functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Initializes and loads the buffers with the data needed for 
 * subsequent RFC1321 API calls. 
 * 
 * The SMRFC1321Load() function basically initializes the specified 
 * RFC1321Data structure. This functions has be called before making 
 * any other RFC1321 function calls.
 *
 * @param pRFCD [in] pointer to a RFC1321Data structure.
 *
 * @return No Return codes.
 *
 * @sa SMRFC1321UnLoad()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRFC1321Load(_OUT RFC1321Data *pRFCD);


//---------------------------------------------------------------------
/**
 * Unloads the buffers created by SMRFC1321Load() function. 
 * 
 * @param pRFCD [in] pointer to a RFC1321Data structure.
 *
 * @return No Return codes.
 *
 * @sa SMRFC1321Load()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRFC1321UnLoad(_INOUT RFC1321Data *pRFCD);


//---------------------------------------------------------------------
/**
 * Appends the Messages to the provided RFC1321Data structure before 
 * computing the digest. 
 * 
 * The SMRFC1321AddMsg() function basically appends the messages to the 
 * specified RFC1321Data structure. 
 * 
 * This can be called as many times before calling the 
 * SMRFC1321ComputeMsgDigest() function.
 *
 * @param pRFCD [in] pointer to a RFC1321Data structure.
 * 
 * @param pMsg [in] message to be encoded.
 *
 * @param msgSize [in] size in bytes of the message.
 *
 * @return No Return codes.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRFC1321AddMsg(
				_INOUT RFC1321Data *pRFCD,
				_IN const void *pMsg,
				_IN u32 msgSize
				);


//---------------------------------------------------------------------
/**
 * Computes the message digest that can be sent as an encrypted message. 
 *
 * The SMRFC1321ComputeMsgDigest() function computes the digest using 
 * the RFC1321 algorithm. 
 *
 * @param pRFCD [in] pointer to a RFC1321Data structure.
 *
 * @return No Return codes.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRFC1321ComputeMsgDigest(_INOUT RFC1321Data *pRFCD);


//---------------------------------------------------------------------
/**
 * Generate a CRC 32 number for checksum.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGenerateCRC32(_IN const u8 *pInBuf, 
								  _IN const u32 inBufSize, 
								  _IN const u32 vKeyLen);



//---------------------------------------------------------------------
// SINGLY linked list support routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Allocate and initializes a singly linked list. 
 *
 * The SMSLListAlloc() function allocates the basic SMSLList structure 
 * and if specified it allocates the context lock variable which will 
 * be used to protect the list when ever this particular list has been 
 * accessed.
 *
 * @return Return a pointer to the SMSLList Structure if succeeded, 
 * other wise returns NULL.
 *
 * @sa SMSLListFree()
 */
//---------------------------------------------------------------------

SMINTF SMSLList * _cdecl SMSLListAlloc(void);


//---------------------------------------------------------------------
/**
 * Free a singly linked list. 
 * The SMSLListFree() function frees up the basic SMSLList structure 
 * and if exists the associated context lock also.
 *
 * @param pSL [in] SMSLList structure pointer to be freed.
 *
 * @return 	No return codes.
 *
 * @sa SMSLListAlloc()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListFree(_IN SMSLList *pSL);


//---------------------------------------------------------------------
/**
 * Initialize a singly linked list. 
 *
 * The SMSLListInitNoAlloc() function initializes the specified 
 * SMSLList structure.
 *
 * @param pSL [in] SMSLList structure pointer.
 *
 * @return No return codes.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListInitNoAlloc(_INOUT SMSLList *pSL);


//---------------------------------------------------------------------
/**
 * Allocate and initializes a singly linked list node. 
 * 
 * The SMSLListEntryAlloc() function allocates the basic SMSLListEntry 
 * structure and if the entryDataSize is not zero then it will allocate 
 * the memory for that amount initializes the pData of SMSLListEntry 
 * structure and returns the pointer to the allocated structure.
 *
 * @param entryDataSize	[in] size to be allocated for the data portion 
 *		of list entry.
 *
 * @return Return a pointer to the SMSLListEntry Structure if succeeded 
 *		other wise returns NULL.
 *
 * @sa SMSLListEntryFree()
 */
//---------------------------------------------------------------------

SMINTF SMSLListEntry * _cdecl SMSLListEntryAlloc(_IN u32 entryDataSize);


//---------------------------------------------------------------------
/**
 * Free a singly linked list node. 
 * The SMSLListEntryFree() function de allocates the specified 
 * SMSLListEntry structure and also the memory allocated for pData 
 * member of SMSLListEntry if it is not null.
 *
 * @param pSLE [in] pointer to the SMSLListEntry structure to be freed.
 *
 * @return No return values.
 *
 * @sa SMSLListEntryAlloc
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListEntryFree(_INOUT SMSLListEntry *pSLE);


//---------------------------------------------------------------------
/**
 * Init a singly linked list node entry. 
 * 
 * The SMSLListEntryInitNoAlloc() function initializes the members of 
 * the specified SMSLListEntry structure if it not NULL.
 *
 * @param pSLE [in] pointer to the SMSLListEntry structure to be 
 *		initialized.
 * 
 * @return No return values.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListEntryInitNoAlloc(_INOUT SMSLListEntry *pSLE);


//---------------------------------------------------------------------
/**
 * Add an SMSLListEntry to the top of a singly linked list. 
 * 
 * The SMSLListInsertEntryAtHead() function adds the specified list 
 * entry pSLE at the head position to the given List pSL. Before 
 * accessing the list it will lock the list if the context lock is 
 * present for the list. Once it is added now the pHead member of the 
 * pSL list is now pointing to the new entry pSLE.
 *
 * @param pSL [inout] pointer the Linked List Structure.
 *
 * @param pSLE [inout] pointer to the List Entry structure to be added.
 *
 * @return No return values.
 *
 * @sa SMSLListInsertEntryAtTail()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListInsertEntryAtHead(_INOUT SMSLList *pSL, 
											 _INOUT SMSLListEntry *pSLE);


//---------------------------------------------------------------------
/**
 * Add an SMSLListEntry to the end of a singly linked list. 
 * 
 * The SMSLListInsertEntryAtTail() function adds the specified list 
 * entry pSLE at the tail position to the given List pSL. Before 
 * accessing the list it will lock the list if the context lock is 
 * present for the list. Once it is added now the pTail member of the 
 * pSL list is now pointing to the new entry pSLE.
 *
 * @param pSL [inout] pointer the Linked List Structure.
 *
 * @param pSLE [inout] pointer to the List Entry structure to be appended.
 *
 * @return No return values.
 *
 * @sa SMSLListInsertEntryAtHead()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListInsertEntryAtTail(_INOUT SMSLList *pSL, 
											 _INOUT SMSLListEntry *pSLE);


//---------------------------------------------------------------------
/**
 * Add a SMSLListEntry to a sorted singly linked list. 
 * 
 * The SMSLListInsertEntry() function inserts the specified list entry 
 * pSLE at the position depending on the provided comparison function 
 * in the given List pSL. The sort key is established by the caller 
 * supplied function pfnCompare. The routine must compare the list 
 * entry elements, then return one of the following values: \n
 * <0 pSLE1 less than pSLE2; \n
 * =0 pSLE1 equivalent to pSLE2; \n
 * > 0 pSLE1 greater than pSLE2. \n
 * 
 * The list is sorted in increasing order, as defined by the comparison 
 * function. To sort a list in decreasing order, reverse the sense of 
 * "greater than" and "less than" in the comparison function.
 *
 * @param pSL [in] pointer the Linked List Structure.
 *
 * @param pSLE [in] pointer to the List Entry structure to be added.
 *
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the compare function to be called.
 *
 * @return SM_STATUS_SUCCESS is returned if the given entry 
 *		successfully added to list at the appropriate position. 
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if any one of the 
 *		parameters is a NULL pointer.
 *
 * @sa SMSLListDeleteEntry()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSLListInsertEntry(_INOUT SMSLList *pSL, 
									  _INOUT SMSLListEntry *pSLE, 
									  _INOUT void *pWalkData, 
									  _IN PFNSLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Walk the linked list calling the walk function for every entry with 
 * passed walk data. 
 * 
 * The SMSLListWalkAtHead() function facilitates a way for the caller 
 * to retrieve a particular node from the list using caller defined 
 * function pfnWalk. The walk function is called with the provided walk 
 * data for each and every entry in the list pSL. The walk is stopped 
 * when a match is found which is indicated by the walk function 
 * returning a 0.The walk continues when a match is not found indicated 
 * by returning non-zero value.
 *
 * @param pSL [in] pointer the singly linked list structure.
 * 
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 * 
 * @return Return the pointer to the List entry that matches the pfnWalk 
 *		function search criteria. Null pointer is return if there no 
 *		match.
 */
//---------------------------------------------------------------------

SMINTF SMSLListEntry * _cdecl SMSLListWalkAtHead(_INOUT SMSLList *pSL, 
												 _INOUT void *pWalkData, 
												 _IN PFNSLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Replace a SMSLListEntry from a singly linked list in the same place 
 * as the node to be replaced. 
 * 
 * The SMSLListReplaceEntry() function searches for pSLEToReplace in 
 * the list pSL, once it finds then it will replace that entry with 
 * pSLEWith in the list. Before accessing the list it will lock the 
 * list if the context lock is present. 
 *
 * @param pSL [in] pointer the Singly Linked List.
 *
 * @param pSLEWith [in] pointer to the new List Entry.
 *
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 * 
 * @return No return values.
 */
//---------------------------------------------------------------------

SMINTF SMSLListEntry * _cdecl SMSLListReplaceEntry(_INOUT SMSLList *pSL, 
												   _INOUT SMSLListEntry *pSLEWith, 
												   _INOUT void *pWalkData, 
												   _IN PFNSLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Delete SMSLListEntry from a singly linked list. 
 * 
 * The SMSLListDeleteEntry() function searches and deletes the 
 * specified list entry pSLE in the given List pSL. Before accessing 
 * the list it will lock the list if the context lock is present. If 
 * the removed entry is at the top of the list then the next element in 
 * the list will be the pHead, similarly if the deleted element is at 
 * the bottom of the list then pTail member of the list pSL will point 
 * to the previous member in the list.
 *
 * @param pSL [in] pointer the Linked List Structure.
 *
 * @param pSLEPrev [in] pointer to previous linked likswt structure.
 *
 * @param pSLE [in] pointer to the List Entry structure to be deleted.
 *
 * @return No return values.
 *
 * @sa SMSLListInsertEntry()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSLListDeleteEntry(_INOUT SMSLList *pSL, 
									   _INOUT SMSLListEntry *pSLEPrev, 
									   _INOUT SMSLListEntry *pSLE);


//---------------------------------------------------------------------
/**
 * Remove a SMSLListEntry from the beginning of a singly linked list. 
 *
 * The SMSLListRemoveEntryAtHead() function removes the entry at the 
 * top of the list or pointed to by pHead is and returns a pointer to 
 * that entry. The pHead member of pSL is now pointing to the next 
 * entry in the list if any. If the list is empty it will return a NULL 
 * pointer.
 *
 * @param pSL [in] pointer the singly linked list structure.
 *
 * @return Return the pointer to the List entry that has been deleted 
 *		from the list. If the List is empty then it returns null pointer.
 *
 * @sa SMSLListRemoveEntryAtTail()
 */
//---------------------------------------------------------------------

SMINTF SMSLListEntry * _cdecl SMSLListRemoveEntryAtHead(_INOUT SMSLList *pSL);


//---------------------------------------------------------------------
/**
 * Remove a SMSLListEntry at the end of a singly linked list. 
 * 
 * The SMSLListRemoveEntryAtTail() function removes the entry at the 
 * bottom of the list or pointed to by pTail and returns a pointer to 
 * that entry. The pTail member of pSL is now pointing to the previous 
 * entry in the list if any. If the list is empty it will return a NULL 
 * pointer.
 * 
 * @param pSL [in] pointer the singly linked list structure.
 *
 * @return Return the pointer to the List entry that has been deleted 
 *		from the list. If the List is empty then it returns null pointer.
 * 
 * @sa SMSLListRemoveEntryAtHead()
 */
//---------------------------------------------------------------------

SMINTF SMSLListEntry * _cdecl SMSLListRemoveEntryAtTail(_INOUT SMSLList *pSL);


//---------------------------------------------------------------------
/**
 * Checks to see if the the singly linked list is empty or contains 
 * data nodes.
 *
 * @param pSL [in] reference to an exisiting Singly Linked List.
 * 
 * @return Returns TRUE if SMSLList has no data nodes.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMSLListIsEmpty(_IN const SMSLList *pSL);



//---------------------------------------------------------------------
// DOUBLY linked list support routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Allocate and initializes a doubly linked list. 
 * The SMDLListAlloc() function allocates the basic SMDLList structure 
 * and if specified it allocates the context lock variable which will 
 * be used to protect the list when ever this particular list has been 
 * accessed.
 *
 * @return Return a pointer to the SMDLList Structure if succeeded 
 *		other wise returns NULL.
 *
 * @sa SMDLListFree
 */
//---------------------------------------------------------------------

SMINTF SMDLList * _cdecl SMDLListAlloc(void);


//---------------------------------------------------------------------
/**
 * Free a doubly linked list. 
 *
 * The SMDLListFree() function frees up the basic SMDLList structure 
 * and if exists the associated context lock also.
 *
 * @param pDL [in] SMDLList structure pointer to be freed.
 *
 * @return No return codes.
 *
 * @sa SMDLListAlloc()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListFree(_INOUT SMDLList *pDL);


//---------------------------------------------------------------------
/**
 * Initialize a doubly linked list. 
 *
 * The SMDLListInitNoAlloc() function initializes the specified 
 * SMDLList structure and if pLock parameter is not null then it will 
 * associate the pLock with the given pDL structure.
 *
 * @param pDL [in] SMDLList structure pointer.
 *
 * @return No return codes.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListInitNoAlloc(_INOUT SMDLList *pDL);


//---------------------------------------------------------------------
/**
 * Allocate and initializes a doubly linked list node. 
 *
 * The SMDLListEntryAlloc() function allocates the basic SMDLListEntry 
 * structure and if the entryDataSize is not zero then it will allocate 
 * the memory for that amount initializes the pData of SMDLListEntry 
 * structure and returns the pointer to the allocated structure.
 *
 * @param entryDataSize	[in] size to be allocated for the data portion 
 *		of list entry.
 *
 * @return Return a pointer to the SMDLListEntry Structure if succeeded 
 *		other wise returns NULL.
 *
 * @sa SMDLListEntryFree()
 */
//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListEntryAlloc(_IN u32 entryDataSize);


//---------------------------------------------------------------------
/**
 * Free a doubly linked list node. 
 * 
 * The SMDLListEntryFree() function de allocates the specified 
 * SMDLListEntry structure and also the memory allocated for pData 
 * member of SMDLListEntry if it is not null.
 * 
 * @param pDLE [in] pointer to the SMDLListEntry structure to be freed.
 *
 * @return No return values.
 *
 * @sa SMDLListEntryAlloc()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListEntryFree(_INOUT SMDLListEntry *pDLE);


//---------------------------------------------------------------------
/**
 * Initialize a doubly linked list node entry. 
 * 
 * The SMDLListEntryInitNoAlloc() function initializes the members of 
 * the specified SMDLListEntry structure if it not NULL.
 *
 * @param pDLE [in] pointer to the SMDLListEntry structure to be 
 *		initialized.
 *
 * @return No return values.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListEntryInitNoAlloc(_INOUT SMDLListEntry *pDLE);


//---------------------------------------------------------------------
/**
 * Add an SMDLListEntry to the top of a doubly linked list. 
 * 
 * The SMDLListInsertEntryAtHead() function adds the specified list 
 * entry pDLE at the head position to the given List pDL. Before 
 * accessing the list it will lock the list if the context lock is 
 * present for the list. Once it is added now the pHead member of the 
 * pDL list is now pointing to the new entry pDLE.
 *
 * @param pDL [in] pointer the Linked List Structure.
 *
 * @param pDLE [in] pointer to the List Entry structure to be added.
 *
 * @return No return values.
 *
 * @sa SMDLListInsertEntryAtTail()
 *
 * @sa SMDLListInsertEntry()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListInsertEntryAtHead(_INOUT SMDLList *pDL, 
											 _INOUT SMDLListEntry *pDLE);


//---------------------------------------------------------------------
/**
 * Add an SMDLListEntry to the end of a doubly linked list. 
 * 
 * The SMDLListInsertEntryAtTail() function adds the specified list 
 * entry pDLE at the tail position to the given List pDL. Before 
 * accessing the list it will lock the list if the context lock is 
 * present for the list. Once it is added now the pTail member of the 
 * pDL list is now pointing to the new entry pDLE.
 *
 * @param pDL [in] pointer the Linked List Structure.
 *
 * @param pDLE [in] pointer to the List Entry structure to be appended.
 *
 * @return No return values.
 *
 * @sa SMDLListInsertEntryAtHead()
 *
 * @sa SMDLListInsertEntry()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListInsertEntryAtTail(_INOUT SMDLList *pDL, 
											 _INOUT SMDLListEntry *pDLE);


//---------------------------------------------------------------------
/**
 * Add a SMDLListEntry to a sorted doubly linked list. 
 *
 * The SMDLListInsertOrdered() function inserts the specified list 
 * entry pDLE at the position depending on the provided comparison 
 * function in the given List pDL. The sort key is established by the 
 * caller supplied function pfnCompare. The routine must compare the 
 * list entry elements, then return one of the following values: \n
 * <0 pDLE1 less than pDLE2; \n
 * =0 pDLE1 equivalent to pDLE2; \n
 * > 0 pDLE1 greater than pDLE2. \n
 *
 * The list is sorted in increasing order, as defined by the comparison 
 * function. To sort a list in decreasing order, reverse the sense of 
 * "greater than" and "less than" in the comparison function.
 *
 * @param pDL [in] pointer the Linked List Structure.
 * 
 * @param pDLE [in] pointer to the List Entry structure to be added.
 *
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 *
 * @return SM_STATUS_SUCCESS is returned if the given entry 
 *		successfully added to list at the appropriate position. 
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if any one of the 
 *		parameters is a NULL pointer.
 *
 * @sa SMDLListInsertEntryAtHead()
 *
 * @sa SMDLListInsertEntryAtTail()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMDLListInsertEntry(_INOUT SMDLList *pDL, 
									  _INOUT SMDLListEntry *pDLE, 
									  _INOUT void *pWalkData, 
									  _IN PFNDLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Walk the linked list calling the walk function for every entry with 
 * passed walk data. 
 * 
 * The SMDLListWalkAtHead() function facilitates a way for the caller 
 * to retrieve a particular node from the list using caller defined 
 * function pfnWalk. The walk function is called with the provided walk 
 * data for each and every entry in the list pDL. The walk is stopped 
 * when a match is found which is indicated by the walk function 
 * returning a 0.The walk continues when a match is not found indicated 
 * by returning non-zero value.
 *
 * @param pDL [in] pointer the doubly linked list structure.
 *
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 *
 * @return Return the pointer to the List entry that matches the pfnWalk 
 *		function search criteria. 
 *
 * @return Null pointer is return if there no match.
 *
 * @sa SMDLListWalkAtTail()
 */
//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListWalkAtHead(_INOUT SMDLList *pDL, 
												 _INOUT void *pWalkData, 
												 _IN PFNDLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Walk the linked list calling the walk function for every entry with 
 * passed walk data. 
 * 
 * The SMDLListWalkAtTail() function facilitates a way for the caller 
 * to retrieve a particular node from the list using caller defined 
 * function pfnWalk. The walk function is called with the provided walk 
 * data for each and every entry in the list pDL. The walk is stopped 
 * when a match is found which is indicated by the walk function 
 * returning a 0.The walk continues when a match is not found indicated 
 * by returning non-zero value.
 *
 * @param pDL [in] pointer the doubly linked list structure.
 *
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 *
 * @return Return the pointer to the List entry that matches the pfnWalk 
 *		function search criteria. 
 *
 * @return Null pointer is return if there no match.
 *
 * @sa SMDLListWalkAtTail()
 */
//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListWalkAtTail(_INOUT SMDLList *pDL, 
												 _INOUT void *pWalkData, 
												 _IN PFNDLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Replace a SMDLListEntry from a doubly linked list in the same place 
 * as the node to be replaced. 
 *
 * The SMDLListReplaceEntry() function searches for pDLEToReplace in 
 * the list pDL, once it finds then it will replace that entry with 
 * pDLEWith in the list. Before accessing the list it will lock the 
 * list if the context lock is present. 
 *
 * @param pDL [in] pointer the doubly linked list.
 *
 * @param pDLEWith [in] pointer to the new List Entry.
 * 
 * @param pWalkData	[in] data to be passed to pfnWalk function.
 *
 * @param pfnWalk [in] pointer to the function to be called.
 *
 * @return No return values.
 */
//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListReplaceEntry(_INOUT SMDLList *pDL, 
												   _INOUT SMDLListEntry *pDLEWith, 
												   _INOUT void *pWalkData, 
												   _IN PFNDLLISTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Delete SMDLListEntry from a singly linked list. 
 *
 * The SMDLListDeleteEntry() function searches and deletes the 
 * specified list entry pDLE in the given List pDL. Before accessing 
 * the list it will lock the list if the context lock is present. If 
 * the removed entry is at the top of the list then the next element in 
 * the list will be the pHead, similarly if the deleted element is at 
 * the bottom of the list then pTail member of the list pDL will point 
 * to the previous member in the list.
 *
 * @param pDL [in] pointer the Linked List Structure.
 * 
 * @param pDLE [in] pointer to the List Entry structure to be deleted.
 *
 * @return No return values.
 *
 * @sa SMDLListInsertEntry()
 *
 * @sa SMDLListRemoveEntryAtHead()
 *
 * @sa SMDLListRemoveEntryAtTail()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMDLListDeleteEntry(_INOUT SMDLList *pDL, 
									   _INOUT SMDLListEntry *pDLE);


//---------------------------------------------------------------------
/**
 * Remove a SMDLListEntry from the beginning of a singly linked list. 
 *
 * The SMDLListRemoveEntryAtHead() function removes the entry at the 
 * top of the list or pointed to by pHead is and returns a pointer to 
 * that entry. The pHead member of pDL is now pointing to the next 
 * entry in the list if any. If the list is empty it will return a NULL 
 * pointer.
 *
 * @param pDL [in] pointer the doubly linked list structure.
 *
 * @return Return the pointer to the List entry that has been deleted 
 *		from the list. If the List is empty then it returns null pointer.
 *
 * @sa SMDLListInsertEntryAtHead()
 *
 * @sa SMDLListRemoveEntry()
 *
 * @sa SMDLListRemoveEntryAtTail()
 */
//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListRemoveEntryAtHead(_INOUT SMDLList *pDL);


//---------------------------------------------------------------------
/**
 * Remove a SMDLListEntry at the end of a doubly linked list. 
 * 
 * The SMDLListRemoveEntryAtTail function removes the entry at the 
 * bottom of the list or pointed to by pTail and returns a pointer to 
 * that entry. The pTail member of pDL is now pointing to the previous 
 * entry in the list if any. If the list is empty it will return a NULL 
 * pointer.
 *
 * @param pDL [in] pointer the doubly linked list structure.
 *
 * @return Return the pointer to the List entry that has been deleted 
 *		from the list. If the List is empty then it returns null pointer.
 *
 * @sa SMDLListInsertEntryAtTail()
 *
 * @sa SMDLListRemoveEntry()
 *
 * @sa SMDLListRemoveEntryAtHead()
 */

//---------------------------------------------------------------------

SMINTF SMDLListEntry * _cdecl SMDLListRemoveEntryAtTail(_INOUT SMDLList *pDL);


//---------------------------------------------------------------------
/**
 * Checks to see if the the doubly linked list is empty or contains 
 * data nodes.
 *
 * @param pDL [in] reference to an exisiting Doubly Linked List.
 *
 * @return Returns TRUE if SMDLList has no data nodes.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMDLListIsEmpty(_IN const SMDLList *pDL);



//---------------------------------------------------------------------
// OS Abstraction support function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Creates a directory given valid path. 
 * 
 * SMMkDir() function creates a new directory given a complete path. 
 * 
 * SMMkDir() can create only one new directory per call, so only the last 
 * component of pPath can name a new directory. 
 *
 * SMMkDir does not translate path delimiters.
 *
 * @param pPath	[in] null terminated  path string.
 *
 * @return SM_STATUS_SUCCESS is returned for successful creation and/or 
 *		the directory exists. For all the other failure cases 
 *		SM_STATUS_UNSUCCESSFUL is returned.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMMkDir(_IN const astring *pPath);


//---------------------------------------------------------------------
/**
 * Deletes a directory given valid path and the directory is empty. 
 * 
 * The SMRmDir() function deletes the directory specified by pPath. 
 * The directory must be empty, and it must not be the current working 
 * directory or the root directory.
 *
 * @param pPath	[in] null terminated path string.
 *
 * @return SM_STATUS_SUCCESS is returned for successful deletion. For 
 *		all the other failure cases SM_STATUS_UNSUCCESSFUL is returned.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMRmDir(_IN const astring *pPath);


//---------------------------------------------------------------------
/**
 * Get a temprary file specifed by path and file name.
 *
 * @param pPath	[in] null terminated path string.
 *
 * @param pFileName	[in] null terminated file name string.
 *
 * @param pSize [inout] Size of the file.
 *
 * @return SM_STATUS_SUCCESS is returned if get successfully. For 
 *		all the other failure cases SM_STATUS_UNSUCCESSFUL is returned.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetTmpFile(
				_IN const astring *pPath, 
				_OUT astring *pFileName, 
				_INOUT u32 *pSize
				);


//---------------------------------------------------------------------
/**
 * Returns the size of file in bytes. 
 * 
 * SMGetSizeOfFile() returns the size of the file specified by 
 * pPathFileName. The size is stored in the location pointed by pSize 
 * variable.
 *
 * @param pPathFileName [in] null terminated  path string.
 *
 * @param pSize	[out] pointer to a u32 variable.
 *
 * @return SM_STATUS_SUCCESS is returned if the size is calculated. For
 *		all the other failure cases SM_STATUS_UNSUCCESSFUL is returned.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetSizeOfFile(
				_IN const astring *pPathFileName,
				_OUT u32 *pSize
				);


//---------------------------------------------------------------------
/**
 * Deletes a files given complete path. 
 * 
 * SMDeleteFile deletes the file specified by pPathFileName. It changes 
 * the file permissions before delete so even read only files are also 
 * deleted without any error.
 *
 * @param pPathFileName	[in] null terminated path file name to be 
 *		deleted.
 *
 * @return SM_STATUS_SUCCESS is returned if the file is removed. For 
 *		all the other failure cases SM_STATUS_UNSUCCESSFUL is returned.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMDeleteFile(_IN const astring *pPathFileName);


//---------------------------------------------------------------------
/**
 * The function checks the console for a recent keystroke. 
 * 
 * If the function returns a nonzero value, a keystroke is waiting in 
 * the buffer. The program can then call getch() or similar calls to 
 * get the keystroke.
 *
 * @return Returns a nonzero value if a key has been pressed. 
 *		Otherwise, it returns 0.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMkbhit(void);



//---------------------------------------------------------------------
// Module management routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Export context management routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Get an export thread context.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMGetExportContext(_OUT SMECInfo *pECI);


//---------------------------------------------------------------------
/**
 * Set an export thread context.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSetExportContext(_OUT SMECInfo *pPrevECI,
									  _IN SMECInfo *pCurrECI);


//---------------------------------------------------------------------
/**
 * Reset an export thread context.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMResetExportContext(_IN SMECInfo *pCurrECI);



//---------------------------------------------------------------------
// Thread management routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Creates a Thread object and executes/attaches the specified function. 
 *
 * The SMThreadStart() function creates a thread to execute within the 
 * virtual address space of the calling process. 
 *
 * @param fpSMTR [in] pointer to a function that is to be executed.
 *
 * @param pSMThreadData	[in] pointer to a data block that will be 
 *		passes as an argument to the function.
 *
 * @return Returns a void pointer once the thread is successfully 
 *		created and started, which should be used when calling the 
 *		SMThreadStop function. 
 *
 * @return NULL pointer is returned if it failed to execute the thread.
 *
 * @sa SMThreadStop()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMThreadStart(_IN FPROCSMTHREADROUTINE fpSMTR, 
								   _INOUT void *pSMThreadData);


//---------------------------------------------------------------------
/**
 * Clears or releases the resources used by the creation of a thread 
 * using SMThreadStart(). 
 *
 * The SMThreadStop function closes the handles of a thread that is 
 * started by using the function SMThreadStart().
 *
 * WINDOWS: It waits for 15 seconds if the thread is not terminated in 
 * that time it just closes the handle of the thread object which does 
 * not means that the thread is terminated so the developer has to make 
 * sure that the function that they have executed in the thread will 
 * exit gracefully once they decide to call SMThreadStop().
 *
 * LINUX:  Here the function SMThreadStop blocks if the thread is still 
 * executing. If some other thread calls the SMThreadStop function to 
 * stop the same thread it will return error immediately.
 *
 * @param pSMThread	[in] pointer to the thread context that is returned 
 *		while creating this thread that is to be stopped.
 *
 * @return SM_STATUS_SUCCESS is returned if the thread is stopped. 
 *
 * @return The failure cases can be: \n
 *		1. SM_STATUS_INVALID_PARAMETER - If pSMThread is NULL. \n
 *		2. SM_STATUS_OBJECT_BAD - If PSMThread is invalid. \n
 *
 * @sa SMThreadStart()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMThreadStop(_IN void *pSMThread);


//---------------------------------------------------------------------
/**
 * Kills a thread and releases the resources used by the creation of the
 * thread using the SMThreadStart API.
 *
 * The SMThreadKill API closes the handles of a thread that is 
 * started by using the SMThreadStart API.
 *
 * WINDOWS: The Windows TerminateThread API is used to kill the thread.
 * The thread has no chance to exit normally.  See MSDN for important
 * information about using the Windows TerminateThread API.
 *
 * LINUX: The Linux pthread_kill API is used to deliver the SIGKILL
 * signal to the thread.  See the man page for pthread_kill for
 * important information about using the Linux pthread_kill API.
 *
 * @param pSMThread	[in] pointer to the thread context that is returned 
 *		while creating this thread that is to be stopped.
 *
 * @return SM_STATUS_SUCCESS is returned if the thread is killed. 
 *
 * @return The failure cases can be: \n
 *		1. SM_STATUS_INVALID_PARAMETER - If pSMThread is NULL. \n
 *		2. SM_STATUS_OBJECT_BAD - If pSMThread is invalid. \n
 *
 * @sa SMThreadStart()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMThreadKill(_IN void *pSMThread);


//---------------------------------------------------------------------
/**
 * Creates a Thread object and executes/attaches the specified function. 
 *
 * The SMThreadStartAndRelease() function creates a thread to execute within 
 * the virtual address space of the calling process. In contrast to 
 * SMThreadStart, there is no internal structure created and any thread
 * handle created is immediately closed. Therefore, the thread routine
 * supplied by the caller is responsible for releasing all resources used
 * by the thread and ultimately proper exit from the thread routine to 
 * release or terminate the thread object.
 *
 * @param fpSMTR [in] pointer to a function that is to be executed.
 *
 * @param pSMThreadData	[in] pointer to a data block that will be 
 *		passes as an argument to the function.
 *
 * @return SM_STATUS_SUCCESS if a thread is started successfully.
 *
 * @return SM_STATUS_INVALID_PARAMETER if fpSMTR is NULL. 
 *
 * @return SM_STATUS_UNSUCCESSFUL if a thread could not be started. 
 *
 * @sa SMThreadStart()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMThreadStartAndRelease(_IN FPROCSMTHREADROUTINE fpSMTR, 
										  _INOUT void *pSMThreadData);


//---------------------------------------------------------------------
/**
 * Yield the thread priority.
 *
 * @return Return a pointer to 0 memory.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMThreadYield(void);


//---------------------------------------------------------------------
/**
 * Get thread ID.
 *
 * @note Do not access or free the returned value since it is
 * a pointer to 0 memory. The pointer value itself contains the
 * thread ID value where you can do comparison but not arithmetic.
 *
 * @return Return a pointer to 0 memory.
 */
//---------------------------------------------------------------------

SMINTF const void * _cdecl SMThreadGetID(void);



//---------------------------------------------------------------------
// Library management (auto-managed) routines 
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This is the alternative of SMLibLinkToExportFN().
 *
 * The function SMFLibLinkToExportFN() take a path file name parameter,
 * while SMLibLinkToExportFN takes a module pointer as parameter.
 *
 * Note that the user ID must be unique within a process since reference 
 * count is monitored by user ID. This is also true across threads. When 
 * two threads use the same ID and one calls SMLibUnLoad the other thread 
 * will cause a process crash when it uses a function pointer returned 
 * from a call to SMFLibLinkToExportFN earlier. This occurs since the call 
 * to SMLibUnLoad unloaded all reference to the library and all exported 
 * symbols.
 *
 * This function should be called before a code block exit if the context
 * is local, i.e. in the same code block. Otherwise, it should be called 
 * in a module's detach/destructor/fini function if the context is global.
 *
 * @param pUID [in] unique string that identifies the user of the library
 *		that is matched in calls to SMFLibLinkToExportFN and SMFLibUnLoad.
 *
 * @param pLibPathFileName [in] points to an complete path name of the 
 *		module to be loaded.
 *
 * @param pExportFNName	[in] Name of the exported function/symbol to 
 *		look for.
 * 
 * @return If the function succeeds, the return value is the address of 
 *		the exported function or variable. If the function fails, the 
 *		return value is NULL.
 *
 * @sa SMLibLoad() 
 *
 * @sa SMLibLinkToExportFN()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMFLibLinkToExportFN(_IN const astring *pUID,
										  _IN const astring *pLibPathFileName,
										  _IN const astring *pExportFNName);


//---------------------------------------------------------------------
/**
 * This is the alternative of SMLibUnLoad().
 *
 * The function SMFLibUnLoad() take a path file name parameter,
 * while SMLibUnLoad takes a module pointer as parameter.
 *
 * Note that the user ID must be unique within a process since reference 
 * count is monitored by user ID. This is also true across threads. When 
 * two threads use the same ID and one calls SMLibUnLoad the other thread 
 * will cause a process crash when it uses a function pointer returned 
 * from a call to SMFLibLinkToExportFN earlier. This occurs since the call 
 * to SMLibUnLoad unloaded all reference to the library and all exported 
 * symbols.
 *
 * This function should be called before a code block exit if the context
 * is local, i.e. in the same code block. Otherwise, it should be called 
 * in a module's detach/destructor/fini function if the context is global.
 *
 * @param pUID [in] unique string that identifies the user of the library
 *		that is matched in calls to SMFLibLinkToExportFN and SMFLibUnLoad.
 *
 * @param pLibPathFileName [in] points to an complete path name of the 
 *		module to be unloaded.
 *
 * @return Return SM_STATUS_SUCCESS if the module is released 
 *		successfully.
 * 
 * @return SM_STATUS_UNSUCCESFUL is returned if the unloading module 
 *		returns an error. 
 *
 * @return SM_STATUS_OBJECT_BAD is the passed void pointer is corrupted 
 *		i.e. not being returned by the SMLibLoad function.
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the pSMLib is a 
 *		null pointer.
 *
 * @sa SMLibUnLoad()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMFLibUnLoad(_IN const astring *pUID,
							   _IN const astring *pLibPathFileName);



//---------------------------------------------------------------------
// Library management routines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Load a specified module. 
 *
 * The SMLibLoad() function maps the specified executable module into 
 * the address space of the calling process.
 *
 * WINDOWS: If the module is a DLL not already mapped for the calling 
 * process, the system calls the DLL's DllMain function with the 
 * DLL_PROCESS_ATTACH value. If the DLL's entry-point function does not 
 * return TRUE, SMLibLoad fails and returns NULL. (The system 
 * immediately calls your entry-point function with DLL_PROCESS_DETACH 
 * and unloads the DLL.)\n 
 * For more information please look at LoadLibrary Documentation in MSDN.
 *
 * LINUX: The SMLibLoad() function is available only to a 
 * dynamically-linked process. A statically-linked process (one where 
 * libc is linked statically) can't call SMLibLoad because a 
 * statically-linked executable: \n
 * doesn't export any of its symbols; \n
 * can't export the required structure for libraries to link against; \n
 * can't fill structures at startup needed to load subsequent shared 
 * objects; \n
 *
 * Objects whose names resolve to the same absolute or relative path 
 * name can be opened any number of times using SMLibLoad(). The object 
 * is loaded into the process's address space only once.
 *
 * When loading shared objects, the application should open a specific 
 * version instead of relying on the version pointed to by a symbolic 
 * link.
 *
 * @param pLibPathFileName [in] points to an complete path name of the 
 *		module to be loaded.
 *
 * @return The module is successfully loaded then it will return a void 
 *		pointer/handle that can be used in subsequent calls to 
 *		SMLibLinkToExportFN().
 *
 * @return NULL pointer is returned for failed cases.
 *
 * @sa SMFLibUnLoad()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMLibLoad(_IN const astring *pLibPathFileName);


//---------------------------------------------------------------------
/**
 * Unload a sepcified module.
 *
 * Disassociates a shared object/module opened/loaded by SMLibLoad() 
 * from the calling process. The SMLibUnLoad() function detaches the 
 * specified executable module from the address space of the calling 
 * process.
 *
 * WINDOWS: Each process maintains a reference count for each loaded 
 * library module. This reference count is incremented each time 
 * SMLibLoad() is called and is decremented each time SMLibUnLoad is 
 * called. A DLL module loaded at process initialization due to 
 * load-time dynamic linking has a reference count of one. This count 
 * is incremented if the same module is loaded by a call to SMLibLoad. \n
 * Before un mapping a library module, the system enables the DLL to 
 * detach from the process by calling the DLL's DLLMain function, if it 
 * has one, with the DLL_PROCESS_DETACH value. Doing so gives the DLL 
 * an opportunity to clean up resources allocated on behalf of the 
 * current process. After the entry-point function returns, the 
 * library module is removed from the address space of the current 
 * process. \n
 * For more information please look at FreeLibrary Documentation in 
 * MSDN.
 *
 * LINUX: The SMLibUnLoad function is available only to a 
 * dynamically-linked process. \n
 * An object won't be removed from the address space until all 
 * references to that object have been closed. \n
 * Referencing a symbol in a closed object can cause undefined 
 * behavior.
 *
 * @param pSMLib [in] pointer to a structure that is returned by a 
 *		previous call to SMLibLoad().
 *
 * @return Return SM_STATUS_SUCCESS if the module is released 
 *		successfully.
 * 
 * @return SM_STATUS_UNSUCCESFUL is returned if the unloading module 
 *		returns an error. 
 *
 * @return SM_STATUS_OBJECT_BAD is the passed void pointer is corrupted 
 *		i.e. not being returned by the SMLibLoad function.
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the pSMLib is a 
 *		null pointer.
 *
 * @sa SMFLibLoad()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMLibUnLoad(_IN void *pSMLib);


//---------------------------------------------------------------------
/**
 * Retrieves the address of an exported function or variable from the 
 * specified module or object. 
 * 
 * The SMLibLinkToExportFN() function locates and returns a function or 
 * symbol in the specified executable module.
 *
 * WINDOWS: The spelling and case of a function name pointed to by 
 * pExportFNName must be identical to that in the EXPORTS statement of 
 * the source DLL's module-definition (.Def) file. \n
 * For more information please look at GetProcAddress Documentation in 
 * MSDN.
 *
 * LINUX: The SMLibLinkToExportFN() function is available only to a 
 * dynamically-linked process. \n
 * Referencing a symbol in a closed object can cause undefined behavior.
 *
 * @param pSMLib [in] void pointer or handle that is returned by a 
 *		previous call to SMLibLoad().
 *
 * @param pExportFNName	[in] Name of the exported function/symbol to 
 *		look for.
 * 
 * @return If the function succeeds, the return value is the address of 
 *		the exported function or variable. If the function fails, the 
 *		return value is NULL.
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMLibLinkToExportFN(_IN void *pSMLib, 
										 _IN const astring *pExportFNName);


//---------------------------------------------------------------------
/**
 * Release the address link that established in funcion 
 * SMLibLinkToExportFN().
 *
 * @param pSMLib [in] void pointer or handle that is returned by a 
 *		previous call to SMLibLoad().
 *
 * @param pExportFNName	[in] Name of the exported function/symbol to 
 *		look for.
 * 
 * @return If the function succeeds, the return value is the address of 
 *		the exported function or variable. If the function fails, the 
 *		return value is NULL.
 *
 * @sa SMLibLinkToExportFN()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMLibUnLinkFromExportFN(_IN void *pSMLib, 
										  _IN const astring *pExportFNName);



//---------------------------------------------------------------------
// IPC and synchronization management routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
// Mutex
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This function creates a named or unnamed mutex object.
 * 
 * @param pMutexName [in] Name for global mutexes to be created for 
 *		use across processes. If pMutexName is NULL, the function 
 *		creates a mutex local to the process.
 *
 * @note Under Linux, it is important that the first four (4) 
 *		characters of the name must be unique.
 *
 * @return On success, it returns a (void *) handle to the newly 
 *		created mutex object.
 *
 * @sa SMMutexDestroy()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMMutexCreate(_IN const astring *pMutexName);

//---------------------------------------------------------------------
/**
* This function creates a named or unnamed mutex object.
* 
* @param pMutexName [in] Name for global mutexes to be created for 
*		use across processes. If pMutexName is NULL, the function 
*		creates a mutex local to the process.
*
* @param aclAttribute [in] Access Control List attribute for the
*       created object. See "SMACL_" definitions for value format
*
* @note Under Linux, it is important that the first four (4) 
*		characters of the name must be unique.
*
* @return On success, it returns a (void *) handle to the newly 
*		created mutex object.
*
* @sa SMMutexDestroy()
*/
//---------------------------------------------------------------------

SMINTF void * _cdecl SMMutexCreate_s(_IN const astring *pMutexName,
									 _IN u32 aclAttribute);


//---------------------------------------------------------------------
/**
 * This function opens a existing mutex object.
 *
 * The SMMutexOpen() function returns a handle to an existing named 
 * mutex object created by SMMutexCreate() function call. On failure, 
 * the function returns a NULL.
 *
 * @param pMutexName [in] Name for global mutexes to be opened for use 
 *		across processes. Name is mandatory.
 *
 * @return On success, it returns a (void *) handle to an already 
 *		created/existing mutex object (SMMutexCreate()). On failure, 
 *		NULL is returned.
 *
 * @sa SMMutexCreate()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMMutexOpen(_IN const astring *pMutexName);


//---------------------------------------------------------------------
/**
 * This function locks a mutex object.
 *
 * The SMMutexLock() function returns if we get a lock on the mutex 
 * object or the time-out (waitTimeOutMilliSecs) interval elapses.
 *
 * @param pMutex [in] handle to an existing mutex object.
 *
 * @param waitTimeOutMilliSecs [in] Specifies the time-out interval, in 
 *		milliseconds. The function returns if the interval elapses, 
 *		even if the mutex's state is unlocked. If waitTimeOutMilliSecs 
 *		is zero, the function tests the object's state and returns 
 *		immediately. If waitTimeOutMilliSecs is INFINITE, the function's 
 *		time-out interval never elapses.
 *		On Linux, the granualrity is only supported upto 10ms. Please refer
 *		to the man pages of nanosleep or usleep
 *
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the passed handle is 
 *		NULL or if we could not acquire a lock on the mutex.
 *
 * @return SM_STATUS_TIMEOUT is returned if time-out 
 *		(waitTimeOutMilliSecs) interval elapsed, and we failed to get a 
 *		lock on the mutex object.
 *
 * @sa SMMutexUnLock()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMMutexLock(_INOUT void *pMutex, 
							  _IN s32 waitTimeOutMilliSecs);


//---------------------------------------------------------------------
/**
 * This function unlockes a mutex object.
 * 
 * The SMMutexUnLock() function releases ownership of the specified 
 * mutex object.
 *
 * @param pMutex [in] handle to an existing mutex object.
 *
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails, SM_STATUS_UNSUCCESSFUL is returned 
 *		if the passed handle is NULL or if we could not unlock the 
 *		mutex.
 *
 * @sa SMMutexLock()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMMutexUnLock(_INOUT void *pMutex);


//---------------------------------------------------------------------
/**
 * The SMMutexDestroy() function destroys an open mutex object handle.
 *
 * @param pMutex [in] handle to an existing mutex object.
 *
 * @return There is no return value.
 *
 * @sa SMMutexCreate()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMMutexDestroy(_INOUT void *pMutex);



//---------------------------------------------------------------------
// Semaphore
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This function creates a named or unnamed semaphore object.
 * 
 * @param pSemaphoreName [in] Name for global semaphores to be created 
 * for use across processes. If pSemaphoreName is NULL, the function 
 * creates an unnamed semaphore.
 *
 * @param initialCount [in] Initialial count for this semaphore object.
 * 
 * @note Under Linux, it is important that the first four (4) 
 * characters of the name must be unique.
 *
 * @return Returns a void * handle to the Semaphore object upon a 
 *		successful return. Upon failure, NULL is returned.
 * 
 * @sa SMSemaphoreDestroy()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSemaphoreCreate(_IN const astring *pSemaphoreName, 
									   _IN s32 initialCount);

//---------------------------------------------------------------------
/**
* This function creates a named or unnamed semaphore object.
* 
* @param pSemaphoreName [in] Name for global semaphores to be created 
* for use across processes. If pSemaphoreName is NULL, the function 
* creates an unnamed semaphore.
*
* @param initialCount [in] Initialial count for this semaphore object.
* 
* @param aclAttribute [in] Access Control List attribute for the
*       created object. See "SMACL_" definitions for value format
*
* @note Under Linux, it is important that the first four (4) 
* characters of the name must be unique.
*
* @return Returns a void * handle to the Semaphore object upon a 
*		successful return. Upon failure, NULL is returned.
* 
* @sa SMSemaphoreDestroy()
*/
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSemaphoreCreate_s(_IN const astring *pSemaphoreName, 
										 _IN s32 initialCount,
										 _IN u32 aclAttribute);


//---------------------------------------------------------------------
/**
 * This function opens a existing semaphore object.
 * 
 * The function SMSemaphoreOpen() returns a handle to an existing named 
 * semaphore object created by SMSemaphoreCreate() function call. On 
 * failure, the function returns a NULL.
 *
 * @param pSemaphoreName [in] Name for global semaphores to be opened 
 *		for use across processes. Name is mandatory.
 *
 * @return On success, it returns a (void *) handle to an already 
 *		created/existing semaphore object (SMSemaphoreCreate()). On 
 *		failure, NULL is returned.
 *
 * @sa SMSemaphoreCreate()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSemaphoreOpen(_IN const astring *pSemaphoreName);


//---------------------------------------------------------------------
/**
 * This function acquire a wait on a semaphore call.
 * 
 * The SMSemaphoreAcquireWait () function returns if we acquire the 
 * semaphore object or the time-out (waitTimeOutMilliSecs) interval 
 * elapses.
 *
 * @param pSemaphore [in] handle to an existing Semaphore object.
 *
 * @param waitTimeOutMilliSecs [in] Specifies the time-out interval, 
 *		in milliseconds. The function returns if the interval elapses. 
 *		If waitTimeOutMilliSecs is zero, the function tests the 
 *		object's state and returns immediately. If waitTimeOutMilliSecs 
 *		is INFINITE, the function's time-out interval never elapses.
 *		On Linux, the granualrity is only supported upto 10ms. Please refer
 *		to the man pages of nanosleep or usleep
 * 
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the passed handle is 
 *		NULL or if we could not acquire the semaphore object;
 *
 * @return SM_STATUS_TIMEOUT is returned if time-out 
 *		(waitTimeOutMilliSecs) interval elapsed, and we failed to get 
 *		the Semaphore object.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSemaphoreAcquireWait(_INOUT void *pSemaphore, 
										 _IN s32 waitTimeOutMilliSecs);


//---------------------------------------------------------------------
/**
 * This function increases the sephamore by a specific account.
 * 
 * The SMSemaphoreRelease() function increases the count of the 
 * specified semaphore object by a specified amount.
 *
 * @param pSemaphore [in] handle to an existing semaphore object.
 *
 * @param releaseCount [in] Specifies the amount by which the semaphore 
 *		object's current count is to be increased. The value must be 
 *		greater than zero. If the specified amount would cause the 
 *		semaphore's count to exceed the maximum count that was 
 *		specified when the semaphore was created, the count is not 
 *		changed and the function returns FALSE.
 *
 * @param pPreviousCount [in] Pointer to a 32-bit variable to receive 
 *		the previous count for the semaphore. This parameter can be 
 *		NULL if the previous count is not required. 
 * 
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the passed handle 
 *		is NULL or the releaseCount > SHRT_MAX (32767);
 *
 * @return SM_STATUS_OBJECT_BAD	if the passed object is not of type 
 *		OSSYNCINFO_TYPE_SEMAPHORE;
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the semaphore could not 
 *		be released or if the semaphore could not be set to releaseCount.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSemaphoreRelease(_INOUT void *pSemaphore, 
									 _IN s32 releaseCount, 
									 _OUT s32 *pPreviousCount);


//---------------------------------------------------------------------
/**
 * Thhis function destroys an open semaphore object handle.
 * 
 * @param pSemaphore [in] handle to an existing mutex object.
 *
 * @return There is no return value.
 *
 * @sa SMSemaphoreCreate()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSemaphoreDestroy(_INOUT void *pSemaphore);



//---------------------------------------------------------------------
// Shared Memory
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Allocates requested size of shared memory object/segment.
 *
 * @param pSharedMemName [in] name of shared memory segment. Name is 
 *		mandatory. It cannot be NULL.
 *
 * @note Under Linux, it is important that the first four (4) 
 *		characters of the name must be unique.
 *
 * @param sizeOfSharedMem [in] size of the desired shared memory segment.
 *
 * @return Upon success, returns a handle to the created shared memory 
 * object. Upon failure, returns NULL.
 *
 * @sa SMSharedMemFree()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSharedMemCreate(_IN const astring *pSharedMemName, 
									   _IN u32 sizeOfSharedMem);

//---------------------------------------------------------------------
/**
* Allocates requested size of shared memory object/segment.
*
* @param pSharedMemName [in] name of shared memory segment. Name is 
*		mandatory. It cannot be NULL.
*
* @note Under Linux, it is important that the first four (4) 
*		characters of the name must be unique.
*
* @param sizeOfSharedMem [in] size of the desired shared memory segment.
*
* @param aclAttribute [in] Access Control List attribute for the
*       created object. See "SMACL_" definitions for value format
*
* @return Upon success, returns a handle to the created shared memory 
* object. Upon failure, returns NULL.
*
* @sa SMSharedMemFree()
*/
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSharedMemCreate_s(_IN const astring *pSharedMemName, 
										 _IN u32 sizeOfSharedMem,
										 _IN u32 aclAttribute);


//---------------------------------------------------------------------
/**
 * Opens a handle to the shared memory object/segment created by 
 * SMSharedMemCreate() function.
 *
 * @param pSharedMemName [in] name of shared memory segment. Name is 
 *		mandatory. It cannot be NULL.
 *
 * @return None.
 *
 * @sa SMSharedMemCreate()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSharedMemOpen(_IN const astring *pSharedMemName);


//---------------------------------------------------------------------
/**
 * Ger address pointer of previoursly created shared memory.
 * 
 * The SMSharedMemGetAddressPtr() function returns a previously mapped 
 * address of an exisiting shared memory object.
 *
 * @param pSharedMem [in] pointer to an existing shared memory object.
 *
 * @return On success the function returns previously mapped address.
 *
 * @sa SMSharedMemCreate()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMSharedMemGetAddressPtr(_INOUT void *pSharedMem);


//---------------------------------------------------------------------
/**
 * Detaches and unmaps the shared memory. 
 *
 * The Creator of the shared memory is allowed to destroy the shared 
 * memory segment completely.
 *
 * @param pSharedMem [in] pointer to an existing shared memory object.
 *
 * @return On success it returns SM_STATUS_SUCCESS. 
 *		SM_STATUS_UNSUCCESSFUL is returned on failure.
 *
 * @return  This command is not implemented under NT or W2K OS. The 
 *		return status is SM_STATUS_CMD_NOT_IMPLEMENTED.
 *
 * @sa SMSharedMemCreate()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSharedMemFree(_INOUT void *pSharedMem);



//---------------------------------------------------------------------
// Critical Sections
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This function executes a given routine as a critical section (fpSMCR). 
 *
 * @param fpSMCSR [in] pointer to a function that is to be executed.
 *
 * @param pSMCritSecData [inout] data to be passed back and forth from 
 *		the fpSMCR routine.
 *
 * @param waitTimeOutMilliSecs [in] amount of time in milliseconds to 
 *		wait for the critical section routine to enter execution if 
 *		blocked by an already executing instance of the critical section.
 *		On Linux, the granualrity is only supported upto 10ms. Please refer
 *		to the man pages of nanosleep or usleep
 * 
 * @return No return value.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMCriticalSectionExecute(_IN FPROCSMCRITSECROUTINE fpSMCSR, 
											_INOUT void *pSMCritSecData, 
											_IN s32 waitTimeOutMilliSecs);



//---------------------------------------------------------------------
// Event Signal
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This function creates a named or unnamed event object.
 *
 * @param pEventName [in] Name for global signals to be created for use 
 *		across processes. If pEventName is NULL, the function creates 
 *		an unnamed signal.
 *
 * @note Under Linux, it is important that the first four (4) 
 *		characters of the name must be unique.
 *
 * @param manualReset [in] Specifies whether a manual-reset or 
 * auto-reset event object is created.
 *
 * @param initialState [in] Specifies the initial state of the event 
 * object. If TRUE, the initial state is signaled; otherwise, it is 
 * nonsignaled.
 *
 * @return Returns a void * handle to the Event object upon a 
 *		successful return. Upon failure, NULL is returned.
 *
 * @sa SMEventDestroy()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMEventCreate(_IN const astring *pEventName, 
								   _IN booln manualReset, 
								   _IN booln initialState);

//---------------------------------------------------------------------
/**
* This function creates a named or unnamed event object.
*
* @param pEventName [in] Name for global signals to be created for use 
*		across processes. If pEventName is NULL, the function creates 
*		an unnamed signal.
*
* @note Under Linux, it is important that the first four (4) 
*		characters of the name must be unique.
*
* @param manualReset [in] Specifies whether a manual-reset or 
* auto-reset event object is created.
*
* @param initialState [in] Specifies the initial state of the event 
* object. If TRUE, the initial state is signaled; otherwise, it is 
* nonsignaled.
*
* @param aclAttribute [in] Access Control List attribute for the
*       created object. See "SMACL_" definitions for value format
*
* @return Returns a void * handle to the Event object upon a 
*		successful return. Upon failure, NULL is returned.
*
* @sa SMEventDestroy()
*/
//---------------------------------------------------------------------

SMINTF void * _cdecl SMEventCreate_s(_IN const astring *pEventName, 
									 _IN booln manualReset, 
									 _IN booln initialState,
									 _IN u32 aclAttribute);


//---------------------------------------------------------------------
/**
 * This function returns a handle to an existing named event object 
 * created by SMEventCreate() function call.
 *
 * @param pEventName [in] Name for global signals to be opened for use 
 * across processes. Name is mandatory.
 *
 * @return On success, it returns a (void *) handle to an already 
 *		created/existing event object (SMEventCreate()). On failure, 
 *		NULL is returned.
 *
 * @sa SMEventCreate()
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMEventOpen(_IN const astring *pEventName);



//---------------------------------------------------------------------
/**
 * The SMEventWait() function returns if we signal the event or the 
 * time-out (waitTimeOutMilliSecs) interval elapses.
 *
 * @param pEvent [in] handle to an existing Event object.
 *
 * @param waitTimeOutMilliSecs [in] Specifies the time-out interval, in 
 *		milliseconds. The function returns if the interval elapses. 
 *		If waitTimeOutMilliSecs is zero, the function tests the 
 *		object's state and returns immediately. If waitTimeOutMilliSecs 
 *		is INFINITE, the function's time-out interval never elapses.
 *		On Linux, the granualrity is only supported upto 10ms. Please refer
 *		to the man pages of nanosleep or usleep
 * 
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails, SM_STATUS_INVALID_PARAMETER is 
 *		returned if the passed handle is NULL SM_STATUS_TIMEOUT is 
 *		returned if time-out (waitTimeOutMilliSecs) interval elapsed, 
 *		and we failed to signal the Event.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMEventWait(_INOUT void *pEvent, 
							  _IN s32 waitTimeOutMilliSecs);



//---------------------------------------------------------------------
/**
 * The SMEventSet function signals the event.
 *
 * @param pEvent [in] handle to an existing event object.
 *
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the passed handle 
 *		is NULL or, the releaseCount > SHRT_MAX (32767);
 *
 * @return SM_STATUS_OBJECT_BAD	If the passed object is not of type 
 *		OSSYNCINFO_TYPE_EVENTSIG. 
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the event could not 
 * be signaled.
 *
 * @sa SMEventReset()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMEventSet(_INOUT void *pEvent);



//---------------------------------------------------------------------
/**
 * The SMEventResets() function sets the specified event object to the
 * non-signaled state. It must be primarily used for manual reset event
 * objects.
 *
 * @param pEvent [in] handle to an existing event object.
 *
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the passed handle 
 *		is NULL or, the releaseCount > SHRT_MAX (32767);
 *
 * @return SM_STATUS_OBJECT_BAD	If the passed object is not of type 
 *		OSSYNCINFO_TYPE_EVENTSIG. 
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the event could not 
 * be signaled.
 *
 * @sa SMEventSet()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMEventReset(_INOUT void *pEvent);



//---------------------------------------------------------------------
/**
 * 
 * This function "pulse" the event.
 *
 * The SMEventPulse() function provides a single operation that sets 
 * (to signaled) the state of the specified event object and then 
 * resets it (to nonsignaled) after releasing the appropriate number 
 * of waiting threads.
 *
 * @note This API is unreliable and should not be used by new applications.
 * It exists mainly for backward compatibility. For more information see MSDN online.
 *
 * @param pEvent [in] handle to an existing event object.
 *
 * @return If the function succeeds, SM_STATUS_SUCCESS is returned.
 *
 * @return If the function fails:
 *
 * @return SM_STATUS_INVALID_PARAMETER is returned if the passed handle 
 *		is NULL or, the releaseCount > SHRT_MAX (32767);
 *
 * @return SM_STATUS_OBJECT_BAD	If the passed object is not of type 
 *		OSSYNCINFO_TYPE_EVENTSIG. 
 *
 * @return SM_STATUS_UNSUCCESSFUL is returned if the event could not 
 * be signaled.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMEventPulse(_INOUT void *pEvent);



//---------------------------------------------------------------------
/**
 * The SMEventDestroy () function destroys any exisiting event object 
 * pointed to by the pEvent handle.
 *
 * @param pEvent [in] handle to an existing event object.
 *
 * @return There is no return value for a destroy.
 *
 * @sa SMEventCreate()
 */
//---------------------------------------------------------------------

SMINTF void  _cdecl SMEventDestroy(_INOUT void *pEvent);



//---------------------------------------------------------------------
// Printf to avoid buffer overflow attacks
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Customized version of sprintf.
 *
 * Like sprintf, however adds the maximum number of characters that can 
 * be added to the output string.  This should be used in _all_ cases 
 * instead of sprintf.
 *
 * @param pOutBuf [in] The output buffer where the format and arguments 
 *		will be copied.  This can be NULL if you want it to just 
 *		calculate how much space you need to expand a certain format.
 *
 * @param outBufSize [in] The total size of the output buffer, 
 *		including space reserved for the '\\0' character.
 *
 * @param pFormatStr [in] printf-style supplied format string.
 *
 * @param ... [in] variable number of arguments.
 *
 * @return The number of characters added to the output buffer.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMsnprintf(_INOUT astring *pOutBuf, 
							 _IN s32 outBufSize, 
							 _IN const astring *pFormatStr, 
							 ...);

//---------------------------------------------------------------------
/**
 * Customized version of vsprintf.
 *
 * Like vsprintf, however adds the maximum number of characters that 
 * can be added to the output string.  This should be used in _all_ 
 * cases instead of vsprintf.
 *
 * @param pOutBuf [in] The output buffer where the format and arguments 
 *		will be copied.  This can be NULL if you want it to just 
 *		calculate how much space you need to expand a certain format.
 *
 * @param outBufSize [in] The total size of the output buffer, 
 *		including space reserved for the '\\0' character.
 *
 * @param pFormatStr [in] printf-style supplied format string.
 *
 * @param pVALArg [in] variable number of arguments.
 *
 * @return The number of characters added to the output buffer.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMvsnprintf(_INOUT astring *pOutBuf, 
							  _IN s32 outBufSize, 
							  _IN const astring *pFormatStr, 
							  va_list pVALArg);


//---------------------------------------------------------------------
// Environment
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/** 
 * Duplicate environment variable value.
 *
 * @param pVarName [in] pointer to environment variable name.
 *
 * @param pValBufSize [out] pointer to variable to receive the size
 *		of the value buffer returned.  This can be NULL if the caller
 *		is not interested in the buffer size.
 *
 * @return This function returns a pointer to a dynamically allocated
 *		buffer containing the environment variable value or NULL if
 *		the environment variable is not found.
 * 
 * @note User must use SMFreeGeneric() to free returned memory.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMEnvDup(_IN const astring *pVarName,
								 _OUT u32 *pValBufSize);


//---------------------------------------------------------------------
// Miscellaneous OS support routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Enable long file name, not limit to 8.3 conversion.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMLongFileNamesEnable(void);


//---------------------------------------------------------------------
/**
 * Disable long file name, follow 8.3 conversion.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMLongFileNamesDisable(void);


//---------------------------------------------------------------------
/**
 * Shutdown the operating system.
 *
 * @param forceIt [in] Force the OS to shutdown despite unfinished tasks.
 *
 * @param restartAfterShutdown [in] If true system will reboot afer 
 *		shutdown.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMOSShutdown(_IN booln forceIt, 
								 _IN booln restartAfterShutdown);


//---------------------------------------------------------------------
// Hash id generation support function prototypes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Allocate hash table memory.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMHashTableBytesNeeded(_IN u32 maxHashID);


//---------------------------------------------------------------------
/**
 * Clear a hash table.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMClearHashTable(_INOUT void *pHashTable, 
								   _IN u32 hashTableSize);


//---------------------------------------------------------------------
/**
 * Query hash IDs.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMAcquireHashID(_INOUT void *pHashTable, 
								  _IN u32 hashTableSize, 
								  _IN u32 hashType);

//---------------------------------------------------------------------
/**
 * Query a hash ID by its identification.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMAcquireHashIDByID(_INOUT void *pHashTable, 
									  _IN u32 hashTableSize, 
									  _IN u32 id);

//---------------------------------------------------------------------
/**
 * Verify a hash ID valid or not.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMVerifyHashID(_IN const void *pHashTable, 
								 _IN u32 hashTableSize, 
								 _IN u32 id);


//---------------------------------------------------------------------
/**
 * Release a hash ID.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMReleaseHashID(_INOUT void *pHashTable, 
								  _IN u32 hashTableSize, 
								  _IN u32 id);

//---------------------------------------------------------------------
/**
 * Get the smallest hash ID that is being used in the hash table.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGetMinInUseID(_IN const void *pHashTable, 
								  _IN u32 hashTableSize);


//---------------------------------------------------------------------
/**
 * Get the boggest hash ID that is being used in the hash table.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGetMaxInUseID(_IN const void *pHashTable, 
								  _IN u32 hashTableSize);


//---------------------------------------------------------------------
/**
 * Find the next hash ID in the hash table.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMFindNextHashID(_IN const void *pHashTable, 
								   _IN u32 hashTableSize, 
								   _INOUT u32 *pHashIndex, 
								   _INOUT u32 *pBitIndex);


//---------------------------------------------------------------------
/**
 * Find the first hash ID in the hash table.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMFindFirstHashID(_IN const void *pHashTable, 
									_IN u32 hashTableSize, 
									_OUT u32 *pHashIndex, 
									_OUT u32 *pBitIndex);



//---------------------------------------------------------------------
// NVPair support routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Given a NV pair string it returns the name string.
 *
 * This API method parses the given pNVPair string to look for first 
 * '=' character, then replaces the "=" character with '\\0' and returns 
 * pNVPair.
 *
 * @note This API does modify the passed string pNVPair so be careful 
 * in reusing that string.
 *
 * @param pNVPair [in] null terminated NV pair string.
 *
 * @return If the passed NV Pair is valid then it will return the 
 *		value string; else it will return NULL pointer.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMNVPGetUTF8ParamName(_IN astring *pNVPair);


//---------------------------------------------------------------------
/**
 * Given a NV pair string it returns the value string.
 *
 * This API method parses the given pNVPair string to look for first 
 * '=' character, then returns the right side portion of '=' character.
 *
 * @param pNVPair [in] null terminated NV pair string.
 *
 * @return If the passed NV Pair is valid then it will return the 
 *		value string; else it will return NULL pointer.
 */
//---------------------------------------------------------------------

SMINTF const astring * _cdecl SMNVPGetUTF8ParamValue(_IN const astring *pNVPair);


//---------------------------------------------------------------------
/**
 * Returns the index of a given name in an list of NV pair strings.
 *
 * This method compares the given string with the name strings in the
 * list of NV pairs, depending on the ignoreCase variable. Once the 
 * match is found it returns the index at which the match is found, 
 * or else returns a -1.
 * 
 * If any of the NV pairs in the list is not a valid NV pair then it 
 * just skips to the next one.
 *
 * @param numNVPair	[in] number of NV pairs in the list.
 *
 * @param ppNVPair [in] pointer to an list of NV Pairs to be searched.
 *
 * @param pName [in] null terminated Name string to be searched.
 *
 * @param ignoreCase [in] boolean variable indicating that case to be 
 *		ignored in the string comparison.
 *
 * @return If the search results in a match of given pName string and 
 *		the list of NV Pairs it will return the index of the match pair. 
 *		-1 is returned if there is no match.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMNVPGetParamIndexByUTF8Name(
				_IN s32 numNVPair, 
				_IN astring **ppNVPair, 
				_IN const astring *pName, 
				_IN booln ignoreCase
				);


//---------------------------------------------------------------------
/**
 * Returns the value of a given name in an list of NV pair strings.
 *
 * This method compares the given string with the name strings in the
 * list of NV pairs, depending on the ignoreCase variable. Once the 
 * match is found it returns the Value portion of the NV pair. If there 
 * is no match then NULL pointer is returned. 
 * 
 * If any of the NV pairs in the list is not a valid NV pair then it 
 * just skips to the next one.
 *
 * @param numNVPair	[in] number of NV pairs in the list.
 *
 * @param ppNVPair [in] pointer to an list of NV Pairs to be searched.
 *
 * @param pName [in] null terminated Name string to be searched.
 *
 * @param ignoreCase [in] boolean variable indicating that case to be 
 *		ignored in the string comparison.
 *
 * @return If the search results in a match of given pName string and  
 *		the list of NV Pairs it will return the pointer to a value 
 *		portion of the matched NV pair. NULL pointer is returned if 
 *		there is no match.
 */
//---------------------------------------------------------------------

SMINTF const astring * _cdecl SMNVPGetUTF8ParamValueByUTF8Name(
				_IN s32 numNVPair, 
				_IN astring **ppNVPair, 
				_IN const astring *pName, 
				_IN booln ignoreCase
				);


//---------------------------------------------------------------------
/**
 * Search the given name in an list of NV pair strings, converts the 
 * value to specified data type.
 *
 * This method compares the given string with the name strings in the
 * list of NV pairs, depending on the ignoreCase variable. Once the 
 * match is found it converts the Value portion of the NV pair to the 
 * specified data type, then writes to the output buffer.
 * 
 * The index of matched NV pair will be returned. If there is no match
 * then -1 will be returned.
 * 
 * If any of the NV pairs in the list is not a valid NV pair then it 
 * just skips to the next one.
 *
 * @param numNVPair	[in] number of NV pairs in the list.
 *
 * @param ppNVPair [in] pointer to an list of NV Pairs to be searched.
 *
 * @param pName [in] null terminated Name string to be searched.
 *
 * @param bIgnoreCase [in] boolean variable indicating that case to be 
 *		ignored in the string comparison.
 *
 * @param pValBuf [in] user allocated output value buffer, of 
 *		sufficient size.
 *
 * @param pVBufSize [inout] size of pValBuf.
 *
 * @param vType [in] type of value data to be converted.
 *
 * @return If the search results in a match of given pName string and 
 *		the list of NV Pairs it will return the index of the match pair. 
 *		-1 is returned if there is no match.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMNVPUTF8ToTypeValue(
				_IN s32 numNVPair,			// Name-Value-Pair count
				_IN astring **ppNVPair,		// Name-Value-Pair list reference
				_IN const astring *pName,	// The Name string to query
				_IN booln bIgnoreCase,		// Set to TRUE to ignore case
				_OUT void *pValBuf,			// user allocated buffer of sufficient size
				_INOUT u32 *pVBufSize,		// size of user buffer
				_IN u32 vType 				// type of value data to be converted 
				);


//---------------------------------------------------------------------
/**
 * Searches NVP list for the name given, compares the value string in 
 * the enum map provided and returns the corresponding enum value.
 *
 * The enum map must have the first element as the invalid value and 
 * the second element as the default value. Set isBitMask if the enum 
 * is a bit mask type.
 *
 * The returned value is the "OR" of the read value in the INI. The 
 * value must be separated by '|'. 
 *
 * On error, SM_ENUMs32_ERROR_VAL is returned.
 * 
 * 
 * @param numNVPair	[in] number of NV pairs in the list.
 *
 * @param ppNVPair [in] pointer to an list of NV Pairs to be searched.
 *
 * @param pName [in] null terminated Name string to be searched.
 *
 * @param bIgnoreCase [in] boolean variable indicating that case to be 
 *		ignored in the string comparison.
 *
 * @param pEnumMap [in] provided enum map for comparision.
 * 
 * @param enumMapCount [in] count of pEnumMap.
 *
 * @param isBitMask [in] whether this is a bit mask.
 *
 * @ return Returns default enum value if INI value does not exist, or 
 *		invalid enum value if INI value does not match any enum string in map, 
 *		or value 0 if no match is found for bitmaps.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMNVPGetEnums32Value(
				_IN s32 numNVPair,			// Name-Value-Pair count
				_IN astring **ppNVPair,		// Name-Value-Pair list reference
				_IN const astring *pName,	// The Name string to query
				_IN booln bIgnoreCase,		// Set to TRUE to ignore case
				_IN const SMEnums32UTF8Map *pEnumMap,
				_IN s32 enumMapCount,
				_IN booln isBitMask
				);


//---------------------------------------------------------------------
/**
 * Given a NV pair string it returns the value string.
 *
 * This API method checks the given pNVPair valid or not by using 
 * function SMGetUTF8StrParamValue. Depending on the resultl success or 
 * failure it validates the given NV pair string.
 *
 * @param pNVPair [in] null terminated NV pair string.
 *
 * @return 	If the passed NV Pair is valid then it will return TRUE; 
 *		else it will return FALSE.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMIsValidParam(_IN const astring *pNVPair);



//---------------------------------------------------------------------
// Properties/Config file manipulation routines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Reads the value from a properties/config file given key names.
 *
 * This function can be used to read a value from an properties/config 
 * file, given file name with path, and the key value.
 *
 * The value can be of any type. 
 * For example it could be an u32 value, or could be an array of bytes. 
 * 
 * If the function could not find the key in the properties/config file, 
 * then it will copy the default value to the buffer provided. 
 *
 * The canBlock parameter can be used to specify whether to block or 
 * return immediately if the file is locked by some other thread/process. 
 * 
 * This function can also be used to get all the keys in the file by 
 * passing the pKey parameter as NULL.  
 *
 * To get the all keys the vType parameter should be INI_ASTRING.
 *
 * @param pKey [in] Key to be searched in the properties/config file. 
 *		If NULL supplied then all key names are read into the supplied 
 *		pValue buffer.  Each key name is '\\0' character delimited,
 *		with the last key name terminated with an additional '\\0'.
 *
 * @param valueType [in] Value type to be read. Type is INI_ASTRING 
 *		if NULL supplied to pKey.
 *
 * @param pValue [out] pointer to value data buffer to be read into.
 *
 * @param pValueSize [inout] size of value data buffer.
 *
 * @param pDefaultValue [in] pointer to a buffer contains default value to 
 *		return in case of failure to locate key.
 *
 * @param defaultValueSize	[in] size of default value data buffer.
 * 
 * @param pPathFileName [in] Path and file name to read.
 *
 * @param bCanBlock [in] Boolean value to enable or disable blocking 
 *		in the function.
 * 
 * @return Upon success return SM_STATUS_SUCCESS. 
 *		On error returns the status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMPropertyFileReadValue(
				_IN const astring *pKey,
				_IN u32 valueType,
				_OUT void *pValue,
				_INOUT u32 *pValueSize,
				_IN const void *pDefaultValue,
				_IN u32 defaultValueSize,
				_IN const astring *pPathFileName,
				_IN booln bCanBlock
				);


//---------------------------------------------------------------------
/**
 * Writes a value to a properties/config file given key names. 
 *
 * This function can be used to write a value to an properties/config  
 * file, given file name with path, and the key value.
 *
 * The value can be of any type. For example it could be an u32 value 
 * or could be an array of bytes.
 *
 * The canBlock parameter can be used to specify whether to block or 
 * return immediately if the file is locked by some other thread/process.
 *
 * @param pKey [in] Key to be searched in the section.
 *
 * @param valueType	[in] Value type to be written.
 *
 * @param pValue [in] pointer to value data buffer to be read into.
 *
 * @param valueSize [in] size of value data buffer.
 *
 * @param pPathFileName [in] Path and file name to read.
 *
 * @param bCanBlock [in] Boolean value to enable or disable blocking 
 *		in the function.
 * 
 * @return On success returns SM_STATUS_SUCCESS, 
 *		on error returns the status code defined in dcsmstatus.h.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMPropertyFileWriteValue(
				_IN const astring *pKey,
				_IN u32 valueType,
				_IN void *pValue,
				_IN u32 valueSize,
				_IN const astring *pPathFileName,
				_IN booln bCanBlock
				);


//---------------------------------------------------------------------
// RedBlackTree container and search routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * This function inserts a pData key into the red black tree.
 *
 * @param ppRootNode [inout] reference to the existing root node.
 *
 * @param pInsertData [in] reference to the data to be inserted.
 *
 * @param pWalkData	[in] contains a reference similar to pInsertData in 
 *		order to help it perform comparison function.
 *
 * @param pfnWalk [in] pointer to user provided function that helps 
 *		determine the insertion point if any in the tree.
 *
 * @return On success it returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_OBJECT_ALREADY_EXISTS is returned if the 
 *		pInsertData already exists in the heap.
 *
 * @sa SMRedBlackTreeDataDelete()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMRedBlackTreeDataInsert(_INOUT SMRedBlackTreeNode **ppRootNode, 
										   _IN SMRedBlackTreeNodeData *pInsertData, 
										   _INOUT void *pWalkData, 
										   _IN PFNSMRBTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Replaces pData key in the red black tree.
 *
 * @param ppRootNode [inout] reference to the existing root node.
 *
 * @param pReplaceData [in] reference to the data to be inserted.
 *
 * @param pWalkData	[in] contains a reference similar to pInsertData in 
 *		order to help it perform comparison function.
 *
 * @param pfnWalk [in] pointer to user provided function that helps 
 *		determine the insertion point if any in the tree.
 *
 * @return On success it returns SM_STATUS_SUCCESS.
 *
 * @return SM_STATUS_OBJECT_ALREADY_EXISTS is returned if the 
 *		pInsertData already exists in the heap.
 */
//---------------------------------------------------------------------

SMINTF SMRedBlackTreeNodeData * _cdecl SMRedBlackTreeDataReplace(_INOUT SMRedBlackTreeNode **ppRootNode, 
																 _IN SMRedBlackTreeNodeData *pReplaceData, 
																 _INOUT void *pWalkData, 
																 _IN PFNSMRBTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Deletes pData key from red black tree.
 *
 * @param ppRootNode [inout] reference to the existing root node.
 *
 * @param pWalkData	[inoout] contains a reference to the key to be 
 * deleted so as to match it with a node in the tree.
 *
 * @param pfnWalk [in] pointer to user provided function that helps 
 * determine delete point if any in the tree.
 *
 * @return Returns SM_STATUS_SUCCESS on success.
 *
 * @return Returns SM_STATUS_NO_SUCH_OBJECT if match not found.
 *
 * @sa SMRedBlackTreeDataInsert()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMRedBlackTreeDataDelete(_INOUT SMRedBlackTreeNode **ppRootNode, 
										   _INOUT void *pWalkData, 
										   _IN PFNSMRBTWALK pfnWalk);


//---------------------------------------------------------------------
/**
 * Delete all nodes in the red black tree.
 *
 * @param ppRootNode [inout] reference to the existing root node.
 *
 * @param pNotifyDelData [inout] pointer to deleted data.
 *
 * @param pfnNotifyDel [in] calls the notify function with the data 
 *		contents of each node.
 *
 * @note The notify function and data are optional.
 *
 * @return None
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRedBlackTreeDelete(_INOUT SMRedBlackTreeNode **ppRootNode, 
										_INOUT void *pNotifyDelData, 
										_IN PFNSMRBTNOTIFYDEL pfnNotifyDel);


//---------------------------------------------------------------------
/**
 * Walk the red black tree starting at a given node. 
 * 
 * The walk is in: \n
 * ascending order if walkType == SM_RBT_WALKTYPE_ASCENDING_ORDER; \n
 * descending order if walkType == SM_RBT_WALKTYPE_DESCENDING_ORDER; \n
 * in correct order (in-order) if walkType == SM_RBT_WALKTYPE_FIND_DATA; \n
 * in order specified by walk function if walkType == SM_RBT_WALKTYPE_WALK_ORDER.
 *
 * @param pNode	[in] reference to an exisiting RBT node.
 *
 * @param pWalkData	[inout] contains a reference to the key to be 
 *		deleted so as to match it with a node in the tree.
 *
 * @param pfnWalk [in] pointer to user provided function that helps 
 *		determine delete point if any in the tree.
 *
 * @param walkType [in] determines the type of walk to be performed. 
 *		See details in the description section.
 *
 * @return if pfnWalk returns 0 then walk is terminated and pData 
 *		key returned from the node at which walk was terminated.
 *
 * @return NULL is returned if pfnWalk returns non-zero or on other 
 *		error.
 */
//---------------------------------------------------------------------

SMINTF SMRedBlackTreeNodeData * _cdecl SMRedBlackTreeDataWalk(_IN const SMRedBlackTreeNode *pNode, 
															  _INOUT void *pWalkData, 
															  _IN PFNSMRBTWALK pfnWalk,
															  _IN s32 walkType);


//---------------------------------------------------------------------
/**
 * Checks to see if the RBT is empty or contains data nodes.
 *
 * @param pNode	[in] reference to an exisiting RBTree node entry.
 *
 * @return Returns TRUE if red black node has no data nodes.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMRedBlackTreeIsEmpty(_IN const SMRedBlackTreeNode *pNode);


//---------------------------------------------------------------------
/**
 * Initializes the tree and root pointer for a RBT.
 *
 * @param ppRootNode [inout] reference to an RBTree root node.
 * 
 * @return None.
 *
 * @sa SMRedBlackTreeDetach()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRedBlackTreeAttach(_INOUT SMRedBlackTreeNode **ppRootNode);


//---------------------------------------------------------------------
/**
 * Releases the tree and root pointer assuming all nodes have been 
 * previously deleted.
 *
 * @param ppRootNode [inout] reference to an RBTree root node.
 *
 * @return None.
 *
 * @sa SMRedBlackTreeAttach()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRedBlackTreeDetach(_INOUT SMRedBlackTreeNode **ppRootNode);



//---------------------------------------------------------------------
// Log support API
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Appends an event data to Operating System event log. 
 * 
 * This function is used to log an event. The entry is written to the 
 * end of the specified log file for the source identified by the pSrc 
 * parameter.
 * 
 * The OS event logging for each OS is described below:
 *
 * Windows - The function internally registers with the OS to get a 
 * handle which represents the source and adds the event description 
 * string as an event to the system log.
 * 
 * Linux - opens the system log with LOG_DAEMON as the facility once 
 * succeeded it adds the event to the system log. If there are any 
 * carriage returns in the string are removed before adding to the 
 * system log.
 *
 * Netware - not supported
 *
 * @param type [in] Specifies the type of event being logged. This 
 *		parameter can be one of the following values: \n
 *		Symbol:						Meaning \n
 *		SMLOG_TYPE_SUCCESS:			Success event \n
 *		SMLOG_TYPE_ERROR:			Error event \n
 *		SMLOG_TYPE_WARNING:			Warning event \n
 *		SMLOG_TYPE_INFORMATION:		Information event \n
 *		SMLOG_TYPE_AUDIT_SUCCESS:	Success audit event \n
 *		SMLOG_TYPE_AUDIT_FAILURE:	Failure audit event \n
 *
 * @param category [in] ID assigned to the message category name.
 *
 * @param eventID [in] ID associated with event, For NT/W2k OS logs 
 *		The event identifier specifies the entry in the message file 
 *		associated with the event source.
 *
 * @param pUTF8Source [in] null terminated sting specifies who 
 *		generated the event.
 *
 * @param pUTF8EventDesc [in] Description of the event.
 *
 * @param pData	[in] pointer to an optional event Data.
 *
 * @param dataBufSize [in] size in bytes of optional event data.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the entry was written to 
 *		the log.
 *
 * @return If the function fails, following are the possible return 
 *		codes:
 *
 * @return SM_STATUS_CMD_BAD - if the facility provided is not 
 *		supported or any of the provided parameters are not valid.
 *
 * @return SM_STATUS_RESOURCE_LOCKED - if the file to be written is 
 *		locked by some other thread or process.
 *
 * @return SM_STATUS_DEVICE_FULL - If the log is full.
 * 
 * @return SM_STATUS_UNSUCCESSFUL - for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMAppendToOSSysLog(_IN u16 type, 
									 _IN u16 category, 
									 _IN u32 eventID, 
									 _IN const astring *pUTF8Source, 
									 _IN const astring *pUTF8EventDesc, 
									 _IN const void *pData, 
									 _IN u32 dataBufSize);


//---------------------------------------------------------------------
/**
 * The function gets the get the XML backup log path file name.
 *
 * @param pPathFileName [in] path and filename of XML log.
 *
 * @param pBufBakPathFileName [in] pointer to string contains return 
 *		value of the backup XML log path file name.
 *
 * @param pBufSize [in] size of the buffer for pBufBakPathFileName.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating  the XML backup log path file 
 *		name available.
 *
 * @return If the function fails, following are the possible return 
 *		codes:
 *
 * @return SM_STATUS_CMD_BAD any of the provided parameters are not 
 *		valid.
 *
 * @return SM_STATUS_DATA_OVERRUN If there was not sufficient buffer 
 *		space provided to copy the backup log path file name.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetBackupXMLLogPathFileName(_IN const astring *pPathFileName, 
												_OUT astring *pBufBakPathFileName, 
												_INOUT u32 *pBufSize);


//---------------------------------------------------------------------
/**
 * Appends an event data to a specified XML log. 
 * The function SMAppendToXMLLog() adds XML event data to a specified 
 * file. The path and file will be created if both or one them does not 
 * exist. The entry is written to the end of the specified log file for 
 * the source identified by the pSrc parameter.  
 *
 * @param pPathFileName [in] path and filename of XML log.
 * 
 * @param type [in] Specifies the type of event being logged. This 
 *		parameter can be one of the following values. 
 *		Symbol:						Meaning \n
 *		SMLOG_TYPE_SUCCESS:			Success event \n
 *		SMLOG_TYPE_ERROR:			Error event \n
 *		SMLOG_TYPE_WARNING:			Warning event \n
 *		SMLOG_TYPE_INFORMATION:		Information event \n
 *		SMLOG_TYPE_AUDIT_SUCCESS:	Success audit event \n
 *		SMLOG_TYPE_AUDIT_FAILURE:	Failure audit event
 *
 * @param category [in] ID assigned to the message category name.
 *
 * @param eventID [in] ID associated with event.
 *
 * @param pUserInfo [in] pointer to string contains domain and logon 
 *		name of user.
 *
 * @param pSource [in] null terminated sting specifies who generated 
 *		the event.
 *
 * @param pEventDesc [in] Description of the event.
 *
 * @param pEventHints [in] Absolute install pathfilename for the event hints file.
 *
 * @param pData	[in] pointer to an optional event Data.
 *
 * @param dataBufSize [in] size in bytes of optional event data.
 *
 * @param maxLogSize [in] maximum allowed log size.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the entry was written to the 
 *		log.
 *
 * @return If the function fails, following are the possible return 
 *		codes:
 *
 * @return SM_STATUS_CMD_BAD - if the facility provided is not supported 
 *		or any of the provided parameters are not valid.
 *
 * @return SM_STATUS_RESOURCE_LOCKED - if the file to be written is 
 *		locked by some other thread or process.
 *
 * @return SM_STATUS_DEVICE_FULL - If the log is full.
 *
 * @return SM_STATUS_UNSUCCESSFUL - for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMAppendToXMLLog(_IN const astring *pPathFileName, 
								   _IN u16 type, 
								   _IN u16 category, 
								   _IN u32 eventID, 
								   _IN const astring *pUserInfo, 
								   _IN const astring *pSource, 
								   _IN const astring *pEventDesc, 
								   _IN const astring *pEventHints, 
								   _IN const void *pData, 
								   _IN u32 dataBufSize, 
								   _IN u32 maxLogSize);


//---------------------------------------------------------------------
/**
 * The function forces the XML log to exist.
 *
 * @param pPathFileName [in] path and filename of XML log.
 *
 * @param category [in] ID assigned to log category.
 *
 * @param pUserInfo [in] pointer to string contains domain and logon 
 *		name of user.
 *
 * @param pSource [in] source name that forced the file creation.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating  the XML log file exisits.
 *
 * @return If the function fails, the return codes are same as for 
 *		SMAppendToXMLLog().
 *
 * @sa SMAppendToXMLLog()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMForceXMLLogExists(_IN const astring *pPathFileName, 
									  _IN u16 category, 
									  _IN const astring *pUserInfo, 
									  _IN const astring *pSource);


//---------------------------------------------------------------------
/**
 * Clears the specified XML log. 
 *
 * The SMClearXMLLog function clears the log file specified and adds a 
 * cleared event to the log with the specified source name. 
 *
 * @param pPathFileName [in] path and filename of XML log.
 *
 * @param category [in] ID assgined to the message category name.
 *
 * @param pUserInfo [in] pointer to string contains domain and logon 
 *		name of user.
 *
 * @param pSource [in] source name for the cleared event to be added.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the log has been cleared.
 *
 * @return If the function fails, following are the possible return 
 *		codes:
 *
 * @return SM_STATUS_CMD_BAD any of the provided parameters are not 
 *		valid.
 *
 * @return SM_STATUS_RESOURCE_LOCKED if the file to be written is 
 *		locked by some other thread or process.
 *
 * @return SM_STATUS_UNSUCCESSFUL for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMClearXMLLog(_IN const astring *pPathFileName, 
								_IN u16 category, 
								_IN const astring *pUserInfo, 
								_IN const astring *pSource);


//---------------------------------------------------------------------
// Product Identifier to Install Path Generation Routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Returns an absolute pathfilename based on product identifer,
 * an optional subpath and the given filename. 
 * 
 * Install path is used for specified product.
 * See the SM_PSUBPATH definitions for pSubPath.
 *
 * Use SMFreeGeneric to release returned memory.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMMakePathFileNameByPID(_IN s32 productID,
												_IN const astring *pSubPath,
												_IN const astring *pFileName);


//---------------------------------------------------------------------
/**
 * Returns an absolute pathfilename based on product identifer,
 * path type, an optional subpath and the given filename.
 * 
 * See the SM_PATH_TYPE definitions for pathType.
 * See the SM_PSUBPATH definitions for pSubPath.
 *
 * Use SMFreeGeneric to release returned memory.
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMMakePathFileNameByPIDAndType(_IN s32 productID,
													   _IN u32 pathType,
													   _IN const astring *pSubPath,
													   _IN const astring *pFileName);


//---------------------------------------------------------------------
/**
 * productID: Identification searched case-insensitive
 * pathType: specifies the type of path needed (log, install, vardata etc)
 * pBufPath: preallocated buffer to store the result
 * pBufSize: result buffer size, always assigned with size of result string
 * Returns SM_STATUS_NO_SUCH_OBJECT when entry for given PID is not found
 *         SM_STATUS_DATA_OVERRUN when output buffer is insufficient
 *         SM_STATUS_OBJECT_BAD when path is not in the file system
 *         SM_STATUS_SUCCESS otherwise
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetPathByProductIDandType(_IN s32 productID, 
											  _IN u32 pathType, 
											  _OUT astring *pBufPath, 
											  _INOUT u32 *pBufSize);


//---------------------------------------------------------------------
/**
 * Looks for the "\@OmInstallPath\@" substring in a string.
 * If found, it replaces the substring with the value of the
 * root OM install path.
 * Note that function only expands the first instance of
 * "\@OmInstallPath\@" in the string.
 * pSrcPath: preallocated source buffer with a string to be expanded
 * pDstPath: preallocated output buffer to store the resulting expanded string
 * pDstSize: result buffer size, always assigned with size of result string
 * Returns SM_STATUS_INVALID_PARAMETER when pSrcPath pointer is NULL
 *         SM_STATUS_INVALID_PARAMETER when output buffer size appears sufficient
 *                                     but output buffer pointer is NULL
 *         SM_STATUS_DATA_OVERRUN when output buffer size is insufficient
 *         SM_STATUS_UNSUCCESSFUL whene general errors occur
 *         SM_STATUS_SUCCESS otherwise
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMExpandOmInstallPathString(
				_IN const astring *pSrcBuf, 
				_OUT astring *pDstBuf, 
				_INOUT u32 *pDstBufSize
				);

				
//---------------------------------------------------------------------
/**
 * pProductSID: Identification searched case-insensitive example: "dataeng"
 * Returns: SM_PRODUCT_ID_INVALID if SID is unknown
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMProductSIDToID(_IN const astring *pProductSID);


//---------------------------------------------------------------------
/**
 * Use SMFreeGeneric to free returned memory
 * productID: SM_PRODUCT_ID_XXX
 * Returns: NULL if ID is unkown
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMProductIDToSID(_IN s32 productID);



//---------------------------------------------------------------------
// Self-Describing Object (SDO) Configuration API
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Allocates memory for an SDO Configuration object.
 *
 * Use the function SDOConfigFree() to free this memory.
 *
 * @return Pointer to a newly allocated SDO Config object. NULL if 
 *		malloc fails.
 *
 * @sa SMSDOConfigFree()
 */
//---------------------------------------------------------------------

SMINTF SDOConfig * _cdecl SMSDOConfigAlloc(void);


//---------------------------------------------------------------------
/**
 * Frees the memory allocated from the call to SDOConfigAlloc. 
 * This function makes necessary removal and free of any data still
 * contained in the SDO configuration object.
 *
 * @param pSDC [in] pointer to the SDO configuration object to be freed.
 *
 * @return None.
 *
 * @sa SMSDOConfigAlloc
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMSDOConfigFree(_IN SDOConfig *pSDC);


//---------------------------------------------------------------------
/**
 * Creates a newly allocated SDOConfig copy of the input SDOConfig.
 *
 * All memory that is referenced by the source SDOConfig will be
 * copied to a new memory area.  Use the function SDOConfigFree
 * to free the memory returned by this function.
 *
 * @param pSDC [in] pointer to the SDO configuration object to be cloned.
 *
 * @return Pointer to a newly cloned SDO Config object. NULL if clone
 *		fails.
 */
//---------------------------------------------------------------------

SMINTF SDOConfig * _cdecl SMSDOConfigClone(_IN const SDOConfig *pSDC);


//---------------------------------------------------------------------
/**
 * Copmare two SDOConfig object.
 *
 * Returns TRUE if and only if the values contained in both the
 * input SDOConfigs are the same; and if both are NULL.
 *
 * @param pSDC1 [in] pointer to the SDO configuration objects to 
 *		be compared.
 *
 * @param pSDC2 [in] pointer to the SDO configuration objects to 
 *		be compared.
 *
 * @return Returns TRUE if and only if the values contained in both the
 * input SDOConfigs are the same; and if both are NULL.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMSDOConfigCompareByValue(_IN const SDOConfig *pSDC1,
											  _IN const SDOConfig *pSDC2);


//---------------------------------------------------------------------
/**
 * Returns the number of data element contained in the SDO
 * Configuration object.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @return The number of data element contained in the SDO 
 *		Configuration object.  Returns a value of -1 on error.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigGetCount(_IN const SDOConfig *pSDC);


//---------------------------------------------------------------------
/**
 * Add or Replace data to the SDO Config object.  
 * 
 * The fieldID is a unique identifyer for the data.  When successful, 
 * the contents of the data buffer will be copied.  When bReplace is 
 * TRUE, an existing data will be destroyed matching the fieldID and 
 * replaced with the input data; if it cannot find the existing data, 
 * it just adds it.  
 *
 * The dataSize is in bytes.
 *
 * @param pSDC [inout] pointer to the SDO configuration object.
 *
 * @param fieldID [in] the field ID of the data to be added.
 *
 * @param dataType [in] the data type signature of the data to be added.
 *
 * @param pDataBuf [in] const pointer to the memory data buffer to be 
 *		added.
 *
 * @param dataSize [in] the size in bytes of the data buffer.
 *
 * @param bReplace [in] boolean flag to replace data when it already 
 *		exist in the object.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigAddData(
				_INOUT SDOConfig *pSDC,
				_IN u16 fieldID,
				_IN u8 dataType,
				_IN const void *pDataBuf,
				_IN const u32 dataSize,
				_IN booln bReplace
				);


//---------------------------------------------------------------------
/**
 * SDO Configuration object data query by Field ID.  
 * The user provides a sufficient data buffer and size to copy the 
 * resulting data into. The data type is also returned.  
 *
 * The dataSize is in bytes. 
 *
 * SM_STATUS_DATA_OVERRUN is returned when buffer is insufficient.
 *
 * Optional parameters may take a NULL value; they are pDataType.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @param fieldID [in] the field ID of the data to query.
 *
 * @param pDataType [out] the type signature of the result data.
 *
 * @param pDataBuf [out] preallocated memory data buffer to store the 
 *		result data.
 *
 * @param pDataSize [inout] the size in bytes, of the input buffer and 
 *		of the result data.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigGetDataByID(
				_IN const SDOConfig *pSDC,
				_IN u16 fieldID,
				_OUT u8 *pDataType,
				_OUT void *pDataBuf,
				_INOUT u32 *pDataSize
				);


//---------------------------------------------------------------------
/**
 * SDO Configuration object data query by insertion index.  
 * The user provides a sufficient data buffer and size to copy the 
 * resulting data into. The data type and field ID is also returned.
 *
 * The dataSize is in bytes.
 *
 * SM_STATUS_DATA_OVERRUN is returned when buffer is insufficient.
 *
 * Optional parameters may take a NULL value; they are pDataType.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @param index [in] the index of the data to query.
 * 
 * @param pFieldID [in] the field ID of the data to query.
 *
 * @param pDataType [out] the type signature of the result data.
 *
 * @param pDataBuf [out] preallocated memory data buffer to store the 
 *		result data.
 *
 * @param pDataSize [inout] the size in bytes, of the input buffer and 
 *		of the result data.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigGetDataByIndex(
				_IN const SDOConfig *pSDC,
				_IN u32 index,
				_OUT u16 *pFieldID,
				_OUT u8 *pDataType,
				_OUT void *pDataBuf,
				_INOUT u32 *pDataSize
				);


//---------------------------------------------------------------------
/**
 * SDO Configuration object data removal by field ID or by insertion
 * index.  
 *
 * If bRemoveAll is set to TRUE, all data contained are removed. 
 *
 * The order of precedence is bRemoveAll, fieldID (when bRemoveAll is 
 * FALSE), and insertion index (when fieldID is zero).
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @param fieldID [in] the field ID the data to query.
 *
 * @param index [in] the index of the data to query.
 *
 * @param bRemoveAll [in] boolean flag to remove all data contained 
 *		within the object.
 *
 * @return Returns SM_STATUS_SUCCESS on successful remove.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigRemoveData(
				_INOUT SDOConfig *pSDC,
				_IN u16 fieldID,
				_IN u32 index,
				_IN booln bRemoveAll
				);


//---------------------------------------------------------------------
// Self-Describing Object (SDO) Transformation API
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Returns the size of SDO configuration when converted to binary format. 
 *
 * Use this value to preallocate memory buffer for the SDO binary 
 * conversion, call to SMSDOConfigToBinary(). 
 *
 * Returns 0 if pSDC is NULL.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @return Returns the binary size in bytes.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMSDOConfigGetBinarySize(_IN const SDOConfig *pSDC);


//---------------------------------------------------------------------
/** 
 * Calculates the size of SDO Configuration when converted to binary format.
 * 
 * It differs from SDOConfigGetBinarySize() such that it browses through
 * the database and recalculates the binary size. The later uses the 
 * saved binary size value.
 * 
 * Use this value to preallocate memory buffer for the SDO Binary
 * conversion, call to SMSDOConfigToBinary().
 *
 * Returns 0 if pSDC is NULL.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @return Returns the binary size in bytes.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMSDOConfigReCalcBinarySize(_IN const SDOConfig *pSDC);


//---------------------------------------------------------------------
/**
 * Transform an SDO Configuration object into an SDO Binary object.
 *
 * The pSDBSize always returns the size (in bytes) of a resulting 
 * SDO Binary.  The user provides a preallocated target buffer.
 * SM_STATUS_DATA_OVERRUN is returned when buffer is insufficient.
 *
 * @param pSDC [in] const pointer to the SDO configuration object.
 *
 * @param pSDB [out] user allocated memory buffer to store the SDO 
 *		Binary object.
 * 
 * @param pSDBSize [inout] size in bytes, of the buffer input and of 
 *		the result object.
 *
 * @return Returns SM_STATUS_SUCCESS on successful transformation.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigToBinary(
				_IN const SDOConfig *pSDC,
				_OUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize
				);


//---------------------------------------------------------------------
/**
 * Transorm an SDO Binary object into an SDO Configuration object.
 * 
 * A newly allocated memory for SDO Configuration is returned on 
 * success. Use SDOConfigFree to free the object returned from here.
 * 
 * @param pSDB [in] const pointer to the SDOBinary object.
 *
 * @return Returns a pointer to a newly allocated SDO Config object 
 *		containing a copy of data in the input SDOBinary object.  
 *		NULL if malloc fails or the source object is NULL or corrupt.
 */
//---------------------------------------------------------------------

SMINTF SDOConfig * _cdecl SMSDOBinaryToConfig(_IN const SDOBinary *pSDB);



//---------------------------------------------------------------------
// Self-Describing Object (SDO) Array Transformation API
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Transform an array of SDOConfig objects into an SDOBinary object
 * of array type.  User provides a preallocated memory for pSDBArray
 * of sufficient size; otherwise, the size will be set to the required
 * size.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOConfigArrayToBinaryArray(
				_IN const SDOConfig **ppSDCArray,
				_IN s32 sdcArrayCount,
				_OUT SDOBinary *pSDBArray,
				_INOUT u32 *pSDBArraySize
				);


//---------------------------------------------------------------------
/**
 * Transform an SDOBinary object of array type into an array of
 * SDOConfig objects.  User provides a preallocated memory for 
 * ppSDCArray, which will be filled with references to the transformed
 * SDOConfig objects from each of the SDOBinary object entries.
 * ***NOTE*** that the user is responsible for calling SMSDOConfigFree
 * on each of the references assign onto the ppSDCArray.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryArrayToConfigArray(
				_IN SDOBinary *pSDBArray,
				_OUT SDOConfig **ppSDCArray,
				_INOUT s32 *pSDCArrayCount
				);


//---------------------------------------------------------------------
// Self-Describing Object (SDO) Binary (BLOB) API
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Returns the number of data contained in the SDO Binary object
 * or -1 if pSDB is NULL or has invalid SDO signature.
 *
 * Optional booln parameter pIsArray is set to TRUE when the
 * SDOBinary object is of type array.
 *
 * @param pSDB [in] const pointer to the SDOBinary object.
 *
 * @param pIsArray [out] set to TRUE when the SDOBinary object is of 
 * type array.
 * 
 * @return The number of data element contained in the SDO Configuration 
 *		object.  Returns a value of -1 on error.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryGetCount(
				_IN const SDOBinary *pSDB,
				_OUT booln *pIsArray
				);


//---------------------------------------------------------------------
/**
 * Return the size of the SDO binary in bytes.  
 * Returns 0 if pSDB is NULL or if it is not an SDO memory.  
 * 
 * Optional pIsArray is set to true if the SDO binary is of type array.
 *
 * @param pSDB [in] const pointer to the SDOBinary object.
 *
 * @param pIsArray [out] set to TRUE when the SDOBinary object is of 
 * type array.
 * 
 * @return The size of SDO binary in bytes.
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMSDOBinaryGetSize(
				_IN const SDOBinary *pSDB,
				_OUT booln *pIsArray
				);


//---------------------------------------------------------------------
/** 
 * Add or Replace data to the SDO Binary object.  
 * 
 * The fieldID is a unique identifyer for the data.  
 *
 * When successful, the contents of the data buffer will be copied.  
 * The user must specify the buffer size, pSDBSize, in bytes allocated 
 * for pSDB.  Any existing data will be destroyed matching the fieldID 
 * and replaced with the input data; if it cannot find the existing 
 * data, it just adds it.  
 * 
 * The dataSize is in bytes.
 *
 * @param pSDB [inout] pointer to the SDOBinary object.
 *
 * @param pSDBSize [inout] size of SDOBinary object.
 *
 * @param fieldID [in] the field ID of the data to query and set.
 *
 * @param dataType [in] the type signature of the data to query and set.
 *
 * @param pDataBuf [in] the data buffer holding the value to set.
 *
 * @param dataSize [in] the size in bytes of the data buffer to set.
 *
 * @return Returns SM_STATUS_SUCCESS on successful set.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinarySetData(
				_INOUT SDOBinary *pSDB,
				_INOUT u32 *pSDBSize,
				_IN u16 fieldID,
				_IN u8 dataType,
				_IN const void *pDataBuf,
				_IN const u32 dataSize
				);


//---------------------------------------------------------------------
/**
 * See SDOConfigGetDataByID(), which is similar except that the source
 * object is an SDO Binary.
 *
 * Optional parameters may take a NULL value; they are pDataType.
 *
 * @param pSDB [inout] pointer to the SDOBinary object.
 *
 * @param fieldID [in] the field ID of the data to query.
 *
 * @param pDataType [out] the type signature of the data found.
 *
 * @param pDataBuf [out] user memory allocated buffer to store data found.
 *
 * @param pDataSize [inout] size in bytes, of the input buffer and of 
 *		the data found.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 *
 * @sa SDOConfigGetDataByID()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryGetDataByID(
				_IN const SDOBinary *pSDB,
				_IN u16 fieldID,
				_OUT u8 *pDataType,
				_OUT void *pDataBuf,
				_INOUT u32 *pDataSize
				);


//---------------------------------------------------------------------
/**
 *
 * See SDOConfigGetDataByIndex, which is similar except that the source
 * object is an SDO Binary.
 *
 * Optional parameters may take a NULL value; they are pDataType.
 *
 * @param pSDB [inout] pointer to the SDOBinary object.
 *
 * @param index [in] the index of the data to query.
 *
 * @param pFieldID [in] the field ID of the data to query.
 *
 * @param pDataType [out] the type signature of the data found.
 *
 * @param pDataBuf [out] user memory allocated buffer to store data found.
 *
 * @param pDataSize [inout] size in bytes, of the input buffer and of 
 *		the data found.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 *
 * @sa SDOConfigGetDataByIndex()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryGetDataByIndex(
				_IN const SDOBinary *pSDB,
				_IN u32 index,
				_OUT u16 *pFieldID,
				_OUT u8 *pDataType,
				_OUT void *pDataBuf,
				_INOUT u32 *pDataSize
				);


//---------------------------------------------------------------------
/**
 * Walk the SDOBinary memory executing provided call backs as appropriate.
 *
 * Setting bIsRecursive to TRUE walks the binary top bottom in order.
 *
 * Returns 0 when walk finished to the end; undefined otherwise.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryWalk(
				_IN const SDOBinary *pSDB,
				_INOUT void *pWalkData,
				_IN PFNSDOBINARYWALKBINARYBEGIN pfnWalkBinaryBegin,
				_IN PFNSDOBINARYWALKFIELDBEGIN pfnWalkFieldBegin,
				_IN PFNSDOBINARYWALKFIELDEND pfnWalkFieldEndOpt,
				_IN PFNSDOBINARYWALKBINARYEND pfnWalkBinaryEndOpt,
				_IN booln bIsRecursive
				);


//---------------------------------------------------------------------
// Self-Describing Object (SDO) Binary Array (BLOB) API
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Initialize the object header of an SDOBinary Array type on the
 * given user memory buffer of sufficient size.  
 * 
 * An SDOBinary object has a size limit of 0xffff. sdbArraySize is the 
 * size in bytes of the input buffer.
 *
 * @param pSDBArray [out] pointer to the SDOBinary object of type Array.
 *
 * @param sdbArraySize [in] size, in bytes, of the SDOBinary object.
 *
 * @return Returns SM_STATUS_SUCCESS on successful init.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryArrayInitHeader(
				_OUT SDOBinary *pSDBArray,
				_IN const u32 sdbArraySize
				);


//---------------------------------------------------------------------
/**
 * Given user memory buffer of type SDOBinary Array of sufficient size.
 *
 * Limited to 0xffff in bytes, and initialized header using the 
 * SMSDOBinaryArrayInitHeader API, the SDOBinary object item will be 
 * appended and copied.  pSDBArraySize is the size in bytes of the
 * input buffer and will be reset to the size of the target buffer
 * after a successful add.
 *
 * @param pSDBArray [inout] pointer to the SDOBinary object of type Array.
 *
 * @param pSDBArraySize [inout] size, in bytes, of the buffer pointed 
 *		to by the array object above.
 *
 * @param pSDBItem [in] pointer to the SDOBinary object to add.
 *
 * @param sdbItemSize [in] size, in bytes, of the object to add.
 *
 * @return Returns SM_STATUS_SUCCESS on successful add.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryArrayAddData(
				_INOUT SDOBinary *pSDBArray, 
				_INOUT u32 *pSDBArraySize,
				_IN const SDOBinary *pSDBItem,
				_IN const u32 sdbItemSize
				);


//---------------------------------------------------------------------
/** 
 * Finds the Nth SDOBinary object contained in the SDOBinary Array
 * and copies it to the user buffer of sufficient size. 
 *
 * When found, pSDBItemSize is reset to the size of the found object.
 *
 * @param pSDBArray [in] const pointer to the SDOBinary object of type 
 *		Array.
 *
 * @param sdbArraySize [in] size, in bytes, of the buffer pointed to by 
 *		the array object above.
 *
 * @param index [in] the index of the object in the array to query.
 *
 * @param pSDBItem [out] user allocated buffer of type SDOBinary.
 *
 * @param pSDBItemSize [inout] size, in bytes, of the buffer above.
 *
 * @return Returns SM_STATUS_SUCCESS on successful query.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSDOBinaryArrayGetByIndex(
				_IN const SDOBinary *pSDBArray,
				_IN const u32 sdbArraySize,
				_IN u32 index,
				_OUT SDOBinary *pSDBItem,
				_INOUT u32 *pSDBItemSize
				);



//---------------------------------------------------------------------
// BASE64 support
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Base 64 encoding function.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMBase64Encode(_OUT char *outputBuf,
								 _IN const u8 *inputBuf,
								 _IN int count,
								 _IN const char *base);


//---------------------------------------------------------------------
/**
 * Base 64 decoding function.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMBase64Decode(_OUT u8 *outputBuf,
								 _IN const char *inputBuf,
								 _IN int count);



//---------------------------------------------------------------------
// XML Generation support functions
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Allocates a buffer of specified initial length, and adds a standard 
 * XML header to the string buffer if specified.
 *
 * The SMXGBufAlloc() function in turn calls the SMSSUTF8StrAlloc() 
 * function and in addition depending on the appendHeader parameter it 
 * will add the XML header to the allocated string buffer. Look at the 
 * SMSSUTF8StrAlloc() for more details. 
 *
 * @param initSize [in] initial buffer size to be allocated.
 * 
 * @param bIncludeHeader [in] boolean variable to specify whether XML 
 *		header to be appended to the resulting string.
 *
 * @return If no error occurs, returns the pointer to a initialized 
 *		_SMSSUTF8Str structure. See the return values for the 
 *		SMSSUTF8StrAlloc() for more details.
 *
 * @sa _SMSSUTF8Str
 *
 * @sa SMSSUTF8StrAlloc()
 */
//---------------------------------------------------------------------

SMINTF SMXMLStrBuf * _cdecl SMXGBufAlloc(_IN u32 initSize,
										 _IN booln bIncludeHeader);


//---------------------------------------------------------------------
/**
 * Frees the contents and reallocates it for buffer reuse.
 * 
 * Frees the contents in an allocated buffer, re-allocate it of 
 * specified initial length, and adds a standard XML header to the
 * string buffer if specified.
 *
 * @param pXMLBuf [inout] existing buffer to be re-allocated.
 *
 * @param initSize [in] initial buffer size to be allocated.
 * 
 * @param bIncludeHeader [in] boolean variable to specify whether XML 
 *		header to be appended to the resulting string.
 *
 * @return If no error occurs, returns the pointer to the re-allocated 
 *		_SMSSUTF8Str structure. See the return values for the 
 *		SMSSUTF8StrAlloc() for more details.
 *
 * @sa _SMSSUTF8Str
 *
 * @sa SMSSUTF8StrAlloc()
 *
 * @sa SMXGBufAlloc()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMXGBufReAllocContent(_INOUT SMXMLStrBuf *pXMLBuf,
											  _IN u32 initSize,
											  _IN booln bIncludeHeader);

//---------------------------------------------------------------------
/** 
 * Frees up the buffer allocated by SMXAllocBuf(). 
 * 
 * The SMXGBufFree() function calls the SMSSUTF8StrFree() function. 
 * See SMSSUTF8StrFree() definition for more details.
 *
 * @param pXMLBuf [in] pointer to a XML string buffer.
 *
 * @return No return codes.
 *
 * @sa SMSSUTF8StrFree()
 *
 * @sa SMXGBufAlloc()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMXGBufFree(_IN SMXMLStrBuf *pXMLBuf);


//---------------------------------------------------------------------
/**
 * Frees up the buffer allocated by SMXGBufAlloc() and returns the 
 * contents string.
 *
 * The SMXGBufFreeGetContent() function in turn calls the 
 * SMSSUTF8StrFree() function and in addition depending it will return 
 * the pointer to a string contained in the specified pXMLBuf Parameter. 
 *
 * Be sure to free the returned string using the function 
 * SMXGBufFreeContent().
 *
 * @param pXMLBuf [in] pointer to a XML string buffer.
 *
 * @return Returns the contained string in the provided pXMLBuf.
 *
 * @sa SMXGBufAlloc()
 *
 * @sa SMXGBufFreeContent()
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMXGBufFreeGetContent(_IN SMXMLStrBuf *pXMLBuf);


//---------------------------------------------------------------------
/**
 * Frees up the buffer returned by SMXGBufFreeGetContent().
 *
 * The SMXGBufFreeContent() function frees up the memory associated 
 * with the specified parameter.
 *
 * @param pXMLStr [in] string buffer to be freed.
 *
 * @return No return codes.
 *
 * @sa SMXGBufFreeGetContent()
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMXGBufFreeContent(_IN astring *pXMLStr);


//---------------------------------------------------------------------
/**
 * Generate an XML attribute to a specified attribute buffer, for 
 * example: unit="MHz".
 *
 * The SMXGBufCatAttribute() function generates an XML attribute and 
 * write it to the specified buffer.
 *
 * @param pXMLAttrBuf [inout] pointer to a XML attribute buffer 
 *		allocated by SMXGBufAlloc().
 *
 * @param pAttrName [in] pointer to an attribute name string.
 *
 * @param pValue [in] pointer to an attribute value string.
 *
 * @param valueSize [in] size of attribute value.
 *
 * @param valueType [in] supported value types, described in function
 *		SMXGBufCatNodeValue().
 *
 * @param pValueOpts [in] pointer to value options.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf or pTagName 
 *		is NULL, otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufCatEndNode()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatAttribute(_INOUT SMXMLStrBuf *pXMLAttrBuf,
									  _IN const astring *pAttrName,
									  _IN const void *pValue,
									  _IN u32 valueSize,
									  _IN u8 valueType,
									  _IN SMXGValOpts *pValueOpts);


//---------------------------------------------------------------------
/**
 * Generate an opening XML tag, for example: \<ProcSpeed unit="MHz"\>.
 *
 * The SMXGBufCatBeginNode() function appends the tag name and the 
 * attribute strings to the given XML string buffer as per the XML spec.
 * If the parameter pXMLAttrBuf is null then appends just the tag name 
 * and the XML ">" at the end of string.
 *
 * The SMXGBufCatBeginNode() function could possibly followed by a 
 * SMXGBufCatNodeValue() call if there is a value for the node. Whether
 * or not a SMXGBufCatNodeValue() is called, a SMXGBufCatEndNode() call
 * must be made in order to close this beginning node.
 *
 * @param pXMLBuf [in] pointer to a XML String buffer allocated by
 *		SMXGBufAlloc.
 *
 * @param pTagName [in] pointer to a tag name string to be appended.
 *
 * @param pXMLAttrBuf [in] points to an optional attribute string.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf or pTagName 
 *		is NULL, otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufCatEndNode()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatBeginNode(_INOUT SMXMLStrBuf *pXMLBuf,
									  _IN const astring *pTagName, 
									  _IN const SMXMLStrBuf *pXMLAttrBuf);


//---------------------------------------------------------------------
/**
 * Generate a XML node value, for example 1600.
 *
 * The SMXGBufCatNodeValue() function appends an XML node value to a 
 * specified XML string buffer.
 *
 * This function is used after a SMXGBufCatBeginNode() call, and should
 * be followed by a SMXGBufCatEndNode() call.
 *
 * @param pXMLBuf [inout] pointer to a XML String buffer allocated by
 *		SMXGBufAlloc.
 *
 * @param pValue [in] pointer to an tag value.
 * 
 * @param valueSize [in] size of tag value.
 *
 * @param valueType [in] supported value types, described in the table:\n
 *		Symbol:								Meaning \n
 *		CSXS_ASTRING:						Null terminated string  \n
 *		CSXS_USTRING:						Null terminated Unicode string \n
 *		CSXS_BINARY:						Binary value \n
 *		CSXS_BOOLN:							Boolean value \n
 *		CSXS_UNSIGNED32_LITTLE_ENDIAN:		u32 little endian format \n
 *		CSXS_SIGNED32_LITTLE_ENDIAN:		s32 little endian format \n
 *		CSXS_UNSIGNED64_LITTLE_ENDIAN:		U64 little endian format \n
 *		CSXS_SIGNED64_LITTLE_ENDIAN:		s64 little endian format \n
 *		CSXS_DOUBLE64_LITTLE_ENDIAN:		d64 little endian format \n
 *		CSXS_TIME_T:						time_t variable \n
 *		CSXS_SMSSASTRING:					SMSSUTF8Str structure \n
 *		CSXS_SMSSSUSTRING:					SMSSUCS2Str structure \n
 *		CSXS_UNSIGNED8_LITTLE_ENDIAN:		u8 little endian format \n
 *		CSXS_SIGNED8_LITTLE_ENDIAN:			s8 little endian format \n
 *		CSXS_UNSIGNED16_LITTLE_ENDIAN:		u16 little endian format \n
 *		CSXS_SIGNED16_LITTLE_ENDIAN:		s16 little endian format \n
 *
 * @param pValueOpts [in] pointer to value options.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf is NULL, 
 *		otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufCatBeginNode()
 *
 * @sa SMXGBufCatEndNode()
 *
 * @sa SMXGBufCatNode()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatNodeValue(_INOUT SMXMLStrBuf *pXMLBuf,
									  _IN const void *pValue,
									  _IN u32 valueSize,
									  _IN u8 valueType,
									  _IN SMXGValOpts *pValueOpts);


//---------------------------------------------------------------------
/**
 * Generate a closing XML tag, for example \</ProcSpeed\>
 *
 * The SMXGBufCatEndNode() function appends the closing tag to the given 
 * XML string buffer as per the XML spec.
 *
 * The SMXGBufCatEndNode() should be called after SMXGBufCatBeginNode().
 *
 * @param pXMLBuf [in] pointer to a XML String buffer allocated by
 *		SMXGBufAlloc.
 *
 * @param pTagName [in] pointer to a tag name string to be appended.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf or pTagName 
 *		is NULL, otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufCatBeginNode()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatEndNode(_INOUT SMXMLStrBuf *pXMLBuf,
									_IN const astring *pTagName);

	
//---------------------------------------------------------------------
/**
 * Generate a complete XML node, for example 
 * \<ProcSpeed unit="MHz"\>1600\</ProcSpeed\>.
 *
 * The SMXGBufCatNode() function appends the tag name, the attribute 
 * string and the value string ( depending the valType) to the given 
 * XML string buffer as per the XML spec.
 *
 * @param pXMLBuf [inout] pointer to a XML String buffer allocated by
 *		SMXGBufAlloc.
 *
 * @param pTagName [in] pointer to a tag name string to be appended.
 *
 * @param pAttrStrBuf [in] pointer to an attribute string.
 * 
 * @param pValue [in] pointer to an tag value.
 * 
 * @param valueSize [in] size of attribute value.
 *
 * @param valueType [in] supported value types, see 
 *		SMXGBufCatNodeValue() for detail.
 *
 * @param pValueOpts [in] pointer to value options.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf or pTagName 
 *		is NULL, otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufCatBeginNode()
 *
 * @sa SMXGBufCatNodeValue()
 *
 * @sa SMXGBufCatEndNode()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatNode(_INOUT SMXMLStrBuf *pXMLBuf,
								 _IN const astring *pTagName,
								 _IN const SMXMLStrBuf *pAttrStrBuf,
								 _IN const void *pValue,
								 _IN u32 valueSize,
								 _IN u8 valueType,
								 _IN SMXGValOpts *pValueOpts);


//---------------------------------------------------------------------
/**
 * Generate an empty XML node, for example: \<ProcSpeed unit="MHz" /\>
 *
 * Appends the specified tag name and attribute string to the given XML 
 * string buffer as per the XML spec.
 *
 * The SMXGBufCatEmptyNode() function appends the tag name and the 
 * attribute strings to the given XML string buffer as per the XML spec. 
 *
 * If the parameter pXMLAttrBuf is null then appends just the tag name  
 * and the XML "/>" at the end of string.
 *
 * @param pXMLBuf [inout] pointer to a XML String buffer allocated by
 *		SMXGBufAlloc().
 *
 * @param pTagName [in] pointer to a tag name string to be appended.
 *
 * @param pAttrStrBuf [in] points to an optional attribute string.
 *
 * @return Returns SM_STATUS_INVALID_PARAMETER if pXMLBuf or pTagName 
 *		is NULL, otherwise returns SM_STATUS_SUCCESS.
 *
 * @sa SMXGBufFreeGetContent()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMXGBufCatEmptyNode(_INOUT SMXMLStrBuf *pXMLBuf,
									  _IN const astring *pTagName, 
									  _IN const SMXMLStrBuf *pAttrStrBuf);



//---------------------------------------------------------------------
/**
 * Convert ASCII string to XML escapte format.
 * 
 * @note Use SMFreeGeneric to free returned value
 */
//---------------------------------------------------------------------

SMINTF astring * _cdecl SMXGEscapeXMLUTF8Str(_IN const astring *pXMLStr,
											 _INOUT u32 *pXMLStrSize,
											 _IN u32 numIteration);



//---------------------------------------------------------------------
// Time routines
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Get reference point of clock ticks.
 */
//---------------------------------------------------------------------

SMINTF s64 _cdecl SMRefTicksGet(void);



//---------------------------------------------------------------------
/**
 * Check difference in reference ticks, set DisplayMessage and 
 * maxTickViolation.
 */
//---------------------------------------------------------------------

SMINTF booln _cdecl SMRefTicksDiffCheckTimer(_IN s64 startClockTick, 
											 _IN s64 stopClockTick, 
											 _IN s64 maxClockTick, 
											 _OUT s64 *pDiffTicks, 
											 _OUT booln *pDisplayMsg, 
											 _OUT booln *pMaxTickViolation);


//---------------------------------------------------------------------
// Read Write Lock routines
//---------------------------------------------------------------------

//---------------------------------------------------------------------
/**
 * Create a read-write lock.
 */
//---------------------------------------------------------------------

SMINTF void * _cdecl SMRWLCreate(_IN const astring *pRWLName);

SMINTF void * _cdecl SMRWLCreate_s(_IN const astring *pRWLName,
								   _IN u32 aclAttribute);


//---------------------------------------------------------------------
/**
 * Destroy a read-write lock.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRWLDestroy(_IN void *pRWL);


//---------------------------------------------------------------------
/**
 * Get a read lock.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMRWLReadLockGet(_IN void *pRWL,
								   _IN s32 waitTimeOutMilliSecs);


//---------------------------------------------------------------------
/**
 * Release a read lock.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRWLReadLockRelease(_IN void *pRWL);


//---------------------------------------------------------------------
/**
 * Get a write lock.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMRWLWriteLockGet(_IN void *pRWL,
									_IN s32 waitTimeOutMilliSecs);


//---------------------------------------------------------------------
/**
 * Release a write lock.
 */
//---------------------------------------------------------------------

SMINTF void _cdecl SMRWLWriteLockRelease(_IN void *pRWL);


//---------------------------------------------------------------------
// SMOSTypeGet
/** 
 * OS Info: Get operating system type
 *
 * This API returns the operating system type.
 *
 * @return Operating system type value (SM_OS_TYPE_XXX) defined
 *		in dcsupdef.h.
 */
//---------------------------------------------------------------------
SMINTF u32 _cdecl SMOSTypeGet(void);



// ********************************************************************
// ********************************************************************
// **                                                                **
// **  Begin: OBSOLETE/DEPRECATED routines (for Disney)              **
// **                                                                **
// ********************************************************************
// ********************************************************************


//---------------------------------------------------------------------
// Disney only string resource support routines (dcunidat.txt)
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Deprecated API: use SMGetUTF8StrFromIDPathFile
 *
 * @deprecated Deprecated API: use SMGetUTF8StrFromIDPathFile
 *
 * @sa SMGetUTF8StrFromIDPathFile()
 */
//---------------------------------------------------------------------

SMINTF u32 _cdecl SMGetUCS2StrFromID(
				_IN const u32 stringID,
				_INOUT u32 *pLanguageId,
				_OUT ustring *pDest,
				_INOUT u32 *pDestSize
				);



//---------------------------------------------------------------------
// Disney only INI support routines (dcprvXX.ini, dcpubXX.ini dcevtXX.ini)
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Deprecated API: use SMReadINIPathFileValue
 *
 * @deprecated Deprecated API: use SMReadINIPathFileValue
 *
 * @sa SMReadINIPathFileValue()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMReadINIFileValue(
				_IN const astring *pSection, // Section Name
				_IN const astring *pKey, // Key
				_IN u32 vType, // type of value data to be written INI_XXX
				_OUT void *pValue, // pointer to value data buffer to be read into
				_INOUT u32 *pvSize, // size of value data buffer
				_IN const void *pDefault, // default value to return incase of failure to locate key
				_IN u32 dSize, // size of default value data buffer
				_IN const astring *pFileName, // File Name
				_IN booln canBlock // True (block on completion), FALSE (don't block)
				);


//---------------------------------------------------------------------
/**
 * Deprecated API: use SMWriteINIPathFileValue
 *
 * @deprecated Deprecated API: use SMWriteINIPathFileValue
 *
 * @sa SMWriteINIPathFileValue()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMWriteINIFileValue(
				_IN const astring *pSection, // Section Name
				_IN const astring *pKey, // Key
				_IN u32 vType, // type of value data to be written INI_XXX
				_IN const void *pValue, // pointer to value data buffer to be written from
				_IN u32 vSize, // size of value data buffer
				_IN const astring *pFileName, // File Name
				_IN booln canBlock // True (block on completion), FALSE (don't block)
				);



//---------------------------------------------------------------------
// Disney specific install path support function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Deprecated API: use SMGetPathByProductIDandType
 *
 * @deprecated Deprecated API: use SMGetPathByProductIDandType
 *
 * @sa SMGetPathByProductIDandType()
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetInstallPath(
				_OUT astring *pBufPath,
				_INOUT u32 *pBufSize
				);

//---------------------------------------------------------------------
/**
 * Get the installation path by product identification.
 *
 * @param productID [in] Identification searched case-insensitive.
 *
 * @param pBufPath [out] preallocated buffer to store the result.
 *
 * @param pBufSize [inout] result buffer size, always assigned with 
 *		size of result string.
 *
 * @return SM_STATUS_NO_SUCH_OBJECT when entry for given PID is not found;
 *
 * @return SM_STATUS_DATA_OVERRUN when output buffer is insufficient;
 *
 * @return SM_STATUS_OBJECT_BAD when path is not in the file system;
 *
 * @return SM_STATUS_SUCCESS otherwise.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetInstallPathByProductID(_IN s32 productID, 
											  _OUT astring *pBufPath, 
											  _INOUT u32 *pBufSize);



//---------------------------------------------------------------------
// Disney only event logging support function prototypes
//---------------------------------------------------------------------


//---------------------------------------------------------------------
/**
 * Get the maximum logging size
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetLogMaxSize(_IN const astring *pPathFileName, 
								  _IN u32 *pMaxSize);


//---------------------------------------------------------------------
/**
 * Set the maximum logging size
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMSetLogMaxSize(_IN const astring *pPathFileName, 
								  _IN u32 maxSize);


//---------------------------------------------------------------------
/**
 * Appends an event data to Operating System event log. 
 * 
 * @deprecated Deprecated API: use SMAppendToOSSysLog().
 *
 * This function is used to log an event. The entry is written to the 
 * end of the specified log file for the source identified by the pSrc 
 * parameter.  
 *
 * The OS event logging for each OS is described below:
 *
 * Windows - The function internally registers with the OS to get a 
 * handle which represents the source and adds the event description 
 * string as an event to the system log.
 *
 * Linux - opens he system log with LOG_DAEMON as the facility once 
 * succeeded it adds the event to the system log. If there are any 
 * carriage returns in the string are removed before adding to the 
 * system log.
 *
 * Netware - not supported.
 *
 * @param facility [in] logging facility.
 * 
 * @param type [in] Specifies the type of event being logged. This 
 *		parameter can be one of the following values:
 *		Symbol:						Meaning \n
 *		SMLOG_TYPE_SUCCESS:			Success event \n
 *		SMLOG_TYPE_ERROR:			Error event \n
 *		SMLOG_TYPE_WARNING:			Warning event \n
 *		SMLOG_TYPE_INFORMATION:		Information event \n
 *		SMLOG_TYPE_AUDIT_SUCCESS:	Success audit event \n
 *		SMLOG_TYPE_AUDIT_FAILURE:	Failure audit event
 *
 * @param category [in] ID assigned to the message category name.
 *
 * @param eventID [in] ID associated with event. For NT/W2k OS logs 
 *		the event identifier specifies the entry in the message file 
 *		associated with the event source.
 *
 * @param pUTF8Source [in] null terminated sting specifies who 
 *		generated the event.
 *
 * @param pUTF8EventDesc [in] Description of the event.
 *
 * @param pData	[in] pointer to an optional event Data.
 *
 * @param dataBufSize [in] size in bytes of optional event data.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the entry was written to 
 *		the log.
 * 
 * @return If the function fails, following are the possible return 
 *		codes:
 *
 * @return SM_STATUS_CMD_BAD - if the facility provided is not 
 *		supported or any of the provided parameters are not valid.
 *
 * @return SM_STATUS_RESOURCE_LOCKED - if the file to be written is 
 *		locked by some other thread or process.
 *
 * @return SM_STATUS_DEVICE_FULL - If the log is full.
 *
 * @return SM_STATUS_UNSUCCESSFUL - for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMAppendToLog(
				_IN u8 facility, // logging facility
				_IN u16 type, // type of log
				_IN u16 category, // ID assigned to the message category name
				_IN u32 eventID, // ID assigned to this event
				_IN const astring *pUTF8Source, // name of source
				_IN const astring *pUTF8EventDesc, // translated event string
				_IN const void *pData, // pointer to optional event data
				_IN u32 dataBufSize // size in bytes of optional event data
				);


//---------------------------------------------------------------------
/**
 * Clears the OS event log. 
 * 
 * The SMClearOSLog function cleats the log file specified and adds an 
 * cleared event to the log with the specified source name. It does not 
 * backup the existing event log file.
 *
 * @param facility [in] logging facility.
 *
 * @param category [in] ID assigned to the message category name.
 *
 * @param pSource [in] source name for the cleared event to be added.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the log has been cleared.
 *
 * @return SM_STATUS_UNSUCCESSFUL - for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMClearLog(
				_IN u8 facility, // logging facility
				_IN u16 category, // ID assigned to the message category name
				_IN const astring *pSource // name of source
				);


//---------------------------------------------------------------------
/**
 * Clears the OS event log. 
 * 
 * The SMClearOSLog function cleats the log file specified and adds an 
 * enhanced (EEMI) cleared event to the log with the specified source name.  
 * It does not backup the existing event log file.
 *
 * @param facility [in] logging facility.
 *
 * @param category [in] ID assigned to the message category name.
 *
 * @param pSource [in] source name for the cleared event to be added.
 *
 * @return If the function succeeds, the return value is 
 *		SM_STATUS_SUCCESS, indicating that the log has been cleared.
 *
 * @return SM_STATUS_UNSUCCESSFUL - for all the other failure cases.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMClearLogEnh(
				_IN u8 facility, // logging facility
				_IN u16 category, // ID assigned to the message category name
				_IN const astring *pSource // name of source
				);


//---------------------------------------------------------------------
/**
 * Get the path file name for the log file.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetLogPathFileName(
				_IN u8 facility, // logging facility
				_OUT astring *pBufPathFileName, // returned path file name
				_INOUT u32 *pBufSize // size of buffer passed
				);


//---------------------------------------------------------------------
/**
 * Get the path file name for the backup log file.
 */
//---------------------------------------------------------------------

SMINTF s32 _cdecl SMGetBackupLogPathFileName(
			   _IN u8 facility, // logging facility
			   _OUT astring *pBufBakPathFileName, // returned path file name
			   _INOUT u32 *pBufSize // size of buffer passed
			   );

// ********************************************************************
// ********************************************************************
// **                                                                **
// **  End: OBSOLETE/DEPRECATED routines (for Disney)                **
// **                                                                **
// ********************************************************************
// ********************************************************************




//---------------------------------------------------------------------
// End
//---------------------------------------------------------------------


#ifdef __cplusplus
}
#endif


#endif // __DCSUPAPI_H_INCLUDED__

